define(["jquery",
		"underscore",
		"utils/ShapeUtils",
		"component/tooltip/Tooltip",
		"utils/webi18n!WFEditorWCRPopup",
		"utils/Observer"
	],
	function ($,
		_,
		ShapeUtils,
		Tooltip,
		i18n,
		Observer) {

		"use strict";

		/**
		 * This class is an anchor to opening WFEditorWCR in a popup
		 * It will add an on click handler to the dom element it was added to as a data-init property
		 * The DOM element should also have the following attributes: data-taskTypeId, data-taskId, data-workflowName
		 * Clicking on the dom element will then require all the scripts needed to open the workflow in a popup
		 *
		 * @class
		 * @exports WFEditorWCRPopupAnchor
		 * @param {String} taskId - id of the task that should be opened on click
		 * @param {String} taskTypeId - id of the task type of the task
		 * @param {String} name - name of the task
		 * @param {Object} privileges
		 */
		var WFEditorWCRPopupAnchor = function (taskId, taskTypeId, name, privileges) {
			Observer.mixin(this);
			/**
			 * @member {WFEditor}
			 * @private
			 */
			this._wFEditor = null;

			/** 
			 * @member {String} 
			 * @private
			 */
			this._taskTypeId = taskTypeId;

			/** 
			 * @member {String} 
			 * @private
			 */
			this._taskId = taskId;

			/** 
			 * @member {String} 
			 * @private
			 */
			this._name = name;

			/**
			 * @member {Object}
			 * @private
			 */
			this._privileges = privileges || {};

			/**
			 * @member {WFEditorWCRPopup}
			 * @private
			 */
			this._popup = null;

			/**
			 * @member {ShapeUtils.Rect} the visible rect of the WFEdidtor upon closing, to store the context for the user
			 * @private
			 */
			this._visibleRect = null;
		};

		/**
		 * opens the workflow on the location of the click event
		 * @param {Event} the mouse click event that triggered this function
		 */
		WFEditorWCRPopupAnchor.prototype.openWorkflow = function (e) {
			requirejs(["WFEditorWCR/wfPopup/MinifiedWFEditorWCR",
				"WFEditorWCR/wfPopup/WFEditorWCRPopup", "utils/webcss!WFEditorWCR/core/wfeditorwcr-style.css"
			], _.bind(function (MinifiedWFEditorWCR, WFEditorWCRPopup) {
				var body = $("body");
				if (!this._wFEditor) {
					this._popup = new WFEditorWCRPopup("popup" + this._taskTypeId, body, this._name);
					this._popup.element.addClass("minified-wfeditor-popup preview-wfeditor-popup");
					// Add the "go to full screen, full editor" button

					var maximizeWFIcon = $("<a/>").addClass("enter-full-icon").append($("<i/>")
							.addClass("icon icon-WCR_WF-LargerWindow12 icon-normal"))
						.on("click", _.bind(function () {
							tooltip.destroy();
							this._goToFullEditor();
						}, this)).on("mousedown", function(e){e.stopPropagation();});

					this._popup.getHeader().append(maximizeWFIcon);

					var tooltip = new Tooltip(null, maximizeWFIcon, body, body, false, i18n.t("wfeditorPopup.tooltip.goToFullEditor"));

					this._wFEditor = new MinifiedWFEditorWCR(this._taskId, this._taskTypeId, this._popup.content, this._popup.getFooter(), this._privileges);
					// resize the popup if the wfeditor wants to be resized
					this._observe(this._wFEditor, "resize", function (width, height) {
						if (this._popup.element.width() < width) {
							this._popup.element.width(width);
						}
						if (this._popup.element.height() < height) {
							this._popup.element.height(height);
						}
						// make sure the popup is visible in the screen
						var footerHeight = $(".wcr-page-footer").height();
						if (($(window).height() - height) < this._popup.element.offset().top - footerHeight || ($(body).height() - height) < this._popup.element.offset().top - footerHeight) {
							this._popup.element.css({
								top: Math.min($(window).height() - height + window.pageYOffset, $(body).height() - height) - footerHeight
							});
						}
					});
					this._registerListeners();
				}
				this.open(e);

			}, this));
		};

		/**
		 * switches between the minified version of the WFEditor to the fully fledged one
		 */
		WFEditorWCRPopupAnchor.prototype._goToFullEditor = function () {
			requirejs(["WFEditorWCR/core/WFEditorWCR", "WFEditorWCR/wfPopup/WFEditorWCRPopup", "utils/webcss!WFEditorWCR/core/wfeditorwcr-style.css"], _.bind(function (WFEditorWCR, WFEditorWCRPopup) {
				if (this._wFEditor && this._taskId) {
					this._visibleRect = this._wFEditor.getVisibleRect();
					this._removeEditor();
				}
				if (!this._wFEditor) {
					var body = $("body");
					this._popup = new WFEditorWCRPopup("popup" + this._taskTypeId, body, this._name);
					this._popup.element.addClass("full-wfeditor-popup preview-wfeditor-popup");
					this._popup.content.attr("tasktypeid", this._taskTypeId);
					this._popup.content.attr("taskid", this._taskId);
					this._popup.content.attr("canedit", this._privileges.hasTaskEditRights ? "1" : "0");
					this._popup.content.attr("autorefresh", this._privileges.autorefresh ? "1" : "0");
					this._popup.content.attr("mastertaskid", this._privileges.isMasterTask ? undefined : "0");
					this._popup.element.css({
						width: $(window).width() * 2 / 3,
						height: $(window).height() * 2 / 3
					});

					this._wFEditor = WFEditorWCR.wcrinit(this._popup.content[0]);
					this._registerListeners();
				}
				this._popup.show(new ShapeUtils.Point($(window).width() / 6, $(window).height() / 6 + window.pageYOffset));

			}, this));
		};

		/**
		 * registeres the listeners on the WFEditor
		 */
		WFEditorWCRPopupAnchor.prototype._registerListeners = function () {
			this._popup.setTitle(this._name);
			this._observe(this._wFEditor, "workflowLoaded", function () {
				if (this._visibleRect) {
					this._wFEditor.setVisibleRect(this._visibleRect);
				}
				this._wFEditor.focus();
			});
			this._popup.element.on("mousedown", _.bind(function (e) {
				// Mouse down on the element should result in the popup being on top
				// This is not the same as the WFEditor having focus as the user
				// may have eg. clicked on an input field inside a parameter panel 
				this._movePopupToFront();
			}, this));
			this._observe(this._wFEditor, "focus", function () {
				this._setFocusedPopup();
			});
			this._observe(this._wFEditor, "blur", function () {
				this._popup.setTitle(this._name);
				this._popup.element.removeClass("focused");
			});
			this._observe(this._popup, "*.stop", function () {
				this._wFEditor.resized();
			});
			
			this._observe(this._popup, "*.start", function () {
				this._wFEditor.focus();
			});
			
			this._popup.getFooter().on("mousedown", _.bind(function () {
				this._wFEditor.focus();
			}, this));
			this._popup.getHeader().on("mousedown", _.bind(function () {
				this._wFEditor.focus();
			}, this));

			this._observe(this._popup, "closed", function (canCloseObj) {
				if (this._taskId || this._wFEditor.canClose()) {
					canCloseObj.canClose = true;
					this._visibleRect = this._wFEditor.getVisibleRect();
					this._removeEditor();
					this._stopObserveAll();
				} else {
					// let the popup know that he shouldn't close
					canCloseObj.canClose = false;
				}
			});
			
		};

		/**
		 * Moves the popup to the front by appending it to the body
		 * @param {Boolean} focus - should the Editor be focused as well?
		 */
		WFEditorWCRPopupAnchor.prototype._movePopupToFront = function (focus) {
			if (!this._popup.element.is('.preview-wfeditor-popup:last')) {
				if(this._wFEditor) {
					this._wFEditor.notifyRemoveFromDom();
				}
				this._popup.element.appendTo($("body"));
				if(this._wFEditor) {
					this._wFEditor.notifyAddToDom();
				}
				if(focus && this._wFEditor) {
					this._wFEditor.focus();
				}
			}
		};

		/**
		 * adds styles to let the user know the popup is focused
		 */
		WFEditorWCRPopupAnchor.prototype._setFocusedPopup = function () {
			// Append the popup at the end of the dom to make sure it is on top even if it is out of focus due to opening of parameter popup
			this._popup.setTitle(this._name + " (" + i18n.t("wfeditorPopup.focused") + ")");
			this._popup.element.addClass("focused");
			this._movePopupToFront(true);
		};

		/**
		 * removes the editor from the dom
		 */
		WFEditorWCRPopupAnchor.prototype._removeEditor = function () {
			this._wFEditor.destroy();
			this._wFEditor = null;
			this._popup.element.remove();
		};

		/**
		 * opens the editor on the location of a click event
		 * @param {Event}
		 */
		WFEditorWCRPopupAnchor.prototype.open = function (event) {
			var position = new ShapeUtils.Point();
			if ((event.pageX || (event.pageX === 0)) && (event.pageY || event.pageY === 0)) {
				position = new ShapeUtils.Point(event.pageX, event.pageY);
			} else if ((event.clientX || event.clientX === 0) && (event.clientY || event.clientY === 0)) {
				position = new ShapeUtils.Point(event.clientX, event.clientY);
			}

			// make sure the popup is visible in the screen
			var height = this._popup.element.height();
			var footerHeight = $(".wcr-page-footer").height();
			if (($(window).height() + window.pageYOffset - height) < position.y - footerHeight || ($("body").height() - height) < position.y - footerHeight) {
				position.y = Math.min($(window).height() - height + window.pageYOffset, ($("body").height() - height)) - footerHeight;
			}

			this._popup.show(position);
		};


		/**
		 * adds click handlers to the context element
		 * @param {DOMObject}
		 */
		WFEditorWCRPopupAnchor.wcrinit = function (context) {
			var privileges = {
				isMasterTask: context.getAttribute("data-mastertaskid") ? context.getAttribute("data-mastertaskid").length === 0 : true, // is a mastertask or not
				autorefresh: context.getAttribute("data-autorefresh") == "1", // should feedback be autorefreshed
				hasTaskEditRights: context.getAttribute("data-canedit") == "1"
			};
			context = $(context);
			var taskTypeId = context.attr("data-taskTypeId");
			var taskId = context.attr("data-taskId");
			var workflowName = context.attr("data-workflowName");
			var body = $("body");
			var wFEditorWCRPopupAnchor = new WFEditorWCRPopupAnchor(taskId, taskTypeId, workflowName, privileges);
			if (taskId && taskTypeId) {
				context.on("click", function (e) {
					wFEditorWCRPopupAnchor.openWorkflow(e);
					e.preventDefault();
				});
				new Tooltip(null, $(context), body, body, false, i18n.t("wfeditorPopup.tooltip.showActiveNodes"));
			} else if (taskTypeId) {
				context.on("click", function (e) {
					wFEditorWCRPopupAnchor._goToFullEditor();
					e.preventDefault();
					e.stopPropagation();
				});
				new Tooltip(null, $(context), body, body, false, i18n.t("wfeditorPopup.tooltip.openWorkflow"));
			}
		};


		return WFEditorWCRPopupAnchor;

	});