define(["underscore"], 
function(_) {		
"use strict";

	/**
	 * Create a new AnimationOptions object of the given type
	 * see http://greweb.me/2012/02/bezier-curve-based-easing-functions-from-concept-to-implementation/
	 * 
	 * MIT License
	 *
	 * KeySpline - use bezier curve for transition easing function
	 * Copyright (c) 2012 Gaetan Renaudeau <renaudeau.gaetan@gmail.com>
	 *
	 * Permission is hereby granted, free of charge, to any person obtaining a
	 * copy of this software and associated documentation files (the "Software"),
	 * to deal in the Software without restriction, including without limitation
	 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
	 * and/or sell copies of the Software, and to permit persons to whom the
	 * Software is furnished to do so, subject to the following conditions:
	 *
	 * The above copyright notice and this permission notice shall be included in
	 * all copies or substantial portions of the Software.
	 *
	 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
	 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
	 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	 * DEALINGS IN THE SOFTWARE.
	 * 
	 * Notice: changed by Esko to become prototyped
	 */

	/**
	 * @param {String} type - One of "ease", "linear", "ease-in", "ease-out", or "ease-in-out"
	 */
	var AnimationOptions = function(type){
	
		this._types = {
			    "ease":        [0.25, 0.1, 0.25, 1.0], 
			    "linear":      [0.00, 0.0, 1.00, 1.0],
			    "ease-in":     [0.42, 0.0, 1.00, 1.0],
			    "ease-out":    [0.00, 0.0, 0.58, 1.0],
			    "ease-in-out": [0.42, 0.0, 0.58, 1.0]
		};
		
		this._options = this._types[type];
		if(!this._options){
			this._options = this._types["linear"];
		}
		
		this._mX1 = this._options[0];
		this._mY1 = this._options[1];
		this._mX2 = this._options[2];
		this._mY2 = this._options[3];				
	};
	
	/**
	 * Get the current position on the bezier based on a [0 1] value
	 * @param {Number} aX - value in [0-1]
	 * @returns {Number} The current position on the bezier based on a [0 1] value
	 */
	AnimationOptions.prototype.get = function(aX) {
		if (this._mX1 == this._mY1 && this._mX2 == this._mY2) return aX; // linear
		return this._calcBezier(this._getTForX(aX), this._mY1, this._mY2);
	};
	 
	/**
	 * @private
	 * @param aA1
	 * @param aA2
	 * @returns {Number}
	 */
	AnimationOptions.prototype._a = function(aA1, aA2) { 
		return 1.0 - 3.0 * aA2 + 3.0 * aA1; 
	};
	
	/**
	 * @private
	 * @param aA1
	 * @param aA2
	 * @returns {Number}
	 */
	AnimationOptions.prototype._b = function(aA1, aA2) { 
		return 3.0 * aA2 - 6.0 * aA1; 
	};
	
	/**
	 * @private
	 * @param aA1
	 * @returns {Number}
	 */
	AnimationOptions.prototype._c = function(aA1){ 
		return 3.0 * aA1;
	};
	 
	/**
	 * @private
	 * Returns x(t) given t, x1, and x2, or y(t) given t, y1, and y2.
	 * @param aT
	 * @param aA1
	 * @param aA2
	 * @returns {Number}
	 */
	AnimationOptions.prototype._calcBezier = function(aT, aA1, aA2) {
		 return ((this._a(aA1, aA2)*aT + this._b(aA1, aA2))*aT + this._c(aA1))*aT;
	};
	 
	/**
	 * @private
	 * Returns dx/dt given t, x1, and x2, or dy/dt given t, y1, and y2.
	 * @param aT
	 * @param aA1
	 * @param aA2
	 * @returns
	 */
	AnimationOptions.prototype._getSlope = function(aT, aA1, aA2) {
		return 3.0 * this._a(aA1, aA2)*aT*aT + 2.0 * this._b(aA1, aA2) * aT + this._c(aA1);
	};
	 
	/**
	 * @private
	 * @param aX
	 * @returns
	 */
	AnimationOptions.prototype._getTForX = function(aX) {
		// Newton raphson iteration
		var aGuessT = aX;
		for (var i = 0; i < 4; ++i) {
		  var currentSlope = this._getSlope(aGuessT, this._mX1, this._mX2);
		  if (currentSlope == 0.0) return aGuessT;
		  var currentX = this._calcBezier(aGuessT, this._mX1, this._mX2) - aX;
		  aGuessT -= currentX / currentSlope;
		}
		return aGuessT;
	};
	
	return AnimationOptions;
});