/**
 * ContentStatement object
 * @author HUBU
 */
define(["utils/XMLUtils",
		"./ContentConstants",
		"./ContentStatementElement",
		"./ContentStatementPermissions",
		"underscore"
	],
	function (XMLUtils,
		ContentConstants,
		ContentStatementElement,
		ContentStatementPermission,
		_) {
		"use strict";


		/**
		 * @class Represents a content statement.
		 * @exports ContentStatement
		 * @constructor
		 */
		var ContentStatement = function (json) {

			if (json == undefined) {
				this.elementType = null;
				this.invariant = null;
				this.index = 0;
				this.type = ContentConstants.TextType.RICHTEXT;
				this.state = null;
				this.updated = null;
				this.element = [];
				this.attributeId = null;
				this.permission = [];
				this.isPermissionsChanged = null;
				this.approvalType = null;
				this.docVersionId = null;
				this.projectId = null;
				this.contentManagerView = 'false';
				this.isApprovalPasswordRequired = null;
				this.locationList = [];
				this.instanceId = null;
			} else {
				this.elementType = json.elementType;
				this.invariant = json.invariant;
				this.index = json.index;
				this.type = json.type;
				this.state = json.state;
				this.updated = json.updated;
				this.attributeId = json.attributeId;
				this.approvalType = json.approvalType;
				this.isPermissionsChanged = json.isPermissionsChanged;
				this.docVersionId = json.docVersionId;
				this.projectId = json.projectId;
				this.contentManagerView = json.contentManagerView;
				this.element = [];
				_.each(json.element, _.bind(function (elem) {
					this.element.push(new ContentStatementElement(elem, this));
				}, this));
				this.permission = [];
				_.each(json.permission, _.bind(function (perm) {
					this.permission.push(new ContentStatementPermission(perm));
				}, this));
				this.isApprovalPasswordRequired = json.isApprovalPasswordRequired;
				this.locationList = json.locationList;
				this.instanceId = json.instanceId;
			}
		};
		
		
		/**
		 * A method to get the ContentStatementElement of this ContentStatement for a given language.
		 * 
		 * @param isMaster
		 */
		ContentStatement.prototype.getLangCodesOfElements = function(isMaster) {
			var langs = [];
			_.each(this.element, _.bind(function (element) {
				if (isMaster && element.master === "true" || (!isMaster && (!element.master || element.master === "false"))) {
					if (!element.deleted && element.state !== ContentConstants.State.EMPTY) {
						langs.push(element.language);
					}
				}
			}, this));
			return langs;
		};
		
		/**
		 * A method to get the master languages which are SignedOff/Approved
		 * 
		 * @param availableMasterLangs - Array of language objects
		 * @returns {Array}
		 */
		ContentStatement.prototype.getSignedOffMasterLangs = function(availableMasterLangs) {
			var signedOffMasterLangs = [];
			
			for(var index=0; index<availableMasterLangs.length; index++) {
				var lang = availableMasterLangs[index];
				_.each(this.element, _.bind(function (element) {
					if (element.master === "true" && ContentConstants.State.APPROVED === element.state && lang.langCode === element.language) {
						signedOffMasterLangs.push(lang);
					}
				}, this));
			}
			
			return signedOffMasterLangs;
		};
		
		
		/**
		 * This will return an array of status for Master/Translation
		 * 
		 * @param isMaster
		 */
		ContentStatement.prototype.getLangStatusOfElements = function(isMaster) {
			var status = [];
			_.each(this.element, _.bind(function (element) {
				if (isMaster && element.master === "true" || !isMaster && element.master === "false") {
					status.push(element.state);
				}
			}, this));
			return status;
		};
		
		/**
		 * A method to get the ContentStatementElement of this ContentStatement for a given language.
		 * 
		 * @param language
		 * @param isMaster
		 */
		ContentStatement.prototype.addLanguage = function(language, isMaster, elementProvider) {
			
			// Parse vars to desired format (be safe)
			var langCode = (typeof language === "object") ? language.langCode : language;
			isMaster = (typeof isMaster === "string") ? isMaster === "true" : isMaster;
			var isInvariant = (typeof this.invariant === "string") ? this.invariant === "true" : this.invariant;
			
			// If this is an invariant statement and we're adding a translation language, this operation should do nothing.
			if (!isMaster && isInvariant) {
				window.console.log("An attempt was made to add a translation language ("+langCode+") to an invariant statement.");
				return;
			}
			
			// We need to create a new ContentStatementElement for this language.
			// First check wether we do not already a ContentStatementElement for this language.
			var csElem = this.getElement(langCode, isMaster);
			if (csElem) {
				// There already exists such a ContentStatementElement, check whether it's marked as deleted.
				if (!csElem.deleted) {
					// Check if the element is Empty.
					if(csElem.state == ContentConstants.State.EMPTY) {
						// Set the state to draft ...
						csElem.state = ContentConstants.State.DRAFT;
						csElem.contentChanged = true;
						// Emit that a language was added.
						pcmEventBus.emit("PCM.ContentStatement.langAdded", this, csElem);
					} else {
						// Language already exists and is added.
						return;
					}
				} else {
					// The ContentStatementElement was marked as deleted, unmark it.
					csElem.deleted = false;
					// Emit that a language was added.
					pcmEventBus.emit("PCM.ContentStatement.langAdded", this, csElem);
				}
			} else {
				// Check if an element provider was passed ...
				if (elementProvider) {
					// Use the elementProvider to get the ContentStatementElement for the new language..
					
					// No ContentStatementElement does exist for this ContentStatement, we can't reuse one and 
					// really need to create and fetch one from the back end. Use the "elementProvider" for this.
					elementProvider.requestNewContentStatementElement(language, isMaster, _.bind(function (csElem) {
						
						if(csElem.state == ContentConstants.State.EMPTY) {
							// Set the state to draft ...
							csElem.state = ContentConstants.State.DRAFT;
						}
						// Set the current parent statement.
						csElem.parentStatement = this;
						// Indicate that the content is changed (so that the backend picks it up for adding).
						csElem.contentChanged = true;
						// Add it to the other elements
						this.element.push(csElem);
						// Emit that a language was added.
						pcmEventBus.emit("PCM.ContentStatement.langAdded", this, csElem);					
					},this), _.bind(function(message) {
						if (console && console.log) {
							console.log(message);
						}
					}, this));
				} else {
					// No elementProvider was passed, we have no choice but to create one ourselves.
					// We do this as a backup and don't give out any permissions! We need the backend for this.
					csElem = new ContentStatementElement();			
					csElem.master = isMaster+"";
					csElem.state = ContentConstants.State.DRAFT;
					csElem.language = langCode;
					csElem.created = null;
					csElem.updated = null;
					csElem.content = {};
					csElem.originalApproval = [];
					csElem.approval = [];
					csElem.comment = [];
					csElem.deleted = false;
					csElem.contentChanged = true;
					csElem.commentChanged = false;
					csElem.approvalChanged = false;
					csElem.isEditable = "false";
					csElem.whyNonEditable = "NO_PCM_PERM_NODEV";
					csElem.isReviewable = "false";
					csElem.whyNonReviewable = "NO_PCM_PERM_NODEV";
					csElem.parentStatement = this;
					csElem.isApprovalPasswordRequired = null;
					// And add it to the list of elements.
					this.element.push(csElem);
					// Emit that a language was added.
					pcmEventBus.emit("PCM.ContentStatement.langAdded", this, csElem);
				}
			}
		};
		
		/**
		 * A method to get the ContentStatementElement of this ContentStatement for a given language.
		 * 
		 * @param language
		 * @param isMaster
		 */
		ContentStatement.prototype.removeLanguage = function(language, isMaster) {
			
			// parse input
			var langCode = (typeof language === "object") ? language.langCode : language;
			var isMaster = (typeof isMaster === "string") ? isMaster === "true" : isMaster;
			
			// Get the element.
			var csElem = this.getElement(langCode, isMaster);
			
			// Check if it can be found.
			if (csElem) {
				
				// Set the deleted mark
				csElem.deleted = true;
				
				// Emit that a language was removed.
				pcmEventBus.emit("PCM.ContentStatement.langRemoved", this, csElem);
				
				// Return the removed ContentStatementElement
				return csElem;
			};
		};
		
		/**
		 * Return the ContentStatementElement of this ContentStatement for a given language, regardless of
		 * whether it's deleted or not.
		 * 
		 * @param langCode
		 * @param isMaster
		 */
		ContentStatement.prototype.getElement = function(langCode, isMaster) {
			
			// Master flag in CS element is stored as string. For comparison keep variable type as string. 
			isMaster = ((typeof isMaster === 'boolean' && isMaster) || 'true' === isMaster) ? 'true' : 'false';
			
			// Get it from the existing language elements.
			var element;
			_.each(this.element, _.bind(function (csElem) {
				// In case of master undefined language is considered as translation.
				csElem.master = (csElem.master && 'true' === csElem.master) ? 'true' : 'false';
				
				if (csElem.language === langCode && csElem.master == isMaster) {
					element = csElem;
				}
			}, this));
			
			return element;
		};
		
		/**
		 * A method to get the ContentStatementElement of this ContentStatement for a given language. It will
		 * only return non deleted elements. If no such element exists for the langcode, an "unadded" one will
		 * be returned.
		 * 
		 * @param langCode
		 * @param isMaster
		 */
		ContentStatement.prototype.provideElement = function(langCode, isMaster) {
			
			// Master flag in CS element is stored as string. For comparison keep variable type as string. 
			isMaster = ((typeof isMaster === 'boolean' && isMaster) || 'true' === isMaster) ? 'true' : 'false';
			
			// Get it from the existing language elements.
			var element;
			_.each(this.element, _.bind(function (csElem) {
				// In case of master undefined language is considered as translation.
				csElem.master = (csElem.master && 'true' === csElem.master) ? 'true' : 'false';
				
				if (csElem.language === langCode && csElem.master == isMaster && !csElem.deleted) {
					element = csElem;
				}
			}, this));
			
			// Create an unAdded one we could not find an added one.
			if (!element && langCode) {
				var element = new ContentStatementElement();
				element.initContentStatementElement(isMaster, {}, ContentConstants.State.EMPTY, langCode, this);
				element.isEditable = 'false';
				element.whyNonEditable = 'NO_PCM_PERM_NOT_ADDED';
				element.isReviewable = 'false';
				element.whyNonReviewable = 'NO_PCM_PERM_NOT_ADDED';
			}
			
			return element;
		};
		
		/**
		 * Create JSON request object from ContentStatement
		 * @param {String} password
		 * @returns {Json} JSON Object
		 */
		ContentStatement.prototype.getUnsavedContentJSONObject = function (password) {
			var jsonObj = {
					"update": []
			};
			
			jsonObj.update.push(this._getUnsavedContentJSONObject(password));
			return jsonObj;
		};
		
		/**
		 * Create JSON request string from ContentStatement
		 * @param {String} password
		 * @returns {String} JSONString
		 */
		ContentStatement.prototype.getUnsavedContentJSON = function (password) {

			var jsonObj = this.getUnsavedContentJSONObject(password);
			var jsonString = JSON.stringify(jsonObj);
			return jsonString;
		};
		
		/**
		 * Create JSON request object from ContentStatement
		 * 
		 * @param {String} password
		 * @returns {Object}
		 */
		ContentStatement.prototype._getUnsavedContentJSONObject = function (password) {
			var jsonObjVar = new Object();
			
			var contentArr = [];
			var commentArr = [];
			var approvalArr = [];
			var deletionArr =[];
			var permissionsArr = [];

			/*
			 * Sample JSON request string
			 * updateJson:
			 * {"update":[{"docversionid":"00002_0000014629",
			 * "elementtype":"Usage Instructions",
			 * "invariant":"true",
			 * "content":[{"value":"Mieux lorsqu'il est utilisé PAR DATE imprimé sur le dessus de cette boîte.","language":"fr-BE"}],
			 * "comment":[{"value":"Content obtained from the Conbury art file","language":"fr-BE"}],
			 * "approval":[{"value":"signed-off","language":"nl-BE"}],
			 * "permissions" :[{"role":"PACKAGING MANAGER", "language":"nl-BE", "tcp":"CONTENT MANAGER"},
			 * 					{"role":"EDITOR [LANGUAGE]", "tcp":"EDITOR"}]
			 * }]}
			 */
			
			//Push approval state into JSON string
			if (this.isPermissionsChanged && this.permission.length > 0)  {
					_.each(this.permission, _.bind(function (permission){
						permissionsArr.push({
						"role": permission.role,
						"tcp": permission.tcp,
						"lang": permission.lang
					});
				}));				
			}
			
			_.each(this.element, _.bind(function (csElement) {
				
				//If the state is "NOT ADDED", skip it.
				if (csElement.state == ContentConstants.State.EMPTY) {
					return;
				} 
				
				//In some places master set as boolean instead of string
				if(typeof csElement.master !== "string"){
					csElement.master = JSON.stringify(csElement.master);
				}
				
				//If the document is marked as invariant, add all the translations to deleted array
				if(this.invariant === "true" && csElement.master === "false"){
					deletionArr.push({
						"language": csElement.language,
						"master": csElement.master
					});
					//No further processing required for this element, as it is marked for delete, so continue with next element
					return;
				}
				
				//Push text content into JSON string
				if (typeof csElement.content !== "undefined" && csElement.contentChanged && !csElement.deleted){
					contentArr.push({
						"value": csElement.content.content,
						"language": csElement.language,
						"master": csElement.master
					});
				}
				
				//Push comments into JSON string
				if (csElement.commentChanged && csElement.comment.length > 0 && !csElement.deleted) {
					_.each(csElement.comment, _.bind(function (elemComment) {
						if(elemComment.isNew){
							commentArr.push({
								"value": elemComment,
								"language": csElement.language,
								"master": csElement.master
							});
						}
					}));
				}

				//Push approval state into JSON string
				if (csElement.approvalChanged && csElement.approval.length > 0 && !csElement.deleted)  {
					//Only single approval status change allowed for save
					var elemApprove = csElement.approval[0];
					approvalArr.push({
						"value": elemApprove.state,
						"language": csElement.language,
						"master": csElement.master
					});
				}
				
				// If csElement is deleted, push it in the deleted array.
				if(csElement.deleted) {
					deletionArr.push({
						"language": csElement.language,
						"master": csElement.master
					});
				}
								
			}, this));
			
			
			jsonObjVar.docversionid = this.docVersionId;
			
			if (contentArr.length > 0) {
				jsonObjVar.content = contentArr;
			}
			if (commentArr.length > 0) {
				jsonObjVar.comment = commentArr;
			}
			if (approvalArr.length > 0) {
				jsonObjVar.approval = approvalArr;
			}
			if (deletionArr.length > 0) {
				jsonObjVar.deletion = deletionArr;
			}			
			if (this.elementTypeChanged) {
				jsonObjVar.elementtype = this.elementType;
			}
			if (this.invariantChanged) {
				jsonObjVar.invariant = this.invariant;
			}
			if (this.approvalTypeChanged) {
				jsonObjVar.approvaltype = this.approvalType;
			}
			if (this.typeChanged) {
				jsonObjVar.type = this.type;
			}
			
			if(permissionsArr.length > 0){
				jsonObjVar.permission = permissionsArr;
			}
			
			if(password) {
				jsonObjVar.password = password;
			}
			
			return jsonObjVar;
		};
		
		/**
		 * Clone this ContentStatement.
		 * 
		 * @returns {ContentStatement} A ContentStatement that is exactly the same.
		 */
		ContentStatement.prototype.clone = function () {
			var clone = new ContentStatement();
			clone.elementType = this.elementType;
			clone.invariant = this.invariant;
			clone.index = this.index;
			clone.type = this.type;
			clone.state = this.state;
			clone.updated = this.updated;
			clone.element = [];
			_.each(this.element, _.bind(function (elem) {
				var clonedElem = elem.clone();
				clonedElem.parentStatement = clone;
				clone.element.push(clonedElem);
			}, this));
			clone.attributeId = this.attributeId;
			clone.permission = [];
			_.each(this.permission, _.bind(function (perm) {
				clone.permission.push(perm.clone());
			}, this));
			clone.approvalType = this.approvalType;
			clone.docVersionId = this.docVersionId;
			clone.projectId = this.projectId;
			clone.contentManagerView = this.contentManagerView;
			clone.isApprovalPasswordRequired = this.isApprovalPasswordRequired;
			return clone;
		};

		/**
		 * Indicate whether this statement changed state so far.
		 * 
		 * @returns {Boolean}
		 */
		ContentStatement.prototype.getContentChanged = function(){
			var isChanged = false;
			if(this.isPermissionsChanged){
				isChanged = true;
				return isChanged;
			}
				
			_.each(this.element, _.bind(function (csElement) {
				if (csElement.contentChanged || csElement.commentChanged || csElement.approvalChanged || csElement.deleted){
					isChanged = true;
					return false;
				}
			}));
			
			return isChanged;
		};
		
		/**
		 * Count of the ContentStatementElement which are modified
		 * 
		 * @returns {Number}
		 */
		ContentStatement.prototype.getContentChangedCount = function(){
			var count = 0;
			_.each(this.element, _.bind(function (csElement) {
				if (csElement.contentChanged || csElement.commentChanged || csElement.approvalChanged || csElement.deleted){
					count++;
				}
			}));
			return count;
		};
		
		/**
		 * Check if any approval action (Approve/Reject) is done and password required
		 * 
		 * @returns isPasswordRequired {boolean}
		 */
		ContentStatement.prototype.isPasswordRequiredForApproval = function() {
			var isPasswordRequired = false;
			
			// If password is required for approval as set by Admin in preferences
			if(this.isApprovalPasswordRequired === 'true') {
				
				// then, check whether any approval action is done on any language
				_.each(this.element, _.bind(function (csElement) {
					if (!isPasswordRequired && csElement.doneApproveORReject){
						isPasswordRequired = true;
						return false;
					}
				}));
			}
			
			return isPasswordRequired;
		};

		return ContentStatement;
	});