/**
 * @author PASH
 */
define(["jquery",
        "underscore",
        "lib/medium/rangy-core",
        "lib/medium/rangy-classapplier",
        "lib/medium/undo",
        "utils/webcss!./ContentStatementElementEditPanelStyle.css"],
		function($, _, rangyCore, rangy, Undo){
	
	"use strict";
	
	/**
	 * Constructor to provide view of content statement edit panel
	 * @param content {String} - Text content string available in the document for current statement element
	 * @param isEditable {boolean} - user permission to allow editing for text content statement.
	 */
	var ContentStatementElementEditPanelView = function(content, isEditable) {
		this._content = content;
		this._isEditable = isEditable;
		
		// Create the wrapping div of the panel.
		this._wrapper = $("<div>", {"class" : "ContentStatementElementEditPanel"});
		
		// We're creating the actual editor and filling it with the current content.
		this._editor = $("<div>", {"class" : "EditArea"}).html(content);
		
		// if user is not allowed to edit than only show the content.
		if (this._isEditable) {
			// Creating Tool bar to provide styling to the text.
			this._toolBarDiv = $("<div>", {"class" : "EditToolbar"});
			
			// Tool bar options.
			this._toolBold = $("<div>", {"class" : "EditToolbarEntry"}).html('<b>B</b>');
			this._toolUnderline = $("<div>", {"class" : "EditToolbarEntry"}).html('<u>U</u>');
			this._toolItalic = $("<div>", {"class" : "EditToolbarEntry"}).html('<i>I</i>');
			this._toolLineThrough = $("<div>", {"class" : "EditToolbarEntry"}).html('<s>S</s>');
			this._toolSub = $("<div>", {"class" : "EditToolbarEntry"}).html('X<sub>2</sub>');
			this._toolSup = $("<div>", {"class" : "EditToolbarEntry"}).html('X<sup>2</sup>');
			
			this._toolBarDiv.append(this._toolBold);
			this._toolBarDiv.append(this._toolUnderline);
			this._toolBarDiv.append(this._toolItalic);
			this._toolBarDiv.append(this._toolLineThrough);
			this._toolBarDiv.append(this._toolSub);
			this._toolBarDiv.append(this._toolSup);
			
			this._wrapper.append(this._toolBarDiv);
			
			this._constructRichText();
		}
		
		this._wrapper.append(this._editor);
	};
	
	/**
	 * Method to prepare rich text editor for text content statement.
	 */
	ContentStatementElementEditPanelView.prototype._constructRichText = function () {
		
		window.rangy = rangy;
		window.Undo = Undo;
		
		require(["lib/medium/medium"], function(Medium) {
			
			var article = this._editor.get(0);
			
			this._medium = new Medium({
			    	element: article,
			    	modifier: 'auto',
			    	placeholder: '',
			    	autofocus: false,
			    	autoHR: false,
			    	mode: Medium.richMode,
			    	modifiers: {
			    		'b': 'bold',
			    		'u': 'underline',
			    		'i': 'italicize',
			    		's': 'strike',
			    		'v': 'paste',
			    		'd': 'sub',
			    		't': 'sup'
			    	},
			    	tags: {
			    		'break': 'br',
			    		'paragraph': 'p',
			    		'innerLevel': ['b', 'u', 'i', 'strike', 'sub', 'sup'],
			    		'outerLevel': ['b', 'u', 'i', 'strike', 'sub', 'sup']
			    	},
			        attributes: null,
				    pasteAsText: false,
			    });

			    // If text is not selected and styling is performed then styling will be applied to whole text.
				article.highlight = _.bind(function() {
					if (document.activeElement !== article) {
						this._medium.select();
					}
				}, this);
				
				this._registerToolBarEvents(this._medium, article.highlight);
				
		}.bind(this));
	   
	};
	
	ContentStatementElementEditPanelView.prototype.getJQElement = function () {
		return this._wrapper;
	};
	
	/**
	 * This method is to get latest text content, if user is not having editor permission then it returns existing content.
	 * @returns {String} - Text content as string. If editing is allowed then read value from editor else return actual value.
	 */
	ContentStatementElementEditPanelView.prototype.getCurrentContent = function () {
		var content = this._content;
		if (this._isEditable) {
			content = this._editor.html();
		}
		
		// return content without modification if contnet is empty.
		if ('' === content.trim()) {
			return content;
		}
		// Check for the content's root node.
		try {
			var contentXml = content,
			xmlDoc = $.parseXML( contentXml ),
			$xml = $( xmlDoc );
			$xml.find("*").eq(0)[0].nodeName;
		} catch (err) {
			// String is not xml, assigning root tags to make it string.
			content = '<p>' + content + '</p>';
		}
		
		return content;
	};
	
	/**
	 * Registering tool bar events for rich text editor.
	 * @param medium {Medium} - object of the medium editor to invoke styling on the text.
	 * @param highlight {function} - A function which helps to select full content in editor is current element is not editor.
	 */
	ContentStatementElementEditPanelView.prototype._registerToolBarEvents = function (medium, highlight) {
		this._toolBold.on("mousedown", function() {
			highlight();
			medium.invokeElement('b', {});
			return false;
		});

		this._toolUnderline.on("mousedown", function() {
			highlight();
			medium.invokeElement('u', {});
			return false;
		});

		this._toolItalic.on("mousedown",function() {
			highlight();
			medium.invokeElement('i', {});
			return false;
		});

		this._toolLineThrough.on("mousedown", function() {
			highlight();
			medium.invokeElement('strike', {});
			return false;
		});

		this._toolSub.on("mousedown",function() {
			highlight();
			medium.invokeElement('sub', {});
			return false;
		});

		this._toolSup.on("mousedown", function() {
			highlight();
			medium.invokeElement('sup', {});
			return false;
		});
	};
	
	return ContentStatementElementEditPanelView;
});