/**
 * @author KADA
 * 
 * 
 */

define(["jquery",
        "underscore",
        "./LanguageIndicatorEntry",
        "utils/webcss!./LanguageIndicatorStyle.css"],
		function($, _, LanguageIndicatorEntry){
	
	"use strict";
	
	var LanguageIndicatorView = function(options, isMaster) {
		
		// Remember the rendering options
		this._options = options;
		this._isMaster = isMaster;
		this._wrapper = $("<div>",{});
		
		// The arrays that keep track of the callbacks.
		this._onLanguageAddCallbacks = [];
		this._onLanguageRemoveCallbacks = [];
		
		// These will hold all the LanguageIndicatorEntries (e.g. "flags") for both sections.
		this._langEntries = [];
		
		// Boolean rememberering wether the current indicator is expanded or not.
		this._expanded = false;
		
		// Determine the number of max entries we're going to show.
		this._maxEntries = 5;
		if (this._options.showCompact) {
			this._maxEntries = 17;
		}
		
		// Create the section for the master languages.
		this._sectionFlags = $("<div>",{'class':'LanguageIndicator-Entries', 'style' : 'max-width: '+this._options.width});
		this._section = $("<div>",{'class':'LanguageIndicator-Section'});
		if (this._options.showCompact) {
			this._sectionFlags.addClass('LanguageIndicator-Entries-Compact');
		}
		if (this._options.showLabels) {
			this._sectionLabel = $("<div>",{'class':'LanguageIndicator-Label'}).text("Master:");
			this._section.append(this._sectionLabel);
		}
			
		this._section.append(this._sectionFlags);
		this._wrapper.append(this._section);
	};
	
	/**
	 * 
	 * @param availableLanguages
	 * @param unAvailableLanguages
	 */
	LanguageIndicatorView.prototype.setLanguages = function(availableLanguages, unavailableLanguages, disabledLanguage) {
		
		// Clear the current flag section and entries.
		this._sectionFlags.empty();
		this._section.empty();
		this._wrapper.empty();
		this._langEntries = [];
		
		// First add all the available languages
		for (var i=0; i <availableLanguages.length; i++) {
			// Find out if we should disable this entry
			var disableEntry = disabledLanguage && disabledLanguage.langCode == availableLanguages[i].langCode; 
			// Create the entry.
			var langEntry = new LanguageIndicatorEntry(availableLanguages[i], true, this._options.showCompact, disableEntry);
			// Add it to the array of all entries
			this._langEntries.push(langEntry);
		}
		
		// Now add all the remaining unavailable languages
		for (var i=0; i <unavailableLanguages.length; i++) {
			// Create the entry.
			var langEntry = new LanguageIndicatorEntry(unavailableLanguages[i], false, this._options.showCompact);
			// Add it to the array of all entries
			this._langEntries.push(langEntry);
		}
		
		// If this is not in read only mode ...
		if (!this._options.readOnly) {
			
			for (var i=0; i < this._langEntries.length; i++) {
				
				// Add the CSS classes to indicate that you can edit the languages.
				this._langEntries[i].setEditable();

				// Add a clickhandler to the entries (only if "editable" though).
				this._langEntries[i].onLanguageClicked(_.bind(function (entry, isMaster) {
					if (entry.isAvailable()) {
						_.each(this._onLanguageRemoveCallbacks, function (callback) {
							callback(entry.getLanguage(), isMaster);
						});
					} else {
						_.each(this._onLanguageAddCallbacks, function (callback) {
							callback(entry.getLanguage(), isMaster);
						});
					}
				}, this, this._langEntries[i], this._isMaster));
			}
		}

		
		// Add the entries to the holder.
		var numShownEntries = 0;
		for (var i=0; i < this._langEntries.length; i++) {
			
			if(this._langEntries[i].getEditable() || this._langEntries[i].isAvailable()) {
				this._sectionFlags.append(this._langEntries[i].getJQElement());
				numShownEntries++;
			}
			
			if (!this._expanded && numShownEntries >= this._maxEntries) {
				this._langEntries[i].getJQElement().hide();
			}
		}
		
		// If the list exceeds the max entries, add controls to collapse or expand it.
		if (numShownEntries >= this._maxEntries) {
			
			var expand = $("<div>",{'class':'LanguageIndicator-Expand'}).text("...");
			expand.on("click", _.bind(function() {
				this.expand(this._isMaster);
			},this));
			this._sectionFlags.append(expand);
			
			
			var collapse = $("<div>",{'class':'LanguageIndicator-Collapse'}).html("&#9650;");
			if (this._options.showCompact) {
				collapse.html("&#9664;");
			}
			collapse.on("click", _.bind(function() {
				this.collapse(this._isMaster);
			},this));
			this._sectionFlags.append(collapse);
			
			if (this._expanded) {
				expand.hide();
				collapse.show();
			} else {
				expand.show();
				collapse.hide();
			}
			
		}
		
		this._section.append(this._sectionFlags);
		this._wrapper.append(this._section);
	};
	
	LanguageIndicatorView.prototype.collapse = function (master) {
		
		// Set that this indicator is collapsed.
		this._expanded = false;
		
		// Determine on which entries we should operate.
		var expand = this._sectionFlags.find('.LanguageIndicator-Expand');
		var collapse = this._sectionFlags.find('.LanguageIndicator-Collapse');
		
		// Hide all the entries that exceed the maximum.
		var numShownEntries = 0;
		for (var i=0; i < this._langEntries.length; i++) {
			
			if(this._langEntries[i].getEditable() || this._langEntries[i].isAvailable()) {
				numShownEntries++;
				this._langEntries[i].getJQElement().show();
				
				if (numShownEntries >= this._maxEntries) {
					this._langEntries[i].getJQElement().hide();
				}
			}
		}
		
		expand.show();
		collapse.hide();
	};
	
	LanguageIndicatorView.prototype.expand = function (master) {
		
		// Set that this indicator is expanded.
		this._expanded = true;
		
		// Determine on which entries we should operate.
		var expand = this._sectionFlags.find('.LanguageIndicator-Expand');
		var collapse = this._sectionFlags.find('.LanguageIndicator-Collapse');
		
		
		// Show all the entries (except those that are unavailable and not editable).
		for (var i=0; i < this._langEntries.length; i++) {
			if(this._langEntries[i].getEditable() || this._langEntries[i].isAvailable()) {
				this._langEntries[i].getJQElement().show();
			}
		}
		
		expand.hide();
		collapse.show();
	};
	
	LanguageIndicatorView.prototype.onLanguageAddAction = function (callback) {
		this._onLanguageAddCallbacks.push(callback);
	};
	
	LanguageIndicatorView.prototype.onLanguageRemoveAction = function (callback) {
		this._onLanguageRemoveCallbacks.push(callback);
	};
	
	LanguageIndicatorView.prototype.getJQElement = function () {
		return this._wrapper;
	};
	
	
	return LanguageIndicatorView;
});