define(["underscore",
        "jquery",
        "./ContentStatementElementWidgetView",
        "./DocXContentStatementElementWidgetView",
        "./ViewerContentStatementElementWidgetView",
        "../text-content-services/languageService",
        "../text-content-services/elementTypeService",
        "../text-content-utils/LanguageUtils",
        "./ApprovalOverviewWidget",
        "utils/webi18n!TextContentStrings",
        "../ContentConstants"],
	function (_, 
			$, 
			ContentStatementElementWidgetView,
			DocXContentStatementElementWidgetView,
			ViewerContentStatementElementWidgetView,
			languageService,
			elementTypeService,
			LanguageUtils,
			ApprovalOverviewWidget,
			TextContentStrings,
			ContentConstants) {
		
		"use strict";

		/**
		 * Create a Widget in order to display and potentially edit the contents of a 'ContentStatementElement'.
		 * 
		 * @param {ContentStatementElement} contentStatementElement - The statement for this widget.
		 * @param {Object} options - An object containing some variables specifying how the widget should behave.
		 * 
		 */
		var ContentStatementElementWidget = function (contentStatementElement, options) {
			
			this._textElement = contentStatementElement;
			this._elementTypeCode = contentStatementElement.parentStatement.elementType;
			this._type = contentStatementElement.parentStatement.type;
			this._options = options;
			
			// Extract the fields from the contentStatementElementWidget that are needed for the view.
			
			var content = this._textElement.content;			
			var languageCode = this._textElement.language;
			var comments = this._textElement.comment;
			var state = this._textElement.state;
			var canEdit = this._textElement.isEditable === 'true'; // Want this to be a boolean.
			var whyCanNotEdit = this._textElement.whyNonEditable;
			var canReview = this._textElement.isReviewable  === 'true'; // Want this to be a boolean.
			var whyCanNotReview = this._textElement.whyNonReviewable;
			var isMaster = this._textElement.master === 'true';
			var isContentManagerView = this._textElement.parentStatement.contentManagerView === 'true';
			var myApprovalStatus = this._textElement.myApprovalStatus;
			
			var markedApprovalState = state;
			if(this._textElement.approvalChanged) {
				markedApprovalState = this._textElement.approval[0].state;
			}

			// Extract the 'viewOptions' from the passed options.
			var viewOptions = {};
			if (this._options && typeof this._options.viewOptions === 'object') {
				viewOptions = this._options.viewOptions;
			}
			
			// callbacks to be executed when any changes happen in the widget
			this._onChangeCallbacks = [];
			
			// create the view of the widget based on the viewOptions
			switch(viewOptions.widgetview) {
			case 'default':
				this._view = new ContentStatementElementWidgetView(content, isMaster, comments, state, languageCode, canEdit, whyCanNotEdit, canReview, whyCanNotReview, isContentManagerView, myApprovalStatus, viewOptions, markedApprovalState, this._type);
				this._updateView(isMaster, isContentManagerView);
				break;
			case 'doctablex':
				this._view = new DocXContentStatementElementWidgetView(content, isMaster, comments, state, languageCode, canEdit, whyCanNotEdit, canReview, whyCanNotReview, isContentManagerView, myApprovalStatus, viewOptions, markedApprovalState, this._type);
				break;
			case 'viewer':
				this._view = new ViewerContentStatementElementWidgetView(content, isMaster, comments, state, languageCode, canEdit, whyCanNotEdit, canReview, whyCanNotReview, isContentManagerView, myApprovalStatus, viewOptions, markedApprovalState, this._type);
				this._setLanguageHeader(isMaster, isContentManagerView);
				break;
			default:
				break;
			}

			// Specify what should happen on Approval actions
			this._view.onApprove(_.bind(function () {
				this._textElement.approval.push({"state": ContentConstants.State.APPROVED});
				this._textElement.approvalChanged = true;
				this._textElement.doneApproveORReject = true;
				this._executeOnChange(ContentConstants.OnChangeCallbackType.APPROVED);
			}, this));
			
			this._view.onReject(_.bind(function () {
				this._textElement.approval.push({"state": ContentConstants.State.REJECTED});
				this._textElement.approvalChanged = true;
				this._textElement.doneApproveORReject = true;
				this._executeOnChange(ContentConstants.OnChangeCallbackType.REJECTED);
			}, this));
			
			this._view.onStartApproval(_.bind(function () {
				this._textElement.approval.push({"state": ContentConstants.State.READY_FOR_APPROVAL});
				this._textElement.approvalChanged = true;
				this._executeOnChange(ContentConstants.OnChangeCallbackType.STARTED_APPROVAL);
			}, this));
			
			this._view.onRevertToDraft(_.bind(function () {
				this._textElement.approval.push({"state": ContentConstants.State.DRAFT});
				this._textElement.approvalChanged = true;
				this._executeOnChange(ContentConstants.OnChangeCallbackType.REVERTED_TO_DRAFT);
			}, this));
			
			// Specify what should happen when the approval action is reverted
			this._view.onRevertApproval(_.bind(function() {
				this._textElement.cancelStatusChange();
				this._executeOnChange(ContentConstants.OnChangeCallbackType.APPROVAL_REVERTED);
			}, this));
			
			// Specify what should happen when a comment is added/edited
			this._view.onComment(_.bind(function(commentText) {
				if($.trim(commentText)) {
					var newComment = {
							content : commentText,
							ts : new Date(),
							user : "you",
							isNew : true
							// add a variable to differentiate between normal and reject comment, if required
					};
					
					// If new comment exists update it, add a new comment otherwise
					if(this._textElement.commentChanged) {
						for(var i=0 ; i < this._textElement.comment.length; i++) {
							if(this._textElement.comment[i].isNew) {
								this._textElement.comment[i] = newComment;
								break;
							}
						}
					} else {
						this._textElement.comment.push(newComment);
						this._textElement.commentChanged = true;
					}
					
					this._executeOnChange(ContentConstants.OnChangeCallbackType.COMMENTED);
				}
			}, this));
			
			// What needs to happen when the approval info button is clicked
			this._view.onApprovalInfo(_.bind(function () {
				// Create a new approval overview widget.
				this._aprovalOverviewWidget = new ApprovalOverviewWidget(this._textElement);
				this._view.showApprovalInfoDialog(this._aprovalOverviewWidget.getJQElement(), this._options);
			}, this));
			
			// What needs to happen when content is changed in editor.
			this._view.onMainContent(_.bind(function (content) {
				var currentContent = {'content': content};
				this.updateCurrentContent(currentContent);
			}, this));
			

		};
		
		ContentStatementElementWidget.prototype.updateCurrentContent = function (currentContent) {
			// Update should be done only if content is actually changed
			if (currentContent.content != this._textElement.content.content) {
				this._textElement.content = currentContent;
				this._textElement.contentChanged = true;
				this._executeOnChange(ContentConstants.OnChangeCallbackType.TEXT_UPDATED);
			}
		};
		
		/**
		 * Make sure the widget is updated with the latest state of the passed contentStatementElement.
		 */
		ContentStatementElementWidget.prototype.update = function (contentStatementElement) {
			this._textElement = contentStatementElement;
			
			// Extract the fields from the contentStatementElementWidget that are needed for the view.
			var content = contentStatementElement.content;
			var languageCode = contentStatementElement.language;
			var comments = contentStatementElement.comment;
			var state = contentStatementElement.state;
			var canEdit = contentStatementElement.isEditable === 'true'; // Want this to be a boolean.
			var whyCanNotEdit = contentStatementElement.whyNonEditable;
			var canReview = contentStatementElement.isReviewable  === 'true'; // Want this to be a boolean.
			var whyCanNotReview = contentStatementElement.whyNonReviewable;
			var isMaster = contentStatementElement.master === 'true';
			var isContentManagerView = contentStatementElement.parentStatement.contentManagerView === 'true';
			var myApprovalStatus = contentStatementElement.myApprovalStatus;
			var type = contentStatementElement.parentStatement.type;
			
			var markedApprovalState = state;
			if(this._textElement.approvalChanged) {
				markedApprovalState = this._textElement.approval[0].state;
			}
			
			this._view.updateWidgetView(content, isMaster, comments, state, languageCode, canEdit, whyCanNotEdit, canReview, whyCanNotReview, isContentManagerView, myApprovalStatus, markedApprovalState, type);
		};
		
		/**
		 * To update the language in header for default view and add a delete icon for non-project languages
		 * @param isMaster
		 * @param isContentManagerView
		 */
		ContentStatementElementWidget.prototype._updateView = function (isMaster, isContentManagerView) {
			this._addDeleteOption(isMaster, isContentManagerView);
			this._setLanguageHeader(isMaster, isContentManagerView);
		};
		
		/**
		 * Adds a delete button in widget when the language is not a project attribute language
		 * 
		 * @param isMaster
		 * @param isContentManagerView
		 */
		ContentStatementElementWidget.prototype._addDeleteOption = function(isMaster, isContentManagerView) {
			// Create delete button if the language is not present in project attribute and user has content manager permission
			var projId = this._textElement.parentStatement.projectId;
			if(isContentManagerView) {
				languageService.getProjectLanguages(projId, isMaster, _.bind(function(langs) {
					var isProjectLang = LanguageUtils.isAProjectAttrLang(langs, this._textElement.language);
					if(!isProjectLang) {
						var revertDeleteHandler = _.bind(function() {
							this._textElement.deleted = false;
						}, this);
						this._view.createDeleteButton(revertDeleteHandler);
						
						// Specify what should happen on Delete action
						this._view.onDelete(_.bind(function () {
							this._textElement.deleted = true;
							this._executeOnChange(ContentConstants.OnChangeCallbackType.DELETED);
						}, this));
					}
				}, this));
			}
		};
		
		/**
		 * Update the language header of the widget
		 * 
		 * @param isMaster
		 * @param isContentManagerView
		 */
		ContentStatementElementWidget.prototype._setLanguageHeader = function(isMaster, isContentManagerView) {
			// Retrieve the languages for this text element.
			var projId = this._textElement.parentStatement.projectId;
			languageService.getAllLanguages(projId, isMaster, function (allLangs) {
				this._view.setLanguage(LanguageUtils.getLangForCode(allLangs, this._textElement.language));
			}.bind(this));
		};
		
		/**
		 * Execute the onchange callback when anything changes in the widget
		 */
		ContentStatementElementWidget.prototype._executeOnChange = function (changetype) {
			_.each(this._onChangeCallbacks, _.bind(function(changetype, callback) {
				callback(changetype, this._textElement);
			}, this, changetype));
		};
		
		/**
		 * Add callback to be executed when any changes is done in the widget
		 */
		ContentStatementElementWidget.prototype.onChange = function (callback) {
			this._onChangeCallbacks.push(callback);
		};
		
		/**
		 * Focus/Blur the widget.
		 */
		ContentStatementElementWidget.prototype.focus = function (focus) {
			this._view.focus(focus);
		};
		
		/**
		 * Disable the widget.
		 */
		ContentStatementElementWidget.prototype.disable = function () {
			this._view.enable(false);
		};
		
		/**
		 * Enable the widget.
		 */
		ContentStatementElementWidget.prototype.enable = function () {
			this._view.enable(true);
		};
		
		/**
		 * Get the current text element
		 */
		ContentStatementElementWidget.prototype.getContentStatementElement = function () {
			return this._textElement;
		};
		
		/**
		 * Return the jQuery HTML element of the widget
		 * @returns
		 */
		ContentStatementElementWidget.prototype.getJQElement = function () {
			return this._view.getJQElement();
		};
		
		// Return the constructor.
		return ContentStatementElementWidget;
		
});
