define(["underscore",
        "jquery",
        "utils/webi18n!TextContentStrings",
        "utils/ConvertUtils",
        "../ContentConstants",
        "component/modal/Modal",
        "../../component/tooltip/Tooltip",
        "./RichTextEditor",
        "utils/webcss!./ContentStatementElementWidgetView.css"],
	function (_,
			$,
			TextContentStrings,
			ConvertUtils,
			ContentConstants,
			Modal,
			Tooltip,
			RichTextEditor) {
	
		"use strict";

		/**
		 * Create the default ContentStatementElementWidgetView.
		 * 
		 * @param {Object} content - The content object from the ContentStatementElementWidget represented by this widget.
		 * @param {Boolean} isMaster - Flag which specifies whether the language is a Master or Translation
		 * @param {Array} comments - The array of Comment objects from the ContentStatementElementWidget represented by this widget.
		 * @param {String} state - The state of the ContentStatementElementWidget represented by this widget.
		 * @param {String} language - The language object of the ContentStatementElementWidget represented by this widget.
		 * @param {Boolean} canEdit - Whether or not the ContentStatementElementWidget represented by this widget can be edited by the current user.
		 * @param {Boolean} whyCanNotEdit - Why the ContentStatementElementWidget represented by this widget can't bed edited by the current user.
		 * @param {Boolean} canReview - Whether or not the ContentStatementElementWidget represented by this widget can be reviewed by the current user.
		 * @param {Boolean} whyCanNotReview - Why the ContentStatementElementWidget represented by this widget can't be reviewed by the current user.
		 * @param {Boolean} isContentManagerView - Specifies whether the current user is a Content_Manager
		 * @param {String} myApprovalStatus - Approval status of TextContent element for a Approver
		 * @param {Object} [viewOptions] - An object containing some specific variables indicating how the widget will be rendered.
		 * @param {String} markedApprovalState - Approval action taken if any, Current status otherwise
		 * @param {String} type - text type as "richtext" or "simple".
		 */
		var ContentStatementElementWidgetView = function (content, isMaster, comments, state, languageCode, canEdit, whyCanNotEdit, canReview, whyCanNotReview, isContentManagerView, myApprovalStatus, viewOptions, markedApprovalState, type) {
			
			this._content = content;
			if (!this._content.content) {
				this._content.content = "";
			}
			this._isMaster = isMaster;
			this._comments = comments;
			this._state = state;
			this._langaugeCode = languageCode;
			this._canEdit = canEdit;
			this._whyCanNotEdit = whyCanNotEdit;
			this._canReview = canReview;
			this._whyCanNotReview = whyCanNotReview;
			this._isContentManagerView = isContentManagerView;
			this._myApprovalStatus = myApprovalStatus;
			this._type = type;
			
			// store the updated changes for approval
			this._markedApprovalState = markedApprovalState;
			
			// Let the viewOptions override the default, extend one these default values.
			this._viewOptions = {
				// Some configuration might override this variable to disable the editing capabilities of this widget.
				editable: true,
				// Some configuration might override this variable to disable the review capabilities of this widget.
				reviewable: true 
			};
			$.extend(this._viewOptions, viewOptions);
			
			// Create the callback arrays
			this._onApproveCallbacks = [];
			this._onRejectCallbacks = [];
			this._onStartApprovalCallbacks = [];
			this._onRevertToDraftCallbacks = [];
			this._onDeleteCallbacks = [];
			this._onCommentCallbacks = [];
			this._onRevertApprovalCallbacks = [];
			this._onApprovalInfoCallbacks = [];
			this._onMainContentCallbacks = [];
			
			// Create the common HTML elements for the view.
			this._approveButton = null;
			this._rejectButton = null;
			this._startApprovalButton = null;
			this._revertToDraftButton = null;
			this._deleteButton = null;
			this._richTextEditor = null;
			this._plainTextEditor = null;
			this._commentTextArea = null;
			this._commentButton = null;
			this._approvalInfoButton = null;
			this._revertApprovalButton = null;
			
			// The height of the content.
			this._contentHeight = 85;
			// Wether to show popovers on why the user does not have certain rights. 
			this._showTooltips = true;
			
			// Create the wrapping div of this widget.
			this._wrapper = $("<div>").addClass("ContentStatementElementWidget");
			
			if(this._viewOptions.widgetview === 'default') {
				// Start constructing the default view
				this._constructView();
			} else {
				return;
			}
			
		};
		
		/**
		 * Construct the HTML elements of this view.
		 */
		ContentStatementElementWidgetView.prototype._constructView = function () {
			this._wrapper.empty();
			
			this._initializeDOMElements();
			
			// create widget elements and append to the wrapper
			this._wrapper.append(
					this._constructApprovalDiv(),
					$('<div>').addClass("TextContent-ContentCol").append(
							$('<div>').addClass("TextContent-InnerContentCol").append(
									this._constructHeaderDiv(),
									this._constructTextDiv()
							)
					),
					this._constructCommentsDiv()
			);
		};
		
		/**
		 * Provides a dialog for Default view to show approval info
		 * @param approvalInfoContent {JQuery} - Approval information as jquery elements
		 */
		ContentStatementElementWidgetView.prototype.showApprovalInfoDialog = function (approvalInfoContent) {
			// Condition check to avoid creating multiple approval info modal
			if (!this._approvalInfoModal) {
				this._approvalInfoModal = new Modal(approvalInfoContent, {"customClass" : "ApprovalOverviewWidgetModal"});

				this._approvalInfoModal.setTitle(TextContentStrings.t("PCM_APPROVAL_INFO_POPUP_TITLE"));
			} else {
				this._approvalInfoModal.setContent(approvalInfoContent);
			}
			this._approvalInfoModal.open();
		};
		
		/**
		 * Method to update the widget and repaint
		 * 
		 * @param text
		 * @param isMaster
		 * @param comments
		 * @param state
		 * @param languageCode
		 * @param canEdit
		 * @param whyCanNotEdit
		 * @param canReview
		 * @param whyCanNotReview
		 * @param isContentManagerView
		 * @param myApprovalStatus
		 * @param markedApprovalState
		 * @param type
		 */
		ContentStatementElementWidgetView.prototype.updateWidgetView = function(content, isMaster, comments, state, languageCode, canEdit, whyCanNotEdit, canReview, whyCanNotReview, isContentManagerView, myApprovalStatus, markedApprovalState, type) {
			this._content = content;
			this._isMaster = isMaster;
			this._comments = comments;
			this._state = state;
			this._langaugeCode = languageCode;
			this._canEdit = canEdit;
			this._whyCanNotEdit = whyCanNotEdit;
			this._canReview = canReview;
			this._whyCanNotReview = whyCanNotReview;
			this._isContentManagerView = isContentManagerView;
			this._myApprovalStatus = myApprovalStatus;
			this._markedApprovalState = markedApprovalState;
			this._type = type;
			
			this._constructView();
		};
		
		/**
		 * Initialize the DOM elements to repaint the widget
		 */
		ContentStatementElementWidgetView.prototype._initializeDOMElements = function() {
			// create the DOM elements
			this._approveButton = $('<div>').attr({"name": "approveButton"});
			this._rejectButton = $('<div>').attr({"name": "rejectButton"});
			this._startApprovalButton = $('<div>').attr({"name": "startApprovalButton"});
			this._revertToDraftButton = $('<div>').attr({"name": "revertToDraftButton"});
			this._deleteButton = $('<div>').attr({"name": "deleteButton"});

			// Create the editors based on text type
			if (ContentConstants.TextType.RICHTEXT === this._type) {
				// Create a rich text editor.
				this._richTextEditor = new RichTextEditor(this._content.content, this._contentHeight);
				this._richTextEditor.getJQElement().addClass("ContentArea");
				
				// Register content change event in rich text editor
				this._richTextEditor.onChange(_.bind(function () {
					var richText = this._richTextEditor.getContent();
					this._executeContentChange(richText);
				}, this));
			} else {
				// Create a plain text editor (e.g. a normal "textarea").
				this._plainTextEditor = $('<textarea>').text(this._content.content);
				this._plainTextEditor.addClass("ContentArea");
				// Register content change event in simple text editor
				this._plainTextEditor.on("input", _.bind(function () {
					var plainText = (this._plainTextEditor.val());
					this._executeContentChange(plainText);
				}, this));
			}
			
			this._commentTextArea = $('<textarea>').attr({"name": "addCommentTextArea"});
			this._commentButton = $('<div>').attr({"name": "commentButton"});
			this._approvalInfoButton = $('<div>').attr({"name": "approvalInfoButton"});
			this._revertApprovalButton = $('<div>').attr({"name": "revertApprovalButton"});
			
			// Bind events to execute callbacks for each action
			this._approveEventHandler = _.bind(function(event) {
				_.each(this._onApproveCallbacks, _.bind(function(callback) {
					callback();
				}, this));
			}, this);
			this._approveButton.click(this._approveEventHandler);
			
			this._rejectEventHandler = _.bind(function(event) {
				_.each(this._onRejectCallbacks, _.bind(function(callback) {
					callback();
				}, this));
			}, this);
			this._rejectButton.click(this._rejectEventHandler);
			
			this._startApprovalEventHandler = _.bind(function(event) {
				_.each(this._onStartApprovalCallbacks, _.bind(function(callback) {
					callback();
				}, this));
			}, this);
			this._startApprovalButton.click(this._startApprovalEventHandler);
			
			this._revertToDraftEventHandler = _.bind(function(event) {
				_.each(this._onRevertToDraftCallbacks, _.bind(function(callback) {
					callback();
				}, this));
			}, this);
			this._revertToDraftButton.click(this._revertToDraftEventHandler);
			
			this._deleteButton.click(_.bind(function(event) {
				_.each(this._onDeleteCallbacks, _.bind(function(callback) {
					callback();
				}, this));
			}, this));
			
			this._commentTextArea.on("input propertychange paste", _.bind(function(event) {
				var commentText = $.trim($(event.target).val());
				_.each(this._onCommentCallbacks, _.bind(function(commentText, callback) {
					callback(commentText);
				}, this, commentText));
			}, this));
			
			this._revertApprovalEventHandler = _.bind(function(event) {
				_.each(this._onRevertApprovalCallbacks, _.bind(function(callback) {
					callback();
				}, this));
			}, this);
			this._revertApprovalButton.click(this._revertApprovalEventHandler);
		};
		
		/**
		 * Method is to update content if its actually changed.
		 * @param currentContent {String} - content from content area
		 */
		ContentStatementElementWidgetView.prototype._executeContentChange = function(currentContent) {
			// Avoid update if content is not modified
			if (currentContent !== this._content.content) {
				_.each(this._onMainContentCallbacks, _.bind(function(currentContent, callback) {
					callback(currentContent);
				}, this, currentContent));
				
				//Ok what are we doing this? Current status is Rejected and user starts typing in text area, automatically the status of the
				//Text Element should change to Draft
				//Important Note: User doesn't get revert option in this case
				if(this._state === ContentConstants.State.REJECTED){
					this._markedApprovalState = ContentConstants.State.DRAFT;
					$("div.TextContent-ApprovalCol", this._wrapper).empty().append(this._constructElementStatus());
				}
			}
		};
		
		/**
		 * Set the language indicator of the view.
		 * 
		 * @param language - language object {langCode, langDesc, langFlag}
		 */
		ContentStatementElementWidgetView.prototype.setLanguage = function (language) {
			var languageDescDiv = this._wrapper.find(".TextContent-TitleRow div[name='languageDescDiv']");
			
			languageDescDiv.addClass("TextContent-Title");
			
			var languageLabel = null;
			if(language && language.langDesc) {
				languageLabel = language.langDesc;
			} else if(language && language.langCode) {
				languageLabel = language.langCode;
			} else {
				languageLabel = this._langaugeCode;
			}
			
			if(this._isMaster) {
				languageDescDiv.append($('<b>').text(languageLabel));
			} else {
				languageDescDiv.append($('<span>').text(languageLabel));
			}
			
			if(language && language.langFlag) {
				languageDescDiv.prepend($('<img>').attr({"src": "images/country_flags/" + language.langFlag}));
			}
			
		};
		
		/**
		 * Create delete button and append to the header
		 * 
		 * @param revertDeleteHandler - click hander for revert button when delete action is performed
		 */
		ContentStatementElementWidgetView.prototype.createDeleteButton = function(revertDeleteHandler) {
			this._deleteButton.addClass("TextContent-Action").attr({"title": TextContentStrings.t("PCM_DELETE_LANGUAGE")}).append($('<i>').addClass("icon icon-WCR_Trash"));
			
			this._changeToDeleteState = function() {
				var deleteStatus = $('<div>').attr({"name": "TextContent-StatusDiv"}).css({'opacity': '1'});
				
				// Display the language status
				deleteStatus.addClass("TextElementStatus_Delete").append(
						$('<div>').addClass("TextContent-ApprovalStatus").append(
								$('<div>').addClass("TextContent-StatusIcon").append($('<i>').addClass("icon icon-WCR_Trash").addClass("DeleteIconAndLabel")),
								$('<div>').addClass("TextContent-StatusLabel DeleteIconAndLabel").text(TextContentStrings.t("PCM_DELETE_LANGUAGE"))
						)
				);
				
				return deleteStatus;
			};
			
			
			this._deleteButton.click(_.bind(function() {
				this._deleteButton.hide();
				
				this._createApprovalDivHandlers();
				$("div.TextContent-ApprovalCol", this._wrapper).empty().append(this._changeToDeleteState()).append(this._constructRevertApprovalAction());
				
				// unbind all events on revert approval button and bind necessary click event
				this._revertApprovalButton.unbind();
				this._revertApprovalButton.click(_.bind(function() {
					this._deleteButton.show();
					$("div.TextContent-ApprovalCol", this._wrapper).empty().append(this._constructElementStatus());
					this._revertApprovalButton = $('<div>').click(this._revertApprovalEventHandler);
				}, this));
				
				// bind the revert approval action handler
				this._revertApprovalButton.click(revertDeleteHandler);
				
			}, this));
			
			// Append the delete button to the header
			this._deleteButton.insertBefore(this._wrapper.find(".TextContent-TitleRow:first div:last"));
		};
		
		/**
		 * Construct the approval column of the widget
		 * @returns approvalDiv - approval column container
		 */
		ContentStatementElementWidgetView.prototype._constructApprovalDiv = function () {
			// Container for Approval column
			var approvalDiv = $('<div>').addClass("TextContent-ApprovalCol");
			
			// add element status
			approvalDiv.append(this._constructElementStatus());
			
			if((this._canEdit && this._state === ContentConstants.State.DRAFT) || 
					((this._canReview || this._canEdit) && this._state === ContentConstants.State.READY_FOR_APPROVAL) || 
					(this._canEdit && this._state === ContentConstants.State.REJECTED)) {
				// create the hover and mouseleave listeners for the actions div
				this._createApprovalDivHandlers();
				
				// If no approval action is taken add approval buttons, add revert button otherwise
				if(this._state === this._markedApprovalState) {
					approvalDiv.append(this._constructApprovalActions());
				} else {
					approvalDiv.append(this._constructRevertApprovalAction());
				}
			} else {
				$("div[name='TextContent-StatusDiv']", approvalDiv).css({'cursor': 'default'});
			}
			
			return approvalDiv;
		};
		
		/**
		 * Creates hover and mouseleave event listeners for the actions
		 */
		ContentStatementElementWidgetView.prototype._createApprovalDivHandlers = function() {
			// create a handler when mouseover element status to view approval actions
			if(!this._actionsDivHandlerIn) {
				this._actionsDivHandlerIn = _.bind(function () {
					this._wrapper.find("div[name='TextContent-StatusDiv']").stop();
					this._wrapper.find("div[name='TextContent-ActionsDiv']").stop();
					this._wrapper.find("div[name='TextContent-StatusDiv']").animate({opacity: 0}, 400);
					this._wrapper.find("div[name='TextContent-ActionsDiv']").animate({opacity: 1}, 400);
				}, this);
			}
			// create a handler when mouseout element status to hide approval actions
			if(!this._actionsDivHandlerOut) {
				this._actionsDivHandlerOut = _.bind(function () {
					this._wrapper.find("div[name='TextContent-StatusDiv']").stop();
					this._wrapper.find("div[name='TextContent-ActionsDiv']").stop();
					this._wrapper.find("div[name='TextContent-ActionsDiv']").animate({opacity: 0}, 400);
					this._wrapper.find("div[name='TextContent-StatusDiv']").animate({opacity: 1}, 400);	
				}, this);
			}
		};
		
		/**
		 * Create TextContent status
		 * 
		 * @returns elementStatusDiv - jQuery element for TextContent status
		 */
		ContentStatementElementWidgetView.prototype._constructElementStatus = function () {
			var elementStatusDiv = $('<div>').attr({"name": "TextContent-StatusDiv"}).css({'opacity': '1'});
			
			if(!this._canEdit && !this._canReview) {
				elementStatusDiv.css({'cursor': 'default'});
			} else {
				elementStatusDiv.css({'cursor': 'pointer'});
				// On mouseleave create the approval actions div after a approval action is taken
				// unbind this event after mouseleave!
				
				/* 
				 * This event shouldn't trigger when the widget is created (creating issue in Firefox browser)!
				 * During widget creation the approval actions are readily available 
				 * But when an approval action is taken the user should move the mouse out of status and move in to get the revert approval button (should be same conversely)
				 */
				
				// TODO: Need a better fix. see jira bug id GRCCOMMONS-2749 (fixing it now by clearing the icons)
				
				elementStatusDiv.mouseleave(_.bind(function() {
					if(this._markedApprovalState === this._state) {
						$("div.TextContent-ApprovalCol", this._wrapper).append(this._constructApprovalActions());
					} else if(this._state !== ContentConstants.State.REJECTED){
						//Dont construct Revert option once the state is changed to DRAFT from rejected mode
						$("div.TextContent-ApprovalCol", this._wrapper).append(this._constructRevertApprovalAction());
					}
					this._wrapper.find('div[name="TextContent-StatusDiv"]').unbind();
				}, this));
			}
			
			var elementStatusIconClass = null;
			var elementStatusClass = null;
			var elementStatusLabelText = null;
			var elementStatusLabelClass = null;
			if(this._state === this._markedApprovalState) {
				switch(this._state) {
				case ContentConstants.State.DRAFT:
					elementStatusIconClass = "icon icon-WCR_Text-Status-Draft ElementStatusIconAndLabel";
					elementStatusClass = "TextContentElement-Status ElementStatusDraft";
					elementStatusLabelClass = "ElementStatusIconAndLabel";
					break;
				case ContentConstants.State.READY_FOR_APPROVAL:
					if(this._canReview && this._myApprovalStatus === ContentConstants.State.READY_FOR_APPROVAL) {
						elementStatusIconClass = "icon icon-WCR_Text-Status-Pending-You ElementStatusIconAndLabel";
						elementStatusLabelClass = "ElementStatusIconAndLabel";
					} else if(this._myApprovalStatus === ContentConstants.State.APPROVED) {
						elementStatusIconClass = "icon icon-WCR_Text-Status-Pending-YouApproved ElementStatusIconAndLabel";
						elementStatusLabelClass = "ElementStatusIconAndLabel";
					} else if(this._myApprovalStatus === ContentConstants.State.REJECTED) {
						elementStatusIconClass = "icon icon-WCR_Text-Status-Pending-YouRejected ElementStatusIconAndLabel";
						elementStatusLabelClass = "ElementStatusIconAndLabel";
					} else {
						elementStatusIconClass = "icon icon-WCR_Text-Status-Pending ElementStatusIconAndLabel";
						elementStatusLabelClass = "ElementStatusIconAndLabel";
					}
					elementStatusClass = "TextContentElement-Status ElementStatusPending";
					break;
				case ContentConstants.State.APPROVED:
					elementStatusIconClass = "icon icon-WCR_Text-Status-Approved ElementStatusIconAndLabel";
					elementStatusClass = "TextContentElement-Status ElementStatusApproved";
					elementStatusLabelClass = "ElementStatusIconAndLabel";
					break;
				case ContentConstants.State.REJECTED:
					elementStatusIconClass = "icon icon-WCR_Text-Status-Rejected ElementStatusIconAndLabel";
					elementStatusClass = "TextContentElement-Status ElementStatusRejected";
					elementStatusLabelClass = "ElementStatusIconAndLabel";
					break;
				default :
					elementStatusIconClass = "icon icon-WCR_Text-Status-NA ElementStatusIconAndLabel";
					elementStatusClass = "TextContentElement-Status";
					elementStatusLabelClass = "ElementStatusIconAndLabel";
					break;	
				}
				
				elementStatusLabelText = this._getLocalizedContentState(this._state, this._myApprovalStatus);
				
			} else {
				switch(this._markedApprovalState) {
				case ContentConstants.State.DRAFT:
					elementStatusIconClass = "icon icon-WCR_Text-Action-Draft RevertTextToDraftIconAndLabel";
					elementStatusClass = "TextContentElement-Status RevertTextToDraft";
					elementStatusLabelClass = "RevertTextToDraftIconAndLabel";
					break;
				case ContentConstants.State.READY_FOR_APPROVAL:
					elementStatusIconClass = "icon icon-WCR_Text-Action-Start StartTextApprovalIconAndLabel";
					elementStatusClass = "TextContentElement-Status StartTextApproval";
					elementStatusLabelClass = "StartTextApprovalIconAndLabel";
					break;
				case ContentConstants.State.APPROVED:
					elementStatusIconClass = "icon icon-WCR_Text-Action-Approve ApproveTextIconAndLabel";
					elementStatusClass = "TextContentElement-Status ApproveText";
					elementStatusLabelClass = "ApproveTextIconAndLabel";
					break;
				case ContentConstants.State.REJECTED:
					elementStatusIconClass = "icon icon-WCR_Text-Action-Reject RejectTextIconAndLabel";
					elementStatusClass = "TextContentElement-Status RejectText";
					elementStatusLabelClass = "RejectTextIconAndLabel";
					break;
				default :
					break;
				}
				
				elementStatusLabelText = this._getLocalizedContentAction(this._markedApprovalState);
			}
			
			
			// Display the language status
			elementStatusDiv.removeClass().addClass(elementStatusClass).append(
					$('<div>').addClass("TextContent-ApprovalStatus").append(
							$('<div>').addClass("TextContent-StatusIcon").append($('<i>').addClass(elementStatusIconClass)),
							$('<div>').addClass("TextContent-StatusLabel").addClass(elementStatusLabelClass).text(elementStatusLabelText)
					)
			);
			
			return elementStatusDiv;
		};
		
		/**
		 * Create Approval action buttons
		 * 
		 * @returns approvalActionsDiv - jQuery element for approval action buttons
		 */
		ContentStatementElementWidgetView.prototype._constructApprovalActions = function () {
			var approvalActionsDiv = $('<div>').attr({"name": "TextContent-ActionsDiv"}).addClass("TextContent-Approval-NextStates").css({'opacity': '0'});
			
			// Create StartApproval button
			this._startApprovalButton.empty().addClass("TextContent-ApprovalAction StartTextApproval").append($('<div>').addClass("ApprovalAction-InnerDiv").append(
					$('<div>').addClass("TextContent-ApprovalSymbol").append($('<i>').addClass("icon icon-WCR_Text-Action-Start").addClass("StartTextApprovalIconAndLabel")),
					$('<div>').addClass("TextContent-ApprovalLabel StartTextApprovalIconAndLabel").text(this._getLocalizedContentAction(ContentConstants.State.READY_FOR_APPROVAL))
					)
			);
			
			// Create RevertToDraft button
			this._revertToDraftButton.empty().addClass("TextContent-ApprovalAction RevertTextToDraft").append($('<div>').addClass("ApprovalAction-InnerDiv").append(
					$('<div>').addClass("TextContent-ApprovalSymbol").append($('<i>').addClass("icon icon-WCR_Text-Action-Draft").addClass("RevertTextToDraftIconAndLabel")),
					$('<div>').addClass("TextContent-ApprovalLabel RevertTextToDraftIconAndLabel").text(this._getLocalizedContentAction(ContentConstants.State.DRAFT))
					)
			);
			
			// Create Approve button
			this._approveButton.empty().addClass("TextContent-ApprovalAction ApproveText").append($('<div>').addClass("ApprovalAction-InnerDiv").append(
					$('<div>').addClass("TextContent-ApprovalSymbol").append($('<i>').addClass("icon icon-WCR_Text-Action-Approve").addClass("ApproveTextIconAndLabel")),
					$('<div>').addClass("TextContent-ApprovalLabel ApproveTextIconAndLabel").text(this._getLocalizedContentAction(ContentConstants.State.APPROVED))
					)
			);
			
			// Create Reject button
			this._rejectButton.empty().addClass("TextContent-ApprovalAction RejectText").append($('<div>').addClass("ApprovalAction-InnerDiv").append(
					$('<div>').addClass("TextContent-ApprovalSymbol").append($('<i>').addClass("icon icon-WCR_Text-Action-Reject").addClass("RejectTextIconAndLabel")),
					$('<div>').addClass("TextContent-ApprovalLabel RejectTextIconAndLabel").text(this._getLocalizedContentAction(ContentConstants.State.REJECTED))
					)
			);
			
			
			// Bind the click events to each approval action button
			this._actionsCallback = _.bind(function() {
				this._revertApprovalButton = $('<div>').click(this._revertApprovalEventHandler);
				$("div.TextContent-ApprovalCol", this._wrapper).empty().append(this._constructElementStatus());
			}, this);
			
			this._startApprovalButton.click(_.bind(function(event) {
				this._markedApprovalState = ContentConstants.State.READY_FOR_APPROVAL;
				this._actionsCallback();
			}, this));
			
			this._revertToDraftButton.click(_.bind(function(event) {
				this._markedApprovalState = ContentConstants.State.DRAFT;
				this._actionsCallback();
			}, this));
			
			this._approveButton.click(_.bind(function(event) {
				this._markedApprovalState = ContentConstants.State.APPROVED;
				this._actionsCallback();
			}, this));
			
			this._rejectButton.click(_.bind(function(event) {
				this._markedApprovalState = ContentConstants.State.REJECTED;
				this._actionsCallback();
			}, this));
			
			
			// If the language is ready for approval, show Approve, Reject and RevertToDraft buttons based on the permissions
			if(this._state === ContentConstants.State.READY_FOR_APPROVAL) {
				// If User is have both EDITOR and APPROVER permissions and TextContent is in Pending-Signoff - add Draft, Approve and Reject buttons
				if(this._canReview && this._canEdit) {
					approvalActionsDiv.append(this._approveButton, this._rejectButton, this._revertToDraftButton);
					approvalActionsDiv.find(".TextContent-ApprovalAction").addClass("TextContent-3ApprovalActions");
					approvalActionsDiv.find(".TextContent-ApprovalSymbol").addClass("ActionIcon-3ApprovalActions");
					approvalActionsDiv.find(".TextContent-ApprovalLabel").addClass("ActionLabel-3ApprovalActions").hide();
				}
				// If User has APPROVER permission - add Approve and Reject buttons
				else if(this._canReview) {
					approvalActionsDiv.append(this._approveButton, this._rejectButton);
					approvalActionsDiv.find(".TextContent-ApprovalAction").addClass("TextContent-2ApprovalActions");
					approvalActionsDiv.find(".TextContent-ApprovalSymbol").addClass("ActionIcon-2ApprovalActions");
					approvalActionsDiv.find(".TextContent-ApprovalLabel").addClass("ActionLabel-2ApprovalActions").hide();
				}
				// If the User has EDITOR permission - add the RevertToDraft button
				else if (this._canEdit) {
					approvalActionsDiv.append(this._revertToDraftButton);
					approvalActionsDiv.find(".TextContent-ApprovalAction").addClass("TextContent-1ApprovalAction");
					approvalActionsDiv.find(".TextContent-ApprovalSymbol").addClass("ActionIcon-1ApprovalAction");
					approvalActionsDiv.find(".TextContent-ApprovalLabel").addClass("ActionLabel-1ApprovalAction").hide();
				}
			}
			else if (this._state === ContentConstants.State.DRAFT) {
				// If the language is in Draft and user has both EDITOR and APPROVER permissions, then add Start Approval & Approve button (User can skip start approval)
				if(this._canEdit && this._canReview) {
					approvalActionsDiv.append(this._approveButton, this._startApprovalButton);
					approvalActionsDiv.find(".TextContent-ApprovalAction").addClass("TextContent-2ApprovalActions");
					approvalActionsDiv.find(".TextContent-ApprovalSymbol").addClass("ActionIcon-2ApprovalActions");
					approvalActionsDiv.find(".TextContent-ApprovalLabel").addClass("ActionLabel-2ApprovalActions").hide();
				} 
				// If the language is in Draft and user has EDITOR permission, then add StartApproval button
				else if(this._canEdit) {
					approvalActionsDiv.append(this._startApprovalButton);
					approvalActionsDiv.find(".TextContent-ApprovalAction").addClass("TextContent-1ApprovalAction");
					approvalActionsDiv.find(".TextContent-ApprovalSymbol").addClass("ActionIcon-1ApprovalAction");
					approvalActionsDiv.find(".TextContent-ApprovalLabel").addClass("ActionLabel-1ApprovalAction").hide();
				}
			}
			// If the language is rejected, add a explict move to draft button
			else if(this._canEdit && this._state === ContentConstants.State.REJECTED) {
				approvalActionsDiv.append(this._revertToDraftButton);
				approvalActionsDiv.find(".TextContent-ApprovalAction").addClass("TextContent-1ApprovalAction");
				approvalActionsDiv.find(".TextContent-ApprovalSymbol").addClass("ActionIcon-1ApprovalAction");
				approvalActionsDiv.find(".TextContent-ApprovalLabel").addClass("ActionLabel-1ApprovalAction");
			}
			
			// add the hover handler for approval actions
			approvalActionsDiv.hover(this._actionsDivHandlerIn, this._actionsDivHandlerOut);
			
			return approvalActionsDiv;
		};
		
		/**
		 * Create the Revert Approval button
		 * @returns approvalActionsDiv - jQuery element for revert approval action button
		 */
		ContentStatementElementWidgetView.prototype._constructRevertApprovalAction = function () {
			var approvalActionsDiv = $('<div>').attr({"name": "TextContent-ActionsDiv"}).addClass("TextContent-Approval-NextStates").css({'opacity': '0'});
			
			// Create revert approval action button
			this._revertApprovalButton.empty().removeClass().addClass("RevertApprovalAction TextContent-1ApprovalAction").append($('<div>').addClass("ApprovalAction-InnerDiv").append(
					$('<div>').addClass("TextContent-ApprovalSymbol ActionIcon-1ApprovalAction").append($('<i>').addClass("icon icon-WCR_Revert").addClass("RevertApprovalActionIconAndLabel")),
					$('<div>').addClass("TextContent-ApprovalLabel ActionLabel-1ApprovalAction RevertApprovalActionIconAndLabel").text(TextContentStrings.t("PCM_REVERT"))
					)
			);
			this._revertApprovalButton.click(_.bind(function(event) {
				this._markedApprovalState = this._state;
				this._startApprovalButton = $('<div>').click(this._startApprovalEventHandler);
				this._revertToDraftButton = $('<div>').click(this._revertToDraftEventHandler);
				this._approveButton = $('<div>').click(this._approveEventHandler);
				this._rejectButton = $('<div>').click(this._rejectEventHandler);
				
				$("div.TextContent-ApprovalCol", this._wrapper).empty().append(this._constructElementStatus());
			}, this));
			
			approvalActionsDiv.append(this._revertApprovalButton);
			
			// add the hover handler for approval actions
			approvalActionsDiv.hover(this._actionsDivHandlerIn, this._actionsDivHandlerOut);
			
			return approvalActionsDiv;
		};
		
		/**
		 * Construct the header of the widget
		 * 
		 * @returns headerDiv - jQuery element for Language header and other actions
		 */
		ContentStatementElementWidgetView.prototype._constructHeaderDiv = function () {
			
			var headerClass = null;
			if(ContentConstants.State.DRAFT === this._state) {
				headerClass = "TextContent-TitleRow TextHeaderDraft";
			} else if(ContentConstants.State.READY_FOR_APPROVAL === this._state) {
				headerClass = "TextContent-TitleRow TextHeaderPending";
			} else if(ContentConstants.State.APPROVED === this._state) {
				headerClass = "TextContent-TitleRow TextHeaderApproved";
			} else if(ContentConstants.State.REJECTED === this._state) {
				headerClass = "TextContent-TitleRow TextHeaderRejected";
			} else {
				headerClass = "TextContent-TitleRow";
			}
			
			// create the header div
			var headerDiv = $('<div>').addClass(headerClass);
			
			// Add language description div to header
			headerDiv.append($('<div>').attr({'name': 'languageDescDiv'}));
			
			
			if(this._state === ContentConstants.State.APPROVED || this._state === ContentConstants.State.REJECTED || this._state === ContentConstants.State.READY_FOR_APPROVAL || this._state === ContentConstants.State.DRAFT) {
				// create and append comment button to header
				this._createCommentButton();
				headerDiv.append(this._commentButton);
				
				// create and append approval info button to header
				this._createApprovalInfoButton();
				headerDiv.append(this._approvalInfoButton);
			}
			
			headerDiv.append($('<div>').css({'clear': 'both'}));
			
			return headerDiv;
		};
		
		/**
		 * Create comment button
		 */
		ContentStatementElementWidgetView.prototype._createCommentButton = function() {
			// Add the Comment button based on the permissions
			var commentIcon = null;
			if(this._comments.length > 0) {
				commentIcon = "icon icon-WCR_Text-Comment-full";
			} else {
				commentIcon = "icon icon-WCR_Text-Comment-outline";
			}
			
			this._commentButton.attr({"title": TextContentStrings.t("PCM_COMMENT")}).addClass("ToggleComment TextContent-Action").append($('<i>').addClass(commentIcon));
			
			if((this._canEdit || this._canReview || this._isContentManagerView) || (this._comments.length > 0)) {
				// click event handler for comment button
				this._commentButton.click(_.bind(function(event) {
					if(this._commentsDiv.hasClass("commentshidden")) {
						this._showCommentSection();
					} else {
						this._hideCommentSection();
					}
				}, this));
				
				// hide the comments when clicked anywhere on the page except the comment button
				$(document).mouseup(_.bind(function (e) {
					if (!this._commentsDiv.is(e.target) 
							&& this._commentsDiv.has(e.target).length === 0 && !this._commentButton.is(e.target) && !this._commentButton.find("i:first").is(e.target)) {
						// Check if we should hide the comments section.
						if(!this._commentsDiv.hasClass("commentshidden")) {
							this._hideCommentSection();
						}
					}
					e.stopPropagation();
				}, this));
			} else {
				this._commentButton.prop({'disabled': 'disabled'}).css({'cursor': 'not-allowed'});
			}
		};
		
		/**
		 * Show the comments section
		 */
		ContentStatementElementWidgetView.prototype._showCommentSection = function() {
			this._commentsDiv.removeClass("commentshidden").slideDown(200);
		};

		/**
		 * Hide the comments section
		 */
		ContentStatementElementWidgetView.prototype._hideCommentSection = function() {
			this._commentsDiv.addClass("commentshidden").slideUp(200);
		};
		
		/**
		 * Create Approval Info button
		 */
		ContentStatementElementWidgetView.prototype._createApprovalInfoButton = function() {
			this._approvalInfoButton.addClass("TextContent-Action").attr({"title": TextContentStrings.t("PCM_APPROVAL_INFO")}).append($('<i>').addClass("icon icon-WCR_Info"));
			
			// What needs to happen when the approval info button is clicked.
			this._approvalInfoButton.on("click", function (){
				// Execute all the callbacks ...
				for (var i=0; i < this._onApprovalInfoCallbacks.length; i++) {
					this._onApprovalInfoCallbacks[i]();
				}
			}.bind(this));
			
		};
		
		/**
		 * Construct the TextContent area
		 * @returns contentDiv
		 */
		ContentStatementElementWidgetView.prototype._constructTextDiv = function () {
			
			// Create the div that will be holding the content.
			this._contentDiv = $('<div>').addClass("Content-ContentRow").attr({"name": "Content-ContentAreaDiv"});
			
			if (ContentConstants.TextType.RICHTEXT === this._type) {
				// Add the rich text area to the content div.
				this._contentDiv.append(this._richTextEditor.getJQElement());
			} else {
				// Add the plain text area to the content div.
				this._contentDiv.append(this._plainTextEditor);
			}
			
			// Reset the "editability" tooltip.
			this._nonEditableTooltip = undefined;
			this.applyEditPermission();

			return this._contentDiv;
			
		};
		
		/**
		 * Apply edit permission if allowed or provide tool tip message on disabled field.
		 */
		ContentStatementElementWidgetView.prototype.applyEditPermission = function () {
			// We can only edit when you have the edit permission and the state is "DRAFT" or "REJECTED".
			if (!(this._canEdit && (this._state === ContentConstants.State.DRAFT || this._state === ContentConstants.State.REJECTED))) {
				// Disable content editing.
				if (ContentConstants.TextType.RICHTEXT === this._type) {
					this._richTextEditor.disable();
				} else {
					this._plainTextEditor.prop("readonly", "readonly");
					this._plainTextEditor.addClass("NotEditable");
				}

				// If we can't edit because we lack permissions, say why ...
				var msgTag = undefined;
				if (!this._canEdit) {
					// Get the message tag.
					msgTag = this._whyCanNotEdit;
					// If the state is "EMPTY", make sure the "NO_PCM_PERM_NOT_ADDED" message is shown.
					if (this._state === ContentConstants.State.EMPTY) {
						msgTag = "NO_PCM_PERM_NOT_ADDED";
					}
				} else if (this._state === ContentConstants.State.READY_FOR_APPROVAL) {
					msgTag = "NO_PCM_PERM_EDIT_READY_FOR_APPROVAL";
				} else if (this._canEdit && this._state === ContentConstants.State.EMPTY) {
					msgTag = "NO_PCM_PERM_NOT_ADDED";
				}
				
				// If we have a reason, display it. 
				if (msgTag && this._showTooltips) {
					if (!this._nonEditableTooltip ) {
						// Create the tooltip
						this._nonEditableTooltip = new Tooltip(undefined, this._contentDiv, this._wrapper, $("body"), false, TextContentStrings.t(msgTag), undefined, undefined, {color: "#EBEBE4", cssClass : "TextContent-Restricted-Popover", restrictBorder : 8});
					} else {
						this._nonEditableTooltip.setTooltipContent(TextContentStrings.t(msgTag));
					}
				}
			} else {
				// Enable content editing.
				if (ContentConstants.TextType.RICHTEXT === this._type) {
					this._richTextEditor.enable();
				} else {
					this._plainTextEditor.prop("readonly", false);
				}
			}
		};	
		
		/**
		/**
		 * update the TextContent area
		 * @param content {"content": content} - An object for the text content statement
		 * @returns textDiv - jQuery element for TextContent textarea
		 */
		ContentStatementElementWidgetView.prototype.updateContent = function (content) {
			
			if (!content) {
				content = {content : ""};
			}
			this._content = content;
			
			// Determine whether we should be able to edit the statement or not.
			if (ContentConstants.TextType.RICHTEXT === this._type) {
				this._richTextEditor.setContent(content.content);
			} else {
				this._plainTextEditor.text(content.content);
			}
			
			this.applyEditPermission();
		};
		
		
		/**
		 * Construct the comments column of the widget
		 * 
		 * @returns commentDiv - jQuery element for comments column
		 */
		ContentStatementElementWidgetView.prototype._constructCommentsDiv = function () {
			// If user does not have permissions and there are no existing comments, then do not create comments div
			if((!this._canEdit && !this._canReview && !this._isContentManagerView) && (this._comments.length === 0)) {
				return;
			}
			
			this._commentsDiv = $('<div>').attr({"name": "commentsDiv"});
			
			// hide the comments initially
			this._commentsDiv.addClass("TextComment commentshidden").hide();
			this._commentsDiv.append(
					$('<table>').addClass("TextComments").append(
							$('<tbody>')
					)
			);
			
			// Show the Add comment content area if the user has permissions to add comment
			if(this._canEdit || this._canReview || this._isContentManagerView) {
				this._commentsDiv.find("tbody:first").append($('<tr>').addClass("Comment").append(
						$('<td>').addClass("CommentAuthor").text(TextContentStrings.t("PCM_COMMENT_CURRENT_USER")),
						$('<td>').addClass("CommentContent").append(
								this._commentTextArea.addClass("CommentField").attr({"placeholder": TextContentStrings.t("PCM_ADD_COMMENT_PLACEHOLDER")})
						)
				));
			}
			
			// Add the existing comments to the comments div
			if(this._comments.length > 0) {
				var textcomments = $.extend(true, [], this._comments);
				textcomments.reverse();
				
				var newComment = null;
				_.each(textcomments, _.bind(function(comment) {
					if(!comment.isNew) {
						this._commentsDiv.find("tbody:first").append(
								$('<tr>').addClass("Comment").append(
										$('<td>').addClass("CommentAuthor").text(comment.user),
										$('<td>').addClass("CommentContent").append(
												$('<div>').addClass("CommentText").text(comment.content),
												$('<div>').addClass("CommentDate").text(ConvertUtils.getLangFormattedDateAndTime(TextContentStrings, new Date(WCXML.xmlencode(comment.created))))
										)
								)
						);
					} else {
						newComment = comment;
					}
				}, this));
				
				if(newComment) {
					this._commentTextArea.text(newComment.content);
				}
			}
			
			return this._commentsDiv;
		};
		
		/**
		 * Get the localized value for a text content state
		 * 
		 * @param {String} [stateCode] 
		 * @returns {String} [localizedState]
		 */
		ContentStatementElementWidgetView.prototype._getLocalizedContentState = function(state, myapprovalstatus) {
			var localizedState = state;
			switch (state) {
			case ContentConstants.State.APPROVED:
				localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_APPROVED");
				break;
			case ContentConstants.State.NEW:
				localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_NEW");
				break;
			case ContentConstants.State.DRAFT:
				localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_DRAFT");
				break;
			case ContentConstants.State.READY_FOR_APPROVAL:
				if(myapprovalstatus === ContentConstants.State.APPROVED) {
					localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_PENDING_YOUAPPROVED");
				} else if(myapprovalstatus === ContentConstants.State.REJECTED) {
					localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_PENDING_YOUREJECTED");
				} else if(myapprovalstatus === ContentConstants.State.READY_FOR_APPROVAL) {
					localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_PENDING_YOUPENDING");
				} else {
					localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_PENDING");
				}
				break;
			case ContentConstants.State.EMPTY:
				localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_EMPTY");
				break;
			case ContentConstants.State.REJECTED:
				localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_REJECTED");
				break;
			default:
				break;
			}
			
			return localizedState;
		};
		
		/**
		 * Get the localized action value for a text content state
		 * 
		 * @param {String} [stateCode] 
		 * @returns {String} [localizedAction]
		 */
		ContentStatementElementWidgetView.prototype._getLocalizedContentAction = function(state) {
			var localizedAction = state;
			switch (state) {
			case ContentConstants.State.APPROVED:
				localizedAction = TextContentStrings.t("PCM_ACTION_APPROVE");
				break;
			case ContentConstants.State.DRAFT:
				localizedAction = TextContentStrings.t("PCM_ACTION_DRAFT");
				break;
			case ContentConstants.State.READY_FOR_APPROVAL:
				localizedAction = TextContentStrings.t("PCM_ACTION_START_APPROVAL");
				break;
			case ContentConstants.State.REJECTED:
				localizedAction = TextContentStrings.t("PCM_ACTION_REJECT");
				break;
			default:
				break;
			}
			
			return localizedAction;
		};
		
		/**
		 * Focus on the widget.
		 */
		ContentStatementElementWidgetView.prototype.focus = function (focus) {
			// Check on which type of editor to focus.
			if (ContentConstants.TextType.RICHTEXT === this._type) {
				if (focus) {
					// Focus on the rich text editor.
					this._richTextEditor.focus();
				} else {
					// Blur the plain rich editor
					this._richTextEditor.blur();
				}
			} else {
				if (focus) {
					// Focus on the plain text editor
					this._plainTextEditor.focus();
				} else {
					// Blur the plain text editor
					this._plainTextEditor.blur();
				}
			}
		};
		
		/**
		 * Enable/Disable the widget.
		 */
		ContentStatementElementWidgetView.prototype.enable = function (enable) {
			// Check which type of editor to disable/enable.
			if (ContentConstants.TextType.RICHTEXT === this._type) {
				if (enable) {
					// Focus on the rich text editor.
					this._richTextEditor.enable();
				} else {
					// Blur the plain rich editor
					this._richTextEditor.disable();
				}
			} else {
				if (enable) {
					// Focus on the plain text editor
					this._plainTextEditor.prop('readonly', false);
				} else {
					// Blur the plain text editor
					this._plainTextEditor.prop("readonly", "readonly");
				}
			}
			// TODO also prevent the approval of the widget, @Anil?
		};
		
		/**
		 * Add callback for Start Approval action
		 * @param callback - callback function to perform the start approval action
		 * @returns
		 */
		ContentStatementElementWidgetView.prototype.onStartApproval = function (callback) {
			return this._onStartApprovalCallbacks.push(callback);
		};
		
		/**
		 * Add callback for Reject action
		 * @param callback - callback function to perform the reject action
		 * @returns
		 */
		ContentStatementElementWidgetView.prototype.onReject = function (callback) {
			return this._onRejectCallbacks.push(callback);
		};
		
		/**
		 * Add callback for Approve action
		 * @param callback - callback function to perform the approve action
		 * @returns
		 */
		ContentStatementElementWidgetView.prototype.onApprove = function (callback) {
			return this._onApproveCallbacks.push(callback);
		};
		
		/**
		 * Add callback for Revert to Draft action
		 * @param callback - callback function to perform the revert to draft action
		 * @returns
		 */
		ContentStatementElementWidgetView.prototype.onRevertToDraft = function (callback) {
			return this._onRevertToDraftCallbacks.push(callback);
		};
		
		/**
		 * Add callback for Delete action
		 * @param callback - callback function to peform delete action using delete icon
		 * @returns
		 */
		ContentStatementElementWidgetView.prototype.onDelete = function (callback) {
			return this._onDeleteCallbacks.push(callback);
		};
		
		/**
		 * Add callback for adding comment
		 * @param - callback function to add comment using comment textarea
		 */
		ContentStatementElementWidgetView.prototype.onComment = function (callback) {
			return this._onCommentCallbacks.push(callback);
		};
		
		/**
		 * Add callback for reverting the approval
		 * @param callback - callback function to revert the approval action (approve/reject/revert-to-draft/start-approval)
		 */
		ContentStatementElementWidgetView.prototype.onRevertApproval = function (callback) {
			return this._onRevertApprovalCallbacks.push(callback);
		};
		
		/**
		 * Add callback that should be executed when the approval info button is clicked.
		 * 
		 * @param callback - callback function to update the textcontent
		 */
		ContentStatementElementWidgetView.prototype.onApprovalInfo = function (callback) {
			return this._onApprovalInfoCallbacks.push(callback);
		};
		
		/**
		 * Add callback that should be executed when the main content is clicked.
		 * 
		 * @param callback - callback function to update the textcontent
		 */
		ContentStatementElementWidgetView.prototype.onMainContent = function (callback) {
			return this._onMainContentCallbacks.push(callback);
		};
		
		/**
		 * Method to return the wrapper jQuery element of the widget
		 * 
		 * @returns wrapper - Return the wrapping jquery element.
		 */
		ContentStatementElementWidgetView.prototype.getJQElement = function () {
			return this._wrapper;
		};
		
		/**
		 * Method to check if the element state is EMPTY
		 * 
		 * @returns {Boolean}
		 */
		ContentStatementElementWidgetView.prototype.isEmptyElement = function () {
			return this._state === ContentConstants.State.EMPTY;
		};
		
		// Return the constructor.
		return ContentStatementElementWidgetView;
});
