define(["underscore",
        "jquery",
        "./ContentStatementElementWidgetView",
        "../ContentConstants",
        "utils/webi18n!TextContentStrings"],
	function (_,
			$,
			ContentStatementElementWidgetView,
			ContentConstants,
			TextContentStrings) {
	
		"use strict";

		/**
		 * Create the default DocXContentStatementElementWidgetView.
		 * 
		 * @param {Object} text - The content object from the ContentStatementElementWidget represented by this widget.
		 * @param {Array} comments - The array of Comment objects from the ContentStatementElementWidget represented by this widget.
		 * @param {String} state - The state of the ContentStatementElementWidget represented by this widget.
		 * @param {String} language - The language object of the ContentStatementElementWidget represented by this widget.
		 * @param {Boolean} canEdit - Whether or not the ContentStatementElementWidget represented by this widget can be edited by the current user.
		 * @param {Boolean} whyCanNotEdit - Why the ContentStatementElementWidget represented by this widget can't be edited by the current user.
		 * @param {Boolean} canReview - Whether or not the ContentStatementElementWidget represented by this widget can be reviewed by the current user.
		 * @param {Boolean} whyCanNotReview - Why the ContentStatementElementWidget represented by this widget can't be reviewed by the current user.
		 * @param {Boolean} isContentManagerView - Specifies whether the current user is a Content_Manager
		 * @param {String} myApprovalStatus - Approval status of TextContent element for a Approver
		 * @param {Object} [viewOptions] - An object containing some specific variables indicating how the widget will be rendered.
		 * @param {String} markedApprovalState - Approval action taken if any, Current status otherwise
		 * @param {String} type - text type as "richtext" or "simple".
		 */
		var DocXContentStatementElementWidgetView = function (text, isMaster, comments, state, languageCode, canEdit, whyCanNotEdit, canReview, whyCanNotReview, isContentManagerView, myApprovalStatus, viewOptions, markedApprovalState, type) {
			
			// call the parent prototype object to inherit properties
			ContentStatementElementWidgetView.call(this, text, isMaster, comments, state, languageCode, canEdit, whyCanNotEdit, canReview, whyCanNotReview, isContentManagerView, myApprovalStatus, viewOptions, markedApprovalState, type);
			
			// call the overridden function to render the doctablex view of the widget
			this._constructView();
		};
		
		DocXContentStatementElementWidgetView.prototype = Object.create(ContentStatementElementWidgetView.prototype);
		DocXContentStatementElementWidgetView.prototype.constructor = DocXContentStatementElementWidgetView;
		DocXContentStatementElementWidgetView.prototype.parent = ContentStatementElementWidgetView.prototype;
		
		/**
		 * First override the content height before executing the initializeDOMElements method.
		 */
		DocXContentStatementElementWidgetView.prototype._initializeDOMElements = function () {
			this._contentHeight = 76;
			ContentStatementElementWidgetView.prototype._initializeDOMElements.call(this);
		};
		
		/**
		 * Construct the HTML elements of this view.
		 * 
		 */
		DocXContentStatementElementWidgetView.prototype._constructView = function () {
			this._wrapper.empty();
			
			this._initializeDOMElements();
			
			this._wrapper.addClass("DocTableXView");
			
			// create widget elements and append to the wrapper
			this._wrapper.append(
					this._constructApprovalDiv(),
					$('<div>').addClass("TextContent-ContentCol TextContent-ContentColDocX").append(
							$('<div>').addClass("TextContent-InnerContentCol").append(
									this._constructTextDiv()
							)
					),
					this._constructRightColDiv(),
					this._constructCommentsDiv()
			);
			
			// Update css for TextContent textarea div
			$("div[name='Content-ContentAreaDiv']", this._wrapper).addClass("Content-ContentRowDocX");
			
			// Update css for comments div
			$(".TextComment", this._wrapper).addClass("TextCommentDocX");
		};
		
		/**
		 * Construct the right column with comment, approval info, mystatus buttons
		 * 
		 * @returns rightPanel - jQuery element for the right side actions column of the widget
		 */
		DocXContentStatementElementWidgetView.prototype._constructRightColDiv = function() {
			var rightPanel = $('<div>').addClass("TextContent-RightCol");
			
			// If the element status is empty - don't add comment, approvalinfo, mystatus buttons
			if(!this.isEmptyElement()) {
				this._createCommentButton();
				rightPanel.append(this._commentButton.removeClass().addClass("TextContent-ActionDocX TextContent-ActionBorder"));
				
				this._createApprovalInfoButton();
				rightPanel.append(this._approvalInfoButton.removeClass().addClass("TextContent-ActionDocX"));
			}
			
			return rightPanel;
		};
		
		/**
		 * Show the comments section
		 */
		DocXContentStatementElementWidgetView.prototype._showCommentSection = function() {
			
			// When showing the comments section in the last row of the DocTableX, it could be hidden as DocTableX hides vertical overflow.
			// Check if this is the case and otherwise make some room by adding some padding to the bottom row. 
			// (While not ideal, it's the best place for this at the moment).
			
			// Select the DocTableX row in which this widget is sitting.
			var docXRow = this._wrapper.closest("tbody.DocEntry");
			// Check if we found a row and wether it's the last row of the table.
			if (docXRow.length > 0 && docXRow.is(':last-child')) {
				// Get the DocTableX cell in which this widget is sitting.
				var parentCell = this._wrapper.parent();
				// Make sure we found a cell.
				if (parentCell.length > 0 && parentCell.prop("tagName").toLowerCase() == "td") {
					// Set the bottom padding to the height of the comments div to make place for it.
					docXRow.find("> tr:first-child > td").css({
						"padding-bottom" : this._commentsDiv.height()+6+"px"
					});	
				}
			}
			
			this._commentsDiv.removeClass("commentshidden").slideDown(200);
		};

		/**
		 * Hide the comments section
		 */
		DocXContentStatementElementWidgetView.prototype._hideCommentSection = function() {
			
			// When showing the comments section in the last row of the DocTableX, it could be hidden as DocTableX hides vertical overflow.
			// Check if this is the case and otherwise make some room by adding some padding to the bottom row.
			// When hiding the comment section we need to remove that space again.
			
			// Select the DocTableX row in which this widget is sitting.
			var docXRow = this._wrapper.closest("tbody.DocEntry");
			// Check if we found a row and wether it's the last row of the table.
			if (docXRow.length > 0 && docXRow.is(':last-child')) {
				// Get the DocTableX cell in which this widget is sitting.
				var parentCell = this._wrapper.parent();
				// Make sure we found a cell.
				if (parentCell.length > 0 && parentCell.prop("tagName").toLowerCase() == "td") {
					// Reset the bottom padding of 6px.
					docXRow.find("> tr:first-child > td").css({
						"padding-bottom" : ""
					});	
				}
			}
			
			this._commentsDiv.addClass("commentshidden").slideUp(200);
		};
		
		
		return DocXContentStatementElementWidgetView;
		
});