define(["jquery",
        "lib/summernote/summernote",
        "utils/webcss!lib/summernote/summernote.css",
        "utils/webcss!./RichTextEditor.css"],
	function ($) {
	
	
	var RichTextEditor = function (content, height) {
		
		// The array holding all the "onChange" callbacks.
		this._onChange = [];
		
		// The wrapping div of the editor.
		this._wrapper = $("<div>");
		this._wrapper.addClass("RichTextEditor");
		
		// Create the div that will be hidden and function as the handle for summernote.
		this._handle = $("<div>");
		
		// Parse the content.
		var parsedContent = undefined;
		try {
			parsedContent = $(content);
		} catch (exception) {
			// This is not valid html ... ugh ... just insert in the editor and hope for the best.
			this._handle.html(content);
		}
		// Check if the content could be parsed.
		if (parsedContent && parsedContent.length == 0) {
			// Parsed content could not be parsed, just dump it in the editor and hope for the best.
			this._handle.html(content);
		}
		// Check if the root element of content is "<body>" (we can't insert that in the editor).
		else if (parsedContent && parsedContent.length == 1 && parsedContent.is("body")) {
			// We need to insert all of the parsed content except the body tag itself.
			this._handle.html(parsedContent.html());
		} 
		// If we get here, then we have perfectly fine rich text. 
		else {
			this._handle.html(content);
		}
						
		// Add the handle to the wrapper.
		this._wrapper.append(this._handle);
		
		// Create the editor.
		this._handle.summernote({
			airMode: true,
			disableDragAndDrop: true,
			popover: {
				air: [['style', ['bold', 'italic', 'underline', 'clear']],
				      ['font', ['strikethrough', 'superscript', 'subscript']]]
			},
			keyMap: {
				  pc: {
				    'ENTER': 'insertParagraph',
				    'CTRL+Z': 'undo',
				    'CTRL+Y': 'redo',
				    'TAB': 'tab',
				    'SHIFT+TAB': 'untab',
				    'CTRL+B': 'bold',
				    'CTRL+I': 'italic',
				    'CTRL+U': 'underline',
				    'CTRL+SHIFT+S': 'strikethrough',
				    'CTRL+BACKSLASH': 'removeFormat'
				  },
				  mac: {
				    'ENTER': 'insertParagraph',
				    'CMD+Z': 'undo',
				    'CMD+SHIFT+Z': 'redo',
				    'TAB': 'tab',
				    'SHIFT+TAB': 'untab',
				    'CMD+B': 'bold',
				    'CMD+I': 'italic',
				    'CMD+U': 'underline',
				    'CMD+SHIFT+S': 'strikethrough',
				    'CMD+BACKSLASH': 'removeFormat',
				    'CMD+SHIFT+NUM7': 'insertUnorderedList',
				    'CMD+SHIFT+NUM8': 'insertOrderedList',
				    'CMD+LEFTBRACKET': 'outdent',
				    'CMD+RIGHTBRACKET': 'indent'
				  }
			},
			callbacks: {
				onPaste: this._handlePaste.bind(this) // Handle the paste event.
			}
		});
		this._handle.summernote("setHeight", (height-10)+"px");
		this._handle.summernote("setStyle", "word-wrap", "break-word");
		
		// When text is not wrapped with p tags then it required to give padding for text with superscript style.
		this._handle.summernote("setStyle", 'padding-top', '5px');
		
		// When the wrapper is clicked ... transfer focus to the editor.
		this._wrapper.on("click", function () {
			this._handle.summernote("focus");
		}.bind(this));
		
		// Execute change call-backs when content is changed in editor.
		this._handle.on('summernote.change', function () {
			$.each(this._onChange, function (index, changeCallback) {
				changeCallback();
			}.bind(this));
		}.bind(this));
	};
	
	/**
	 * Filter the god damn frigging paste event.
	 */
	RichTextEditor.prototype._handlePaste = function(e) {
		// Defer the handling of this event.
		window.setTimeout(function (e) {
			var allowedTags = ["p", "b", "i", "u", "sup", "br", "sub", "strike", "s"];
			var allowedEmptyTags = ["br"];
			var allowedAttributes = [];
			var changed = this._handle.summernote('cleanEditable', allowedTags, allowedEmptyTags, allowedAttributes);
			if (changed) {
				$.each(this._onChange, function (index, changeCallback) {
					changeCallback();
				}.bind(this));
			}
		}.bind(this, e), 0);
	};

	
	/**
	 * Disable editing.
	 */
	RichTextEditor.prototype.disable = function () {
		this._wrapper.addClass("Disabled");
		this._handle.summernote('disable');
	};
	
	/**
	 * Enable editing.
	 */
	RichTextEditor.prototype.enable = function () {
		this._wrapper.removeClass("Disabled");
		this._handle.summernote('enable');
	};
	
	/**
	 * Set the god damn content.
	 * 
	 * @returns
	 */
	RichTextEditor.prototype.setContent = function (content) {
		this._handle.summernote('code', content);
	};
	
	/**
	 * Get the god damn content.
	 * 
	 * @returns
	 */
	RichTextEditor.prototype.focus = function () {
		this._handle.summernote('focus');
		this._handle.summernote('restoreRange');
	};
	
	/**
	 * Get the god damn content.
	 * 
	 * @returns
	 */
	RichTextEditor.prototype.blur = function () {
		this._handle.summernote('saveRange');
		this._handle.summernote('blur');
		// Also clear the current selection.
		if (window.getSelection()) {
			window.getSelection().removeAllRanges();
		} else if (document.selection) {
			document.selection.empty();
		}
	};
	
	/**
	 * Get the god damn content.
	 * 
	 * @returns
	 */
	RichTextEditor.prototype.getContent = function () {
		
		var newContent = this._handle.summernote('code');
		var parsedNewContent = $.parseHTML(newContent);
		// Check if newCotent has one single root tag.
		if (parsedNewContent && parsedNewContent.length > 1) {
			// Create a "body" root tag to contain all the elements.
			var root = $("<body>");
			root.append(parsedNewContent);
			return root[0].outerHTML;
		} else {
			return newContent;
		}
		
		
	};
	
	/**
	 * Register an "onChange" callback.
	 * 
	 * @param callback
	 * @returns {Array}
	 */
	RichTextEditor.prototype.onChange = function (callback) {
		this._onChange.push(callback);
	};
	
	/**
	 * Return the wrapping jQuery element.
	 * @returns
	 */
	RichTextEditor.prototype.getJQElement = function () {
		return this._wrapper;
	};
	
	return RichTextEditor;
});
