define(["jquery",
        "packaging-content-management/ContentStatement",
        "packaging-content-management/ContentStatementElement",
        "services/passwordValidationService",
        "utils/webi18n!TextContentStrings"], 
        function ($, 
        		ContentStatement,
        		ContentStatementElement,
        		passwordValidationService,
        		TextContentStrings) {

	
	
	var ContentStatementService = function () {
		
        // These private variables take care of preventing duplicate requests.
        this._contentStatementIsBeingRetrieved = {};
        this._onContentStatementRetrieved = {};
        
	};


	/**
	 * Request an existing contentent statement from the backend.
	 * 
	 * @param {String} docVerId - The ID of the document version of the the contentstatement that should be returned.
	 * @param {Function} callback - Will be executed on succesfull return from the backend (and be passed the content statement).
	 */
	ContentStatementService.prototype.getContentStatement = function (docVerId, callback, errorCallback) {
		
		var params = {
				"ajax": "1",
				"docverid": docVerId,
		};
		
		// The baseUrl of the service.
        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
		var url = jspBaseUrl+"GetDocTypeTextContent.jsp?json=1&" + $.param(params, true);
		$.getJSON(url, function(jsonData) {
        	var contentStatement = new ContentStatement(jsonData);
        	callback(contentStatement);
        }.bind(this)).fail(function(jqXHR, textStatus, errorThrown) {
        	if(errorCallback) {
            	var errorCode = null;
            	errorCode = $(jqXHR.responseText).find('error').find('code').text();
            	
            	if(errorCode && ('INVALIDPARAMETER_TAG' === errorCode || '002' === errorCode || '003' === errorCode)) {
            		errorCallback(TextContentStrings.t("TEXTCONTENT_DOCVERSION_NOT_FOUND"));
            		return;
            	}
        		errorCallback(TextContentStrings.t("TEXTCONTENT_LOAD_ERROR"));
        	}
        });
	};
	
	
	/**
	 * Request a new contentent statement in a different project from the backend. (does not yet exist)
	 * 
	 * @param {String} docVerId - The ID of the document version from wich the content statement should be created/copied.
	 * @param {String} targetProjID - The ID of the project in which the new content statement should be created/copied.
	 * @param {Function} callback - Will be executed on succesfull return from the backend (and be passed the content statement).
	 */
	ContentStatementService.prototype.getNewContentStatementFor = function (docVerId, targetProjID, retainStatus, callback, errorCallback) {
		
		var params = {
				"ajax": "1",
				"isNew": "1",
				"docverid": docVerId,
				"retainstatus" : retainStatus
		};
		
		// Set also a projectID in the parameters when specified. 
		if (targetProjID && targetProjID != null) {
			params.targetprojid = targetProjID;
		}

		// The baseUrl of the service.
        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
		var url = jspBaseUrl+"GetDocTypeTextContent.jsp?json=1&" + $.param(params, true);
		$.getJSON(url, function(jsonData) {
        	var contentStatement = new ContentStatement(jsonData);
        	callback(contentStatement);
        }.bind(this)).fail(function(jqXHR, textStatus, errorThrown) {
        	if(errorCallback) {
            	var errorCode = null;
            	errorCode = $(jqXHR.responseText).find('error').find('code').text();
            	
            	if(errorCode && ('INVALIDPARAMETER_TAG' === errorCode || '002' === errorCode || '003' === errorCode)) {
            		errorCallback(TextContentStrings.t("TEXTCONTENT_DOCVERSION_NOT_FOUND"));
            		return;
            	}
        		errorCallback(TextContentStrings.t("TEXTCONTENT_LOAD_ERROR"));
        	}
        });
	};
	
	
	/**
	 * Save the changes in a Content Statement object.
	 * 
	 * The Content Statement should already exist!
	 * 
	 * @param {ContentStatement} contentStatement - The Content Statement object to be saved.
	 * @param {Function} callback - To be executed on save action (Will be passed the updated content statement or
	 * 								the content statement for which the update failed and a possible error message).
	 */
	ContentStatementService.prototype.saveContentStatement = function (contentStatement, callback) {
		this.saveContentStatements([contentStatement], callback);
	};
	
	/**
	 * Save the changes in multiple Content Statement objects.
	 * 
	 * These Content Statements should already exist!
	 * 
	 * @param {ContentStatement} contentStatements - The Content Statements objects to be saved.
	 * @param {Function} callback - To be executed on save action (Will be passed the updated contentStatements, 
	 * 								the contentStatements for which the update failed and error messages).
	 */
	ContentStatementService.prototype.saveContentStatements = function (contentStatements, callback) {
		
		var password = passwordValidationService.getValidatedPassword();
		
		var saveJsons = [];
		var docVersionIds = [];
		var docVersionIdMap = [];
		for (var i=0; i < contentStatements.length; i++) {
			saveJsons.push(contentStatements[i].getUnsavedContentJSON(password));
			docVersionIds.push(contentStatements[i].docVersionId);
			docVersionIdMap[contentStatements[i].docVersionId] = contentStatements[i];
		}
		
		var url = window.webcenterConfig.jspBaseUrl + 'updateDocTypeTextContent.jsp';
		$.ajax(url, {
			dataType: "json",
			cache: false,
			noAjaxParam: false,
			contentType: "application/x-www-form-urlencoded; charset=UTF-8",
			type: "POST",
			data: {
				updatejsons: saveJsons,
				docverids : docVersionIds
			},
			success: function (json) {
				
				// Get all the error messages.
				var errorMsgs = new Array();
				// Get all the updated content statements;
				var updatedContentStatements = {};
				// Get all the failed content statements;
				var failedContentStatements = {};
				
				// If a general error occured.
				if (json.error) {
					// Add the general error occured.
					errorMsgs.push(json.error);
					// All content statements failed.
					failedContentStatements = contentStatements.slice(0, contentStatements.length);
				}
				// Find out which updates succeeded and get the new updated content statements.
				else if (json["updates"]) {
					// Iterate over all update entries ...
					for (var i=0; i < json.updates.length; i++) {
						var update = json.updates[i];
						// Check if an error occured during the update ...
						if (update["error"]) {
							errorMsgs[update["id"]] = update["error"];
						}
						// Check if we have a newly updated version of the content statement.
						if (update["new"]) {
							updatedContentStatements[update["id"]] = new ContentStatement(update["new"]);
						}
						// If no such newly updated version could be found ...
						else {
							// We now know that this contentStatement could not be updated.
							// Add the original content statement to the list of failed content statements.
							var failedContentStatement = docVersionIdMap[update["id"]];
							failedContentStatements[update["id"]] = failedContentStatement;
						}
					}
				}
				
				// If a callback was specified.
				if (callback) {
					callback(updatedContentStatements, failedContentStatements, errorMsgs);
				}
				
			},
			error: function (jqXHR, textStatus, errorThrown) {
				if(callback) {
					callback([], contentStatements.slice(0, contentStatements.length), [TextContentStrings.t("TEXTCONTENT_SAVE_FAILED")]);
				}
			}
		});
		
	};
	
	/**
	 * Get the Content statement element language when a new language has been added
	 */
	ContentStatementService.prototype.getContentStatementElement = function (docVerId, language, isMaster, callback, errorCallback) {
		
		var langCode = language;
		if (typeof langCode == "object") {
			langCode = language.langCode;
		}
		
		var params = {
				"ajax": "1",
				"docverid": docVerId,
				"language": langCode,
				"contentType": isMaster,
		};

		// The baseUrl of the service.
        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
		var url = jspBaseUrl+"CreateDocTypeTextContentElement.jsp?" + $.param(params, true);
		$.getJSON(url, function(jsonData) {
        	var element = new ContentStatementElement(jsonData);
        	callback(element);
        }.bind(this)).fail(function(jqXHR, textStatus, errorThrown) {
        	if(errorCallback) {
            	var errorCode = null;
            	errorCode = $(jqXHR.responseText).find('error').find('code').text();
            	
            	if(errorCode && ('INVALIDPARAMETER_TAG' === errorCode || '002' === errorCode || '003' === errorCode)) {
            		errorCallback(TextContentStrings.t("TEXTCONTENT_DOCVERSION_NOT_FOUND"));
            		return;
            	}
        		errorCallback(TextContentStrings.t("TEXTCONTENT_LOAD_ERROR"));
        	}
        });
		
		
	};
	
	
	/**
	 * Get the Content statement element language when a new language has been added in case of new project
	 */
	ContentStatementService.prototype.getNewContentStatementElementFor = function (docVerId, language, isMaster, targetProjId, retainStatus, callback, errorCallback) {
		
		var langCode = language;
		if (typeof langCode == "object") {
			langCode = language.langCode;
		}
		
		var params = {
				"ajax": "1",
				"isNew": "1",
				"docverid": docVerId,
				"language": langCode,
				"contentType": isMaster,
				"retainstatus": retainStatus
		};

		// Set also a projectID in the parameters when specified. 
		if (targetProjId && targetProjId != null) {
			params.targetprojid = targetProjId;
		}
		
		// The baseUrl of the service.
        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
		var url = jspBaseUrl+"CreateDocTypeTextContentElement.jsp?" + $.param(params, true);
		$.getJSON(url, function(jsonData) {
        	var element = new ContentStatementElement(jsonData);
        	callback(element);
        }.bind(this)).fail(function(jqXHR, textStatus, errorThrown) {
        	if(errorCallback) {
            	var errorCode = null;
            	errorCode = $(jqXHR.responseText).find('error').find('code').text();
            	
            	if(errorCode && ('INVALIDPARAMETER_TAG' === errorCode || '002' === errorCode || '003' === errorCode)) {
            		errorCallback(TextContentStrings.t("TEXTCONTENT_DOCVERSION_NOT_FOUND"));
            		return;
            	}
        		errorCallback(TextContentStrings.t("TEXTCONTENT_LOAD_ERROR"));
        	}
        });
		
		
	};

	
	/**
	 * Request an existing contentent statement from the backend.
	 * 
	 * @param {String} docVerId - The ID of the document version of the the contentstatement that should be returned.
	 * @param {Function} callback - Will be executed on succesfull return from the backend (and be passed the content statement).
	 */
	ContentStatementService.prototype.getContentStatementList = function (docVerId, callback, errorCallback) {
		
		var params = {
				"ajax": "1",
				"docverid": docVerId,
		};
		
		// The baseUrl of the service.
        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
		var url = jspBaseUrl+"GetDocTypeTextContent.jsp?json=1&" + $.param(params, true);
		$.getJSON(url, function(jsonData) {
			var contentStatementList = [];
			for(var i=0;i<jsonData.length;i++) {
				var contentStatement = new ContentStatement(jsonData[i]);
				contentStatementList.push(contentStatement);
			}
			callback(contentStatementList);
		}.bind(this)).fail(function(jqXHR, textStatus, errorThrown) {
			if(errorCallback) {
            	var errorCode = null;
            	errorCode = $(jqXHR.responseText).find('error').find('code').text();
            	
            	if(errorCode && ('INVALIDPARAMETER_TAG' === errorCode || '002' === errorCode || '003' === errorCode)) {
            		errorCallback(TextContentStrings.t("TEXTCONTENT_DOCVERSION_NOT_FOUND"));
            		return;
            	}
        		errorCallback(TextContentStrings.t("TEXTCONTENT_LOAD_ERROR"));
        	}
        });
	};
	
	return new ContentStatementService();
	
});