define(["jquery",
        "packaging-content-management/PCMEventBus",
        "packaging-content-management/text-content-utils/LanguageUtils"], function ($, pcmEventBus, LanguageUtils) {

	
	
	var LanguageService = function () {
		
		// These private variables will cache the returned languages and smartnames
        this._cachedLanguages = {};
        
        // These private variables take care of preventing duplicate requests for project languages.
        this._langsAreBeingRetrieved = {};
        this._onLangsRetrieved = {};
        
        // These private variables take care of preventing duplicate request for all languages.
        this._allLangsAreBeingRetrieved = false;
        this._onAllLangsRetrieved = [];
        
        // Add a listener to update our cache when a language was removed/added from the project.
        pcmEventBus.on("PCM.projMasterLangChanged", _.bind(function (langAttrName, addedLangDesc, removedLangDesc, projectID) {
        	
			if(this._cachedLanguages[projectID]) {
				var isMaster = true;
				// Determine wether we need to work on Master/Translation text.
				var projLangs = this._cachedLanguages[projectID].master;
				var allLangs = this._cachedLanguages.all.master;
				if ('MASTER_TEXT_LANGUAGE' !== langAttrName) {
					isMaster = false;
					projLangs = this._cachedLanguages[projectID].translations;
					allLangs = this._cachedLanguages.all.translations;
				}
				
				// Let's start by mapping the passed language descriptions to real language objects.
					
				// Start searching for the passed language description in the "All Languages" array.
				var addedLang = [];
				var removedLang = [];
				for (var i=0; i < addedLangDesc.length;i++) {
					var lang = LanguageUtils.getLangForDescription(allLangs, addedLangDesc[i]);
					if (lang) {
						addedLang.push(lang);
					}
				}
				for (var i=0; i < removedLangDesc.length;i++) {
					var lang = LanguageUtils.getLangForDescription(allLangs, removedLangDesc[i]);
					if (lang) {
						removedLang.push(lang);
					}
				}
				
				// Now add or remove them to the project languages.
				if(addedLang.length > 0) {
					if(isMaster) {
						this._cachedLanguages[projectID].master = projLangs.concat(addedLang);
					} else {
						this._cachedLanguages[projectID].translations = projLangs.concat(addedLang);
					}
					
				}
				if(removedLang.length > 0) {
					if(isMaster) {
						LanguageUtils.removeFrom(this._cachedLanguages[projectID].master, removedLang);
					} else {
						LanguageUtils.removeFrom(this._cachedLanguages[projectID].translations, removedLang);
					}
				}
			}
        }, this));
        
	};
	

	/**
	 * Return the project master or translation languages for a certain text content document.
	 * 
	 * @param projectId
	 * @param isMaster
	 * @param onProjectLangsCallback
	 * @returns
	 */
	LanguageService.prototype.getProjectLanguages = function (projectId, isMaster, onProjectLangsCallback) {
		if (this._cachedLanguages[projectId]) {
			// We have a cache entry for that projectId, return from cache.
			this._returnCachedLanguagesFor(projectId, isMaster, true, onProjectLangsCallback);
		} else {
			// We don't have a cache entry, this means that they still need to be fetched.
			// Execute the "onProjectLangsCallback" when the're retrieved by adding it to the "this._onLangsRetrieved" array.
			if (!this._onLangsRetrieved[projectId]) {
				this._onLangsRetrieved[projectId] = [];
			}
			this._onLangsRetrieved[projectId].push(_.bind(function () {
				this._returnCachedLanguagesFor(projectId, isMaster, true, onProjectLangsCallback);
			}, this));
			
			// Fetch the languages.
			this._getLanguages(projectId);
		}
	};

	
	/**
	 * Return all the master or translation languages for whole WebCenter.
	 * 
	 * @param projectId
	 * @param isMaster
	 * @param onAllLangsCallback
	 * @returns
	 */
	LanguageService.prototype.getAllLanguages = function (projectId, isMaster, onAllLangsCallback) {
		if (this._cachedLanguages.all) {
			// We have a cache entry for that projectId, return from cache.
			this._returnCachedLanguagesFor(projectId, isMaster, false, onAllLangsCallback);
		} else {
			// We don't have a cache entry, this means that they still need to be fetched.
			// Execute the "onProjectLangsCallback" when the're retrieved by adding it to the "this._onLangsRetrieved" array.
			if (!this._onLangsRetrieved[projectId]) {
				this._onLangsRetrieved[projectId] = [];
			}
			this._onLangsRetrieved[projectId].push(_.bind(function () {
				this._returnCachedLanguagesFor(projectId, isMaster, false, onAllLangsCallback);
			}, this));
			
			// Fetch the languages.
			this._getLanguages(projectId);
		}
	};
	
	/**
	 * Get both the translation and master languages.
	 */
	LanguageService.prototype.getBothAllLanguages = function (onAllLangsCallback) {
		
		if (this._cachedLanguages.all){
			onAllLangsCallback(this._cachedLanguages.all.master, this._cachedLanguages.all.translations);
		} else {
			
			// We don't have a cache entry, this means that they still need to be fetched.
			// Execute the "onProjectLangsCallback" when the're retrieved by adding it to the "this._onLangsRetrieved" array.
			this._onAllLangsRetrieved.push(_.bind(function () {
				onAllLangsCallback(this._cachedLanguages.all.master, this._cachedLanguages.all.translations);
			}, this));
			
			// We need to fetch all the languages.
			this._getAllLanguages();
		}
	};
	
	/**
	 * A utility function to return some cached languages.
	 * 
	 * @param projectId
	 * @param isMaster
	 * @param onlyProject
	 * @param onLangsCallback
	 * @returns
	 */
	LanguageService.prototype._returnCachedLanguagesFor = function (projectId, isMaster, onlyProject, onLangsCallback) {
		
		// Check if need for all the languages or just the project languages.
		if (onlyProject) {
			// Get the cached languages for that project.
			var docVerLang =  this._cachedLanguages[projectId];
			if (isMaster) {
				onLangsCallback(docVerLang.master);
			} else {
				onLangsCallback(docVerLang.translations);
			}
		} else {
			// Get the cached languages for whole WebCenter.
			var allLang = this._cachedLanguages.all;
			if (isMaster) {
				onLangsCallback(allLang.master);
			} else {
				onLangsCallback(allLang.translations);
			}
		}
	};
	
	/**
	 * 
	 * @param projectId
	 * @param onLangsCallback
	 * @param textContentsLangs - In Viewer TextContentWidget when no Project language attributes are present then the languages in Textcontents are considered
	 */
	LanguageService.prototype.getLanguages = function(projectId, onLangsCallback, textContentsLangs) {
		
		if (this._cachedLanguages.all && this._cachedLanguages[projectId]) {
			// We have a cache entry for these languages.
			var allLang = this._cachedLanguages.all;
			var docVerLang =  this._cachedLanguages[projectId];
			onLangsCallback(docVerLang.master, docVerLang.translations, allLang.master, allLang.translations);
		} else {
			// We don't have a cache entry for the languages, this means that they still need to be fetched.
			// Execute the "onLangsCallback" when the're retrieved by adding it to the "this._onLangsRetrieved" array.
			if (!this._onLangsRetrieved[projectId]) {
				this._onLangsRetrieved[projectId] = [];
			}
			this._onLangsRetrieved[projectId].push(_.bind(function () {
				var allLang = this._cachedLanguages.all;
				var docVerLang =  this._cachedLanguages[projectId];
				onLangsCallback(docVerLang.master, docVerLang.translations, allLang.master, allLang.translations);
			}, this));
			
			// Fetch the languages.
			this._getLanguages(projectId, textContentsLangs);
		}
	};
	
	
	/**
	 * Make a language retrieval request to WebCenter.
	 */
	LanguageService.prototype._getLanguages = function (projectId, languages) {
    	
		// Check if we're not already retrieving the languages for this projectId.
		if (this._langsAreBeingRetrieved[projectId] === undefined || this._langsAreBeingRetrieved[projectId] === false) {
			
    		// Indicate that they're being fetched now for this projectId 
			this._langsAreBeingRetrieved[projectId] = true;
    		
			// The baseUrl of the service.
	        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
			
	        var url = jspBaseUrl+"GetProjectLanguageAttr.jsp?json=1&projectId="+projectId;
	        
	        if(languages) {
	        	url += "&languages=" + languages;
	        }
	        
    		// Do the async call to the service
    		$.getJSON(url, _.bind(function( data ) {
    		
    			// Parse the json string.
    			this._cachedLanguages.all = {};
    			this._cachedLanguages.all.master = this._parseLangs(data.languages.complete_source_language.language);
    			this._cachedLanguages.all.translations = this._parseLangs(data.languages.complete_translation_language.language);
    			this._cachedLanguages[projectId] = {};
    			
    			if (data.languages.selected_source_language) {
        			this._cachedLanguages[projectId].master = this._parseLangs(data.languages.selected_source_language.language);
    			} else {
    				this._cachedLanguages[projectId].master = [];
    			}
    			
    			if (data.languages.selected_translation_language) {
    				this._cachedLanguages[projectId].translations = this._parseLangs(data.languages.selected_translation_language.language);
    			} else {
    				this._cachedLanguages[projectId].translations = [];
    			}
    			
    			// Execute the onLangsRetrieved callbacks.
    			var callbacks = this._onLangsRetrieved[projectId];
    			if (callbacks) {
    				for (var i=0; i < callbacks.length; i++) {
        				callbacks[i]();
        			}
    			}
    			
        	}, this)).fail(function(jqxhr, textStatus, error) {
        		
        		
        	});
        }
	};
	
	
	/**
	 * Make a language retrieval request to WebCenter without providing prjectId. <br>
	 * Useful for those functionality where projectId is not available like- Search.
	 */
	LanguageService.prototype._getAllLanguages = function () {
		
		if (this._allLangsAreBeingRetrieved === false) {
			
			// Indicate that they're being fetched now 
			this._allLangsAreBeingRetrieved = true;
    		
			// The baseUrl of the service.
	        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
			
    		// Do the async call to the service
    		$.getJSON( jspBaseUrl+"GetProjectLanguageAttr.jsp?json=1", _.bind(function( data ) {
    			
    			// Parse the json string.
    			this._cachedLanguages.all = {};
    			this._cachedLanguages.all.master = this._parseLangs(data.languages.complete_source_language.language);
    			this._cachedLanguages.all.translations = this._parseLangs(data.languages.complete_translation_language.language);
    			
    			// Execute the onLangsRetrieved callbacks.
    			for (var i=0; i < this._onAllLangsRetrieved.length; i++) {
    				this._onAllLangsRetrieved[i]();
        		}
  
    			
    			this._allLangsAreBeingRetrieved = false;
    			
    		}, this)).fail(function(jqxhr, textStatus, error) {
        		
        		
        	});
		}
	};
	
	/**
	 * Make a language retrieval request to WebCenter.
	 */
	LanguageService.prototype._parseLangs = function (langsFromXml) {
		var langs = [];
		
		if (langsFromXml instanceof Array) {
			for (var i=0; i <langsFromXml.length; i++) {
				var lang = {};
				lang.langCode = langsFromXml[i].language_code;
				lang.langDesc = langsFromXml[i].descriptor;
				lang.langFlag = langsFromXml[i].flag_file_name;
				lang.langIndex = langsFromXml[i].language_index;
				langs.push(lang);
			}
		} else if (langsFromXml instanceof Object) {
			var lang = {};
			lang.langCode = langsFromXml.language_code;
			lang.langDesc = langsFromXml.descriptor;
			lang.langFlag = langsFromXml.flag_file_name;
			lang.langIndex = langsFromXml.language_index;
			langs.push(lang);
		}
		
		return langs;
	};
	
    return new LanguageService();

});