/**
 * Text Information view generation.
 * @author PASH
 */
define(["jquery",
        "underscore",
        "utils/webi18n!TextContentStrings",
        "packaging-content-management/text-information-view/InitTextInformationView",
        "packaging-content-management/ContentStatement",
        "packaging-content-management/ContentConstants",
        "packaging-content-management/text-content-services/contentStatementService"], 
	function($,
			_,
			webi18n,
			InitTextInformationView,
			ContentStatement,
			ContentConstants,
			contentStatementService) {
		
		"use strict";
		
		
		/**
		 * Constructor for TextInformationView
		 * @param documentVersionId {documentVersionId} a reference to the documentVersionId
		 * @param projectId {projectId} a reference to the projectId
		 * @param i18nPCM {i18nPCM} array of localized strings from wcstrings.
		 */
		var TextInformationView = function(documentVersionId, projectId, passwordRequired, showReadOnly) {
			
			this.documentVersionId = documentVersionId;
			this.projectId = projectId;
			this.passwordRequired = passwordRequired;
			this.showReadOnly = showReadOnly;
			this.webi18n = webi18n;
			
			this._view = new InitTextInformationView(webi18n);
			
			// Load the content statement and update the view.
			contentStatementService.getContentStatement(documentVersionId, function (contentStatement) {
				this._contentStatement = contentStatement;
				this.updateView();
			}.bind(this), function(message) {
				this._view.displayLoadError(message, false);
			}.bind(this));
			
			// Save the content savement.
			this._view.onSaveClicked(function () {
				contentStatementService.saveContentStatements([this._contentStatement], function (updatedContentStatements, failedContentStatements, errorMsgs) {
					var updatedContentStatementIds = Object.keys(updatedContentStatements);
					var errorMsgIds = Object.keys(errorMsgs);
					if (updatedContentStatementIds.length > 0) {
						// Get the updated content statement.
						var updatedContentStatement = updatedContentStatements[updatedContentStatementIds[0]];
						// We need to refresh the page but without the docversionId parameter since we want the latest version.
						// Therefore we need the following parameters
						var docID = this._getParameterByName("docID");
						var menu = this._getParameterByName("menu_file");
						// Generate the new query string and redirect to it.
						var query = jQuery.param({'docID': docID, 'menu_file': menu, 'docVerID': updatedContentStatement.docVersionId});
						window.location.replace("./doctypetextview.jsp?"+query);
					} else if (errorMsgIds.length > 0) {
						this._view.displaySaveError(errorMsgs[errorMsgIds[0]], true);
					}
				}.bind(this));
			}.bind(this));
			
			
			this._view.onInvariantChange(function (val) {
				var isInvariant = val;
				this._contentStatement.invariant = isInvariant+"";
				this._contentStatement.invariantChanged = true;
				if(isInvariant) {
					this._view._propertiesContainer.find(".TranslationLangRow:first").hide();
					this._view._translationElementsContainer.hide();
				} else {
					this._view._propertiesContainer.find(".TranslationLangRow:first").show();
					this._view._translationElementsContainer.show();
				}
			}.bind(this));
			
			this._view.onElementTypeChange(function (val) {
				this._contentStatement.elementType = val;
				this._contentStatement.elementTypeChanged = true;
			}.bind(this));
			
			this._view.onApprovalTypeChange(function (val) {
				this._contentStatement.approvalType = val;
				this._contentStatement.approvalTypeChanged = true;
			}.bind(this));
			
			this._view.onTextTypeChange(_.bind(function (val) {
				var isTextTypeUpdated = true;
				
				// renders the text content elements again with new text type.
				var elements = this._contentStatement.element;

				// Verify whether confirmation for approval state change is required to ask to the user or not.
				// If all the text contents are in draft state then not need to ask for the confirmation.
				if (elements) {
					$.each(elements, _.bind(function (index, element) {
						if (!(element.state == ContentConstants.State.DRAFT || element.state == ContentConstants.State.EMPTY || element.state == ContentConstants.State.NONE)) {
							var result = confirm(this.webi18n.t("PCM_TYPE_CHANGE_SIMPLE"));
							if (!result) {
								isTextTypeUpdated = false;
								// If text type is changed to simple, then confirm again for the action because approval process will start again.
								if (ContentConstants.TextType.SIMPLE === val) {
									this._view.setTextType(ContentConstants.TextType.RICHTEXT);
								} else {
									this._view.setTextType(ContentConstants.TextType.SIMPLE);
								}
							}
							return false;
						}
					}, this));
				}
				
				// check whether text type is modified or change rolled back by user.
				if (isTextTypeUpdated) {
					this._contentStatement.type = val;
					this._contentStatement.typeChanged = true;
					
					// renders the text content elements again with new text type.
					var elements = this._contentStatement.element;
					if (elements) {
						$.each(elements, _.bind(function (index, element) {
							// If text type is changed to simple, then remove all the HTML tags of styling from the content.
							if (ContentConstants.TextType.SIMPLE === val) {
								var simpleContent = this._removeHTMLTags(element.content.content);
								element.setContent({"content": simpleContent});
							} else 
								// If text type is changed to rich, then encode content text.
								if (ContentConstants.TextType.RICHTEXT === val) {
								var encodedContent = WCXML.xmlencode(element.content.content);
								element.setContent({"content": encodedContent});
							}
						}, this));
						
						this._view.getTextContentRows(elements, this._contentStatement.elementType, this._contentStatement.invariant);
					}
				}
				
				// Text type is modified or not.
				return isTextTypeUpdated;
			}, this));
			
			this._view.onLangChange(function () {
				
				var elements = this._contentStatement.element;
				if (elements) {
					this._view.getTextContentRows(elements);
				}
				
				// show the translation if not invariant and translations are available
				if(this._contentStatement.invariant !== 'true' && this._view._translationElementsContainer.children().length > 1) {
					this._view._translationElementsContainer.show();
				} else if(this._contentStatement.invariant === 'true' || this._view._translationElementsContainer.children().length === 1) {
					this._view._translationElementsContainer.hide();
				}
				
			}.bind(this));
		};
		
		/**
		 * Method is to remove all the HTML tags from text content string
		 * @param textContentWithHTML {String} - text content string with HTML tags.
		 * @returns {String} - text content string without HTML tags.
		 */
		TextInformationView.prototype._removeHTMLTags = function (textContentWithHTML) {
			return $('<p>').html(textContentWithHTML).text();
		};
		
		/** 
		 * Initializing Text Information view
		 * @returns
		 */
		TextInformationView.prototype.updateView = function() {
			
			// Hide the loading message.
			this._view.hideLoadingMessage();

			// renders the last updated date at top of the document
			this._view.getTextUpdatedDate(this._contentStatement.updated);
			
			// renders the element type, invariant option 
			this._view.getMetaDataRow(this._contentStatement.contentManagerView, this._contentStatement.elementType, this._contentStatement.invariant);
			
			// renders the approval dropdown option
			this._view.getApprovalSetupRow(this._contentStatement.contentManagerView, this._contentStatement.approvalType);
			
			// renders the type of the document drop down option
			this._view.getTextTypeRow(this._contentStatement.contentManagerView, this._contentStatement.type);
			
			this._view.getProjectLanguagesRow(this._contentStatement);
			
			this._view.setIsPasswordRequiredFunc(_.bind(this._contentStatement.isPasswordRequiredForApproval, this._contentStatement));
			
			// renders the text content elements
			var elements = this._contentStatement.element;
			if (elements) {
				this._view.getTextContentRows(elements, this._contentStatement.elementType, this._contentStatement.invariant);
			}
		};
		
		/**
		 * Get a query parameter by name;
		 * 
		 * @param name
		 * @returns
		 */
		TextInformationView.prototype._getParameterByName = function (name){
		    name = name.replace(/[\[]/, "\\[").replace(/[\]]/, "\\]");
		    var regex = new RegExp("[\\?&]" + name + "=([^&#]*)"),
		        results = regex.exec(location.search);
		    return results === null ? "" : decodeURIComponent(results[1].replace(/\+/g, " "));
		};
		
		return TextInformationView;
	}
);
