define(["wcr/webcenter_html",
        "./PageSection",
        "wcr/AddDocument",
        "wcr/DocumentsTable",
        "wcr/validate"], function (WCHTML, PageSection) {

	
	/**
	*	ChecklistSection constructor (subclass of PageSection)
	* 
	* PageSection serves as a generic superclass of various page sections used within task execution page
	* and others.
	*/
	function ChecklistSection(sectionID, expandToggleID, bodyID, data, state)
	{
		try {
			// call superclass constructor with parameters containing IDs of main elements of the section
			PageSection.call(this, sectionID, expandToggleID, bodyID, state);
		
			// total number of checklist items
			this.itemsNumber = data.itemsNumber;
			// array of checklist items (rows of checklist table)
			this.items = data.items;

			// deep copy of original checklist items to store original values until the new values are saved in DB
			this.origItems = new Array(data.items.length);
			for (var i=0 ; i<data.items.length ; i++) {
				this.origItems[i] = new Object();
				for (var prop in data.items[i]) {
					if (data.items[i].hasOwnProperty(prop)){
						this.origItems[i][prop] = data.items[i][prop];
					}
				}
			}
			
			this.readOnly = true;
		}
		catch (err) {
			alert("Problem in ChecklistSection constructor: " + err);
		}
	}

	// define ChecklistSection as a subclass of PageSection
	ChecklistSection.prototype = new PageSection;


	ChecklistSection.prototype.initialize = function(name)
	{
		PageSection.prototype.initialize.call(this, name);
		try {
			// set a flag defining whether this task's section is modifiable
			this.readOnly = (this.parent.isTaskCompleted || !this.parent.isChangeable);
	    
			// hide the section if there is nothing to show
			if (this.itemsNumber < 1) this.setState("HIDE");
			
			if (!this.readonly) {
				// when the section is not read-only, count checked checklist items and if everything is selected, 
				// check "Select All" checkbox too
				this.setSelectAllCheckbox();
			}
		}
		catch (err) {
			alert("Problem in ChecklistSection.initialize() method: " + err);
		}
	};


	ChecklistSection.prototype.show = function()
	{
		try {
			PageSection.prototype.show.call(this);
		}
		catch (err) {
			alert("Problem in ChecklistSection.show() method: " + err);
		}
	};


	ChecklistSection.prototype.generateDynamicContent = function()
	{
		try {
			// the checkklist was already generated - skip it
			if (this.dynamicContentReady) return;
			
			for (var i = 0; i < this.itemsNumber; i++) {
				this.generateOperatorColumn(i);
				this.generateDateColumn(i);
			}
			// dynamic content of this section generated
			this.dynamicContentReady = true;
		}
		catch (err) {
			alert("Problem in ChecklistSection.generateDynamicContent() method: " + err);
		}
	};

	ChecklistSection.prototype.generateOperatorColumn = function(idx)
	{
		try {
			var operatorID = this.items[idx].operatorID;
			if (!operatorID) operatorID = "";
			
			var html = '';
			if (this.readOnly && operatorID.length > 0) {
				html += this.items[idx].operatorFullName;
			}
			else if (operatorID.length > 0) {
				html += '<a href="#" onclick="javascript:ShowUserDetails(\'' + operatorID + '\'); return false;">' + this.items[idx].operatorFullName + '</a>';
			}
			// put prepared HTML string to operator name column
			$("#ChecklistUserName" + idx).html(html);
		}
		catch (err) {
			alert("Problem in ChecklistSection.generateOperatorColumn() method: " + err);
		}
	};

	ChecklistSection.prototype.generateDateColumn = function(idx)
	{
		try {
			// the conversion of check date in milliseconds could failed
			var checkDate = this.items[idx].checkDate;
			if (isNaN(checkDate) || (checkDate <= 0)) checkDate = 0;
			
			var html = '';
			if (checkDate > 0) {
				html += WCHTML.getReadableDateAndTime(new Date(checkDate));
			}
			else if (this.items[idx].operatorID.length > 0) {
				html += '<i>' + this.parent.langArr.GENERATED_WHEN_SAVED + '</i>';
			}
			// put prepared HTML string to check date (completion date) column
			$("#ChecklistDate" + idx).html(html);
		}
		catch (err) {
			alert("Problem in ChecklistSection.generateOperatorColumn() method: " + err);
		}
	};


	ChecklistSection.prototype.selectAll = function(checkAllItem)
	{
		try {
			if (this.readOnly) return;
		
			// store current this object for use within jQuery.each() function
			var callerThis = this;
			
			if (checkAllItem.checked) {
				if (confirm(this.parent.langArr.CHECKLISTITEM_ALL_CONFIRM))
				{
					// select all checklist items
					$("input[name=SelectChecklistItem]").each(function (idx) {
						// do the action only if required
						if (!this.checked) {
							this.checked = true;
							callerThis.selectItem.call(callerThis, this, idx, true);
						}
					});
				}
				else {
					// deselect "Select All" checkbox again if user changed his/her mind
					checkAllItem.checked = false;
				}
			}
			else {
				// clear selection of all checklist items
				$("input[name=SelectChecklistItem]").each(function (idx) {
					// do the action only if required
					if (this.checked) {
						this.checked = false;
						callerThis.selectItem.call(callerThis, this, idx, true);
					}
				});
			}
			// (the section is marked as modified within this.selectItem() method)
			
			// redraw checklist table as the contents changed
			this.generateDynamicContent();
		}
		catch (err) {
			alert("Problem in ChecklistSection.selectAll() method: " + err);
		}
	};


	ChecklistSection.prototype.selectItem = function(item, idx, fromCheckAll)
	{
		try {
			// when checklist table is read-only, do nothing
			if (this.readOnly) return;
			
			var origItem = this.origItems[idx];
			
			if (item.checked && !origItem.checked) {
				// ("!origItem.checked" diasallow to change already checked checklist item without first 
				// unchecking it and save the changes to DB - this let project history to be updated accordingly)
				this.items[idx].operatorID = this.parent.loggedInUserID;
				this.items[idx].operatorFullName = this.parent.loggedInUserFullName;
				this.items[idx].checkDate = 0;   // the date is not generated until it's saved in DB!
			}
			else {
				this.items[idx].operatorID = origItem.operatorID;
				this.items[idx].operatorFullName = origItem.operatorFullName;
				this.items[idx].checkDate = origItem.checkDate;
			}
			
//			// store new values also to input fields to be accesible for JSP after submit
//			// (checkbox input itself is already checked by user and is static otherwise)
//			$("input[name=ChecklistOperatorID]")[idx].value = this.items[idx].operatorID;
//			$("input[name=ChecklistDate]")[idx].value = this.items[idx].checkDate;
			
			// mark this section as modified
			this.setModified(true);
			
			// count checked checklist items and if everything is selected, check "Select All" checkbox too
			this.setSelectAllCheckbox();
			
			// redraw checklist table as the contents changed
			if (!fromCheckAll) this.generateDynamicContent();

		}
		catch (err) {
			alert("Problem in ChecklistSection.selectChecklistItem() method: " + err);
		}
	};

	ChecklistSection.prototype.setSelectAllCheckbox = function()
	{
		try {
			var selCount = $("input[name=SelectChecklistItem]:checked").length;

			// select or deselect "Select All" checkbox based on number of selected Checklist rows
			var allSelected = (selCount > 0) && (selCount == this.itemsNumber);
			var selAllCheckbox = $("input[name=selectWholeChecklist]")[0];
			if (selAllCheckbox) {
				selAllCheckbox.checked = allSelected;
				// if all are selected disable "Select All" checkbox
				selAllCheckbox.disabled = allSelected;
			}
		}
		catch (err) {
			alert("Problem in ChecklistSection.setSelectAllCheckbox() method: " + err);
		}
	};

	return ChecklistSection;
	
});


