
define(["wcr/webcenter_html",
        "jquery",
        "wcr/AddDocument",
        "wcr/DocumentsTable",
        "wcr/validate"], function (WCHTML, $) {

	
	
	/**
	*	PageSection constructor
	* 
	* PageSection serves as a generic superclass of various page sections used within task execution page
	* and others.
	*/
	function PageSection(sectionID, expandToggleID, bodyID, state)
	{
		try {
			// set the initial state of the page section (possible values: SHOW, HIDE, COLLAPSE, REQUIRED, ERROR)
			// default for page section is that it's shown (expanded)
			if (!state || (state.length == 0)) state = "SHOW";
			this.setState(state);
			
			// page or other section inside which this section resides
			this.parent = null;
			// flag showing whether any item owned by the section was changed 
			// note it doesn't include state of other sections - they should have their own modified flag!
			this.modified = false;
			
			// IDs of the whole GUI section, its header and body
			this.sectionID = sectionID;
			this.expandToggleID = expandToggleID;
			this.bodyID = bodyID;
			
			// stores the future state of section, if it will be expanded or collapsed
			this.expand = (this.state != "COLLAPSE");

			// flag to store if the dynamic part of page section was already prepared or not (late initialization)
			this.dynamicContentReady = false;
			
//			try {
//				var sectionBody = $("#" + this.bodyID);
//				sectionBody.prepend('<input type="hidden" name="projectThreadID" value=""/>');
//			}
//			catch (ignoredError) {}
		}
		catch (err) {
			alert("Problem in PageSection constructor: " + err);
		}
	}


	/**
	* Method to initialize page section after it's added to task execution page.
	* To be overridden, but this basic version should be always called at first.
	*/
	PageSection.prototype.initialize = function(name)
	{
		this.name = name;
	};


	/**
	* General method to display the section GUI
	*/
	PageSection.prototype.show = function()
	{
		var section = $("#" + this.sectionID);
		var sectionBody = $("#" + this.bodyID);

		section.hide();

		if (this.state == "COLLAPSE") {
			// this is the same as SHOW/REQUIRED, but the initial this.expand state is false (== collapsed), not true
			// and thus there is no need to call this.generateDynamicContent() method...
		}
		else if ((this.state == "SHOW") || (this.state == "REQUIRED")) {
			this.generateDynamicContent();
		}
		else {
			sectionBody.html("Error in section " + this.bodyID + ".");
		}
		
		if (this.state != "HIDE") {
			section.show();
			this.slide(false);
		}
		// (no catching of exceptions here - try/catch block should be in show() methods of subclasses, 
		// from which this method is called !)
	};


	PageSection.prototype.setState = function(state) 
	{
		try {
			// possible values of page section state: SHOW, HIDE, COLLAPSE, REQUIRED, ERROR
			if ((state != "SHOW") && (state != "HIDE") && (state != "COLLAPSE") && 
					(state != "REQUIRED") && (state != "ERROR")) state = "HIDE";
			this.state = state;
		}
		catch (err) {
			alert("Problem in PageSection.setState() method: " + err);
		}
	};


	PageSection.prototype.setModified = function(modified) 
	{	
		try {
			// calling this method without parameter is the same as calling it with parameter == true
			if ((typeof modified == 'undefined') || (modified == null)) modified = true;
			this.modified = modified;
			// when page section changed, invalidate the dynamic content by default
			if (modified) this.dynamicContentReady = false;
		}
		catch (err) {
			alert("Problem in PageSection.setModified() method: " + err);
		}
//		if really modified, display the "change" button and the rest...
		taskExPage.checkIfModifiedAndUpdateButtons();
	};

	PageSection.prototype.isModified = function() 
	{
		return this.modified;
	};


	/**
	* Function to expand and collapse page sections.
	*/
	PageSection.prototype.slide = function(switchStatus)
	{
		try {
			// the page section doesn't support collapsing if expand toggle ID is not set
			if (!this.expandToggleID || (this.expandToggleID.length == 0)) return;
			
			var headerExpander = $("#" + this.expandToggleID);
			var body = $("#" + this.bodyID);

			// invert the state of page section expansion whenever it's not explicitly said not to do it
			if ((switchStatus == null) || switchStatus) this.expand = !this.expand;

			if (this.expand) {
				// prepare the content of the section first
				this.generateDynamicContent();
				if (headerExpander) {
					headerExpander.attr("class", "icon icon-normal icon-WCR_Down");
				}
				// make the section content visible (animated)
				body.slideDown("fast");
			} 
			else {
				if (headerExpander) {
					headerExpander.attr("class", "icon icon-normal icon-WCR_Right");
				}
				body.slideUp("fast");
			}
		}
		catch (err) {
			alert("Problem in PageSection.slide() method: " + err);
		}
	};


	//JANH not used!!!
	/**
	* Method to reset GUI of the section.
	* ??? To be overridden ???
	*/
	PageSection.prototype.reset = function()
	{
		// dynamic content of the page section will be regenerated
		this.dynamicContentReady = false;
	};

	//JANH not used!!!
	/**
	* A generalized method to set all needed/known data related to the section.
	* The input parameter is expected to be an array of (identifier, value) pairs.
	* To be overridden.
	*/
	PageSection.prototype.setSectionData = function(data)
	{
		alert("PageSection.setSectionData() should be implemented in subclass");
	};

	/**
	* Method to validate data entered by user in particular section.
	* To be overridden.
	*/
	PageSection.prototype.validateFormParams = function()
	{
		return true;
	};

	/**
	* Method to generate dynamic content of a page section.
	* To be overridden.
	*/
	PageSection.prototype.generateDynamicContent = function()
	{
		// by default there is no dynamic content in page section
	};
	
	return PageSection;
	
});

