
define('layout/Template',["utils/ConvertUtils",
		"jquery",
		"underscore"
	],
	function (ConvertUtils,
		$,
		_) {
		"use strict";

		/**
		 * Create a new Template instance
		 *
		 * @exports Template
		 * @memberof Layout
		 *
		 * @class Template A templating mechanism providing i18n and variables
		 * @example
		 * <div>
		 * 	<h1>This is an example template for <%=d.someVariablename%></h1>
		 * 	<p>
		 * 		template syntax: <%=d.variable%> <%=d.i18n('Language_key')%>
		 *  </p>
		 * </div>
		 *
		 * Template usage:
		 *
		 * var t = new Template(i18n, "some tempalte code");
		 * t.d.someVariable = 'a value';
		 * t.d.someOtherVariable = 'another value';
		 *
		 * t.setContext($("<div/>"));
		 * t.render();
		 *
		 * @param {Object} i18n - This translator object should have at least a method 't' to translate keys
		 * @param {String} template - The template to set
		 */
		var Template = function (i18n, template) {
			this.bindings = {};
			this._context = undefined;
			this.d = {
				i18n: function () {
					return i18n.t.apply(i18n, arguments);
				},
				date: function (value) {
					return ConvertUtils.getLangFormattedDateAndTime(i18n, value);
				},
				/**
				 * Call another sub-template.
				 * The template file should already be loaded before calling this function!! So 'define' it where the main template is created.
				 * @param {String} templateName - Name of the sub template e.g. "pagelist/PagelistBlock"
				 * @param {Object} [data] - Optional extra data for the sub template
				 * @param {String} [dataName] - Optional name of where the extra data can be accessed (via d.dataName). If undefined "d" (so d.d) contains the data (if any)
				 */
				callTemplate: _.bind(function (templateName, data, dataName) {
					if (dataName === undefined) {
						dataName = "d";
					}
					if (data !== undefined) {
						this.d[dataName] = data;
					}
					// this assumes the template is already loaded
					return require("tpl!" + templateName)(this.d);
				}, this)
			};

			this.setTemplate(template);
		};

		/**
		 * Set the template
		 * @param {String} template
		 */
		Template.prototype.setTemplate = function (template) {
			this._parsedTemplate = template;
		};

		/**
		 * Set the HTMLObject in which the template should be rendered
		 * @param {JQueryObj} context
		 */
		Template.prototype.setContext = function (context) {
			if (this._context) {
				this.unbind();
			}
			this._context = context;
			if (context) {
				this.bind();
			}
		};
		
		/**
		 * Return the context of the template.
		 * 
		 * @returns {JQueryObj} The context of the template.
		 */
		Template.prototype.getContext = function () {
			return this._context;
		};

		/**
		 * Render the template
		 */
		Template.prototype.render = function () {
			if (this._context) {
				this._context.html(this._parsedTemplate(this.d));
			}
		};

		/**
		 * Return the rendered HTML
		 */
		Template.prototype.getRenderedHtml = function () {
			return this._parsedTemplate(this.d);
		};

		/**
		 * Bind an event to the template contents
		 * @param {String} type - The type of the event i.e. mouseover, click, ...
		 * @param {String|JQueryIdentifier} identifier - Filter to identify the elements too which the event should be bound
		 * @param {Function} callback - callback which should be callend when the event occurs (arguments: source, event)
		 * @param {Object} [binding] - bind the callback to "binding"
		 */
		Template.prototype.bind = function (type, identifier, callback, binding) {
			if (type === undefined) {
				// rebind all events
				_.each(this.bindings, function (bindingsForType, type) {
					this._context.on(type, bindingsForType.handler);
				}, this);
			} else {
				if (!this.bindings[type]) {
					// no bindings yet for this type, add binding handler
					this.bindings[type] = {};
					this.bindings[type].handler = _.bind(this._handleEvent, this, type);
					this.bindings[type].identifiers = {};
					if (this._context) {
						this._context.on(type, this.bindings[type].handler);
					}
				}

				var handlers = this.bindings[type];
				if (!handlers.identifiers[identifier]) {
					// no callbacks with the same identifier defined, add it
					handlers.identifiers[identifier] = [];
				}

				// push the callback
				if (binding) {
					callback = _.bind(callback, binding);
				}
				handlers.identifiers[identifier].push(callback);
			}
		};

		/**
		 * Unbind an event
		 *
		 * CAUTION: unbind only works for events bound without the "binding" parameter
		 * @param {String} type - The type of the event i.e. mouseover, click, ...
		 * @param {String|JQueryIdentifier} identifier - Filter to identify the elements too which the event should be bound (note same identifier should be specified as in in the binding step)
		 * @param {Function} callback - callback which should be callend when the event occurs (arguments: source, event) (note same callback should be specified as in the binding step
		 */
		Template.prototype.unbind = function (type, identifier, callback) {
			if (type === undefined) {
				// unbind all handlers
				_.each(this.bindings, function (bindingsForType, type) {
					this._context.on(type, bindingsForType.handler);
				}, this);
			} else {
				if (this.bindings[type]) {
					if (this.bindings[type][identifier]) {
						this.bindings[type][identifier] = _.without(this.bindings[type][identifier], callback);
					}

					if (identifier === undefined && callback === undefined) {
						if (this._context) {
							this._context.off(type, this.bindings[type].handler);
						}

						delete this.bindings[type];
					} else if (callback === undefined) {
						delete this.bindings[type][identifier];
					}
				}
			}
		};

		/**
		 * Handle an event, execute the correct callbacks
		 * @param type
		 * @param event
		 */
		Template.prototype._handleEvent = function (type, event) {
			// find all possible identifiers for this event type
			_.each(this.bindings[type].identifiers, function (callbacks, identifier) {
				// find elements mathcing the identifier in the event bubble chain
				var targets = $(event.target).parentsUntil(this._context, identifier);
				// the target itself can also be a possible target!
				if ($(event.target).is(identifier)) {
					targets = $(event.target, targets);
				}

				// find all elements matching the identifier on the event bubble path
				_.each(targets, function (target) {
					_.each(callbacks, function (callback) {
						callback(target, event);
					});
				}, this);
			}, this);
		};


		return Template;
	});

define('tpl!pages/workflow_connection/aesetupjmswizard', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<div class="AEJMFServiceSetupWizard">\r\n	<b>'+
((__t=(d.i18n('WORKFLOWCONNECTION_AE_SETUP_TITLE')))==null?'':_.escape(__t))+
'</b>\r\n	<ol class="AEJMFServiceSetupWizardStepList">\r\n		<li>\r\n			<div>\r\n				'+
((__t=(d.i18n('WORKFLOWCONNECTION_AE_S1_SAMELAN_Q')))==null?'':_.escape(__t))+
'\r\n			</div>\r\n			<div class="AEJMFServiceSetupWizardQuestion">\r\n				<input type="radio" name="AEinLAN" id="SameLAN" value="true" style="vertical-align: middle;" ';
 if (d.isLAN) { 
__p+=' checked="checked" ';
 } 
__p+=' />\r\n				<span style="vertical-align: middle;" >\r\n					'+
((__t=(d.i18n('WORKFLOWCONNECTION_AE_S1_SAMELAN_A1')))==null?'':_.escape(__t))+
'\r\n				</span><br/>\r\n				<input type="radio" name="AEinLAN" id="ConnectedAppTunnel" value="false" style="vertical-align: middle;" ';
 if (!d.isLAN) { 
__p+=' checked="checked" ';
 } 
__p+=' />\r\n				<span style="vertical-align: middle;" >\r\n					'+
((__t=(d.i18n('WORKFLOWCONNECTION_AE_S1_SAMELAN_A2')))==null?'':_.escape(__t))+
'\r\n					\r\n					'+
((__t=(d.help('common/wc/task/ta_wc_ConfigureBackStageApprovalClients')))==null?'':__t)+
'\r\n				</span>\r\n			</div>\r\n		</li>\r\n		';
 if (!d.isLAN) { 
__p+='\r\n		<li>\r\n			<div>\r\n				'+
((__t=(d.i18n('WORKFLOWCONNECTION_AE_S2_APPROVAL_AND_TUNNEL_CLIENT_Q')))==null?'':_.escape(__t))+
'\r\n				';
 if(d.bindingSmartNames.length == 0) { 
__p+='\r\n					<img class="BindingSmartNameLoadingIndicator" src="images/ajax_loader.gif"/>\r\n				';
 } 
__p+='\r\n			</div>\r\n			<div class="AEJMFServiceSetupWizardQuestion">\r\n				<div class="input-group">					\r\n				<select name="TunnelBindings" id="TunnelBindings" class="form-control">\r\n					';
 if(d.bindingSmartNames.length > 0) { 
__p+='\r\n						<option value=""></option>\r\n						';
 _.each(d.bindingSmartNames, function(smartName){ 
__p+='\r\n							<option value="['+
((__t=(smartName.getFullName()))==null?'':_.escape(__t))+
']">'+
((__t=(smartName.getArguments()[0]))==null?'':_.escape(__t))+
' ('+
((__t=(smartName.getArguments()[1]))==null?'':_.escape(__t))+
')</option>\r\n						';
 }); 
__p+='\r\n					';
 } 
__p+='			\r\n				</select>\r\n				\r\n				<span class="input-group-btn add-padding-left">&nbps;<i data-toggle="tooltip" data-placement="top" title="'+
((__t=(d.i18n('WORKFLOWCONNECTION_TUNNEL_LIC_WARNING')))==null?'':_.escape(__t))+
'" class="icon icon-normal icon-WCR_Msg-Warning"></i></span>\r\n				\r\n				</div>\r\n			</div>\r\n			<div class="BindingSmartNameLoadingErrorBox" style="display: none"></div>\r\n			<div class="AEJMFServiceSetupWizardHint help-block">\r\n				<strong>'+
((__t=(d.i18n('TIP')))==null?'':_.escape(__t))+
'</strong>'+
((__t=(d.i18n('WORKFLOWCONNECTION_AE_S2_APPROVAL_AND_TUNNEL_CLIENT_HINT')))==null?'':_.escape(__t))+
'\r\n			</div>\r\n		</li>\r\n		';
 } 
__p+='\r\n		';
 if (d.isLAN) { 
__p+='\r\n		<li>\r\n			<div>\r\n				'+
((__t=(d.i18n('WORKFLOWCONNECTION_AE_S2_HOSTNAME_Q')))==null?'':_.escape(__t))+
'\r\n			</div>\r\n			<div class="AEJMFServiceSetupWizardQuestion">\r\n				<input name="AEDomainName" type="text" class="form-control" />\r\n			</div>\r\n			<div class="AEJMFServiceSetupWizardHint help-block">\r\n				<strong>'+
((__t=(d.i18n('TIP')))==null?'':_.escape(__t))+
'</strong>'+
((__t=(d.i18n('WORKFLOWCONNECTION_AE_S2_HOSTNAME_HINT')))==null?'':_.escape(__t))+
'\r\n			</div>\r\n		</li>\r\n		';
 } 
__p+='\r\n		<li>\r\n			<div>\r\n				<button type="button" name="SetupURLButton" class="btn btn-default" onclick="">\r\n					'+
((__t=(d.i18n('WORKFLOWCONNECTION_AE_S3_SETUP_URL')))==null?'':_.escape(__t))+
'\r\n				</button>\r\n			</div>\r\n		</li>\r\n	</ol>\r\n</div>';
return __p;
}});
/**
 * Encodes/Decodes SmartName arguments and names from/to a string.
 */

/*
 *
 * !!! THE IMPLEMENTATION IN THIS FILE SHOULD MATCH THE IMPLEMENTATION IN THE JAVA FILE "SmartNameArgumentEncoding.java".
 *
 */

define('component/smartnames/encoding/smartnameargumentencoding',["utils/wcrerror", 
        "utils/extendedstring"],
        function(WCRError) {
	"use strict";
	var exports={};
	
	var SmartNameArgumentEncoding = function (argumentDelimeterChar, argumentEscapeChar) {
		
		this._delimeterChar = argumentDelimeterChar;
		this._escapeChar = argumentEscapeChar;
	};
	
	(function () {
		
		/**
		 * Encode the name and arguments of a SmartName.
		 * 
		 * Executing {@link #decodeSmartNameArguments(String)} on the result of this method should return exactly the same
		 * input arguments used for this method (name will be the first element in the returned array).
		 * 
		 * @param name [String] The name of the SmartName.
		 * @param arguments [Array] The arguments of the SmartName.
		 * @return [String] The encoded String.
		 */
		this.encodeSmartName = function (name, args) {
			
			// The result...
			var result = name;
			for (var i=0; i <args.length; i++) {
				// Escape every escape char.
				var arg1 = args[i].replaceAll(this._escapeChar, this._escapeChar + this._escapeChar);
				// Escape every delimeter char
				var arg2 = arg1.replaceAll(this._delimeterChar, this._escapeChar + this._delimeterChar);
				// Concatenate the argumente delimited by the argumentDelimeterChar.
				result += this._delimeterChar;
				result += arg2;
			}

			return result.toString();
			
		};
		
		
		/**
		 * Decode a String that represents a SmartName with its arguments.
		 * 
		 * The string representation of the smart name concatenates the arguments to the name but seperates them through the
		 * delimeter character of this encoding. If an argument contains this character, it needs to be escaped by the
		 * escape character of this encoding. Occurences of the escape character need to be escaped by themselves.
		 * 
		 * @param str [String] The String that needs to be decoded.
		 * @return The [Array] first element of this list is the name of the SmartName, followed by the seperate arguments.
		 */
		this.decodeSmartNameArguments = function (str) {
			var result = [];
			
			// This mini FSM parses the string.
			var escaped = false;
			var part = "";
			for (var i = 0; i < str.length; i++) {
                var c = str.charAt(i);
                if (c === this._escapeChar) {
                    if (escaped) {
                        part+=this._escapeChar;
                    }
                    escaped = !escaped;
                } else if (c === this._delimeterChar) {
    				// Check if we are escaped
    				if (escaped) {
    					part += this._delimeterChar;
    					escaped = false;
    				} else {
    					result.push(part);
    					part = "";
    				};
    			} else {
    				if (escaped) {
    					// A normal character was escaped ... throw error
    					throw new WCRError("SMARTNAME_ARGUMENT_PARSING_ILLEGAL_ESCAPE_CHARACTER", "Parsing of arguments of SmartName failed. Found a single escape character in the string.");
    				} else {
    					// normal letter, add it.
    					part += c;
    				};
    			};
			}
			// Add the last part the results.
			result.push(part);
			
			// Parsing is done and we should not be escaped anymore ... throw error otherwise
			if (escaped) {
				throw new WCRError("SMARTNAME_ARGUMENT_PARSING_ILLEGAL_ESCAPE_CHARACTER", "Parsing of arguments of SmartName failed. Found a single escape character in the string.");
			}

			// Return result
			return result;
		};
		
	}).call(SmartNameArgumentEncoding.prototype);
	
	
	/**
	 * The default SmartNameArgumentEncoding
	 */
	exports.DEFAULT = new SmartNameArgumentEncoding(":","\\");
	
	
	/**
	 * Export Constructor (for JUnit test)
	 */
	exports.constructor = SmartNameArgumentEncoding;
	
	return exports;
});

/**
 * SmartNames - Encoding of SmartNames & normal text in a string
 */

/*
 *
 * !!! THE IMPLEMENTATION IN THIS FILE SHOULD MATCH THE IMPLEMENTATION IN THE JAVA FILE "SmartName.java".
 *
 */

define('component/smartnames/encoding/smartname',["../encoding/smartnameargumentencoding"], function (SmartNameArgumentEncoding) {
	"use strict";


	/**
	 * The implementation of this constructor depends on the fact whether the "arguments" parameter is 
	 * specified or not.
	 * 
	 * Situation I: "arguments" is undefined:
	 *  
	 * 		Create a SmartName from the string representation of the smart name.
	 * 
	 * 		The string representation of the smart name concatenates the arguments to the name but seperates
	 * 		them through the ":" character. If an argument contains this character, it needs to be escaped by
	 * 		a "\". Occurences of "\" need to be escaped by "\" themselves.
	 * 
	 * Situation II: "arguments" is not undefined.
	 * 
	 * 		Create a SmartName, from a name and arguments.
	 * 
	 * 		An example of a SmartName with arguments is "[ProjAttr:Artist_Name]", herein is "ProjAttr" the 
	 * 		name and "Artist_Name" is an argument of the SmartName.	
	 * 
	 * @param smartName
	 *          The string representations of the smart name or the name of the SmartName (depending on whether 
	 *          arguments is specified).
	 * @param arguments
	 * 			When specified, the arguments of the SmartName.
	 */
	var SmartName = function () {

		if (arguments.length === 1) {
			// Split the string on ":", the name is the first part, the rest are arguments.
			// When an argument contains ":" chars, they need to be escaped by a "\". Occurences 
			// of "\" need to be escaped by "\" themselves.
			var parts = SmartNameArgumentEncoding.DEFAULT.decodeSmartNameArguments(arguments[0]);

			// The name is always the first entry in the returned array.
			this._name = parts[0];

			// The other elements are arguments.
			if (parts.length > 1) {
				this._arguments = parts.slice(1, parts.length);
			} else {
				this._arguments = [];
			}

		} else if (arguments.length > 1) {

			// name and arguments are seperately provided, no need to parse.
			this._name = arguments[0];

			// If only 2 arguments are given and the second one is an array, these are the SmartName arguments.
			if (arguments.length == 2 && arguments[1].constructor == Array) {
				this._arguments = arguments[1];
			} else {
				// Otherwise all other arguments are the SmartName arguments.
				this._arguments = Array.prototype.slice.call(arguments);
				this._arguments = this._arguments.slice(1, this._arguments.length);
			}


		} else if (arguments.length < 1) {
			throw new Error("A SmartName requires at least one 'name' parameter.");
		}

	};

	(function () {

		/**
		 * Get the name of the SmartName
		 * 
		 * @return [String] the name of the SmartName
		 */
		this.getName = function () {
			return this._name;
		};

		/**
		 * Get the arguments of the SmartName
		 * 
		 * @return [Array] the arguments of the SmartName
		 */
		this.getArguments = function () {
			return this._arguments;
		};

		/**
		 * Return the full name of this string including the contacenated escaped arguments.
		 * 
		 * @return [String] The full name of this string including the contacenated escaped arguments.
		 */
		this.getFullName = function () {
			return SmartNameArgumentEncoding.DEFAULT.encodeSmartName(this._name, this._arguments);
		};

		/**
		 * Are these SmartNames the same?
		 * 
		 * @return [Boolean] Are these SmartNames the same?
		 */
		this.equals = function (other) {
			if (other instanceof SmartName) {
				var nameMatches = other.getName() === this.getName();
				var argumentsMatches = this._compareArguments(other);
				return nameMatches && argumentsMatches;
			} else {
				return false;
			}
		};

		/**
		 * A private helper function of the public "equals" method.
		 */
		this._compareArguments = function (other) {
			if (this._arguments.length === other.getArguments().length) {
				for (var i = 0; i < this._arguments.length; i++) {
					if (this._arguments[i] !== other.getArguments()[i]) {
						return false;
					}
				}
				return true;
			} else {
				return false;
			}
		};

	}).call(SmartName.prototype);

	return SmartName;
});

/**
 * TunnelService
 *
 * @author KADA
 * @since 26.2.2015
 */
define('pages/workflow_connection/tunnel-service',["jquery","component/smartnames/encoding/smartname"], function ($, SmartName) {
	
	"use strict";
	
	var TunnelService = (function (){
			
			// Instance stores a reference to the Singleton
			var instance;
			
			function init() {
				
				// The baseUrl of the service.
		        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
				
				return {
					
					getBindingSmartNames : function (onSuccesCallback, onFailCallback) {
						
						// Do the async call to the service
						$.ajaxSetup({ cache: false });
	            		$.getJSON( jspBaseUrl+"tunnelmonitor.wcr?action=GET_BINDING_SMARTNAMES", function( data ) {

	            			var smartNames = [];
	            			for (var i=0; i < data.length; i++) {
	            				smartNames.push(new SmartName(data[i].name, data[i].arguments));
	            			}
	            			
	            			onSuccesCallback(smartNames);
	            			
	                	}).fail(function(jqxhr, textStatus, error) {
	                		// Try to parse the error.
	        	    		var msg;
	        	    		try {
	        	    			msg =JSON.parse(jqxhr.responseText).error.message;
	        	    		} catch (e) {
	        	    			msg = error;
	        	    		}
	        	    		// Execute the callback.
	                		onFailCallback(msg);
	                	});
					}
				};
			}
			
			return {
				// Get the Singleton instance if one exists
				// or create one if it doesn't
				getInstance : function() {
					if (!instance) {
						instance = init();
					}
					return instance;
				}
			};
	})();
	    
	return TunnelService;

});

define('lib/text!pages/workflow_connection/aesetupjmswizard-style.css',[],function () { return '.AEJMFServiceSetupWizard {\r\n\tmargin-top: 5px;\r\n\tmargin-left: 5px;\r\n}\r\nhr.AEJMFServiceSetupWizardTop {\r\n    border: 0;\r\n    height: 0;\r\n    border-top: 1px solid rgba(0, 0, 0, 0.1);\r\n    border-bottom: 1px solid rgba(255, 255, 255, 0.3);\r\n}\r\np.AEJMFServiceSetupWizardIntro {\r\n\tmargin-top: 10px;\r\n}\r\nul.AEJMFServiceSetupWizardStepList {\r\n\tmargin-left: 10px;\r\n\tmargin-top: 10px;\r\n\tmargin-bottom: 10px;\r\n}\r\nul.AEJMFServiceSetupWizardStepList>li{\r\n\tmargin-top: 10px;\r\n}\r\nol.AEJMFServiceSetupWizardStepList {\r\n\tmargin: 10px;\r\n\tmargin-bottom: 5px;\r\n}\r\nol.AEJMFServiceSetupWizardStepList>li {\r\n\tmargin-top: 20px;\r\n}\r\n.AEJMFServiceSetupWizardQuestion {\r\n\tmargin-top: 10px;\r\n}\r\n.AEJMFServiceSetupWizardHint {\r\n\tmargin-top: 8px;\r\n}\r\n.BindingSmartNameLoadingIndicator {\r\n\theight: 12px;\r\n}\r\n.TunnelWorkflowConnectionLicWarn {\r\n\theight: 12px;\r\n}\r\n.BindingSmartNameLoadingErrorBox {\r\n\tcolor: red;\r\n\tmargin-top: 4px;\r\n}\r\n.BindingSmartNameLoadingErrorIcon {\r\n\tvertical-align: bottom;\r\n\theight: 10px;\r\n}';});

/**
 * When a dependency is made to this module, the "aesetupjmswizard-style.css" will get inserted in the DOM.
 */

define('pages/workflow_connection/aesetupjmswizard-style',['require','exports','module','lib/text!./aesetupjmswizard-style.css','lib/dom'],function(require, exports, module) {
	exports.cssText = require("lib/text!./aesetupjmswizard-style.css");
	var dom = require("lib/dom");
	dom.importCssString(exports.cssText);
});

/**
 * AEEngineSetup
 *
 * @author KADA
 * @since 9.2.2015
 */
define('pages/workflow_connection/aesetupjmswizard',["jquery",
        "layout/Template",
        "tpl!pages/workflow_connection/aesetupjmswizard",
        "utils/webi18n!AESetupJMSWizard",
        "./tunnel-service",
        "component/helpIcon/HelpIcon",
        "./aesetupjmswizard-style"
        ], function ($, Template, tpl, webI18n, TunnelService, HelpIcon) {
	
	"use strict";

	/**
	 * 
	 */
	var AESetupJMSWizard = function (element) {
		
		// Wrap the element with jQuery if this is not the case ...
		if (!(element instanceof jQuery)) {
        	this._element = $(element);
        }
		
		// Create the template
		this._t = new Template(webI18n, tpl);
		
		// Add the template to the element.
		this._t.setContext(this._element);
		
		// Setup initial data
		this._t.d.isLAN = false;
		this._t.d.hostName = "";
		this._t.d.tunnelName = "";
		this._t.d.bindingSmartNames = [];
		this._t.d.help = HelpIcon.getInlineHelpTopicHTML;
		
		// What needs to happen when the "AEinLAN" dropdrown is changed.
		this._t.bind("change", "input[name='AEinLAN']", function(source, event){
			
			// Find out if we're now in a LAN 
			this._t.d.isLAN = !this._t.d.isLAN ;
			
			// We need te rerender the template (to show different steps).
			this._t.render();
			
		}, this);
		
		// When the "Setup URL" button is clicked.
		this._t.bind("click", "button[name='SetupURLButton']", function(source, event){
			// Setup the url in the final URL input.
			this.setupURL($(this._finalURLInputSelector));
		}, this);
		
		// Render
		this._t.render();
		
		// Fetch all the active HTTP tunnel bindings
		this._tunnelService = TunnelService.getInstance();
		
		// Start fetching the binding SmartNames
		this.startMonitoringBindingSmartNames(3000);
		
	};
	
	/**
	 * The jQuery selector to get the final URL input.
	 */
	AESetupJMSWizard.prototype._finalURLInputSelector = "#JMFService";
	
	/**
	 * The jQuery selector to get the button to test the final URL input.
	 */
	AESetupJMSWizard.prototype._finalURLCheckButtonSelector = "#urlcheckJMFService";
	
	
	/**
	 * Start monitoring the Binding SmartNames.
	 */
	AESetupJMSWizard.prototype.startMonitoringBindingSmartNames = function (interval) {
		
		var self = this;
		function monitor () {
			setTimeout(function(){
				// Request the current Binding SmartNames
				self._tunnelService.getBindingSmartNames(function (bindingSmartNames) {
					// OnSuccess, update the Binding Smartnames
					self.updateBindingSmartNames(bindingSmartNames);
					// Request new Binding SmartNames (after 1 sec).
					monitor();
				}, function (errorMsg) {
					// Something went wrong requesting the tunnelInfo, show the error (if not paused).
					self.showErrorFetchingBindingSmartNames(errorMsg);
					// Request new tunnelMonitorService update.
					monitor();
				});
			}, interval);
		}
		monitor();
	};
	
	
	/**
	 * Update the bindings.
	 * 
	 * @param bindingSmartNames
	 */
	AESetupJMSWizard.prototype.updateBindingSmartNames = function(bindingSmartNames) {
		
		// Save the smartnames in the template ...
		this._t.d.bindingSmartNames = bindingSmartNames;
		
		// Hide potential error message.
		this.hideErrorFetchingBindingSmartNames();
		
		// Get the dropdown (if it exists)
		var dropDown = this._element.find("select[name='TunnelBindings']");
		
		// Check if the dropdown does exist.
		if (dropDown.length > 0) {
			
			// Remove the loading icon:
			$(".BindingSmartNameLoadingIndicator").hide();
			
			// Remember the selected option value
			var selectedOptionVal = dropDown.val();
			
			// Clear all options.
			dropDown.find('option').remove();
			
			// Always add an "empty" option.
			dropDown.append($("<option></option>").attr("value","").text(""));
			
			// Add all these smartnames to the dropdown.
			for (var i=0; i < bindingSmartNames.length; i++) {
				var smartNameStr = "["+bindingSmartNames[i].getFullName()+"]";
				var smartNameTxt = bindingSmartNames[i].getArguments()[0]+" ("+bindingSmartNames[i].getArguments()[1]+")";
				dropDown.append($("<option></option>").attr("value",smartNameStr).text(smartNameTxt)); 
			}
			
			// Try to select the previous selected value.
			dropDown.find("option[value='"+selectedOptionVal+"']").attr("selected","selected");
		}
	};
	
	
	/**
	 * Set and show the error msg.
	 * 
	 * @param errorMsg
	 */
	AESetupJMSWizard.prototype.showErrorFetchingBindingSmartNames = function(errorMsg) {
		var errorBox = $("div.BindingSmartNameLoadingErrorBox");
		if (errorBox.length > 0) {
			errorBox.empty();
			var errorSpan = $("<span/>");
			errorSpan.text(webI18n.t("WORKFLOWCONNECTION_AE_FETCHING_ERROR_BINDING_SMARTNAMES")+errorMsg);
			errorBox.append("<img class='BindingSmartNameLoadingErrorIcon' src='images/rejected.gif'/>");
			errorBox.append(errorSpan);
			errorBox.show();
		}
		
	};
	
	/**
	 * Hide the error msg.
	 * 
	 * @param errorMsg
	 */
	AESetupJMSWizard.prototype.hideErrorFetchingBindingSmartNames = function() {
		var errorBox = $("div.BindingSmartNameLoadingErrorBox");
		if (errorBox.length > 0) {
			errorBox.hide();
		}
		
	};
	
	/**
	 * 
	 */
	AESetupJMSWizard.prototype.setupURL = function(urlInput) {
		
		var protocol = "http://";
		var port = ":4411";
		var servicePath = "/JDFP/JMF";
		
		// Is AE in a LAN?
		if (!this._t.d.isLAN) {
			// Retrieve the dropdown that contains the Tunnel Binding SmartNames
			var tunnelBindingSmartNameDropDown = this._element.find("select[name='TunnelBindings']");
			// Check if the dropdown exists
			if (tunnelBindingSmartNameDropDown.length > 0) {
				// Get the selected Tunnel Binding SmartName
				var tunnelBindingSmartName = tunnelBindingSmartNameDropDown.val();
				// Check if one is selected.
				if (tunnelBindingSmartName != "") {
					// Create the URL
					var URL = protocol+tunnelBindingSmartName+servicePath;
					// Insert it in the URL Input
					urlInput.val(URL);
					// Test the url
					$( this._finalURLCheckButtonSelector ).trigger("click");
				} else {
					alert(webI18n.t("WORKFLOWCONNECTION_AE_SETUP_NEED_TUNNEL"));
				}
			} else {
				alert(webI18n.t("WORKFLOWCONNECTION_AE_SETUP_NEED_TUNNEL"));
			}
		} else {
			// Retrieve the domainName that contains the Tunnel Binding SmartNames
			var domainNameInput = this._element.find("input[name='AEDomainName']");
			// Check if the input exists
			if (domainNameInput.length > 0) {
				// Get the selected Tunnel Binding SmartName
				var domainName = domainNameInput.val();
				// Check if a host is specified.
				if (domainName != "") {
					// Create the URL
					var URL = protocol+domainName+port+servicePath;
					// Insert it in the URL Input
					urlInput.val(URL);
					// Test the url
					$( this._finalURLCheckButtonSelector ).trigger("click");
				} else {
					alert(webI18n.t("WORKFLOWCONNECTION_AE_SETUP_NEED_HOST"));
				}
			} else {
				alert(webI18n.t("WORKFLOWCONNECTION_AE_SETUP_NEED_HOST"));
			}
		}
	};
	
	
    /**
     * Method to automatically apply this module to a div element ...
     */
	AESetupJMSWizard.wcrinit = function(element){
        new AESetupJMSWizard(element);
    };
	
	return AESetupJMSWizard;
	
});
	
