define(["Template",
		"tpl!upload/ApprovalFields",
		"utils/webi18n!UploadTool",
		"utils/AjaxCache",
		"jquery",
        "underscore"],
	function (Template,
			tpl,
			i18n,
			AjaxCache,
			$,
			_) {
		"use strict";

		/**
		 * Add the approval input fields
		 * 
		 * @class
		 * @exports ApprovalFields 
		 * @constructor
		 */
		var ApprovalFields = function(configuration) {
			this.ctx = configuration.element || $("<div/>");
			
			/**
			 * @member {Map} - maps folder ID on folder configuration
			 */
			this.folderConfigMap = {};
			
			this.t = new Template(i18n, tpl);
			this.t.setContext(this.ctx);	
			this.t.d.prefix = _.uniqueId();
			this.t.d.baseUrl = window.webcenterConfig.jspBaseUrl || './';
		};

		/**
		 * Set configuration
		 * @param {Object} config
		 */
		ApprovalFields.prototype.setConfig = function(config, projectId, folderId){
			this.approvalRestartOption = config.approvalRestartOption;
			this.approvalRestartUploaderCanOverrule = config.approvalRestartUploaderCanOverrule;
			
			this.projectId = projectId;
			folderId = folderId || "";
			this.folderConfigMap[folderId] = {
				existProjectApproval: config.existProjectApproval,
				canStartApprovalCycle: config.canStartApprovalCycle
			};
			
			this.setFolders([folderId]);
		};
		
		/**
		 * Set selected folders and update visibility
		 * @param {Array.String} list of folderids
		 */
		ApprovalFields.prototype.setFolders = function(folderIds){
			var unknownFolders = [];
			this.existProjectApproval = false;
			this.canStartApprovalCycle = false;
			
			// first check all known folder
			_.each(folderIds, _.bind(function(folderId){
				var config = this.folderConfigMap[folderId];
				if(config){
					this.existProjectApproval = this.existProjectApproval || config.existProjectApproval;
					this.canStartApprovalCycle = this.canStartApprovalCycle || config.canStartApprovalCycle;
				} else {
					unknownFolders.push(folderId);
				}
			}, this));
			
			// only if not everything should be shown AND there are still unknown projects, 
			// do an Ajax for the unknown projects, as they may add visibility
			if((!this.existProjectApproval || !this.canStartApprovalCycle) && unknownFolders.length > 0){
				var promises = [];
				// do an ajax call for each folder because: 
				// - normally it's just one anyway (event triggers whenever a folder changes)
				// - having it per folder in the cache is better for reuse
				_.each(unknownFolders, _.bind(function(folderId){
					var url = "asyncupload.wcr?action=GET_FOLDER_INFO";
					url += "&projectId=" + this.projectId;
					url += "&folderId=" + folderId;

					var promise = AjaxCache.ajax({
						url: url,
						dataType: "json",
						cache: false
					});
					promises.push(promise);
					$.when(promise).done(_.bind(function (data) {
						this.folderConfigMap[folderId] = data.result;
					}, this));
				}, this));
				
				// when all folders are loaded, retry setting the configuration for all folders
				$.when.apply($, promises).done(_.bind(function(){
					this.setFolders(folderIds);
				}, this));
				
			} else {
				this.render();
			}
		};

		/**
		 * Render the fields
		 */
		ApprovalFields.prototype.render = function(){
			// get previous values before removing from DOM
			var data = {
					apply: this.ctx.find(":input[name=doApplyDefApproval]").is(":checked"),
					start: this.ctx.find(":input[name=doStartApproval]").is(":checked"),
					date: this.ctx.find(":input[name=due_date]").val(),
					datelong: this.ctx.find(":input[name=longFormat_due_date]").val(),
					time: this.ctx.find(":input[name=due_periodOfDay]").val()
			};
			
			// render
			this.t.render();
			this.ctx.find(".form-group").hide();
			
			// add datepicker
			this.ctx.find(":input[name=longFormat_due_date]").datepicker({
				dateFormat: "M dd, yy", 
				onClose: _.bind(function(dateText, inst) {
					var element = this.ctx.find(":input[name=due_periodOfDay]")[0];
					if(element && element.selectedIndex == 0){
						element.selectedIndex = (36 + 1);
					}
					var dateVal = dateText;
					if (dateText != '')	{
						var date = new Date(dateText);
						dateVal = formatDate(date, "MM/dd/yyyy");
					}
					this.ctx.find(":input[name=due_date]").val(dateVal);																	
				}, this)
			});
			
			// implement interactivity
			if(this.existProjectApproval){
				var copyRow = this.ctx.find("div[name=doApplyDefApprovalSettings]");
				copyRow.show();
				
				var copyElem = this.ctx.find(":input[name=doApplyDefApproval]");
				copyElem.on("change", _.bind(function(e){
					if(copyElem.is(":checked")){
						// show other fields
						if(this.approvalRestartUploaderCanOverrule && this.canStartApprovalCycle){
							this.ctx.find("div[name=doStartApprovalRow]").show();
						} else {
							if(this.approvalRestartOption == "1"){
								this.ctx.find("div[name=AppAlwaysStartRow]").show();
							} else {
								this.ctx.find("div[name=AppNeverStartRow]").show();
							}
						}
						this.ctx.find("div[name=AppDueDateLabelRow]").show();
						this.ctx.find("div[name=AppDueDateRow]").show();
					} else {
						this.ctx.find(".form-group").not(copyRow).hide();
					}
				}, this));
			}
			
			// fill out data
			if(data.apply && this.existProjectApproval){
				this.ctx.find(":input[name=doApplyDefApproval]").attr('checked', true).change();
				
				var startApproval = data.start && this.approvalRestartUploaderCanOverrule && this.canStartApprovalCycle;
				this.ctx.find(":input[name=doStartApproval]").prop('checked', startApproval);
				this.ctx.find(":input[name=due_date]").val(data.data);
				this.ctx.find(":input[name=longFormat_due_date]").val(data.datelong);
				this.ctx.find(":input[name=due_periodOfDay]").val(data.time);
			}
		};
		
		/**
		 * Add data to form if needed
		 */
		ApprovalFields.prototype.getElement = function(){
			return this.ctx;
		};
		
		/**
		 * Add data from approval fields to passed form.
		 * @param {FormData} formData - form to which the data should be appended 
		 */
		ApprovalFields.prototype.addToForm = function(formData){
			if(this.existProjectApproval){
				var copyRow = this.ctx.find("input[name=doApplyDefApproval]").is(":checked");
				if(copyRow){
					formData.append("copyapproval", "1");
					
					formData.append("duedate", this.ctx.find("input[name=due_date]").val());
					formData.append("duedatetime", this.ctx.find("select[name=due_periodOfDay]").val());
					
					if((this.approvalRestartOption == "1" && !this.approvalRestartUploaderCanOverrule) || this.ctx.find("input[name=doStartApproval]").is(":checked")){
						formData.append("startcycle", "1");
					}
				}
			}
		};

		return ApprovalFields;
});