define(["doctablex/DocTableXFactory",
        "doctablex/DocTableXRenderConfig",
        "doctablex/StaticDocumentRow",
        "component/smartnames/savedsearchcontext/SavedSearchContextInfo",
        "utils/webi18n!DocSrcHdlrStrings",
        "utils/webi18n!UploadTool",
        "utils/Observable",
        "utils/EventBasedValidator",
        "wcr/documentSourceHandler",
        "jquery",
		"underscore"
	],
	function (DocTableXFactory,
			DocTableXRenderConfig,
			StaticDocumentRow,
			SavedSearchContextInfo,
			DocSrcHdlrStrings,
			i18n,
			Observable,
			EventBasedValidator,
			documentSourceHandler,
			$,
			_) {
		"use strict";

		/**
		 * This class is in control of the DocTableX.
		 * All operations that need to be pushed to the DocTable go through here
		 * 
		 * @class
		 * @exports DocTableController 
		 * @mixes Observable
		 * @constructor
		 * @param {jQueryElem} ctx - context to render tool in
		 * @param {String} projectId
		 */
		var DocTableController = function(ctx, projectId, folderId) {
			this._id = _.uniqueId();
			Observable.mixin(this, this._id);
			
			// create context for visible and hidden elements
			this._ctx = ctx;
			this._doctableDomElem = $("<div />").attr('id', this._id);
			this._hiddenDomElem = $("<div />");
			this._docsourceElem = $("<div />").addClass("hiddenElement").attr('id', _.uniqueId());
			ctx.append(this._doctableDomElem).append(this._hiddenDomElem);
			this._hiddenDomElem.append(this._docsourceElem);
			
			this._docTableObj = null;
			this._dshObj = null;
			this.projectId = projectId;
			this.folderId = folderId;
			this.folderName = null;
			
			/**
			 * @member {Map} uploadedRows: keep track of which rows have been uploaded and which haven't
			 */
			this._uploadedRows = {};
			this._uploadedDocumentsInfo = {};
		};
		
		/**
		 * Initialize DocTableX based on project configuration
		 */
		DocTableController.prototype.setConfig = function(config){
			var docSourceXML = config.docSourceXML;
			var docRenderConfigXML = config.docRenderConfigXML; 
			var showThumbnails = config.showThumbnails;
			var currentUser = ObjectHandler.getCloneOfObject(gCurrentUser);
			var targetFolderName = config.targetFolderName;
			
			// get render config and document sources
	        var docTableRenderOptions = this._getDocTableRenderOptions(docRenderConfigXML, showThumbnails);
            this._dshObj = this._getDocSourceConfig(docSourceXML, currentUser);
			
	        // create documents table
            this._docTableObj = DocTableXFactory.createDocTableX({
	            prefix : this._id + '_',  	// main prefix to be used by this DocTableX instance
	            currentUser : currentUser,
	            projectID : this.projectId,
	            phElemID : this._id,   		// HTML element ID, where this doc table will be put
	            dshObj : this._dshObj,  	// link between DocTableX and DocumentSourceHandler objects
	            renderOptions : docTableRenderOptions, 
	            onChangeCallback : _.bind(this._tableUpdated, this),
	            saveXMLRootTag : 'drs',
	            isEditable : true,
	            allowMultiple : true,
	            showGreenLinksOnTop : true
	        });
			
            this.setTargetFolder(targetFolderName);
			this._dshObj.setAdvancedClickMethods('UploadTool', this._docTableObj.thisInDOM);
			this._docTableObj.displayTable();
		};
		
		/**
		 * Create the rendering options for the DocTableX
		 * 
		 * @param {String} docRenderConfigXML
		 * @param {Boolean} showThumbnails
		 * @returns {DocTableRenderOptions}
		 */
		DocTableController.prototype._getDocTableRenderOptions = function(docRenderConfigXML, showThumbnails){
			var docTableRenderConfigObj = new DocTableXRenderConfig({
				defaultName : 'Custom Rendering Configuration',
				section : 'UploadTool',
				configObj : null
			});

			// load the rendering configuration from provided XML (if any)
			if (docRenderConfigXML) {
				docTableRenderConfigObj.readFromXMLString(docRenderConfigXML);
			}

			// add few more implicit (not configurable) render options
			var docTableRenderOptions = docTableRenderConfigObj.getConfigObj();
			docTableRenderOptions.renderMethod = 'table'; // TODO BVON: based on config
	        docTableRenderOptions.showGreenLinksOnTop = true;
	        docTableRenderOptions.showGreenLinksAtBottom = false;
	        docTableRenderOptions.showThumbnails = showThumbnails;
	        
	        return docTableRenderOptions;
		};
		
		/**
		 * Create document source handler for DocTableX
		 * 
		 * @param {String} docSourceXML
		 * @param {Object} currentUser
		 * @returns {DocumentSourceHandler}
		 */
		DocTableController.prototype._getDocSourceConfig = function(docSourceXML, currentUser){
			var savedSearchContextInfo = new SavedSearchContextInfo();
     		savedSearchContextInfo.setProjectId(this.projectId);
            var dshObj = new DocumentSourceHandler({
                docSourceXML : docSourceXML,
                docSourceTableId : this._docsourceElem.attr('id'),
                functionPrefix : 'javascript:document.getElementById(\'' + this._docsourceElem.attr('id') + '\').dshObj',			
                prefix : this._doctableDomElem.attr('id'),		
                currentLanguage : currentUser ? currentUser.lang : null,
                textClass : 'darktext_greenlink_11',
                linksId : null,   						// managed by DocTableX internally
                isDocReferenceSection : false,   		// makes visible the "By reference" document source
                savedSearchContextInfo : savedSearchContextInfo
            }, DocSrcHdlrStrings);

            // set the WebCenter defaults if not defined
            if (!docSourceXML) {						
            	var section = "UploadTool";
                dshObj.setWebCenterDefault(section); 			
            }
            
            return dshObj;
		};
		
		/**
		 * Emit event when DocTable is updated
		 */
		DocTableController.prototype._tableUpdated = function(){
			this._emit("doctable.updated", this._docTableObj);
		};
		
		/**
		 * Get DOM Element containing the DocTable 
		 * 
		 * @returns {JQueryObj}
		 */
		DocTableController.prototype.getElement = function(){
			return this._doctableDomElem;
		};
		
		/**
		 * Add file to the DocTable
		 * 
		 * @param {File} file
		 */
		DocTableController.prototype.addFile = function(file){
			if(!this._dls && this._dshObj){
				this._dls = this._dshObj.createDlsObj(0);
				if(this.folderName){
					this._dls.targetFolder = this.folderName;
				}
			}
			
			this._docTableObj.addFileRow(this._dls, file, false);
		};
		
		/**
		 * Repaint the DocTable
		 */
		DocTableController.prototype.repaintTable = function(){
			this._docTableObj.displayTable();
		};

		/**
		 * Change the target folder for upload
		 * 
		 * @param {String} folderName
		 */
		DocTableController.prototype.setTargetFolder = function(folderName){
			// change all rows
			var rows = this._docTableObj.docRows;
			if(rows.length > 0){
				_.each(rows, _.bind(function(row){
					if(!row.targetFolderName || (this.folderName && (this.folderName == row.targetFolderName))){
						row.setTargetFolderByName(folderName);
					}
				}, this));
			}
			
			// change dls
			if(this._dls){
				this._dls.targetFolder = folderName;
			}
			
			this.folderName = folderName;
		};
		
		DocTableController.prototype.startUpload = function(){
			var ctr = 0;
			var hasFailures = false;
			var hasWarnings = false;
			
			// only do the upload for changed rows that have not been submitted yet
			var rows = _.filter(this._docTableObj.docRows, _.bind(function(row){
				return !this._uploadedRows[row.rowID] && row.getIsChanged();
			}, this));
			if(rows.length > 0){
				this._emit("upload.start", rows);
				
				// first do a unique name check
				var nameValidationObj = this.checkAllNames();
				
				_.each(rows, _.bind(function(row){

					// now do a frontend check
					var validObj = {
		   	    		errors : nameValidationObj[row.rowID] || [], // start with already obtained errors
			    		isOK : function() {
			    			return (this.errors.length == 0);
			    		}, 
			    		newDocs: [],
			    		newDocsDOMElems: {}
			    	};
					row.validateDocRow(validObj, 1);
					if(!validObj.isOK()){
						var errors = [];
						_.each(validObj.errors, function(err){
							errors.push(err.msg);
						});
						this.rowUploadFailed(row, errors);
						hasFailures = true;
						return;
					}
					
					this._uploadedRows[row.rowID] = true;
					
					var formData = new FormData();
					formData.append("projectId", this.projectId);
					 // for upload rows, we need to attach the file
					if(row.fileInputName){
						formData.append(row.fileInputName, row.getFile());
					}
					
					row.setIsEditable(false);
					row.displayRow();
					
					// add document XML, encapsulated in docTableX root tag to mimic 1-row-doctable
					var xml = '<' + this._docTableObj.saveXMLRootTag + '>';
					xml += row.convertToXML();
					xml += '</' + this._docTableObj.saveXMLRootTag + '>';
					formData.append("documentXML", xml);
					
					// gather other form data with intercept
					this._emit("form.beforesubmit", formData);
					
					// raise counter
					ctr++;

					$.ajax({
						url: "asyncupload.wcr?action=DO_UPLOAD",
						dataType : "json",
						type: 'POST',
						enctype: 'multipart/form-data',
						cache: false,
				        processData: false,
				        contentType: false,
						data: formData,
						xhr: _.bind(function() {
							// handle progress events
				            var myXhr = $.ajaxSettings.xhr();
				            if(myXhr.upload){
				                myXhr.upload.addEventListener('progress',_.bind(function(e){
				                	var progress = e.loaded/e.total;
				                	this._emit("upload.progress", row, progress);
				                }, this), false);
				            }
				            return myXhr;
				        }, this)
					})
					.done(_.bind(function(data){
						if(data.error){
							this.rowUploadFailed(row, [data.error.msg]);
							hasFailures = true;
						} else if(data.result.warning){
							var warnings = _.pluck(data.result.warning, 'msg');
							this._emit("upload.warning", row, warnings);
							this._uploadedDocumentsInfo[row.rowID] = data.result.documents[0]; // upload was successful, so add info
							this._uploadedDocumentsInfo[row.rowID].warnings = warnings;
							hasWarnings = true;
						} else {
							this._uploadedDocumentsInfo[row.rowID] = data.result.documents[0];
							this._emit("upload.success", row);
						}
					}, this))
					.fail(_.bind(function(err, status, errStr){
		                var errorMsg = i18n.t("REQUEST_FAILED");
		                if (errStr == 'SESSION_EXP') {
		                    errorMsg += ' ' + i18n.t("SESSION_EXPIRED");
		                } else {
		                	errorMsg += ' ' + i18n.t("UPLOAD_FAILED", status);
		                }
						
						this.rowUploadFailed(row, [errorMsg]);
						hasFailures = true;
					}, this))
					.always(_.bind(function(data){
						ctr--;
						if(ctr == 0){
							this.uploadFinished(hasFailures, hasWarnings);
						}
					}, this));
				}, this));
				
				// all rows failed on frontend Validation
				if(ctr == 0){
					this.uploadFinished(hasFailures, hasWarnings);
				}
				
				this._docTableObj.displayTable();
			}
		};
		
		/**
		 *	Handle failure of upload: set editable again and send event
		 */
		DocTableController.prototype.uploadFinished = function(hasFailures, hasWarnings){
			if(hasFailures){
				this.rowUploadSomeFailed();
				this._emit("done.failure");
			} else if(hasWarnings){
				this.rowUploadSomeFailed();
				this._emit("done.warnings");
			} else {
				this._emit("done.success");
			}
		};
		
		/**
		 *	Handle failure of upload: set editable again and send event
		 */
		DocTableController.prototype.rowUploadFailed = function(row, err){
			this._emit("upload.progress", row, 1); // set progress of upload done for if it wouldn't be finished
			this._emit("upload.failed", row, err);
			row.setIsEditable(true);
			row.displayRow();
			
			this._uploadedRows[row.rowID] = false;
		};
		
		/**
		 * If some rows failed to upload, convert the successful ones into StaticDocumentRows
		 */
		DocTableController.prototype.rowUploadSomeFailed = function(){
			var rows = this._docTableObj.docRows;
			if(rows.length > 0){
				_.each(rows, _.bind(function(row){
					if(row.constructor == StaticDocumentRow.prototype.constructor){
						// if it's a static row, you should be able to edit it again, even if it submitted successfully
						// also, put the uploaded flag on false, so it gets submitted again next time
						this._uploadedRows[row.rowID] = false;
						row.setIsEditable(true);
						row.displayRow();
						return;
					}
					
					// when successfully uploaded, the array will contain document information
					if(this._uploadedDocumentsInfo[row.rowID]){
						var aDoc = this._uploadedDocumentsInfo[row.rowID];
			        	var docInfo = {
		        			docVerID: aDoc.docverid,
				            docID: aDoc.docid,
				            docName: aDoc.name,
				            docDesc: aDoc.description,
				            aCatID: aDoc.categoryid
			        	};

			            // create the row instance and append it to the documents table
			        	var newRow = this._docTableObj.replaceRowWithNewStaticRow(row, docInfo);
			        	this._uploadedDocumentsInfo[row.rowID] = null;
			        	
			        	if(aDoc.warnings){
			        		this._emit("upload.warning", newRow, aDoc.warnings);
			        	} else {
			        		this._emit("upload.success", newRow);
			        	}
			        	
					}
				},this));
				this._docTableObj.displayTable();
			}
		};
		
		/**
		 * Are there documents in the DocumentsTable?
		 * 
		 * @returns {Boolean} 
		 */
		DocTableController.prototype.hasDocuments = function(){
			return this._docTableObj && this._docTableObj.docRows && this._docTableObj.docRows.length > 0;
		};
		
		/**
		 * Clear all files of the DocTable
		 */
		DocTableController.prototype.clear = function(){
			this._docTableObj.reset(true);
		};
		
		/**
		 * Clear succesful entries
		 */
		DocTableController.prototype.clearSuccessful = function(){
			var rows = this._docTableObj.docRows;
			if(rows.length > 0){
				for(var i=rows.length-1 ; i>=0 ; i--){
					var row = rows[i];
					if($("#"+row.phElemID).find(".icon-WCR_Msg-Success").length > 0){
						var idx = this._docTableObj.findRow(row);
						this._docTableObj.deleteDocumentRow(idx);
					}
				}
			}
		};
		
		/**
		 * get All folder id's used in the doctableX
		 * @return {Array.String}
		 */
		DocTableController.prototype.getAllFolderIds = function(){
			var rows = this._docTableObj.docRows;
			var folderIds = [];
			_.each(rows, function(row){
				if(row.selTargetFolder){
					var folderId = row.selTargetFolder.id || "";
					if(folderId === "0"){
						folderId = "";
					}
					folderIds.push(folderId);
				} else {
					folderIds.push("");
				}
			});
			return _.uniq(folderIds);
		};
		
		/**
		 * Check the names of all documents.
		 * Returns a map
		 * @return {Map<String,Array<Error>>}
		 */
		DocTableController.prototype.checkAllNames = function(){
			var evInst = EventBasedValidator.getInstance();
	    	var uniqueNamesObj = evInst.createUniqueNamesObj();
	    	evInst.validateNamesGlobally(uniqueNamesObj, true);

	    	var map = {};
	    	_.each(uniqueNamesObj.errors, function(error){
	    		var metadataArr = uniqueNamesObj.getErrorMetadata(error);
	    		_.each(metadataArr, function(metadata){
	    			if(metadata.row instanceof StaticDocumentRow){
	    				if(!map[metadata.row.rowID]){
	    					map[metadata.row.rowID] = [];
	    				}
	    				map[metadata.row.rowID].push(error);
	    			}
		    	});
	    	});
	    	
	    	return map;
		};

		return DocTableController;
	});