define(["component/message/MessageComponent",
        "component/popover/Popover",
        "utils/Observer",
        "utils/webi18n!UploadTool",
        "utils/MethodExtender",
        "doctablex/DocTableXView",
        "doctablex/StaticDocumentRowView"],
	function (MessageComponent,
			Popover,
			Observer,
			i18n,
			MethodExtender,
			DocTableXView,
			StaticDocumentRowView) {
		"use strict";

		/**
		 * Makes sure all feedback of the upload tool is handled
		 * It extends the DocTableX classes with specific feedback functionality
		 * 
		 * @class
		 * @exports UploadFeedBack 
		 * @constructor
		 * 
		 * @param {jQueryObj} ctx - context
		 */
		var UploadFeedBack = function(ctx) {
			Observer.mixin(this);
			
			this._ctx = ctx;
			this._messageModel = new MessageComponent.MessageModel();
			this._messageWidget = new MessageComponent.MessageWidget(this._messageModel, this._ctx);
			
			this.feedback = {};
			
			// extend DocTableX render method with feedback
			if(!DocTableXView.prototype.hasUploadFeedbackExtension){
				MethodExtender.extendMethod(DocTableXView.prototype, "render", function(docTableView){
					if(docTableView.dm.docRows.length > 0){
						var table = $("#"+docTableView.phElemID + "_mainTable");
						var tHeadRow = table.find(">thead>tr");
						tHeadRow.prepend("<th class='td-narrow'></th>");
					}
				});
				DocTableXView.prototype.hasUploadFeedbackExtension = true;
			}
			
			// extend StaticDocumentRow render method with feedback
			if(!StaticDocumentRowView.prototype.hasUploadFeedbackExtension){
				MethodExtender.extendMethod(StaticDocumentRowView.prototype, "render", _.bind(function(docRowView){
					// defer needed because overriding methods replace the whole element
					_.defer(_.bind(function(){
						var rowElem = $("#"+docRowView.phElemID);
						if(rowElem.find(".UploadFeedback").length === 0){
							docRowView.feedbackElem = $("<td />").addClass("UploadFeedback");
							rowElem.prepend(docRowView.feedbackElem);
							
							var feedback = this.feedback[docRowView.dm.rowID];
							if(feedback){
								if(feedback.success){
									this.addSuccessFeedback(docRowView.dm);
								} else if (feedback.error){
									this.addFailFeedback(docRowView.dm, feedback.error);
								} else if (feedback.warning){
									this.addWarnFeedback(docRowView.dm, feedback.warning);
								} else if (feedback.progress){
									this.addProgressFeedback(docRowView.dm, feedback.progress);
								}
							}
						}
					}, this));
				}, this));
				StaticDocumentRowView.prototype.hasUploadFeedbackExtension = true;
			}
		};
		
		/**
		 * Register listeners on an observable
		 * @param {Observable} observable 
		 */
		UploadFeedBack.prototype.addObservable = function(observable){
			// total progress handlers
			this._observe(observable, "done.success", _.bind(function () {
				var message = new MessageComponent.Message(MessageComponent.MessageType.SUCCESS, i18n.t("FEEDBACK_SUCCESS"), undefined, true);
				this._messageModel.addToMessageQueue(message);
			}, this));
			
			this._observe(observable, "done.failure", _.bind(function () {
				var message = new MessageComponent.Message(MessageComponent.MessageType.ERROR, i18n.t("FEEDBACK_FAILURE"));
				this._messageModel.addToMessageQueue(message);
			}, this));
			
			this._observe(observable, "done.warnings", _.bind(function () {
				var message = new MessageComponent.Message(MessageComponent.MessageType.WARN, i18n.t("FEEDBACK_WARNING"), undefined, true);
				this._messageModel.addToMessageQueue(message);
			}, this));
			
			// single document handlers
			this._observe(observable, "upload.success", _.bind(function (row) {
				this.addSuccessFeedback(row);
			}, this));
			
			this._observe(observable, "upload.progress", _.bind(function (row, progress) {
				this.addProgressFeedback(row, progress);
			}, this));
			
			this._observe(observable, "upload.failed", _.bind(function (row, err) {
				this.addFailFeedback(row, err);
			}, this));
			
			this._observe(observable, "upload.warning", _.bind(function (row, err) {
				this.addWarnFeedback(row, err);
			}, this));
		};
		
		/**
		 * Add "success" feedback to DOM
		 * @param {StaticDocumentRow} row - row to add feedback to
		 */
		UploadFeedBack.prototype.addSuccessFeedback = function(row){
			this.feedback[row.rowID] = {success: true};
			if(row.view.feedbackElem){
				row.view.feedbackElem.html($("<i />").addClass("icon icon-normal icon icon-WCR_Msg-Success"));
			}
		};
		
		/**
		 * Add "failure" feedback to DOM
		 * @param {StaticDocumentRow} row - row to add feedback to
		 * @param {Array.String} errArr - Array of error strings
		 */
		UploadFeedBack.prototype.addFailFeedback = function(row, err){
			this.feedback[row.rowID] = {error: err};
			if(row.view.feedbackElem){
				var i = $("<i />").addClass("icon icon-normal icon-WCR_Msg-Error");
				row.view.feedbackElem.html(i);

				// show popover
				var pop = new Popover(i, $("body"), $("body"), "right", "#f2dede", "alert alert-danger");
				pop.setContent(this.createErrorContent(err));
				i.hover(function(){
					pop.show();
				}, function(){
					pop.hide();
				});
			}
		};
		
		/**
		 * Add "warning" feedback to DOM
		 * @param {StaticDocumentRow} row - row to add feedback to
		 * @param {Array.String} errArr - Array of warning strings
		 */
		UploadFeedBack.prototype.addWarnFeedback = function(row, errArr){
			this.feedback[row.rowID] = {warning: errArr};
			if(row.view.feedbackElem){
				var i = $("<i />").addClass("icon icon-normal icon-WCR_Msg-Warning");
				row.view.feedbackElem.html(i);

				// show popover
				var pop = new Popover(i, $("body"), $("body"), "right", "#fcf8e3", "alert alert-warning");
				pop.setContent(this.createErrorContent(errArr));
				i.hover(function(){
					pop.show();
				}, function(){
					pop.hide();
				});
			}
		};
		
		/**
		 * Add "warning" feedback to DOM
		 * @param {StaticDocumentRow} row - row to add feedback to
		 * @param {Number} progress - progress between 0 and 1
		 */
		UploadFeedBack.prototype.addProgressFeedback = function(row, progress){
			var progStr;
			if(progress == 1){
				progStr = i18n.t("FEEDBACK_PROCESSING") + '...';
			} else {
				progStr = Math.round(10000*progress)/100 + '%';
			}
			this.feedback[row.rowID] = {progress: progress};
			if(row.view.feedbackElem){
				row.view.feedbackElem.html($("<label />").append(progStr));
			}
		};
		
		/**
		 * Create jQuery object for error feedback, containing all error strings
		 * @param {Array.String} errArr - Array of warning strings
		 * @returns {jQueryObj}
		 */
		UploadFeedBack.prototype.createErrorContent = function(errArr){
			if(errArr.length == 1){
				return errArr[0];
			} else {
				var ul = $("<ul />");
				_.each(errArr, function(err){
					ul.append($("<li />").html(err));
				});
				return ul;
			}
		};

		return UploadFeedBack;
});