define(["upload/DocTableController",
		"upload/ApprovalFields",
		"upload/UploadFeedBack",
		"component/dropzone/FileDropzone",
		"component/progressbar/ProgressBar",
		"component/fileinput/MultiFileInput",
		"utils/Observer",
		"utils/Observable",
		"utils/webi18n!UploadTool",
		"utils/AjaxCache",
		"utils/ExecutionQueue",
		"Template",
		"tpl!upload/UploadTool",
		"jquery",
		"underscore",
		"utils/webcss!./UploadToolStyle.css"
	],
	function (DocTableController,
		ApprovalFields,
		UploadFeedBack,
		FileDropzone,
		ProgressBar,
		MultiFileInput,
		Observer,
		Observable,
		i18n,
		AjaxCache,
		ExecutionQueue,
		Template,
		tpl,
		$,
		_) {
		"use strict";

		/**
		 * This is the main class for the HTML5 Upload Tool
		 * 
		 * @class
		 * @exports UploadTool 
		 * @constructor
		 * @mixes Observer 
		 * @param {jQueryElem} ctx - context to render tool in
		 * @param {Object} config
		 */
		var UploadTool = function (ctx, config) {
			Observer.mixin(this);
			Observable.mixin(this, "UploadTool");

			this._ctx = ctx;
			this._ctx.addClass("UploadTool");

			this.t = new Template(i18n, tpl);
			this.t.setContext(this._ctx);
			this.t.render();

			this.projectId = config.projectId;
			this.folderId = config.folderId;
			this.taskId = config.taskId;
			this.returnUrl = config.returnUrl;

			/**
			 * @member {ProgressBar}
			 * @private
			 */
			this._progressBar = null;

			/**
			 * @member {Object.<Number>} - object holding the amount of progress that has been made for each document uploaded
			 * @private 
			 */
			this._progresses = {};

			// create dropzone
			this._dropzone = new FileDropzone({
				element: this._ctx.find("div[name=dropzone]")
			});

			// Multi-upload input field
			this._multiFileInput = new MultiFileInput({
				element: this._ctx.find("div[name=UploadField]")
			}, i18n);
			
			// hide "loading" message
			this._ctx.find(".MultiUploadAddingFiles").hide();

			// create doctable element and controller
			var docTableContainer = this._ctx.find(".MultiUploadDocTable");
			this._docTableController = new DocTableController(docTableContainer, this.projectId, this.taskId);
			docTableContainer.hide(); // hide when there are no files

			// approval
			this.approvalFields = new ApprovalFields({
				element: this._ctx.find(".ApprovalFields")
			});

			// Action Buttons
			var submitBtn = this._ctx.find("button[name=submit]");
			submitBtn.on("click", _.bind(function () {
				this._docTableController.startUpload();
			}, this));
			submitBtn.attr("disabled", "disabled");

			if (!this.returnUrl) {
				this._ctx.find("button[name=return]").hide();
			} else {
				this._ctx.find("button[name=return]").on("click", _.bind(function () {
					window.location = this.returnUrl;
				}, this));
			}

			this._ctx.find("button[name=clear]").on("click", _.bind(function () {
				this._docTableController.clear();
			}, this));

			var clearSuccessful = this._ctx.find("button[name=clearsuccessful]");
			clearSuccessful.on("click", _.bind(function () {
				this._docTableController.clearSuccessful();
				clearSuccessful.hide();
			}, this));
			clearSuccessful.hide();

			// register needed listeners on members 
			this._registerListeners();

			// load config from server
			this.loadConfigFromServer();

			// Upload feedback
			this._feedback = new UploadFeedBack(this._ctx.find(".FeedBackContainer"));
			this._feedback.addObservable(this._docTableController);

			// warning message when leaving the page
			if (this.returnUrl) {
				$(window).on("beforeunload", _.bind(this._onBeforeUnload, this));
			}
		};
	
		/**
		 * creates an array from a filelist object as the ExecutionQueue doens't handle the fileList object well on IE
		 * @param {FileList}
		 * @return {Array}
		 */
		UploadTool.prototype._createObjectFromFileList = function(fileList) {
			var arr = [];
			for(var i = 0; i < fileList.length; i++) {
				arr.push(fileList[i]);
			}
			return arr;
		}

		/**
		 * Register listeners on observable fields
		 */
		UploadTool.prototype._registerListeners = function () {
			// listen to the upload input and handle the incoming files
			var addFilesToTable = _.bind(function (fileList) {
				this._ctx.find(".MultiUploadAddingFiles").show();
				var queue = new ExecutionQueue();
				var filesArray = this._createObjectFromFileList(fileList);
				queue.forEach(filesArray, function (file) {
					this._docTableController.addFile(file);
				}, this);
				queue.run(function(){
					this._docTableController.repaintTable();
					this._ctx.find(".MultiUploadAddingFiles").hide();
				}, this);
			}, this);
			this._observe(this._multiFileInput, "change", addFilesToTable);
			this._observe(this._dropzone, "filesDropped", addFilesToTable);

			this._observe(this._docTableController, "upload.start", function (rows) {
				if (this._progressBar) {
					this._progressBar.getElement().remove();
				}
				this._progresses = {};
				this._progressBar = new ProgressBar({
					maxProgress: rows.length
				});
				this._progressBar.startProgress();
				this._ctx.find(".ActionButtons").before(this._progressBar.getElement());
				
				// disable action buttons when uploading
				this._ctx.find(".ActionButtons button").attr("disabled", "disabled");
			});

			this._observe(this._docTableController, "upload.progress", function (row, progress) {
				this._progresses[row.phElemID] = progress;
				this._progressBar.progressChanged(_.reduce(this._progresses, function (memo, progress) {
					return memo + progress;
				}, 0));
			});

			// listen to completion to redirect if successful
			this._observe(this._docTableController, "done.success", function (fileList) {
				this.uploadSuccessful = true;
				if (this.returnUrl) {
					window.location = this.returnUrl;
				}
			});
			this._observe(this._docTableController, "done.failure", function () {
				this._progressBar.progressFailed();
				this._ctx.find("button[name=clearsuccessful]").show();
			});
			this._observe(this._docTableController, "done.warnings", function () {
				this._progressBar.addClass("progress-bar-warning");
				this._ctx.find("button[name=clearsuccessful]").show();
			});
			this._observe(this._docTableController, "done.*", function () {
				if (this._docTableController.hasDocuments()) {
					this._ctx.find(".ActionButtons button").removeAttr("disabled");
				}
			});

			// listen to update
			this._observe(this._docTableController, "doctable.updated", _.bind(function () {
				// show/hide submit
				if (this._docTableController.hasDocuments()) {
					this._ctx.find(".MultiUploadDocTable").show();
					this._ctx.find(".MultiUploadDocTableEmpty").hide();
					this._ctx.find("button[name=submit]").removeAttr("disabled");
				} else {
					this._ctx.find(".MultiUploadDocTable").hide();
					this._ctx.find(".MultiUploadDocTableEmpty").show();
					this._ctx.find("button[name=submit]").attr("disabled", "disabled");
					
					// hide progress bar when table is cleared
					if (this._progressBar) {
						this._progressBar.getElement().remove();
					}
				}
				
				// update approval fields
				this._updateApprovalFields();
			}, this));


			// intercept form submission to add parameters
			this._observe(this._docTableController, "form.beforesubmit", function (formData) {
				if (this.taskId) {
					formData.append("taskId", this.taskId);
				}
				this.approvalFields.addToForm(formData);
			});
		};

		/**
		 * Load configuration data from the server using an ajax call and propagate the result to the modules.
		 * Only one shared call is done for all components together.
		 */
		UploadTool.prototype.loadConfigFromServer = function () {
			var url = "asyncupload.wcr?action=GET_UPLOAD_INFO";
			url += "&projectId=" + this.projectId;
			url += "&taskId=" + this.taskId;
			url += "&folderId=" + this.folderId;

			var promise = AjaxCache.ajax({
				url: url,
				dataType: "json",
				cache: false
			});

			$.when(promise).done(_.bind(function (data) {
					if (data.error) {
						this._emit("error.loadfailed", data.error);
					} else {
						this._docTableController.setConfig(data.result);
						this.approvalFields.setConfig(data.result, this.projectId, this.folderId);

						// if there are folders, fill out data
						if (data.result.folders && data.result.folders.length > 0) {
							var select = this._ctx.find(".FolderSelector select");

							select.append($('<option value="" />').html(i18n.t("ROOT_FOLDER_TAG")));
							_.each(data.result.folders, function (folder) {
								var option = $('<option />').attr("value", folder.id).html(folder.name);
								select.append(option);
							});

							if (this.folderId) {
								select.val(this.folderId);
							}

							select.on("change", _.bind(function () {
								var id = select.val();
								var text = select.find("option:selected").text();
								if(!id){
									// for root folder, we should apply "" instead of "Root Folder"
									text = "";
								}
								
								this._docTableController.setTargetFolder(text);
								this.folderId = id;
								
								this._updateApprovalFields();
							}, this));
						} else {
							this._ctx.find(".FolderSelector").hide();
						}
					}
				}, this))
				.fail(_.bind(function (err) {
					this._emit("error.loadfailed", err);
				}, this));
		};

		/**
		 * Return a warning string if there are files that have not been uploaded yet.
		 * @returns {String}
		 */
		UploadTool.prototype._onBeforeUnload = function () {
			if (this._docTableController.hasDocuments() && !this.uploadSuccessful) {
				return i18n.t("UPLOAD_TOOL_SAVE_WARNING");
			}
		};
		
		/**
		 * Update approval fields with selected folders
		 * @returns {String}
		 */
		UploadTool.prototype._updateApprovalFields = function () {
			var folderIds = this._docTableController.getAllFolderIds();
			folderIds.push(this.folderId);
			this.approvalFields.setFolders(folderIds);
		};

		return UploadTool;
	});
