define([
		"underscore",
		"lib/EasingFunctions"
	],
	function (
		_,
		EasingFunctions
	) {
		"use strict";

		return {
			/**
			 * Mixin Animatable functionality to an object.
			 *
			 * Animatable makes it possible for objects to be animated. It generates animatable properties
			 * on the object which can be animated by calling animate and request its value at any given time
			 * with getValue-method. If the object is an Observable, Animatable will emit animationStarted and animationFinished events.
			 * 
			 * The Mixin can be applied to both a prototype or a object instance
			 * 
			 * @param {Object} object - the object in which the Animatable functionality should be mixed in
			 */
			mixin: function (object) {
				/**
				 * Add animatable properties to the object, provide for each property a function that returns its correct value.
				 * It is possible to add 1 property (by using 2 parameters), or a set of properties (only use the properties parameter)
				 * @param {Map.<String,Function>|String} properties - Either a object mapping propertyNames on there value functions or a property name (combined with a value function as second argument)
				 * @param {Function} [function] -  the functions that returns the correct values for the property (in case of two arguments)
				 */
				object.addAnimatableProperties = function (properties, valueFunction) {
					//first make sure the correct properties for the animatable are set
					if (!this._initialized) {
						this._initialize(); //call the private initialize function
					}

					if (arguments.length == 2){
						this._animatableProperties[properties] = valueFunction;						
					} else {
						this._animatableProperties = _.extend(this._animatableProperties, properties);
					}
				};

				/**
				 * Initialize the mixin animatable to the object, installs the needed properties for the animatable
				 */
				object._initialize = function () {
					/**
					 * @member {Map.<String|Function>} maps the animatable properties to the function that returns their values
					 * @private
					 */
					this._animatableProperties = {};

					/**
					 * @member {Map.<String|Object>} maps the animatable property to its current active animation. If there
					 * 									is no animation the property has no entry in this map.
					 * @private
					 */
					this._animations = {};

					/**
					 * @member {Boolean} indicates if the animatable mixin has be initialized on the object
					 * @private
					 */
					this._initialized = true;
				};

				/**
				 * Get the correct value for the animatable properties, uses the animation (if one for the property) or the default function.
				 * @param   {String} propertyName
				 * @param   {Boolean} [isStartAnimation] used to indicate start animation, does not take the animation into account
				 * @returns {any} value of the property
				 */
				object.getValue = function (propertyName, isStartAnimation) {
					//check if there is an animation active for the property, if this is the case get the value from the animation
					if (this._animations[propertyName] && !isStartAnimation) {
						var animationObject = this._animations[propertyName];
						var currentTime = Date.now();
						var timeOffset = currentTime - animationObject.startTime;
						if (timeOffset > animationObject.duration) { //if the animation is done (time offset is bigger than the duration)
							delete this._animations[propertyName]; //delete entry in the animations map
							
							if (this._emit) {
								this._emit(["animation", propertyName, "finished"].join("."));
								if (!this.isAnimating()){
									this._emit("animationFinished");
								}
							}							
							
							return this._animatableProperties[propertyName]();
						} else {
							//uses an easing function for the value
							return EasingFunctions[animationObject.easingFunction](null, timeOffset, animationObject.startValue, animationObject.endValue - animationObject.startValue, animationObject.duration);
						}
					} else {
						return this._animatableProperties[propertyName]();
					}
				};

				/**
				 * Starts an animation on one of the animatable properties. Make sure the function of the animatable
				 * property returns the end value for the animation.
				 * @param {String} propertyName
				 * @param {Integer} startValue
				 * @param {Integer} duration
				 * @param {String} [easingFunction] - if set this animation will use the provided easing function, otherwise it falls back to the easign assigned to the animation property or the default easing function
				 */
				object.animate = function (propertyName, startValue, duration, easingFunction) {
					var endValue = this.getValue(propertyName, true); //endvalue is the currently set value in the animatable property
					var wasAnimating = this.isAnimating();

					if (startValue !== endValue) { //only start animations if the start value is not equal to the endvalue
						if (!easingFunction){
							easingFunction = "easeOutBounce";
						}
						
						this._animations[propertyName] = {
							startValue: startValue,
							endValue: endValue,
							duration: duration,
							easingFunction: easingFunction,
							startTime: Date.now() //set the current time as the start time for the animation
						};
						
						if (this._emit){
							this._emit(["animation", propertyName, "started"].join("."));							
							if (!wasAnimating){
								this._emit("animationStarted");
							}
						}
					}
				};

				/**
				 * Has the object any properties that are being animated
				 * @returns {Boolean}
				 */
				object.isAnimating = function () {
					if (this._initialized) {
						return !_.isEmpty(this._animations);
					} else {
						return false;
					}
				};
			}
		};

	});