define(["jquery",
		"underscore"],
function($, _){
	var ConvertUtils = {
			 
		 /**
		  * Get a language formatted date of the form "Month dayOfMonth, year
		  * NOTE: make sure the string MONTH[1-12] are loaded in the i18n instance
		  * @param {I18n} i18n - instance of i18n
		  * @param {Date} date - the date to language format
		  * @returns {String} Language formatted date string as "Jan 19, 2014" e.g.
		  */
		 getLangFormattedDate : function (i18n,date) {
			  var months = [];

              months[0]= i18n.t("MONTH1");
              months[1]= i18n.t("MONTH2");
              months[2]= i18n.t("MONTH3");
              months[3]= i18n.t("MONTH4");
              months[4]= i18n.t("MONTH5");
              months[5]= i18n.t("MONTH6");
              months[6]= i18n.t("MONTH7");
              months[7]= i18n.t("MONTH8");
              months[8]= i18n.t("MONTH9");
              months[9]= i18n.t("MONTH10");
              months[10]= i18n.t("MONTH11");
              months[11]= i18n.t("MONTH12");
			  
			  if (date==null)
			    date=new Date();
			  var curMonth=date.getMonth(); 
			  var curDate=date.getDate(); 
			  var curYear=date.getFullYear(); 
			  
			  return ""+months[curMonth]+" "+curDate+", "+curYear; 
		 },
        /**
		  * Get a language formatted date and time of the form "Month dayOfMonth, year at hour:minutes"
		  * NOTE: make sure the string MONTH[1-12] are loaded in the i18n instance
		  * @param {I18n} i18n - instance of i18n
		  * @param {Date} date - the date to language format
		  * @returns {String} Language formatted date string as "Jan 19, 2014 at 11:00" e.g.
		  */
		 getLangFormattedDateAndTime : function (i18n,date) {
			  var months = [];	
			  
              months[0]= i18n.t("MONTH1");
              months[1]= i18n.t("MONTH2");
              months[2]= i18n.t("MONTH3");
              months[3]= i18n.t("MONTH4");
              months[4]= i18n.t("MONTH5");
              months[5]= i18n.t("MONTH6");
              months[6]= i18n.t("MONTH7");
              months[7]= i18n.t("MONTH8");
              months[8]= i18n.t("MONTH9");
              months[9]= i18n.t("MONTH10");
              months[10]= i18n.t("MONTH11");
              months[11]= i18n.t("MONTH12");	
			  
			  if (date==null)
				    date=new Date();
			  var curMonth=date.getMonth(); 
			  var curDate=date.getDate(); 
			  var curYear=date.getFullYear(); 
			  var curHour=date.getHours();
			  if(curHour<10){
				  curHour = "0"+curHour;
			  }
			  var curMinutes=date.getMinutes();
			  if(curMinutes<10){
				  curMinutes = "0"+curMinutes;
			  }
			  return i18n.t("DateAndTimeString",months[curMonth],curDate,curYear,curHour,curMinutes); 
         },
         /**
		  * Get an English formatted date of the form "Month dayOfMonth, year
		  * NOTE: make sure the string MONTH[1-12] are loaded in the i18n instance
		  * @param {I18n} i18n - instance of i18n
		  * @param {String} translatedDate - Language formatted date string as "Déc 19, 2014" e.g.
		  * @returns {String} English formatted date string as "Dec 19, 2014" e.g.
		  */
		 getEnglishFormattedDate : function (i18n,translatedDate) {
			  var months = {};

              months[i18n.t("MONTH1")]= "Jan";
              months[i18n.t("MONTH2")]= "Feb";
              months[i18n.t("MONTH3")]= "Mar";
              months[i18n.t("MONTH4")]= "Apr";
              months[i18n.t("MONTH5")]= "May";
              months[i18n.t("MONTH6")]= "Jun";
              months[i18n.t("MONTH7")]= "Jul";
              months[i18n.t("MONTH8")]= "Aug";
              months[i18n.t("MONTH9")]= "Sep";
              months[i18n.t("MONTH10")]= "Oct";
              months[i18n.t("MONTH11")]= "Nov";
              months[i18n.t("MONTH12")]= "Dec";
              
              var splitStr = $.trim(translatedDate).split(' ');
              if(months[splitStr[0]]){
            	  splitStr[0] = months[splitStr[0]];
            	  return splitStr.join('');
              } else {
            	  return translatedDate;
              }
		 },
        /**
		  * Make a capitalized version of the first letter of the given string
		  * @param {String} string - String of which you want to capitalize the first letter
		  * @returns {String} String with a capitalized first letter
		  */
		 capitaliseFirstLetter : function (string) {
		     return string.charAt(0).toUpperCase() + string.slice(1);
		 },
        
        /**
		  * Count the number of lines and characters in a raw HTML string
		  * @param {String} rawHtmlString - The raw HTML string
		  * @returns {Object} o.lineCount contains the line count, o.charCount the character count
		  */
		 getCharacterLineCount : function(rawHtmlString) {
			var lineCount = 1;
			var charCount = 0;

			if (rawHtmlString) {				
				var text = this.convertHTMLStringToRaw(rawHtmlString);
				text = text.replace(/\r\n/g, '\n');
				
				// count lines
				var newLines = 0;
				var n = text.indexOf('\n', 0);
				while (n != -1) {
					lineCount++;
					newLines++;
					n = text.indexOf('\n', n + 1);
				}
				// count characters
				charCount = text.length - newLines;
			}

			return {
				lineCount : lineCount,
				charCount : charCount
			};
		},
		/**
		 * Convert an html string to RAW string (remove all html tags and convert html entitites
		 * @param {String} htmlString
		 * @param {String} [newLineCharacter=\n]  - the character used for separating/indicating newlines in the output
		 * @returns {String} the raw text string
		 */
		convertHTMLStringToRaw: function(htmlString, newLineCharacter){
			if (newLineCharacter === undefined){
				newLineCharacter = "\n";
			}
			// remove all white space between tags, except real spaces
			var rawString = htmlString.replace(/>[\f\n\r\t\v\u00a0\u1680\u180e\u2000\u2001\u2002\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200a\u2028\u2029\u202f\u205f\u3000]*([ ]*)[\f\n\r\t\v\u00a0\u1680\u180e\u2000\u2001\u2002\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200a\u2028\u2029\u202f\u205f\u3000]*</g, ">$1<");
			
			// get body only
			rawString = rawString.replace(/.*<body>((\r\n|\n|.)*)<\/body>.*/i, "$1");
			
			// remove the first blocklevel element
			rawString = rawString.replace(/^<(blockquote|body|center|dd|dir|div|dl|dt|form|h1|h2|h3|h4|h5|h6|head|hr|html|isindex|li|menu|noframes|ol|p|pre|td|th|title|ul)[^>]*>/, "");
			// all elements that cause a line break are replaced by a \n
			rawString = rawString.replace(/<(blockquote|body|br|center|dd|dir|div|dl|dt|form|h1|h2|h3|h4|h5|h6|head|hr|html|isindex|li|menu|noframes|ol|p|pre|td|th|title|ul)[^>]*>/g,newLineCharacter);
			// strip all html elements
			rawString = rawString.replace(/<[^>]*>/g, "");
			// convert spaces
			rawString = rawString.replace(/&nbsp;|&#160;/gi, ' ');
			// decode entitites
			rawString = this.decodeHtmlEntities(rawString);
			
			return rawString;
		}, 
		
		/**
		 * Convert a RAW string to xHTML (change \r\n or \n to <br/>)
		 * @param {String} rawString
		 * @returns {String} the xHTML string
		 */
		convertRawStringToHTML: function(rawString){
			var htmlString = this.encodeHtmlEntities(rawString);
			
			// replace newlines with breaks
			htmlString = htmlString.replace(/(\r)?\n/g,"<br />");
			
			return htmlString;
		},
        /**
		 * Decode HTML entities in the HTML string
		 * @param html - HTML string to decode
		 * @returns {String} html parameter with decoded entities
		 */
		decodeHtmlEntities : function(html){
			return $('<textarea>').html(html).text();
		},

		/**
		 * Encode HTML entities in the HTML string
		 * @param text - text string to encode
		 * @returns {String} text parameter with encoded entities
		 */
		encodeHtmlEntities : function(text){
			return $("<div>").text(text).html();
		}
		   
    };
    
    return ConvertUtils;
});