define(["underscore"], function (_) {
	"use strict";


	/**
	 * ExecutionQueue can be used to execute heave operations that can't be executed in a WebWorker
	 * You can split up the work in smaller tasks using .run()
	 * When you need to process an array or map, use forEach
	 * it will split up the loop in smaller chunks and make sure the browser is not blocked for longer than 15 ms at a time
	 *
	 * @exports ExecutionQueue
	 * @class
	 *
	 * @param {Boolean} [synchronized=false] - if true, all operations will block EventQueue and be executed synchronously
	 * @param {Boolean} [updateLoadingIndicator=!synchronized] - Default value: true when assynchronous, false when synchronous
	 */
	var ExecutionQueue = function (synchronized, updateLoadingIndicator) {
		/**
		 * @member {Number} interval when deferring tasks
		 * @private
		 */
		this._interval = 1;
		/**
		 * @member {Number} maximum execution time of a chunk when doing foreach
		 */
		this._loopChunkTime = 15;
		/**
		 * @member {Array.<Array>} execution queue, [fn, context, assync, key, description]
		 * @private
		 */
		this._queue = [];
		/**
		 * @member {Boolean} is the queue executing?
		 * @private
		 */
		this._isRunning = false;
		/**
		 * @member {Number} timer ID (used to cancel the timer if needed
		 * @private
		 */
		this._timer = null;
		/**
		 * @member {Boolean} is the queue in synchronized mode?
		 */
		this._synchronized = synchronized;
		
		/**
		 * @member {Number} timestamp for when the runtime was blocked for the first time. this is used when the execution queue batches run calls
		 */
		this._blockingJSRuntimeSince = 0;
		/**
		 * @member {Boolean} batch run calls. By default consecutive run calls are deferred. If this is true, we try to run as much run calls as possible in the loopChunkTime. if we exceed the loopchunktime, we defer the next run call
		 */
		this._batchRunCalls = false;
		
		/**
		 * @member {Boolean} should start/stop be made to LoadingIndicator when starting/stopping tasks
		 * @private
		 */
		this._shouldUpdateLoadingIndicator = updateLoadingIndicator || !synchronized;

		/**
		 * @member {Array.<Function>} list of callbacks waiting for the queue to finish executing
		 * @private
		 */
		this._finishedCallbacks = [];
	};

	/**
	 * Put a function to the queue
	 * if the executor is not running, it will start runnning
	 * @param {Function} fn - task that needs to be executed, if assync is true the first parameter is a callback function that needs to be called once the operation has finished
	 * @param {Any} [context] - context for when executing fn
	 * @param {Boolean} [assync=false] - if true fn() can be assynchronous, once finish it should call the callback provided as first argument of fn
	 * @param {String} [key] - If LoadingIndicator should be shown during action you can provide a key for loading action here
	 * @param {String} [description} - If LoadingIndicator should be shown during action you can provide an optional description here
	 */
	ExecutionQueue.prototype.run = function (fn, context, assync, key, description) {
		this._queue.push([fn, context, assync, key, description]);
		if (!this._isRunning){
			this._startTimer();
		}
	};

	/**
	 * Run over a big list/map or do heave operations on a list
	 * Whenever execution takes more than loopChunkTime, the loop is stopped and processed further after a timeout
	 * This enables to execute heavy operations without blocking the browser too much
	 * @param {Array, Object} collection - collection that needs processing
	 * @param {Function} fn - arguments: (value, key/index)
	 * @param {Any} [context] - context for when calling fn
	 * @param {String} [key] - If LoadingIndicator should be shown during action you can provide a key for loading action here
	 * @param {String} [description} - If LoadingIndicator should be shown during action you can provide an optional description here
	 */
	ExecutionQueue.prototype.forEach = function (collection, fn, context, key, description) {
		var loopChunkTime = this._loopChunkTime;
		var queue = this;

		// run an assync task
		this.run(function (whenFinished) {
			var currentIndex = 0;
			var isArrayLike = _.isArray(collection);

			var keys;
			var maxIndex;

			if (isArrayLike) {
				maxIndex = collection.length;
			} else {
				keys = _.keys(collection);
				maxIndex = keys.length;
			}

			// execute a chunk
			var executeChunk = function () {
				var startTime = queue._getCurrentTime();
				while (queue._getCurrentTime() - startTime < loopChunkTime && currentIndex < maxIndex) {
				
					if (isArrayLike) {
						fn.call(context || window, collection[currentIndex], currentIndex);
						currentIndex++;
					} else {
						var key = keys[currentIndex];
						if (collection.hasOwnProperty(key)) {
							fn.call(context || window, collection[key], key);
						}
						currentIndex++;
					}
				}

				if (currentIndex < maxIndex) {
					// we are not finished yet, process another chunk
					queue._defer(executeChunk);
				} else {
					// we are finished!
					whenFinished();
				}
			};

			// process the first chunk
			executeChunk();
		}, window, true, key, description);
	};

	/**
	 * Start processing the queue if needed. Or notify that processing has finshed if there are no tasks anymore
	 */
	ExecutionQueue.prototype._startTimer = function () {
		if (!this._timer && this._queue.length > 0) {
			this._isRunning = true;
			this._blockingJSRuntimeSince = 0;
			this._timer = this._defer(_.bind(this._executeNext, this));
		} else if (this._queue.length === 0) {
			this._setFinished();
		}
	};
	/**
	 * Execute the next operation (and start the timer for the following operation)
	 */
	ExecutionQueue.prototype._executeNext = function () {
		if (this._queue.length > 0) {
			if (this._batchRunCalls && !this._blockingJSRuntimeSince ){
				this._blockingJSRuntimeSince = this._getCurrentTime();
			}
			var operation = this._queue.shift();

			if (operation[3]) {
				this._updateLoadingIndicator(true, operation[3], operation[4]);
			}

			if (operation[2]) {
				// assync operation
				operation[0].call(operation[1] || window, _.bind(this._operationFinished, this, operation[3]));
			} else {
				// synchronous operation
				operation[0].call(operation[1] || window);
				this._operationFinished(operation[3]);
			}
		} else {
			this._setFinished();
		}
	};

	/**
	 * Executed when an operation has finished
	 * @param {String} [key] - If LoadingIndicator should be shown during action you can provide a key for loading action here
	 */
	ExecutionQueue.prototype._operationFinished = function (key) {
		this._timer = null;		
		if (this._batchRunCalls && this._getCurrentTime() - this._blockingJSRuntimeSince < this._loopChunkTime){
			this._executeNext();
		} else {
			this._startTimer();
		}

		if (key) {
			this._updateLoadingIndicator(false, key);
		}
	};

	/**
	 * Clear the queue
	 */
	ExecutionQueue.prototype.clear = function () {
		this._queue = [];
		clearTimeout(this._timer);
		this._timer = null;
	};

	/**
	 * Wait till the queue has finished processing
	 * @param {Function} callback - called whenever the queue is finished and all tasks are processed
	 */
	ExecutionQueue.prototype.whenFinished = function (callback) {
		if (!this._isRunning) {
			callback();
		} else {
			this._finishedCallbacks.push(callback);
		}
	};

	/**
	 * mark the queue as finished (and execute needed callbacks)
	 */
	ExecutionQueue.prototype._setFinished = function () {
		this._isRunning = false;

		_.each(this._finishedCallbacks, function (callback) {
			callback();
		});

		this._finishedCallbacks = [];
	};

	/**
	 * Execute a task (deferred if queue is in asynchronous mode)
	 * @param {Function} fn
	 */
	ExecutionQueue.prototype._defer = function (fn) {
		if (!this._synchronized) {
			return setTimeout(fn, this._interval);
		} else {
			fn();
			return null;
		}
	};

	/**
	 * Update loading indicator if needed
	 * @param {Boolean} start - is an operation started or stopped
	 * @param {String} key - LoadingIndicator key
	 * @param {String} [description] - message to show in the LoadingIndicator
	 */
	ExecutionQueue.prototype._updateLoadingIndicator = function (start, key, description) {
		if (this._shouldUpdateLoadingIndicator) {
			requirejs(["component/loadingindicator/LoadingIndicator"], function (LoadingIndicator) {
				if (start) {
					LoadingIndicator.start(key, description);
				} else {
					LoadingIndicator.stop(key);
				}
			});
		}
	};
	
	/**
	 * Did the queue finish running
	 * @returns {Boolean}
	 */
	ExecutionQueue.prototype.isFinished = function(){
		return !this._isRunning;
	};
	
	/**
	 * When running in Batch mode, consecutive .run() calls will be ran in a non-deferred way if the executionqueue hasn't been blocking the queue for longer than 15ms
	 * If you are queuing a lot of small run tasks, this option can be very interesting to enable.
	 * By default all .run() calls are deferred
	 */
	ExecutionQueue.prototype.setBatchRunCalls = function(batchRunCalls){
		this._batchRunCalls = batchRunCalls;
	};
	
	/**
	 * Get the current time
	 * @returns {Number} current time in ms
	 */
	ExecutionQueue.prototype._getCurrentTime = function(){
		return Date.now();
	};
		

	return ExecutionQueue;
});