define(["utils/HTTPUtils",
        "utils/XMLUtils",
        "underscore",
        "jquery"
       ],
function(HTTPUtils,
          XMLUtils,
          _, 
          $){
    "use strict";
    
    
	/**
	 * This module provides internationalization to the wcr-static project
	 * 
	 * @param locale {string} locale which should be used
	 * @param wcrInstanceUrl {String} url for the WebCenterInstance (e.g. http://localhost:8081/wcr-web)
	 * @param timestamp {String} timestamp - this is used a cache busting parameter for requesting language strings
	 */
	var I18n = function(locale, wcrInstanceUrl, timestamp){
		this._locale = locale ? locale : "en";
		this._timestamp = timestamp ? timestamp : 0;
        this.wcrInstanceUrl = wcrInstanceUrl;
		this.bundles = {};
		this.loading = {};
		this.bundleOrder  = [];
        this.bundleLoadedListeners = {};
	};
	
	/**
	 * Change the locale
	 * 
	 * @param {String} locale - the new locale
	 */
	I18n.prototype.setLocale = function(locale){
		this._locale = locale;
		this.bundles = {};
        var onAllLoaded = function(){};
		for (var i = 0; i < this.bundleOrder.length; i++){
			this.loadBundle(this.bundleOrder[i], onAllLoaded);
		}
	};
	
	/**
	 * Get the locale of the module this object is used for
	 * @returns {String}
	 */
	I18n.prototype.getLocale = function(){
		return this._locale;
	};
		
	/**
	 * Load one or more bundles
	 * 
	 * If fallback is enabled, the fallback will be loaded automatically!
	 * 
	 * @param {String} bundleName - the name of the bundle
	 * @param {Function} [callback] - is called when the bundle is loaded
	 * @param {Object} [binding] - bind the callback to "binding"
	 */
	I18n.prototype.loadBundle = function(bundleName, callback, binding){
		if (callback && binding){
			callback = _.bind(callback, binding);
		}
		
		if (_.isArray(bundleName)){
			// multiple bundles
			// make new callback, which only is executed after bundName.length-1 times
			var multiCallback = _.after(bundleName.length, _.once(callback));
			_.each(bundleName, _.bind(function(bundle){
				this.loadSingleBundle(bundle, multiCallback);
			}, this));
		} else {
			this.loadSingleBundle(bundleName, callback);
		}
	};
	
	/**
	 * Load a bundle
	 * 
	 * WARNING about newlines: "\n" in the translation file is interpreted as a JavaScript newline character!! (see I18n:loadSingleBundle)
	 * 
	 * If fallback is enabled, the fallback will be loaded automatically!
	 * 
	 * @param {String} bundleName - the name of the bundle
	 * @param {Function} [callback] - is called when the bundle is loaded
	 * @param {Function} [callback] - is called when the bundle failed to load
	 */
	I18n.prototype.loadSingleBundle = function(bundleName, callback, errorcallback){
		// shortcut if bundle already loaded
		if (this.bundles[bundleName]){
			if (callback){
				callback(this.bundles[bundleName]);
			}
		} else {
			// bundle info not yet available, register callback
			if (callback){
				this.whenBundleLoaded(bundleName, callback);
			}			
			
			if (!this.loading[bundleName]){
				// the bundle info is not yet being loaded
				this.bundleOrder.push(bundleName);
				this.loading[bundleName] = 1; 
                
				var self = this;
				// t and lk are just for cache busting purposes
				// we use both the timestamp of reloading the languages and the user defined locale as parameters
                $.ajax(HTTPUtils.constructURL("DownloadLocKeys.jsp", { scope : bundleName, t : this._timestamp, lk: this._locale}, this.wcrInstanceUrl), {
                    dataType: "xml",
                    success: function(data, textStatus, jqXHR){
                        // if xml, check if login failed
                        if (data) {
                            var code = data.getElementsByTagName("code");
                            if (code[0]) {
                            	errorcallback(XMLUtils.textContent(code[0]));
                                return;
                            }
                        }

                        var bundle = {};

                        var langs = data.documentElement.getElementsByTagName("lang_string");
                        for(var i=0; i<langs.length;i++){
                            var lang = langs[i];
                            bundle[lang.getAttribute("key")] = lang.getAttribute("value").replace(/\\n/g,'\n');
                        }

                        self.bundles[bundleName] = {};
                        delete self.loading[bundleName];

                        if(bundle){
                        	self.bundles[bundleName] = bundle;
                        }

                        self.fireBundleLoaded(bundleName, self.bundles[bundleName]);
                    },
                    error: function(jqXHR, textStatus, errorThrown){
                        //do error handling
                    	errorcallback();
                    },
                    cache: true,
                    type: "GET"
                });
			}
		}
	};
    
    /**
     * Register a callback to be fired when bundle with bundleName is loaded
     * @param {String} bundleName 
     * @param {Function} callback 
     */
    I18n.prototype.whenBundleLoaded = function(bundleName, callback){
        if(!this.bundleLoadedListeners[bundleName]){
            this.bundleLoadedListeners[bundleName] = [];
        }
        this.bundleLoadedListeners[bundleName].push(callback);
    };
    
    /**
     * Fire all callbacks that were registered 
     * @param {String} bundleName 
     * @param {Object} bundle     
     */
    I18n.prototype.fireBundleLoaded = function(bundleName, bundle){
        if(this.bundleLoadedListeners[bundleName]){
            _.each(this.bundleLoadedListeners[bundleName], function(callback){
                callback(bundle);
            });
            delete this.bundleLoadedListeners[bundleName];
        }
    };
	
	/**
	 * Try to find a value for key, respecting the order in which bundles are loaded
	 * @param {String} key
	 * @returns {String|undefined}
	 */
	I18n.prototype.findByKey = function(key){
		for (var i = this.bundleOrder.length-1; i >= 0; i--){
			if (this.bundles[this.bundleOrder[i]] && this.bundles[this.bundleOrder[i]][key]){
				return this.bundles[this.bundleOrder[i]][key]; 
			}
		}
		
		return undefined;
	};
	
	/**
	 * Try to find a value for a key in a certain bundle.
	 */
	I18n.prototype.findByKeyInBundle = function(key, bundleName){
		if (this.bundles[bundleName] && this.bundles[bundleName][key]) {
			return this.bundles[bundleName][key];
		} else {
			return undefined;
		}
	};
	
	/**
	 * Get the translation of a certain key
	 * 
	 * @param {String} key - key from which you want the translation
	 * @param {...String} [replacements] - if extra parameters are given, they replace {0}, {1}, ... in the translated string
	 * @returns translation - if no translation is found, the key is returned
	 */
	I18n.prototype.t = function(key){
		var value = this.findByKey(key);
		
		if (value !== undefined){			
			// replace all given options
			for (var i = 1; i < arguments.length; i++){
				value = value.replace("{"+(i-1)+"}", arguments[i]);
			}
			
			return value;
		}
					
		// we didn't find any value for key, return key
		return key;
	};
	
	/**
	 * Get the translation of a date based on a key
	 */
	I18n.prototype.d = function(key, value){
		var dateFormat =  this.findByKey(key);
		if (dateFormat !== undefined){
			return value.toString(dateFormat);
		} else {
			return undefined;
		}
	};
	
	return I18n;	
});
