/**
 * JEPE - The "webloggedin" Experiment. (Now called "LoggedInTool")
 * 
 * Can be used to make sure a user is logged in before calling some Save or Submit action.
 * 
 * Usage: see wcr-components
 */
define(["jquery", "utils/Modal", "utils/webi18n!LoginPopup"], function($, Modal, i18n) {
	
	
	/**
	 * @constructor
	 */
	var LoggedInTool = function(){
		// current state...
		this._checking = false;// we are checking if the user is logged in or the popup is open
		this._callbackswhenok = [];
		this._callbackswhenerror = [];
		this._callbackswhencancel = [];
		
		// Timer used for checking if the user closed the login tab every half a second.
		this._timer = null;
		
		// Make the popup
		this._initialize();
	};
	
	
	
	//
	// ---
	//
	// PUBLIC METHODS
	//
	// ---
	//
	
	/**
	 * [Main method...]
	 */
	LoggedInTool.prototype.requireLogin = function(options) {
		var success = options["success"];
		var error = options["error"];
		var cancel = options["cancel"];// Note: cancel is not supported yet! (+ we need an option for that?)
		
		
		if(success){
			this._callbackswhenok.push(success);
		}
		if(error){
			this._callbackswhenerror.push(error);
		}
		if(cancel){
			this._callbackswhencancel.push(cancel);
		}
		
		if(!this._checking) {
			this._checking = true;
			this._checkIfOkNow();
		}
	};
	
	/**
	 * [Utility method...]
	 * 
	 * Call to check if the user is logged in.
	 * 
	 * Can return three possible values:
	 *  - true => user is logged in
	 *  - false => user is not logged in
	 *  - null => I don't know, something is wrong with the server
	 * 
	 * @param {Function} callback should have one argument, which will be one of the above values
	 */
	LoggedInTool.prototype.checkSessionAlive = function (callback) {
		$.ajax({
			url: (webcenterConfig?webcenterConfig.jspBaseUrl:'') + "OpenSession.jsp?nocache="+Math.random(),
			type: "GET",
			data: "xml",
			error: function(jqXHR, textStatus, errorThrown){
				callback(null);
			},
			success: function(xml, textStatus, jqXHR) {
				if($(xml).is("session")){
					// returned session information, ok, we are probably logged in!
					callback(true);
				} else if($("error code", xml).text()==="001") {
					// we are not logged in
					callback(false);
				} else {
					// no correct return value...
					callback(null);
				}
			}
		});
	};
	
	/**
	 * [Utility method...]
	 * 
	 * Log out
	 */
	LoggedInTool.prototype._logout = function(){
		window.location.href = webcenterConfig.webInstAbsPath+"/logoff.jsp";
	};
	
	
	//
	// ---
	//
	// PRIVATE METHODS
	//
	// ---
	//
	
	/**
	 * Initialize the component (make the popup)
	 */
	LoggedInTool.prototype._initialize = function(){
		//
		// Make the popup
		//
		var content = $("<div>").append($("<div>").append($("<p>").text(i18n.t("LOGINPOPUP_POPUP_MESSAGE"))));
		var footer = $("<div>");
		
		this._popup = new Modal(content, {
			allowClose: false,
			width: "modal-sm"
		});
		
		//
		// "Open new tab" button
		//
		var openButton = $("<button>", {
			"type":"button",
			"class":"btn btn-primary",
			"text":i18n.t("LOGINPOPUP_POPUP_RELOGINBUTTON")
		}).click(_.bind(this._openLoginPage, this));
		content.append($('<div class="btn-group btn-group-justified" role="group"/>').append($('<div class="btn-group" role="group"/>').append(openButton)));
		
		//
		// "Retry" button
		//
		var tryAgainButton = $("<button>", {
			"type":"button",
			"class":"btn btn-default"
		})
		.append($("<i>", {"class":"icon icon-WCR_Refresh"}))
		.append($("<span>", {"text":" "}))
		.append($("<span>", {"text":i18n.t("LOGINPOPUP_POPUP_RETRYACTION")}))
		.click(_.bind(this._checkIfOkNow, this));
		footer.append(tryAgainButton);
		
		//
		// "Log out" button
		//
		var logoutButton = $("<button>", {
			"type":"button",
			"class":"btn btn-default"
		})
		.append($("<i>", {"class":"icon icon-WCR_Exit"}))
		.append($("<span>", {"text":" "}))
		.append($("<span>", {"text":i18n.t("LOGINPOPUP_POPUP_LOGOUT")}))
		.click(_.bind(this._logout, this));
		footer.append(logoutButton);
		
		
		this._popup.setFooter(footer);
		
		this._popup.setTitle(i18n.t("LOGINPOPUP_POPUP_TITLE"));
	};
	
	/**
	 * Main function...
	 * 
	 * Checks if the user is logged in.
	 * If he is, we close the popup and call all callbacks
	 * If he is not, we open the popup
	 * If the server is in an error state, ...
	 */
	LoggedInTool.prototype._checkIfOkNow = function(){
		if(!this._checking) {
			return;
		}
		// Hide the popup while checking... Not really necessary, but gives a nice effect.
		this._popup.close();
		// Check
		this.checkSessionAlive(_.bind(function(alive){
			if(!this._checking) {
				return;
			}
			if(alive===false) {
				// Not logged in, ask again...
				
				// WORKAROUND: Due to bug in bootstrap, we cannot open the popup again after we closed it. So, make a new one!
				this._initialize();
				// end workaround
				
				this._popup.open();
			}else if(alive===null) {
				// Something is wrong with the servers...
				this._popup.close();
				this._fireServerErrorAndEnd();
			}else{
				// Ok, the user logged in again
				this._popup.close();
				this._fireLoggedInAndEnd(null /* TODO: pass information we got from the session alive call along...*/);
			}
		},this));
	};
	
	/**
	 * Call to open the login page in a new tab.
	 * Uses the timer variable.
	 * 
	 * Will callback when the user closed the tab.
	 * 
	 * @param callbackToRetry callback which is called when the user closes the tab.
	 */
	LoggedInTool.prototype._openLoginPage = function(){
	    var win = window.open(webcenterConfig.webInstAbsPath+"/login.jsp?urlwanted=closewindow.jsp", 
	    	"LoginPopupLoginInNewTab");
	    clearInterval(this._timer);
	    this._timer = setInterval(_.bind(function() {
	        if (win.closed) {
	        	clearInterval(this._timer);
	        	this._checkIfOkNow();
	        }
	    }, this), 500);
	};
	
	/**
	 * Fire the logged in event.
	 */
	LoggedInTool.prototype._fireLoggedInAndEnd = function(logininfo){
		var lasterror = undefined;
		for(var i=0;i<this._callbackswhenok.length;i++) {
			try{
				this._callbackswhenok[i](logininfo);
			} catch(e) {
				lasterror = e;
			}
		}
		this._endOfLoginCheck();
		if(lasterror) {
			throw lasterror;
		}
	};
	
	/**
	 * Fire the server error event
	 */
	LoggedInTool.prototype._fireServerErrorAndEnd = function(){
		console.log("[ERR] webloggedin failed!!! Server returned wrong information. Cannot check if logged in.");
		for(var i=0;i<this._callbackswhenerror.length;i++) {
			this._callbackswhenerror[i]();
		}
		this._endOfLoginCheck();
	};
	
	/**
	 * Fire the server error event
	 */
	LoggedInTool.prototype._fireCancelAndEnd = function(){
		console.log("[WARN] User not logged in, but user cancelled.");
		for(var i=0;i<this._callbackswhencancel.length;i++) {
			this._callbackswhencancel[i]();
		}
		this._endOfLoginCheck();
	};
	
	/**
	 * Called at the end of the login checking...
	 * 
	 * At this moment the callbacks are called and the popup is closed...
	 * 
	 * Turn back to initial state...
	 */
	LoggedInTool.prototype._endOfLoginCheck = function(){
		this._checking = false;
		this._callbackswhenok = [];
		this._callbackswhenerror = [];
		this._callbackswhencancel = [];
		clearInterval(this._timer);
	};
	
	//
	// NOTE: SINGLETON ! Because we only want to check once at the same time!
	//
	var singleton = new LoggedInTool();
	return singleton;
	
});