define([
		"underscore"
	],
	function (
		_
	) {
		"use strict";

		/**
		 * PriorityOrderedObjects keeps track of objects with a given priority. Multiple objects
		 * can have a the same priority and it's still possible to search for an object by key.
		 *
		 * @exports PriorityOrderedObject
		 * @class
		 */
		var PriorityOrderedObject = function () {
			// Priority -> Array with keys
			this._priority = [];
			// Key -> Item
			this._object = {};
		};

		/**
		 * Add the object with key with a given priority.
		 * @param {String} key   - Key of the item
		 * @param {Object} item  - Any item
		 * @param {Number} priority - the priority of the item
		 */
		PriorityOrderedObject.prototype.addItemWithPriority = function (key, item, priority) {
			if (this._object[key]) {
				throw new Error("Duplicate key exception");
			}
			if (this._priority[priority]) {
				this._priority[priority].push(key);
			} else {
				this._priority[priority] = [key];
			}
			this._object[key] = item;
		};

		/**
		 * Get the item by key
		 * @param {String} key - Key of the item
		 * @returns {Object} The item with the specified key
		 */
		PriorityOrderedObject.prototype.getItemByKey = function (key) {
			return this._object[key];
		};

		/**
		 * Check if  a certain key is present in the ordered object
		 * @param {String}  key - key of the item
		 * @returns {Boolean}
		 */
		PriorityOrderedObject.prototype.containsKey = function (key) {
			return this._object[key] !== undefined;
		};

		/**
		 * Remove the item with the given key
		 * @param {String} key - Key of the item
		 * @returns {Any} the deleted object, undefined if nothing was removed
		 */
		PriorityOrderedObject.prototype.removeByKey = function (key) {
			var deleted;
			if (this._object[key] !== undefined) {
				var array = _.find(this._priority, function (keyArray) {
					if (keyArray) {
						return keyArray.indexOf(key) >= 0;
					}
				});

				if (array) {
					array.splice(array.indexOf(key), 1);
					deleted = this._object[key];
					delete this._object[key];
				}
			}

			return deleted;
		};

		/**
		 * Return an array of the objects as they are ordered
		 * @returns {Array}
		 */
		PriorityOrderedObject.prototype.getOrderedObjectList = function () {
			var orderedObjectList = [];
			_.each(this._priority, function (keyArray) {
				_.each(keyArray, function (key) {
					orderedObjectList.push(this._object[key]);
				}, this);
			}, this);
			return orderedObjectList;
		};

		/**
		 * Get the items as object
		 * @returns {Object} The items as object
		 */
		PriorityOrderedObject.prototype.getItemsAsObject = function () {
			return _.clone(this._object);
		};

		/**
		 * Returns the number of items in the priority ordered object.
		 * @returns {Number}
		 */
		PriorityOrderedObject.prototype.getLength = function () {
			var length = 0;
			_.each(this._priority, function (keyArray) {
				if (keyArray) {
					length += keyArray.length;
				}
			}, this);
			return length;
		};

		/**
		 * Is the priority ordered object empty.
		 * @returns {Boolean}
		 */
		PriorityOrderedObject.prototype.isEmpty = function () {
			return _.isEmpty(this._object);
		};

		return PriorityOrderedObject;

	});