define(["jquery", "underscore"], function($,_){
	
	/**
	 * Utility class for responsive buttons with more dropdown
	 * @constructor
	 * @class
	 * @param {JQueryObj} container - the container containing all elements
	 * @param {JQueryObj} moreElement - reference to the more element
	 * @param {JQueryObj} moreDropdown - reference to the container for more items
	 * @param {Function} getMaximumWidth - function to get the maximum with for the container element
	 */
	var ResponsiveFit = function(container, moreElement, moreDropdown, getMaximumWidth){
		this._container = container;
		this._moreElement = moreElement;
		this._moreContainer = moreDropdown;
		
		this._maintainSequence = false;
		
		if (getMaximumWidth){
			this._getMaximumWidth = getMaximumWidth;	
		} 		
	};
	
	/**
	 * Call this to maintain the orinal sequence of actions 
	 * all actions will get a sequence number based on their current order
	 * @param {Boolean} maintSequence
	 */
	ResponsiveFit.prototype.setMaintainSequence = function(maintainSequence){
		if (maintainSequence){			
			this._storeSequence();			
		} 
		
		this._maintainSequence = maintainSequence;
	};
	
	/**
	 * Store the order of elements at initialization
	 */
	ResponsiveFit.prototype._storeSequence = function () {
		
		var mainChildren = this._getMainChildren();
		_.each(mainChildren, function (child, index) {
			$(child).attr("data-sequence", index);
		});
		
		var moreStartIndex = mainChildren.length;
		
		var more = this._getMoreChildren();
		_.each(more, function (child, index) {
			$(child).attr("data-sequence", moreStartIndex+index);
		});		
	};
	
	
	
	/**
	 * Get the children in the main container
	 */
	ResponsiveFit.prototype._getMainChildren = function(){
		return  this._container.children().not(this._moreElement);
	};
	
	ResponsiveFit.prototype._getMaximumWidth = function(){
		return this._container.innerWidth();
	};
	
	/**
	 * Get the children in the more container
	 */
	ResponsiveFit.prototype._getMoreChildren = function(){
		return this._moreContainer.children();
	};
	
	/**
	 * Get the width of an element
	 * @param {JQueryObj} child - element to get width of
	 */
	ResponsiveFit.prototype._getElementWidth = function(child){
		return child.outerWidth(true);
	};
	
	ResponsiveFit.prototype._canMoveToMore = function(child){
		return true;
	};
	
	ResponsiveFit.prototype._canMoveToMain = function(child){
		return true;
	};
	
	ResponsiveFit.prototype._appendToMore = function(child){	
		this._moreContainer.append(child);
		return child;
	};
	
	/**
	 * Prepend a child in the more container
	 * @param {JQueryObj} child 
	 */
	ResponsiveFit.prototype._prependToMore = function(child){
		if (this._maintainSequence){	
			// we should maintain the sequence of items
			var children = this._getMoreChildren();
			var order = parseInt(child.attr("data-sequence"), 10);
			var nextIndex = _.findIndex(children, function (item) {
				var itemOrder = parseInt($(item).attr("data-sequence"), 10);
				return (!isNaN(itemOrder) && order < itemOrder);
			});

			if (nextIndex > -1) {
				child.insertBefore(children[nextIndex]);
			} else {
				this._moreContainer.append(child);
			}
		} else {			
			this._moreContainer.prepend(child);
		}
		return child;
	};
	
	/**
	 * Append an element to the main container
	 * @parma {JQueryObj} child
	 */
	ResponsiveFit.prototype._appendToMain = function(child){
		if (this._maintainSequence){
			// we should maintain the original sequence of items
			var children = this._getMainChildren();
			var order = parseInt(child.attr("data-sequence"), 10);
			var nextIndex = _.findIndex(children, function (item) {
				var itemOrder = parseInt($(item).attr("data-sequence"), 10);
				return (!isNaN(itemOrder) && order < itemOrder);
			});

			if (nextIndex > -1) {
				child.insertBefore(children[nextIndex]);
			} else {
				child.insertBefore(this._moreElement);
			}
		} else {
			child.insertBefore(this._moreElement);	
		}
		return child;
	};
	
	/**
	 * Apply the Responsive Fit magic
	 */
	ResponsiveFit.prototype.resize = function(){
		var maxWidth = this._getMaximumWidth();	
		var child;
		var mainChildren = this._getMainChildren();
		
		
		var availableWidth = maxWidth;
	
		if (this._getMoreChildren().length > 0){
			availableWidth -= this._getElementWidth(this._moreElement);
		}
		
		var i = 0;
		var j = 0;
		while (i < mainChildren.length){
			child = $(mainChildren[i++]);			
			availableWidth -= this._getElementWidth(child);
		}
		
		if (availableWidth <= 0){
			// there was not enough space
			// go back one element
			i--;
			
			
			// display the more element if needed
			if (this._getMoreChildren().length === 0){
				// the more element was not yet visible, display it
				this._moreElement.toggleClass("hiddenElement", false);
				availableWidth -= this._getElementWidth(this._moreElement);
				
			}
				
			j = mainChildren.length-1;
			// append to more container (this should be starting from the back... otherwise the sequence goes wrong)
			while (availableWidth < 0 && j >= 0){
				child = $(mainChildren[j--]);
				if (this._canMoveToMore(child)){
					availableWidth += this._getElementWidth(child);
					this._prependToMore(child);
				}			
			}			
		} else {
			// there is more space available, we might be able to move out some elements of the more button
			var finished = false;
			j = 0;
			while (availableWidth >= 0 && !finished && j < this._getMoreChildren().length){
				child = $(this._getMoreChildren()[j]);
				var widthToBeSubtractedOnUndo = 0;
				
				if( this._canMoveToMain(child)){
					child = this._appendToMain(child);
					var childWidth = this._getElementWidth(child);
					availableWidth -=childWidth;


					if (this._moreContainer.children().length === 0){
						widthToBeSubtractedOnUndo =  this._getElementWidth(this._moreElement);
						availableWidth += widthToBeSubtractedOnUndo;
						this._updateMoreVisibility();
					} 

					if (availableWidth < 0){
						// undo :P
						this._prependToMore(child);
						availableWidth += childWidth;	
						availableWidth -=  widthToBeSubtractedOnUndo;
						this._updateMoreVisibility();
						finished = true;
					}
				} else {
					j++;
				}
			}
		}
		
		this._updateMoreVisibility();
	};
	
	/**
	 * Update the visibility of the more element (based on the children)
	 */
	ResponsiveFit.prototype._updateMoreVisibility = function(){
		this._moreElement.toggleClass("hiddenElement", this._getMoreChildren().length == 0);	
	};
	
	/**
	 * Move all elements out of the more container to the main container
	 */
	ResponsiveFit.prototype.moveAllToMain = function(){
		_.each(this._getMoreChildren(), function(child){
			this._appendToMain($(child));
		}, this);	
		this._updateMoreVisibility();
	};
	
	return ResponsiveFit;
	
});