/**
 * @author PASH
 * SMSDrop stands for "Search Multiple Selection Drop-down".
 * SMSDrop is provides one drop-down for one id. <br>
 * Each drop-down is specifically created with the SMSDrop object and all the operations should be performed with the object created for specific drop-down.
 * By using this utility user can select multiple options from the drop-down by checking check-box.<br>
 * User can start typing to minimize the list of option, list will be populated according to the string entered in search input field.<br>
 * Selection for all or remove all the selection feature is also available.
 */
define(["jquery",
        "underscore",
        "utils/webi18n!SMSDropStrings",
        "wcr/hover"],
function($,
		_,
		webi18n) {
	"use strict";
	
	/**
	 * Constructor of SMSDrop
	 * @param : id {id is required to create element for drop-down}
	 * @param : separaterFunc {separaterFunc is a function to join selected values to get as a result. By default values will be separated with ' | '.}
	 */
	var SMSDrop = function(id, separaterFunc) {
		if (!id) {
			alert(webi18n.t("WRONGIDERRORMSG", id));
			return;
		}
		
		if (separaterFunc && $.type(separaterFunc) === "function") {
			this.separaterFunc = separaterFunc;
		} else {
			// Default function if function is not available in parameter
			this.separaterFunc = function(selectedOptions){
			    var stringValue = "";
			    $.each(selectedOptions, function(index, option) {
			    	stringValue += option.key;
			    	if (index < selectedOptions.length - 1) {
			    		stringValue += ' | ';
			    	}
			    });
			    return stringValue;
			};
		}
		
		this.id = id;
		this.optionValues = [];
		this.values = [];
		this.createEvent = true;
		this.onChangeCallBacks = [];
	};
	
	/**
	 * Getter method to get JQuery element for drop-down
	 * @returns JQuery element
	 */
	SMSDrop.prototype.getJQElement = function() {
		return this.renderSelectMultiple();
	};

	/**
	 * Method is to clear all the drop down options
	 */
	SMSDrop.prototype.clearAll = function() {
		this.optionValues = [];
		this.values = [];
		this.displaySelection();
	};
	
	/**
	 * Event listener to executes function whenever value for drop down is changing.
	 * @param callBack {callBack is a function which will execute on change of drop down}
	 */
	SMSDrop.prototype.onChange = function(callBack) {
		this.onChangeCallBacks.push(callBack);
		
	};
	
	/**
	 * Method to get input to generate options for the drop-down.
	 * @param optionValues {Array with key and value pair objects (like: optionValues.push({key:'value of option', value:'text of option'}); )}
	 * @param selectedValues {Boolean or an Array for the preselected values (like: selectedValues.push({key:'value of option', value:'text of option'})), 
	 * This parameter should keep values of the drop-down option or it should be true for all the preselected options in the drop down }
	 */
	SMSDrop.prototype.addElementOptions = function(optionValues, selectedValues) {
		if (optionValues && optionValues.length > 0) {
			this.optionValues = optionValues;
			this.optionValues.sort(this.sortByValue);
		}
		
		if (selectedValues) {
			if ($.isArray(selectedValues) && selectedValues.length > 0) {
				this.values = $.extend(true, [], selectedValues);
			} else if (typeof selectedValues === 'boolean' && selectedValues)  {
				this.values = $.extend(true, [], this.optionValues);
			}
		}
		this.displaySelection();
	};
	
	/**
	 *  This method is to get value for selected options in the drop-down.
	 * @returns {Array of values like - ({key: 'key', value: 'value'}) }
	 */
	SMSDrop.prototype.val = function() {
		return this.values;
	};
	
	/**
	 * This method is works as a compare method to help sorting based on the values.
	 * @param obj1 {key: 'key1', value: 'value1'}
	 * @param obj2 {key: 'key2', value: 'value2'}
	 * @returns
	 */
	SMSDrop.prototype.sortByValue = function(obj1, obj2) {
		if (val1 && val2 && val1.value & val2.value) {
			var val1 = obj1.value.toLowerCase();
			var val2 = obj2.value.toLowerCase();
			return (val1 == val2) ? 0 : (val1 > val2) ? 1 : -1;
		}
	};
	
	/**
	 * Method is to render field for selected options and a button to show drop-down <br>
	 * This method ensures that duplicate drop down is not populated with same id. <br>
	 * It removes previous drop down and create a fresh down each time.
	 * @returns {table as JQuery object}
	 */
	SMSDrop.prototype.renderSelectMultiple = function() {
		var outerDiv;
		if ($('#outdiv_' + this.id).length) {
			$('#outdiv_' + this.id).remove();
			this.createEvent = false;
		}
		outerDiv = $('<div>');
		outerDiv.attr('id', 'outdiv_' + this.id);
		
		var valueHolder = $('<input>');
		valueHolder.attr({'id': this.id, 'type': 'text'});
		valueHolder.css('display', 'none');
		
		var table = $('<table>');
		table.attr({'valign': 'top', 'padding': '0'});
		
		outerDiv.append(table);
		table.append(valueHolder);
		
		var row = $('<tr>');
		row.attr('padding', '0');
		table.append(row);

		var col1 = $('<td>');
		col1.css('padding', '0px');
		row.append(col1);
		
		var col2 = $('<td>');
		col2.css({'padding': '0 0 0 0', 'vertical-align': 'top'});
		row.append(col2);

		var img = $('<img id="' + this.id + '_popUpIcon" src="' + webcenterConfig.imageBaseUrl + 'ar_tp.gif">');
		col2.append(img);
		this.dropButtonEvent();

		var mvDiv = $('<div>');
		mvDiv.attr('id', 'mvdiv_' + this.id);
		mvDiv.css({'background-color': 'white', 'vertical-align': 'top', 
			'padding': '0 0 0 0', 'border': '1px solid grey', 
			'min-width': '230px', 'min-height': '15px',
			'max-height': '75px', 'overflow-y': 'auto'});
		col1.append(mvDiv);
		
		// executing all the call back function on inputChange event.
		valueHolder.on('inputChange', _.bind(function() {
			$.each(this.onChangeCallBacks, function(index, callBack) {
				callBack();
			});
		}, this));
		
		return outerDiv;
	};
	
	/**
	 * Initiating multiple selection drop-down.
	 * @param event {Existing object to know about event happened on the element.}
	 */
	SMSDrop.prototype.executeWithNotification = function(event) {
		"use strict";

		var dropdownItems = [];  // dropdownmenu expects and array of html code snippets, so we give it an array of 1 element
		dropdownItems.push('<img src="' + webcenterConfig.imageBaseUrl + 'ajax_loader.gif" />');
		var menuWidth = '400px', menuHeight = '300px', scrollHor = false, nohide = true, bgcolor = "#ffffff", nohiddonclick = false;
		
		$("#dropmenudiv").css('z-index', 10000);	// 10000 is an assumption that will be the highest z-index to display drop down.
		$("#dropmenudiv").addClass('showOverPopup'); // Adding class to deal with _allowInteraction for popup.
		
		dropdownmenu($('#mvdiv_' + this.id)[0], event, dropdownItems, menuWidth, menuHeight, scrollHor, nohide, bgcolor, nohiddonclick);
		
		this.displayMultiselect(event);				
	};

	/**
	 * Creates top section of drop-down and initiate method to populate options
	 * @param event
	 */
	SMSDrop.prototype.displayMultiselect = function(event) {
		var tableDBValue = $('<table>');
		tableDBValue.attr('id', 'dbValues' + this.id);
		tableDBValue.width($('#mvdiv_' + this.id).width());
		
		var rowSelectDeselectAll = $('<tr>');
		rowSelectDeselectAll.attr('class','darktext_greenlink_11');
		tableDBValue.append(rowSelectDeselectAll);
		
		var colSelectAll = $('<td>').css('display', 'flex');
		colSelectAll.attr('class', 'nobr');
		colSelectAll.css({'cursor': 'default', 'align': 'center'});
		rowSelectDeselectAll.append(colSelectAll);
		
		var linkSelectAll = $('<a>');
		linkSelectAll.attr({'class': 'linkSelectAll', 'href': '#'});
		linkSelectAll.css('padding-right', '30px');
		linkSelectAll.text(webi18n.t('SELECTALL'));
		colSelectAll.append(linkSelectAll);
		
		var linkDeselectAll = $('<a>');
		linkDeselectAll.attr({'class': 'linkDeselectAll', 'href': '#'});
		linkDeselectAll.text(webi18n.t('DESELECTALL'));
		colSelectAll.append(linkDeselectAll);
		
		var rowFilterInput = $('<tr>');
		rowFilterInput.attr('class','darktext_greenlink_11');
		tableDBValue.append(rowFilterInput);
		
		var colFilterInput = $('<td>');
		rowFilterInput.append(colFilterInput);
		
		var fieldFilterInput = $('<input>');
		fieldFilterInput.prop({'type': 'text', 'autocomplete': 'off', 'class': 'FieldInputFilter'});
		colFilterInput.append(fieldFilterInput);
		
		var rowDropDownOptions = $('<tr>');
		rowDropDownOptions.attr('class','darktext_greenlink_11');
		tableDBValue.append(rowDropDownOptions);
		
		var colDropDownOptions = $('<td>');
		rowDropDownOptions.append(colDropDownOptions);

		var dropmenucontainer = $('<div>');
		dropmenucontainer.attr('id', 'dropmenucontainer');
		dropmenucontainer.css({'width': '100%', 'max-height': '300px', 'display': 'block', 'overflow': 'auto', 'overflow-x': 'hidden'});
		colDropDownOptions.append(dropmenucontainer);
		
		// call a method to generate list of options with check box.
		dropmenucontainer.append(this.multiValueDropdownOptions());
		
		var selectDropmenucontainer = $('<div>');
		selectDropmenucontainer.attr('id', 'selectdropmenucontainer');
		selectDropmenucontainer.css({'width': '100%', 'display': 'block'});
		selectDropmenucontainer.append(tableDBValue);
		
		var dropdownItems = [];
		dropdownItems.push(selectDropmenucontainer.html());
		var menuWidth = '400px', menuHeight = '300px', scrollHor = false, nohide = false, bgcolor = "#ffffff", nohiddonclick = true;
		
		dropmenuobj.style.visibility = 'hidden';
		dropdownmenu($('#mvdiv_' + this.id)[0], event, dropdownItems, menuWidth, menuHeight, scrollHor, nohide, bgcolor, nohiddonclick);
		dropmenuobj.style.visibility = 'visible';
		
		this.addMultiSelectEvents();
	};
	
	/**
	 * Creates options with check box in drop-down
	 * @returns
	 */
	SMSDrop.prototype.multiValueDropdownOptions = function() {
		var tableDropDownOptions = $('<table>');
		
		var optionValues = this.optionValues;
		var values = this.values;
		
		$.each(optionValues, function(index, object) {
			var rowOption = $('<tr>');
			rowOption.attr({'class': 'darktext_greenlink_11', 'name': 'MultiValueSelectTr'});
			tableDropDownOptions.append(rowOption);
			
			var colOption = $('<td>');
			rowOption.append(colOption);
			
			var checkOption = $('<input>');
			checkOption.attr({'type': 'checkbox', 'value': object.key, 'class': 'multioption', 'id': object.key + '_' + this.id});
			checkOption.css('top', '-35px');
			
			var selected = $.grep(values, _.bind(function( n, i ) {
				return (n.key === object.key);
			}, this));
			
			if (selected.length > 0) {
				checkOption.attr("checked", true);
			}
			
			colOption.append(checkOption);
			
			var checkLable = $('<label>');
			checkLable.attr('for', object.key + '_' + this.id);
			checkLable.text(object.value);
			colOption.append(checkLable);
		});
	
		this.displaySelection();
		return tableDropDownOptions;
	};
	
	/**
	 * Adding click event for display or hide drop-down
	 */
	SMSDrop.prototype.dropButtonEvent = function() {
		var self = this;
		var dropButtenEventFunction = function(event) {
			if(!dropmenuobj || dropmenuobj.style.visibility !== 'visible') {
				self.executeWithNotification(event);
			} else {
				dropmenuobj.style.visibility = 'hidden';
			}
		};
		
		// if events are already bound then do not bind again.
		if (self.createEvent) {
			$(document).on("click", "#" + this.id + "_popUpIcon", dropButtenEventFunction);
			$(document).on("click", "#mvdiv_" + this.id, dropButtenEventFunction);
		}
	};
	
	/**
	 * Handles all other events for drop-down
	 */
	SMSDrop.prototype.addMultiSelectEvents = function() {
		this.eventSelectAll();
		this.eventDeSelectAll();
		this.eventCheckBox();
		this.eventFilterOptions();
	};
	
	/**
	 * Event handling for filter input field
	 */
	SMSDrop.prototype.eventFilterOptions = function() {
		$('#dbValues' + this.id).find('.FieldInputFilter').focus();
		$('#dbValues' + this.id).find('.FieldInputFilter').on("input change", _.bind(function() {
			this.showFilteredDropdownOptions($('#dbValues' + this.id).find('.FieldInputFilter')[0].value);
		}, this));
	};
	
	/**
	 * Event to select all the options by click on hyper link for "Select All"
	 */
	SMSDrop.prototype.eventSelectAll = function() {
		$('#dbValues' + this.id).find('.linkSelectAll').on("click", _.bind(function() {
			var list = $('#dbValues' + this.id).find('.multioption');
			$.each(list, _.bind(function(index, item) {
				var display = $(item).parent().css('display');
				// Ignore selection if not displayed in the drop down options
				if (display !== 'none') {
					item.checked = true;
					var value = item.value;
					var text = $("label[for='" + item.id + "']").text();
					this.updateValue(true, value, text);
				}
			}, this));
			this.displaySelection();
			
			// Trigger the event for inputChange.
			$('#' + this.id).trigger("inputChange", $('#' + this.id).val());
		}, this));
	};
	
	/**
	 * Method to deselect all the options by click on hyper link for "Deselect All"
	 */
	SMSDrop.prototype.eventDeSelectAll = function() {
		$('#dbValues' + this.id).find('.linkDeselectAll').on("click", _.bind(function() {
			var list = $('#dbValues' + this.id).find('.multioption');
			$.each(list, function(index, item) {
				item.checked = false;	// uncheck all the check boxes
			});
			this.values = [];	// Empty selected values
			this.displaySelection();
			
			updateHoverPosition($('#mvdiv_' + this.id)[0], 0, 0);	// updated hover position related to the div.
			$('#' + this.id).trigger("inputChange", '');
		}, this));
	};
	
	/**
	 * Displays selected values
	 */
	SMSDrop.prototype.displaySelection = function() {
		$('#mvdiv_' + this.id).empty();
		$('#' + this.id).val("");
		
		$('#' + this.id).attr("value", '');
		
		var innerTable = $('<ul>');
		innerTable.attr('class', 'darktext_darklink_11');
		innerTable.css({'padding': '0px', 'padding-top': '0px', 'padding-bottom': '0px', 'margin': '0px', 'list-style-type': 'none'});
		$('#mvdiv_' + this.id).append(innerTable);
		
		this.values.sort(this.sortByValue);
		
		var selectedValues = this.values;
		$.each(selectedValues, function(index, item) {
			var innerRow1 = $('<li>');
			innerRow1.append(item.value);
			innerTable.append(innerRow1);
		});
		var strValues = this.separaterFunc(this.values);
		if (strValues) {
			$('#' + this.id).attr('value', strValues);
			$('#' + this.id).val(strValues);
		}
	};
	
	/**
	 * Update result array for the selection or de selection
	 * @param checked {boolean value to know selection or de selection}
	 * @param value {value of the option}
	 * @param text {text fo the option}
	 */
	SMSDrop.prototype.updateValue = function(checked, key, value) {
		var index = this.getObjIndexByKey(this.values, key);
		if (checked) {
			if (index === -1) {
				this.values.push({key: key, value: value});
			}
		} else {
			if (index != -1) {
				this.values.splice(index, 1);
			}
		}
	};
	
	/**
	 * Method to get object index from the array based on the key
	 * @param array in structure of [{key: 'key', value: 'value'}]
	 * @param key as string
	 * @returns {Number} returns -1 if object is not found.
	 */
	SMSDrop.prototype.getObjIndexByKey = function(array, key) {
		var i = -1;
		$.each(array, function(index, value){
		   if(value.key === key){ i = index; return false; }
		});
		return i;
	};
	
	/**
	 * Handling change event for check box of the drop-down
	 */
	SMSDrop.prototype.eventCheckBox = function() {
		var self = this;
		$('#dbValues' + this.id).find('.multioption').on('change', function() {
			var value = this.value;
			var text = $("label[for='" + this.id + "']").text();
			self.updateValue(this.checked, value, text);
			self.displaySelection();
			
			$('#' + self.id).trigger("inputChange", $('#' + self.id).val());
		});
	};
	
	/**
	 * Method is used to display only filtered options based of requested term
	 * @param reqTerm {String provided in filter input field.}
	 */
	SMSDrop.prototype.showFilteredDropdownOptions = function(reqTerm) {
		var matchedValueHash = {};
		
		for (var i = 0; i < this.optionValues.length; i++) {
        	if (this.optionValues[i].value.toLowerCase().indexOf(reqTerm.toLowerCase()) !== -1) {
        		matchedValueHash[this.optionValues[i].key] = this.optionValues[i].value;
        	}
        }

		var list = $('#dbValues' + this.id).find('.multioption');
		$.each(list, _.bind(function(index, item) {
			var value = matchedValueHash[item.value];
			
			if (value || item.checked) {
				if (value) {
					var startHighLight = value.toLowerCase().indexOf(reqTerm.toLowerCase());
					var firstPart = value.substr(0, startHighLight);
					var highLightPart = value.substr(startHighLight, reqTerm.length);
					var lastPart = value.substr(startHighLight + reqTerm.length, value.length);
					
					var optionTextPart1 = $('<span>').text(firstPart);
					var optionTextPart2 = $('<b>').text(highLightPart);
					var optionTextPart3 = $('<span>').text(lastPart);
					
					$("label[for='" + item.id + "']").empty();
					$("label[for='" + item.id + "']").append(optionTextPart1).append(optionTextPart2).append(optionTextPart3);
				}
				
				$(item).parent().css('display', 'block');
			} else {
				$(item).parent().css('display', 'none');
			}
		}, this));
	};
	
	return SMSDrop;
});










	
