define("utils/init", ["utils/ExecutionQueue", "jquery"], function (ExecutionQueue, $) {
	var exports = {};


	/**
	 * Initialize all wcr components on the page (or in a certain subtree)
	 * 
	 * (Calls wcrinit on the module defined by the data-init attribute) 
	 * 
	 * @param root the subtree to process...
	 * @param callback callback when done...
	 */
	exports.doinit = function (root, callback) {
		if (root !== undefined && root != null) {
			root = $(root);
		}
		var found = 0;
		var inited = 0;

		var execQueue = new ExecutionQueue();
		execQueue.setBatchRunCalls(true); // we rely on multiple .run() calls on the ExecutionQueue, by default these are all deferred, but we want to batch them so we work around 15ms without deferring to execute the next run call

		// Searches for all data-init attributes...
		$("*[data-init]", root).each(function (i, e) {
			// If that node was not initialized before... 
			if (!e.datainitdone) {
				e.datainitdone = true;
				found++;
				// look inside the attribute and get the module we need
				
				var module_to_init = $(e).attr("data-init");
			
				require([module_to_init], function (mod) {
					// make sure to run the module initializations in the execution queue
					execQueue.run((function (i, e, mod, module_to_init) {
						return function () {
							// now call the wcrinit(div) method on that module
							if (mod instanceof Object && mod.wcrinit) {
								// but save the function to destroy the element first...
								if (mod.wcrdestroy !== undefined) {
									e.destroyfunction = mod.wcrdestroy;
								}

								// initialize the actual component
								mod.wcrinit(e);

								// init was called...
								e.initcalled = true;

								// call init listeners on this element
								if (e.initlisteners !== undefined) {
									$.each(e.initlisteners, function (listeneri, listenerf) {
										listenerf.call(e);
									});
									e.initlisteners = undefined;
								}
							} else {
								require(["utils/wcrerror"], function (WCRError) {
									throw new WCRError(null, "Could not initialize module \"" + module_to_init + "\" on component. This module does not support node initialization...");
								});
							}

							// for the callback of this function
							inited++;
							if (inited === found && callback !== undefined) {
								callback();
							}
						};
					})(i, e, mod, module_to_init));
				});
			}

		});



		if (inited === found && callback !== undefined) {
			callback();
		}
	};

	/**
	 * Destroy all wcr components on the page (or in a certain subtree)
	 * ! Do this before removing html from the page...
	 * 
	 * (Calls wcrdestroy on the module defined by the data-init attribute)
	 * 
	 * This callback should be used by the module to remove listeners and stop timers...
	 * 
	 * @param root the subtree to process...
	 * @param callback callback when done...
	 */
	exports.dodestroy = function (root, callback) {
		if (root !== undefined && root != null) {
			root = $(root);
		}
		// Searches for all data-init attributes...
		$("*[data-init]", root).each(function (i, e) {
			// If that node was not initialized before... 
			if (e.datainitdone === true) {
				e.datainitdone = false;
				var destroy = function () {
					// check if there is a destroyfunction...
					if (e.destroyfunction !== undefined) {
						e.destroyfunction(e);
						delete e.destroyfunction;
					}
				};

				if (!e.initcalled) {
					// init has been called but is still loading the require module...
					if (e.initlisteners === undefined) {
						e.initlisteners = [];
					}
					e.initlisteners.push(destroy);
				} else {
					// the require module is loaded too...
					destroy();
				}
			}
		});
	};

	/**
	 * Wait for the component of a certain element to be initialized and call the callback if it is.
	 * 
	 * @param jQueryComponent a jQuery object (possibly containing multiple elements)
	 * @param callback the callback to execute (no arguments)
	 */
	exports.whenInitialized = function (jQueryComponent, callback) {
		if (jQueryComponent instanceof $) {
			var waitfor = jQueryComponent.length;
			if (waitfor === 0) {
				callback();
			}

			function onedonewaiting() {
				waitfor--;
				if (waitfor === 0) {
					callback();
				}
			}
			jQueryComponent.each(function (i, e) {
				if (e.datainitdone && e.initcalled) {
					// already initialized
					onedonewaiting();
				} else if (!$(e).attr("data-init")) {
					// not a component
					onedonewaiting();
				} else {
					// has data-init & not (fully) initialized yet
					if (e.initlisteners === undefined) {
						e.initlisteners = [];
					}
					e.initlisteners.push(onedonewaiting);
				}
			});
		} else {
			if (console && console.log){
				console.log("init.whenInitialized called with wrong first argument, expects jQuery selector");
			}
		}
	};

	return exports;
});