define(["underscore"], function (_) {

	"use strict";



	/**
	 * A simple implementation of a prefix tree - for quick prefix searching
	 * 
	 * @class
	 * @exports PrefixTree
	 */
	var PrefixTree = function () {
		this._root = new PrefixTreeEntry();
	};

	/**
	 * an entry in the prefix tree
	 */
	var PrefixTreeEntry = function () {
		this.objects = [];
		this.childs = {};
	};

	/**
	 * adds an object to the prefix tree
	 * @param {String}   key    - the key for this object
	 * @param {Object} object 
	 */
	PrefixTree.prototype.add = function (key, object) {
		addEntry(this._root, key.toLowerCase(), object);
	};


	/**
	 * Searches for all objects that have a key with the given prefix
	 * @param   {String}   prefix    
	 * @param   {Function} [filter] - filter the objects with this function, return true if it should be added, false if it should be ignored  
	 * @returns {Array.<Object>} 
	 */
	PrefixTree.prototype.find = function (prefix, filter) {
		var entries = [];
		findEntries(this._root, prefix.toLowerCase(), filter, entries);
		return entries;
	};

	/**
	 * help method to add an entry
	 * @param {PrefixTreeEntry}   prefixTreeEntry 
	 * @param {String}   prefix          
	 * @param {Object} object          
	 */
	var addEntry = function (prefixTreeEntry, prefix, object) {
		if (prefix === "") {
			prefixTreeEntry.objects.push(object);
		} else {
			var char = prefix.charAt(0);
			var remainder = prefix.substring(1);
			if (!prefixTreeEntry.childs[char]) {
				prefixTreeEntry.childs[char] = new PrefixTreeEntry();
			}
			addEntry(prefixTreeEntry.childs[char], remainder, object);
		}
	};

	/**
	 * help method to find entries
	 * @param   {Object}   prefixTreeEntry 
	 * @param   {PrefixTreeEntry}   prefix          
	 * @param   {Function} filter          
	 * @param   {Array.<Object>} entries - the array to which the entries are added          
	 */                                   
	var findEntries = function (prefixTreeEntry, prefix, filter, entries) {
		if (typeof prefixTreeEntry === "undefined") {
			return;
		}
		if (prefix === "") {
			_.each(prefixTreeEntry.objects, function (object) {
				if (!filter || filter(object)) {
					entries.push(object);
				}
			});
			_.each(prefixTreeEntry.childs, function (childTreeEntry) {
				findEntries(childTreeEntry, "", filter, entries);
			});
		} else {
			var char = prefix.charAt(0);
			var remainder = prefix.substring(1);
			findEntries(prefixTreeEntry.childs[char], remainder, filter, entries);
		}

	};




	return PrefixTree;


});
