/**
 * 
 * ########################################################################
 * #                                                                      #
 * #  A new way to load the css in WebCenter. (for re-usable components)  #
 * #                                                                      #
 * ########################################################################
 * 
 * ## Usage: 
 * define(["utils/webcss!./NameOfCssDocument.css"], function(){
 * 		// Your module which uses the css here.
 * });
 * 
 * OR (with less files)
 * 
 * define(["utils/webcss!./NameOfLessDocument.less"], function(){
 * 		// Your module which uses the less (css) here.
 * });
 * 
 * ## TECHNICAL DETAILS:
 * 
 * What this module does?
 *  - It loads the content of the css file using AMD
 *  - It makes all paths to images in the css relative to the page...
 *  - It puts the css file in the header of the document (inline)
 * 
 * Note: less is compiled by webcss-text... This module does not know anything about less...
 * 
 * An alternative would be:
 *  - To just add a &lt;link rel="stylesheet" src="sourceOfStylesheet"&gt; in the header of the document.
 * 
 * Why don't we use the alternative way? 
 *  [+] When optimizing wcr-static, the css is now bundled in the javascript component.
 *      (No need to do a separate call to load the css)
 *  [+] The css is available in the document before the component is executed
 *  [-] Paths in the css are relative to the stylesheet. 
 *      We wouldn't have a problem with that if we used the alternative way.
 *      However, if the css is put inline in the page, the browser does not know where the file came from...
 *      That is why this module needs to convert the urls. (make all paths to images in the css relative to the page...)
 * 
 * ## KNOWN LIMITATIONS:
 *  - does not support @ include in css (yet)
 *    If you need this, use less instead... (in less files, @ include is supported)
 * 
 * ## OPTIMIZATION NOTES:
 *  - if you don't want to bundle a certain css in the javascript, exclude "lib/webcss-text!<name of css>" when building.
 *    DO NOT use this for .less files (only .css). For .less files, when they are not bundled, they will be compiled in the browser!
 * 
 * @author JEPE
 */
define(["lib/dom"], function(dom){
	var loader = {};
	
	/**
	 * Instead of a path relative to the css file, return an path relative to the document we are working in 
	 */
	function modifyPath(req, name, path){
		if (path.indexOf("/") === 0 || path.indexOf("data:") === 0) {
			// absolute path... no problem
			return path;
		}
		// Split the path to the image
		var pathPieces = path.split("/");
		
		// Split the URL to the css file
		var urlToCssFile = req.toUrl(name);
		var urlToCssFilePieces = urlToCssFile.split("/");
		// remove the file name from the path
		urlToCssFilePieces.splice(-1,1);
		
		// Join both paths...
		var newPath = [].concat(urlToCssFilePieces).concat(pathPieces);
		
		return newPath.join("/");
	}

	function makeCssLocal (req, name, content){
		// Make all paths in css local to page instead of stylesheet...
		// 1) Find all url(" ... ") things in the css.
		var urlPattern = /url\(('?"?)([^'")]*)('?"?)\)/g;
		newContent = content.replace(urlPattern, function(a, b, c, d){
			// 2) Modify the path to be local to the page
			return "url("+b+modifyPath(req, name, c) + d +")";
		});
		
		return newContent;
	}

	/**
	 * Implements a custom require loaded...
	 * 
	 * Allow using "utils/webcss!./nameOfCss.css"
	 * 
	 * See http://requirejs.org/docs/plugins.html for information about implementing custom loaders...
	 * @param name
	 * @param req
	 * @param onload
	 * @param config
	 */
	loader.load = function (name, req, onload, config) {
		// Load the css as text...
		req(["utils/webcss-text!"+name], function(cssContent){
			// Do not do further steps in build mode... 
			if(config.isBuild) {
				onload();
				return;
			}
			
			// Make the css local
			var newCssContent = makeCssLocal(req, name, cssContent);
			
			// Import the css (would crash if called in build mode)
			dom.importCssString(newCssContent);
			
			// Return something...
			onload({});
		});
	};

	return loader;
});