/**
 * WebCenter Language Strings Loader for use in require.
 * 
 * Note: See wcr-component on how to use this component!
 * 
 * @author JEPE & KADA
 */
define([ "utils/I18n", "jquery" ], function(I18n, $) {
	"use strict";
	
	/**
	 * For debugging purposes...
	 */
	function debuglog(msg){
		//console.log(msg);
	}
	
	var WebI18n = {};
	
	/**
	 * Public, could be used externally...
	 * 
	 * Makes the wrapper around a map of language strings...
	 * 
	 * Has useful methods like t, tFallback, replacing messages, ...
	 * 
	 * Used internally when using require("utils/webi18n!...") to load language strings.
	 * 
	 * @param langmap an Object which maps key on language string
	 */
	WebI18n.makeBundle = function(langmap) {
		/**
		 * A bundle loaded with utils/webi18n!&lt;bundlename&gt; has three methods 
		 */
		var bundle = {};
	
		/**
		 * t => get the translation of a key
		 * 
		 * Also possible to use parameterization!
		 * 
		 * Returns the key if the value does not exist
		 * 
		 * @param key the key to lookup
		 * @returns the translation
		 */
		bundle.t = function(key) {
			var args = [key, key].concat(Array.prototype.slice.call(arguments).slice(1));
			return bundle.tFallback.apply(bundle, args);
		};
		
		/**
		 * tFallback => get the translation of a key or use some other value if the key does not exist...
		 * 
		 * @param key the key to lookup
		 * @param defaultValue the value to use if the key does not exist
		 * @returns the translation
		 */
		bundle.tFallback = function(key, defaultValue) {
			var value = langmap[key];
			if (value !== undefined){			
				// replace all given options
				for (var i = 2; i < arguments.length; i++){
					value = value.replace("{"+(i-2)+"}", arguments[i]);
				}
				return value;
			} else {
				return defaultValue;
			}
		};
		
		/**
		 * asMap => get the values as an object...
		 * 
		 * @returns an object with key-value pairs...
		 */
		bundle.asMap = function() {
			return langmap;
		};
		
		return bundle;
	};
	
	/**
	 * @private
	 * Internal -- loading the language bundle.
	 * 
	 * Note: might try twice if you are not logged in! 
	 * (uses webloggedin! to log in in such a case)
	 * 
	 * @param name
	 * @param callback
	 * @param errorcallback
	 * @param isretry
	 */
	WebI18n._doActualLoad = function(name, callback, errorcallback, isretry){
		debuglog("Loading bundle... \""+name+"\"");
		
		// Load the bundle...
		window.webI18n.loadSingleBundle(name, function(objectbundle) {
			// Success!
			debuglog("Bundle loaded... \""+name+"\"");
			callback(objectbundle);
		}, function(errorcode){
			// Failed...
			debuglog("Bundle failed to load... \""+name+"\" (errorcode: \""+errorcode+"\")");
			errorcallback();
		});
	};
	
	
	/**
	 * Allow using utils/webi18n!<bundlename>
	 * 
	 * To understand this method, see RequireJS documentation on how to create custom loaders...
	 */
	WebI18n.load = function (name, req, onload, config) {
		if (config.isBuild) {
			// do not trace dependencies in this loader...
			onload();
		} else {
			
			// Check if no i18n instance exists yet ...
			// i18n is in global scope since we only want to have one to make use of it's caching abilities.
			if (!window.webI18n) {
				window.webI18n = new I18n(window.webcenterConfig.userLang, window.webcenterConfig.jspBaseUrl, window.webcenterConfig.languageLoadTimestamp);
			}
			
			WebI18n._doActualLoad(name, function(objectbundle){
				// success...
				var bundle = WebI18n.makeBundle(objectbundle);
				onload(bundle);
			}, function(){
				// Something bad happened... (or the session of the user expired)
				// We could do: onload.error(); to fail the loading of the require module using webi18n.
				// But let's just return no language strings instead, that way at least everything still works 
				// (but you will see the language keys instead of the translations) 
				onload(WebI18n.makeBundle({}));
			});
		}
	};
	
	return WebI18n;
});