define(["web-viewer/data/AbstractDocumentVersion",
		"web-viewer/data/AbstractDocumentPage",
		"web-viewer/approval/ApprovalConstants",
		"web-viewer/utils/ViewerUtils",
		"utils/XMLUtils",
		"underscore"
	],
	function (AbstractDocumentVersion,
		AbstractDocumentPage,
		ApprovalConstants,
		ViewerUtils,
		XMLUtils,
		_) {

		/**
		 * Inject approval functionality in the data model
		 * The AbstractDocumentPage prototype gets extended with methods for retrieving and storing approval information
		 * The approval module only supports the WebCenter data adapter (in the future, this implementation here can be changed to work together with other data adapters to)
		 *
		 * @exports ApprovalDataExtensions
		 * @memberof ApprovalModule
		 * @param {Core.ViewerFacade} f
		 */
		var ApprovalDataExtensions = function (f) {
			var dataAdapter = f.getDataAdapter();

			// only the WebCenter data-adapter is supported
			if (!dataAdapter.WebCenter) {
				f.emit("error.SEVERE", "Can't initialize Approval on a non-webcenter data-adapter");
				return;
			}

			//---------------- AbstractDocumentVersion extensions -------------------------------------------------------------

			/**
			 * Initialize the approval data extensions on the document
			 * @private
			 */
			AbstractDocumentVersion.prototype._initApproval = function () {
				if (!this._approval) {
					this._approval = {
						status: "initialized",
						appObject: undefined
					};
				}
			};

			/**
			 * Request the approval setup if needed, execute callback when the information is available
			 * @param {Function} callback - first param true if success
			 */
			AbstractDocumentVersion.prototype.requestApprovalSetup = function (callback) {
				this._initApproval();

				if (this._approval.status === "available" || this._approval.status === "notAvailable") {
					if (callback) {
						callback(true);
					}
				} else {
					// setup is not yet available, register callback
					if (callback) {
						this.once("approval.loaded", callback);
					}
					// boot the complex approval
					this._bootComplexApproval();
				}
			};


			/**
			 * Initialize complex approval step 1
			 *
			 * Some base information on the approval setup needs to be loaded
			 * the call loads extra information on the user, needed for the complex approval
			 */
			AbstractDocumentVersion.prototype._bootComplexApproval = function () {

				if (this._approval.status == "initialized") {
					f.emit("loading.start", "bootComplexApproval");
					// start loading
					this._approval.status = "loading";
					// first get boot info for approval
					var bootInfoURL = dataAdapter.WebCenter("complexapproval.jsp", {
						'action': "GetBootInfo",
						'projectID': this.getDocument().get('projectid'),
						'docVerID': this.get('docversionid')
					});

					f.ajax(bootInfoURL, {
						success: _.bind(function (data) {
							data = XMLUtils.xml2obj(data);
							if (data.error) {
								// error, log error and notify listeners
								this._complexApprovalInitialized(false);

								f.i18n.loadBundle("viewer", function () {
									f.emit("error.SEVERE", f.i18n.t("ApprovalLoadingError"), data.error);
								});
							} else if (data.complexApprovalBootInfo.isLinkedDocument == "1") {
								// linked document, still try to get the approval state of the document
								this._approval.isLinked = true;
								this._loadApprovalState();
							} else {
								// start loading the complex approval document
								this._loadComplexApproval(data);
							}
						}, this),
						error: _.bind(function (jqXHR, textStatus, errorThrown) {
							// error, log error and notify listeners
							this._complexApprovalInitialized(false);

							f.i18n.loadBundle("viewer", function () {
								f.emit("error.SEVERE", f.i18n.t("ApprovalLoadingError"), jqXHR, textStatus, errorThrown);
							});
						}, this)
					});


				}
			};

			/**
			 * Load the approval state for a document using the getapprovalstate.jsp call
			 */
			AbstractDocumentVersion.prototype._loadApprovalState = function () {
				//getapprovalstate.jsp?projectid=00002_0000136693&docid=00002_0000001174&docversionid=00002_0000002663
				var readURL = dataAdapter.WebCenter("getapprovalstate.jsp", {
					projectid: this.getDocument().get("projectid"),
					docid: this.getDocument().get('docid'),
					docversionid: this.get('docversionid')
				});

				f.ajax(readURL, {
					success: _.bind(function (data) {
						data = XMLUtils.xml2obj(data);
						if (data && data.status && data.status.approval) {
							this._approval.approvalState = ApprovalConstants.ViewerStateToComplexApprovalState[data.status.approval.approvalstate];
						}
						this._complexApprovalInitialized(false);
					}, this),
					error: _.bind(function (jqXHR, textStatus, errorThrown) {
						this._complexApprovalInitialized(false);
					}, this)
				});
			};


			/**
			 * Check if
			 * @returns
			 */
			AbstractDocumentVersion.prototype.approvalIsLinkedDocument = function () {
				if (this._approval) {
					return this._approval.isLinked;
				} else {
					return false;
				}
			};

			/**
			 * Initialize complex approval step 2
			 *
			 * Load the complex approval object
			 */
			AbstractDocumentVersion.prototype._loadComplexApproval = function (data) {
				var readURL = dataAdapter.WebCenter("complexapproval.jsp", {
					'projectID': this.getDocument().get('projectid'),
					'folderID': '',
					'docID': this.getDocument().get('docid'),
					'returnURL': 'docdetailsapprovers.jsp',
					'docVerID': this.get('docversionid'),
					'menu_file': '',
					'urlwanted': 'docdetailsapprovers.jsp'
				});
				var bootInfo = data.complexApprovalBootInfo;
				var userObject = {
					userId: bootInfo.userId,
					userName: bootInfo.userName,
					lastName: bootInfo.lastName,
					firstName: bootInfo.firstName,
					isProjectManager: bootInfo.isProjectManager == '1',
					isAdmin: bootInfo.isAdmin == '1',
					canForceApprove: bootInfo.canForceApprove == '1',
					canForceReject: bootInfo.canForceReject == '1',
					canChangeSetup: bootInfo.canChangeSetup == '1',
					canStartApproval: bootInfo.canStartApproval == '1',
					canSeeOthers: bootInfo.canSeeOthers == '1'
				};
				var readProjectFolderCAPP = readURL + "&action=Inherit";
				var actionURL = readURL;

				var stagedApprovalAllowed = bootInfo.stagedApprovalAllowed == '1';
				var languageCode = bootInfo.languageCode;

				var nextversion = false;
				var isViewable = bootInfo.isViewable == '1';

				if (!this._approval.appObjDiv) {
					var id = ViewerUtils.generateUniqueDOMId(this.get('docversionid') + "approvalAppObj");
					var div = $("<div/>").attr("id", id).hide();
					this._approval.appObjDiv = id;
					f.getContext().append(div);
				}

				if (!this._approval.appPanelDiv) {
					var id = ViewerUtils.generateUniqueDOMId(this.get('docversionid') + "approvalAppPanel");
					var div = $("<div/>").attr("id", id).hide();
					this._approval.appPanelDiv = id;
					f.getContext().append(div);
				}

				// create div for dropdownmenu if non existant
				// dropmenudiv is used by complex approval to show the approval dialogs after clicking a button
				if ($("#dropmenudiv").length === 0) {
					$(f.getContext()).append($('<div onmouseover="clearhidemenu()" onmouseout="dynamichide(event)"/>').attr("id", "dropmenudiv").css("visibility", "hidden").addClass("approvalActionDropdown wcr"));
				} else {
					// the dropmenudiv exists already (added by hover.js), move it into the viewer context and add css class for styling
					$("#dropmenudiv").addClass("approvalActionDropdown wcr").appendTo(f.getContext());
				}

				// require needed dependencies
				var wcrJSPath = dataAdapter.getPathToJSFiles();
				require([wcrJSPath + "detailswin.js",
						wcrJSPath + "hover.js",
						wcrJSPath + "date.js",
						wcrJSPath + "dateShift.js",
						wcrJSPath + "datetools.js",
						wcrJSPath + "webcenter_ajax.js",
						wcrJSPath + "webcenter_xml.js",
						wcrJSPath + "webcenter_html.js",
						wcrJSPath + "ComplexApproval.js"
					],
					_.bind(function () {
						f.i18n.loadBundle("approval", _.bind(function (languageKeys) {
							// we are sure that all dependencies are loaded, initialize complex approval
							this._approval.appObject = new complexApproval(this._approval.appObjDiv,
								this._approval.appPanelDiv,
								readURL,
								actionURL,
								readProjectFolderCAPP,
								userObject,
								languageKeys,
								'xmlcontent',
								nextversion,
								stagedApprovalAllowed,
								languageCode,
								isViewable,
								true,
								_.bind(function () {
									this._complexApprovalInitialized(true);
								}, this),
								_.bind(this._approvalUpdated, this),
								dataAdapter.getPathToWCRImages());
						}, this));
					}, this)
				);
			};

			/**
			 * Action to be done when approval is updated
			 */
			AbstractDocumentVersion.prototype._approvalUpdated = function (success) {
				// if document is a pagelist, invalidate all page approval info
				// we also want to get the global approval statuses again
				if (this.isPageList() && this._approval.appObject && this._approval.appObject.getApprovalStatus() != "NotDefined") {
					// invalidate all pages
					var pages = this.getAllPages();
					_.each(pages, function (page) {
						page.invalidatePageApproval();
					});

					// reload global approval for document by reading it from the server again
					// we need to do this, because we don't know which pages were approved (some may have failed) and we don't want to load the approval for each page
					// 
					// reloading flag is used, because update event is called whenever the data is loaded 
					// The flag will prevent infinite loops and make sure the "update" event is only propagated when the approval is fully loaded
					if (!this._approval.reloading) {
						this._approval.reloading = true;
						this._approval.appObject.readCAppFromXML(this._approval.appObject.readURL);
					} else {
						this._approval.reloading = false;
						this._emit("approval.update", true);
					}
				} else {
					// emit an update
					this._emit("approval.update", true);
				}
			};

			/**
			 * Initialize complex approval step 3
			 *
			 * The complexApproval object now is correctly initialized (or initialization failed), notify listeners!
			 * @param {Boolean} success - true if complex approval was correctly initialized
			 */
			AbstractDocumentVersion.prototype._complexApprovalInitialized = function (success) {
				f.emit("loading.stop", "bootComplexApproval");

				this._approval.status = success ? "available" : "notAvailable";
				this._emit("approval.loaded", success);
			};


			/**
			 * Get the ID of the div in which the Complex Approval object is stored
			 * @returns {String} ID of the div in which the Complex Approval object is stored
			 */
			AbstractDocumentVersion.prototype.getApprovalObjectDivId = function () {
				this._initApproval();
				return this._approval.appObjDiv;
			};

			/**
			 * Get the ID of the div in which the Complex Approval could draw itself (AppPanel div)
			 * @returns {String} ID of the div in which the Complex Approval could draw itself (AppPanel div)
			 */
			AbstractDocumentVersion.prototype.getApprovalPanelDivId = function () {
				this._initApproval();
				return this._approval.appPanelDiv;
			};

			/**
			 * Get the details data structure of the approval
			 * @returns {Object}
			 */
			AbstractDocumentVersion.prototype.getApprovalDetails = function () {
				return this._approval.appObject.getApprovalDetailsData();
			};

			/**
			 * Check if an approval setup was defined
			 * @returns {Boolean}
			 */
			AbstractDocumentVersion.prototype.hasApprovalSetup = function () {
				return this._approval && this._approval.appObject && this._approval.appObject.getApprovalStatus() != "NotDefined";
			};

			/**
			 * Check if the approval setup is being loaded
			 * @returns {Boolean}
			 */
			AbstractDocumentVersion.prototype.isApprovalLoading = function () {
				return this._approval && this._approval.status === "loading";
			};


			/**
			 * Show the mouseover message when hovering over a specific comment
			 * @param {HTMLObject} target - the target over which was hovered
			 * @param {Event} event - the mouseover event
			 * @param {Number} step - internal number of the step which is hovered
			 * @param {Number} approver - internal number of the approver which is hovered
			 */
			AbstractDocumentVersion.prototype.showApprovalCommentsHover = function (target, event, step, approver) {
				this._approval.appObject.showCommentHover(target, event, step, approver);
			};


			/**
			 * Check if the currently logged in user is an approval master
			 * if the info is not yet available, undefined will be returned (if the info is available callback is called)
			 * @param {Function} [callback] - function to be called when the info becomes available on a later stage (is not called when info is available already)
			 * @returns {Boolean} (or undefined when the info is not available yet)
			 */
			AbstractDocumentVersion.prototype.isUserApprovalMaster = function (callback) {
				this._initApproval();

				if (this._approval.isCurrentUserApprovalMaster === undefined) {
					if (callback) {
						this.once("approval.approvalMasterInfo", callback);
					}

					// we didn't fetch this information yet, fetch it
					if (!this._approval.isApprovalMasterInfoBeingFetched) {
						this._approval.isApprovalMasterInfoBeingFetched = true;

						// Generate URL to get te approval state (assuming we have a webcenter document)
						var url = dataAdapter.WebCenter("getapprovalstate.jsp", {
							'projectid': this.getDocument().getProjectID(),
							'docid': this.getDocument().getDocumentID(),
							'docversionid': this.getDocumentVersionID()
						});

						f.ajax(url, {
							success: _.bind(function (data) {
								if (data) {
									data = XMLUtils.xml2obj(data);

									if (data.status.value == "1") {
										// there is an error
										f.i18n.loadBundle("viewer", function () {
											f.emit("error.SEVERE", f.i18n.t("ApprovalLoadingError"), data.status.message);
										});
									} else {
										this._approval.isCurrentUserApprovalMaster = (data.status.approval.approvalmaster == "1");
									}

									this._approval.isApprovalMasterInfoBeingFetched = false;
								} else {
									// there is an error
									f.i18n.loadBundle("viewer", function () {
										f.emit("error.SEVERE", f.i18n.t("ApprovalLoadingError"), "Data was not set");
									});
								}

								this._emit("approval.approvalMasterInfo");
							}, this),
							error: _.bind(function (jqXHR, textStatus, errorThrown) {
								this._approval.isApprovalMasterInfoBeingFetched = false;

								f.i18n.loadBundle("viewer", function () {
									f.emit("error.SEVERE", f.i18n.t("ApprovalLoadingError"), jqXHR, textStatus, errorThrown);
								});

								this._emit("approval.approvalMasterInfo");
							}, this)
						});
					}

				}


				// return the current value
				return this._approval.isCurrentUserApprovalMaster;
			};


			//---------------- AbstractDocumentPage extensions -------------------------------------------------------------
			/**
			 * Check if an approval setup was defined
			 * @returns {Boolean}
			 */
			AbstractDocumentPage.prototype.hasApprovalSetup = function () {
				return this.getDocumentVersion().hasApprovalSetup();
			};

			/**
			 * Get a list of possible approval actions
			 * @returns {Array.<Object>}
			 */
			AbstractDocumentPage.prototype.getApprovalActions = function () {
				var actions = {
					list: [],
					text: '',
					'showContent': false
				};

				if (this.getDocumentVersion()._approval && this.getDocumentVersion()._approval.status === "available") {
					actions = this.getDocumentVersion()._approval.appObject.getApprovalActions(this.getDocumentVersion().isPageList());
				}

				return actions;
			};


			/**
			 * Get the approval Object for this page
			 * @returns {Object} (or undefined if the approval object is not defined)
			 */
			AbstractDocumentPage.prototype.getApprovalObject = function () {
				var docVersion = this.getDocumentVersion();
				if (docVersion._approval && docVersion._approval.appObject && docVersion._approval.status === "available") {
					return docVersion._approval.appObject;
				} else {
					return undefined;
				}
			};



			/**
			 * Get the current approval status
			 * @returns {String}
			 */
			AbstractDocumentPage.prototype.getApprovalStatus = function () {
				var docVersion = this.getDocumentVersion();
				if (docVersion._approval && docVersion._approval.appObject) {
					return docVersion._approval.appObject.getApprovalStatus();
				} else if (docVersion._approval && docVersion._approval.approvalState) {
					return docVersion._approval.approvalState;
				} else {
					return "NotDefined";
				}
			};

			/**
			 * Check if there is an ongoing approval cycle on the document
			 * @returns {Boolean}
			 */
			AbstractDocumentPage.prototype.isUnderApproval = function () {
				return this.isApprovalStarted() && !(this.isApproved() || this.isRejected());
			};

			/**
			 * Check if the page has been approved (this could be forced or conditional)
			 * @returns {Boolean}
			 */
			AbstractDocumentPage.prototype.isApproved = function () {
				var status = this.getApprovalStatus();
				return status === "Approved" || status === "ForceApproved" || status === "ForceApprovedConditional" || status === "ApprovedConditional";
			};

			/**
			 * Check if the page has been rejected (this could be forced)
			 * @returns {Boolean}
			 */
			AbstractDocumentPage.prototype.isRejected = function () {
				var status = this.getApprovalStatus();

				return status === "Rejected" || status === "ForceRejected";
			};

			/**
			 * Check if the current user is an approver
			 * @returns {Boolean}
			 */
			AbstractDocumentPage.prototype.isUserApprover = function () {
				var approval = this.getApprovalObject();
				if (approval) {
					return approval.isUserApprover();
				}

				return false;
			};

			/**
			 * Check if the current user is an approval master
			 *
			 * @param {Function} callback - if the info is not availble yet, the info is requested and callback is called
			 * @returns {Boolean} or undefined if info is not available
			 */
			AbstractDocumentPage.prototype.isUserApprovalMaster = function (callback) {
				var docVersion = this.getDocumentVersion();

				return docVersion.isUserApprovalMaster(callback);
			};

			/**
			 * Check if the current user is in the pending approval cycle
			 * @returns {Boolean}
			 */
			AbstractDocumentPage.prototype.isUserPendingApprover = function () {
				var approval = this.getApprovalObject();

				if (approval) {
					return approval.isUserPendingApprover();
				}
			};

			/**
			 * Check if the approval has started
			 * @returns {Boolean}
			 */
			AbstractDocumentPage.prototype.isApprovalStarted = function () {
				var approval = this.getApprovalObject();

				if (approval) {
					return approval.isApprovalStarted();
				}

				return false;
			};


			/**
			 * Request the approval setup for the page, and wait for it before the callback is called.
			 *
			 * If approval setup is available, callback is called immediately
			 * @param {Function} callback
			 */
			AbstractDocumentPage.prototype.whenApprovalSetupAvailable = function (callback) {
				var docVersion = this.getDocumentVersion();
				docVersion.requestApprovalSetup(callback);
			};

			/**
			 * Initialize the approval data extensions on the document page
			 * @private
			 */
			AbstractDocumentPage.prototype._initPageApproval = function () {
				if (!this._approval) {
					this._approval = {
						status: "initialized",
						dataObj: undefined
					};
				}
			};

			/**
			 * Invalidate the approval data for the document page
			 */
			AbstractDocumentPage.prototype.invalidatePageApproval = function () {
				if (this._approval) {
					this._approval.status = "initialized";
					this._approval.dataObj = undefined;
				}
			};

			/**
			 * Tell whether or not the approval is fully loaded
			 * @private
			 */
			AbstractDocumentPage.prototype.isApprovalLoaded = function () {
				if (this._approval && (this._approval.status === "available" || this._approval.status === "notAvailable")) {
					return true;
				}
				return false;
			};

			/**
			 * Set Page approval initialization status
			 * @param {Boolean} [success] - Define whether or not approval is available for the page
			 * @private
			 */
			AbstractDocumentPage.prototype._pageApprovalInitialized = function (success) {
				this._approval.status = success ? "available" : "notAvailable";
				this._emit("approval.loaded", success);
			};

			/**
			 * Request the approval status of the page. This will overwrite the approval object with info of this page.
			 * If the info is not available yet, it will be fetched.
			 * @param {Function} [callback] - function to be executed when approval object is updated
			 */
			AbstractDocumentPage.prototype.requestPageApprovalSetup = function (callback) {
				this._initPageApproval();

				// handle callback
				if (this._approval.status === "notAvailable") {
					if (callback) {
						callback();
					}
				} else if (this._approval.status === "available") {
					this._overwritePageApprovalSetup(callback);
				} else {
					this.once("approval.loaded", callback);
				}

				// get data if needed
				if (this._approval.status === "initialized" || this._approval.status === "outdated") {
					this._approval.status = 'loading';
					var historyUrl = dataAdapter.WebCenter("GetDocumentHistory.jsp", {
						'projectid': this.getDocument().get('projectid'),
						'docid': this.getDocument().get('docid'),
						'docversionid': this.getDocumentVersion().get("docversionid"),
						'approvalinfo': 1,
						'annotationinfo': 0,
						'fulluserinfo': 0,
						'pageno': this.getPagenum(),
						'pagename': this.get("pagename") ? this.get("pagename") : this.getPagenum(),
						'pageindex': this.get("pageindex") ? this.get("pageindex") : this.getPagenum() - 1
					});

					// get object from doc history via Ajax call
					f.ajax(historyUrl, {
						success: _.bind(function (data) {
							data = XMLUtils.xml2obj(data);
							if (data.error) {
								// error, log error and notify listeners
								this._pageApprovalInitialized(false);

								f.i18n.loadBundle("viewer", function () {
									f.emit("error.SEVERE", f.i18n.t("ApprovalLoadingError"), data.error);
								});
							} else {
								// update approval object
								this._approval.dataObj = data;
								this._overwritePageApprovalSetup(callback);
							}
						}, this),
						error: _.bind(function (jqXHR, textStatus, errorThrown) {
							// error, log error and notify listeners
							this._pageApprovalInitialized(false);

							f.i18n.loadBundle("viewer", function () {
								f.emit("error.SEVERE", f.i18n.t("ApprovalLoadingError"), jqXHR, textStatus, errorThrown);
							});
						}, this)
					});
				}
			};

			/**
			 * This method will do the actual overwriting of the approval object with the data received from the document history ajax call.
			 * @param {Function} [callback] - function to be executed when approval object is updated
			 * @private
			 */
			AbstractDocumentPage.prototype._overwritePageApprovalSetup = function (callback) {
				this.whenApprovalSetupAvailable(_.bind(function () {
					var data = this._approval.dataObj;
					var appObject = this.getApprovalObject();
					// need to determine whether we have an array or an object first (because of xml2obj use)
					var docVersionInfo = null;
					var docVersionInfoArr = [];
					if (data.document_info && data.document_info.document_version) {
						if (_.isArray(data.document_info.document_version)) {
							docVersionInfoArr = data.document_info.document_version;
						} else {
							docVersionInfoArr = [data.document_info.document_version];
						}
					}
					// now select the correct version. If we don't find the correct version, take the highest one that is before the current version
					for (var i = 0; i < docVersionInfoArr.length; i++) {
						if (parseInt(docVersionInfoArr[i].version_number) === this.getDocumentVersion().getVersionId()) {
							docVersionInfo = docVersionInfoArr[i];
							break;
						} else if (parseInt(docVersionInfoArr[i].version_number) < this.getDocumentVersion().getVersionId()) {
							// temporarily store the highest version we have so far.
							// If we don't find a page with the current version, show approval for highest version before this version.
							docVersionInfo = docVersionInfoArr[i];
						}
					}

					if (appObject && docVersionInfo && appObject.getApprovalStatus() !== "NotDefined") {
						// apply status to the approval stage
						var appStatus = ApprovalConstants.ViewerStateToComplexApprovalState[docVersionInfo.document_approval_code];
						if (!appStatus) {
							if (appObject.getApprovalStatus() === "NotStarted") {
								appStatus = "NotStarted";
							} else {
								appStatus = "Pending";
							}
						}
						appObject.approvalStatus = appStatus;
						if (appObject.steps[0]) {
							// note, only one stage available for pagelists, so we'll always only have step[0]
							appObject.steps[0].approvalStatus = appStatus;
						}

						// now go deeper and look at the actions per approver
						if (docVersionInfo.stages && docVersionInfo.stages.stage) {
							var approvers = docVersionInfo.stages.stage.approver_list.approver;
							if (approvers && !_.isArray(approvers)) {
								// approvers is not an array, but a single object
								approvers = [approvers];
							}
							_.each(approvers, _.bind(function (submittedApproval) {
								var appStage = appObject.steps[0];
								if (appStage) {
									var approver = _.find(appStage.approvers, function (approver) {
										return approver.memberObj.userId === submittedApproval.userID || (submittedApproval.group && approver.memberObj.groupId === submittedApproval.group.groupID);
									});

									if (approver) {
										// match entries of document history with entires of approval object
										if (approver.memberObj.userId === submittedApproval.userID || (submittedApproval.group && approver.memberObj.groupId === submittedApproval.group.groupID)) {
											// need to determine whether we have an array or an object first (because of xml2obj use)
											var appAction = null;
											if (submittedApproval.approval_action) {
												if (_.isArray(submittedApproval.approval_action)) {
													// take the final action; if a document is approvad, approval will always be the final action, 
													// unless it's forced approved afterwards, in which case the force approval will be the last action
													appAction = submittedApproval.approval_action[submittedApproval.approval_action.length - 1];
												} else {
													appAction = submittedApproval.approval_action;
												}
											}

											// read approval info and overwrite complex approval object
											if (appAction) {
												var status = appAction.approval_status_ID;
												var date = null;
												var version = "";
												var approvedBy = "";
												var commentArr = [];

												if (status !== "00001_0000000002") {
													if (appAction.approval_date) {
														date = new Date(appAction.approval_date.replace(' ', 'T'));
													}
													version = appAction.doc_version;
													approvedBy = appAction.actual_user_approver ? appAction.actual_user_approver.userID : '';
												}

												// go over the comments of all approval actions
												// need to determine whether we have an array or an object for "approval_action" first (because of xml2obj use)
												var approvalActions = [];
												if (submittedApproval.approval_action && submittedApproval.approval_action.length === undefined) {
													approvalActions.push(submittedApproval.approval_action);
												} else if (submittedApproval.approval_action) {
													approvalActions = submittedApproval.approval_action;
												}
												// create comment objects
												_.each(approvalActions, function (action) {
													// create approval comment object
													if (action.approval_comment) {
														var userId = '';
														if (approver.memberObj.userId) {
															userId = approver.memberObj.userId;
														} else if (appAction.actual_user_approver) {
															userId = appAction.actual_user_approver.userID;
														}
														var commentDate = null;
														if (action.approval_comment.date) {
															commentDate = new Date(action.approval_comment.date.replace(' ', 'T'));
														}
														commentArr.push({
															commentText: action.approval_comment.comment,
															commentTime: commentDate,
															commentByUserId: userId,
															commentVersion: action.doc_version
														});
													}
												});

												this._overwriteApproverEntry(approver, status, date, version, approvedBy, commentArr);
											} else {
												// set as pending without extra info
												this._overwriteApproverEntry(approver);
											}
										}
									}
								}
							}, this));
						}

						// update current page model
						this.set("document_approval_code", ApprovalConstants.ComplexApprovalStateToViewerState[appObject.getApprovalStatus()]);
						this.set("user_approval_code", ApprovalConstants.ComplexApprovalStateToViewerState[appObject.currentUserApprovalStatus(0)]);

						// Execute the initial callback, which was passed along
						if (callback) {
							callback();
						}

						this._pageApprovalInitialized(true);
					} else {
						this._pageApprovalInitialized(false);
					}
				}, this));
			};

			/**
			 * Overwrites one specific approver entry of approval.
			 * @param {Object} [approver] - approver to overwrite
			 * @param {String} [status] - status of entry
			 * @param {Date} [date] - date of approval
			 * @param {String} [version] - version in which entry was approved
			 * @param {String} [approvedBy] - ID of user that did approval for group
			 * @param {Array} [comments] - Array of submitted comments
			 * @private
			 */
			AbstractDocumentPage.prototype._overwriteApproverEntry = function (approver, status, date, version, approvedBy, comments) {
				if (approver) {
					if (status === undefined) {
						approver.approvalStatus = ApprovalConstants.ViewerStateToComplexApprovalState["00001_0000000002"];
					} else {
						status = ApprovalConstants.ViewerStateToComplexApprovalState[status];
						if (!status) {
							status = ApprovalConstants.ViewerStateToComplexApprovalState["00001_0000000002"];
						}
						approver.approvalStatus = status;
					}

					if (date === undefined) {
						approver.approvalTime = null;
					} else {
						approver.approvalTime = date;
					}

					if (version === undefined) {
						approver.approvedVersion = "";
					} else {
						approver.approvedVersion = version;
					}

					if (approvedBy === undefined) {
						approver.approvedByUserId = "";
					} else {
						approver.approvedByUserId = approvedBy;
					}

					if (comments === undefined) {
						approver.commentArr = [];
					} else {
						approver.commentArr = comments;
					}
				}

			};

		};

		return ApprovalDataExtensions;
	});