define(["web-viewer/layout/AbstractWidget",
		"Template",
		"tpl!web-viewer/approval/ApprovalDetailsWidget",
		"component/tooltip/Tooltip",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/compare/CompareConstants",
		"web-viewer/utils/ListenerUtil",
		"underscore"
	],
	function (AbstractWidget,
		Template,
		tpl,
		Tooltip,
		LayoutConstants,
		CompareConst,
		ListenerUtil,
		_) {
		"use strict";
		/**
		 * Details table widget of complex approval
		 *
		 * @exports ApprovalDetailsWidget
		 * @memberof ApprovalModule
		 * @constructor
		 * @mixes {Utils.ListenerUtilMixin}
		 *
		 *
		 * @param {Core.ViewerFacade} f
		 */
		var ApprovalDetailsWidget = function (f) {
			AbstractWidget.call(this, f, LayoutConstants.Widget.APPROVAL);
			ListenerUtil.mixin(this, f);

			// set widget title
			f.i18n.loadBundle("viewer", _.bind(function () {
				this.setTitle(f.i18n.t("ApprovalWidgetTitle"));
			}, this));

			/**
			 * currently loaded doc
			 * @private
			 */
			this._currentDocVersion = null;


			// The template used for rendering the widget
			this.t = new Template(f.i18n, tpl);
			this.showInfo = true;

			// when the user hovers over a comment, show the mouseover of the complex approval
			this.t.bind("mouseover", ".comments", function (source, event) {
				if (this._currentDocVersion && this._currentDocVersion.hasApprovalSetup()) {
					var data = ($(source).attr("data-bind") || "").split("|");
					var step = data[0];
					var approver = data[1];

					this._currentDocVersion.showApprovalCommentsHover(source, event, step, approver);
				}
			}, this);
			
			this.setIcon("icon icon-WCR_Approve-round")

			this.t.bind("mouseout", ".comments", function (source, event) {
				delayhidemenu(); // note this a global function introduced by hover.js which is included by approvalmodule (hooray!)
			});

			// add a tool tip for when the step status is hovered
			this.whenContextAvailable(function () {
				this.tooltip = new Tooltip(this.f.i18n, this.context, this.f.getContext(), this.f.getContext(), this.f.isMobile(), "");
				this.tooltip.disableTooltip();
				// filter on all elements with attribute tooltip
				this.t.bind("mouseover", "[tooltip]", function (source, event) {
					this.tooltip.setTooltipContent($(source).attr("tooltip"));
					this.tooltip.context = $(source);
					this.tooltip.show();
				}, this);
				// filter on all elements with attribute tooltip
				this.t.bind("mouseout", "[tooltip]", function (source, event) {
					this.tooltip.hide();
				}, this);
			}, this);




			// react on compare mode changes
			this._registerListener("module.compare.changed.mode", this.onCompareModeChanged, this);
			// make sure we update the toolbar component when compare becomes available
			this.f.whenModuleAvailable("compare", function (compareModule) {
				this.onCompareModeChanged(null, compareModule.getActiveMode());
			}, this);

			// react when the active document has changed
			this.f.getDocumentController().whenActiveDocumentVersionChanged(this.selectDocument, this);

			// if the new document is a page list, listen to page change events
			if (this._currentDocVersion.isPageList()) {
				var updateListener = function (oldPage, newPage) {
					if (oldPage) {
						// remove listeners
						this._unregisterAllListenersForObject(oldPage);

						//also remove listener from active page when in difference mode and vice versa
						if (this.showDifferenceInfo) {
							this._unregisterAllListenersForObject(this.f.getActivePage());
						} else {
							this._unregisterAllListenersForObject(this.f.getDocumentController().getDifferencePage());
						}

					}

					if (newPage) {
						// register listeners, and request approval setup	
						this._registerListener("approval.loaded", this.render, this, newPage);
						newPage.requestPageApprovalSetup(_.bind(this.render, this));
					}
				};
				this.f.getDocumentController().whenDifferencePageChanged(updateListener, this);
				this.f.getDocumentController().whenActivePageChanged(updateListener, this);
			}

			this.selectDocument();
		};

		// inherit from approval details widget
		ApprovalDetailsWidget.prototype = Object.create(AbstractWidget.prototype);
		ApprovalDetailsWidget.prototype.constructor = ApprovalDetailsWidget;
		ApprovalDetailsWidget.widgetName = LayoutConstants.Widget.APPROVAL;

		/**
		 * Set the context of the widget
		 */
		ApprovalDetailsWidget.prototype.setContext = function (context, detailsExtra) {
			AbstractWidget.prototype.setContext.call(this, context, detailsExtra);
			var loader = $("<span/>").text("Loading");
			this.context.append(loader);

			this.f.i18n.loadBundle("viewer", function () {
				if (loader) {
					loader.text(this.f.i18n.t("Loading"));
				}
			}, this);

			this.context.addClass("enableTextSelection");

			this.t.setContext(context);
		};

		/**
		 * The active document has changed/ or compare mode, select the correct document
		 */
		ApprovalDetailsWidget.prototype.selectDocument = function () {
			var oldDocVersion = this._currentDocVersion;

			if (oldDocVersion) {
				this._unregisterAllListenersForObject(oldDocVersion);
			}

			if (this.showDifferenceInfo) {
				this._currentDocVersion = this.f.getDocumentController().getDifferencePage().getDocumentVersion();
			} else {
				this._currentDocVersion = this.f.getDocumentController().getActiveDocumentVersion();
			}


			if (this._currentDocVersion) {
				this._registerListener("approval.loaded", this.render, this, this._currentDocVersion);
				this._registerListener("approval.update", this.render, this, this._currentDocVersion);

				if (!this._currentDocVersion.isPageList()) {
					this._currentDocVersion.requestApprovalSetup(_.bind(this.render, this));
				}
			}
		};


		/**
		 * Compare mode has changed, change the information we show
		 * @param oldMode
		 * @param newMode
		 */
		ApprovalDetailsWidget.prototype.onCompareModeChanged = function (oldMode, newMode) {
			if (!newMode || newMode.getName() === CompareConst.mode.SHOW_CURRENT || (newMode.getName() === CompareConst.mode.READERSPREAD)) {
				this.showInfo = true;
				this.showDifferenceInfo = false;
				this.selectDocument();
			} else if (newMode && newMode.getName() === CompareConst.mode.SHOW_DIFFERENCE) {
				this.showInfo = true;
				this.showDifferenceInfo = true;
				this.selectDocument();
			} else {
				this.showInfo = false;
			}

			this.render();
		};

		/**
		 * Render the widget
		 */
		ApprovalDetailsWidget.prototype.render = function () {
			// render template, notify layout manager of possible layout change
			// we need to have the correct language information
			this.f.i18n.loadBundle(["viewer", "approval"], function () {
				if (this.showInfo) {
					// we can show approval info
					this.t.d.hasApprovalSetup = this._currentDocVersion && this._currentDocVersion.hasApprovalSetup();
					if (this.t.d.hasApprovalSetup) {
						// there is an approval setup, setup data for template
						var data = this._currentDocVersion.getApprovalDetails();
						this.t.d.steps = data.steps;
						this.t.d.isComplexApproval = data.isComplexApproval;

						_.each(this.t.d.steps, function (step) {
							step.tooltip = this.f.i18n.t('CApp' + step.status);
							step.translatedStatusName = this.f.i18n.t('CApp' + step.status);
							if (step.approvalTime) {
								step.tooltip += " " + this.f.i18n.t('AtTime') + " " + this.t.d.date(step.approvalTime);
							}
						}, this);
					} else {
						if (this._currentDocVersion && this._currentDocVersion.isApprovalLoading()) {
							this.t.d.message = this.f.i18n.t("Loading");
						} else if (this._currentDocVersion && this._currentDocVersion.approvalIsLinkedDocument()) {
							// for a linked document, we cannot display details
							this.t.d.message = this.f.i18n.t("approvalCannotDisplayDetailsForLinkedDocument");
						} else {
							// no approval setup defined
							this.t.d.message = this.f.i18n.t("CAppNotDefined");
						}
					}
				} else {
					// we cannot show approval info because some compare mode is active
					this.t.d.hasApprovalSetup = false;
					this.t.d.message = this.f.i18n.t("approvalHiddenBecauseCompare");
				}

				this.t.render();
				this._fireResized();
			}, this);
		};


		return ApprovalDetailsWidget;
	});