
define('web-viewer/approval/ApprovalDataExtensions',["web-viewer/data/AbstractDocumentVersion",
		"web-viewer/data/AbstractDocumentPage",
		"web-viewer/approval/ApprovalConstants",
		"web-viewer/utils/ViewerUtils",
		"utils/XMLUtils",
		"underscore"
	],
	function (AbstractDocumentVersion,
		AbstractDocumentPage,
		ApprovalConstants,
		ViewerUtils,
		XMLUtils,
		_) {

		/**
		 * Inject approval functionality in the data model
		 * The AbstractDocumentPage prototype gets extended with methods for retrieving and storing approval information
		 * The approval module only supports the WebCenter data adapter (in the future, this implementation here can be changed to work together with other data adapters to)
		 *
		 * @exports ApprovalDataExtensions
		 * @memberof ApprovalModule
		 * @param {Core.ViewerFacade} f
		 */
		var ApprovalDataExtensions = function (f) {
			var dataAdapter = f.getDataAdapter();

			// only the WebCenter data-adapter is supported
			if (!dataAdapter.WebCenter) {
				f.emit("error.SEVERE", "Can't initialize Approval on a non-webcenter data-adapter");
				return;
			}

			//---------------- AbstractDocumentVersion extensions -------------------------------------------------------------

			/**
			 * Initialize the approval data extensions on the document
			 * @private
			 */
			AbstractDocumentVersion.prototype._initApproval = function () {
				if (!this._approval) {
					this._approval = {
						status: "initialized",
						appObject: undefined
					};
				}
			};

			/**
			 * Request the approval setup if needed, execute callback when the information is available
			 * @param {Function} callback - first param true if success
			 */
			AbstractDocumentVersion.prototype.requestApprovalSetup = function (callback) {
				this._initApproval();

				if (this._approval.status === "available" || this._approval.status === "notAvailable") {
					if (callback) {
						callback(true);
					}
				} else {
					// setup is not yet available, register callback
					if (callback) {
						this.once("approval.loaded", callback);
					}
					// boot the complex approval
					this._bootComplexApproval();
				}
			};


			/**
			 * Initialize complex approval step 1
			 *
			 * Some base information on the approval setup needs to be loaded
			 * the call loads extra information on the user, needed for the complex approval
			 */
			AbstractDocumentVersion.prototype._bootComplexApproval = function () {

				if (this._approval.status == "initialized") {
					f.emit("loading.start", "bootComplexApproval");
					// start loading
					this._approval.status = "loading";
					// first get boot info for approval
					var bootInfoURL = dataAdapter.WebCenter("complexapproval.jsp", {
						'action': "GetBootInfo",
						'projectID': this.getDocument().get('projectid'),
						'docVerID': this.get('docversionid')
					});

					f.ajax(bootInfoURL, {
						success: _.bind(function (data) {
							data = XMLUtils.xml2obj(data);
							if (data.error) {
								// error, log error and notify listeners
								this._complexApprovalInitialized(false);

								f.i18n.loadBundle("viewer", function () {
									f.emit("error.SEVERE", f.i18n.t("ApprovalLoadingError"), data.error);
								});
							} else if (data.complexApprovalBootInfo.isLinkedDocument == "1") {
								// linked document, still try to get the approval state of the document
								this._approval.isLinked = true;
								this._loadApprovalState();
							} else {
								// start loading the complex approval document
								this._loadComplexApproval(data);
							}
						}, this),
						error: _.bind(function (jqXHR, textStatus, errorThrown) {
							// error, log error and notify listeners
							this._complexApprovalInitialized(false);

							f.i18n.loadBundle("viewer", function () {
								f.emit("error.SEVERE", f.i18n.t("ApprovalLoadingError"), jqXHR, textStatus, errorThrown);
							});
						}, this)
					});


				}
			};

			/**
			 * Load the approval state for a document using the getapprovalstate.jsp call
			 */
			AbstractDocumentVersion.prototype._loadApprovalState = function () {
				//getapprovalstate.jsp?projectid=00002_0000136693&docid=00002_0000001174&docversionid=00002_0000002663
				var readURL = dataAdapter.WebCenter("getapprovalstate.jsp", {
					projectid: this.getDocument().get("projectid"),
					docid: this.getDocument().get('docid'),
					docversionid: this.get('docversionid')
				});

				f.ajax(readURL, {
					success: _.bind(function (data) {
						data = XMLUtils.xml2obj(data);
						if (data && data.status && data.status.approval) {
							this._approval.approvalState = ApprovalConstants.ViewerStateToComplexApprovalState[data.status.approval.approvalstate];
						}
						this._complexApprovalInitialized(false);
					}, this),
					error: _.bind(function (jqXHR, textStatus, errorThrown) {
						this._complexApprovalInitialized(false);
					}, this)
				});
			};


			/**
			 * Check if
			 * @returns
			 */
			AbstractDocumentVersion.prototype.approvalIsLinkedDocument = function () {
				if (this._approval) {
					return this._approval.isLinked;
				} else {
					return false;
				}
			};

			/**
			 * Initialize complex approval step 2
			 *
			 * Load the complex approval object
			 */
			AbstractDocumentVersion.prototype._loadComplexApproval = function (data) {
				var readURL = dataAdapter.WebCenter("complexapproval.jsp", {
					'projectID': this.getDocument().get('projectid'),
					'folderID': '',
					'docID': this.getDocument().get('docid'),
					'returnURL': 'docdetailsapprovers.jsp',
					'docVerID': this.get('docversionid'),
					'menu_file': '',
					'urlwanted': 'docdetailsapprovers.jsp'
				});
				var bootInfo = data.complexApprovalBootInfo;
				var userObject = {
					userId: bootInfo.userId,
					userName: bootInfo.userName,
					lastName: bootInfo.lastName,
					firstName: bootInfo.firstName,
					isProjectManager: bootInfo.isProjectManager == '1',
					isAdmin: bootInfo.isAdmin == '1',
					canForceApprove: bootInfo.canForceApprove == '1',
					canForceReject: bootInfo.canForceReject == '1',
					canChangeSetup: bootInfo.canChangeSetup == '1',
					canStartApproval: bootInfo.canStartApproval == '1',
					canSeeOthers: bootInfo.canSeeOthers == '1'
				};
				var readProjectFolderCAPP = readURL + "&action=Inherit";
				var actionURL = readURL;

				var stagedApprovalAllowed = bootInfo.stagedApprovalAllowed == '1';
				var languageCode = bootInfo.languageCode;

				var nextversion = false;
				var isViewable = bootInfo.isViewable == '1';

				if (!this._approval.appObjDiv) {
					var id = ViewerUtils.generateUniqueDOMId(this.get('docversionid') + "approvalAppObj");
					var div = $("<div/>").attr("id", id).hide();
					this._approval.appObjDiv = id;
					f.getContext().append(div);
				}

				if (!this._approval.appPanelDiv) {
					var id = ViewerUtils.generateUniqueDOMId(this.get('docversionid') + "approvalAppPanel");
					var div = $("<div/>").attr("id", id).hide();
					this._approval.appPanelDiv = id;
					f.getContext().append(div);
				}

				// create div for dropdownmenu if non existant
				// dropmenudiv is used by complex approval to show the approval dialogs after clicking a button
				if ($("#dropmenudiv").length === 0) {
					$(f.getContext()).append($('<div onmouseover="clearhidemenu()" onmouseout="dynamichide(event)"/>').attr("id", "dropmenudiv").css("visibility", "hidden").addClass("approvalActionDropdown wcr"));
				} else {
					// the dropmenudiv exists already (added by hover.js), move it into the viewer context and add css class for styling
					$("#dropmenudiv").addClass("approvalActionDropdown wcr").appendTo(f.getContext());
				}

				// require needed dependencies
				var wcrJSPath = dataAdapter.getPathToJSFiles();
				require([wcrJSPath + "detailswin.js",
						wcrJSPath + "hover.js",
						wcrJSPath + "date.js",
						wcrJSPath + "dateShift.js",
						wcrJSPath + "datetools.js",
						wcrJSPath + "webcenter_ajax.js",
						wcrJSPath + "webcenter_xml.js",
						wcrJSPath + "webcenter_html.js",
						wcrJSPath + "ComplexApproval.js"
					],
					_.bind(function () {
						f.i18n.loadBundle("approval", _.bind(function (languageKeys) {
							// we are sure that all dependencies are loaded, initialize complex approval
							this._approval.appObject = new complexApproval(this._approval.appObjDiv,
								this._approval.appPanelDiv,
								readURL,
								actionURL,
								readProjectFolderCAPP,
								userObject,
								languageKeys,
								'xmlcontent',
								nextversion,
								stagedApprovalAllowed,
								languageCode,
								isViewable,
								true,
								_.bind(function () {
									this._complexApprovalInitialized(true);
								}, this),
								_.bind(this._approvalUpdated, this),
								dataAdapter.getPathToWCRImages());
						}, this));
					}, this)
				);
			};

			/**
			 * Action to be done when approval is updated
			 */
			AbstractDocumentVersion.prototype._approvalUpdated = function (success) {
				// if document is a pagelist, invalidate all page approval info
				// we also want to get the global approval statuses again
				if (this.isPageList() && this._approval.appObject && this._approval.appObject.getApprovalStatus() != "NotDefined") {
					// invalidate all pages
					var pages = this.getAllPages();
					_.each(pages, function (page) {
						page.invalidatePageApproval();
					});

					// reload global approval for document by reading it from the server again
					// we need to do this, because we don't know which pages were approved (some may have failed) and we don't want to load the approval for each page
					// 
					// reloading flag is used, because update event is called whenever the data is loaded 
					// The flag will prevent infinite loops and make sure the "update" event is only propagated when the approval is fully loaded
					if (!this._approval.reloading) {
						this._approval.reloading = true;
						this._approval.appObject.readCAppFromXML(this._approval.appObject.readURL);
					} else {
						this._approval.reloading = false;
						this._emit("approval.update", true);
					}
				} else {
					// emit an update
					this._emit("approval.update", true);
				}
			};

			/**
			 * Initialize complex approval step 3
			 *
			 * The complexApproval object now is correctly initialized (or initialization failed), notify listeners!
			 * @param {Boolean} success - true if complex approval was correctly initialized
			 */
			AbstractDocumentVersion.prototype._complexApprovalInitialized = function (success) {
				f.emit("loading.stop", "bootComplexApproval");

				this._approval.status = success ? "available" : "notAvailable";
				this._emit("approval.loaded", success);
			};


			/**
			 * Get the ID of the div in which the Complex Approval object is stored
			 * @returns {String} ID of the div in which the Complex Approval object is stored
			 */
			AbstractDocumentVersion.prototype.getApprovalObjectDivId = function () {
				this._initApproval();
				return this._approval.appObjDiv;
			};

			/**
			 * Get the ID of the div in which the Complex Approval could draw itself (AppPanel div)
			 * @returns {String} ID of the div in which the Complex Approval could draw itself (AppPanel div)
			 */
			AbstractDocumentVersion.prototype.getApprovalPanelDivId = function () {
				this._initApproval();
				return this._approval.appPanelDiv;
			};

			/**
			 * Get the details data structure of the approval
			 * @returns {Object}
			 */
			AbstractDocumentVersion.prototype.getApprovalDetails = function () {
				return this._approval.appObject.getApprovalDetailsData();
			};

			/**
			 * Check if an approval setup was defined
			 * @returns {Boolean}
			 */
			AbstractDocumentVersion.prototype.hasApprovalSetup = function () {
				return this._approval && this._approval.appObject && this._approval.appObject.getApprovalStatus() != "NotDefined";
			};

			/**
			 * Check if the approval setup is being loaded
			 * @returns {Boolean}
			 */
			AbstractDocumentVersion.prototype.isApprovalLoading = function () {
				return this._approval && this._approval.status === "loading";
			};


			/**
			 * Show the mouseover message when hovering over a specific comment
			 * @param {HTMLObject} target - the target over which was hovered
			 * @param {Event} event - the mouseover event
			 * @param {Number} step - internal number of the step which is hovered
			 * @param {Number} approver - internal number of the approver which is hovered
			 */
			AbstractDocumentVersion.prototype.showApprovalCommentsHover = function (target, event, step, approver) {
				this._approval.appObject.showCommentHover(target, event, step, approver);
			};


			/**
			 * Check if the currently logged in user is an approval master
			 * if the info is not yet available, undefined will be returned (if the info is available callback is called)
			 * @param {Function} [callback] - function to be called when the info becomes available on a later stage (is not called when info is available already)
			 * @returns {Boolean} (or undefined when the info is not available yet)
			 */
			AbstractDocumentVersion.prototype.isUserApprovalMaster = function (callback) {
				this._initApproval();

				if (this._approval.isCurrentUserApprovalMaster === undefined) {
					if (callback) {
						this.once("approval.approvalMasterInfo", callback);
					}

					// we didn't fetch this information yet, fetch it
					if (!this._approval.isApprovalMasterInfoBeingFetched) {
						this._approval.isApprovalMasterInfoBeingFetched = true;

						// Generate URL to get te approval state (assuming we have a webcenter document)
						var url = dataAdapter.WebCenter("getapprovalstate.jsp", {
							'projectid': this.getDocument().getProjectID(),
							'docid': this.getDocument().getDocumentID(),
							'docversionid': this.getDocumentVersionID()
						});

						f.ajax(url, {
							success: _.bind(function (data) {
								if (data) {
									data = XMLUtils.xml2obj(data);

									if (data.status.value == "1") {
										// there is an error
										f.i18n.loadBundle("viewer", function () {
											f.emit("error.SEVERE", f.i18n.t("ApprovalLoadingError"), data.status.message);
										});
									} else {
										this._approval.isCurrentUserApprovalMaster = (data.status.approval.approvalmaster == "1");
									}

									this._approval.isApprovalMasterInfoBeingFetched = false;
								} else {
									// there is an error
									f.i18n.loadBundle("viewer", function () {
										f.emit("error.SEVERE", f.i18n.t("ApprovalLoadingError"), "Data was not set");
									});
								}

								this._emit("approval.approvalMasterInfo");
							}, this),
							error: _.bind(function (jqXHR, textStatus, errorThrown) {
								this._approval.isApprovalMasterInfoBeingFetched = false;

								f.i18n.loadBundle("viewer", function () {
									f.emit("error.SEVERE", f.i18n.t("ApprovalLoadingError"), jqXHR, textStatus, errorThrown);
								});

								this._emit("approval.approvalMasterInfo");
							}, this)
						});
					}

				}


				// return the current value
				return this._approval.isCurrentUserApprovalMaster;
			};


			//---------------- AbstractDocumentPage extensions -------------------------------------------------------------
			/**
			 * Check if an approval setup was defined
			 * @returns {Boolean}
			 */
			AbstractDocumentPage.prototype.hasApprovalSetup = function () {
				return this.getDocumentVersion().hasApprovalSetup();
			};

			/**
			 * Get a list of possible approval actions
			 * @returns {Array.<Object>}
			 */
			AbstractDocumentPage.prototype.getApprovalActions = function () {
				var actions = {
					list: [],
					text: '',
					'showContent': false
				};

				if (this.getDocumentVersion()._approval && this.getDocumentVersion()._approval.status === "available") {
					actions = this.getDocumentVersion()._approval.appObject.getApprovalActions(this.getDocumentVersion().isPageList());
				}

				return actions;
			};


			/**
			 * Get the approval Object for this page
			 * @returns {Object} (or undefined if the approval object is not defined)
			 */
			AbstractDocumentPage.prototype.getApprovalObject = function () {
				var docVersion = this.getDocumentVersion();
				if (docVersion._approval && docVersion._approval.appObject && docVersion._approval.status === "available") {
					return docVersion._approval.appObject;
				} else {
					return undefined;
				}
			};



			/**
			 * Get the current approval status
			 * @returns {String}
			 */
			AbstractDocumentPage.prototype.getApprovalStatus = function () {
				var docVersion = this.getDocumentVersion();
				if (docVersion._approval && docVersion._approval.appObject) {
					return docVersion._approval.appObject.getApprovalStatus();
				} else if (docVersion._approval && docVersion._approval.approvalState) {
					return docVersion._approval.approvalState;
				} else {
					return "NotDefined";
				}
			};

			/**
			 * Check if there is an ongoing approval cycle on the document
			 * @returns {Boolean}
			 */
			AbstractDocumentPage.prototype.isUnderApproval = function () {
				return this.isApprovalStarted() && !(this.isApproved() || this.isRejected());
			};

			/**
			 * Check if the page has been approved (this could be forced or conditional)
			 * @returns {Boolean}
			 */
			AbstractDocumentPage.prototype.isApproved = function () {
				var status = this.getApprovalStatus();
				return status === "Approved" || status === "ForceApproved" || status === "ForceApprovedConditional" || status === "ApprovedConditional";
			};

			/**
			 * Check if the page has been rejected (this could be forced)
			 * @returns {Boolean}
			 */
			AbstractDocumentPage.prototype.isRejected = function () {
				var status = this.getApprovalStatus();

				return status === "Rejected" || status === "ForceRejected";
			};

			/**
			 * Check if the current user is an approver
			 * @returns {Boolean}
			 */
			AbstractDocumentPage.prototype.isUserApprover = function () {
				var approval = this.getApprovalObject();
				if (approval) {
					return approval.isUserApprover();
				}

				return false;
			};

			/**
			 * Check if the current user is an approval master
			 *
			 * @param {Function} callback - if the info is not availble yet, the info is requested and callback is called
			 * @returns {Boolean} or undefined if info is not available
			 */
			AbstractDocumentPage.prototype.isUserApprovalMaster = function (callback) {
				var docVersion = this.getDocumentVersion();

				return docVersion.isUserApprovalMaster(callback);
			};

			/**
			 * Check if the current user is in the pending approval cycle
			 * @returns {Boolean}
			 */
			AbstractDocumentPage.prototype.isUserPendingApprover = function () {
				var approval = this.getApprovalObject();

				if (approval) {
					return approval.isUserPendingApprover();
				}
			};

			/**
			 * Check if the approval has started
			 * @returns {Boolean}
			 */
			AbstractDocumentPage.prototype.isApprovalStarted = function () {
				var approval = this.getApprovalObject();

				if (approval) {
					return approval.isApprovalStarted();
				}

				return false;
			};


			/**
			 * Request the approval setup for the page, and wait for it before the callback is called.
			 *
			 * If approval setup is available, callback is called immediately
			 * @param {Function} callback
			 */
			AbstractDocumentPage.prototype.whenApprovalSetupAvailable = function (callback) {
				var docVersion = this.getDocumentVersion();
				docVersion.requestApprovalSetup(callback);
			};

			/**
			 * Initialize the approval data extensions on the document page
			 * @private
			 */
			AbstractDocumentPage.prototype._initPageApproval = function () {
				if (!this._approval) {
					this._approval = {
						status: "initialized",
						dataObj: undefined
					};
				}
			};

			/**
			 * Invalidate the approval data for the document page
			 */
			AbstractDocumentPage.prototype.invalidatePageApproval = function () {
				if (this._approval) {
					this._approval.status = "initialized";
					this._approval.dataObj = undefined;
				}
			};

			/**
			 * Tell whether or not the approval is fully loaded
			 * @private
			 */
			AbstractDocumentPage.prototype.isApprovalLoaded = function () {
				if (this._approval && (this._approval.status === "available" || this._approval.status === "notAvailable")) {
					return true;
				}
				return false;
			};

			/**
			 * Set Page approval initialization status
			 * @param {Boolean} [success] - Define whether or not approval is available for the page
			 * @private
			 */
			AbstractDocumentPage.prototype._pageApprovalInitialized = function (success) {
				this._approval.status = success ? "available" : "notAvailable";
				this._emit("approval.loaded", success);
			};

			/**
			 * Request the approval status of the page. This will overwrite the approval object with info of this page.
			 * If the info is not available yet, it will be fetched.
			 * @param {Function} [callback] - function to be executed when approval object is updated
			 */
			AbstractDocumentPage.prototype.requestPageApprovalSetup = function (callback) {
				this._initPageApproval();

				// handle callback
				if (this._approval.status === "notAvailable") {
					if (callback) {
						callback();
					}
				} else if (this._approval.status === "available") {
					this._overwritePageApprovalSetup(callback);
				} else {
					this.once("approval.loaded", callback);
				}

				// get data if needed
				if (this._approval.status === "initialized" || this._approval.status === "outdated") {
					this._approval.status = 'loading';
					var historyUrl = dataAdapter.WebCenter("GetDocumentHistory.jsp", {
						'projectid': this.getDocument().get('projectid'),
						'docid': this.getDocument().get('docid'),
						'docversionid': this.getDocumentVersion().get("docversionid"),
						'approvalinfo': 1,
						'annotationinfo': 0,
						'fulluserinfo': 0,
						'pageno': this.getPagenum(),
						'pagename': this.get("pagename") ? this.get("pagename") : this.getPagenum(),
						'pageindex': this.get("pageindex") ? this.get("pageindex") : this.getPagenum() - 1
					});

					// get object from doc history via Ajax call
					f.ajax(historyUrl, {
						success: _.bind(function (data) {
							data = XMLUtils.xml2obj(data);
							if (data.error) {
								// error, log error and notify listeners
								this._pageApprovalInitialized(false);

								f.i18n.loadBundle("viewer", function () {
									f.emit("error.SEVERE", f.i18n.t("ApprovalLoadingError"), data.error);
								});
							} else {
								// update approval object
								this._approval.dataObj = data;
								this._overwritePageApprovalSetup(callback);
							}
						}, this),
						error: _.bind(function (jqXHR, textStatus, errorThrown) {
							// error, log error and notify listeners
							this._pageApprovalInitialized(false);

							f.i18n.loadBundle("viewer", function () {
								f.emit("error.SEVERE", f.i18n.t("ApprovalLoadingError"), jqXHR, textStatus, errorThrown);
							});
						}, this)
					});
				}
			};

			/**
			 * This method will do the actual overwriting of the approval object with the data received from the document history ajax call.
			 * @param {Function} [callback] - function to be executed when approval object is updated
			 * @private
			 */
			AbstractDocumentPage.prototype._overwritePageApprovalSetup = function (callback) {
				this.whenApprovalSetupAvailable(_.bind(function () {
					var data = this._approval.dataObj;
					var appObject = this.getApprovalObject();
					// need to determine whether we have an array or an object first (because of xml2obj use)
					var docVersionInfo = null;
					var docVersionInfoArr = [];
					if (data.document_info && data.document_info.document_version) {
						if (_.isArray(data.document_info.document_version)) {
							docVersionInfoArr = data.document_info.document_version;
						} else {
							docVersionInfoArr = [data.document_info.document_version];
						}
					}
					// now select the correct version. If we don't find the correct version, take the highest one that is before the current version
					for (var i = 0; i < docVersionInfoArr.length; i++) {
						if (parseInt(docVersionInfoArr[i].version_number) === this.getDocumentVersion().getVersionId()) {
							docVersionInfo = docVersionInfoArr[i];
							break;
						} else if (parseInt(docVersionInfoArr[i].version_number) < this.getDocumentVersion().getVersionId()) {
							// temporarily store the highest version we have so far.
							// If we don't find a page with the current version, show approval for highest version before this version.
							docVersionInfo = docVersionInfoArr[i];
						}
					}

					if (appObject && docVersionInfo && appObject.getApprovalStatus() !== "NotDefined") {
						// apply status to the approval stage
						var appStatus = ApprovalConstants.ViewerStateToComplexApprovalState[docVersionInfo.document_approval_code];
						if (!appStatus) {
							if (appObject.getApprovalStatus() === "NotStarted") {
								appStatus = "NotStarted";
							} else {
								appStatus = "Pending";
							}
						}
						appObject.approvalStatus = appStatus;
						if (appObject.steps[0]) {
							// note, only one stage available for pagelists, so we'll always only have step[0]
							appObject.steps[0].approvalStatus = appStatus;
						}

						// now go deeper and look at the actions per approver
						if (docVersionInfo.stages && docVersionInfo.stages.stage) {
							var approvers = docVersionInfo.stages.stage.approver_list.approver;
							if (approvers && !_.isArray(approvers)) {
								// approvers is not an array, but a single object
								approvers = [approvers];
							}
							_.each(approvers, _.bind(function (submittedApproval) {
								var appStage = appObject.steps[0];
								if (appStage) {
									var approver = _.find(appStage.approvers, function (approver) {
										return approver.memberObj.userId === submittedApproval.userID || (submittedApproval.group && approver.memberObj.groupId === submittedApproval.group.groupID);
									});

									if (approver) {
										// match entries of document history with entires of approval object
										if (approver.memberObj.userId === submittedApproval.userID || (submittedApproval.group && approver.memberObj.groupId === submittedApproval.group.groupID)) {
											// need to determine whether we have an array or an object first (because of xml2obj use)
											var appAction = null;
											if (submittedApproval.approval_action) {
												if (_.isArray(submittedApproval.approval_action)) {
													// take the final action; if a document is approvad, approval will always be the final action, 
													// unless it's forced approved afterwards, in which case the force approval will be the last action
													appAction = submittedApproval.approval_action[submittedApproval.approval_action.length - 1];
												} else {
													appAction = submittedApproval.approval_action;
												}
											}

											// read approval info and overwrite complex approval object
											if (appAction) {
												var status = appAction.approval_status_ID;
												var date = null;
												var version = "";
												var approvedBy = "";
												var commentArr = [];

												if (status !== "00001_0000000002") {
													if (appAction.approval_date) {
														date = new Date(appAction.approval_date.replace(' ', 'T'));
													}
													version = appAction.doc_version;
													approvedBy = appAction.actual_user_approver ? appAction.actual_user_approver.userID : '';
												}

												// go over the comments of all approval actions
												// need to determine whether we have an array or an object for "approval_action" first (because of xml2obj use)
												var approvalActions = [];
												if (submittedApproval.approval_action && submittedApproval.approval_action.length === undefined) {
													approvalActions.push(submittedApproval.approval_action);
												} else if (submittedApproval.approval_action) {
													approvalActions = submittedApproval.approval_action;
												}
												// create comment objects
												_.each(approvalActions, function (action) {
													// create approval comment object
													if (action.approval_comment) {
														var userId = '';
														if (approver.memberObj.userId) {
															userId = approver.memberObj.userId;
														} else if (appAction.actual_user_approver) {
															userId = appAction.actual_user_approver.userID;
														}
														var commentDate = null;
														if (action.approval_comment.date) {
															commentDate = new Date(action.approval_comment.date.replace(' ', 'T'));
														}
														commentArr.push({
															commentText: action.approval_comment.comment,
															commentTime: commentDate,
															commentByUserId: userId,
															commentVersion: action.doc_version
														});
													}
												});

												this._overwriteApproverEntry(approver, status, date, version, approvedBy, commentArr);
											} else {
												// set as pending without extra info
												this._overwriteApproverEntry(approver);
											}
										}
									}
								}
							}, this));
						}

						// update current page model
						this.set("document_approval_code", ApprovalConstants.ComplexApprovalStateToViewerState[appObject.getApprovalStatus()]);
						this.set("user_approval_code", ApprovalConstants.ComplexApprovalStateToViewerState[appObject.currentUserApprovalStatus(0)]);

						// Execute the initial callback, which was passed along
						if (callback) {
							callback();
						}

						this._pageApprovalInitialized(true);
					} else {
						this._pageApprovalInitialized(false);
					}
				}, this));
			};

			/**
			 * Overwrites one specific approver entry of approval.
			 * @param {Object} [approver] - approver to overwrite
			 * @param {String} [status] - status of entry
			 * @param {Date} [date] - date of approval
			 * @param {String} [version] - version in which entry was approved
			 * @param {String} [approvedBy] - ID of user that did approval for group
			 * @param {Array} [comments] - Array of submitted comments
			 * @private
			 */
			AbstractDocumentPage.prototype._overwriteApproverEntry = function (approver, status, date, version, approvedBy, comments) {
				if (approver) {
					if (status === undefined) {
						approver.approvalStatus = ApprovalConstants.ViewerStateToComplexApprovalState["00001_0000000002"];
					} else {
						status = ApprovalConstants.ViewerStateToComplexApprovalState[status];
						if (!status) {
							status = ApprovalConstants.ViewerStateToComplexApprovalState["00001_0000000002"];
						}
						approver.approvalStatus = status;
					}

					if (date === undefined) {
						approver.approvalTime = null;
					} else {
						approver.approvalTime = date;
					}

					if (version === undefined) {
						approver.approvedVersion = "";
					} else {
						approver.approvedVersion = version;
					}

					if (approvedBy === undefined) {
						approver.approvedByUserId = "";
					} else {
						approver.approvedByUserId = approvedBy;
					}

					if (comments === undefined) {
						approver.commentArr = [];
					} else {
						approver.commentArr = comments;
					}
				}

			};

		};

		return ApprovalDataExtensions;
	});

define('web-viewer/approval/ApprovalToolbarComponent',["component/toolbar/ToolbarComponent",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/utils/ListenerUtil",
		"underscore"
	],
	function (ToolbarComponent,
		ButtonGroup,
		ViewerToolbarButton,
		LayoutConstants,
		ListenerUtil,
		_
	) {
		/**
		 * Toolbar component for approval
		 *
		 * @exports ApprovalToolbarComponent
		 * @memberof ApprovalModule
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Approval.ApprovalModule} module
		 */
		var ApprovalToolbarComponent = function (f, approvalModule) {
			ToolbarComponent.call(this, LayoutConstants.ToolbarComponent.APPROVAL, f);
			ListenerUtil.mixin(this, f);

			this.f = f;
			this._approvalModule = approvalModule;
			this._registerListener("updated", this.updateComponent, this, this._approvalModule);
			// start by displaying loading message
			this.currentState = "Loading";

			this.status = $("<div/>").addClass("Loading approvalBtn").text("Loading...");
			this.statusIndicator = $("<div/>").addClass("btn-group btnGroupWithTitle").append(this.status);

			f.i18n.loadBundle("viewer", function () {
				this.statusIndicator.append($("<div/>").addClass("btnGroupTitle").text(f.i18n.t("ApprovalBtnGroupTitle")));
			}, this);


			this.element.append(this.statusIndicator).addClass("approvalControls");
			this.buttonGroup = new ButtonGroup("approval");
			this.buttonGroup.setTitle(f.i18n, "viewer", "ApprovalBtnGroupTitle");
			this.addButtonGroup(this.buttonGroup);
			this.buttonGroup.element.hide();

			this.updateComponent();
		};

		ApprovalToolbarComponent.prototype = Object.create(ToolbarComponent.prototype);
		ApprovalToolbarComponent.prototype.constructor = ApprovalToolbarComponent;
		ApprovalToolbarComponent.toolbarName = LayoutConstants.ToolbarComponent.APPROVAL;


		ApprovalToolbarComponent.prototype.setStatus = function (status) {
			this.status.toggleClass(this.currentState, false);

			// hide the component if no approval defined
			if (status !== "NotDefined") {
				this.show();
			} else {
				this.hide();
			}
			this.currentState = status;
			this.status.toggleClass(this.currentState, true);

			this.f.i18n.loadBundle("approval", function () {
				this.status.empty();
				this.status
					.append($("<span/>").addClass("enableTextSelection").text(this.f.i18n.t("CApp" + this.currentState)))
					.prepend($("<i class='icon-normal approvalStatus'/>").addClass(this.currentState));
			}, this);
		};

		/**
		 * Updates the toolbar component when the approval setup or compare mode has changed
		 */
		ApprovalToolbarComponent.prototype.updateComponent = function () {
			// get the active document
			var docController = this.f.getDocumentController();
			var page = docController.getActivePage();


			if (page) {
				this.show();
				this.setStatus("Loading");
				this.toggleButtons(false);


				this.setStatus(page.getApprovalStatus());


				// clear buttons
				this.buttonGroup.clearButtons();

				// add buttons			
				var actions = this._approvalModule.getAvailableActions();
				for (var i = 0; i < actions.list.length; i++) {
					var action = actions.list[i];

					var button = new ViewerToolbarButton(this.f, action.name, action.name, true, action.action);
					button.element.addClass(action.name);
					button.setTooltip(action.text);

					this.buttonGroup.addButton(button);
				}
				this.toggleButtons(actions.list.length > 0);

				// hide if there is nothing to show
				if (actions.list.length === 0 && !this._approvalModule.showStatus()) {
					this.hide();
				}
			} else {
				// no document active
				this.buttonGroup.clearButtons();
				this.buttonGroup.hide();
				this.hide();
			}

			this._emit("changed.content");
		};

		/**
		 * Toggle whether buttons should be visible or not
		 * @param show
		 */
		ApprovalToolbarComponent.prototype.toggleButtons = function (show) {
			this.buttonGroup.element.toggle(show);
			this.statusIndicator.toggle(!show);
		};


		return ApprovalToolbarComponent;
	});

define('web-viewer/approval/ApprovalMobileToolbarComponent',["component/toolbar/ToolbarComponent",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/layout/component/DropdownToolbarButton",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/utils/ListenerUtil",
		"underscore"
	],
	function (ToolbarComponent,
		ButtonGroup,
		ViewerToolbarButton,
		DropdownToolbarButton,
		LayoutConstants,
		ListenerUtil,
		_
	) {
		"use strict";
		/**
		 * mobile Toolbar component for approval
		 *
		 * @exports ApprovalMobileToolbarComponent
		 * @memberof ApprovalModule
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Approval.ApprovalModule} module
		 */
		var ApprovalMobileToolbarComponent = function (f, approvalModule) {
			ToolbarComponent.call(this, LayoutConstants.ToolbarComponent.APPROVAL, f);
			ListenerUtil.mixin(this, f);

			this.f = f;
			this._approvalModule = approvalModule;
			this._registerListener("updated", this.updateComponent, this, this._approvalModule);
			// start by displaying loading message
			this.currentState = "Loading";


			this._icon = $("<i>").addClass("icon");

			this._dropdownMenu = new DropdownToolbarButton(this.f, "approval", this._icon, false);

			this.element.append(this._dropdownMenu.element).addClass("approvalControls");


			this.updateComponent();
		};

		ApprovalMobileToolbarComponent.prototype = Object.create(ToolbarComponent.prototype);
		ApprovalMobileToolbarComponent.prototype.constructor = ApprovalMobileToolbarComponent;
		ApprovalMobileToolbarComponent.toolbarName = LayoutConstants.ToolbarComponent.APPROVAL;


		/**
		 * Update the display of the status in the approval toolbar component
		 * @param {String} status - the approval status that should be displayed
		 */
		ApprovalMobileToolbarComponent.prototype.setStatus = function (status) {
			this._dropdownMenu.element.toggleClass(this.currentState, false).toggleClass(status, true);

			// hide the component if no approval defined
			if (status !== "NotDefined") {
				this.show();
			} else {
				this.hide();
			}
			this.currentState = status;

			this._icon.removeClass().addClass("icon icon-WCR_" + status);
			this.f.i18n.loadBundle("approval", function () {
				this._dropdownMenu.element.text(this.f.i18n.t("CApp" + this.currentState));
				this._dropdownMenu.element.prepend(this._icon.addClass("statusIcon"));
				this._dropdownMenu.element.append($("<i>").addClass("icon icon-WCR_Sort-Down dropdownIndicator"));
			}, this);
		};

		/**
		 * Updates the toolbar component when the approval setup or compare mode has changed
		 */
		ApprovalMobileToolbarComponent.prototype.updateComponent = function () {
			// get the active document
			var docController = this.f.getDocumentController();
			var page = docController.getActivePage();


			if (page) {
				this.show();
				this.setStatus("Loading");
				this.setStatus(page.getApprovalStatus());


				// clear buttons
				this._dropdownMenu.clear();

				// add buttons			
				var actions = this._approvalModule.getAvailableActions();
				for (var i = 0; i < actions.list.length; i++) {
					var action = actions.list[i];

					this._dropdownMenu.addAction({
						name: action.name,
						content: action.text,
						icon: "icon-WCR_" + action.name,
						action: action.action,
						cssClass: action.name,
						enabled: true
					});
				}
				if (actions.list.length === 0) {
					this._dropdownMenu.disableDropdown();
					if (!this._approvalModule.showStatus()) {
						this.hide();
					}
					this.element.toggleClass("clickable", false);
				} else {
					this._dropdownMenu.enableDropdown();
					this.element.toggleClass("clickable", true);
				}

			} else {
				// no document active
				this.buttonGroup.clearButtons();
				this.buttonGroup.hide();
				this.hide();
			}

			this._emit("changed.content");
		};




		return ApprovalMobileToolbarComponent;
	});

define('tpl!web-viewer/approval/ApprovalDetailsWidget', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<div class="approvalDetails">\r\n';
 if(d.hasApprovalSetup){ 
__p+='\r\n<table>\r\n';
 _.each(d.steps, function(step){ 
__p+='\r\n	<tbody class="step '+
((__t=(step.status))==null?'':_.escape(__t))+
'">\r\n	<tr>\r\n		<td colspan="2">\r\n		';
 if (step.name.length > 0){ 
__p+='\r\n			<em>'+
((__t=(step.number))==null?'':_.escape(__t))+
': </em>\r\n				'+
((__t=(step.name))==null?'':_.escape(__t))+
' \r\n		';
 } else { 
__p+='\r\n			'+
((__t=(step.translatedStatusName))==null?'':_.escape(__t))+
'\r\n		';
 } 
__p+='\r\n			<i class="icon-normal '+
((__t=(step.status))==null?'':_.escape(__t))+
' approvalStatus" tooltip="'+
((__t=(step.tooltip))==null?'':_.escape(__t))+
'"></i>\r\n		</td>\r\n	</tr>\r\n	</tbody>\r\n\r\n		';
 _.each(step.approvers, function(approver){ 
__p+='\r\n			<tbody class="approver">\r\n				<tr>\r\n					<th>'+
((__t=(d.i18n('CAppApprover')))==null?'':_.escape(__t))+
': </th>\r\n					<td>'+
((__t=(approver.user.name))==null?'':_.escape(__t))+
'</td>\r\n				</tr>\r\n			\r\n				<tr><th>'+
((__t=(d.i18n('CAppUserType')))==null?'':_.escape(__t))+
': </th>\r\n				<td>'+
((__t=(d.i18n(approver.typeLangTag)))==null?'':_.escape(__t))+
'</td></tr>\r\n				\r\n				';
 if (step.status != 'NotStarted' && !(step.status != 'Pending' && approver.status == 'Pending')){ 
__p+='\r\n				<tr><th>'+
((__t=(d.i18n('CAppStatusVersion')))==null?'':_.escape(__t))+
': </th>\r\n				<td><i class="'+
((__t=(approver.status))==null?'':_.escape(__t))+
' icon-normal approvalStatus"></i><span class="'+
((__t=(approver.status))==null?'':_.escape(__t))+
'">'+
((__t=(d.i18n('CApp'+approver.status)))==null?'':_.escape(__t))+
'</span> \r\n					';
 if(approver.status != 'Pending') { 
__p+='\r\n					<span class="'+
((__t=((approver.isDifferentVersionApproved?'markedVersion':'')))==null?'':_.escape(__t))+
'">\r\n						('+
((__t=(approver.approvedVersion))==null?'':_.escape(__t))+
')\r\n					</span> \r\n					';
 } 
__p+='</td></tr>\r\n				';
 } 
__p+='\r\n				';
 if (approver.approvalTime){ 
__p+='\r\n					<tr><th>'+
((__t=(d.i18n('CAppApprovalDate')))==null?'':_.escape(__t))+
': </th>\r\n					<td>'+
((__t=(d.date(approver.approvalTime)))==null?'':_.escape(__t))+
'</td></tr>\r\n					\r\n				';
 } 
__p+='\r\n				';
 if (approver.approvedBy){ 
__p+='\r\n					<tr><th>'+
((__t=(d.i18n('CAppApprovedBy')))==null?'':_.escape(__t))+
': </th>\r\n					<td>'+
((__t=(approver.approvedBy.name))==null?'':_.escape(__t))+
'</td></tr>\r\n				';
 } 
__p+='\r\n				\r\n				';
 if (approver.comments){ 
__p+='\r\n					<tr class="comments" data-bind="'+
((__t=(step.internalNumber))==null?'':_.escape(__t))+
'|'+
((__t=(approver.internalNumber))==null?'':_.escape(__t))+
'"><th>'+
((__t=(d.i18n('CAppApprovalCommentCondition')))==null?'':_.escape(__t))+
': </th>\r\n					<td>		';
 _.each(approver.comments, function(comment){ 
__p+='\r\n								'+
((__t=(comment.commentText))==null?'':_.escape(__t))+
'<br/>\r\n							';
 });
__p+='\r\n					</td></tr>\r\n				';
 } 
__p+='\r\n			</tbody>\r\n		';
 }); 
__p+='\r\n\r\n';
 }); 
__p+='\r\n</table>\r\n';
 } else { 
__p+='\r\n	<span>'+
((__t=( d.message ))==null?'':_.escape(__t))+
'</span>\r\n';
 } 
__p+='\r\n</div>';
return __p;
}});
define('web-viewer/approval/ApprovalDetailsWidget',["web-viewer/layout/AbstractWidget",
		"Template",
		"tpl!web-viewer/approval/ApprovalDetailsWidget",
		"component/tooltip/Tooltip",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/compare/CompareConstants",
		"web-viewer/utils/ListenerUtil",
		"underscore"
	],
	function (AbstractWidget,
		Template,
		tpl,
		Tooltip,
		LayoutConstants,
		CompareConst,
		ListenerUtil,
		_) {
		"use strict";
		/**
		 * Details table widget of complex approval
		 *
		 * @exports ApprovalDetailsWidget
		 * @memberof ApprovalModule
		 * @constructor
		 * @mixes {Utils.ListenerUtilMixin}
		 *
		 *
		 * @param {Core.ViewerFacade} f
		 */
		var ApprovalDetailsWidget = function (f) {
			AbstractWidget.call(this, f, LayoutConstants.Widget.APPROVAL);
			ListenerUtil.mixin(this, f);

			// set widget title
			f.i18n.loadBundle("viewer", _.bind(function () {
				this.setTitle(f.i18n.t("ApprovalWidgetTitle"));
			}, this));

			/**
			 * currently loaded doc
			 * @private
			 */
			this._currentDocVersion = null;


			// The template used for rendering the widget
			this.t = new Template(f.i18n, tpl);
			this.showInfo = true;

			// when the user hovers over a comment, show the mouseover of the complex approval
			this.t.bind("mouseover", ".comments", function (source, event) {
				if (this._currentDocVersion && this._currentDocVersion.hasApprovalSetup()) {
					var data = ($(source).attr("data-bind") || "").split("|");
					var step = data[0];
					var approver = data[1];

					this._currentDocVersion.showApprovalCommentsHover(source, event, step, approver);
				}
			}, this);
			
			this.setIcon("icon icon-WCR_Approve-round")

			this.t.bind("mouseout", ".comments", function (source, event) {
				delayhidemenu(); // note this a global function introduced by hover.js which is included by approvalmodule (hooray!)
			});

			// add a tool tip for when the step status is hovered
			this.whenContextAvailable(function () {
				this.tooltip = new Tooltip(this.f.i18n, this.context, this.f.getContext(), this.f.getContext(), this.f.isMobile(), "");
				this.tooltip.disableTooltip();
				// filter on all elements with attribute tooltip
				this.t.bind("mouseover", "[tooltip]", function (source, event) {
					this.tooltip.setTooltipContent($(source).attr("tooltip"));
					this.tooltip.context = $(source);
					this.tooltip.show();
				}, this);
				// filter on all elements with attribute tooltip
				this.t.bind("mouseout", "[tooltip]", function (source, event) {
					this.tooltip.hide();
				}, this);
			}, this);




			// react on compare mode changes
			this._registerListener("module.compare.changed.mode", this.onCompareModeChanged, this);
			// make sure we update the toolbar component when compare becomes available
			this.f.whenModuleAvailable("compare", function (compareModule) {
				this.onCompareModeChanged(null, compareModule.getActiveMode());
			}, this);

			// react when the active document has changed
			this.f.getDocumentController().whenActiveDocumentVersionChanged(this.selectDocument, this);

			// if the new document is a page list, listen to page change events
			if (this._currentDocVersion.isPageList()) {
				var updateListener = function (oldPage, newPage) {
					if (oldPage) {
						// remove listeners
						this._unregisterAllListenersForObject(oldPage);

						//also remove listener from active page when in difference mode and vice versa
						if (this.showDifferenceInfo) {
							this._unregisterAllListenersForObject(this.f.getActivePage());
						} else {
							this._unregisterAllListenersForObject(this.f.getDocumentController().getDifferencePage());
						}

					}

					if (newPage) {
						// register listeners, and request approval setup	
						this._registerListener("approval.loaded", this.render, this, newPage);
						newPage.requestPageApprovalSetup(_.bind(this.render, this));
					}
				};
				this.f.getDocumentController().whenDifferencePageChanged(updateListener, this);
				this.f.getDocumentController().whenActivePageChanged(updateListener, this);
			}

			this.selectDocument();
		};

		// inherit from approval details widget
		ApprovalDetailsWidget.prototype = Object.create(AbstractWidget.prototype);
		ApprovalDetailsWidget.prototype.constructor = ApprovalDetailsWidget;
		ApprovalDetailsWidget.widgetName = LayoutConstants.Widget.APPROVAL;

		/**
		 * Set the context of the widget
		 */
		ApprovalDetailsWidget.prototype.setContext = function (context, detailsExtra) {
			AbstractWidget.prototype.setContext.call(this, context, detailsExtra);
			var loader = $("<span/>").text("Loading");
			this.context.append(loader);

			this.f.i18n.loadBundle("viewer", function () {
				if (loader) {
					loader.text(this.f.i18n.t("Loading"));
				}
			}, this);

			this.context.addClass("enableTextSelection");

			this.t.setContext(context);
		};

		/**
		 * The active document has changed/ or compare mode, select the correct document
		 */
		ApprovalDetailsWidget.prototype.selectDocument = function () {
			var oldDocVersion = this._currentDocVersion;

			if (oldDocVersion) {
				this._unregisterAllListenersForObject(oldDocVersion);
			}

			if (this.showDifferenceInfo) {
				this._currentDocVersion = this.f.getDocumentController().getDifferencePage().getDocumentVersion();
			} else {
				this._currentDocVersion = this.f.getDocumentController().getActiveDocumentVersion();
			}


			if (this._currentDocVersion) {
				this._registerListener("approval.loaded", this.render, this, this._currentDocVersion);
				this._registerListener("approval.update", this.render, this, this._currentDocVersion);

				if (!this._currentDocVersion.isPageList()) {
					this._currentDocVersion.requestApprovalSetup(_.bind(this.render, this));
				}
			}
		};


		/**
		 * Compare mode has changed, change the information we show
		 * @param oldMode
		 * @param newMode
		 */
		ApprovalDetailsWidget.prototype.onCompareModeChanged = function (oldMode, newMode) {
			if (!newMode || newMode.getName() === CompareConst.mode.SHOW_CURRENT || (newMode.getName() === CompareConst.mode.READERSPREAD)) {
				this.showInfo = true;
				this.showDifferenceInfo = false;
				this.selectDocument();
			} else if (newMode && newMode.getName() === CompareConst.mode.SHOW_DIFFERENCE) {
				this.showInfo = true;
				this.showDifferenceInfo = true;
				this.selectDocument();
			} else {
				this.showInfo = false;
			}

			this.render();
		};

		/**
		 * Render the widget
		 */
		ApprovalDetailsWidget.prototype.render = function () {
			// render template, notify layout manager of possible layout change
			// we need to have the correct language information
			this.f.i18n.loadBundle(["viewer", "approval"], function () {
				if (this.showInfo) {
					// we can show approval info
					this.t.d.hasApprovalSetup = this._currentDocVersion && this._currentDocVersion.hasApprovalSetup();
					if (this.t.d.hasApprovalSetup) {
						// there is an approval setup, setup data for template
						var data = this._currentDocVersion.getApprovalDetails();
						this.t.d.steps = data.steps;
						this.t.d.isComplexApproval = data.isComplexApproval;

						_.each(this.t.d.steps, function (step) {
							step.tooltip = this.f.i18n.t('CApp' + step.status);
							step.translatedStatusName = this.f.i18n.t('CApp' + step.status);
							if (step.approvalTime) {
								step.tooltip += " " + this.f.i18n.t('AtTime') + " " + this.t.d.date(step.approvalTime);
							}
						}, this);
					} else {
						if (this._currentDocVersion && this._currentDocVersion.isApprovalLoading()) {
							this.t.d.message = this.f.i18n.t("Loading");
						} else if (this._currentDocVersion && this._currentDocVersion.approvalIsLinkedDocument()) {
							// for a linked document, we cannot display details
							this.t.d.message = this.f.i18n.t("approvalCannotDisplayDetailsForLinkedDocument");
						} else {
							// no approval setup defined
							this.t.d.message = this.f.i18n.t("CAppNotDefined");
						}
					}
				} else {
					// we cannot show approval info because some compare mode is active
					this.t.d.hasApprovalSetup = false;
					this.t.d.message = this.f.i18n.t("approvalHiddenBecauseCompare");
				}

				this.t.render();
				this._fireResized();
			}, this);
		};


		return ApprovalDetailsWidget;
	});

define('tpl!web-viewer/approval/PageListApproval', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<div class="comparePagelistPopup range">\r\n	'+
((__t=(d.i18n("AppPages")))==null?'':_.escape(__t))+
'\r\n	<ul>\r\n	';
 if(!d.approveRemaining){ 
__p+='\r\n		<li><label><input type="radio" name="pageRange" value="0" ';
 if(d.range===0){ 
__p+=' checked ';
}
__p+=' /> '+
((__t=(d.i18n("AppPagesCurrentPage")))==null?'':_.escape(__t))+
'</label></li>\r\n	';
}
__p+='\r\n		<li>\r\n			<label><input type="radio" name="pageRange" value="1" ';
 if(d.range===1){ 
__p+=' checked ';
}
__p+='  /> '+
((__t=(d.i18n("AppPagesMarkedPages")))==null?'':_.escape(__t))+
' ('+
((__t=( (d.markedPages === "" ? d.i18n("NoneMarked") : d.markedPages) ))==null?'':_.escape(__t))+
')\r\n		 </label></li>\r\n		<li>\r\n			<label><input type="radio" name="pageRange" value="2" ';
 if(d.range===2){ 
__p+=' checked ';
}
__p+=' /> '+
((__t=(d.i18n("AppPagesSelectedPages")))==null?'':_.escape(__t))+
': </label>\r\n			<input type="text" name="selectedPages" ';
 if(d.range!==1){ 
__p+=' disabled="disabled" ';
}
__p+=' />\r\n		</li>\r\n		<li><label><input type="radio" name="pageRange" value="3" ';
 if(d.range===3){ 
__p+=' checked ';
}
__p+=' /> '+
((__t=(d.approveRemaining?d.i18n("AppPagesRemPages"):d.i18n("AppPagesAllPages")))==null?'':_.escape(__t))+
'</label></li>\r\n	</ul>\r\n</div>';
return __p;
}});
/**
 * @namespace ApprovalModule
 */
define('web-viewer/approval/ApprovalModule',["web-viewer/core/AbstractModule",
		"web-viewer/approval/ApprovalDataExtensions",
		"web-viewer/approval/ApprovalToolbarComponent",
		"web-viewer/approval/ApprovalMobileToolbarComponent",
		"web-viewer/approval/ApprovalDetailsWidget",
		"Template",
		"tpl!web-viewer/approval/PageListApproval",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/compare/CompareConstants",
		"web-viewer/layout/LayoutConstants",
		"underscore"
	],
	function (AbstractModule,
		ApprovalDataExtensions,
		ApprovalToolbarComponent,
		ApprovalMobileToolbarComponent,
		ApprovalDetailsWidget,
		Template,
		tpl,
		ListenerUtil,
		CompareConst,
		LayoutConstants,
		_) {
		"use strict";

		/**
		 * Create a new instance of the approval module
		 * 
		 * @exports ApprovalModule
		 * @memberof ApprovalModule
		 * @class
		 * @augments Core.AbstractModule
		 * @constructor
		 * 
		 * @param {Core.ViewerFacade} f
		 * @param {Boolean} silenced
		 */
		var ApprovalModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "approval");
			ListenerUtil.mixin(this, f);

			this._allowApproval = true;
			this._showStatus = true;


			// start preloading language bundle as soon as module is loaded
			f.i18n.loadBundle("approval");

			// install the data extensions
			ApprovalDataExtensions(f);

			// publish approval buttons toolbar component 
			this._publishToolbarComponent(ApprovalToolbarComponent.toolbarName, _.bind(function (layoutType) {
				if (layoutType === LayoutConstants.LayoutType.MOBILE) {
					return new ApprovalMobileToolbarComponent(this.f, this);
				} else {
					return new ApprovalToolbarComponent(this.f, this);
				}
			}, this));

			// publish details widget
			this._publishWidget(ApprovalDetailsWidget.widgetName, _.bind(function (layoutType) {
				this._approvalWidget = new ApprovalDetailsWidget(f);
				return this._approvalWidget;
			}, this));


			this.f.getDocumentController().whenActivePageChanged(_.bind(this._whenActivePageChanged, this));
			// react on compare mode changes
			this.f.on("module.compare.changed.mode", _.bind(this._update, this));
			// make sure we update the toolbar component when compare becomes available
			this.f.whenModuleAvailable("compare", this._update, this);
		};

		// Inherit from AbstractModule
		ApprovalModule.prototype = Object.create(AbstractModule.prototype);
		ApprovalModule.prototype.constructor = ApprovalModule;


		/**
		 * When the active page has changed, react on it, and execute some actions
		 */
		ApprovalModule.prototype._whenActivePageChanged = function (oldPage, newPage) {
			if (oldPage) {
				var oldDocVersion = oldPage.getDocumentVersion();
				this._unregisterAllListenersForObject(oldDocVersion);
			}

			if (newPage) {
				var newDocVersion = newPage.getDocumentVersion();

				this._registerListener("approval.loaded", this._update, this, newDocVersion);
				this._registerListener("approval.update", this._update, this, newDocVersion);

				if (newDocVersion.isPageList()) {
					// we have a page list
					this._registerListener("approval.update", function () {
						newPage.requestPageApprovalSetup(_.bind(this._update, this));
					}, this, newDocVersion);
					// request the approval for the page
					newPage.requestPageApprovalSetup(_.bind(this._update, this));
				} else {
					// request the approval setup for the docversion
					newDocVersion.requestApprovalSetup(_.bind(this._update, this));
				}

				// listen to successful approval/rejection to show a message
				this._registerListener("approval.update", function () {
					var approvalObj = newPage.getApprovalObject();
					if (approvalObj) {
						approvalObj.approveRejectCallback = _.bind(function (success, action, error) {
							if (success) {
								this.f.emit("userMessage", "success", this.f.i18n.t("approvalSuccess" + action), true);

								if (!newPage.isBlankPage()) {
									// reload annotations when approval was successful
									var annotationModule = this.f.getModule("annotation");
									if (annotationModule) {
										annotationModule.reload();
									}
								}

								// SDK support
								// First approvalcallbackurl
								var approvalCallbackUrl = this.f.getProperty("approvalcallbackurl");
								if (approvalCallbackUrl) {
									this.f.ajax(approvalCallbackUrl, {
										dataType: undefined, // intelligent guess of what will come back
										noAjaxParam: true,
										success: _.bind(this._closeOnApprove, this), // success or error of approvalcallbackurl
										error: _.bind(this._closeOnApprove, this) // we always check if we need to close on approve
									});
								} else {
									// if we have approvalCallbackUrl, we wait until that one is finished (callbacks), otherwise we just check closeOnApprove
									this._closeOnApprove();
								}
							}
							this._emit("updated");
						}, this);
					}
				}, this, newDocVersion);
			}
		};

		/**
		 * Check if the viewer needs to be closed on approve, and do so if needed
		 */
		ApprovalModule.prototype._closeOnApprove = function () {
			// Then closeonapprove
			var closeOnApprove = this.f.getProperty('closeonapprove');
			if (closeOnApprove && closeOnApprove != "0" && closeOnApprove != "false") {
				this.f.emit("viewer.request.close", this.f.getProperty("returnurl"));
			}
		};

		/**
		 * Approval info has updated (or compare module has become available or changed)
		 */
		ApprovalModule.prototype._update = function () {
			// check which things to show based on compare mode
			// get the active document

			var compareMode = null;
			var compareModule = this.f.getModule("compare");
			if (compareModule) {
				compareMode = compareModule.getActiveMode();
			}
			this._allowApproval = (!compareMode || compareMode.getName() === CompareConst.mode.SHOW_CURRENT || compareMode.getName() === CompareConst.mode.READERSPREAD);
			this._showStatus = (!compareMode || compareMode.getName() !== CompareConst.mode.SHOW_DIFFERENCE);

			this._emit("updated");
		};


		/**
		 * Check if there are unsaved annotations, and only then execute the action
		 * @param action
		 */
		ApprovalModule.prototype._checkAndExecuteAction = function (action, event) {
			var finalAction = function () {
				action(event);
			};
			if (this.f.getDocumentController().getActiveDocumentVersion().isPageList()) {
				finalAction = _.bind(this.getPageListApprovalAction, this, action, event);
			}


			if (this._allowApproval) {
				var annotModule = this.f.getModule("annotation");
				if (annotModule) {
					var hasUnsavedAnnotations = annotModule.promptToSaveUnsavedAnnotations(finalAction, function () {
						// when the user cancel saving unsaved annotations we do not want to do anything...
					}, _.bind(function () {
						annotModule.reload(); //reload all the annotations, they are not saved
						finalAction();
					}, this));
					if (!hasUnsavedAnnotations) { //no prompt is shown as there are no unsaved annotations
						finalAction(); //do the final actions
					}
				} else {
					finalAction();
				}
			} else {
				// approval is not allowed, show alert
				this.f.loadBundle("viewer", function () {
					this.f.emit("userMessage", "warn", this.f.t("AppDiffModeError"));
				}, this);
			}

		};

		/**
		 * Get the list of available actions
		 * @returns {Array.<Object>} with for each action (name, action, text)
		 */
		ApprovalModule.prototype.getAvailableActions = function () {
			var page = this.f.getActivePage();
			var actions = page.getApprovalActions();

			_.each(actions.list, function (action) {
				action.action = _.bind(this._checkAndExecuteAction, this, action.action);
			}, this);

			return actions;
		};

		/**
		 * Is approval allowed?
		 * @returns {Boolean}
		 */
		ApprovalModule.prototype.allowApproval = function () {
			return this._allowApproval;
		};

		/**
		 * Should the approval status be shown?
		 * @returns {Boolean}
		 */
		ApprovalModule.prototype.showStatus = function () {
			return this._showStatus;
		};


		/**
		 * Extend the approval action with extra parameters for page lists
		 * @param action
		 * @param event
		 * @returns the modified action
		 */
		ApprovalModule.prototype.getPageListApprovalAction = function (action, event) {
			//check whether we want to approve this page or remaining pages:
			var approveRemaining = false;
			if (event.target.getAttribute("class") && event.target.getAttribute("class").indexOf("Remaining") > -1) {
				approveRemaining = true;
			}

			var page = this.f.getDocumentController().getActivePage();
			var approvalObj = page.getApprovalObject();
			var widgetContext = $("<div/>");
			this.t = new Template(this.f.i18n, tpl);
			// set default value: 'current' when page not approved; 'all' when already approved (approve remaining pages)
			this.t.d.range = approveRemaining ? 3 : 0;
			this.t.d.approveRemaining = approveRemaining;
			this.t.setContext(widgetContext);

			// add some listeners to make the panel interactive
			this.t.bind("change", "[name=pageRange]", function (source, event) {
				this.t.d.range = parseInt(source.value, 10);
				if (this.t.d.range === 2) {
					widgetContext.find("[name=selectedPages]").removeAttr("disabled");
				} else {
					widgetContext.find("[name=selectedPages]").attr("disabled", "disabled");
				}
			}, this);

			this.t.bind("change", "[name=selectedPages]", function (source, event) {
				this.t.d.selectedRange = source.value;
			}, this);

			// rerender if marked pages have changed
			var renderPopup = _.bind(function () {
				// only render if visible (popup/visibility defined by complexapproval.js)
				if (widgetContext.css("visibility") !== 'hidden') {
					var pagelistModule = this.f.getModule("pagelist");
					if (pagelistModule) {
						this.t.d.markedPages = pagelistModule.getAllMarkedPagesAsString();
					}
					this.t.render();
				}
			}, this);

			// register listener on marked pages changed
			if (this.pageListApprovalInitialized === undefined) {
				this.f.on("doc.**.pagelistMarked.changed", renderPopup);
				this.pageListApprovalInitialized = true;
			}

			approvalObj.setExtraContent(widgetContext);
			renderPopup();

			approvalObj.setBeforeSubmitAction(_.bind(function (approvalObj) {
				approvalObj.forceStep(undefined);
				switch (this.t.d.range) {
				case 0:
					approvalObj.setPageRange(page.get("pageindex"));
					break;
				case 1:
					approvalObj.setPageRange(this.t.d.markedPages, true, true);
					break;
				case 2:
					approvalObj.setPageRange(this.t.d.selectedRange, true, true);
					break;
				case 3:
					approvalObj.setPageRange("all");
					break;
				}
			}, this, approvalObj));

			// Do the real approve/reject action
			action(event);
		};

		/**
		 * Get the approval details widget
		 * @returns {Approval.ApprovalDetailsWidget}
		 */
		ApprovalModule.prototype.getApprovalWidget = function () {
			return this._approvalWidget;
		};

		return ApprovalModule;
	});
