define(["component/toolbar/ToolbarComponent",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/utils/ListenerUtil",
		"underscore"
	],
	function (ToolbarComponent,
		ButtonGroup,
		ViewerToolbarButton,
		LayoutConstants,
		ListenerUtil,
		_
	) {
		/**
		 * Toolbar component for approval
		 *
		 * @exports ApprovalToolbarComponent
		 * @memberof ApprovalModule
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Approval.ApprovalModule} module
		 */
		var ApprovalToolbarComponent = function (f, approvalModule) {
			ToolbarComponent.call(this, LayoutConstants.ToolbarComponent.APPROVAL, f);
			ListenerUtil.mixin(this, f);

			this.f = f;
			this._approvalModule = approvalModule;
			this._registerListener("updated", this.updateComponent, this, this._approvalModule);
			// start by displaying loading message
			this.currentState = "Loading";

			this.status = $("<div/>").addClass("Loading approvalBtn").text("Loading...");
			this.statusIndicator = $("<div/>").addClass("btn-group btnGroupWithTitle").append(this.status);

			f.i18n.loadBundle("viewer", function () {
				this.statusIndicator.append($("<div/>").addClass("btnGroupTitle").text(f.i18n.t("ApprovalBtnGroupTitle")));
			}, this);


			this.element.append(this.statusIndicator).addClass("approvalControls");
			this.buttonGroup = new ButtonGroup("approval");
			this.buttonGroup.setTitle(f.i18n, "viewer", "ApprovalBtnGroupTitle");
			this.addButtonGroup(this.buttonGroup);
			this.buttonGroup.element.hide();

			this.updateComponent();
		};

		ApprovalToolbarComponent.prototype = Object.create(ToolbarComponent.prototype);
		ApprovalToolbarComponent.prototype.constructor = ApprovalToolbarComponent;
		ApprovalToolbarComponent.toolbarName = LayoutConstants.ToolbarComponent.APPROVAL;


		ApprovalToolbarComponent.prototype.setStatus = function (status) {
			this.status.toggleClass(this.currentState, false);

			// hide the component if no approval defined
			if (status !== "NotDefined") {
				this.show();
			} else {
				this.hide();
			}
			this.currentState = status;
			this.status.toggleClass(this.currentState, true);

			this.f.i18n.loadBundle("approval", function () {
				this.status.empty();
				this.status
					.append($("<span/>").addClass("enableTextSelection").text(this.f.i18n.t("CApp" + this.currentState)))
					.prepend($("<i class='icon-normal approvalStatus'/>").addClass(this.currentState));
			}, this);
		};

		/**
		 * Updates the toolbar component when the approval setup or compare mode has changed
		 */
		ApprovalToolbarComponent.prototype.updateComponent = function () {
			// get the active document
			var docController = this.f.getDocumentController();
			var page = docController.getActivePage();


			if (page) {
				this.show();
				this.setStatus("Loading");
				this.toggleButtons(false);


				this.setStatus(page.getApprovalStatus());


				// clear buttons
				this.buttonGroup.clearButtons();

				// add buttons			
				var actions = this._approvalModule.getAvailableActions();
				for (var i = 0; i < actions.list.length; i++) {
					var action = actions.list[i];

					var button = new ViewerToolbarButton(this.f, action.name, action.name, true, action.action);
					button.element.addClass(action.name);
					button.setTooltip(action.text);

					this.buttonGroup.addButton(button);
				}
				this.toggleButtons(actions.list.length > 0);

				// hide if there is nothing to show
				if (actions.list.length === 0 && !this._approvalModule.showStatus()) {
					this.hide();
				}
			} else {
				// no document active
				this.buttonGroup.clearButtons();
				this.buttonGroup.hide();
				this.hide();
			}

			this._emit("changed.content");
		};

		/**
		 * Toggle whether buttons should be visible or not
		 * @param show
		 */
		ApprovalToolbarComponent.prototype.toggleButtons = function (show) {
			this.buttonGroup.element.toggle(show);
			this.statusIndicator.toggle(!show);
		};


		return ApprovalToolbarComponent;
	});
