define(["web-viewer/webcenter/WebCenterDocumentPage",
		"web-viewer/utils/ViewerUtils",
		"utils/Cache",
		"attributecategory/AttributeCategory",
		"underscore"
	],
	function (WebCenterDocumentPage,
		ViewerUtils,
		Cache,
		AttributeCategory,
		_) {
		"use strict";
		var ATTRIBUTE_DATA_KEY = "attributeCategory";


		/**
		 * AttributeCategoryDataExtensions install methods to WebCenterDocumentPage to manage an attributeCategory for the page
		 *
		 * @exports AttributeCategoryDataExtensions
		 * @memberof AttributeCategoryModule
		 * @param   {ViewerFacade} f [[Description]]
		 */
		var AttributeCategoryDataExtensions = function (f) {

			// Cache used for retrieving the groups and roles a user is assigned to in a project
			var userGroupRolesCache = new Cache(20, true);
			userGroupRolesCache._loadItem = function (projectID) {
				var url = f.getDataAdapter().WebCenter("loadrolesandgroupsforuser.wcr", {
					projectid: projectID
				});

				f.ajax(url, {
					dataType: "JSON",
					success: function (data) {
						if (data && data.result && data.result.rolesAndGroups) {
							// convert to Uppercase for consistency
							var rolesAndGroups = _.map(data.result.rolesAndGroups, function (roleGroup) {
								return roleGroup.toUpperCase();
							});
							userGroupRolesCache._registerItem(projectID, rolesAndGroups);
						} else {
							// we couldn't load usergroupsroles for this project
							userGroupRolesCache._registerItem(projectID);
						}
					},
					error: function () {
						// we couldn't load usergroupsroles for this project
						userGroupRolesCache._registerItem(projectID);
					}
				});
			};


			/**
			 * Get the Data Object for the AttributeCategoryDataExtensions
			 * @returns {Object} data-object, containing the attribute category status, object and DOM elements
			 */
			WebCenterDocumentPage.prototype._getAttributeCategoryData = function () {
				return this.get(ATTRIBUTE_DATA_KEY);
			};

			/**
			 * Execute a callback when the AttributeCategory is loaded (its structure is available, and views can be requested)
			 * @param {Function} callback - callback callend when the attribute category is available, or loading failed. First parameters indicates whether load was successfull. Second parameter can be an errorstring if something failed
			 * @param {Object}   [binding]  - binding for the callback
			 */
			WebCenterDocumentPage.prototype.whenAttributeCategoryAvailable = function (callback, binding) {
				if (binding) {
					callback = _.bind(callback, binding);
				}

				if (!this.hasAttributeCategory()) {
					// No attribute category is defined for this page
					callback(false);
				} else {
					var data = this._getAttributeCategoryData();
					if (data && data.loaded) {
						// The attribute category is already loaded and available, execute the callback
						callback(data.object);
					} else {
						// The category should be initialized, or is still loading
						// register the callback
						this.once("attributeCategory.loaded", callback);
						if (!data || !data.loading) {
							// the category is not yet initialized, initialize and start loading the category
							this._initializeAttributeCategory();
						}
					}
				}
			};

			/**
			 * Bootstrap and initialize the Attribute Category
			 */
			WebCenterDocumentPage.prototype._initializeAttributeCategory = function () {
				// load language strings
				f.loadBundle("attributeCategory", function (bundle) {
					// create the data exetensions data object
					var data = {};


					var document = this.getDocument();
					var docVersionID = this.getDocumentVersion().getDocumentVersionID();

					// generate a unique ID for storing the attribute category in the DOM
					data.id = ViewerUtils.generateUniqueDOMId("aCatObj_");

					// make sure the values are not yet loaded (they should only be fetched when there is a custom Viewer view
					var aCatOptions = {
						delayedLoading: true
					};

					// set the ID for the attribute storage
					aCatOptions.ACatObj = data.id;
					aCatOptions.ACatPanel = data.id + "_panel";

					// create the context div (in this the attribtue category panel, form and iframe to submit will be placed)
					data.contextDiv = $("<div>").hide();

					// iframe for submitting category
					data.iframe = $("<iframe>")
						.attr("name", aCatOptions.ACatObj + "_iframe")
						.css({
							height: 1,
							width: 1,
							position: "absolute",
							top: "-90000px",
							left: "-10000px"
						})
						.appendTo(data.contextDiv);


					// form for submitting category
					data.form = $("<form>")
						.attr("name", "actionform")
						.attr("method", "post")
						.attr("enctype", "multipart/form-data")
						.attr("target", aCatOptions.ACatObj + "_iframe")
						.attr("action", this.f.getDataAdapter().WebCenter("dodocdetailsactions.jsp"))
						.appendTo(data.contextDiv);
					data.form.append($("<input>")
						.attr("type", "hidden")
						.attr("name", "actionVal")
						.val("SaveAttributesEmbedded"));
					data.form.append($("<input>")
						.attr("type", "hidden")
						.attr("name", "docVerID")
						.val(docVersionID));

					// panel div for rendering category
					data.panelDiv = $("<div>")
						.attr("id", aCatOptions.ACatPanel)
						.appendTo(data.form);

					data.contextDiv.appendTo(f.getContext());

					// global object to store category in it
					data.objectDiv = $("<div>").attr("id", aCatOptions.ACatObj).hide();
					data.objectDiv.appendTo(f.getContext());


					// register some callbacks
					aCatOptions.callbackOnLoad = _.bind(this._attributeCategoryFireOnLoad, this);
					aCatOptions.callbackOnFail = _.bind(this._attributeCategoryFireOnFail, this);
					aCatOptions.callbackOnRender = _.bind(this._attributeCategoryFireOnRender, this);

					// we are requesting the category for a document
					aCatOptions.type = "document";
					aCatOptions.projectID = document.getProjectID();
					aCatOptions.documentID = document.getDocumentID();
					aCatOptions.docVersionID = docVersionID;
					if (aCatOptions.docVersionID) {
						// to be used to detect cyclic dependencies in document references
						aCatOptions.docTableParentDocVerIDs = [docVersionID];
					}

					// set the language strings
					aCatOptions.lanugageStrings = bundle;

					aCatOptions.currentUser = _.clone(window.gCurrentUser);
					aCatOptions.currentUser.canEditAttributes = this.canEditAttributes();

					// url for fetching the values
					aCatOptions.valuesURL = f.getDataAdapter().WebCenter("GetAttributes.jsp", {
						docverid: docVersionID
					});

					// indicate that the category is being loaded
					data.loading = true;

					// initialize (not we didn't  provide the readURL option, so the category won't start doing anything yet)
					data.object = new AttributeCategory(aCatOptions);
					data.object.valueChangedListeners.push(_.bind(this._attributeCategoryFireOnChange, this));

					// store the data extensions data object
					this.set(ATTRIBUTE_DATA_KEY, data);

					// start loading category
					data.object.readACatFromXML(f.getDataAdapter().WebCenter("GetAttributeCategoryXML.jsp", {
						categoryid: this.getAttributeCategoryID()
					}));
				}, this);

			};

			/**
			 * Get the attributecategory object for this page
			 * @returns {AttributeCategory}
			 */
			WebCenterDocumentPage.prototype.getAttributeCategory = function () {
				var data = this._getAttributeCategoryData();
				if (data) {
					return data.object;
				}

				// no attribute category available
				return;
			};

			/**
			 * Fire onload event
			 */
			WebCenterDocumentPage.prototype._attributeCategoryFireOnLoad = function () {
				var data = this._getAttributeCategoryData();
				data.loading = false;
				data.loaded = true;
				this._emit("attributeCategory.loaded", true);
			};

			/**
			 * Fire on fail event
			 */
			WebCenterDocumentPage.prototype._attributeCategoryFireOnFail = function (error) {
				this._emit("attributeCategory.loaded", false, error);
			};

			/**
			 * Fire the render event
			 */
			WebCenterDocumentPage.prototype._attributeCategoryFireOnRender = function () {
				this._emit("attributeCategory.render");
			};

			/**
			 * Fire the change event
			 */
			WebCenterDocumentPage.prototype._attributeCategoryFireOnChange = function () {
				this._emit("attributeCategory.change");
			};
			
			/**
			 * Fire the modal popup closed event
			 */
			WebCenterDocumentPage.prototype.attributeCategoryFireModalPopupClosed = function () {
				this._emit("attributeCategory.modalPopupClosed");
			};

			/**
			 * Fire a save finished event
			 * @param {Boolean}       success - is the save operation successful?
			 * @param {String|Object} error   - error information
			 */
			WebCenterDocumentPage.prototype._attributeCategoryFireOnSaveFinished = function (success, error) {
				this._emit("attributeCategory.saveFinished", success, error);
			};

			/**
			 * Get the panel (with form and ifrmae to submit) for display the attribute category
			 * @returns {JQueryObj} a JQuery reference to the DOM element in which the attribtue category panel, form and iframe sit
			 */
			WebCenterDocumentPage.prototype.getAttributeCategoryPanel = function () {
				var data = this._getAttributeCategoryData();
				if (data && data.loaded) {
					return data.contextDiv;
				}
			};


			/**
			 * This is a helper function to make sure all dates are set correctly when saving the category
			 * NOTE: this is legacy WCR code, but it seems to work
			 * JANH NOTE: It doesn't work for any prefixed attribute category! (project creation page,
			 * 		task execution page, all document references) But it seems not needed at those places...
			 *
			 * @param   {Object}  thisForm - form of the attribute category
			 * @returns {Boolean}
			 */
			function assignDate(thisForm) {
				var i, cobj, dateElementName, dElement, setValueOjbName, setValueOjb, isRS;
				for (i = 0; i < thisForm.elements.length; ++i) {
					cobj = thisForm.elements[i];

					if (cobj.tagName == 'INPUT' &&
						(cobj.type == 'text' || cobj.type == 'hidden') &&
						cobj.name.substring(0, 5) == 'adate') {
						dateElementName = cobj.name.substring(5);
						dElement = document.getElementById(dateElementName);
						dElement.value = cobj.value;
					}

					if ((cobj.tagName == 'INPUT' && (cobj.type == 'text' || cobj.type == 'hidden') &&
							cobj.name.substring(0, 2) == 'UN' &&
							cobj.value == '00001_0000000020')) {
						isRS = document.getElementById("isRS" + cobj.name.substring(2));
						if ((isRS != null && isRS.value != "1") || (isRS == null)) {
							// not a restricted set
							dElement = document.getElementById(cobj.name.substring(2));
							setValueOjbName = "adate" + dElement.name;
							setValueOjb = document.getElementById(setValueOjbName);

							dElement.value = setValueOjb.value;
						}
					}
				}
				return true;
			}

			/**
			 * Save the attributes
			 * @param {Function} callback - callback function (first parameter: success, second parameter: error string if any)
			 */
			WebCenterDocumentPage.prototype.saveAttributes = function (callback) {
				var data = this._getAttributeCategoryData();
				// make sure no save operation is ongoing
				if (data && !data.saving && data.loaded && this.canEditAttributes()) {
					data.saving = true;

					// notify that things are changing
					this._attributeCategoryFireOnChange();

					// register listener
					this.once("attributeCategory.saveFinished", callback);

					// make sure the dates are set correctly
					assignDate(data.form.get(0));

					// validate and prepare the attribute category for submit
					if (data.object.prepareForSubmit(false, true)) {
						var iframe = data.iframe.get(0);
						// register onload on the iframe
						iframe.onload = _.bind(function () {
							data.saving = false;

							// the iframe has loaded, handle result
							try {
								if (iframe.contentDocument) {
									if (iframe.contentDocument.getElementsByTagName("ok").length > 0) {
										data.object.attributesChanged = false;
										// save operation was successful

										var attributeCategory = this.getAttributeCategory();
										if (attributeCategory && attributeCategory.hasActiveViewDocReference()) {
											this.f.getDataAdapter().updateDocumentData(this.f.getActivePage());
										}

										this._attributeCategoryFireOnSaveFinished(true);
									} else if (iframe.contentDocument.getElementsByTagName("error").length > 0) {
										// an errr occurred, parse all errors
										var error = "";
										var errorTags = iframe.contentDocument.getElementsByTagName("error");

										for (var i = 0; i < errorTags.length; i++) {
											if (i > 0) {
												error += ", ";
											}
											error += errorTags[i].textContent;
										}
										this._attributeCategoryFireOnSaveFinished(false, error);
									}
								}
							} catch (err) {
								this._attributeCategoryFireOnSaveFinished(false, err);
							}
							this._attributeCategoryFireOnChange();
						}, this);

						// submit the form in the iframe
						data.form.submit();

					} else {
						// attribute category is not valid, user choose not to save, so don't call callback for any error... The attribute category will give alert
						data.saving = false;
						this._attributeCategoryFireOnChange();
					}
				} else {
					// save operation is already ongoing
					data.saving = false;
					callback(false, this.f.t("AttrSaveErrorOngoing"));
					this._attributeCategoryFireOnChange();
				}
			};

			/**
			 * Can the user save attributes? (is the category loaded and does he have edit rights)
			 * @returns {Boolean}
			 */
			WebCenterDocumentPage.prototype.canSaveAttributes = function () {
				var data = this._getAttributeCategoryData();
				return data && this.canEditAttributes();
			};

			/**
			 * Is a save operation ongoing?
			 * @returns {Boolean}
			 */
			WebCenterDocumentPage.prototype.isAttributeCategorySaveOngoing = function () {
				var data = this._getAttributeCategoryData();
				return data && data.saving;
			};

			/**
			 * Are there any unsaved changes on the attribute category
			 * @returns {Boolean} [[Description]]
			 */
			WebCenterDocumentPage.prototype.hasUnsavedAttributeChanges = function () {
				var data = this._getAttributeCategoryData();

				if (data && data.loaded && this.canEditAttributes()) {
					return data.object.attributesChanged;
				}

				return false;
			};

			/**
			 * Is there an attribute category defined on the page
			 * @returns {Boolean}
			 */
			WebCenterDocumentPage.prototype.hasAttributeCategory = function () {
				return this.getAttributeCategoryID() && true;
			};

			/**
			 * Get a set of avialable Roles and Groups that have a view in the category
			 * @param {Function} callback - parameter of the callback is an object with a properties "viewerView" [contains the name of the Viewer view, or undefined if non-existant], "roleSpecificViews" [keys: available roles and values the attribute category view assigned to that role/group], "all": [array of all views]
			 * @example
			 *    {
			 *        viewerView: "Viewer",
			 *        roleSpecificViews: {
			 *             GROUP: "Viewer Group",
			 *             "SOME ROLE": "Viewer Some ROLE"
			 *        },
			 *        all: ["Viewer", "Viewer Group", "Viewer Some ROLE", "Default" , ...]
			 *    }
			 */
			WebCenterDocumentPage.prototype.whenAttributeCategoryViewsAvailable = function (callback) {
				var data = this._getAttributeCategoryData();
				if (data && data.loaded) {

					// check if we already did this operation before
					if (data.views && data.views.loaded) {
						// execute callback, we have the needed data
						callback(data.views);
						return;
					} else {

						// register listener
						this.once("attributeCategory.viewsAvailable", callback);

						if (!data.views) {
							data.views = {};
							data.views.viewerView = null;
							data.views.all = data.object.getViewNames();
							data.views.roleSpecificViews = {};
							data.views.loaded = false;

							// make a list of all attribute category views that could match a Role or Group
							// and detect if the Viewer view is present
							var roleViews = {};
							_.each(data.views.all, function (viewName) {
								var upperCaseView = viewName.toUpperCase();
								if (upperCaseView == "VIEWER") {
									// we found the viewer view
									data.views.viewerView = viewName;
								} else if (upperCaseView.indexOf("VIEWER") === 0) {
									// we found an attribute category view that could match a Role or Group
									roleViews[viewName.toUpperCase().replace(/^VIEWER\s*/, "")] = viewName;
								}
							});


							if (_.size(roleViews) > 0) {
								// there are one or more attribute category views that could mat a role or group, fetch the roles and groups for the active user in the project of this file
								userGroupRolesCache.get(this.getDocument().getProjectID(), _.bind(function (userGroupRoles) {
									data.views.roleSpecificViews = {};
									// run through the roles/groups the user is assigned to, if a match is found with the attribute category views, add it to the available views
									_.each(userGroupRoles, function (groupRole) {
										if (roleViews[groupRole]) {
											data.views.roleSpecificViews[groupRole] = roleViews[groupRole];
										}
									});
									data.views.loaded = true;
									// we have everything we need, execute the callback
									this._emit("attributeCategory.viewsAvailable", data.views);
								}, this));
							} else {
								data.views.loaded = true;
								// no custom views are set, just execute the callback
								this._emit("attributeCategory.viewsAvailable", data.views);
							}

						}
					}
				}
			};

			/**
			 * Is the "Viewer" view defined for this page
			 * @returns {Boolean}
			 */
			WebCenterDocumentPage.prototype.hasViewerView = function () {
				var data = this._getAttributeCategoryData();
				return data && data.views.viewerView;
			};

			/**
			 * Get the name of the Viewer view
			 * @returns {String} name of the viewer view
			 */
			WebCenterDocumentPage.prototype.getViewerView = function () {
				var data = this._getAttributeCategoryData();
				if (data) {
					return data.views.viewerView;
				}
			};

			/**
			 * Get all the views defined on the attribute category of the document
			 * if no views are defined, or the views are not yet loaded, an empty array is returned
			 * @returns {Array.<String>} list of available views
			 */
			WebCenterDocumentPage.prototype.getAllAttributeCategoryViews = function () {
				var data = this._getAttributeCategoryData();
				if (data && data.views) {
					return data.views.all;
				} else {
					return [];
				}
			};

			/**
			 * Reset the attribute category (this destroys the existing data and reloads/reinitializes the category
			 * @param {Function} callback - called when finished
			 * @param {Object}   [binding]  - binding for the callback
			 */
			WebCenterDocumentPage.prototype.resetAttributeCategory = function (callback, binding) {
				var oldData = this._getAttributeCategoryData();

				if (binding) {
					callback = _.bind(callback, binding);
				}

				if (oldData && oldData.object) {
					// remove TinyMCE editors
					oldData.object.destroy();

				}


				// reset the data
				this.set(ATTRIBUTE_DATA_KEY);

				// load category
				this.whenAttributeCategoryAvailable(function () {
					if (oldData) {
						if (oldData.objectDiv) {
							oldData.objectDiv.remove();
						}
						if (oldData.contextDiv) {
							oldData.contextDiv.remove();
						}
					}

					callback();
				});

			};
		};

		return AttributeCategoryDataExtensions;
	});