define(["web-viewer/layout/AbstractWidget",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/layout/component/MiniButton",
		"web-viewer/layout/component/DropdownMiniButton",
		"underscore",
		"jquery"
	],
	function (AbstractWidget,
		LayoutConstants,
		ListenerUtil,
		MiniButton,
		DropdownMiniButton,
		_,
		$) {
		"use strict";

		/**
		 * The AttributeCategoryWidget shows the attribute category for the active document
		 * 
		 * @exports AttributeCategoryWidget
		 * @augments Layout.AbstractWidget
		 * @class The AttributeCategoryWidget shows the attribute category for the active docmetn
		 * @constructor
		 * @mixes {Utils.ListenerUtilMixin}
		 * @param {Core.ViewerFacade} f - viewerFacade
		 */
		var AttributeCategoryWidget = function AttributeCategoryWidget(f, attributeCategorModule) {
			AbstractWidget.call(this, f, LayoutConstants.Widget.ATTRIBUTE_CATEGORY);
			ListenerUtil.mixin(this, f);

			/**
			 * @member {AbstractDocumentPage} the page for which the attributes are shown
			 */
			this._page = null;

			/**
			 * @member {JQueryObj} the DOM element containing all DOM elements for the attributes
			 */
			this._panel = null;

			// intialize Save button
			this._saveButton = new MiniButton(this.f, "saveAttributes", "icon-WCR_Save", true, _.bind(function () {
				this._emit("saveAttributes");
			}, this), "fontIconDisabled icon-WCR_Save");
			this._saveButton.element.addClass("displayNone");
			this._saveButton.setTooltip("viewer", "AttrSaveAttributes");

			// initialize view swichter
			this._viewSwitcher = new DropdownMiniButton(this.f, "viewSwitcher", "icon-WCR_User", ["bottom", "top"]);
			this._viewSwitcher.element.addClass("displayNone");
			this._viewSwitcher.setTooltip("viewer", "AttrSwitchView");




			// settitle
			this.f.loadBundle("viewer", function () {
				this.setTitle(this.f.t("AttributeCategoryWidgetTitle"));
			}, this);

			this.setIcon("icon icon-WCR_Attributes");


			// load CSS to display attributes correctly in the viewer
			this._loadCss = _.once(function () {
				// wrap popitmenu so it is displayed correctly
				$("#popitmenu").wrap($("<div>").addClass("wcr"));

			});
		};

		AttributeCategoryWidget.prototype = Object.create(AbstractWidget.prototype);
		AttributeCategoryWidget.prototype.constructor = AttributeCategoryWidget;
		AttributeCategoryWidget.widgetName = LayoutConstants.Widget.ATTRIBUTE_CATEGORY;



		/**
		 * @override
		 */
		AttributeCategoryWidget.prototype.setContext = function (context, headerExtra) {
			AbstractWidget.prototype.setContext.call(this, context, headerExtra);

			this.context.addClass("enableTextSelection");

			headerExtra.append(this._saveButton.element);
			headerExtra.append(this._viewSwitcher.element);

			if (this._panel) {
				this.context.append(this._panel.show()).addClass("wcr");
			}

			this._loadCss();
		};

		/**
		 * Set the page for the widget
		 * @param {AbstractDocumentPage} [page]            - The page for which the attribute category is shown. If null, the current attribtue category is removed from the widget
		 * @param {String}               [activeGroupRole] - Name of the currently active group/role view
		 * @param {Array.<String>}       [availableViews]  - List of available roles/groups for display the attribute category custom views   
		 */
		AttributeCategoryWidget.prototype.setPage = function (page, activeGroupRole, availableViews) {
			var panel;

			if (page) {
				panel = page.getAttributeCategoryPanel();
			}


			if (this._page && this._page !== page) {
				// remove listeners from the current page
				this._unregisterAllListenersForObject(this._page);
				// move the attribute category context out of the widget
				this.f.getContext().append(this._panel.hide());
			}

			this._page = page;
			this._panel = panel;

			if (this._page) {
				// listen to change and render events of the attribute category
				this._registerListener("attributeCategory.render", this._fireResized, this, this._page);
				this._registerListener("attributeCategory.change", this._updateSaveButton, this, this._page);
			}

			// clear the view switcher
			this._viewSwitcher.clear();
			if (this._page && availableViews && availableViews.length > 1) {
				// there is more than 1 views available, initialize and display the view switcher
				var selectGroupRole = _.bind(function (groupRole) {
					this._emit("changedGroupRole", groupRole);
				}, this);

				// add available views
				this._groupRoleOptions = [];
				_.each(availableViews, function (groupRole) {
					this._viewSwitcher.addOption({
						name: groupRole,
						content: groupRole,
						enabled: true,
						selected: groupRole == activeGroupRole,
						onchange: _.partial(selectGroupRole, groupRole)
					});
					this._groupRoleOptions.push(groupRole);
				}, this);

				this._viewSwitcher.element.removeClass("displayNone");
			} else {
				// hide the view swithcer
				this._viewSwitcher.element.addClass("displayNone");
			}

			if (this.context && this._panel) {
				// append the attribute category panel to the widget
				this.context.append(this._panel.show()).addClass("wcr");
				this._fireResized();
			}

			// render the save button correctly
			this._updateSaveButton();
		};

		/**
		 * Update the view selector dropdown
		 * @param {String} activeGroupRole - the currently active Group/Role
		 */
		AttributeCategoryWidget.prototype.updateGroupRoleSelection = function (activeGroupRole) {
			_.each(this._groupRoleOptions, function (groupRole) {
				this._viewSwitcher.toggleOption(groupRole, false);
			}, this);
			this._viewSwitcher.toggleOption(activeGroupRole, true);
		};

		/**
		 * Update the rendering of the save button
		 */
		AttributeCategoryWidget.prototype._updateSaveButton = function () {
			if (this._page) {
				if (this._page.canSaveAttributes()) {
					// the user can change attributes, show the button and indicate whether any changes are made
					this._saveButton.element.removeClass("displayNone");
					// while saving the button should be disabled
					this._saveButton.setEnabled(!this._page.isAttributeCategorySaveOngoing() && this._page.hasUnsavedAttributeChanges());
					this._saveButton.setBadge(this._page.hasUnsavedAttributeChanges() ? "!" : "");
				} else {
					// hide the save button, the user can't edit attributes
					this._saveButton.element.addClass("displayNone");
				}
			} else {
				// hide the save button, no page is defined
				this._saveButton.element.addClass("displayNone");
			}
		};

		/**
		 * When the widget is being moved, we need to make sure to disable the tinyMCE fields
		 * This method is called before the widget is being moved
		 * @override
		 */
		AttributeCategoryWidget.prototype.movingStarted = function () {
			AbstractWidget.prototype.movingStarted.apply(this, arguments);

			if (this._page) {
				var attributeCategory = this._page.getAttributeCategory();
				if (attributeCategory) {
					attributeCategory.beforeMove();
				}
			}
		};

		/**
		 * When the widget is being moved, we need to make sure to disable the tinyMCE fields
		 * This method is called after the widget was moved, reintialize the tinyMCE instances
		 * @override
		 */
		AttributeCategoryWidget.prototype.movingStopped = function () {
			AbstractWidget.prototype.movingStopped.apply(this, arguments);

			if (this._page) {
				var attributeCategory = this._page.getAttributeCategory();
				if (attributeCategory) {
					attributeCategory.afterMove();
				}
			}
		};

		/**
		 * @override
		 */
		AttributeCategoryWidget.prototype.willFill = function () {
			return true;
		};

		return AttributeCategoryWidget;
	});