define(["web-viewer/webcenter/WebCenterDocumentPage",
		"web-viewer/data/AbstractDocumentPage",
		"utils/ShapeUtils",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/cad/CadFile",
		"web-viewer/cad/CadStyle",
		"underscore"
	],
	function (WebCenterDocumentPage,
		AbstractDocumentPage,
		ShapeUtils,
		ViewerUtils,
		CadFile,
		CadStyle,
		_) {
		"use strict";

		/**
		 * Inject cad functionality in the data model
		 * The AbstractDocumentPage prototype gets extended with methods for cad
		 * The cad module only supports the WebCenter data adapter (in the future, this implementation here can be changed to work together with other data adapters to)
		 *
		 * @exports CadDataExtensions
		 * @memberof CadModule
		 * @param {Core.ViewerFacade} f
		 */
		var CadDataExtensions = function (f) {
			var dataAdapter = f.getDataAdapter();

			// only the WebCenter data-adapter is supported
			if (!dataAdapter.WebCenter) {
				f.emit("error.SEVERE", "Can't initialize Cad on a non-webcenter data-adapter");
				return;
			}

			/**
			 * Initialize the page with needed data object, so the cad module can work correctly
			 * @private
			 * @returns {Object} CadData object
			 */
			AbstractDocumentPage.prototype._initCadModule = function () {
				var cadData = this.get("cadData");
				if (!cadData) {
					cadData = {
						loaded: false
					};

					this.set("cadData", cadData);
				}
				return cadData;
			};

			/**
			 * Load the cad information for this page
			 * If the information is already available, it won't be reloaded (use reloadCad if this is needed)
			 *
			 * @param {Function} onCadAvailable (first argument indicates whether the cad is loaded successfully or not
			 */
			AbstractDocumentPage.prototype.requestCad = function (onCadAvailable) {
				this.whenParametersAvailable(function () {
					onCadAvailable(this.initializeCadFile());
				}, this);
			};

			/**
			 * Parse the raw cad data and style and store it in the page
			 * @returns {Boolean} true if successful
			 */
			AbstractDocumentPage.prototype.initializeCadFile = function () {
				var cadData = this._initCadModule();
				if (this.hasCad() && this.isParametersAvailable()) {
					if (cadData) {
						if (cadData.loaded && cadData.cad) {
							// CAD is loaded and initialized
							return true;
						} else if (cadData.loaded && cadData.rawCad && cadData.rawStyle) {
							// CAD raw2 data is available, parse it and make it available
							var stylesMsg = cadData.rawStyle.getElementsByTagName('message')[0];
							var cadMsg = cadData.rawCad.getElementsByTagName('message')[0];
							if (stylesMsg || cadMsg) { // if there is a status element, the loading failed...
								var msg = null;
								if (stylesMsg) {
									msg = stylesMsg.textContent;
								}
								if (cadMsg) {
									if (msg) {
										msg += " ";
									} else {
										msg = "";
									}

									msg += stylesMsg.textContent;
								}
								cadData.loaded = false;
								this.f.debug.error("Loading CAD failed: " + msg);
								return false;
							} else {
								var style = new CadStyle(cadData.rawStyle);
								cadData.cad = new CadFile(style, cadData.rawCad);

								// rawCad and rawStyle is no longer needed
								delete cadData.rawCad;
								delete cadData.rawStyle;

								return true;
							}
						} else {
							// someting must've gone wrong, could not initialize CAD data
							return false;
						}
					}
				} else {
					// parameters are not available, or page has no CAD defined
					return false;
				}
			};

			/**
			 * Get the cad file
			 * @returns {CadFile} Cad File
			 */
			AbstractDocumentPage.prototype.getCad = function () {
				var cadData = this._initCadModule();
				return cadData.cad;
			};


			/**
			 * Get CAD offset
			 * @returns {ShapeUtils.Size} the cad offset
			 */
			AbstractDocumentPage.prototype.getCadOffset = function () {
				if (this._data.offsetx !== undefined || this._data.offsety !== undefined) {
					var offsetx = 0;
					if (this._data.offsetx !== undefined) {
						offsetx = ViewerUtils.parseFloat(this._data.offsetx);
					}

					var offsety = 0;
					if (this._data.offsety !== undefined) {
						offsety = ViewerUtils.parseFloat(this._data.offsety);
					}

					return new ShapeUtils.Size(offsetx, offsety);
				} else {
					return null;
				}
			};

			/**
			 * Is the CAD data/file available?
			 * @returns {Boolean}
			 */
			AbstractDocumentPage.prototype.isCadAvailable = function () {
				var cadData = this._initCadModule();
				if (cadData && cadData.loaded) {
					if (!cadData.cad) {
						// try parsing cad data
						this.initializeCadFile();
					}
					if (cadData.cad) {
						return true;
					}
				}

				// no CAD data found
				return false;
			};


			/**
			 * Returns the origin of the CAD-file, only if the CAD-origin is above and/or left of the graphics' (0,0)-point
			 * if this is not the case it returns (0,0) If only the original origin is requested it will return positive origins.
			 *
			 * @param {Boolean} getOriginalOrigin - returns the original origin even if it's positive
			 *
			 * @returns {ShapeUtils.Point}
			 */
			WebCenterDocumentPage.prototype.getOrigin = function (getOriginalOrigin) {
				var originX = 0;
				var originY = 0;
				if (this._parameters) {
					if (this.hasCad() && getOriginalOrigin) {
						if (this.isCadAvailable && this.isCadAvailable()) {
							return this.getCad().getOriginalOrigin();
						} else {
							return null;
						}
					}
				}
				return new ShapeUtils.Point(originX, -originY);
			};
		};


		return CadDataExtensions;
	});