
define('web-viewer/cad/CadStyle',["underscore",
		"utils/ShapeUtils",
		"utils/XMLUtils"
	],
	function (_,
		ShapeUtils,
		XMLUtils) {

		/**
		 * @exports CadColor
		 * @memberof CadModule
		 * @class A cad color describes the color definition of a color with a certain number (used in the plotting styles)
		 * @constructor
		 * @param {String} r - red value
		 * @param {String} g - green value
		 * @param {String} b - blue value
		 * @param {String} number - color number
		 */
		var CadColor = function (r, g, b, number) {
			this.r = r;
			this.g = g;
			this.b = b;
			this.number = number;
		};

		/**
		 * @exports CadPlottingStyle
		 * @memberof CadModule
		 * @class A plotting style describes how a cad line should be drawn
		 * @constructor
		 * @param {String} type
		 * @param {String} tool
		 * @param {String} style
		 * @param {String} width
		 * @param {CadColor} color
		 * @param {String} name
		 */
		var CadPlottingStyle = function (type, tool, style, width, color, name) {
			this.type = type;
			this.tool = tool;
			this.style = style;
			this.width = width;
			this.color = color;
			this.name = name;
		};

		/**
		 * @exports CadStyle
		 * @memberof CadModule
		 * @class
		 * A cad style indicates how a cad line should be drawn.
		 * It contains the color definitions (per colors, the rgb value), the plotting styles (line width, colorref, style...) and the name of a line type.
		 * This can be altered in ArtiosCad to have a different visualisation per line type
		 * @constructor
		 * @param {XMLObject} stylesxml
		 */
		var CadStyle = function (stylesxml) {

			this.colors = {};
			this.names = {};
			this.plottingStyles = {};

			/*
			 * Colors
			 */
			var colordefs = stylesxml.getElementsByTagName('ColorDef')[0].getElementsByTagName('Color');
			for (var i = 0; i < colordefs.length; i++) {
				var colordef = colordefs[i];
				var c = new CadColor(XMLUtils.attributeValue(colordef, 'red'), XMLUtils.attributeValue(colordef, 'green'), XMLUtils.attributeValue(colordef, 'blue'), XMLUtils.attributeValue(colordef, 'color'));
				this.colors[c.number] = c;
			}



			/*
			 * Names
			 */
			var namedefs = stylesxml.getElementsByTagName('NameDef')[0].getElementsByTagName('Name');
			for (var i = 0; i < namedefs.length; i++) {
				var namedef = namedefs[i];
				var name = XMLUtils.attributeValue(namedef, 'name');
				var type = XMLUtils.attributeValue(namedef, 'type');
				this.names[type] = name;
			}

			/*
			 * Plotting Styles
			 */
			var pstyles = stylesxml.getElementsByTagName('PlottingStyle')[0].getElementsByTagName('Style');
			for (var i = 0; i < pstyles.length; i++) {
				var pstyle = pstyles[i];
				var styletype = XMLUtils.attributeValue(pstyle, 'type');
				if (styletype == '*') {
					continue;
				}
				var stylecolor = XMLUtils.attributeValue(pstyle, 'color');
				var styletool = XMLUtils.attributeValue(pstyle, 'tool');
				var stylestyle = XMLUtils.attributeValue(pstyle, 'style');
				var stylewidth = XMLUtils.attributeValue(pstyle, 'width');

				var stylecolorobject = this.colors[stylecolor];
				var stylename = this.names[styletype];
				var plotstyle = new CadPlottingStyle(styletype, styletool, stylestyle, stylewidth, stylecolorobject, stylename);

				this.plottingStyles[styletype] = plotstyle;
			}
		};

		return CadStyle;
	});

define('web-viewer/cad/CadDataExtensions',["web-viewer/webcenter/WebCenterDocumentPage",
		"web-viewer/data/AbstractDocumentPage",
		"utils/ShapeUtils",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/cad/CadFile",
		"web-viewer/cad/CadStyle",
		"underscore"
	],
	function (WebCenterDocumentPage,
		AbstractDocumentPage,
		ShapeUtils,
		ViewerUtils,
		CadFile,
		CadStyle,
		_) {
		"use strict";

		/**
		 * Inject cad functionality in the data model
		 * The AbstractDocumentPage prototype gets extended with methods for cad
		 * The cad module only supports the WebCenter data adapter (in the future, this implementation here can be changed to work together with other data adapters to)
		 *
		 * @exports CadDataExtensions
		 * @memberof CadModule
		 * @param {Core.ViewerFacade} f
		 */
		var CadDataExtensions = function (f) {
			var dataAdapter = f.getDataAdapter();

			// only the WebCenter data-adapter is supported
			if (!dataAdapter.WebCenter) {
				f.emit("error.SEVERE", "Can't initialize Cad on a non-webcenter data-adapter");
				return;
			}

			/**
			 * Initialize the page with needed data object, so the cad module can work correctly
			 * @private
			 * @returns {Object} CadData object
			 */
			AbstractDocumentPage.prototype._initCadModule = function () {
				var cadData = this.get("cadData");
				if (!cadData) {
					cadData = {
						loaded: false
					};

					this.set("cadData", cadData);
				}
				return cadData;
			};

			/**
			 * Load the cad information for this page
			 * If the information is already available, it won't be reloaded (use reloadCad if this is needed)
			 *
			 * @param {Function} onCadAvailable (first argument indicates whether the cad is loaded successfully or not
			 */
			AbstractDocumentPage.prototype.requestCad = function (onCadAvailable) {
				this.whenParametersAvailable(function () {
					onCadAvailable(this.initializeCadFile());
				}, this);
			};

			/**
			 * Parse the raw cad data and style and store it in the page
			 * @returns {Boolean} true if successful
			 */
			AbstractDocumentPage.prototype.initializeCadFile = function () {
				var cadData = this._initCadModule();
				if (this.hasCad() && this.isParametersAvailable()) {
					if (cadData) {
						if (cadData.loaded && cadData.cad) {
							// CAD is loaded and initialized
							return true;
						} else if (cadData.loaded && cadData.rawCad && cadData.rawStyle) {
							// CAD raw2 data is available, parse it and make it available
							var stylesMsg = cadData.rawStyle.getElementsByTagName('message')[0];
							var cadMsg = cadData.rawCad.getElementsByTagName('message')[0];
							if (stylesMsg || cadMsg) { // if there is a status element, the loading failed...
								var msg = null;
								if (stylesMsg) {
									msg = stylesMsg.textContent;
								}
								if (cadMsg) {
									if (msg) {
										msg += " ";
									} else {
										msg = "";
									}

									msg += stylesMsg.textContent;
								}
								cadData.loaded = false;
								this.f.debug.error("Loading CAD failed: " + msg);
								return false;
							} else {
								var style = new CadStyle(cadData.rawStyle);
								cadData.cad = new CadFile(style, cadData.rawCad);

								// rawCad and rawStyle is no longer needed
								delete cadData.rawCad;
								delete cadData.rawStyle;

								return true;
							}
						} else {
							// someting must've gone wrong, could not initialize CAD data
							return false;
						}
					}
				} else {
					// parameters are not available, or page has no CAD defined
					return false;
				}
			};

			/**
			 * Get the cad file
			 * @returns {CadFile} Cad File
			 */
			AbstractDocumentPage.prototype.getCad = function () {
				var cadData = this._initCadModule();
				return cadData.cad;
			};


			/**
			 * Get CAD offset
			 * @returns {ShapeUtils.Size} the cad offset
			 */
			AbstractDocumentPage.prototype.getCadOffset = function () {
				if (this._data.offsetx !== undefined || this._data.offsety !== undefined) {
					var offsetx = 0;
					if (this._data.offsetx !== undefined) {
						offsetx = ViewerUtils.parseFloat(this._data.offsetx);
					}

					var offsety = 0;
					if (this._data.offsety !== undefined) {
						offsety = ViewerUtils.parseFloat(this._data.offsety);
					}

					return new ShapeUtils.Size(offsetx, offsety);
				} else {
					return null;
				}
			};

			/**
			 * Is the CAD data/file available?
			 * @returns {Boolean}
			 */
			AbstractDocumentPage.prototype.isCadAvailable = function () {
				var cadData = this._initCadModule();
				if (cadData && cadData.loaded) {
					if (!cadData.cad) {
						// try parsing cad data
						this.initializeCadFile();
					}
					if (cadData.cad) {
						return true;
					}
				}

				// no CAD data found
				return false;
			};


			/**
			 * Returns the origin of the CAD-file, only if the CAD-origin is above and/or left of the graphics' (0,0)-point
			 * if this is not the case it returns (0,0) If only the original origin is requested it will return positive origins.
			 *
			 * @param {Boolean} getOriginalOrigin - returns the original origin even if it's positive
			 *
			 * @returns {ShapeUtils.Point}
			 */
			WebCenterDocumentPage.prototype.getOrigin = function (getOriginalOrigin) {
				var originX = 0;
				var originY = 0;
				if (this._parameters) {
					if (this.hasCad() && getOriginalOrigin) {
						if (this.isCadAvailable && this.isCadAvailable()) {
							return this.getCad().getOriginalOrigin();
						} else {
							return null;
						}
					}
				}
				return new ShapeUtils.Point(originX, -originY);
			};
		};


		return CadDataExtensions;
	});

define('web-viewer/cad/CadLayerModel',["underscore",
		"web-viewer/viewport/model/ViewportLayerModel",
		"web-viewer/utils/LineworkWrapper",
		"web-viewer/utils/LineworkPoint",
		"utils/ShapeUtils",
		"web-viewer/utils/ViewerUtils"
	],
	function (_,
		LayerModel,
		LineworkWrapper,
		LineworkPoint,
		ShapeUtils,
		ViewerUtils) {
		"use strict";

		/**
		 * @exports ViewserverCADLayerModel
		 * @class The cad layer model keeps all the info for a cad document on a layer
		 * @augments Viewport.LayerModel
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {String} ID
		 * @param {CadModel} cadModel
		 * @param {Boolean} isForReferencePage
		 */
		var CadLayerModel = function (f, ID, cadModel, isForReferencePage) {
			LayerModel.call(this, f, ID);

			this.cadModel = cadModel;

			this._isForReferencePage = isForReferencePage;

			this._buffer = null;

			//Take the offset of the cad file itself first
			this._offsetX = -this.cadModel.getCad().getOrigin().x;
			this._offsetY = this.cadModel.getCad().getOrigin().y;

			this._calculateTransformation();

			//Register for model change events
			this.cadModel.on("modelChanged", _.bind(function () {
				this._calculateTransformation();
			}, this));
		};

		// inherit
		CadLayerModel.prototype = Object.create(LayerModel.prototype);
		// correct the constructor pointer 
		CadLayerModel.prototype.constructor = CadLayerModel;

		/**
		 * returns the transparency of the layer
		 * @returns {Number} alpha
		 */
		CadLayerModel.prototype.getAlpha = function () {
			return this.cadModel.getAlpha();
		};

		/**
		 * Check if the layer model has CAD
		 * @returns {Boolean}
		 */
		CadLayerModel.prototype.hasLinework = function () {
			return true;
		};

		/**
		 * Does the layermodel have a buffer
		 * @returns {Boolean}
		 */
		CadLayerModel.prototype.hasBuffer = function () {
			return this._buffer !== null;
		};

		/**
		 * Returns the layers's buffer
		 * @returns {null|Fragment}
		 */
		CadLayerModel.prototype.getBuffer = function () {
			return this._buffer;
		};

		/**
		 * Set the buffer for the layer
		 * @param {null|Fragment} buffer
		 */
		CadLayerModel.prototype.setBuffer = function (buffer) {
			this._buffer = buffer;
		};

		/**
		 * Get the color for the cad linework
		 * @returns {String} the color "#RGB"
		 */
		CadLayerModel.prototype.getLineworkColor = function () {
			return this.cadModel.getCadColor();
		};

		/**
		 * @override
		 */
		CadLayerModel.prototype.getLinework = function (isAnimating, isAnimatingRotation, drawrect) {
			if (drawrect !== undefined && this.cadModel.getCad() !== null) {
				var cadSize = this.cadModel.getCad().getSize();
				var originLineX = new LineworkWrapper();

				var startX = new LineworkPoint();
				startX.makeMoveTo(0, drawrect.top());
				originLineX.pointArray.push(startX);
				var endX = new LineworkPoint();
				endX.makeLineTo(0, drawrect.bottom());
				originLineX.pointArray.push(endX);

				var originLineY = new LineworkWrapper();

				var startY = new LineworkPoint();
				startY.makeMoveTo(drawrect.left(), cadSize.height);
				originLineY.pointArray.push(startY);
				var endY = new LineworkPoint();
				endY.makeLineTo(drawrect.right(), cadSize.height);
				originLineY.pointArray.push(endY);

				originLineX.color = '#' + ViewerUtils.rgbToHex(0, 0, 255);
				originLineY.color = '#' + ViewerUtils.rgbToHex(0, 0, 255);
				originLineX.dash = [5, 4, 2, 4];
				originLineY.dash = [5, 4, 2, 4];

				var cadLines = this.cadModel.getVisibleCadLines().slice(0);
				cadLines.push(originLineX);
				cadLines.push(originLineY);

				return cadLines;
			} else {
				return this.cadModel.getVisibleCadLines();
			}
		};

		/**
		 * @override
		 */
		CadLayerModel.prototype.getBackgroundOptions = function () {
			var options = {};

			//Get the background of the cad layer. This is typically 0 (transparent) when showing cad over graphics. And 1 for only cad
			//This could also be another value, example is when the graphics need to be dimmed to highlight the CAD more
			options.fillStyle = this.cadModel.getBackgroundString();

			return options;
		};

		/**
		 * Get the rotation of the page in the CadModel
		 * @returns {Number}
		 */
		CadLayerModel.prototype.getRotation = function () {
			return this.cadModel.getPage().getRotation();
		};

		/**
		 * Is the page in the CadModel flipped
		 * @returns {Boolean}
		 */
		CadLayerModel.prototype.isFlipped = function () {
			return this.cadModel.getPage().isFlipped();
		};

		/**
		 * Calculates the transformation for the CAD layers (current CAD and reference CAD)
		 */
		CadLayerModel.prototype._calculateTransformation = function () {
			this.resetTransformation(); //first reset the transformation and calculate the new one

			var page = this.cadModel.getPage(); //get CAD page for which this layer draws the linework
			page.getCad().setLocation(null); //set the location of the CAD to null

			this._offsetX = -this.cadModel.getCad().getOrigin().x;
			this._offsetY = this.cadModel.getCad().getOrigin().y;

			var offset = new ShapeUtils.Size(this._offsetX, this._offsetY);
			if (page.hasStateParam("compareOffset")) { //if there is a compare offset defined add it to the general offset
				var compareOffset = page.getStateParam("compareOffset");
				offset.width -= compareOffset.x;
				offset.height += compareOffset.y;
			}

			var rotation = this.cadModel.getPage().getRotation();

			var isFlipped = this.cadModel.getPage().isFlipped();
			var worldBounds = this.f.getModule("viewport").getWorldBounds();
			if (!this._isForReferencePage) { //if the layer is not displaying the reference document page CAD


				this.calculateTransformation(worldBounds, page.getPageSize(), offset, rotation, isFlipped, 0, false);
			} else { // the layer is displaying the CAD of the reference document page
				var activeOrigin = this.f.getActivePage().getOrigin(true);
				var referenceOrigin = page.getOrigin(true);

				var activePageSize = this.f.getActivePage().getPageSize();
				var referencePageSize = page.getPageSize();

				//calculate the point it needs to rotate around (the centre of the active docuemnt page)
				var rotatePoint = new ShapeUtils.Point(activePageSize.width / 2, activePageSize.height / 2);
				//calculate the point to flip the reference document page around (the centre of the reference document page)
				var flipPoint = new ShapeUtils.Point(-activeOrigin.x + referenceOrigin.x + referencePageSize.width / 2, 0);
				this.calculateTransformation(worldBounds, page.getPageSize(), offset, 0, false, rotation, isFlipped, rotatePoint, flipPoint);

				//now pass the rectangle of the CAD through the transformation to get the new CAD rectangle (and set the location in the CadFIle)
				var origin = page.getCad().getOrigin();
				//need to get the origin point of the CAD relative to the world (0,0) point
				var point = new ShapeUtils.Point(origin.x - activeOrigin.x + referenceOrigin.x, -origin.y + activePageSize.height + activeOrigin.y - referenceOrigin.y);
				referencePageSize.height = -referencePageSize.height;
				var rect = new ShapeUtils.Rect(point, referencePageSize);

				var newRect = this.getTransform().transformedRect(rect);
				//set the new rect as location in the CadFile
				page.getCad().setLocation(newRect);
			}

			this.modelChanged();
		};

		return CadLayerModel;
	});

define('web-viewer/cad/CadOverlayLayerModel',["underscore",
		"web-viewer/cad/CadLayerModel",
		"utils/ShapeUtils"
	],
	function (_,
		CadLayerModel,
		ShapeUtils) {
		"use strict";

		/**
		 * @exports ViewserverCADLayerModel
		 * @class Extends the CadLayerModel, only used for the CAD overlay, in combination with Graphics layer
		 * @augments Viewport.CadLayerModel
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {String} ID
		 * @param {CadModel} cadModel
		 * @param {Boolean} isForReferencePage - is this layer showing the reference document page or not
		 */
		var CadOverlayLayerModel = function (f, ID, cadModel, isForReferencePage) {
			CadLayerModel.call(this, f, ID, cadModel, isForReferencePage);
		};

		// inherit
		CadOverlayLayerModel.prototype = Object.create(CadLayerModel.prototype);
		// correct the constructor pointer 
		CadOverlayLayerModel.prototype.constructor = CadOverlayLayerModel;

		/**
		 * Override the _calculateTransformation of the CadLayerModel, this is specific for CAD overlay
		 */
		CadOverlayLayerModel.prototype._calculateTransformation = function () {
			if (this.f.getModule("viewport")) {
				var worldBounds = this.f.getModule("viewport").getWorldBounds();

				this.resetTransformation();

				var page = this.cadModel.getPage();
				var graphicsPage = this.f.getDocumentController().getActivePage();
				//get the Size of the Active (Graphics) document page, this is the page the CAD overlay is linked to
				var graphicsSize = graphicsPage.getPageSize();
				page.getCad().setLocation(null); //reset the location 

				this._offsetX = 0;
				this._offsetY = -this.cadModel.getCad().getSize().height; //cad is always seen from bottom left corner

				var offset = new ShapeUtils.Size(this._offsetX, this._offsetY);
				var compareOffset = new ShapeUtils.Point();
				if (page.hasStateParam("compareOffset")) {
					compareOffset = page.getStateParam("compareOffset").copy();
					offset.width -= compareOffset.x;
					offset.height += compareOffset.y;
				}

				offset.width += graphicsPage.getDocumentPageOrigin().x;
				offset.height += graphicsPage.getDocumentPageOrigin().y;

				var rotation = this.cadModel.getPage().getRotation();
				if (rotation % 360 === 90) {
					offset.height -= graphicsSize.width;
				} else if (rotation % 360 === 180) {
					offset.width -= graphicsSize.width;
					offset.height -= graphicsSize.height;
				} else if (rotation % 360 === 270) {
					offset.width -= graphicsSize.height;
				}

				var isFlipped = this.cadModel.getPage().isFlipped();

				if (isFlipped && rotation % 360 === 0) {
					offset.width -= graphicsSize.width;
				} else if (isFlipped && rotation % 360 === 90) {
					offset.width -= graphicsSize.height;
				} else if (isFlipped && rotation % 360 === 180) {
					offset.width += graphicsSize.width;
				} else if (isFlipped && rotation % 360 === 270) {
					offset.width += graphicsSize.height;
				}

				this.calculateTransformation(worldBounds, page.getPageSize(), offset, 0, false, rotation, isFlipped);

				var origin = page.getCad().getOrigin();
				var pageSize = page.getPageSize();
				origin.y = -origin.y;
				var rect = new ShapeUtils.Rect(origin, pageSize);
				var newRect = this.getTransform().transformedRect(rect);

				page.getCad().setLocation(newRect);


				this.modelChanged();
			}
		};

		return CadOverlayLayerModel;
	});

define('web-viewer/cad/CadOneupName',["web-viewer/utils/LineworkWrapper",
		"web-viewer/utils/DrawUtils",
		"utils/ShapeUtils"
	],
	function (LineworkWrapper, DrawUtils, ShapeUtils) {
		/**
		 * @exports CadOneupName
		 * @memberof CadModule
		 * @class CadOneupName is a LineworkWrapper object that is used to display name of a oneup
		 * @augments Utils.LineworkWrapper
		 * @constructor
		 * @param {String} name - Name of the oneup
		 * @param {ShapeUtils.Rect} rect - rect for where the one up needs to be in the world
		 */
		var CadOneupName = function (name, rect) {
			LineworkWrapper.call(this);

			this.type = DrawUtils.LWTYPE_TEXT;
			this.staticsize = false;

			this.textoptions['rotation'] = 0;
			this.textoptions['font-family'] = 'Arial';
			this.textoptions['font-size'] = '21';
			this.textoptions['text'] = name;
			this.textoptions['textAlignment'] = 'center';

			this.color = 'blue';

			this.rect = new ShapeUtils.Rect(rect.center(), null);
		};

		// inherit
		CadOneupName.prototype = Object.create(LineworkWrapper.prototype);
		// correct the constructor pointer 
		CadOneupName.prototype.constructor = CadOneupName;

		return CadOneupName;

	});

define('web-viewer/cad/CadModel',["web-viewer/core/PrivateChannel",
		"web-viewer/cad/CadConstants",
		"web-viewer/cad/CadOneupName",
		"web-viewer/utils/ViewerUtils",
		"underscore"
	],
	function (
		PrivateChannel,
		CadConstants,
		CadOneupName,
		ViewerUtils,
		_) {
		"use strict";
		/**
		 * @exports CadModel
		 * @memberof CadModule
		 * @class The cad model is the main model within the viewer that keeps track of the current cad and their layer visibility.
		 * @constructor
		 * @param {Core.ViewerFacade} f 
		 * @param {String} ID - used for the private channel of the CadModel
		 */
		var CadModel = function (f, ID) {
			PrivateChannel.installTo(this, f, ['cadModel', ID]);

			/**
			 * Facade
			 */
			this.f = f;

			/**
			 * {CadFile} Cad object for the model
			 * @private
			 */
			this._cad = null;

			/**
			 * {AbstractDocumentPage} The page which is used for the CAD
			 * @private
			 */
			this._page = null;

			/**
			 * {Boolean} indicated if the cad is the only content of the file
			 * @private
			 */
			this._onlycad = false;

			/**
			 * {Number} Visibility of Bridges and Nicks. 1: none, 2: only bridges, 3: only nicks, 4: both
			 * @private
			 */
			this._bridgesAndNicks = CadConstants.ShowBridgesNicks.NONE;

			/**
			 * {Number} Visibility of Cad and Graphics. 1: both, 2: only graphics, 3: only cad
			 * @private
			 */
			this._cadGraphicsMode = 1;

			/**
			 * {Number} Visibility of graphics. 1:normal, 2: lighten, 3: lighten more
			 * @private
			 */
			this._graphicsMode = 1;

			/**
			 * {Boolean} true if the viewport should show oneupnames
			 * @private
			 */
			this._showOneupNames = false; //Show oneup names

			/**
			 * {Array.<CadLine/CadText>} List of currently showed cad lines
			 * @private
			 */
			this._currentCadLines = null;

			/**
			 * @member {String} color of the cad linework ('#' + RGB string)
			 * @private
			 */
			this._cadColor = null;

			/**
			 * @member {String} color of the cad background in hex format (without #)
			 * @private
			 */
			this._cadBackgroundColor = null;

		};

		/**
		 * Get the cad file
		 * @returns {CadFile} Cad File
		 */
		CadModel.prototype.getCad = function () {
			return this._cad;
		};


		/**
		 * @returns {Boolean} true if the cad is the only content
		 */
		CadModel.prototype.isOnlyCad = function () {
			return this._onlycad;
		};

		/**
		 * @returns {Array.CadLine|CadText} the visible Cad Lines
		 */
		CadModel.prototype.getVisibleCadLines = function () {
			return this._currentCadLines;
		};

		/**
		 * Set the cad file that is represented in the model
		 * @param {CadFile} cad - cad file
		 * @para {Boolean} onlyCad - is there only CAD (white background) or are there graphics
		 * @param {ShapeUtils.Size} offset - offset for the CAD (when there is graphics with CAD)
		 * @param {AbstractDocumentPage} page - the page set in the CadModel
		 */
		CadModel.prototype.setCad = function (cad, onlyCad, offset, page) {
			this._cad = cad;

			this._onlycad = onlyCad;

			this.offset = offset;

			this._page = page;

			this.invalidateLayers();
		};

		/**
		 * Get the document page where the CAD sits in
		 * @returns {AbstractDocumentPage} 
		 */
		CadModel.prototype.getPage = function () {
			return this._page;
		};

		/**
		 * Invalidate the current layers and reload the new ones
		 */
		CadModel.prototype.invalidateLayers = function () {
			this._currentCadLines = this._getLinesForSelectedLayers();
			this._emit("modelChanged");
		};

		/**
		 * fire model changed on the cad model
		 */
		CadModel.prototype.modelChanged = function () {
			this._emit("modelChanged");
		};

		/**
		 * Returns the CAD offset, returns null if there is no offset
		 * @returns {Size|null}
		 */
		CadModel.prototype.getCadOffset = function () {
			if (this.offset !== undefined) {
				return this.offset;
			} else {
				return null;
			}
		};

		/**
		 * Get the cad lines for the selected layers
		 * @private
		 * @returns {Array.<CadLine|CadText>}
		 */
		CadModel.prototype._getLinesForSelectedLayers = function () {
			var lines = [];
			for (var i = 0; i < this._cad.cadlayers.length; i++) {
				var l = this._cad.cadlayers[i];
				if (_.indexOf(this._cad.getSelectedLayers(), l) > -1) {
					lines.push(this._getCadLinesForLayer(l));

					//Iterate over the layer oneups and check if they need to be visible
					_.each(l.oneUps, _.bind(function (oneup) {
						_.each(oneup.cadlayers, _.bind(function (layer) {
							if (_.indexOf(this._cad.getSelectedClasses(), layer.layerclass) >= 0) {
								lines.push(this._getCadLinesForLayer(layer));
							}
						}, this));

						if (this._showOneupNames) {
							var oneupName = new CadOneupName(oneup.name, oneup.contentRect);
							lines.push(oneupName);
						}

					}, this));

				}
			}
			lines = _.flatten(lines);
			lines = _.compact(lines);
			return lines;
		};

		/**
		 * Get CadLine/CadText for one specific layer
		 * @private
		 * @param {CadLayer} layer 
		 * @returns {Array.<CadLine|CadText>} 
		 */
		CadModel.prototype._getCadLinesForLayer = function (layer) {
			var array = [];
			for (var j = 0; j < layer.lines.length; j++) {

				var lwwrapper = layer.lines[j];
				var add = false;

				if (lwwrapper.type === CadConstants.CadTypes.Nick) {
					var showNicks = (this._bridgesAndNicks === CadConstants.ShowBridgesNicks.NICKS || this._bridgesAndNicks === CadConstants.ShowBridgesNicks.BOTH);
					add = showNicks;
				} else if (lwwrapper.bridgeopt) {
					var showBridges = (this._bridgesAndNicks === CadConstants.ShowBridgesNicks.BRIDGES || this._bridgesAndNicks === CadConstants.ShowBridgesNicks.BOTH);

					if (layer.lines[j].bridgeopt === 1) {
						if (showBridges) {
							add = true;
						}
					} else {
						if (!showBridges) {
							add = true;
						}
					}
				} else {
					add = true;
				}

				if (add) {
					array.push(lwwrapper);
				}
			}
			if (array.length === 0) {
				array = layer.lines;
			}
			return array;
		};

		/**
		 * Get all the cad layers
		 * @returns {Array.<CadFile.layer>} 
		 */
		CadModel.prototype.getAllCadLayers = function () {
			return this._cad.cadlayers;
		};

		/**
		 * Get all the cad classes
		 * @returns {Array.<String>}
		 */
		CadModel.prototype.getAllClasses = function () {
			return this._cad.getOneupClasses();
		};

		/**
		 * @returns {Array.<CadFile.layer>} layers that are currently selected
		 */
		CadModel.prototype.getSelectedLayers = function () {
			return this._cad.getSelectedLayers();
		};

		/**
		 * Toggle a layers visibility. 
		 * @param {Number} index - index of the layer (-1 for all)
		 * @param {Boolean} [hideAllLayers] - if index = -1 define if layers should be hidden or not
		 */
		CadModel.prototype.toggleLayer = function (index, hideAllLayers) {
			this._cad.toggleLayer(index, hideAllLayers);
			this.invalidateLayers();
		};

		/**
		 * Hides all the layers
		 */
		CadModel.prototype.hideLayers = function () {
			this._cad.hideLayers();
			this.invalidateLayers();
		};

		/**
		 * Set the visibility of bridges and nicks
		 * @param {Number} 1: none, 2: only bridges, 3: only nicks, 4: both
		 */
		CadModel.prototype.setBridgesAndNicks = function (bridgesAndNicks) {
			var oldBridgesAndNicks = this._bridgesAndNicks;
			if (oldBridgesAndNicks !== bridgesAndNicks) {
				this._bridgesAndNicks = bridgesAndNicks;
				this.invalidateLayers();
			}
		};

		/**
		 * Returns the visibility of the nicks/bridges
		 * @returns {Number} 1: none, 2: only bridges, 3: only nicks, 4: both
		 */
		CadModel.prototype.getBridgesAndNicksVisibility = function () {
			return this._bridgesAndNicks;
		};

		/**
		 * Show the layer selector
		 * @returns {Boolean} true if the layer dialog should be visible
		 */
		CadModel.prototype.showLayers = function () {
			return true;
		};

		/**
		 * Show the class selector
		 * @returns {Boolean} true if the class dialog should be visible
		 */
		CadModel.prototype.showClasses = function () {
			return ((this._cad.doctype == CadConstants.CadDocType.Layout) && (this._cad.getOneupClasses().length > 0));
		};

		/**
		 * Get the list of the currently selected classes
		 * @returns {Array.<String>} selected classes
		 */
		CadModel.prototype.getSelectedClasses = function () {
			return this._cad.getSelectedClasses();
		};

		/**
		 * Toggle a class visibility. 
		 * @param {Number} classindex - index of the class (-1 for all)
		 * @param {Boolean} [hideAllClasses] - if index = -1 define if layers should be hidden or not
		 */
		CadModel.prototype.toggleClass = function (index, hideAllClasses) {
			this._cad.toggleClass(index, hideAllClasses);

			this.invalidateLayers();
		};

		/**
		 * Hides all the classes
		 */
		CadModel.prototype.hideClasses = function () {
			this._cad.hideClasses();
			this.invalidateLayers();
		};

		/**
		 * Toggle the visibility of the oneup names
		 * @param {Boolean} [toggle] True to show it, false otherwise
		 */
		CadModel.prototype.toggleShowOneupNames = function (toggle) {
			if (toggle !== undefined) {
				if (this._showOneupNames !== toggle) {
					this._showOneupNames = toggle;
					this.invalidateLayers();
				}
			} else {
				this._showOneupNames = !this._showOneupNames;
				this.invalidateLayers();
			}
		};

		/**
		 * @returns {Boolean} true if the UI should show the oneup names
		 */
		CadModel.prototype.isShowOneupNames = function () {
			return this.canShowOneupNames() && this._showOneupNames;
		};

		/**
		 * @returns {Boolean} true if the UI can show oneup names (typically only layout files)
		 */
		CadModel.prototype.canShowOneupNames = function () {
			return this._cad && this._cad.doctype == CadConstants.CadDocType.Layout;
		};

		/**
		 * Set the graphics mode
		 * @param {Number} mode - 1:normal, 2: lighten, 3: lighten more 
		 */
		CadModel.prototype.setGraphicsMode = function (mode) {
			this._graphicsMode = mode;
			this._emit("modelChanged");
		};

		/**
		 * Get the graphics mode
		 * @returns {Number} mode - 1:normal, 2: lighten, 3: lighten more 
		 */
		CadModel.prototype.getGraphicsMode = function () {
			return this._graphicsMode;
		};

		/**
		 * Set the color of the cad in the model
		 * @param {String} cadColor - '#' + RGB string
		 */
		CadModel.prototype.setCadColor = function (_cadColor) {
			this._cadColor = _cadColor;
			this._emit("modelChanged");
		};

		/**
		 * Returns the cad linework color
		 * @returns {String}
		 */
		CadModel.prototype.getCadColor = function () {
			return this._cadColor;
		};

		/**
		 * Resets the color for the cad linework
		 */
		CadModel.prototype.resetCadColor = function () {
			this._cadColor = null;
		};


		/**
		 * Get the cad graphics mode
		 * @returns {Number} 1: both, 2: only graphics, 3: only cad
		 */
		CadModel.prototype.getCadGraphicsMode = function () {
			return this._cadGraphicsMode;
		};

		/**
		 * Set the cad graphics mode
		 * @param mode
		 */
		CadModel.prototype.setCadGraphicsMode = function (mode) {
			if (mode !== this._cadGraphicsMode) {
				this._cadGraphicsMode = mode;

				this._emit("modelChanged");

			}
		};

		/**
		 * Get the alpha value of the background of the cad line
		 * @returns {Number} - alpha value that needs to be used to display the cad background
		 * - When only cad is shown OR cad graphics mode is to show only CAD, return 1 (full white background)
		 * - When the graphics mode is "lighten", return an alpha of .4
		 * - When the graphics mode is "lighten more", return an alpha of .8
		 */
		CadModel.prototype.getBackgroundAlpha = function () {
			if (this._onlycad || this._cadGraphicsMode === 3) {
				return 1;
			} else if (this.getGraphicsMode() === 2) {
				return 0.4;
			} else if (this.getGraphicsMode() === 3) {
				return 0.8;
			} else {
				return 0;
			}
		};

		/**
		 * Get the cad background color that needs to be displayed.
		 * @returns {String} - Color in hex format (without #).
		 */
		CadModel.prototype.getBackgroundColor = function () {
			if (this._cadBackgroundColor === null) {
				return "FFFFFF"; // default white
			} else {
				return this._cadBackgroundColor;
			}
		};

		/**
		 * Set the cad background color
		 * @param {String} [hexColor] - Background color in hex format (without #). If not set, it will reset to the default (white)
		 */
		CadModel.prototype.setBackgroundColor = function (hexColor) {
			if (hexColor === undefined) {
				hexColor = "FFFFFF";
			}
			if (this._cadBackgroundColor !== hexColor) {
				this._cadBackgroundColor = hexColor;
				this._emit("modelChanged");
			}
		};

		/**
		 * Get the background as string "rgba(r,g,b,a)".
		 * @returns {String}
		 */
		CadModel.prototype.getBackgroundString = function () {
			var backgroundAlpha = this.getBackgroundAlpha();
			var backgroundColor = this.getBackgroundColor();
			var rgbBackgroundColor = ViewerUtils.rgbColor(backgroundColor);
			if (rgbBackgroundColor !== null) {
				return "rgba(" + rgbBackgroundColor.r + "," +
					rgbBackgroundColor.g + "," +
					rgbBackgroundColor.b + "," +
					backgroundAlpha + ")";
			} else {
				return "rgba(255,255,255," + backgroundAlpha + ")"; // white
			}
		};

		/**
		 * Get the transparency of the CAD
		 * @returns {Float} 
		 */
		CadModel.prototype.getAlpha = function () {
			return this._cadAlpha;
		};

		/**
		 * Set the transparence of the CAD 
		 * @param {Number} cadAlpha
		 */
		CadModel.prototype.setAlpha = function (cadAlpha) {
			this._cadAlpha = cadAlpha;
		};

		return CadModel;
	});

define('tpl!web-viewer/cad/CadWidget', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<div>\r\n	<h3>'+
((__t=(d.i18n("CadLayers")))==null?'':_.escape(__t))+
':</h3>\r\n	<ul>\r\n		<li class="all">\r\n			<i class="eyeBox cadLayer '+
((__t=((d.areAllLayersSelected()?'selected icon icon-WCR_Eye':'')))==null?'':_.escape(__t))+
'" data-bind="all"></i> \r\n			<label class="cadLayer" data-bind="all">'+
((__t=(d.i18n("CadAllLayers")))==null?'':_.escape(__t))+
'</label>\r\n		</li>\r\n	';
 _.each(d.layers, function(layer, index){ 
__p+='\r\n		<li>\r\n			<i class="eyeBox cadLayer '+
((__t=((d.isLayerSelected(layer)?'selected icon icon-WCR_Eye':'')))==null?'':_.escape(__t))+
'" data-bind="'+
((__t=(index))==null?'':_.escape(__t))+
'"></i> \r\n			<label class="cadLayer" data-bind="'+
((__t=(index))==null?'':_.escape(__t))+
'">'+
((__t=(d.getLayerLocName(layer)))==null?'':_.escape(__t))+
'</label>\r\n		</li>	\r\n	';
 }); 
__p+='\r\n	</ul>\r\n</div>\r\n';
 if (d.showClasses) { 
__p+='\r\n<div>\r\n	<h3>'+
((__t=(d.i18n("CadClasses")))==null?'':_.escape(__t))+
':</h3>\r\n	<ul>\r\n		<li class="all">\r\n			<i class="eyeBox cadClass '+
((__t=((d.areAllClassesSelected()?'selected icon icon-WCR_Eye':'')))==null?'':_.escape(__t))+
'" data-bind="all"></i> 		\r\n			<label class="cadClass" data-bind="all">'+
((__t=(d.i18n("CadAllClasses")))==null?'':_.escape(__t))+
'</label>\r\n		</li>\r\n	';
 _.each(d.classes, function(cl, index){ 
__p+='\r\n		<li>\r\n			<i class="eyeBox cadClass '+
((__t=((d.isClassSelected(cl, true)?'selected icon icon-WCR_Eye':'')))==null?'':_.escape(__t))+
'" data-bind="'+
((__t=(index))==null?'':_.escape(__t))+
'"></i> \r\n			<label class="cadClass" data-bind="'+
((__t=(index))==null?'':_.escape(__t))+
'">'+
((__t=(d.getClassLocName(cl)))==null?'':_.escape(__t))+
'</label>\r\n		</li>	\r\n	';
 }); 
__p+='\r\n    ';
 if(d.referenceClasses){ 
__p+='\r\n        ';
 _.each(d.referenceClasses, function(cl, index){ 
__p+='\r\n		<li>\r\n			<i class="eyeBox cadClass '+
((__t=((d.isClassSelected(cl, false)?'selected icon icon-WCR_Eye':'')))==null?'':_.escape(__t))+
'" data-bind="'+
((__t=(d.classes.length+index))==null?'':_.escape(__t))+
'"></i>\r\n			<label class="cadClass" data-bind="'+
((__t=(index))==null?'':_.escape(__t))+
'">'+
((__t=(d.getClassLocName(cl)))==null?'':_.escape(__t))+
'</label>\r\n		</li>	\r\n	';
 }); }
__p+='\r\n	</ul>\r\n</div>\r\n';
 } 
__p+='';
return __p;
}});
define('web-viewer/cad/CadWidget',["web-viewer/layout/AbstractWidget",
		"tpl!web-viewer/cad/CadWidget",
		"web-viewer/cad/CadConstants",
		"Template",
		"utils/ShapeUtils",
		"web-viewer/layout/component/MiniButton",
		"web-viewer/layout/component/DropdownMiniButton",
		"web-viewer/layout/component/ViewerToolbarButton",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/LayoutConstants",
		"jquery",
		"underscore"
	],
	function (AbstractWidget,
		tpl,
		CadConstants,
		Template,
		ShapeUtils,
		MiniButton,
		DropdownMiniButton,
		ViewerToolbarButton,
		ButtonGroup,
		LayoutConstants,
		$,
		_) {
		"use strict";
		/**
		 * @exports CadWidget
		 * @memberof CadModule
		 * @augments Layout.AbstractWidget
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {CadModel} cadModel
		 * @param {CadModule} cadModule
		 */
		var CadWidget = function (f, cadModel, cadModule) {
			AbstractWidget.call(this, f, LayoutConstants.Widget.CAD); // with this call all the methods of AbstractWidget.js are installed to this object

			/**
			 * Facade
			 * @private
			 */
			this._f = f;

			/**
			 * {CadModule}
			 * @private
			 */
			this._cadModule = cadModule;
			/**
			 * {CadModel}
			 * @private
			 */
			this._cadModel = cadModel;

			/**
			 * Div that holds the content of the template
			 * @private
			 */
			this._contentDiv = $("<div/>");

			this.setIcon("icon icon-WCR_CAD-Both");
			
			// initialize the layer template
			this.layerTemplate = new Template(f.i18n, tpl);
			this.layerTemplate.setContext(this._contentDiv);

			//Layers
			this.layerTemplate.d.layers = this._cadModel.getAllCadLayers();
			this.layerTemplate.d.selectedLayers = this._cadModel.getSelectedLayers();

			//Bind the cad layer selection to this event
			this.layerTemplate.bind("mouseup", ".cadLayer", function (source, event) {
				var id = $(source).attr("data-bind");
				if (id == "all") {
					id = -1;
				}

				var activeCadLayerCount = this._cadModel.getAllCadLayers().length;
				if (id >= activeCadLayerCount && this._referenceCadModel) {
					id = id - activeCadLayerCount;
					if (event.which === 3) {
						this._referenceCadModel.hideLayers();
						this._referenceCadModel.toggleLayer(id);
					} else {
						this._referenceCadModel.toggleLayer(id);
					}
				} else if (id === -1) {
					var hideAllLayers = this._areAllLayersSelected();
					this._cadModel.toggleLayer(id, hideAllLayers);
					if (this._referenceCadModel) {
						this._referenceCadModel.toggleLayer(id, hideAllLayers);
					}
				} else {
					if (event.which === 3) {
						//right click
						this._cadModel.hideLayers();
						this._cadModel.toggleLayer(id);
					} else {
						this._cadModel.toggleLayer(id);
					}
				}

			}, this);

			this.layerTemplate.bind("contextmenu", ".cadLayer,.cadClass", function (el, e) {
				e.preventDefault();
				return false;
			}, this);

			//Function that indicates if all the cad layers are selected or not
			this.layerTemplate.d.areAllLayersSelected = _.bind(this._areAllLayersSelected, this);

			//Function that indicates if all the cad layers are selected or not
			this.layerTemplate.d.isLayerSelected = _.bind(function (layer) {
				return _.indexOf(this._cadModel.getSelectedLayers(), layer) >= 0 || (this._referenceCadModel && _.indexOf(this._referenceCadModel.getSelectedLayers(), layer) >= 0);
			}, this);


			//Classes
			this.layerTemplate.d.classes = this._cadModel.getAllClasses();
			this.layerTemplate.d.selectedClasses = this._cadModel.getSelectedClasses();
			this.layerTemplate.d.showClasses = this._cadModel.showClasses();

			//Function that indicates if all the cad layers are selected or not
			this.layerTemplate.d.areAllClassesSelected = _.bind(this._areAllClassesSelected, this);

			//Function that indicates if all the cad layers are selected or not
			this.layerTemplate.d.isClassSelected = _.bind(function (className, isCurrentCAD) {
				if (isCurrentCAD) {
					return _.indexOf(this._cadModel.getSelectedClasses(), className) >= 0;
				} else {
					return (this._referenceCadModel && _.indexOf(this._referenceCadModel.getSelectedClasses(), className) >= 0);
				}
			}, this);

			//Get the localised name of the layer
			this.layerTemplate.d.getLayerLocName = _.bind(function (layer) {
				var name;
				if (layer.layername) {
					name = layer.layername;
				} else {
					name = this._f.i18n.t(CadConstants.CadLayerLocName[layer.layertype]);
					if (name === undefined) {
						name = layer.layertype;
					}
				}
				if (layer.layerclass !== undefined && layer.layerclass != "0") {
					var className = layer.layerclass;
					if (CadConstants.CadClassName[layer.layerclass] !== undefined) {
						className = this._f.i18n.t(CadConstants.CadClassName[layer.layerclass]);
					}
					if (className.toUpperCase() != name.toUpperCase()) {
						if (!name || name.length === 0) {
							name = className;
						} else {
							name += " (" + className + ")";
						}
					}
				}
				return name;
			}, this);

			//Get the localised name of the class
			this.layerTemplate.d.getClassLocName = _.bind(function (classNumber) {
				return this._f.i18n.t(CadConstants.CadClassName[classNumber]);
			}, this);


			//Bind the cad class selection to this event
			this.layerTemplate.bind("mouseup", ".cadClass", function (source, event) {
				var id = $(source).attr("data-bind");
				if (id == "all") {
					id = -1;
				}

				var activeCadClassesCount = this._cadModel.getCad().getOneupClasses().length;
				if (id >= activeCadClassesCount && this._referenceCadModel) {
					id = id - activeCadClassesCount;
					if (event.which === 3) {
						//right click
						this._referenceCadModel.hideClasses();
						this._referenceCadModel.toggleClass(id);
					} else {
						this._referenceCadModel.toggleClass(id);
					}
				} else if (id === -1) {
					var hideAllClasses = this._areAllClassesSelected();
					if (this._referenceCadModel) {
						this._referenceCadModel.toggleClass(id, hideAllClasses);
					}
					this._cadModel.toggleClass(id, hideAllClasses);
				} else {
					if (event.which === 3) {
						//right click
						this._cadModel.hideClasses();
						this._cadModel.toggleClass(id);
					} else {
						this._cadModel.toggleClass(id);
					}
				}

			}, this);


			this._boundUpdateUI = _.bind(function () {
				this._updateUI();
			}, this);
			/*
			 * When the model changes, update the toggles and the selected layers
			 */
			this._cadModel.on("modelChanged", this._boundUpdateUI);

			/**
			 * Show both button
			 * @private
			 */
			this._showBothButton = new ViewerToolbarButton(f, "showBoth", "icon-WCR_CAD-Both", true, _.bind(function () {
				this._cadModule.setCadGraphicsMode(CadConstants.CadGraphicsMode.BOTH);
			}, this));
			this._showBothButton.setTooltip("viewer", "CadViewBothTT");

			/**
			 * Show only graphics
			 * @private
			 */
			this._showOnlyGraphicsButton = new ViewerToolbarButton(f, "showOnlyGraphics", "icon-WCR_CAD-OnlyGraphics", true, _.bind(function () {
				this._cadModule.setCadGraphicsMode(CadConstants.CadGraphicsMode.GRAPHICS_ONLY);
			}, this));
			this._showOnlyGraphicsButton.setTooltip("viewer", "CadViewGraphicsTT");

			/**
			 * Show only cad
			 * @private
			 */
			this._showOnlyCadButton = new ViewerToolbarButton(f, "showOnlyCad", "icon-WCR_CAD-OnlyCAD", true, _.bind(function () {
				this._cadModule.setCadGraphicsMode(CadConstants.CadGraphicsMode.CAD_ONLY);
			}, this));
			this._showOnlyCadButton.setTooltip("viewer", "CadViewCadTT");

			/**
			 * Show 3D widget with a generated 3D model of the active document page
			 * @private
			 */
			this._3DView = new MiniButton(this.f, "show3DView", "icon-WCR_3D-Create", true, _.bind(function () {
				//first check if the compareModule exists, if that's the case let it check if the aligment is correctly saved
				var compareModule = this.f.getModule("compare");
				if (compareModule) { //pass opening of 3D through the compare module
					compareModule.checkAligmentAndShow3D();
				} else { //no compare module (only CAD is open) load the 3D View
					this.f.requestModule("view3D", _.bind(function (view3DModule) {
						view3DModule.show3D();
					}, this), false);
				}
			}, this), "fontIconDisabled icon-WCR_3D-Create");


			/**
			 * Button group to switch cadGraphicsMode
			 * @private
			 */
			this._showButtonGroup = new ButtonGroup("buttonGroup", [this._showBothButton, this._showOnlyGraphicsButton, this._showOnlyCadButton]);
			this._showButtonGroup.element.addClass("cadShowOptions toggleBtnGroup");

			/**
			 * Make the options dropdown
			 * @private
			 */
			this._optionButton = new DropdownMiniButton(f, "cadOptions", "icon-WCR_Settings-submenu");

			f.i18n.loadBundle("viewer", _.bind(function () {
				this._optionButton.addOption({
					name: 'CadBridgesShowNone',
					content: f.i18n.t("CadBridgesShowNone"),
					onchange: _.bind(this._showNone, this),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'CadBridgesShowNicks',
					content: f.i18n.t("CadBridgesShowNicks"),
					onchange: _.bind(this._showNicks, this)
				});

				this._optionButton.addOption({
					name: 'CadBridgesShowBridges',
					content: f.i18n.t("CadBridgesShowBridges"),
					onchange: _.bind(this._showBridges, this)
				});

				this._optionButton.addOption({
					name: 'CadBridgesShowBoth',
					content: f.i18n.t("CadBridgesShowBoth"),
					onchange: _.bind(this._showBoth, this)
				});

				this._optionButton.toggleOption("CadBridgesShowNone", true);

				this._optionButton.addDivider();

				this._optionButton.addOption({
					name: 'CadOneupTT',
					content: f.i18n.t("CadOneupTT"),
					onchange: _.bind(this._toggleShowOneupNames, this)
				});

				this._optionButton.addDivider();

				this._optionButton.addOption({
					name: 'CadNormal',
					content: f.i18n.t("CadNormal"),
					onchange: _.bind(this._setGraphicsNormal, this)
				});

				this._optionButton.addOption({
					name: 'CadLighten',
					content: f.i18n.t("CadLighten"),
					onchange: _.bind(this._setGraphicsLighten, this)
				});
				this._optionButton.addOption({
					name: 'CadLightenMore',
					content: f.i18n.t("CadLightenMore"),
					onchange: _.bind(this._setGraphicsLightenMore, this)
				});

				this.setTitle(this.f.i18n.t("CadTitle"));

				this._updateUI();
			}, this));

			/*
			 * Overwrite the setContext to attach the header and the template
			 */
			this._override("setContext", function (context, headerExtra) {
				this._super(context);

				headerExtra.append(this._3DView.element);
				this._3DView.setTooltip("viewer", "Open3DViewTT");

				//Append header			
				headerExtra.append(this._optionButton.element);
				headerExtra.addClass("flexEnd");

				//Append content (buttons + content)
				context.append(this._showButtonGroup.element);
				context.append(this._contentDiv);
				context.addClass("cadWidget");

				//Load translations if needed
				f.i18n.loadBundle("viewer", _.bind(function () {
					this._updateUI();
				}, this));
			});

		};

		CadWidget.prototype = Object.create(AbstractWidget.prototype);
		CadWidget.prototype.constructor = CadWidget;
		CadWidget.widgetName = LayoutConstants.Widget.CAD;

		/**
		 * @override
		 */
		CadWidget.prototype.getMinimumSize = function () {
			return new ShapeUtils.Size(250, 150);
		};

		/**
		 * @override
		 */
		CadWidget.prototype.willFill = function () {
			return false;
		};

		/**
		 * Are all the layers currently selected (visible)?
		 * @returns {Boolean}
		 * @private
		 */
		CadWidget.prototype._areAllLayersSelected = function () {
			return this._cadModel.getAllCadLayers().length == this._cadModel.getSelectedLayers().length && (!this._referenceCadModel || this._referenceCadModel.getAllCadLayers().length == this._referenceCadModel.getSelectedLayers().length);
		};

		/**
		 * Are all classes currently selected (visible)?
		 * @returns {Boolean}
		 */
		CadWidget.prototype._areAllClassesSelected = function () {
			return this._cadModel.getCad().getOneupClasses().length == this._cadModel.getSelectedClasses().length && (!this._referenceCadModel || this._referenceCadModel.getCad().getOneupClasses().length == this._referenceCadModel.getSelectedClasses().length);
		};

		/**
		 * Change the cadModel of the widget
		 * @param {CadModel} cadModel
		 * @param {CadModel} referenceCadModel
		 */
		CadWidget.prototype.setCadModel = function (cadModel, referenceCadModel) {
			if (this._cadModel !== cadModel) {
				this._cadModel.off("modelChanged", this._boundUpdateUI);
				this._cadModel = cadModel;
				this._cadModel.on("modelChanged", this._boundUpdateUI);
			}
			if (this._referenceCadModel !== referenceCadModel) {
				if (this._referenceCadModel) {
					this._referenceCadModel.off("modelChanged", this._boundUpdateUI);
				}
				if (referenceCadModel) {
					this._referenceCadModel = referenceCadModel;
					this._referenceCadModel.on("modelChanged", this._boundUpdateUI);
					this._referenceCadModel.setBridgesAndNicks(this._cadModel.getBridgesAndNicksVisibility());
				} else {
					this._referenceCadModel = null;
				}
			}
			this._updateUI();
		};

		/**
		 * Update the UI components
		 * @private
		 */
		CadWidget.prototype._updateUI = function () {
			var activePage = this.f.getActivePage();
			var overlay = this.f.getDocumentController().getCadOverlay();
			if (this._3DView && this._3DView.element){
				//check if the active page either has CAD or there is CAD overlay (linked or docref) 
				if (!this.f.isMobile() &&
					activePage.get("canGenerate3D") &&
					(activePage.hasCad() || (activePage.hasGraphics() && overlay && (activePage.hasLinkedPage() || activePage.canSaveCadOverlay()))) &&
					activePage.canEditAttributes()) {
					this._3DView.element.removeClass("displayNone");
				} else {
					this._3DView.element.addClass("displayNone");
				}
			}

			// Nicks/Bridges visibility
			this._optionButton.toggleOption("CadBridgesShowNone", this._cadModel.getBridgesAndNicksVisibility() === CadConstants.ShowBridgesNicks.NONE);
			this._optionButton.toggleOption("CadBridgesShowBridges", this._cadModel.getBridgesAndNicksVisibility() === CadConstants.ShowBridgesNicks.BRIDGES);
			this._optionButton.toggleOption("CadBridgesShowNicks", this._cadModel.getBridgesAndNicksVisibility() === CadConstants.ShowBridgesNicks.NICKS);
			this._optionButton.toggleOption("CadBridgesShowBoth", this._cadModel.getBridgesAndNicksVisibility() === CadConstants.ShowBridgesNicks.BOTH);

			// Show one up names
			this._optionButton.setActionEnabled("CadOneupTT", this._cadModel.canShowOneupNames() || (this._referenceCadModel && this._referenceCadModel.canShowOneupNames()));
			this._optionButton.toggleOption("CadOneupTT", this._cadModel.isShowOneupNames() || (this._referenceCadModel !== null && this._referenceCadModel !== undefined && this._referenceCadModel.isShowOneupNames()));

			// Graphics mode		 
			this._optionButton.toggleOption("CadNormal", this._cadModel.getGraphicsMode() === 1);
			this._optionButton.toggleOption("CadLighten", this._cadModel.getGraphicsMode() === 2);
			this._optionButton.toggleOption("CadLightenMore", this._cadModel.getGraphicsMode() === 3);

			var showOptions = !this._cadModel.isOnlyCad();
			this._optionButton.setActionEnabled("CadNormal", showOptions);
			this._optionButton.setActionEnabled("CadLighten", showOptions);
			this._optionButton.setActionEnabled("CadLightenMore", showOptions);

			//Set the tooltip of the "both" button with the mode of the graphics
			var mode = "CadNormal";
			if (this._cadModel.getGraphicsMode() === 2) {
				mode = "CadLighten";
			} else if (this._cadModel.getGraphicsMode() === 3) {
				mode = "CadLightenMore";
			}
			this._showBothButton.setTooltip("viewer", "CadViewBothTT", [this._f.i18n.t(mode)]);

			// Update template
			this.layerTemplate.d.classes = this._cadModel.getCad().getOneupClasses();
			this.layerTemplate.d.showClasses = this._cadModel.showClasses();
			this.layerTemplate.d.layers = this._cadModel.getAllCadLayers();

			if (this._referenceCadModel) {
				this.layerTemplate.d.referenceClasses = this._referenceCadModel.getCad().getOneupClasses();
				this.layerTemplate.d.showClasses = this._cadModel.showClasses() || this._referenceCadModel.showClasses();
				this.layerTemplate.d.layers = this.layerTemplate.d.layers.concat(this._referenceCadModel.getAllCadLayers());
			}

			this.layerTemplate.render();

			// Show graphics
			this._showBothButton.setActive(this._cadModel.getCadGraphicsMode() === CadConstants.CadGraphicsMode.BOTH);
			this._showOnlyGraphicsButton.setActive(this._cadModel.getCadGraphicsMode() === CadConstants.CadGraphicsMode.GRAPHICS_ONLY);
			this._showOnlyCadButton.setActive(this._cadModel.getCadGraphicsMode() === CadConstants.CadGraphicsMode.CAD_ONLY);

			//Hide cad/graphics selection if there is only cad
			if (this._cadModel.isOnlyCad()) {
				this._showButtonGroup.hide();
			} else {
				this._showButtonGroup.show();
			}

			this._fireResized();

		};

		/**
		 * Show bridges
		 * @private
		 */
		CadWidget.prototype._showBridges = function (toggle) {
			if (toggle) {
				this._cadModel.setBridgesAndNicks(CadConstants.ShowBridgesNicks.BRIDGES);
				if (this._referenceCadModel) {
					this._referenceCadModel.setBridgesAndNicks(CadConstants.ShowBridgesNicks.BRIDGES);
				}
			} else {
				this._showNone(true);
			}
		};

		/**
		 * Show nicks
		 * @private
		 */
		CadWidget.prototype._showNicks = function (toggle) {
			if (toggle) {
				this._cadModel.setBridgesAndNicks(CadConstants.ShowBridgesNicks.NICKS);
				if (this._referenceCadModel) {
					this._referenceCadModel.setBridgesAndNicks(CadConstants.ShowBridgesNicks.NICKS);
				}
			} else {
				this._showNone(true);
			}
		};

		/**
		 * Show both bridges and nicks
		 * @private
		 */
		CadWidget.prototype._showBoth = function (toggle) {
			if (toggle) {
				this._cadModel.setBridgesAndNicks(CadConstants.ShowBridgesNicks.BOTH);
				if (this._referenceCadModel) {
					this._referenceCadModel.setBridgesAndNicks(CadConstants.ShowBridgesNicks.BOTH);
				}
			} else {
				this._showNone(true);
			}
		};

		/**
		 * Show no bridges or nicks
		 * @private
		 */
		CadWidget.prototype._showNone = function (toggle) {
			if (toggle) {
				this._cadModel.setBridgesAndNicks(CadConstants.ShowBridgesNicks.NONE);
				if (this._referenceCadModel) {
					this._referenceCadModel.setBridgesAndNicks(CadConstants.ShowBridgesNicks.NONE);
				}
			}
		};

		/**
		 * Show/hide oneup names
		 * @private
		 */
		CadWidget.prototype._toggleShowOneupNames = function () {
			if (this._cadModel.canShowOneupNames()) {
				this._cadModel.toggleShowOneupNames();
			}
			if (this._referenceCadModel && this._referenceCadModel.canShowOneupNames()) {
				this._referenceCadModel.toggleShowOneupNames();
			}
		};

		/**
		 * Set grapics level to normal
		 * @private
		 */
		CadWidget.prototype._setGraphicsNormal = function () {
			this._cadModel.setGraphicsMode(1);
		};

		/**
		 * Set grapics level to lighten
		 * @private
		 */
		CadWidget.prototype._setGraphicsLighten = function () {
			this._cadModel.setGraphicsMode(2);
		};

		/**
		 * Set grapics level to lighten more
		 * @private
		 */
		CadWidget.prototype._setGraphicsLightenMore = function () {
			this._cadModel.setGraphicsMode(3);
		};

		return CadWidget;
	});

/**
 *  @namespace CadModule
 */
define('web-viewer/cad/CadModule',["underscore",
		"web-viewer/core/AbstractModule",
		"web-viewer/cad/CadDataExtensions",
		"web-viewer/cad/CadLayerModel",
		"web-viewer/cad/CadOverlayLayerModel",
		"web-viewer/cad/CadModel",
		"web-viewer/cad/CadWidget",
		"web-viewer/cad/CadConstants",
		"web-viewer/compare/CompareConstants",
		"web-viewer/utils/ListenerUtil",
		"utils/ShapeUtils"
	],
	function (_,
		AbstractModule,
		CadDataExtensions,
		CadLayerModel,
		CadOverlayLayerModel,
		CadModel,
		CadWidget,
		CadConstants,
		CompareConstants,
		ListenerUtil,
		ShapeUtils
	) {
		"use strict";

		/**
		 * @exports CadModule
		 * @memberof CadModule
		 * @class The cad module is the main module that controls the loading of cad and displaying
		 * @augments Core.AbstractModule
		 * @param {Core.ViewerFacade} f
		 * @param {Boolean} silenced
		 * @constructor
		 */
		var CadModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "cad");
			ListenerUtil.mixin(this, f);

			this.f = f;

			/**
			 * CadModel
			 * @private
			 */
			this._cadModel = new CadModel(this.f, "MainCad");

			/**
			 * CadModel for difference compare
			 * @private
			 */
			this._referenceDiffCadModel = new CadModel(this.f, "referenceDiffCad");

			/**
			 * @member {CadModel} CadModel for side by side, reference document page
			 * @private
			 */
			this._referenceCadModel = new CadModel(this.f, "referenceCad");

			/**
			 * @member {CadModel} CadModel for difference compare in side by side, current document page
			 * @private
			 */
			this._currentDiffCadModel = new CadModel(this.f, "currentDiffCad");

			/**
			 * @member {CadModel} CadModel for the cad overlay
			 * @private
			 */
			this._cadOverlayModel = new CadModel(this.f, "cadOverlay");

			/**
			 * CadWidget
			 * @private
			 */
			this._widget = null;

			/**
			 * This is the model currently assigned to the CAD Widget
			 * @private
			 **/
			this._cadModelForWidget = this._cadModel;

			/**
			 * This is the model currently assigned to the CAD Widget for the reference document page
			 * @private
			 **/
			this._referenceCadModelForWidget = null;

			/**
			 * Layer models per view
			 * @private
			 */
			this._layerModels = {};

			/**
			 * Fetched cadStyles, cache to prevent multiple calls for the same cadStyle
			 * @private
			 */
			this._cadStyles = {};

			this.compareDiffMode = CompareConstants.diffMode.UNDEFINED;


			// inject cad functionality in data objects
			CadDataExtensions(f);


			this.f.on("module.compare.changed.diffMode", _.bind(function (oldMode, newMode) {
				if (this.compareDiffMode !== newMode) {
					this._whenCompareModeChanged(this.compareDiffMode, newMode);
				}
			}, this));

			this.f.getCurrentUser().on("options.changed.PREFERENCES_HIGHLIGHTCOLORS_DIFF_ADD", _.bind(function () {
				if ((this.compareDiffMode === CompareConstants.diffMode.SHOWBOTHCHANGES || this.compareDiffMode === CompareConstants.diffMode.SHOWBOTHSIDEBYSIDE) && this._referenceDiffCadModel !== null) {
					var color = this.f.getCurrentUser().getColor("PREFERENCES_HIGHLIGHTCOLORS_DIFF_ADD");
					if (this.compareDiffMode === CompareConstants.diffMode.SHOWBOTHSIDEBYSIDE) {
						this._currentDiffCadModel.setCadColor(color);
					} else {
						this._cadModel.setCadColor(color);
					}
				}
			}, this));

			this.f.getCurrentUser().on("options.changed.PREFERENCES_HIGHLIGHTCOLORS_DIFF_SUB", _.bind(function () {
				if ((this.compareDiffMode === CompareConstants.diffMode.SHOWBOTHCHANGES || this.compareDiffMode === CompareConstants.diffMode.SHOWBOTHSIDEBYSIDE) && this._cadModel !== null) {
					var color = this.f.getCurrentUser().getColor("PREFERENCES_HIGHLIGHTCOLORS_DIFF_SUB");
					this._referenceDiffCadModel.setCadColor(color);
				}
			}, this));

			var docController = this.f.getDocumentController();



			// initialize the CAD before parameters available (and endUpdate, so the viewport will use correct info for verifyWorld)
			docController.on("parameters.beforeEndUpdate", _.bind(function (activeChanged, differenceChanged, cadOverlayChanged) {
				var docController = this.f.getDocumentController();
				_.each([docController.getActivePage(), docController.getDifferencePage(), docController.getCadOverlay()], function (page) {
					if (page && page.hasCad()) {
						page.initializeCadFile();
					}
				}, this);

				if (activeChanged || differenceChanged || cadOverlayChanged) {
					this._whenPagesChangedAndParametersAvailable();
				} else if (this._layerModels['CadOverlay']) {
					this._cadOverlayModel.modelChanged();
				}
			}, this));

			docController.whenReferencePageStateParametersChange(function (activePage, differencePage) {
				if (this._layerModels['referenceDiffCad']) {
					this._referenceDiffCadModel.modelChanged();
				}
				if (this._layerModels['referenceCad']) {
					this._referenceCadModel.modelChanged();
				}

			}, this);

			docController.whenCadOverlayPageStateParametersChange(function (activePage, differencePage, cadOverlay) {
				if (this._layerModels['CadOverlay']) {
					this._cadOverlayModel.modelChanged();
				}
			}, this);

			if (!this.f.isParameterUpdateOngoing() && this.f.getActivePage()) {
				this._whenPagesChangedAndParametersAvailable();
			}

		};

		// inherit
		CadModule.prototype = Object.create(AbstractModule.prototype);
		// correct the constructor pointer 
		CadModule.prototype.constructor = CadModule;

		/**
		 * Get an array with all the pages (either active, difference or overlay) having CAD
		 * @returns {Array.<AbstractDocumentPage>} list of pages having CAD
		 */
		CadModule.prototype.getAllActivePagesHavingCad = function () {
			var docController = this.f.getDocumentController();
			return _.filter([docController.getActivePage(), docController.getDifferencePage(), docController.getCadOverlay()], function (page) {
				return page && page.hasCad();
			});
		};

		/**
		 * Execute a callback function when all CAD files are successfully initialized and available
		 * @param   {Function} callback - the callback, called when all cad files are loaded
		 * @param   {Object}   [binding]  - binding for the callback
		 */
		CadModule.prototype.whenAllCadFilesInitialized = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}

			var pagesHavingCad = this.getAllActivePagesHavingCad();

			if (pagesHavingCad.length > 0) {
				callback = _.after(pagesHavingCad.length, callback);
				_.each(pagesHavingCad, function (page) {
					page.requestCad(callback);
				});
			} else {
				// no CAD files are loaded, still execute the callback
				callback();
			}
		};

		/**
		 * Initialize the cad layer and widget when a document with cad data is loaded
		 * @param {AbstractDocumentPage} currentPage - New page that is loaded
		 */
		CadModule.prototype._whenPagesChangedAndParametersAvailable = function (activePage, differencePage, cadOverlay) {
			this.whenAllCadFilesInitialized(this._updateCadRendering, this);
		};

		/**
		 * React on a change of compare mode, if the cadModule wasn't initialized don't execute it
		 * @private
		 * @param {CompareConstants.diffMode} oldMode
		 * @param {CompareConstants.diffMode} newMode
		 */
		CadModule.prototype._whenCompareModeChanged = function (oldMode, newMode) {

			if (newMode === CompareConstants.diffMode.SHOWBOTHCHANGESHIGHLIGHT || newMode === CompareConstants.diffMode.SHOWBOTHCHANGESONLY) {
				newMode = CompareConstants.diffMode.SHOWBOTHCHANGES;
			} else if (newMode === CompareConstants.diffMode.SHOWBOTHSIDEBYSIDEHIGHLIGHT || newMode === CompareConstants.diffMode.SHOWBOTHSIDEBYSIDECHANGESONLY) {
				newMode = CompareConstants.diffMode.SHOWBOTHSIDEBYSIDE;
			}
			this.compareDiffMode = newMode;

			this.whenAllCadFilesInitialized(this._updateCadRendering, this);
		};


		/**
		 * Update the rendering of CAD (configure CadModels correctly, add/remove layers, update colors for CadModels)
		 * !!NOTE this method should only be called when CAD is available for all loaded documetns (active, difference, overlay)
		 */
		CadModule.prototype._updateCadRendering = function () {
			var docController = this.f.getDocumentController();

			var activePage = docController.getActivePage();
			var differencePage = docController.getDifferencePage();
			var cadOverlay = docController.getCadOverlay();

			// we use the viewport module for adding and removing layers
			this.f.whenModuleAvailable("viewport", _.bind(function (viewportModule) {
				viewportModule.whenWidgetModelAvailable(function () {
					var subtractedHighlightColor = this.f.getCurrentUser().getColor("PREFERENCES_HIGHLIGHTCOLORS_DIFF_ADD");
					var addedHighlightColor = this.f.getCurrentUser().getColor("PREFERENCES_HIGHLIGHTCOLORS_DIFF_SUB");

					// reset the origins of all cad files
					_.each([activePage, differencePage, cadOverlay], function (page) {
						if (page && page.hasCad()) {
							page.getCad().resetOrigin();
						}
					});

					if ((activePage && activePage.hasCad()) || (cadOverlay && cadOverlay.hasCad())) {
						// disable animations
						this.f.emit("viewport.toggleAnimation", "cad", false);

						this._referenceCadModelForWidget = null;

						// set the correct cad data
						if (cadOverlay) {
							this._cadOverlayModel.setCad(cadOverlay.getCad(), false, cadOverlay.getCadOffset(), cadOverlay);
							if (!this._layerModels["CadOverlay"]) {
								this._layerModels["CadOverlay"] = new CadOverlayLayerModel(this.f, "CadOverlay", this._cadOverlayModel, false);
								viewportModule.addLayer("CadOverlay", this._layerModels["CadOverlay"]);
							}
							this._cadModelForWidget = this._cadOverlayModel;
							if (this._layerModels["Cad"]) {
								viewportModule.removeLayer("Cad");
								delete this._layerModels["Cad"];
							}
						} else if (this.compareDiffMode !== CompareConstants.diffMode.DIFFVERSION) {
							if (this._layerModels["CadOverlay"]) {
								viewportModule.removeLayer("CadOverlay");
								delete this._layerModels["CadOverlay"];
							}
							this._cadModel.setCad(activePage.getCad(), true, activePage.getCadOffset(), activePage);
							this._addCadLayer("Cad", this._cadModel, false);
							this._cadModelForWidget = this._cadModel;
						}

						//objects will be used if the active/reference page has CAD
						var activeCad;
						var diffCad;

						if ((this.compareDiffMode === CompareConstants.diffMode.DIFFVERSION || this.compareDiffMode === CompareConstants.diffMode.SHOWBOTHSIDEBYSIDE) && !cadOverlay && differencePage && differencePage.hasCad()) {
							// we are in CAD COMPARE, show only difference
							activeCad = activePage.getCad();
							diffCad = differencePage.getCad();

							var activePageOffset = activeCad.getOrigin();
							var activeSize = activeCad.getSize();
							var diffSize = diffCad.getSize();
							diffCad.setOrigin(activePageOffset.x, activePageOffset.y - (diffSize.height - activeSize.height));
						}

						// now let's make sure compare is rendered correctly
						if (this.compareDiffMode === CompareConstants.diffMode.DIFFVERSION &&
							!cadOverlay && differencePage && differencePage.hasCad()) {

							if (this._layerModels["Cad"]) {
								viewportModule.removeLayer("Cad", "Main");
								delete this._layerModels["Cad"];
							}

							this._referenceCadModel.resetCadColor();

							this._cadModelForWidget = this._referenceCadModel;
							this._referenceCadModel.setCad(diffCad, true, differencePage.getCadOffset(), differencePage);
							this._addCadLayer("referenceCad", this._referenceCadModel, true);
						} else if (this.compareDiffMode === CompareConstants.diffMode.SHOWBOTHSIDEBYSIDE &&
							!cadOverlay && differencePage && differencePage.hasCad()) {
							// we are in CAD Compare, side by side mode (no cad overlay is defined)
							var cadOffset = activePage.getCadOffset();
							var referenceCadOffset = differencePage.getCadOffset();

							this._cadModel.resetCadColor();

							// add layer for the reference CAD linework
							this._referenceCadModel.setCad(diffCad, true, referenceCadOffset, differencePage);
							this._addCadLayer("referenceCad", this._referenceCadModel, true);
							this._referenceCadModelForWidget = this._referenceCadModel;

							// add layer for showing a colored version of the active page
							this._currentDiffCadModel.setCadColor(subtractedHighlightColor);
							this._currentDiffCadModel.setCad(activeCad, true, cadOffset, activePage);

							this._addCadLayer("currentDiffCad", this._currentDiffCadModel, false);

							// add a layer for showing a colored version of the difference page
							this._referenceDiffCadModel.setCadColor(addedHighlightColor);
							this._addLayerForReferenceDiffCad(activePage, differencePage);

						} else if ((this.compareDiffMode === CompareConstants.diffMode.SHOWBOTHCHANGES || this.compareDiffMode === CompareConstants.diffMode.SHOWBOTHONTOP) &&
							!cadOverlay && differencePage && differencePage.hasCad()) {
							// we are in CAD compare mode, difference or show-both-on-top mode (no cad overlay is defined)

							// show the reference cad
							this._addLayerForReferenceDiffCad(activePage, differencePage);

							if (this.compareDiffMode === CompareConstants.diffMode.SHOWBOTHCHANGES) {
								// we need to color the CAD linework
								this._cadModel.setCadColor(subtractedHighlightColor);
								this._referenceDiffCadModel.setCadColor(addedHighlightColor);
							} else {
								this._cadModel.resetCadColor();
								this._referenceDiffCadModel.resetCadColor();
							}

							this._referenceCadModelForWidget = this._referenceDiffCadModel;

							if (this._layerModels["referenceCad"]) {
								viewportModule.removeLayer("referenceCad");
								delete this._layerModels["referenceCad"];
							}
						} else {
							// remove layers from side by side compare or show both on top if needed
							_.each(["referenceCad", "currentDiffCad", "referenceDiffCad"], function (layerName) {
								if (this._layerModels[layerName]) {
									viewportModule.removeLayer(layerName);
								}
								delete this._layerModels[layerName];
							}, this);

							// reset the CAD linework colors
							this._cadModel.resetCadColor();
							this._referenceDiffCadModel.resetCadColor();
						}

						// show widget
						if (!this._widget) {
							// the widget does not exist yet, publish it
							this._publishWidget(CadWidget.widgetName, _.bind(function (layoutType) {
								this._widget = new CadWidget(this.f, this._cadModelForWidget, this);

								// check if the widget is still needed
								if (this.getAllActivePagesHavingCad().length > 0) {
									this._widget.setCadModel(this._cadModelForWidget, this._referenceCadModelForWidget);
								} else {
									this._widget.setEnabled(false);
								}

								return this._widget;
							}, this));
						} else {
							// set the correct CAD model for the widget (this could be the difference cad is "show reference mode" is activated
							this._widget.setCadModel(this._cadModelForWidget, this._referenceCadModelForWidget);
							// enable the widget
							this._widget.setEnabled(true);
						}

						this._updateCadGraphicsMode();

					} else {
						// NO CAD is defined

						// enable animations
						this.f.emit("viewport.toggleAnimation", "cad", true);

						// remove all added CAD layers            
						_.each(this._layerModels, function (layerModel, layerName) {
							viewportModule.removeLayer(layerName);
						});
						this._layerModels = {};

						// disable widget
						if (this._widget) {
							this._widget.setEnabled(false);
						}

						viewportModule.toggleLayer("Graphics", true);
					}
				}, this);
			}, this));
		};

		/**
		 * Add a layer to the viewport (if the layer is not yet existing)
		 * @param {String}   layerName - name of the CAD layer
		 * @param {CadModel} cadModel  - the CAD model to be used for the layer
		 * @param {Boolean} isForReferencePage - is the layer model for the reference page
		 */
		CadModule.prototype._addCadLayer = function (layerName, cadModel, isForReferencePage) {
			if (!this._layerModels[layerName]) {
				this._layerModels[layerName] = new CadLayerModel(this.f, layerName, cadModel, isForReferencePage);
				this.f.getModule("viewport").addLayer(layerName, this._layerModels[layerName]);
			}
		};

		/**
		 * Add a layer for the reference cad file to the difference panel
		 * @param {AbstractDocumentPage} cadDiff
		 * @param {CadFile} currentCad - the active document page's CAD file
		 * @private
		 */
		CadModule.prototype._addLayerForReferenceDiffCad = function (activePage, differencePage) {
			var activeCad = activePage.getCad();
			var diffCad = differencePage.getCad();

			var diffCadOffset = differencePage.getCadOffset();


			var activeCadOffset = activeCad.getOrigin();
			var activeSize = activeCad.getSize();
			var diffSize = diffCad.getSize();
			diffCad.setOrigin(activeCadOffset.x, activeCadOffset.y - (diffSize.height - activeSize.height));

			this._referenceDiffCadModel.setAlpha(0.5);
			this._referenceDiffCadModel.setCad(diffCad, false, diffCadOffset, differencePage);

			this._addCadLayer("referenceDiffCad", this._referenceDiffCadModel, true);
		};

		/**
		 * @override
		 */
		CadModule.prototype.getState = function () {
			if (this._cadModel !== null && this._cadModel.getPage()) {
				var state = {
					// Cad model
					cadModel: {
						bridgesAndNicks: this._cadModel.getBridgesAndNicksVisibility(),
						cadGraphicsMode: this.getCadGraphicsMode(),
						graphicsMode: this._cadModel.getGraphicsMode(),
						showOneupNames: this._cadModel.isShowOneupNames(),
						cadColor: this._cadModel.getCadColor(),
						//run over all the layers and set value to true if the layer is selected
						selectedLayers: _.map(this._cadModel.getAllCadLayers(), _.bind(function (layer) {
							//maps layerId on a boolean, is the layer selected or not
							return _.indexOf(this._cadModel.getSelectedLayers(), layer) >= 0;
						}, this)),
						selectedClasses: _.map(this._cadModel.getAllClasses(), _.bind(function (className) {
							//maps ClassId on a boolean, is the class selected or not
							return _.indexOf(this._cadModel.getSelectedClasses(), className) >= 0;
						}, this))
					}
				};
				return state;
			} else {
				return {};
			}
		};

		/**
		 * @override
		 * @param {Object} state
		 */
		CadModule.prototype.setState = function (state) {
			// This module is ready
			if (state !== undefined) {
				// Cad model
				var cadModelState = state.cadModel;

				if (cadModelState !== undefined) {
					this._cadModel.setBridgesAndNicks(cadModelState.bridgesAndNicks);

					this._cadModel.setGraphicsMode(cadModelState.graphicsMode);
					this._cadModel.toggleShowOneupNames(cadModelState.showOneupNames);
					this._cadModel.setCadColor(cadModelState.cadColor);
					if (cadModelState.selectedLayers !== undefined) {
						_.each(cadModelState.selectedLayers, _.bind(function (isLayerSelected, index) {
							if (!isLayerSelected) {
								this._cadModel.toggleLayer(index);
							}
						}, this));
					}
					if (cadModelState.selectedClasses !== undefined) {
						_.each(cadModelState.selectedClasses, _.bind(function (isClassSelected, index) {
							if (!isClassSelected) {
								this._cadModel.toggleClass(index);
							}
						}, this));
					}
					this.setCadGraphicsMode(cadModelState.cadGraphicsMode);
				}
				this._whenCompareModeChanged(-1, this.compareDiffMode);
			}
		};

		/**
		 * Get the CadModel of the currently active document page
		 * @returns {CadModel}
		 */
		CadModule.prototype.getCurrentPageCadModel = function () {
			if (this._currentDiffCadModel.getVisibleCadLines() !== null && this._currentDiffCadModel.getVisibleCadLines().length > 0 && this.compareDiffMode === CompareConstants.diffMode.SHOWBOTHSIDEBYSIDE) {
				return this._currentDiffCadModel;
			} else {
				return this._cadModel;
			}
		};

		/**
		 * Get the CadModel of the reference document page
		 * @returns {CadModel}
		 */
		CadModule.prototype.getReferencePageCadModel = function () {
			if (this.compareDiffMode === CompareConstants.diffMode.SHOWBOTHCHANGES || this.compareDiffMode === CompareConstants.diffMode.SHOWBOTHSIDEBYSIDE || this.compareDiffMode === CompareConstants.diffMode.SHOWBOTHONTOP) {
				return this._referenceDiffCadModel;
			} else if (this.compareDiffMode === CompareConstants.diffMode.DIFFVERSION) {
				return this._referenceCadModel;
			} else {
				return null;
			}
		};

		/**
		 * Get the cadModel for the CAD overlay
		 * @returns {CadModel}
		 */
		CadModule.prototype.getCadOverlayModel = function () {
			if (this._cadOverlayModel.getVisibleCadLines() !== null && this._cadOverlayModel.getVisibleCadLines().length > 0) {
				return this._cadOverlayModel;
			} else {
				return null;
			}
		};

		/**
		 * Set the CAD background color
		 * @param {String} [hexColor] - New background color in hex format (without #). If not set, it will reset to the default.
		 */
		CadModule.prototype.setCadBackgroundColor = function (hexColor) {
			// set it in all models
			this._cadModel.setBackgroundColor(hexColor);
			this._referenceDiffCadModel.setBackgroundColor(hexColor);
			this._referenceCadModel.setBackgroundColor(hexColor);
			this._currentDiffCadModel.setBackgroundColor(hexColor);
		};

		/**
		 * Get the transformation for the reference CAD file
		 * @returns {Transformation}
		 */
		CadModule.prototype.getReferenceTransform = function () {
			var layerModel = this._layerModels["referenceCad"];
			if (!layerModel) {
				layerModel = this._layerModels["referenceDiffCad"];
			}
			if (layerModel) {
				return layerModel.getTransform().copy();
			}
		};

		/**
		 * Get the transformation for the CAD overlay
		 * @returns {Transformation}
		 */
		CadModule.prototype.getOverlayTransform = function () {
			var layerModel = this._layerModels["CadOverlay"];
			if (layerModel) {
				return layerModel.getTransform().copy();
			}
		};


		/**
		 * Get the currently active CADGraphics mode
		 * @returns {CadConstants.CadGraphicsMode}
		 */
		CadModule.prototype.getCadGraphicsMode = function () {
			return this._cadModel.getCadGraphicsMode();
		};

		/**
		 * Set the cad graphics mode
		 * @param mode
		 */
		CadModule.prototype.setCadGraphicsMode = function (mode) {
			_.each([this._cadModel,
				this._referenceDiffCadModel,
				this._referenceCadModel,
				this._currentDiffCadModel,
				this._cadOverlayModel
			], function (model) {
				model.setCadGraphicsMode(mode);
			});

			this._updateCadGraphicsMode();
		};

		/**
		 * Toggle the correct layers on the viewport (for CADGraphics rendering)
		 */
		CadModule.prototype._updateCadGraphicsMode = function () {
			var cadGraphicsMode = this.getCadGraphicsMode();

			var showGraphics = cadGraphicsMode === CadConstants.CadGraphicsMode.BOTH || cadGraphicsMode === CadConstants.CadGraphicsMode.GRAPHICS_ONLY;
			var showCad = cadGraphicsMode === CadConstants.CadGraphicsMode.BOTH || cadGraphicsMode === CadConstants.CadGraphicsMode.CAD_ONLY;

			var viewportModule = this.f.getModule("viewport");


			viewportModule.toggleLayer("Graphics", showGraphics);
			_.each(this._layerModels, function (model, layerName) {
				viewportModule.toggleLayer(layerName, showCad);
			});
		};

		return CadModule;
	});
