
//     Underscore.js 1.8.3
//     http://underscorejs.org
//     (c) 2009-2015 Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
//     Underscore may be freely distributed under the MIT license.

(function() {

  // Baseline setup
  // --------------

  // Establish the root object, `window` in the browser, or `exports` on the server.
  var root = this;

  // Save the previous value of the `_` variable.
  var previousUnderscore = root._;

  // Save bytes in the minified (but not gzipped) version:
  var ArrayProto = Array.prototype, ObjProto = Object.prototype, FuncProto = Function.prototype;

  // Create quick reference variables for speed access to core prototypes.
  var
    push             = ArrayProto.push,
    slice            = ArrayProto.slice,
    toString         = ObjProto.toString,
    hasOwnProperty   = ObjProto.hasOwnProperty;

  // All **ECMAScript 5** native function implementations that we hope to use
  // are declared here.
  var
    nativeIsArray      = Array.isArray,
    nativeKeys         = Object.keys,
    nativeBind         = FuncProto.bind,
    nativeCreate       = Object.create;

  // Naked function reference for surrogate-prototype-swapping.
  var Ctor = function(){};

  // Create a safe reference to the Underscore object for use below.
  var _ = function(obj) {
    if (obj instanceof _) return obj;
    if (!(this instanceof _)) return new _(obj);
    this._wrapped = obj;
  };

  // Export the Underscore object for **Node.js**, with
  // backwards-compatibility for the old `require()` API. If we're in
  // the browser, add `_` as a global object.
  if (typeof exports !== 'undefined') {
    if (typeof module !== 'undefined' && module.exports) {
      exports = module.exports = _;
    }
    exports._ = _;
  } else {
    root._ = _;
  }

  // Current version.
  _.VERSION = '1.8.3';

  // Internal function that returns an efficient (for current engines) version
  // of the passed-in callback, to be repeatedly applied in other Underscore
  // functions.
  var optimizeCb = function(func, context, argCount) {
    if (context === void 0) return func;
    switch (argCount == null ? 3 : argCount) {
      case 1: return function(value) {
        return func.call(context, value);
      };
      case 2: return function(value, other) {
        return func.call(context, value, other);
      };
      case 3: return function(value, index, collection) {
        return func.call(context, value, index, collection);
      };
      case 4: return function(accumulator, value, index, collection) {
        return func.call(context, accumulator, value, index, collection);
      };
    }
    return function() {
      return func.apply(context, arguments);
    };
  };

  // A mostly-internal function to generate callbacks that can be applied
  // to each element in a collection, returning the desired result — either
  // identity, an arbitrary callback, a property matcher, or a property accessor.
  var cb = function(value, context, argCount) {
    if (value == null) return _.identity;
    if (_.isFunction(value)) return optimizeCb(value, context, argCount);
    if (_.isObject(value)) return _.matcher(value);
    return _.property(value);
  };
  _.iteratee = function(value, context) {
    return cb(value, context, Infinity);
  };

  // An internal function for creating assigner functions.
  var createAssigner = function(keysFunc, undefinedOnly) {
    return function(obj) {
      var length = arguments.length;
      if (length < 2 || obj == null) return obj;
      for (var index = 1; index < length; index++) {
        var source = arguments[index],
            keys = keysFunc(source),
            l = keys.length;
        for (var i = 0; i < l; i++) {
          var key = keys[i];
          if (!undefinedOnly || obj[key] === void 0) obj[key] = source[key];
        }
      }
      return obj;
    };
  };

  // An internal function for creating a new object that inherits from another.
  var baseCreate = function(prototype) {
    if (!_.isObject(prototype)) return {};
    if (nativeCreate) return nativeCreate(prototype);
    Ctor.prototype = prototype;
    var result = new Ctor;
    Ctor.prototype = null;
    return result;
  };

  var property = function(key) {
    return function(obj) {
      return obj == null ? void 0 : obj[key];
    };
  };

  // Helper for collection methods to determine whether a collection
  // should be iterated as an array or as an object
  // Related: http://people.mozilla.org/~jorendorff/es6-draft.html#sec-tolength
  // Avoids a very nasty iOS 8 JIT bug on ARM-64. #2094
  var MAX_ARRAY_INDEX = Math.pow(2, 53) - 1;
  var getLength = property('length');
  var isArrayLike = function(collection) {
    var length = getLength(collection);
    return typeof length == 'number' && length >= 0 && length <= MAX_ARRAY_INDEX;
  };

  // Collection Functions
  // --------------------

  // The cornerstone, an `each` implementation, aka `forEach`.
  // Handles raw objects in addition to array-likes. Treats all
  // sparse array-likes as if they were dense.
  _.each = _.forEach = function(obj, iteratee, context) {
    iteratee = optimizeCb(iteratee, context);
    var i, length;
    if (isArrayLike(obj)) {
      for (i = 0, length = obj.length; i < length; i++) {
        iteratee(obj[i], i, obj);
      }
    } else {
      var keys = _.keys(obj);
      for (i = 0, length = keys.length; i < length; i++) {
        iteratee(obj[keys[i]], keys[i], obj);
      }
    }
    return obj;
  };

  // Return the results of applying the iteratee to each element.
  _.map = _.collect = function(obj, iteratee, context) {
    iteratee = cb(iteratee, context);
    var keys = !isArrayLike(obj) && _.keys(obj),
        length = (keys || obj).length,
        results = Array(length);
    for (var index = 0; index < length; index++) {
      var currentKey = keys ? keys[index] : index;
      results[index] = iteratee(obj[currentKey], currentKey, obj);
    }
    return results;
  };

  // Create a reducing function iterating left or right.
  function createReduce(dir) {
    // Optimized iterator function as using arguments.length
    // in the main function will deoptimize the, see #1991.
    function iterator(obj, iteratee, memo, keys, index, length) {
      for (; index >= 0 && index < length; index += dir) {
        var currentKey = keys ? keys[index] : index;
        memo = iteratee(memo, obj[currentKey], currentKey, obj);
      }
      return memo;
    }

    return function(obj, iteratee, memo, context) {
      iteratee = optimizeCb(iteratee, context, 4);
      var keys = !isArrayLike(obj) && _.keys(obj),
          length = (keys || obj).length,
          index = dir > 0 ? 0 : length - 1;
      // Determine the initial value if none is provided.
      if (arguments.length < 3) {
        memo = obj[keys ? keys[index] : index];
        index += dir;
      }
      return iterator(obj, iteratee, memo, keys, index, length);
    };
  }

  // **Reduce** builds up a single result from a list of values, aka `inject`,
  // or `foldl`.
  _.reduce = _.foldl = _.inject = createReduce(1);

  // The right-associative version of reduce, also known as `foldr`.
  _.reduceRight = _.foldr = createReduce(-1);

  // Return the first value which passes a truth test. Aliased as `detect`.
  _.find = _.detect = function(obj, predicate, context) {
    var key;
    if (isArrayLike(obj)) {
      key = _.findIndex(obj, predicate, context);
    } else {
      key = _.findKey(obj, predicate, context);
    }
    if (key !== void 0 && key !== -1) return obj[key];
  };

  // Return all the elements that pass a truth test.
  // Aliased as `select`.
  _.filter = _.select = function(obj, predicate, context) {
    var results = [];
    predicate = cb(predicate, context);
    _.each(obj, function(value, index, list) {
      if (predicate(value, index, list)) results.push(value);
    });
    return results;
  };

  // Return all the elements for which a truth test fails.
  _.reject = function(obj, predicate, context) {
    return _.filter(obj, _.negate(cb(predicate)), context);
  };

  // Determine whether all of the elements match a truth test.
  // Aliased as `all`.
  _.every = _.all = function(obj, predicate, context) {
    predicate = cb(predicate, context);
    var keys = !isArrayLike(obj) && _.keys(obj),
        length = (keys || obj).length;
    for (var index = 0; index < length; index++) {
      var currentKey = keys ? keys[index] : index;
      if (!predicate(obj[currentKey], currentKey, obj)) return false;
    }
    return true;
  };

  // Determine if at least one element in the object matches a truth test.
  // Aliased as `any`.
  _.some = _.any = function(obj, predicate, context) {
    predicate = cb(predicate, context);
    var keys = !isArrayLike(obj) && _.keys(obj),
        length = (keys || obj).length;
    for (var index = 0; index < length; index++) {
      var currentKey = keys ? keys[index] : index;
      if (predicate(obj[currentKey], currentKey, obj)) return true;
    }
    return false;
  };

  // Determine if the array or object contains a given item (using `===`).
  // Aliased as `includes` and `include`.
  _.contains = _.includes = _.include = function(obj, item, fromIndex, guard) {
    if (!isArrayLike(obj)) obj = _.values(obj);
    if (typeof fromIndex != 'number' || guard) fromIndex = 0;
    return _.indexOf(obj, item, fromIndex) >= 0;
  };

  // Invoke a method (with arguments) on every item in a collection.
  _.invoke = function(obj, method) {
    var args = slice.call(arguments, 2);
    var isFunc = _.isFunction(method);
    return _.map(obj, function(value) {
      var func = isFunc ? method : value[method];
      return func == null ? func : func.apply(value, args);
    });
  };

  // Convenience version of a common use case of `map`: fetching a property.
  _.pluck = function(obj, key) {
    return _.map(obj, _.property(key));
  };

  // Convenience version of a common use case of `filter`: selecting only objects
  // containing specific `key:value` pairs.
  _.where = function(obj, attrs) {
    return _.filter(obj, _.matcher(attrs));
  };

  // Convenience version of a common use case of `find`: getting the first object
  // containing specific `key:value` pairs.
  _.findWhere = function(obj, attrs) {
    return _.find(obj, _.matcher(attrs));
  };

  // Return the maximum element (or element-based computation).
  _.max = function(obj, iteratee, context) {
    var result = -Infinity, lastComputed = -Infinity,
        value, computed;
    if (iteratee == null && obj != null) {
      obj = isArrayLike(obj) ? obj : _.values(obj);
      for (var i = 0, length = obj.length; i < length; i++) {
        value = obj[i];
        if (value > result) {
          result = value;
        }
      }
    } else {
      iteratee = cb(iteratee, context);
      _.each(obj, function(value, index, list) {
        computed = iteratee(value, index, list);
        if (computed > lastComputed || computed === -Infinity && result === -Infinity) {
          result = value;
          lastComputed = computed;
        }
      });
    }
    return result;
  };

  // Return the minimum element (or element-based computation).
  _.min = function(obj, iteratee, context) {
    var result = Infinity, lastComputed = Infinity,
        value, computed;
    if (iteratee == null && obj != null) {
      obj = isArrayLike(obj) ? obj : _.values(obj);
      for (var i = 0, length = obj.length; i < length; i++) {
        value = obj[i];
        if (value < result) {
          result = value;
        }
      }
    } else {
      iteratee = cb(iteratee, context);
      _.each(obj, function(value, index, list) {
        computed = iteratee(value, index, list);
        if (computed < lastComputed || computed === Infinity && result === Infinity) {
          result = value;
          lastComputed = computed;
        }
      });
    }
    return result;
  };

  // Shuffle a collection, using the modern version of the
  // [Fisher-Yates shuffle](http://en.wikipedia.org/wiki/Fisher–Yates_shuffle).
  _.shuffle = function(obj) {
    var set = isArrayLike(obj) ? obj : _.values(obj);
    var length = set.length;
    var shuffled = Array(length);
    for (var index = 0, rand; index < length; index++) {
      rand = _.random(0, index);
      if (rand !== index) shuffled[index] = shuffled[rand];
      shuffled[rand] = set[index];
    }
    return shuffled;
  };

  // Sample **n** random values from a collection.
  // If **n** is not specified, returns a single random element.
  // The internal `guard` argument allows it to work with `map`.
  _.sample = function(obj, n, guard) {
    if (n == null || guard) {
      if (!isArrayLike(obj)) obj = _.values(obj);
      return obj[_.random(obj.length - 1)];
    }
    return _.shuffle(obj).slice(0, Math.max(0, n));
  };

  // Sort the object's values by a criterion produced by an iteratee.
  _.sortBy = function(obj, iteratee, context) {
    iteratee = cb(iteratee, context);
    return _.pluck(_.map(obj, function(value, index, list) {
      return {
        value: value,
        index: index,
        criteria: iteratee(value, index, list)
      };
    }).sort(function(left, right) {
      var a = left.criteria;
      var b = right.criteria;
      if (a !== b) {
        if (a > b || a === void 0) return 1;
        if (a < b || b === void 0) return -1;
      }
      return left.index - right.index;
    }), 'value');
  };

  // An internal function used for aggregate "group by" operations.
  var group = function(behavior) {
    return function(obj, iteratee, context) {
      var result = {};
      iteratee = cb(iteratee, context);
      _.each(obj, function(value, index) {
        var key = iteratee(value, index, obj);
        behavior(result, value, key);
      });
      return result;
    };
  };

  // Groups the object's values by a criterion. Pass either a string attribute
  // to group by, or a function that returns the criterion.
  _.groupBy = group(function(result, value, key) {
    if (_.has(result, key)) result[key].push(value); else result[key] = [value];
  });

  // Indexes the object's values by a criterion, similar to `groupBy`, but for
  // when you know that your index values will be unique.
  _.indexBy = group(function(result, value, key) {
    result[key] = value;
  });

  // Counts instances of an object that group by a certain criterion. Pass
  // either a string attribute to count by, or a function that returns the
  // criterion.
  _.countBy = group(function(result, value, key) {
    if (_.has(result, key)) result[key]++; else result[key] = 1;
  });

  // Safely create a real, live array from anything iterable.
  _.toArray = function(obj) {
    if (!obj) return [];
    if (_.isArray(obj)) return slice.call(obj);
    if (isArrayLike(obj)) return _.map(obj, _.identity);
    return _.values(obj);
  };

  // Return the number of elements in an object.
  _.size = function(obj) {
    if (obj == null) return 0;
    return isArrayLike(obj) ? obj.length : _.keys(obj).length;
  };

  // Split a collection into two arrays: one whose elements all satisfy the given
  // predicate, and one whose elements all do not satisfy the predicate.
  _.partition = function(obj, predicate, context) {
    predicate = cb(predicate, context);
    var pass = [], fail = [];
    _.each(obj, function(value, key, obj) {
      (predicate(value, key, obj) ? pass : fail).push(value);
    });
    return [pass, fail];
  };

  // Array Functions
  // ---------------

  // Get the first element of an array. Passing **n** will return the first N
  // values in the array. Aliased as `head` and `take`. The **guard** check
  // allows it to work with `_.map`.
  _.first = _.head = _.take = function(array, n, guard) {
    if (array == null) return void 0;
    if (n == null || guard) return array[0];
    return _.initial(array, array.length - n);
  };

  // Returns everything but the last entry of the array. Especially useful on
  // the arguments object. Passing **n** will return all the values in
  // the array, excluding the last N.
  _.initial = function(array, n, guard) {
    return slice.call(array, 0, Math.max(0, array.length - (n == null || guard ? 1 : n)));
  };

  // Get the last element of an array. Passing **n** will return the last N
  // values in the array.
  _.last = function(array, n, guard) {
    if (array == null) return void 0;
    if (n == null || guard) return array[array.length - 1];
    return _.rest(array, Math.max(0, array.length - n));
  };

  // Returns everything but the first entry of the array. Aliased as `tail` and `drop`.
  // Especially useful on the arguments object. Passing an **n** will return
  // the rest N values in the array.
  _.rest = _.tail = _.drop = function(array, n, guard) {
    return slice.call(array, n == null || guard ? 1 : n);
  };

  // Trim out all falsy values from an array.
  _.compact = function(array) {
    return _.filter(array, _.identity);
  };

  // Internal implementation of a recursive `flatten` function.
  var flatten = function(input, shallow, strict, startIndex) {
    var output = [], idx = 0;
    for (var i = startIndex || 0, length = getLength(input); i < length; i++) {
      var value = input[i];
      if (isArrayLike(value) && (_.isArray(value) || _.isArguments(value))) {
        //flatten current level of array or arguments object
        if (!shallow) value = flatten(value, shallow, strict);
        var j = 0, len = value.length;
        output.length += len;
        while (j < len) {
          output[idx++] = value[j++];
        }
      } else if (!strict) {
        output[idx++] = value;
      }
    }
    return output;
  };

  // Flatten out an array, either recursively (by default), or just one level.
  _.flatten = function(array, shallow) {
    return flatten(array, shallow, false);
  };

  // Return a version of the array that does not contain the specified value(s).
  _.without = function(array) {
    return _.difference(array, slice.call(arguments, 1));
  };

  // Produce a duplicate-free version of the array. If the array has already
  // been sorted, you have the option of using a faster algorithm.
  // Aliased as `unique`.
  _.uniq = _.unique = function(array, isSorted, iteratee, context) {
    if (!_.isBoolean(isSorted)) {
      context = iteratee;
      iteratee = isSorted;
      isSorted = false;
    }
    if (iteratee != null) iteratee = cb(iteratee, context);
    var result = [];
    var seen = [];
    for (var i = 0, length = getLength(array); i < length; i++) {
      var value = array[i],
          computed = iteratee ? iteratee(value, i, array) : value;
      if (isSorted) {
        if (!i || seen !== computed) result.push(value);
        seen = computed;
      } else if (iteratee) {
        if (!_.contains(seen, computed)) {
          seen.push(computed);
          result.push(value);
        }
      } else if (!_.contains(result, value)) {
        result.push(value);
      }
    }
    return result;
  };

  // Produce an array that contains the union: each distinct element from all of
  // the passed-in arrays.
  _.union = function() {
    return _.uniq(flatten(arguments, true, true));
  };

  // Produce an array that contains every item shared between all the
  // passed-in arrays.
  _.intersection = function(array) {
    var result = [];
    var argsLength = arguments.length;
    for (var i = 0, length = getLength(array); i < length; i++) {
      var item = array[i];
      if (_.contains(result, item)) continue;
      for (var j = 1; j < argsLength; j++) {
        if (!_.contains(arguments[j], item)) break;
      }
      if (j === argsLength) result.push(item);
    }
    return result;
  };

  // Take the difference between one array and a number of other arrays.
  // Only the elements present in just the first array will remain.
  _.difference = function(array) {
    var rest = flatten(arguments, true, true, 1);
    return _.filter(array, function(value){
      return !_.contains(rest, value);
    });
  };

  // Zip together multiple lists into a single array -- elements that share
  // an index go together.
  _.zip = function() {
    return _.unzip(arguments);
  };

  // Complement of _.zip. Unzip accepts an array of arrays and groups
  // each array's elements on shared indices
  _.unzip = function(array) {
    var length = array && _.max(array, getLength).length || 0;
    var result = Array(length);

    for (var index = 0; index < length; index++) {
      result[index] = _.pluck(array, index);
    }
    return result;
  };

  // Converts lists into objects. Pass either a single array of `[key, value]`
  // pairs, or two parallel arrays of the same length -- one of keys, and one of
  // the corresponding values.
  _.object = function(list, values) {
    var result = {};
    for (var i = 0, length = getLength(list); i < length; i++) {
      if (values) {
        result[list[i]] = values[i];
      } else {
        result[list[i][0]] = list[i][1];
      }
    }
    return result;
  };

  // Generator function to create the findIndex and findLastIndex functions
  function createPredicateIndexFinder(dir) {
    return function(array, predicate, context) {
      predicate = cb(predicate, context);
      var length = getLength(array);
      var index = dir > 0 ? 0 : length - 1;
      for (; index >= 0 && index < length; index += dir) {
        if (predicate(array[index], index, array)) return index;
      }
      return -1;
    };
  }

  // Returns the first index on an array-like that passes a predicate test
  _.findIndex = createPredicateIndexFinder(1);
  _.findLastIndex = createPredicateIndexFinder(-1);

  // Use a comparator function to figure out the smallest index at which
  // an object should be inserted so as to maintain order. Uses binary search.
  _.sortedIndex = function(array, obj, iteratee, context) {
    iteratee = cb(iteratee, context, 1);
    var value = iteratee(obj);
    var low = 0, high = getLength(array);
    while (low < high) {
      var mid = Math.floor((low + high) / 2);
      if (iteratee(array[mid]) < value) low = mid + 1; else high = mid;
    }
    return low;
  };

  // Generator function to create the indexOf and lastIndexOf functions
  function createIndexFinder(dir, predicateFind, sortedIndex) {
    return function(array, item, idx) {
      var i = 0, length = getLength(array);
      if (typeof idx == 'number') {
        if (dir > 0) {
            i = idx >= 0 ? idx : Math.max(idx + length, i);
        } else {
            length = idx >= 0 ? Math.min(idx + 1, length) : idx + length + 1;
        }
      } else if (sortedIndex && idx && length) {
        idx = sortedIndex(array, item);
        return array[idx] === item ? idx : -1;
      }
      if (item !== item) {
        idx = predicateFind(slice.call(array, i, length), _.isNaN);
        return idx >= 0 ? idx + i : -1;
      }
      for (idx = dir > 0 ? i : length - 1; idx >= 0 && idx < length; idx += dir) {
        if (array[idx] === item) return idx;
      }
      return -1;
    };
  }

  // Return the position of the first occurrence of an item in an array,
  // or -1 if the item is not included in the array.
  // If the array is large and already in sort order, pass `true`
  // for **isSorted** to use binary search.
  _.indexOf = createIndexFinder(1, _.findIndex, _.sortedIndex);
  _.lastIndexOf = createIndexFinder(-1, _.findLastIndex);

  // Generate an integer Array containing an arithmetic progression. A port of
  // the native Python `range()` function. See
  // [the Python documentation](http://docs.python.org/library/functions.html#range).
  _.range = function(start, stop, step) {
    if (stop == null) {
      stop = start || 0;
      start = 0;
    }
    step = step || 1;

    var length = Math.max(Math.ceil((stop - start) / step), 0);
    var range = Array(length);

    for (var idx = 0; idx < length; idx++, start += step) {
      range[idx] = start;
    }

    return range;
  };

  // Function (ahem) Functions
  // ------------------

  // Determines whether to execute a function as a constructor
  // or a normal function with the provided arguments
  var executeBound = function(sourceFunc, boundFunc, context, callingContext, args) {
    if (!(callingContext instanceof boundFunc)) return sourceFunc.apply(context, args);
    var self = baseCreate(sourceFunc.prototype);
    var result = sourceFunc.apply(self, args);
    if (_.isObject(result)) return result;
    return self;
  };

  // Create a function bound to a given object (assigning `this`, and arguments,
  // optionally). Delegates to **ECMAScript 5**'s native `Function.bind` if
  // available.
  _.bind = function(func, context) {
    if (nativeBind && func.bind === nativeBind) return nativeBind.apply(func, slice.call(arguments, 1));
    if (!_.isFunction(func)) throw new TypeError('Bind must be called on a function');
    var args = slice.call(arguments, 2);
    var bound = function() {
      return executeBound(func, bound, context, this, args.concat(slice.call(arguments)));
    };
    return bound;
  };

  // Partially apply a function by creating a version that has had some of its
  // arguments pre-filled, without changing its dynamic `this` context. _ acts
  // as a placeholder, allowing any combination of arguments to be pre-filled.
  _.partial = function(func) {
    var boundArgs = slice.call(arguments, 1);
    var bound = function() {
      var position = 0, length = boundArgs.length;
      var args = Array(length);
      for (var i = 0; i < length; i++) {
        args[i] = boundArgs[i] === _ ? arguments[position++] : boundArgs[i];
      }
      while (position < arguments.length) args.push(arguments[position++]);
      return executeBound(func, bound, this, this, args);
    };
    return bound;
  };

  // Bind a number of an object's methods to that object. Remaining arguments
  // are the method names to be bound. Useful for ensuring that all callbacks
  // defined on an object belong to it.
  _.bindAll = function(obj) {
    var i, length = arguments.length, key;
    if (length <= 1) throw new Error('bindAll must be passed function names');
    for (i = 1; i < length; i++) {
      key = arguments[i];
      obj[key] = _.bind(obj[key], obj);
    }
    return obj;
  };

  // Memoize an expensive function by storing its results.
  _.memoize = function(func, hasher) {
    var memoize = function(key) {
      var cache = memoize.cache;
      var address = '' + (hasher ? hasher.apply(this, arguments) : key);
      if (!_.has(cache, address)) cache[address] = func.apply(this, arguments);
      return cache[address];
    };
    memoize.cache = {};
    return memoize;
  };

  // Delays a function for the given number of milliseconds, and then calls
  // it with the arguments supplied.
  _.delay = function(func, wait) {
    var args = slice.call(arguments, 2);
    return setTimeout(function(){
      return func.apply(null, args);
    }, wait);
  };

  // Defers a function, scheduling it to run after the current call stack has
  // cleared.
  _.defer = _.partial(_.delay, _, 1);

  // Returns a function, that, when invoked, will only be triggered at most once
  // during a given window of time. Normally, the throttled function will run
  // as much as it can, without ever going more than once per `wait` duration;
  // but if you'd like to disable the execution on the leading edge, pass
  // `{leading: false}`. To disable execution on the trailing edge, ditto.
  _.throttle = function(func, wait, options) {
    var context, args, result;
    var timeout = null;
    var previous = 0;
    if (!options) options = {};
    var later = function() {
      previous = options.leading === false ? 0 : _.now();
      timeout = null;
      result = func.apply(context, args);
      if (!timeout) context = args = null;
    };
    return function() {
      var now = _.now();
      if (!previous && options.leading === false) previous = now;
      var remaining = wait - (now - previous);
      context = this;
      args = arguments;
      if (remaining <= 0 || remaining > wait) {
        if (timeout) {
          clearTimeout(timeout);
          timeout = null;
        }
        previous = now;
        result = func.apply(context, args);
        if (!timeout) context = args = null;
      } else if (!timeout && options.trailing !== false) {
        timeout = setTimeout(later, remaining);
      }
      return result;
    };
  };

  // Returns a function, that, as long as it continues to be invoked, will not
  // be triggered. The function will be called after it stops being called for
  // N milliseconds. If `immediate` is passed, trigger the function on the
  // leading edge, instead of the trailing.
  _.debounce = function(func, wait, immediate) {
    var timeout, args, context, timestamp, result;

    var later = function() {
      var last = _.now() - timestamp;

      if (last < wait && last >= 0) {
        timeout = setTimeout(later, wait - last);
      } else {
        timeout = null;
        if (!immediate) {
          result = func.apply(context, args);
          if (!timeout) context = args = null;
        }
      }
    };

    return function() {
      context = this;
      args = arguments;
      timestamp = _.now();
      var callNow = immediate && !timeout;
      if (!timeout) timeout = setTimeout(later, wait);
      if (callNow) {
        result = func.apply(context, args);
        context = args = null;
      }

      return result;
    };
  };

  // Returns the first function passed as an argument to the second,
  // allowing you to adjust arguments, run code before and after, and
  // conditionally execute the original function.
  _.wrap = function(func, wrapper) {
    return _.partial(wrapper, func);
  };

  // Returns a negated version of the passed-in predicate.
  _.negate = function(predicate) {
    return function() {
      return !predicate.apply(this, arguments);
    };
  };

  // Returns a function that is the composition of a list of functions, each
  // consuming the return value of the function that follows.
  _.compose = function() {
    var args = arguments;
    var start = args.length - 1;
    return function() {
      var i = start;
      var result = args[start].apply(this, arguments);
      while (i--) result = args[i].call(this, result);
      return result;
    };
  };

  // Returns a function that will only be executed on and after the Nth call.
  _.after = function(times, func) {
    return function() {
      if (--times < 1) {
        return func.apply(this, arguments);
      }
    };
  };

  // Returns a function that will only be executed up to (but not including) the Nth call.
  _.before = function(times, func) {
    var memo;
    return function() {
      if (--times > 0) {
        memo = func.apply(this, arguments);
      }
      if (times <= 1) func = null;
      return memo;
    };
  };

  // Returns a function that will be executed at most one time, no matter how
  // often you call it. Useful for lazy initialization.
  _.once = _.partial(_.before, 2);

  // Object Functions
  // ----------------

  // Keys in IE < 9 that won't be iterated by `for key in ...` and thus missed.
  var hasEnumBug = !{toString: null}.propertyIsEnumerable('toString');
  var nonEnumerableProps = ['valueOf', 'isPrototypeOf', 'toString',
                      'propertyIsEnumerable', 'hasOwnProperty', 'toLocaleString'];

  function collectNonEnumProps(obj, keys) {
    var nonEnumIdx = nonEnumerableProps.length;
    var constructor = obj.constructor;
    var proto = (_.isFunction(constructor) && constructor.prototype) || ObjProto;

    // Constructor is a special case.
    var prop = 'constructor';
    if (_.has(obj, prop) && !_.contains(keys, prop)) keys.push(prop);

    while (nonEnumIdx--) {
      prop = nonEnumerableProps[nonEnumIdx];
      if (prop in obj && obj[prop] !== proto[prop] && !_.contains(keys, prop)) {
        keys.push(prop);
      }
    }
  }

  // Retrieve the names of an object's own properties.
  // Delegates to **ECMAScript 5**'s native `Object.keys`
  _.keys = function(obj) {
    if (!_.isObject(obj)) return [];
    if (nativeKeys) return nativeKeys(obj);
    var keys = [];
    for (var key in obj) if (_.has(obj, key)) keys.push(key);
    // Ahem, IE < 9.
    if (hasEnumBug) collectNonEnumProps(obj, keys);
    return keys;
  };

  // Retrieve all the property names of an object.
  _.allKeys = function(obj) {
    if (!_.isObject(obj)) return [];
    var keys = [];
    for (var key in obj) keys.push(key);
    // Ahem, IE < 9.
    if (hasEnumBug) collectNonEnumProps(obj, keys);
    return keys;
  };

  // Retrieve the values of an object's properties.
  _.values = function(obj) {
    var keys = _.keys(obj);
    var length = keys.length;
    var values = Array(length);
    for (var i = 0; i < length; i++) {
      values[i] = obj[keys[i]];
    }
    return values;
  };

  // Returns the results of applying the iteratee to each element of the object
  // In contrast to _.map it returns an object
  _.mapObject = function(obj, iteratee, context) {
    iteratee = cb(iteratee, context);
    var keys =  _.keys(obj),
          length = keys.length,
          results = {},
          currentKey;
      for (var index = 0; index < length; index++) {
        currentKey = keys[index];
        results[currentKey] = iteratee(obj[currentKey], currentKey, obj);
      }
      return results;
  };

  // Convert an object into a list of `[key, value]` pairs.
  _.pairs = function(obj) {
    var keys = _.keys(obj);
    var length = keys.length;
    var pairs = Array(length);
    for (var i = 0; i < length; i++) {
      pairs[i] = [keys[i], obj[keys[i]]];
    }
    return pairs;
  };

  // Invert the keys and values of an object. The values must be serializable.
  _.invert = function(obj) {
    var result = {};
    var keys = _.keys(obj);
    for (var i = 0, length = keys.length; i < length; i++) {
      result[obj[keys[i]]] = keys[i];
    }
    return result;
  };

  // Return a sorted list of the function names available on the object.
  // Aliased as `methods`
  _.functions = _.methods = function(obj) {
    var names = [];
    for (var key in obj) {
      if (_.isFunction(obj[key])) names.push(key);
    }
    return names.sort();
  };

  // Extend a given object with all the properties in passed-in object(s).
  _.extend = createAssigner(_.allKeys);

  // Assigns a given object with all the own properties in the passed-in object(s)
  // (https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object/assign)
  _.extendOwn = _.assign = createAssigner(_.keys);

  // Returns the first key on an object that passes a predicate test
  _.findKey = function(obj, predicate, context) {
    predicate = cb(predicate, context);
    var keys = _.keys(obj), key;
    for (var i = 0, length = keys.length; i < length; i++) {
      key = keys[i];
      if (predicate(obj[key], key, obj)) return key;
    }
  };

  // Return a copy of the object only containing the whitelisted properties.
  _.pick = function(object, oiteratee, context) {
    var result = {}, obj = object, iteratee, keys;
    if (obj == null) return result;
    if (_.isFunction(oiteratee)) {
      keys = _.allKeys(obj);
      iteratee = optimizeCb(oiteratee, context);
    } else {
      keys = flatten(arguments, false, false, 1);
      iteratee = function(value, key, obj) { return key in obj; };
      obj = Object(obj);
    }
    for (var i = 0, length = keys.length; i < length; i++) {
      var key = keys[i];
      var value = obj[key];
      if (iteratee(value, key, obj)) result[key] = value;
    }
    return result;
  };

   // Return a copy of the object without the blacklisted properties.
  _.omit = function(obj, iteratee, context) {
    if (_.isFunction(iteratee)) {
      iteratee = _.negate(iteratee);
    } else {
      var keys = _.map(flatten(arguments, false, false, 1), String);
      iteratee = function(value, key) {
        return !_.contains(keys, key);
      };
    }
    return _.pick(obj, iteratee, context);
  };

  // Fill in a given object with default properties.
  _.defaults = createAssigner(_.allKeys, true);

  // Creates an object that inherits from the given prototype object.
  // If additional properties are provided then they will be added to the
  // created object.
  _.create = function(prototype, props) {
    var result = baseCreate(prototype);
    if (props) _.extendOwn(result, props);
    return result;
  };

  // Create a (shallow-cloned) duplicate of an object.
  _.clone = function(obj) {
    if (!_.isObject(obj)) return obj;
    return _.isArray(obj) ? obj.slice() : _.extend({}, obj);
  };

  // Invokes interceptor with the obj, and then returns obj.
  // The primary purpose of this method is to "tap into" a method chain, in
  // order to perform operations on intermediate results within the chain.
  _.tap = function(obj, interceptor) {
    interceptor(obj);
    return obj;
  };

  // Returns whether an object has a given set of `key:value` pairs.
  _.isMatch = function(object, attrs) {
    var keys = _.keys(attrs), length = keys.length;
    if (object == null) return !length;
    var obj = Object(object);
    for (var i = 0; i < length; i++) {
      var key = keys[i];
      if (attrs[key] !== obj[key] || !(key in obj)) return false;
    }
    return true;
  };


  // Internal recursive comparison function for `isEqual`.
  var eq = function(a, b, aStack, bStack) {
    // Identical objects are equal. `0 === -0`, but they aren't identical.
    // See the [Harmony `egal` proposal](http://wiki.ecmascript.org/doku.php?id=harmony:egal).
    if (a === b) return a !== 0 || 1 / a === 1 / b;
    // A strict comparison is necessary because `null == undefined`.
    if (a == null || b == null) return a === b;
    // Unwrap any wrapped objects.
    if (a instanceof _) a = a._wrapped;
    if (b instanceof _) b = b._wrapped;
    // Compare `[[Class]]` names.
    var className = toString.call(a);
    if (className !== toString.call(b)) return false;
    switch (className) {
      // Strings, numbers, regular expressions, dates, and booleans are compared by value.
      case '[object RegExp]':
      // RegExps are coerced to strings for comparison (Note: '' + /a/i === '/a/i')
      case '[object String]':
        // Primitives and their corresponding object wrappers are equivalent; thus, `"5"` is
        // equivalent to `new String("5")`.
        return '' + a === '' + b;
      case '[object Number]':
        // `NaN`s are equivalent, but non-reflexive.
        // Object(NaN) is equivalent to NaN
        if (+a !== +a) return +b !== +b;
        // An `egal` comparison is performed for other numeric values.
        return +a === 0 ? 1 / +a === 1 / b : +a === +b;
      case '[object Date]':
      case '[object Boolean]':
        // Coerce dates and booleans to numeric primitive values. Dates are compared by their
        // millisecond representations. Note that invalid dates with millisecond representations
        // of `NaN` are not equivalent.
        return +a === +b;
    }

    var areArrays = className === '[object Array]';
    if (!areArrays) {
      if (typeof a != 'object' || typeof b != 'object') return false;

      // Objects with different constructors are not equivalent, but `Object`s or `Array`s
      // from different frames are.
      var aCtor = a.constructor, bCtor = b.constructor;
      if (aCtor !== bCtor && !(_.isFunction(aCtor) && aCtor instanceof aCtor &&
                               _.isFunction(bCtor) && bCtor instanceof bCtor)
                          && ('constructor' in a && 'constructor' in b)) {
        return false;
      }
    }
    // Assume equality for cyclic structures. The algorithm for detecting cyclic
    // structures is adapted from ES 5.1 section 15.12.3, abstract operation `JO`.

    // Initializing stack of traversed objects.
    // It's done here since we only need them for objects and arrays comparison.
    aStack = aStack || [];
    bStack = bStack || [];
    var length = aStack.length;
    while (length--) {
      // Linear search. Performance is inversely proportional to the number of
      // unique nested structures.
      if (aStack[length] === a) return bStack[length] === b;
    }

    // Add the first object to the stack of traversed objects.
    aStack.push(a);
    bStack.push(b);

    // Recursively compare objects and arrays.
    if (areArrays) {
      // Compare array lengths to determine if a deep comparison is necessary.
      length = a.length;
      if (length !== b.length) return false;
      // Deep compare the contents, ignoring non-numeric properties.
      while (length--) {
        if (!eq(a[length], b[length], aStack, bStack)) return false;
      }
    } else {
      // Deep compare objects.
      var keys = _.keys(a), key;
      length = keys.length;
      // Ensure that both objects contain the same number of properties before comparing deep equality.
      if (_.keys(b).length !== length) return false;
      while (length--) {
        // Deep compare each member
        key = keys[length];
        if (!(_.has(b, key) && eq(a[key], b[key], aStack, bStack))) return false;
      }
    }
    // Remove the first object from the stack of traversed objects.
    aStack.pop();
    bStack.pop();
    return true;
  };

  // Perform a deep comparison to check if two objects are equal.
  _.isEqual = function(a, b) {
    return eq(a, b);
  };

  // Is a given array, string, or object empty?
  // An "empty" object has no enumerable own-properties.
  _.isEmpty = function(obj) {
    if (obj == null) return true;
    if (isArrayLike(obj) && (_.isArray(obj) || _.isString(obj) || _.isArguments(obj))) return obj.length === 0;
    return _.keys(obj).length === 0;
  };

  // Is a given value a DOM element?
  _.isElement = function(obj) {
    return !!(obj && obj.nodeType === 1);
  };

  // Is a given value an array?
  // Delegates to ECMA5's native Array.isArray
  _.isArray = nativeIsArray || function(obj) {
    return toString.call(obj) === '[object Array]';
  };

  // Is a given variable an object?
  _.isObject = function(obj) {
    var type = typeof obj;
    return type === 'function' || type === 'object' && !!obj;
  };

  // Add some isType methods: isArguments, isFunction, isString, isNumber, isDate, isRegExp, isError.
  _.each(['Arguments', 'Function', 'String', 'Number', 'Date', 'RegExp', 'Error'], function(name) {
    _['is' + name] = function(obj) {
      return toString.call(obj) === '[object ' + name + ']';
    };
  });

  // Define a fallback version of the method in browsers (ahem, IE < 9), where
  // there isn't any inspectable "Arguments" type.
  if (!_.isArguments(arguments)) {
    _.isArguments = function(obj) {
      return _.has(obj, 'callee');
    };
  }

  // Optimize `isFunction` if appropriate. Work around some typeof bugs in old v8,
  // IE 11 (#1621), and in Safari 8 (#1929).
  if (typeof /./ != 'function' && typeof Int8Array != 'object') {
    _.isFunction = function(obj) {
      return typeof obj == 'function' || false;
    };
  }

  // Is a given object a finite number?
  _.isFinite = function(obj) {
    return isFinite(obj) && !isNaN(parseFloat(obj));
  };

  // Is the given value `NaN`? (NaN is the only number which does not equal itself).
  _.isNaN = function(obj) {
    return _.isNumber(obj) && obj !== +obj;
  };

  // Is a given value a boolean?
  _.isBoolean = function(obj) {
    return obj === true || obj === false || toString.call(obj) === '[object Boolean]';
  };

  // Is a given value equal to null?
  _.isNull = function(obj) {
    return obj === null;
  };

  // Is a given variable undefined?
  _.isUndefined = function(obj) {
    return obj === void 0;
  };

  // Shortcut function for checking if an object has a given property directly
  // on itself (in other words, not on a prototype).
  _.has = function(obj, key) {
    return obj != null && hasOwnProperty.call(obj, key);
  };

  // Utility Functions
  // -----------------

  // Run Underscore.js in *noConflict* mode, returning the `_` variable to its
  // previous owner. Returns a reference to the Underscore object.
  _.noConflict = function() {
    root._ = previousUnderscore;
    return this;
  };

  // Keep the identity function around for default iteratees.
  _.identity = function(value) {
    return value;
  };

  // Predicate-generating functions. Often useful outside of Underscore.
  _.constant = function(value) {
    return function() {
      return value;
    };
  };

  _.noop = function(){};

  _.property = property;

  // Generates a function for a given object that returns a given property.
  _.propertyOf = function(obj) {
    return obj == null ? function(){} : function(key) {
      return obj[key];
    };
  };

  // Returns a predicate for checking whether an object has a given set of
  // `key:value` pairs.
  _.matcher = _.matches = function(attrs) {
    attrs = _.extendOwn({}, attrs);
    return function(obj) {
      return _.isMatch(obj, attrs);
    };
  };

  // Run a function **n** times.
  _.times = function(n, iteratee, context) {
    var accum = Array(Math.max(0, n));
    iteratee = optimizeCb(iteratee, context, 1);
    for (var i = 0; i < n; i++) accum[i] = iteratee(i);
    return accum;
  };

  // Return a random integer between min and max (inclusive).
  _.random = function(min, max) {
    if (max == null) {
      max = min;
      min = 0;
    }
    return min + Math.floor(Math.random() * (max - min + 1));
  };

  // A (possibly faster) way to get the current timestamp as an integer.
  _.now = Date.now || function() {
    return new Date().getTime();
  };

   // List of HTML entities for escaping.
  var escapeMap = {
    '&': '&amp;',
    '<': '&lt;',
    '>': '&gt;',
    '"': '&quot;',
    "'": '&#x27;',
    '`': '&#x60;'
  };
  var unescapeMap = _.invert(escapeMap);

  // Functions for escaping and unescaping strings to/from HTML interpolation.
  var createEscaper = function(map) {
    var escaper = function(match) {
      return map[match];
    };
    // Regexes for identifying a key that needs to be escaped
    var source = '(?:' + _.keys(map).join('|') + ')';
    var testRegexp = RegExp(source);
    var replaceRegexp = RegExp(source, 'g');
    return function(string) {
      string = string == null ? '' : '' + string;
      return testRegexp.test(string) ? string.replace(replaceRegexp, escaper) : string;
    };
  };
  _.escape = createEscaper(escapeMap);
  _.unescape = createEscaper(unescapeMap);

  // If the value of the named `property` is a function then invoke it with the
  // `object` as context; otherwise, return it.
  _.result = function(object, property, fallback) {
    var value = object == null ? void 0 : object[property];
    if (value === void 0) {
      value = fallback;
    }
    return _.isFunction(value) ? value.call(object) : value;
  };

  // Generate a unique integer id (unique within the entire client session).
  // Useful for temporary DOM ids.
  var idCounter = 0;
  _.uniqueId = function(prefix) {
    var id = ++idCounter + '';
    return prefix ? prefix + id : id;
  };

  // By default, Underscore uses ERB-style template delimiters, change the
  // following template settings to use alternative delimiters.
  _.templateSettings = {
    evaluate    : /<%([\s\S]+?)%>/g,
    interpolate : /<%=([\s\S]+?)%>/g,
    escape      : /<%-([\s\S]+?)%>/g
  };

  // When customizing `templateSettings`, if you don't want to define an
  // interpolation, evaluation or escaping regex, we need one that is
  // guaranteed not to match.
  var noMatch = /(.)^/;

  // Certain characters need to be escaped so that they can be put into a
  // string literal.
  var escapes = {
    "'":      "'",
    '\\':     '\\',
    '\r':     'r',
    '\n':     'n',
    '\u2028': 'u2028',
    '\u2029': 'u2029'
  };

  var escaper = /\\|'|\r|\n|\u2028|\u2029/g;

  var escapeChar = function(match) {
    return '\\' + escapes[match];
  };

  // JavaScript micro-templating, similar to John Resig's implementation.
  // Underscore templating handles arbitrary delimiters, preserves whitespace,
  // and correctly escapes quotes within interpolated code.
  // NB: `oldSettings` only exists for backwards compatibility.
  _.template = function(text, settings, oldSettings) {
    if (!settings && oldSettings) settings = oldSettings;
    settings = _.defaults({}, settings, _.templateSettings);

    // Combine delimiters into one regular expression via alternation.
    var matcher = RegExp([
      (settings.escape || noMatch).source,
      (settings.interpolate || noMatch).source,
      (settings.evaluate || noMatch).source
    ].join('|') + '|$', 'g');

    // Compile the template source, escaping string literals appropriately.
    var index = 0;
    var source = "__p+='";
    text.replace(matcher, function(match, escape, interpolate, evaluate, offset) {
      source += text.slice(index, offset).replace(escaper, escapeChar);
      index = offset + match.length;

      if (escape) {
        source += "'+\n((__t=(" + escape + "))==null?'':_.escape(__t))+\n'";
      } else if (interpolate) {
        source += "'+\n((__t=(" + interpolate + "))==null?'':__t)+\n'";
      } else if (evaluate) {
        source += "';\n" + evaluate + "\n__p+='";
      }

      // Adobe VMs need the match returned to produce the correct offest.
      return match;
    });
    source += "';\n";

    // If a variable is not specified, place data values in local scope.
    if (!settings.variable) source = 'with(obj||{}){\n' + source + '}\n';

    source = "var __t,__p='',__j=Array.prototype.join," +
      "print=function(){__p+=__j.call(arguments,'');};\n" +
      source + 'return __p;\n';

    try {
      var render = new Function(settings.variable || 'obj', '_', source);
    } catch (e) {
      e.source = source;
      throw e;
    }

    var template = function(data) {
      return render.call(this, data, _);
    };

    // Provide the compiled source as a convenience for precompilation.
    var argument = settings.variable || 'obj';
    template.source = 'function(' + argument + '){\n' + source + '}';

    return template;
  };

  // Add a "chain" function. Start chaining a wrapped Underscore object.
  _.chain = function(obj) {
    var instance = _(obj);
    instance._chain = true;
    return instance;
  };

  // OOP
  // ---------------
  // If Underscore is called as a function, it returns a wrapped object that
  // can be used OO-style. This wrapper holds altered versions of all the
  // underscore functions. Wrapped objects may be chained.

  // Helper function to continue chaining intermediate results.
  var result = function(instance, obj) {
    return instance._chain ? _(obj).chain() : obj;
  };

  // Add your own custom functions to the Underscore object.
  _.mixin = function(obj) {
    _.each(_.functions(obj), function(name) {
      var func = _[name] = obj[name];
      _.prototype[name] = function() {
        var args = [this._wrapped];
        push.apply(args, arguments);
        return result(this, func.apply(_, args));
      };
    });
  };

  // Add all of the Underscore functions to the wrapper object.
  _.mixin(_);

  // Add all mutator Array functions to the wrapper.
  _.each(['pop', 'push', 'reverse', 'shift', 'sort', 'splice', 'unshift'], function(name) {
    var method = ArrayProto[name];
    _.prototype[name] = function() {
      var obj = this._wrapped;
      method.apply(obj, arguments);
      if ((name === 'shift' || name === 'splice') && obj.length === 0) delete obj[0];
      return result(this, obj);
    };
  });

  // Add all accessor Array functions to the wrapper.
  _.each(['concat', 'join', 'slice'], function(name) {
    var method = ArrayProto[name];
    _.prototype[name] = function() {
      return result(this, method.apply(this._wrapped, arguments));
    };
  });

  // Extracts the result from a wrapped and chained object.
  _.prototype.value = function() {
    return this._wrapped;
  };

  // Provide unwrapping proxy for some methods used in engine operations
  // such as arithmetic and JSON stringification.
  _.prototype.valueOf = _.prototype.toJSON = _.prototype.value;

  _.prototype.toString = function() {
    return '' + this._wrapped;
  };

  // AMD registration happens at the end for compatibility with AMD loaders
  // that may not enforce next-turn semantics on modules. Even though general
  // practice for AMD registration is to be anonymous, underscore registers
  // as a named module because, like jQuery, it is a base library that is
  // popular enough to be bundled in a third party lib, but not be part of
  // an AMD load request. Those cases could generate an error when an
  // anonymous define() is called outside of a loader request.
  if (typeof define === 'function' && define.amd) {
    define('underscore', [], function() {
      return _;
    });
  }
}.call(this));

define('lib/EasingFunctions',[],
	function () {
		/*
		 * jQuery Easing v1.3 - http://gsgd.co.uk/sandbox/jquery/easing/
		 *
		 * Uses the built in easing capabilities added In jQuery 1.1
		 * to offer multiple easing options
		 *
		 * TERMS OF USE - jQuery Easing
		 *
		 * Open source under the BSD License.
		 *
		 * Copyright Â© 2008 George McGinley Smith
		 * All rights reserved.
		 *
		 * Redistribution and use in source and binary forms, with or without modification,
		 * are permitted provided that the following conditions are met:
		 *
		 * Redistributions of source code must retain the above copyright notice, this list of
		 * conditions and the following disclaimer.
		 * Redistributions in binary form must reproduce the above copyright notice, this list
		 * of conditions and the following disclaimer in the documentation and/or other materials
		 * provided with the distribution.
		 *
		 * Neither the name of the author nor the names of contributors may be used to endorse
		 * or promote products derived from this software without specific prior written permission.
		 *
		 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
		 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
		 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
		 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
		 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
		 *  GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
		 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
		 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
		 * OF THE POSSIBILITY OF SUCH DAMAGE.
		 *
		 */

		var EasingFunctions = {
			easeInQuad: function (x, t, b, c, d) {
				return c * (t /= d) * t + b;
			},
			easeOutQuad: function (x, t, b, c, d) {
				return -c * (t /= d) * (t - 2) + b;
			},
			easeInOutQuad: function (x, t, b, c, d) {
				if ((t /= d / 2) < 1) return c / 2 * t * t + b;
				return -c / 2 * ((--t) * (t - 2) - 1) + b;
			},
			easeInCubic: function (x, t, b, c, d) {
				return c * (t /= d) * t * t + b;
			},
			easeOutCubic: function (x, t, b, c, d) {
				return c * ((t = t / d - 1) * t * t + 1) + b;
			},
			easeInOutCubic: function (x, t, b, c, d) {
				if ((t /= d / 2) < 1) return c / 2 * t * t * t + b;
				return c / 2 * ((t -= 2) * t * t + 2) + b;
			},
			easeInQuart: function (x, t, b, c, d) {
				return c * (t /= d) * t * t * t + b;
			},
			easeOutQuart: function (x, t, b, c, d) {
				return -c * ((t = t / d - 1) * t * t * t - 1) + b;
			},
			easeInOutQuart: function (x, t, b, c, d) {
				if ((t /= d / 2) < 1) return c / 2 * t * t * t * t + b;
				return -c / 2 * ((t -= 2) * t * t * t - 2) + b;
			},
			easeInQuint: function (x, t, b, c, d) {
				return c * (t /= d) * t * t * t * t + b;
			},
			easeOutQuint: function (x, t, b, c, d) {
				return c * ((t = t / d - 1) * t * t * t * t + 1) + b;
			},
			easeInOutQuint: function (x, t, b, c, d) {
				if ((t /= d / 2) < 1) return c / 2 * t * t * t * t * t + b;
				return c / 2 * ((t -= 2) * t * t * t * t + 2) + b;
			},
			easeInSine: function (x, t, b, c, d) {
				return -c * Math.cos(t / d * (Math.PI / 2)) + c + b;
			},
			easeOutSine: function (x, t, b, c, d) {
				return c * Math.sin(t / d * (Math.PI / 2)) + b;
			},
			easeInOutSine: function (x, t, b, c, d) {
				return -c / 2 * (Math.cos(Math.PI * t / d) - 1) + b;
			},
			easeInExpo: function (x, t, b, c, d) {
				return (t == 0) ? b : c * Math.pow(2, 10 * (t / d - 1)) + b;
			},
			easeOutExpo: function (x, t, b, c, d) {
				return (t == d) ? b + c : c * (-Math.pow(2, -10 * t / d) + 1) + b;
			},
			easeInOutExpo: function (x, t, b, c, d) {
				if (t == 0) return b;
				if (t == d) return b + c;
				if ((t /= d / 2) < 1) return c / 2 * Math.pow(2, 10 * (t - 1)) + b;
				return c / 2 * (-Math.pow(2, -10 * --t) + 2) + b;
			},
			easeInCirc: function (x, t, b, c, d) {
				return -c * (Math.sqrt(1 - (t /= d) * t) - 1) + b;
			},
			easeOutCirc: function (x, t, b, c, d) {
				return c * Math.sqrt(1 - (t = t / d - 1) * t) + b;
			},
			easeInOutCirc: function (x, t, b, c, d) {
				if ((t /= d / 2) < 1) return -c / 2 * (Math.sqrt(1 - t * t) - 1) + b;
				return c / 2 * (Math.sqrt(1 - (t -= 2) * t) + 1) + b;
			},
			easeInElastic: function (x, t, b, c, d) {
				var s = 1.70158;
				var p = 0;
				var a = c;
				if (t == 0) return b;
				if ((t /= d) == 1) return b + c;
				if (!p) p = d * .3;
				if (a < Math.abs(c)) {
					a = c;
					var s = p / 4;
				} else var s = p / (2 * Math.PI) * Math.asin(c / a);
				return -(a * Math.pow(2, 10 * (t -= 1)) * Math.sin((t * d - s) * (2 * Math.PI) / p)) + b;
			},
			easeOutElastic: function (x, t, b, c, d) {
				var s = 1.70158;
				var p = 0;
				var a = c;
				if (t == 0) return b;
				if ((t /= d) == 1) return b + c;
				if (!p) p = d * .3;
				if (a < Math.abs(c)) {
					a = c;
					var s = p / 4;
				} else var s = p / (2 * Math.PI) * Math.asin(c / a);
				return a * Math.pow(2, -10 * t) * Math.sin((t * d - s) * (2 * Math.PI) / p) + c + b;
			},
			easeInOutElastic: function (x, t, b, c, d) {
				var s = 1.70158;
				var p = 0;
				var a = c;
				if (t == 0) return b;
				if ((t /= d / 2) == 2) return b + c;
				if (!p) p = d * (.3 * 1.5);
				if (a < Math.abs(c)) {
					a = c;
					var s = p / 4;
				} else var s = p / (2 * Math.PI) * Math.asin(c / a);
				if (t < 1) return -.5 * (a * Math.pow(2, 10 * (t -= 1)) * Math.sin((t * d - s) * (2 * Math.PI) / p)) + b;
				return a * Math.pow(2, -10 * (t -= 1)) * Math.sin((t * d - s) * (2 * Math.PI) / p) * .5 + c + b;
			},
			easeInBack: function (x, t, b, c, d, s) {
				if (s == undefined) s = 1.70158;
				return c * (t /= d) * t * ((s + 1) * t - s) + b;
			},
			easeOutBack: function (x, t, b, c, d, s) {
				if (s == undefined) s = 1.70158;
				return c * ((t = t / d - 1) * t * ((s + 1) * t + s) + 1) + b;
			},
			easeInOutBack: function (x, t, b, c, d, s) {
				if (s == undefined) s = 1.70158;
				if ((t /= d / 2) < 1) return c / 2 * (t * t * (((s *= (1.525)) + 1) * t - s)) + b;
				return c / 2 * ((t -= 2) * t * (((s *= (1.525)) + 1) * t + s) + 2) + b;
			},
			easeOutBounce: function (x, t, b, c, d) {
				if ((t /= d) < (1 / 2.75)) {
					return c * (7.5625 * t * t) + b;
				} else if (t < (2 / 2.75)) {
					return c * (7.5625 * (t -= (1.5 / 2.75)) * t + .75) + b;
				} else if (t < (2.5 / 2.75)) {
					return c * (7.5625 * (t -= (2.25 / 2.75)) * t + .9375) + b;
				} else {
					return c * (7.5625 * (t -= (2.625 / 2.75)) * t + .984375) + b;
				}
			}

		}

		return EasingFunctions;

	});

define('utils/Animatable',[
		"underscore",
		"lib/EasingFunctions"
	],
	function (
		_,
		EasingFunctions
	) {
		"use strict";

		return {
			/**
			 * Mixin Animatable functionality to an object.
			 *
			 * Animatable makes it possible for objects to be animated. It generates animatable properties
			 * on the object which can be animated by calling animate and request its value at any given time
			 * with getValue-method. If the object is an Observable, Animatable will emit animationStarted and animationFinished events.
			 * 
			 * The Mixin can be applied to both a prototype or a object instance
			 * 
			 * @param {Object} object - the object in which the Animatable functionality should be mixed in
			 */
			mixin: function (object) {
				/**
				 * Add animatable properties to the object, provide for each property a function that returns its correct value.
				 * It is possible to add 1 property (by using 2 parameters), or a set of properties (only use the properties parameter)
				 * @param {Map.<String,Function>|String} properties - Either a object mapping propertyNames on there value functions or a property name (combined with a value function as second argument)
				 * @param {Function} [function] -  the functions that returns the correct values for the property (in case of two arguments)
				 */
				object.addAnimatableProperties = function (properties, valueFunction) {
					//first make sure the correct properties for the animatable are set
					if (!this._initialized) {
						this._initialize(); //call the private initialize function
					}

					if (arguments.length == 2){
						this._animatableProperties[properties] = valueFunction;						
					} else {
						this._animatableProperties = _.extend(this._animatableProperties, properties);
					}
				};

				/**
				 * Initialize the mixin animatable to the object, installs the needed properties for the animatable
				 */
				object._initialize = function () {
					/**
					 * @member {Map.<String|Function>} maps the animatable properties to the function that returns their values
					 * @private
					 */
					this._animatableProperties = {};

					/**
					 * @member {Map.<String|Object>} maps the animatable property to its current active animation. If there
					 * 									is no animation the property has no entry in this map.
					 * @private
					 */
					this._animations = {};

					/**
					 * @member {Boolean} indicates if the animatable mixin has be initialized on the object
					 * @private
					 */
					this._initialized = true;
				};

				/**
				 * Get the correct value for the animatable properties, uses the animation (if one for the property) or the default function.
				 * @param   {String} propertyName
				 * @param   {Boolean} [isStartAnimation] used to indicate start animation, does not take the animation into account
				 * @returns {any} value of the property
				 */
				object.getValue = function (propertyName, isStartAnimation) {
					//check if there is an animation active for the property, if this is the case get the value from the animation
					if (this._animations[propertyName] && !isStartAnimation) {
						var animationObject = this._animations[propertyName];
						var currentTime = Date.now();
						var timeOffset = currentTime - animationObject.startTime;
						if (timeOffset > animationObject.duration) { //if the animation is done (time offset is bigger than the duration)
							delete this._animations[propertyName]; //delete entry in the animations map
							
							if (this._emit) {
								this._emit(["animation", propertyName, "finished"].join("."));
								if (!this.isAnimating()){
									this._emit("animationFinished");
								}
							}							
							
							return this._animatableProperties[propertyName]();
						} else {
							//uses an easing function for the value
							return EasingFunctions[animationObject.easingFunction](null, timeOffset, animationObject.startValue, animationObject.endValue - animationObject.startValue, animationObject.duration);
						}
					} else {
						return this._animatableProperties[propertyName]();
					}
				};

				/**
				 * Starts an animation on one of the animatable properties. Make sure the function of the animatable
				 * property returns the end value for the animation.
				 * @param {String} propertyName
				 * @param {Integer} startValue
				 * @param {Integer} duration
				 * @param {String} [easingFunction] - if set this animation will use the provided easing function, otherwise it falls back to the easign assigned to the animation property or the default easing function
				 */
				object.animate = function (propertyName, startValue, duration, easingFunction) {
					var endValue = this.getValue(propertyName, true); //endvalue is the currently set value in the animatable property
					var wasAnimating = this.isAnimating();

					if (startValue !== endValue) { //only start animations if the start value is not equal to the endvalue
						if (!easingFunction){
							easingFunction = "easeOutBounce";
						}
						
						this._animations[propertyName] = {
							startValue: startValue,
							endValue: endValue,
							duration: duration,
							easingFunction: easingFunction,
							startTime: Date.now() //set the current time as the start time for the animation
						};
						
						if (this._emit){
							this._emit(["animation", propertyName, "started"].join("."));							
							if (!wasAnimating){
								this._emit("animationStarted");
							}
						}
					}
				};

				/**
				 * Has the object any properties that are being animated
				 * @returns {Boolean}
				 */
				object.isAnimating = function () {
					if (this._initialized) {
						return !_.isEmpty(this._animations);
					} else {
						return false;
					}
				};
			}
		};

	});

define('utils/OrderedObject',["underscore"],
	function (
		_) {
		"use strict";

		/**
		 * Create a new ordered object.
		 * @exports OrderedObject
		 * @memberof Utils
		 * @class
		 * An ordered object is an object that supports the advantages of an object (key lookup),
		 * while having the items ordered as well (index lookup).
		 * @constructor
		 */
		var OrderedObject = function () {
			// Index -> Key
			this.array = [];
			// Key -> Item
			this.object = {};
		};

		/**
		 * Add the given item for the given key at the given index.
		 * @param {String} key - Key of the item
		 * @param {Object} item - Any item
		 * @param {Number} index - The item is added at the specified index
		 */
		OrderedObject.prototype.addItemAtIndex = function (key, item, index) {
			if (this.getIndexByKey(key) >= 0) {
				throw new Error("Duplicate key exception");
			}
			this.array.splice(index, 0, key);
			this.object[key] = item;
		};

		/**
		 * Add the given item for the given key. The item is added at the end.
		 * @param {String} key - Key of the item
		 * @param {Object} item - Any item
		 */
		OrderedObject.prototype.addItem = function (key, item) {
			if (this.getIndexByKey(key) >= 0) {
				throw new Error("Duplicate key exception");
			}
			this.array.push(key);
			this.object[key] = item;
		};

		/**
		 * If a certain key is already existent, update the item for that key, else add the item in the end of the list
		 * @param {String} key - Key of the item
		 * @param {Object} item - Any item
		 */
		OrderedObject.prototype.addOrReplaceItem = function (key, item) {
			if (this.getIndexByKey(key) >= 0) {
				// replace item
				this.object[key] = item;
			} else {
				this.addItem(key, item);
			}
		};

		/**
		 * Sets the order array for the orderedObject, array has as values the keys of the orderedobject
		 * @param {Array} newOrder
		 * @throws {Error} when newOrder doesn't contain the same elements as the current ordered object keys
		 */
		OrderedObject.prototype.setOrder = function (newOrder) {
			if (_.difference(this.array, newOrder).length === 0) {
				this.array = newOrder.slice(0);
			} else {
				throw new Error("Invalid array given for order. Should contain the same elements as in the ordered object.");
			}
		};


		/**
		 * Remove the item with the given key
		 * @param {String} key - Key of the item
		 * @returns {Any} the deleted object, undefined if nothing was removed
		 */
		OrderedObject.prototype.removeByKey = function (key) {
			var deleted;
			var i = this.getIndexByKey(key);
			if (this.object[key] !== undefined && i >= 0) {
				this.array.splice(i, 1);

				//this.array = _.without(this.array, key);
				deleted = this.object[key];
				delete this.object[key];
			}

			return deleted;
		};

		/**
		 * Remove the item at the given index
		 * @param {Number} index - Index of the item
		 * @returns {Any} the deleted object, undefined if nothing was removed
		 */
		OrderedObject.prototype.removeByIndex = function (index) {
			var deleted;

			if (this.array[index]) {
				var k = this.array[index];
				deleted = this.object[k];
				delete this.object[k];
				this.array.splice(index, 1);
			}
			return deleted;
		};
		/**
		 * Omit all items which are not mentioned in keys
		 * @param {Array.<String>} keys - only these items can stay
		 */
		OrderedObject.prototype.omitOthers = function (keys) {
			var toBeOmmitted = _.difference(this.array, keys);

			this.array = _.difference(this.array, toBeOmmitted);
			this.object = _.omit(this.object, toBeOmmitted);
		};

		/**
		 * Clear all the elements in the Ordered Object
		 */
		OrderedObject.prototype.clear = function () {
			this.array = [];
			this.object = {};
		};

		/**
		 * Get the index of the item with the given key
		 * @param key - Key of the item
		 * @returns {Number} Index of the item with the given key, -1 if not found
		 */
		OrderedObject.prototype.getIndexByKey = function (key) {
			return this.array.indexOf(key);
		};

		/**
		 * Get the key of the item by index
		 * @param {Number} index - Index of the item
		 * @returns {String} Key of the item at the given index, undefined if not defined
		 */
		OrderedObject.prototype.getKeyByIndex = function (index) {
			return this.array[index];
		};

		/**
		 * Get the key of the item with the given value
		 * @param {Object} value - value of the item
		 * @returns {String} Key of the item at the given index, undefined if not defined
		 */
		OrderedObject.prototype.getKeyByValue = function (value) {
			var key;

			_.find(this.object, function (v, k) {
				if (value === v) {
					key = k;
					return true;
				} else {
					return false;
				}
			}, this);

			return key;
		};

		/**
		 * Get the item at the specified index
		 * @param {Number} index - Index of the item
		 * @returns {Object} The item at the specified index
		 */
		OrderedObject.prototype.getItemByIndex = function (index) {
			return this.object[this.array[index]];
		};

		/**
		 * Get the item by key
		 * @param {String} key - Key of the item
		 * @returns {Object} The item with the specified key
		 */
		OrderedObject.prototype.getItemByKey = function (key) {
			return this.object[key];
		};

		/**
		 * Check if  a certain key is present in the ordered object
		 * @param {String} key - key of the item
		 * @returns {Boolean}
		 */
		OrderedObject.prototype.containsKey = function (key) {
			return this.object[key] !== undefined;
		};


		/**
		 * Get the keys, this list is ordered correctly
		 * @returns {Array.<String>} The keys as array ordered.
		 */
		OrderedObject.prototype.getKeys = function () {
			return this.array.slice(0);
		};

		/**
		 * Get the items as object
		 * @returns {Object} The items as object
		 */
		OrderedObject.prototype.getItemsAsObject = function () {
			return _.clone(this.object);
		};

		/**
		 * Get the number of items
		 * @returns {Number} Number of items
		 */
		OrderedObject.prototype.getLength = function () {
			return this.array.length;
		};

		/**
		 * Move an item from a certain position to another position
		 * @param {Number} fromIndex - original index of the item to move
		 * @param {Number} toIndex - new index for element
		 */
		OrderedObject.prototype.moveItemByIndex = function (fromIndex, toIndex) {
			// first splice out the element at from index, and then inject it in the correct location
			var elementToMove = this.array.splice(fromIndex, 1)[0];
			this.array.splice(toIndex, 0, elementToMove);
		};


		/**
		 * Move an item from a certain position to another position
		 * @param {String} key - Key of the item
		 * @param {Number} toIndex - new index for element
		 */
		OrderedObject.prototype.moveItemByKeyToIndex = function (key, toIndex) {
			this.moveItemByIndex(this.getIndexByKey(key), toIndex);
		};

		/**
		 * Move an item to the end of the list
		 * @param {String} key - Key of the item
		 */
		OrderedObject.prototype.moveItemByKeyToEnd = function (key) {
			this.moveItemByKeyToIndex(key, this.array.length - 1);
		};

		/**
		 * Gets the next Object, if there isn't a next Object it returns undefined
		 * @param {String} key - Key of the item
		 * @returns {Object|undefined}
		 */
		OrderedObject.prototype.getNext = function (key) {
			if (this.hasNext(key)) {
				var index = this.getIndexByKey(key);
				var nextKey = this.array[++index];
				return this.object[nextKey];
			} else {
				return undefined;
			}
		};

		/**
		 * Gets the previous object, if there isn't a previous object it returns undefined
		 * @param {String} key - Key of the item
		 * @returns {Object|undefined}
		 */
		OrderedObject.prototype.getPrevious = function (key) {
			if (this.hasPrevious(key)) {
				var index = this.getIndexByKey(key);
				var previousKey = this.array[--index];
				return this.object[previousKey];
			} else {
				return undefined;
			}
		};

		/**
		 * returns true if there is a next Object
		 * @param {String} key
		 * @returns {Boolean}
		 */
		OrderedObject.prototype.hasNext = function (key) {
			var index = this.getIndexByKey(key);
			return index !== -1 && ++index < this.array.length;
		};

		/**
		 * returns true if there is a previous object
		 * @param {String} key
		 * @returns {Boolean}
		 */
		OrderedObject.prototype.hasPrevious = function (key) {
			var index = this.getIndexByKey(key);
			return index !== -1 && --index >= 0;
		};

		/**
		 * Return an array of the objects as they are ordered
		 * @returns {Array}
		 */
		OrderedObject.prototype.getOrderedObjectList = function () {
			var orderedObjectList = [];
			_.each(this.array, _.bind(function (key) {
				orderedObjectList.push(this.object[key]);
			}, this));
			return orderedObjectList;
		};

		/**
		 * Get the first item of the ordered object
		 * @returns {Any}
		 */
		OrderedObject.prototype.getFirst = function () {
			return this.getItemByIndex(0);
		};

		/**
		 * Get the last item of the ordered object
		 * @returns {Any}
		 */
		OrderedObject.prototype.getLast = function () {
			return this.getItemByIndex(this.getLength() - 1);
		};

		return OrderedObject;
	});

/**
 * @namespace Utils
 */
define('utils/Cache',["utils/OrderedObject",
		"underscore"
	],
	function (OrderedObject, _) {
		"use strict";

		/**
		 * @memberof Utils
		 * @exports Cache
		 * @class
		 * A simple LRU cache implementation (With loading mechanism)
		 * @constructor
		 * @param {Number} [cacheSize] - size of the cache (default: 30)
		 * @param {Boolean} [executeCallbacksIfInCache] - if true, callbacks will also be executed when the item is available in the cache (default: true)
		 */
		var Cache = function (cacheSize, executeCallbacksIfInCache) {
			this.store = new OrderedObject();
			this.deletedItems = {};

			this.executeCallbackIfInCache = executeCallbacksIfInCache !== undefined ? executeCallbacksIfInCache : true;

			cacheSize = cacheSize || 30;

			this.getCacheSize = function () {
				return cacheSize;
			};
		};

		/**
		 * Get a property from the cache, or load it if needed
		 * @param {String} name - name of the item
		 * @param {Function} callback - callback to be called when item is available
		 * @returns {Any}
		 */
		Cache.prototype.get = function (name, callback) {
			var storeItem = this.store.getItemByKey(name);

			// if no cache hit is found, try to see if there is an alternative in the cache
			if (!storeItem) {
				var alternativeName = this.getAlternativeName(name);
				if (alternativeName !== null) {
					// an alternative might have been found
					storeItem = this.store.getItemByKey(alternativeName);
				}
			}

			if (!storeItem) {
				// item was never loaded
				storeItem = {
					name: name,
					item: undefined,
					callbacks: [],
					loaded: false,
					error: false
				};

				// insert in store
				this.store.addItem(name, storeItem);

				// push the callback 
				if (callback) {
					storeItem.callbacks.push(callback);
				}

				// start loading
				this._loadItem(name);
			} else {
				// the item is already in the cache in some way
				if (storeItem.loaded) {
					// the item is in the cache and is loaded, maybe we need to fire the callback
					if (callback && this.executeCallbackIfInCache) {
						// we need to execute the callback
						callback(storeItem.item);
					}
				} else {
					// the item is still being loaded, add callback if needed
					if (callback) {
						storeItem.callbacks.push(callback);
					}
				}

				// move the item to the end of the store (so the cache is LRU)
				this.store.moveItemByKeyToEnd(name);
			}

			return storeItem.item;
		};


		/**
		 * This method should be overridden so it loads the requested item
		 * When the item is loaded, _registerItem should be called
		 * @param {String} name - name of the item
		 */
		Cache.prototype._loadItem = function (name) {
			console.error("This method should be overriden!");
		};


		/**
		 * Register an item in the cache
		 * @param {String} name - name of the item
		 * @param {Any|undefined} item - the item, if undefined, the item couldn't be loaded (callbacks will be called with "undefined" as parameter, and item is removed from cache)
		 */
		Cache.prototype._registerItem = function (name, item) {
			var inCache = true;
			var storeItem = this.store.getItemByKey(name);
			if (storeItem === undefined) {
				inCache = false;
				// item was already removed from cache, check if it is still in deleted
				storeItem = this.deletedItems[name];

				if (storeItem === undefined) {
					throw new Error("Trying to register item that was never requested");
				}

				// remove the item from the deleted items list
				delete this.deletedItems[name];
			}


			// item is still in cache
			storeItem.loaded = true;
			storeItem.item = item;
			storeItem.error = (item === undefined);

			if (inCache) {
				// the item is in the cache
				if (storeItem.error) {
					// remove it 
					this.store.removeByKey(name);
				} else {
					// make space for the new item that was loaded
					while (this.store.getLength() > this.getCacheSize()) {
						var deleted = this.store.removeByIndex(0);
						// if an item is still loading and has callbacks, mark it as deleted
						if (!deleted.loaded && deleted.callbacks.length > 0) {
							this.deletedItems[deleted.name] = deleted;
						}
					}
				}
			}

			_.each(storeItem.callbacks, function (callback) {
				callback(item);
			});

			storeItem.callbacks = [];
		};

		/**
		 * Remove item from cache
		 * @param name
		 */
		Cache.prototype.remove = function (name) {
			var storeItem = this.store.removeByKey(name);
			if (!storeItem.loaded && storeItem.callbacks.length > 0) {
				this.deletedItems[storeItem.name] = storeItem;
			}
		};

		/**
		 * In this method an alternative name could be generated for an item
		 * @param name
		 * @returns {String}
		 */
		Cache.prototype.getAlternativeName = function (name) {
			return null;
		};

		return Cache;
	});

define('utils/ColorUtils',[],
	function () {
		"use strict";

		return {
			/**
			 * Converts an integer (0 to 16581375) into a RGB color string (e.g. #012345)
			 * @param   {Number} colorInt
			 * @returns {String}
			 */
			intToHexString: function (colorInt) {
				var r = Math.floor(colorInt / (255 * 255)).toString(16);
				var g = Math.floor(colorInt / 255).toString(16);
				var b = Math.floor(colorInt % 255).toString(16);
				return "#" + (r.length == 1 ? "0" + r : r) + (g.length == 1 ? "0" + g : g) + (b.length == 1 ? "0" + b : b);
			},
			/**
			 * Converts RGB values (numbers 0 to 255) into a RGB color string (e.g. #012345)
			 * @param   {Number} r
			 * @param   {Number} g
			 * @param   {Number} b
			 * @returns {String}
			 */
			rgbToHexString: function (r, g, b) {
				r = r.toString(16);
				g = g.toString(16);
				b = b.toString(16);
				return "#" + (r.length == 1 ? "0" + r : r) + (g.length == 1 ? "0" + g : g) + (b.length == 1 ? "0" + b : b);
			}

		};
	});

define('utils/ConvertUtils',["jquery",
		"underscore"],
function($, _){
	var ConvertUtils = {
			 
		 /**
		  * Get a language formatted date of the form "Month dayOfMonth, year
		  * NOTE: make sure the string MONTH[1-12] are loaded in the i18n instance
		  * @param {I18n} i18n - instance of i18n
		  * @param {Date} date - the date to language format
		  * @returns {String} Language formatted date string as "Jan 19, 2014" e.g.
		  */
		 getLangFormattedDate : function (i18n,date) {
			  var months = [];

              months[0]= i18n.t("MONTH1");
              months[1]= i18n.t("MONTH2");
              months[2]= i18n.t("MONTH3");
              months[3]= i18n.t("MONTH4");
              months[4]= i18n.t("MONTH5");
              months[5]= i18n.t("MONTH6");
              months[6]= i18n.t("MONTH7");
              months[7]= i18n.t("MONTH8");
              months[8]= i18n.t("MONTH9");
              months[9]= i18n.t("MONTH10");
              months[10]= i18n.t("MONTH11");
              months[11]= i18n.t("MONTH12");
			  
			  if (date==null)
			    date=new Date();
			  var curMonth=date.getMonth(); 
			  var curDate=date.getDate(); 
			  var curYear=date.getFullYear(); 
			  
			  return ""+months[curMonth]+" "+curDate+", "+curYear; 
		 },
        /**
		  * Get a language formatted date and time of the form "Month dayOfMonth, year at hour:minutes"
		  * NOTE: make sure the string MONTH[1-12] are loaded in the i18n instance
		  * @param {I18n} i18n - instance of i18n
		  * @param {Date} date - the date to language format
		  * @returns {String} Language formatted date string as "Jan 19, 2014 at 11:00" e.g.
		  */
		 getLangFormattedDateAndTime : function (i18n,date) {
			  var months = [];	
			  
              months[0]= i18n.t("MONTH1");
              months[1]= i18n.t("MONTH2");
              months[2]= i18n.t("MONTH3");
              months[3]= i18n.t("MONTH4");
              months[4]= i18n.t("MONTH5");
              months[5]= i18n.t("MONTH6");
              months[6]= i18n.t("MONTH7");
              months[7]= i18n.t("MONTH8");
              months[8]= i18n.t("MONTH9");
              months[9]= i18n.t("MONTH10");
              months[10]= i18n.t("MONTH11");
              months[11]= i18n.t("MONTH12");	
			  
			  if (date==null)
				    date=new Date();
			  var curMonth=date.getMonth(); 
			  var curDate=date.getDate(); 
			  var curYear=date.getFullYear(); 
			  var curHour=date.getHours();
			  if(curHour<10){
				  curHour = "0"+curHour;
			  }
			  var curMinutes=date.getMinutes();
			  if(curMinutes<10){
				  curMinutes = "0"+curMinutes;
			  }
			  return i18n.t("DateAndTimeString",months[curMonth],curDate,curYear,curHour,curMinutes); 
         },
         /**
		  * Get an English formatted date of the form "Month dayOfMonth, year
		  * NOTE: make sure the string MONTH[1-12] are loaded in the i18n instance
		  * @param {I18n} i18n - instance of i18n
		  * @param {String} translatedDate - Language formatted date string as "Déc 19, 2014" e.g.
		  * @returns {String} English formatted date string as "Dec 19, 2014" e.g.
		  */
		 getEnglishFormattedDate : function (i18n,translatedDate) {
			  var months = {};

              months[i18n.t("MONTH1")]= "Jan";
              months[i18n.t("MONTH2")]= "Feb";
              months[i18n.t("MONTH3")]= "Mar";
              months[i18n.t("MONTH4")]= "Apr";
              months[i18n.t("MONTH5")]= "May";
              months[i18n.t("MONTH6")]= "Jun";
              months[i18n.t("MONTH7")]= "Jul";
              months[i18n.t("MONTH8")]= "Aug";
              months[i18n.t("MONTH9")]= "Sep";
              months[i18n.t("MONTH10")]= "Oct";
              months[i18n.t("MONTH11")]= "Nov";
              months[i18n.t("MONTH12")]= "Dec";
              
              var splitStr = $.trim(translatedDate).split(' ');
              if(months[splitStr[0]]){
            	  splitStr[0] = months[splitStr[0]];
            	  return splitStr.join('');
              } else {
            	  return translatedDate;
              }
		 },
        /**
		  * Make a capitalized version of the first letter of the given string
		  * @param {String} string - String of which you want to capitalize the first letter
		  * @returns {String} String with a capitalized first letter
		  */
		 capitaliseFirstLetter : function (string) {
		     return string.charAt(0).toUpperCase() + string.slice(1);
		 },
        
        /**
		  * Count the number of lines and characters in a raw HTML string
		  * @param {String} rawHtmlString - The raw HTML string
		  * @returns {Object} o.lineCount contains the line count, o.charCount the character count
		  */
		 getCharacterLineCount : function(rawHtmlString) {
			var lineCount = 1;
			var charCount = 0;

			if (rawHtmlString) {				
				var text = this.convertHTMLStringToRaw(rawHtmlString);
				text = text.replace(/\r\n/g, '\n');
				
				// count lines
				var newLines = 0;
				var n = text.indexOf('\n', 0);
				while (n != -1) {
					lineCount++;
					newLines++;
					n = text.indexOf('\n', n + 1);
				}
				// count characters
				charCount = text.length - newLines;
			}

			return {
				lineCount : lineCount,
				charCount : charCount
			};
		},
		/**
		 * Convert an html string to RAW string (remove all html tags and convert html entitites
		 * @param {String} htmlString
		 * @param {String} [newLineCharacter=\n]  - the character used for separating/indicating newlines in the output
		 * @returns {String} the raw text string
		 */
		convertHTMLStringToRaw: function(htmlString, newLineCharacter){
			if (newLineCharacter === undefined){
				newLineCharacter = "\n";
			}
			// remove all white space between tags, except real spaces
			var rawString = htmlString.replace(/>[\f\n\r\t\v\u00a0\u1680\u180e\u2000\u2001\u2002\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200a\u2028\u2029\u202f\u205f\u3000]*([ ]*)[\f\n\r\t\v\u00a0\u1680\u180e\u2000\u2001\u2002\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200a\u2028\u2029\u202f\u205f\u3000]*</g, ">$1<");
			
			// get body only
			rawString = rawString.replace(/.*<body>((\r\n|\n|.)*)<\/body>.*/i, "$1");
			
			// remove the first blocklevel element
			rawString = rawString.replace(/^<(blockquote|body|center|dd|dir|div|dl|dt|form|h1|h2|h3|h4|h5|h6|head|hr|html|isindex|li|menu|noframes|ol|p|pre|td|th|title|ul)[^>]*>/, "");
			// all elements that cause a line break are replaced by a \n
			rawString = rawString.replace(/<(blockquote|body|br|center|dd|dir|div|dl|dt|form|h1|h2|h3|h4|h5|h6|head|hr|html|isindex|li|menu|noframes|ol|p|pre|td|th|title|ul)[^>]*>/g,newLineCharacter);
			// strip all html elements
			rawString = rawString.replace(/<[^>]*>/g, "");
			// convert spaces
			rawString = rawString.replace(/&nbsp;|&#160;/gi, ' ');
			// decode entitites
			rawString = this.decodeHtmlEntities(rawString);
			
			return rawString;
		}, 
		
		/**
		 * Convert a RAW string to xHTML (change \r\n or \n to <br/>)
		 * @param {String} rawString
		 * @returns {String} the xHTML string
		 */
		convertRawStringToHTML: function(rawString){
			var htmlString = this.encodeHtmlEntities(rawString);
			
			// replace newlines with breaks
			htmlString = htmlString.replace(/(\r)?\n/g,"<br />");
			
			return htmlString;
		},
        /**
		 * Decode HTML entities in the HTML string
		 * @param html - HTML string to decode
		 * @returns {String} html parameter with decoded entities
		 */
		decodeHtmlEntities : function(html){
			return $('<textarea>').html(html).text();
		},

		/**
		 * Encode HTML entities in the HTML string
		 * @param text - text string to encode
		 * @returns {String} text parameter with encoded entities
		 */
		encodeHtmlEntities : function(text){
			return $("<div>").text(text).html();
		}
		   
    };
    
    return ConvertUtils;
});

define('utils/Command',[], function () {
	"use strict";

	/**
	 * A Command is an action that can be undone and redone.
	 * Commands are used by the UndoRedoController
	 *
	 * @class Command
	 *
	 *@param {Function} doCallback - function to execute the command
	 * @param {Function} undoCallback - function to revert the actions of the doCallback
	 * @param {String} description - description of the command
	 */
	var Command = function (doCallback, undoCallback, description) {

		this._do = doCallback;
		this._undo = undoCallback;
		this._description = description;
	};

	/**
	 * Execute the command
	 */
	Command.prototype.executeDo = function () {
		this._do();
	};

	/**
	 * Revert the actions done by the command
	 */
	Command.prototype.executeUndo = function () {
		this._undo();
	};

	/**
	 * Get the description of the command
	 * returns {String}
	 */
	Command.prototype.getDescription = function () {
		return this._description;
	};

	return Command;
});

/**
 * Port of /wcr-web/date.js
 *
 * @author KADA
 * @since 11.2.2015
 */
define('utils/dateformatter',["jquery"], function ($) {
	
	"use strict";
	
	// ===================================================================
	// Author: Matt Kruse <matt@mattkruse.com>
	// WWW: http://www.mattkruse.com/
	//
	// NOTICE: You may use this code for any purpose, commercial or
	// private, without any further permission from the author. You may
	// remove this notice from your final code if you wish, however it is
	// appreciated by the author if at least my web site address is kept.
	//
	// You may *NOT* re-distribute this code in any way except through its
	// use. That means, you can include it in your product, or your web
	// site, or any other form where the code is actually being used. You
	// may not put the plain javascript up on your site for download or
	// include it in your javascript libraries for download. 
	// If you wish to share this code with others, please just point them
	// to the URL instead.
	// Please DO NOT link directly to my .js files from your site. Copy
	// the files to your server and use them there. Thank you.
	// ===================================================================

	// HISTORY
	// ------------------------------------------------------------------
	// May 17, 2003: Fixed bug in parseDate() for dates <1970
	// March 11, 2003: Added parseDate() function
	// March 11, 2003: Added "NNN" formatting option. Doesn't match up
	//	                 perfectly with SimpleDateFormat formats, but 
	//	                 backwards-compatability was required.

	// ------------------------------------------------------------------
	// These functions use the same 'format' strings as the 
	// java.text.SimpleDateFormat class, with minor exceptions.
	// The format string consists of the following abbreviations:
	// 
	// Field        | Full Form          | Short Form
	// -------------+--------------------+-----------------------
	// Year         | yyyy (4 digits)    | yy (2 digits), y (2 or 4 digits)
	// Month        | MMM (name or abbr.)| MM (2 digits), M (1 or 2 digits)
	//	            | NNN (abbr.)        |
	// Day of Month | dd (2 digits)      | d (1 or 2 digits)
	// Day of Week  | EE (name)          | E (abbr)
	// Hour (1-12)  | hh (2 digits)      | h (1 or 2 digits)
	// Hour (0-23)  | HH (2 digits)      | H (1 or 2 digits)
	// Hour (0-11)  | KK (2 digits)      | K (1 or 2 digits)
	// Hour (1-24)  | kk (2 digits)      | k (1 or 2 digits)
	// Minute       | mm (2 digits)      | m (1 or 2 digits)
	// Second       | ss (2 digits)      | s (1 or 2 digits)
	// AM/PM        | a                  |
	//
	// NOTE THE DIFFERENCE BETWEEN MM and mm! Month=MM, not mm!
	// Examples:
	//  "MMM d, y" matches: January 01, 2000
	//	                      Dec 1, 1900
	//	                      Nov 20, 00
	//  "M/d/yy"   matches: 01/20/00
	//	                      9/2/00
	//  "MMM dd, yyyy hh:mm:ssa" matches: "January 01, 2000 12:30:45AM"
	// ------------------------------------------------------------------
	
	var DateFormatter = {	

			/**
			 * WebCenter standard formats for dates/times.
			 */
			TimeFormatWithSeconds : "HH:mm:ss a",
			TimeFormat : "HH:mm a",
			DateFormat : "dd.MM.yyyy",
			DateAndTimeFormat : "dd.MM.yyyy HH:mm a",
			
			/**
			 * Some private variables/functions that are needed.
			 */
			_MONTH_NAMES : new Array('January','February','March','April','May','June',
					'July','August','September','October','November','December','Jan',
					'Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'),
			_DAY_NAMES : new Array('Sunday','Monday','Tuesday','Wednesday','Thursday',
					'Friday','Saturday','Sun','Mon','Tue','Wed','Thu','Fri','Sat'),
			_LZ : function(x){
				return(x<0||x>9?"":"0")+x;
			},
			_isInteger: function(val) {
				var digits="1234567890";
				for (var i=0; i < val.length; i++) {
					if (digits.indexOf(val.charAt(i))==-1) { 
						return false;
					};
				}
				return true;
			},
			_getInt : function(str,i,minlength,maxlength) {
				for (var x=maxlength; x>=minlength; x--) {
					var token=str.substring(i,i+x);
					if (token.length < minlength) { 
						return null; 
					}
					if (this._isInteger(token)) { 
						return token; 
					};
				}
				return null;
			},
			
			/**
			 * Returns true if date string matches format of format string and is a valid
			 * date. Else returns false.
			 * It is recommended that you trim whitespace around the value before passing 
			 * it to this function, as whitespace is NOT ignored!
			 */
			isDate: function (val,format) {
				var date=getDateFromFormat(val,format);
				if (date==0) { return false; }
				return true;
			},
			
			/**
			 * Returns a date in the output format specified.
			 * The format string uses the same abbreviations as in getDateFromFormat()
			 */
			formatDate : function(date, format) {
				
				if (format === undefined) {
					format = this.DateAndTimeFormat;
				}
	
				var result = "";
				var i_format = 0;
				var c = "";
				var token = "";
				var y = date.getYear() + "";
				var M = date.getMonth() + 1;
				var d = date.getDate();
				var E = date.getDay();
				var H = date.getHours();
				var m = date.getMinutes();
				var s = date.getSeconds();
	
				// Convert real date parts into formatted versions
				var value = new Object();
				if (y.length < 4) {
					y = "" + (y - 0 + 1900);
				}
				value["y"] = "" + y;
				value["yyyy"] = y;
				value["yy"] = y.substring(2, 4);
	
				value["M"] = M;
				value["MM"] = this._LZ(M);
				value["MMM"] = this._MONTH_NAMES[M - 1];
				value["NNN"] = this._MONTH_NAMES[M + 11];
	
				value["d"] = d;
				value["dd"] = this._LZ(d);
	
				value["E"] = this._DAY_NAMES[E + 7];
				value["EE"] = this._DAY_NAMES[E];
				value["H"] = H;
				value["HH"] = this._LZ(H);
				if (H == 0) {
					value["h"] = 12;
				} else if (H > 12) {
					value["h"] = H - 12;
				} else {
					value["h"] = H;
				}
				value["hh"] = this._LZ(value["h"]);
				if (H > 11) {
					value["K"] = H - 12;
				} else {
					value["K"] = H;
				}
				value["k"] = H + 1;
				value["KK"] = this._LZ(value["K"]);
				value["kk"] = this._LZ(value["k"]);
				if (H > 11) {
					value["a"] = "PM";
				} else {
					value["a"] = "AM";
				}
				value["m"] = m;
				value["mm"] = this._LZ(m);
				value["s"] = s;
				value["ss"] = this._LZ(s);
				while (i_format < format.length) {
					c = format.charAt(i_format);
					token = "";
					while ((format.charAt(i_format) == c)
							&& (i_format < format.length)) {
						token += format.charAt(i_format++);
					}
					if (value[token] != null) {
						result = result + value[token];
					} else {
						result = result + token;
					};
				}
				return result;
			},
			

				
			// ------------------------------------------------------------------
			// getDateFromFormat( date_string , format_string )
			//
			// This function takes a date string and a format string. It matches
			// If the date string matches the format string, it returns the 
			// getTime() of the date. If it does not match, it returns 0.
			// ------------------------------------------------------------------
			getDateFromFormat : function(val,format) {
				
				val=val+"";
				format=format+"";
				var i_val=0;
				var i_format=0;
				var c="";
				var token="";
				var x,y;
				var now=new Date();
				var year=now.getYear();
				var month=now.getMonth()+1;
				var date=1;
				var hh=now.getHours();
				var mm=now.getMinutes();
				var ss=now.getSeconds();
				var ampm="";
				
				while (i_format < format.length) {
					// Get next token from format string
					c=format.charAt(i_format);
					token="";
					while ((format.charAt(i_format)==c) && (i_format < format.length)) {
						token += format.charAt(i_format++);
					}
					// Extract contents of value based on format token
					if (token=="yyyy" || token=="yy" || token=="y") {
						if (token=="yyyy") { x=4;y=4; }
						if (token=="yy")   { x=2;y=2; }
						if (token=="y")    { x=2;y=4; }
						year=this._getInt(val,i_val,x,y);
						if (year==null) {
							return 0;
						}
						i_val += year.length;
						if (year.length==2) {
							if (year > 70) { 
								year=1900+(year-0);
							} else { 
								year=2000+(year-0);
							}
						}
					} else if (token=="MMM"||token=="NNN") {
						month=0;
						for (var i=0; i<this._MONTH_NAMES.length; i++) {
							var month_name=this._MONTH_NAMES[i];
							if (val.substring(i_val,i_val+month_name.length).toLowerCase()==month_name.toLowerCase()) {
								if (token=="MMM"||(token=="NNN"&&i>11)) {
									month=i+1;
									if (month>12) { month -= 12; }
									i_val += month_name.length;
									break;
								}
							}
						}
						if ((month < 1)||(month>12)){return 0;}
					} else if (token=="EE"||token=="E"){
						for (var i=0; i<this._DAY_NAMES.length; i++) {
							var day_name=this._DAY_NAMES[i];
							if (val.substring(i_val,i_val+day_name.length).toLowerCase()==day_name.toLowerCase()) {
								i_val += day_name.length;
								break;
								}
							}
					} else if (token=="MM"||token=="M") {
						month=this._getInt(val,i_val,token.length,2);
						if(month==null||(month<1)||(month>12)){return 0;}
						i_val+=month.length;
					} else if (token=="dd"||token=="d") {
						date=this._getInt(val,i_val,token.length,2);
						if(date==null||(date<1)||(date>31)){return 0;}
						i_val+=date.length;
					} else if (token=="hh"||token=="h") {
						hh=this._getInt(val,i_val,token.length,2);
						if(hh==null||(hh<1)||(hh>12)){return 0;}
						i_val+=hh.length;
					} else if (token=="HH"||token=="H") {
						hh=this._getInt(val,i_val,token.length,2);
						if(hh==null||(hh<0)||(hh>23)){return 0;}
						i_val+=hh.length;
					} else if (token=="KK"||token=="K") {
						hh=this._getInt(val,i_val,token.length,2);
						if(hh==null||(hh<0)||(hh>11)){return 0;}
						i_val+=hh.length;
					} else if (token=="kk"||token=="k") {
						hh=this._getInt(val,i_val,token.length,2);
						if(hh==null||(hh<1)||(hh>24)){return 0;}
						i_val+=hh.length;hh--;
					} else if (token=="mm"||token=="m") {
						mm=this._getInt(val,i_val,token.length,2);
						if(mm==null||(mm<0)||(mm>59)){return 0;}
						i_val+=mm.length;
					} else if (token=="ss"||token=="s") {
						ss=this._getInt(val,i_val,token.length,2);
						if(ss==null||(ss<0)||(ss>59)){return 0;}
						i_val+=ss.length;
					} else if (token=="a") {
						if (val.substring(i_val,i_val+2).toLowerCase()=="am") {
							ampm="AM";
						}
						else if (val.substring(i_val,i_val+2).toLowerCase()=="pm") {
							ampm="PM";
						}
						else {
							return 0;
						}
						i_val+=2;
					}
					else {
						if (val.substring(i_val,i_val+token.length)!=token) {
							return 0;
						}
						else {
							i_val+=token.length;
						}
					}
				}
				
				// If there are any trailing characters left in the value, it
				// doesn't match
				if (i_val != val.length) {
					return 0;
				}
				// Is date valid for month?
				if (month==2) {
					// Check for leap year
					if ( ( (year%4==0)&&(year%100 != 0) ) || (year%400==0) ) { // leap
																				// year
						if (date > 29){
							return 0;
						}
					} else { if (date > 28) {
						return 0; 
						}
					}
				}
				if ((month==4)||(month==6)||(month==9)||(month==11)) {
					if (date > 30) {
						return 0;
					}
				}
				// Correct hours value
				if (hh<12 && ampm=="PM") {
					hh=hh-0+12;
				}
				else if (hh>11 && ampm=="AM") {
					hh-=12;
				}
				
				var newdate=new Date(year,month-1,date,hh,mm,ss);
				return newdate.getTime();
			}

	};
	
	return DateFormatter;

});






















define('utils/DomUtils',[], function(){
    "use strict";
    var DomUtils = {
        /**
         * Dynamically load CSS file
         * 
         * @param {String}   filename - path to css from requireJS base
         * @param {Function} [onload]   - callback function when css is loaded
         */
        loadCss: function(filename, onload){
            this.loadCssFromUrl(requirejs.toUrl(filename), onload); 
        },
    
        /**
         * Dynamically load CSS file
         * 
         * @param {String}   url - url to css
         * @param {Function} [onload]   - callback function when css is loaded
         */
	    loadCssFromUrl: function(url, onload){
	        var sheet = null;
	        var timerfunc = null;
	        var fileref = document.createElement("link");
	        fileref.setAttribute("rel", "stylesheet");
	        fileref.setAttribute("type", "text/css");
	
	
	        timerfunc = function (){
	            for (var i=0;i<document.styleSheets.length;i++){
	                sheet = document.styleSheets[i].href;
	                if(sheet !== null && sheet.substr(sheet.length-url.length) == url){
	                    if (onload) {
	                        onload();
	                        onload = null; // make sure the callback is only called once
	                    }
	                    return;
	                }
	            }
	            setTimeout(timerfunc,50);  
	        };
	
	        if (document.all){ //Uses onreadystatechange for Internet Explorer
	            fileref.attachEvent('onreadystatechange',function() {
	                if(fileref.readyState == 'complete' || fileref.readyState == 'loaded') {
	                    if (onload) {
	                        onload();
	                        onload = null; // make sure the callback is only called once
	                    }
	                }
	
	            });
	        } else {    //Checks if the stylesheet has been loaded every 50 ms for others
	            setTimeout(timerfunc,50);
	        }
	
	        fileref.setAttribute("href", url);	    
	        document.getElementsByTagName("head")[0].appendChild(fileref);       
	    }
	    
	};

    
    return DomUtils;
});

define('utils/ShapeUtils',["underscore"],
	function (_) {
		"use strict";

		/**
		 * Create a new point
		 * @class Point
		 * @namespace ShapeUtils
		 * @param {Number} [x = 0] x coordinate of the point
		 * @param {Number} [y = 0] y coordinate of the point
		 */
		var Point = function (x, y) {
			this.x = (x ? x : 0);
			this.y = (y ? y : 0);
		};

		/**
		 * String representation of the point.
		 * @returns {String} "xxy" where the second x means \times
		 */
		Point.prototype.toString = function () {
			return this.x + "x" + this.y;
		};

		/**
		 * Make a deep copy of this point
		 * @returns {ShapeUtils.Point} point with the same x and y
		 */
		Point.prototype.copy = function () {
			return new Point(this.x, this.y);
		};

		/**
		 * Check if the point equals otherpoint
		 * @param {ShapeUtils.Point} otherpoint - point to check equality of this point with
		 * @returns {Boolean} True if x and y are the same, false otherwise
		 */
		Point.prototype.equals = function (otherpoint) {
			return this.x == otherpoint.x && this.y == otherpoint.y;
		};

		/**
		 * Create a size object from this point to the given point to
		 * @param {ShapeUtils.Point} to - To point
		 * @returns {ShapeUtils.Size} Size object from this point to the given to point calculated as to.x - this.x, to.y-this.y.
		 */
		Point.prototype.sizeTo = function (to) {
			return new ShapeUtils.Size(to.x - this.x, to.y - this.y);
		};

		/**
		 * Add an offset to this point
		 * @param {ShapeUtils.Size} offsetSize - width and height to add as offset
		 */
		Point.prototype.offset = function (offsetSize) {
			this.x += offsetSize.width;
			this.y += offsetSize.height;
		};

		/**
		 * Add an offset to this point
		 * @param {Number} x - Number to add to x field of point
		 * @param {Number} y - Number to add to y field of point
		 */
		Point.prototype.offsetXY = function (x, y) {
			this.x += x;
			this.y += y;
		};

		/**
		 * Check if this point is near another point within a certain margin
		 * @param {ShapeUtils.Point} otherpoint - other point to check closeness with
		 * @param {Number} margin - max distance (included) between otherpoint and this point
		 * @returns {Boolean} True if the other point is within margin distance in absolute value of this point for x AND y, false otherwise.
		 */
		Point.prototype.near = function (otherpoint, margin) {
			return Math.abs(this.x - otherpoint.x) <= margin && Math.abs(this.y - otherpoint.y) <= margin;
		};

		/**
		 * Get the euclidian distance between to points
		 * @param {ShapeUtils.Point} otherpoint
		 */
		Point.prototype.getDistance = function (otherpoint) {
			return Math.sqrt(Math.pow(this.x - otherpoint.x, 2) + Math.pow(this.y - otherpoint.y, 2));

		};

		/**
		 * Round the x and y values to the nearest integer
		 * @returns {ShapeUtils.Point} this point so you can use chaining
		 */
		Point.prototype.round = function () {
			this.x = Math.round(this.x);
			this.y = Math.round(this.y);
			return this;
		};

		/**
		 * If this Point isn't in/on Rect already the Point will be altered to the closest point on the border of rect. otherwise nothing will happen
		 * @param {ShapeUtils.Rect} rect
		 */
		Point.prototype.containPointInRect = function (rect) {
			if (this.x > rect.normRight()) {
				this.x = rect.normRight();
			} else if (this.x < rect.normLeft()) {
				this.x = rect.normLeft();
			}
			if (this.y > rect.normBottom()) {
				this.y = rect.normBottom();
			} else if (this.y < rect.normTop()) {
				this.y = rect.normTop();
			}
		};



		/**
		 * Create new Size object
		 *
		 * @class Size
		 * @namespace ShapeUtils
		 * @param {Number} width - Width of the size object
		 * @param {Number} height - Height of the size object
		 */
		var Size = function (width, height) {
			this.width = (width ? width : 0);
			this.height = (height ? height : 0);
		};

		/**
		 * Create a string representation of the size object
		 * @returns {String} "widthxheight" where the x means \times
		 */
		Size.prototype.toString = function () {
			return this.width + " x " + this.height;
		};

		/**
		 * Deep copy of this size object
		 * @returns {ShapeUtils.Size} size object with the same width and height
		 */
		Size.prototype.copy = function () {
			return new Size(this.width, this.height);
		};

		/**
		 * Check if this size object equals another one
		 * @param {ShapeUtils.Size} other - size object to check equality with
		 * @returns {Boolean} True if width and height are equal, false otherwise
		 */
		Size.prototype.equals = function (other) {
			return (this.width == other.width && this.height == other.height);
		};

		/**
		 * Add a width/height to this object's width/height
		 * @param {ShapeUtils.Size} size - width and height to add to this object's width and height
		 */
		Size.prototype.add = function (size) {
			this.width += size.width;
			this.height += size.height;
		};


		/**
		 * Create new Rect(angle) object
		 *
		 * @class Rect
		 * @namespace ShapeUtils
		 * @param {ShapeUtils.Point} origin - point object containing x and y
		 * @param {ShapeUtils.Size} size - size object containing width and height
		 */
		var Rect = function (origin, size) {
			this.origin = (origin ? origin : new Point());
			this.size = (size ? size : new Size());
		};

		/**
		 * Normalizes rects with negative width or height to positive width or height (changes origin and inverts signs of width/height)
		 */
		Rect.prototype.normalize = function () {
			if (this.size.width < 0) {
				this.origin.x += this.size.width;
				this.size.width = Math.abs(this.size.width);
			}
			if (this.size.height < 0) {
				this.origin.y += this.size.height;
				this.size.height = Math.abs(this.size.height);
			}
		};

		/**
		 * Check if this rectangle contains the given point
		 * NOTE PJPR: this probably will go wrong on non normalized rectangles
		 * @param {ShapeUtils.Point} pointToCheck - point to check
		 * @returns {Boolean} True if the pointToCheck lies in this rectangle, false otherwise
		 */
		Rect.prototype.containsPoint = function (pointToCheck) {
			var contains = false;
			if (this.origin.x <= pointToCheck.x && pointToCheck.x <= Number(this.origin.x) + Number(this.size.width)) {
				if (this.origin.y <= pointToCheck.y && pointToCheck.y <= Number(this.origin.y) + Number(this.size.height)) {
					contains = true;
				}
			}
			return contains;
		};


		/**
		 * String representation of this rectangle
		 * @returns {String} string representation of this rectangle of the format "Origin: xxy size: widthxheight" with x meaning \times
		 */
		Rect.prototype.toString = function () {
			return "Origin: " + this.origin.toString() + " size: " + this.size.toString();
		};

		/**
		 * Check if this rectangle equals another rectangle within a certain precision
		 * @param {ShapeUtils.Rect} r1 - Rectangle to check equality of this rectangle with
		 * @param {Number} [precision = 0.000001] - Precision of the equality check
		 * @returns {Boolean} True if the difference of x, y, width and height of this rectangle with the respective values of r1 are smaller than precision, false otherwise
		 */
		Rect.prototype.rectEquals = function (r1, precision) {
			if (r1 === undefined || r1 === null) {
				return false;
			}
			if (precision === undefined) {
				precision = 0.000001;
			}

			return (Math.abs(r1.origin.x - this.origin.x) <= precision &&
				Math.abs(r1.origin.y - this.origin.y) <= precision &&
				Math.abs(r1.size.width - this.size.width) <= precision &&
				Math.abs(r1.size.height - this.size.height) <= precision);
		};



		/**
		 * Check if two rectangle overlap or intersect
		 *
		 * @param {ShapeUtils.Rect} r - the rectangle to check intersection with
		 * @param {Float} [p] percentage that this rect overlaps r
		 * @returns {Boolean} true if there is an overlap or some points of the rectangles intersect
		 */
		Rect.prototype.rectIntersect = function (r, p) {
			var r1 = r;
			var r2 = this;
			var intersect = !(r1.normLeft() > r2.normRight() ||
				r1.normRight() < r2.normLeft() ||
				r1.normTop() > r2.normBottom() ||
				r1.normBottom() < r2.normTop());
			if (p && intersect) {
				var overlap = this.calculateOverlap(r);
				overlap.normalize();
				var areaOverlap = overlap.size.width * overlap.size.height;
				var areaP = r.size.width * r.size.height;
				return areaOverlap / areaP > p;

			} else {
				return intersect;
			}
		};


		/**
		 * Get the right coordinate
		 * DANGER this doesn't work correctly for non-normalized rectangles!
		 * @returns {Number} The right coordinate calculated as x + width
		 */
		Rect.prototype.right = function () {
			return this.origin.x + this.size.width;
		};

		/**
		 * Get the left coordinate
		 * DANGER this doesn't work correctly for non-normalized rectangles!
		 * @returns {Number} The left coordinate calculated as x
		 */
		Rect.prototype.left = function () {
			return this.origin.x;
		};

		/**
		 * Get the top coordinate
		 * DANGER this doesn't work correctly for non-normalized rectangles!
		 * @returns {Number} The top coordinate calculated as y
		 */
		Rect.prototype.top = function () {
			return this.origin.y;
		};

		/**
		 * Get the bottom coordinate
		 * DANGER this doesn't work correctly for non-normalized rectangles!
		 * @returns {Number} The bottom coordinate calculated as y + height
		 */
		Rect.prototype.bottom = function () {
			return this.origin.y + this.size.height;
		};

		/**
		 * Get the width of the rectangle
		 * DANGER this doesn't work correctly for non-normalized rectangles!
		 * @returns {Number} The width of the rectangle
		 */
		Rect.prototype.width = function () {
			return this.size.width;
		};

		/**
		 * Get the height of the rectangle
		 * DANGER this doesn't work correctly for non-normalized rectangles!
		 * @returns {Number} The height of the rectangle
		 */
		Rect.prototype.height = function () {
			return this.size.height;
		};

		/**
		 * Get the top-left coordinate
		 * DANGER this doesn't work correctly for non-normalized rectangles!
		 * @returns {ShapeUtils.Point} The top-left coordinate calculated as left(), top()
		 */
		Rect.prototype.topleft = function () {
			return new Point(this.left(), this.top());
		};

		/**
		 * Get the top-right coordinate
		 * DANGER this doesn't work correctly for non-normalized rectangles!
		 * @returns {ShapeUtils.Point} The top-right coordinate calculated as right(), top()
		 */
		Rect.prototype.topright = function () {
			return new Point(this.right(), this.top());
		};

		/**
		 * Get the bottom-left coordinate
		 * DANGER this doesn't work correctly for non-normalized rectangles!
		 * @returns {ShapeUtils.Point} The bottom-left coordinate calculated as left(), bottom()
		 */
		Rect.prototype.bottomleft = function () {
			return new Point(this.left(), this.bottom());
		};

		/**
		 * Get the bottom-right coordinate
		 * DANGER this doesn't work correctly for non-normalized rectangles!
		 * @returns {ShapeUtils.Point} The bottom-right coordinate calculated as right(), bottom()
		 */
		Rect.prototype.bottomright = function () {
			return new Point(this.right(), this.bottom());
		};

		/**
		 * Get the top-center coordinate
		 * DANGER this doesn't work correctly for non-normalized rectangles!
		 * @returns {ShapeUtils.Point} The top-center coordinate calculated as center().x, top()
		 */
		Rect.prototype.topcenter = function () {
			return new Point(this.center().x, this.top());
		};

		/**
		 * Get the bottom-center coordinate
		 * DANGER this doesn't work correctly for non-normalized rectangles!
		 * @returns {ShapeUtils.Point} The bottom-center coordinate calculated as center().x, bottom()
		 */
		Rect.prototype.bottomcenter = function () {
			return new Point(this.center().x, this.bottom());
		};

		/**
		 * Get the center-left coordinate
		 * DANGER this doesn't work correctly for non-normalized rectangles!
		 * @returns {ShapeUtils.Point} The center-left coordinate calculated as left(), center().y
		 */
		Rect.prototype.centerleft = function () {
			return new Point(this.left(), this.center().y);
		};

		/**
		 * Get the center-right coordinate
		 * DANGER this doesn't work correctly for non-normalized rectangles!
		 * @returns {ShapeUtils.Point} The center-right coordinate calculated as right(), center().y
		 */
		Rect.prototype.centerright = function () {
			return new Point(this.right(), this.center().y);
		};

		// ---- following functions return correct top/right/bottom/left values for non-normalized rectangles

		/**
		 * Right side of the rectangle (x coordinate)
		 * works correctly for non-normalized rectangles
		 * @returns {Number}
		 */
		Rect.prototype.normRight = function () {
			if (this.size.width >= 0) {
				return this.origin.x + this.size.width;
			} else {
				return this.origin.x;
			}
		};

		/**
		 * Left side of the rectangle (x coordinate)
		 * works correctly for non-normalized rectangles
		 * @returns {Number}
		 */
		Rect.prototype.normLeft = function () {
			if (this.size.width >= 0) {
				return this.origin.x;
			} else {
				return this.origin.x + this.size.width;
			}
		};

		/**
		 * Top side of the rectangle (y coordinate)
		 * works correctly for non-normalized rectangles
		 * @returns {Number}
		 */
		Rect.prototype.normTop = function () {
			if (this.size.height >= 0) {
				return this.origin.y;
			} else {
				return this.origin.y + this.size.height;
			}
		};

		/**
		 * Bottom side of the rectangle (y coordinate)
		 * works correctly for non-normalized rectangles
		 * @returns {Number}
		 */
		Rect.prototype.normBottom = function () {
			if (this.size.height >= 0) {
				return this.origin.y + this.size.height;
			} else {
				return this.origin.y;
			}
		};

		/**
		 * Get the normalized (absolute) height of the rectangle
		 * @returns {Number}
		 */
		Rect.prototype.normHeight = function () {
			return Math.abs(this.size.height);
		};

		/**
		 * Get the normalized (absolute) width of the rectangle
		 * @returns {Number}
		 */
		Rect.prototype.normWidth = function () {
			return Math.abs(this.size.height);
		};


		/**
		 * Get the center of the rectangle
		 * @returns {ShapeUtils.Point} - point containing the center of the rectangle calculated as x + width/2 and y + height/2
		 */
		Rect.prototype.center = function () {
			return new Point(this.origin.x + this.size.width / 2, this.origin.y + this.size.height / 2);
		};

		/**
		 * Set the center of the rectangle. The x and y coordinates are adapted.
		 * @param {ShapeUtils.Point} center - New center of the rectangle
		 */
		Rect.prototype.setCenter = function (center) {
			this.origin.x = center.x - this.size.width / 2;
			this.origin.y = center.y - this.size.height / 2;
		};

		/**
		 * Check if this rect contains otherRect within a certain margin
		 * @param {ShapeUtils.Rect} otherRect - other rectangle that is contained by this rectangle
		 * @param {Number} margin - extra margin that is required for the otherRect within this rectangle
		 * @returns {Boolean} True if this rectangle contains otherRect, false otherwise.
		 */
		Rect.prototype.containsRect = function (otherRect, margin) {
			if (!margin) {
				margin = 0;
			}
			var x1 = this.origin.x - margin;
			var y1 = this.origin.y - margin;
			var w1 = this.size.width + margin;
			var h1 = this.size.height + margin;
			var x2 = otherRect.origin.x;
			var y2 = otherRect.origin.y;
			var w2 = otherRect.size.width;
			var h2 = otherRect.size.height;


			if ((w1 | h1 | w2 | h2) < 0) {
				// At least one of the dimensions is negative...
				return false;
			}
			if (x2 < x1 || y2 < y1) {
				return false;
			}
			w1 += x1;
			w2 += x2;
			if (w2 <= x2) {
				// X+W overflowed or W was zero, return false if...
				// either original w or W was zero or
				// x+w did not overflow or
				// the overflowed x+w is smaller than the overflowed X+W
				if (w1 >= x1 || w2 > w1) {
					return false;
				}
			} else {
				// X+W did not overflow and W was not zero, return false if...
				// original w was zero or
				// x+w did not overflow and x+w is smaller than X+W
				if (w1 >= x1 && w2 > w1) {
					return false;
				}
			}
			h1 += y1;
			h2 += y2;
			if (h2 <= y2) {
				if (h1 >= y1 || h2 > h1) {
					return false;
				}
			} else {
				if (h1 >= y1 && h2 > h1) {
					return false;
				}
			}
			return true;
		};

		/**
		 * Make a deep copy of this rectangle
		 * @returns {ShapeUtils.Rect} rectangle with the same origin and size
		 */
		Rect.prototype.copy = function () {
			return new Rect(this.origin.copy(), this.size.copy());
		};

		/**
		 * Update rectangle
		 * @param {ShapeUtils.Point} p1 - origin (or new endpoint if only 1 argument)
		 * @param {ShapeUtils.Point} p2 - endpoint
		 */
		Rect.prototype.update = function (p1, p2) {
			if (arguments.length == 1) {
				this.size.width = p1.x - this.origin.x;
				this.size.height = p1.y - this.origin.y;
			} else {
				this.origin.x = p1.x;
				this.origin.y = p1.y;

				this.size.width = p2.x - p1.x;
				this.size.height = p2.y - p2.y;
			}
		};

		/**
		 * Calculate the rectangle that covers the overlap of this rectangle and rectangle b
		 * @param {ShapeUtils.Rect} b - rectangle to calculate overlap with
		 * @returns {ShapeUtils.Rect} the overlap rectangle
		 */
		Rect.prototype.calculateOverlap = function (b) {
			var origin = new Point(Math.max(this.normLeft(), b.normLeft()), Math.max(this.normTop(), b.normTop()));
			var endpoint = new Point(Math.min(this.normRight(), b.normRight()), Math.min(this.normBottom(), b.normBottom()));

			var result = new Rect(origin);
			result.update(endpoint);
			return result;
		};

		/**
		 * Calculates the combination of two rectangles
		 * @param {ShapeUtils.Rect} b - rectangle to combine this rectangle with
		 * @returns {ShapeUtils.Rect} An area that covers this rectangle and rectangle b
		 */
		Rect.prototype.calculateArea = function (b) {
			var origin = new Point(Math.min(this.normLeft(), b.normLeft()), Math.min(this.normTop(), b.normTop()));
			var endpoint = new Point(Math.max(this.normRight(), b.normRight()), Math.max(this.normBottom(), b.normBottom()));

			var result = new Rect(origin);
			result.update(endpoint);
			return result;
		};

		/**
		 * Returns a normalized rectangle definition
		 * @returns {NormalizedRectArray} [x1, y1, x2, y2]
		 */
		Rect.prototype.getNormalizedRectArray = function () {
			return [this.normLeft(), this.normTop(), this.normRight(), this.normBottom()];
		};


		/**
		 * Create a new Vector. This represents a vector in polar coordinates.
		 * Note: A vector 0,0 has length 0 and by default angle 0, if angleWhenZero is set this is the angle for the 0,0 vector
		 * @class Vector
		 * @namespace ShapeUtils
		 * @param {Number} [x = 0] - x coordinate of the Vector
		 * @param {Number} [y = 0] - y coordinate of the Vector
		 * @param {Number} [angle = 0] - the angle (in radians) when the length is 0
		 */
		var Vector = function (x, y, angle) {
			this.x = (x ? x : 0);
			this.y = (y ? y : 0);

			if (this.x == 0 && this.y == 0) {
				this.angleWhenZero = angle ? angle : 0;
			} else {
				this.angleWhenZero = this.getAngleInRadians();
			}
		};

		/**
		 * String representation of the Vector.
		 * @returns {String} "x,y"
		 */
		Vector.prototype.toString = function () {
			return this.x + "," + this.y;
		};

		/**
		 * Make a deep copy of this Vector
		 * @returns {Vector} Vector with the same x and y
		 */
		Vector.prototype.copy = function () {
			return new Vector(this.x, this.y, this.angleWhenZero);
		};

		/**
		 * Check if the Vector equals otherVector
		 * @param {ShapeUtils.Vector} otherVector - Vector to check equality of this Vector with
		 * @returns {Boolean} True if x and y are the same, false otherwise
		 */
		Vector.prototype.equals = function (otherVector) {
			return this.x == otherVector.x && this.y == otherVector.y && this.getAngleInRadians() == otherVector.getAngleInRadians();
		};

		/**
		 * Return the length (norm) of the vector
		 * @returns {Number} Length of the vector
		 */
		Vector.prototype.getLength = function () {
			return Math.sqrt(Math.pow(this.x, 2) + Math.pow(this.y, 2));
		};


		/**
		 * Set the length (norm) of the vector
		 * It changes the x and y accordingly.
		 */
		Vector.prototype.setLength = function (newLength) {
			var length = this.getLength();
			var angle = this.getAngleInRadians();
			if (length === 0) {
				this.x = Math.cos(angle) * newLength;
				this.y = Math.sin(angle) * newLength;
			} else {
				var factor = newLength / length;
				this.x = this.x * factor;
				this.y = this.y * factor;
			}

			this.angleWhenZero = angle;
		};

		/**
		 * Get the angle of the vector in degrees in [-180, 180]
		 * @returns {Number} Angle of the vector in degrees in [-180, 180]
		 */
		Vector.prototype.getAngleInDegrees = function () {
			return this.getAngleInRadians() * 180 / Math.PI;
		};

		/**
		 * Get the angle of the vector in radians
		 * @returns {Number} Angle of the vector in radians in [-PI, PI]
		 */
		Vector.prototype.getAngleInRadians = function () {
			if (this.x === 0 && this.y === 0) {
				return this.angleWhenZero;
			}
			return Math.atan2(this.y, this.x);
		};

		/**
		 * Set the angle of the vector in degrees.
		 * Keep the length the same.
		 * @param angleInDegrees [-180, 180]
		 */
		Vector.prototype.setAngleInDegrees = function (angleInDegrees) {
			this.setAngleInRadians(angleInDegrees * Math.PI / 180);
		};

		/**
		 * Set the angle of the vector in radians
		 * Keep the length the same.
		 * @param angleInRadians [-PI, PI]
		 */
		Vector.prototype.setAngleInRadians = function (angleInRadians) {
			var length = this.getLength();

			this.angleWhenZero = angleInRadians;
			this.x = length * Math.cos(angleInRadians);
			this.y = length * Math.sin(angleInRadians);
		};


		/**
		 * ShapeUtils contains utils for geometrical shapes (Point, Size, Rect)
		 *
		 * @exports ShapeUtils
		 * @borrows Point as Point
		 * @borrows Size as Size
		 * @borrows Rect as Rect
		 */
		var ShapeUtils = {

			Point: Point,
			Size: Size,
			Rect: Rect,
			Vector: Vector,

			/**
			 * Create a new Rect from x,y, width and height
			 * @param {Number} x
			 * @param {Number} y
			 * @param {Number} width
			 * @param {Number} height
			 * @returns {ShapeUtils.Rect}
			 */
			rectMake: function (x, y, width, height) {
				return new Rect(new Point(x, y), new Size(width, height));
			},
			/**
			 * Create a ShapeUtils.Rect from a normalized rect array ([x1, y1, x2,y2])
			 * @param {NormalizedRectArray} normalizedRect
			 * @returns {ShapeUtils.Rect}
			 */
			rectFromNormalizedRectArray: function (normalizedRect) {
				return new Rect(new Point(normalizedRect[0], normalizedRect[1]), new Size(normalizedRect[2] - normalizedRect[0], normalizedRect[3] - normalizedRect[1]));
			},
			/**
			 * Calculate the bounding box of two normalized rects
			 * @param {NormalizedRectArray} rectA
			 * @param {NormalizedRectArray} rectB
			 * @returns {NormalizedRectArray} the bounding box for the two rects
			 */
			calculateBoundingBoxRectArrays: function (rectA, rectB) {
				return [
					Math.min(rectA[0], rectB[0]),
					Math.min(rectA[1], rectB[1]),
					Math.max(rectA[2], rectB[2]),
					Math.max(rectA[3], rectB[3])
				];
			},
			/**
			 * Check if two normalized rectangles are equal
			 * @param {NormalizedRectArray} rectA
			 * @param {NormalizedRectArray} rectB
			 * @returns {Boolean} true if rectangles equal
			 */
			normalizedRectEquals: function (rectA, rectB) {
				return rectA[0] === rectB[0] &&
						rectA[1] === rectB[1] &&
						rectA[2] === rectB[2] &&
						rectA[3] === rectB[3];
			},
			/**
			 * Create a new rectangle from the overlap of two rectangles
			 * @param {ShapeUtils.Rect} r1
			 * @param {ShapeUtils.Rect} r2
			 * @returns {ShapeUtils.Rect}
			 */
			rectIntersection: function (r1, r2) {
				var x1, x2, y1, y2;

				x1 = Math.max(r1.left(), r2.left());
				y1 = Math.max(r1.top(), r2.top());
				x2 = Math.min(r1.right(), r2.right());
				y2 = Math.min(r1.bottom(), r2.bottom());

				return this.rectMake(x1, y1, x2 - x1, y2 - y1);
			},

			/**
			 * Check if a rectangle is empty/null
			 * @param {ShapeUtils.Rect} r
			 * @returns {Boolean} true if the rectangle is positioned at (0,0) and has a size of (0,0)
			 */
			rectIsNull: function (r) {
				return r.origin.x == 0 && r.origin.y == 0 && r.size.width == 0 && r.size.height == 0;
			}

		};

		return ShapeUtils;
	});

define('utils/DashUtils',["underscore",
		"utils/ShapeUtils"
	],
	function (_,
		ShapeUtils
	) {
		"use strict";

		function drawDashedLine(ctx, x, y, x2, y2, dashArray, zoom) {
			if (!dashArray) {
				dashArray = [10, 5];
			}

			if (zoom !== undefined && zoom !== null) {
				dashArray = _.map(dashArray, function (value) {
					return Math.abs(value / zoom);
				});
			}
			var supportsLinesDash = true;
			if (ctx.setLineDash !== undefined) { //check if setLineDash exists and dashed lines are supported
				ctx.setLineDash(dashArray);
			} else if (ctx.mozDash !== undefined) { //for older versions of firefox
				ctx.mozDash = dashArray;
			} else {
				supportsLinesDash = false; //dashed lines are not supported
			}

			ctx.save();
			ctx.beginPath();
			var dx = (x2 - x),
				dy = (y2 - y);
			var len = Math.sqrt(dx * dx + dy * dy);
			var rot = Math.atan2(dy, dx);

			ctx.translate(x, y);
			ctx.moveTo(0, 0);
			ctx.rotate(rot);
			if (supportsLinesDash) {
				ctx.lineTo(len, 0);
			} else {
				var dc = dashArray.length;
				var di = 0,
					draw = true;
				x = 0;
				while (len > x) {
					x += dashArray[di++ % dc];
					if (x > len) {
						x = len;
					}
					if (draw) {
						ctx.lineTo(x, 0);
					} else {
						ctx.moveTo(x, 0);
					}
					draw = !draw;
				}
			}
			ctx.stroke();
			ctx.restore();
		}

		/**
		 * @memberof Utils
		 * @exports DashUtils
		 */
		var DashUtils = {
			/**
			 * Draw a dashed line between two points
			 * @param {CanvasRenderingContext2D} ctx - Canvas context
			 * @param {Number} x - x coordinate of the first point
			 * @param {Number} y - y coordinate of the first point
			 * @param {Number} x2 - x coorindate of the second point
			 * @param {Number} y2 - x coordinate of the second point
			 * @param {Array<Number>} dashArray - dash lenghts
			 */
			drawDash: function (context, x, y, x2, y2, dashArray) {
				var dashCount = dashArray.length;

				var dx = (x2 - x),
					dy = (y2 - y);
				var xSlope = dx > dy;
				var slope = (xSlope) ? dy / dx : dx / dy;

				if (slope > 9999) {
					slope = 9999;
				} else if (slope < -9999) {
					slope = -9999;
				}

				var distRemaining = Math.sqrt(dx * dx + dy * dy);
				var dashIndex = 0,
					draw = true;
				while (distRemaining >= 0.1) {
					var dashLength = dashArray[dashIndex++ % dashCount];
					if (dashLength === 0) {
						dashLength = 0.001;
					}
					if (dashLength > distRemaining) {
						dashLength = distRemaining;
					}
					var step = Math.sqrt(dashLength * dashLength / (1 + slope * slope));
					if (xSlope) {
						x += dx < 0 && dy < 0 ? step * -1 : step;
						y += dx < 0 && dy < 0 ? slope * step * -1 : slope * step;
					} else {
						x += dx < 0 && dy < 0 ? slope * step * -1 : slope * step;
						y += dx < 0 && dy < 0 ? step * -1 : step;
					}
					context[draw ? 'lineTo' : 'moveTo'](x, y);
					distRemaining -= dashLength;
					draw = !draw;
				}

				context.moveTo(x2, y2);
			},
			/**
			 * Draw a dashed line
			 * @param {Context2D} ctx - canvas context
			 * @param {Number} x - start x
			 *                   @param {Number} y - start y
			 * @param {Number} x2 - end x
			 * @param {Number} y2 - end y
			 * @param {Array.<Number>} dashArray - dash pattern [px draw, px empty, px draw, px empty, ...]
			 */
			drawDashedLine: drawDashedLine,
			/**
			 * Draw a dashed rectangle
			 * @param {Context2D} ctx - canvas context
			 * @param {ShapeUtils.Rect} rect - rectangle to draw
			 * @param {Array.<Number>} dashArray - dash pattern
			 * @param {Number} [zoom] - which zoomlevel is used (use this if you want to define the dasharray in pixels and not world coordinates)
			 */
			drawDashedRect: function (ctx, rect, dashArray, zoom) {
				// we have to subtract/add half the lineWidth to be able to draw sharp lines
				var left = rect.left() + 0.5 * ctx.lineWidth;
				var right = rect.right() - 0.5 * ctx.lineWidth;
				var top = rect.top() + 0.5 * ctx.lineWidth;
				var bottom = rect.bottom() - 0.5 * ctx.lineWidth;

				// top
				drawDashedLine(ctx, left, top, right, top, dashArray, zoom);
				// bottom
				drawDashedLine(ctx, left, bottom, right, bottom, dashArray, zoom);
				// left
				drawDashedLine(ctx, left, top + 0.5 * ctx.lineWidth, left, bottom - 0.5 * ctx.lineWidth, dashArray, zoom);
				// right
				drawDashedLine(ctx, right, top + 0.5 * ctx.lineWidth, right, bottom - 0.5 * ctx.lineWidth, dashArray, zoom);
			}
		};
		return DashUtils;
	});

define('utils/HTTPUtils',["jquery",
		"underscore",
        "utils/ConvertUtils"],
	function ($,
		_,
		ConvertUtils) {
		"use strict";

		var HTTPUtils = {
			/**
			 * Construct an url (with urlencoded arguments)
			 * @param servlet {String} the servlet to call
			 * @param args {Object} parameters for url (key-value pairs)
			 * @param base {String} url (optional)
			 * @returns {String}
			 */
			constructURL: function (servlet, args, base) {
				var url = "";
				if (base) {
					url += base;
				}
				url += servlet;

				var pre = "?";
				for (var key in args) {
					if (args.hasOwnProperty(key)) {
						if (_.isArray(args[key])) {
							for (var i = 0; i < args[key].length; i++) {
								url += pre + encodeURIComponent(key) + "=" + encodeURIComponent(ConvertUtils.decodeHtmlEntities(args[key][i]));
								pre = "&";
							}
						} else {
							url += pre + encodeURIComponent(key) + "=" + encodeURIComponent(ConvertUtils.decodeHtmlEntities(args[key]));
						}
					}
					pre = "&";
				}


				return url;
			}
		};

		return HTTPUtils;
	});

define('utils/XMLUtils',[],
	function () {
		"use strict";

		var XMLUtils = null;
		XMLUtils = {
			attributeValue: function (xml, attribute) {
				var att = xml.attributes.getNamedItem(attribute);
				if (att) {
					return att.nodeValue;
				} else {
					return null;
				}
			},
			nodeTextContent: function (parent, nodename) {
				var child = parent.getElementsByTagName(nodename)[0];
				if (child) {
					return XMLUtils.textContent(child);
				} else {
					return null;
				}
			},
			textContent: function (node) {
				var content = node.textContent;
				if (content === undefined) content = node.nodeTypedValue; //ie7/8
				return content;
			},
			/**
			 * This is a utility function for converting an xml to an object for fast traversal.
			 * If a node has attributes they are available via obj.attributes[xxx], otherwise .attributes is an empty object, but
			 * KNOWN LIMITATIONS: 
			 * - attributes object will not be added for leaf nodes: the leaf node will just be the text value
			 * - a node cannot have a node called "attributes"
			 * @param {DOMNode} node - The node you want to convert to an object
			 * @param {Number} [_nrOfChildren] - Don't define this parameter. It is used by this function in recursion for optimization
			 * @returns {Object}
			 */
			xml2obj: function (node, _nrOfChildren) {
				var obj = new Object();
				var nodes = node.childNodes;

				for (var i = 0; i < nodes.length; i++) {
					var childnode = nodes[i];
					var name = childnode.nodeName;
					if (name == "#text") continue;

					var nrOfChildren = this.numberOfChildren(childnode);
					var value = undefined;
					if (nrOfChildren > 0) {
						value = this.xml2obj(childnode, nrOfChildren);
					} else {
						value = this.textContent(childnode);
					}

					if (obj[name] !== undefined) {
						if (Object.prototype.toString.call(obj[name]) !== '[object Array]') {
							var o = obj[name];
							obj[name] = [];
							obj[name].push(o);
						}

						obj[name].push(value);
					} else {
						obj[name] = value;
					}
				}

				// read the attributes (for nodes with children or if the root node has children) and attributes is defined (not the case if you pass #document)
				if ((_nrOfChildren !== undefined || this.numberOfChildren(node) > 0) && node.attributes) {
					obj.attributes = {};
					for (var a = 0; a < node.attributes.length; a++) {
						obj.attributes[node.attributes[a].name] = node.attributes[a].value;
					}
				}

				return obj;
			},
			numberOfChildren: function (node) {
				var number = 0;
				if (node.childNodes && node.childNodes.length > 0) {
					for (var i = 0; i < node.childNodes.length; i++) {
						if (node.childNodes[i].nodeType == 1) {
							number++;
						}
					}
					return number;
				} else {
					return number;
				}
			},
			/**
			 * Get the first child element by tag name
			 * @param   {Node}   node - XML element to search
			 * @param   {String}   tagName - The tag name to search
			 * @returns {Node} The element of the first element matching the tag name or undefined if not found
			 */
			getFirstChildElementByTagName: function (node, tagName) {
				for (var child = node.firstChild; child; child = child.nextSibling) {
					if (child.nodeType == 1 && (child.tagName.toLowerCase() === tagName)) {
						return child;
					}
				}
				return undefined;
			}
		};

		return XMLUtils;
	});

define('utils/I18n',["utils/HTTPUtils",
        "utils/XMLUtils",
        "underscore",
        "jquery"
       ],
function(HTTPUtils,
          XMLUtils,
          _, 
          $){
    "use strict";
    
    
	/**
	 * This module provides internationalization to the wcr-static project
	 * 
	 * @param locale {string} locale which should be used
	 * @param wcrInstanceUrl {String} url for the WebCenterInstance (e.g. http://localhost:8081/wcr-web)
	 * @param timestamp {String} timestamp - this is used a cache busting parameter for requesting language strings
	 */
	var I18n = function(locale, wcrInstanceUrl, timestamp){
		this._locale = locale ? locale : "en";
		this._timestamp = timestamp ? timestamp : 0;
        this.wcrInstanceUrl = wcrInstanceUrl;
		this.bundles = {};
		this.loading = {};
		this.bundleOrder  = [];
        this.bundleLoadedListeners = {};
	};
	
	/**
	 * Change the locale
	 * 
	 * @param {String} locale - the new locale
	 */
	I18n.prototype.setLocale = function(locale){
		this._locale = locale;
		this.bundles = {};
        var onAllLoaded = function(){};
		for (var i = 0; i < this.bundleOrder.length; i++){
			this.loadBundle(this.bundleOrder[i], onAllLoaded);
		}
	};
	
	/**
	 * Get the locale of the module this object is used for
	 * @returns {String}
	 */
	I18n.prototype.getLocale = function(){
		return this._locale;
	};
		
	/**
	 * Load one or more bundles
	 * 
	 * If fallback is enabled, the fallback will be loaded automatically!
	 * 
	 * @param {String} bundleName - the name of the bundle
	 * @param {Function} [callback] - is called when the bundle is loaded
	 * @param {Object} [binding] - bind the callback to "binding"
	 */
	I18n.prototype.loadBundle = function(bundleName, callback, binding){
		if (callback && binding){
			callback = _.bind(callback, binding);
		}
		
		if (_.isArray(bundleName)){
			// multiple bundles
			// make new callback, which only is executed after bundName.length-1 times
			var multiCallback = _.after(bundleName.length, _.once(callback));
			_.each(bundleName, _.bind(function(bundle){
				this.loadSingleBundle(bundle, multiCallback);
			}, this));
		} else {
			this.loadSingleBundle(bundleName, callback);
		}
	};
	
	/**
	 * Load a bundle
	 * 
	 * WARNING about newlines: "\n" in the translation file is interpreted as a JavaScript newline character!! (see I18n:loadSingleBundle)
	 * 
	 * If fallback is enabled, the fallback will be loaded automatically!
	 * 
	 * @param {String} bundleName - the name of the bundle
	 * @param {Function} [callback] - is called when the bundle is loaded
	 * @param {Function} [callback] - is called when the bundle failed to load
	 */
	I18n.prototype.loadSingleBundle = function(bundleName, callback, errorcallback){
		// shortcut if bundle already loaded
		if (this.bundles[bundleName]){
			if (callback){
				callback(this.bundles[bundleName]);
			}
		} else {
			// bundle info not yet available, register callback
			if (callback){
				this.whenBundleLoaded(bundleName, callback);
			}			
			
			if (!this.loading[bundleName]){
				// the bundle info is not yet being loaded
				this.bundleOrder.push(bundleName);
				this.loading[bundleName] = 1; 
                
				var self = this;
				// t and lk are just for cache busting purposes
				// we use both the timestamp of reloading the languages and the user defined locale as parameters
                $.ajax(HTTPUtils.constructURL("DownloadLocKeys.jsp", { scope : bundleName, t : this._timestamp, lk: this._locale}, this.wcrInstanceUrl), {
                    dataType: "xml",
                    success: function(data, textStatus, jqXHR){
                        // if xml, check if login failed
                        if (data) {
                            var code = data.getElementsByTagName("code");
                            if (code[0]) {
                            	errorcallback(XMLUtils.textContent(code[0]));
                                return;
                            }
                        }

                        var bundle = {};

                        var langs = data.documentElement.getElementsByTagName("lang_string");
                        for(var i=0; i<langs.length;i++){
                            var lang = langs[i];
                            bundle[lang.getAttribute("key")] = lang.getAttribute("value").replace(/\\n/g,'\n');
                        }

                        self.bundles[bundleName] = {};
                        delete self.loading[bundleName];

                        if(bundle){
                        	self.bundles[bundleName] = bundle;
                        }

                        self.fireBundleLoaded(bundleName, self.bundles[bundleName]);
                    },
                    error: function(jqXHR, textStatus, errorThrown){
                        //do error handling
                    	errorcallback();
                    },
                    cache: true,
                    type: "GET"
                });
			}
		}
	};
    
    /**
     * Register a callback to be fired when bundle with bundleName is loaded
     * @param {String} bundleName 
     * @param {Function} callback 
     */
    I18n.prototype.whenBundleLoaded = function(bundleName, callback){
        if(!this.bundleLoadedListeners[bundleName]){
            this.bundleLoadedListeners[bundleName] = [];
        }
        this.bundleLoadedListeners[bundleName].push(callback);
    };
    
    /**
     * Fire all callbacks that were registered 
     * @param {String} bundleName 
     * @param {Object} bundle     
     */
    I18n.prototype.fireBundleLoaded = function(bundleName, bundle){
        if(this.bundleLoadedListeners[bundleName]){
            _.each(this.bundleLoadedListeners[bundleName], function(callback){
                callback(bundle);
            });
            delete this.bundleLoadedListeners[bundleName];
        }
    };
	
	/**
	 * Try to find a value for key, respecting the order in which bundles are loaded
	 * @param {String} key
	 * @returns {String|undefined}
	 */
	I18n.prototype.findByKey = function(key){
		for (var i = this.bundleOrder.length-1; i >= 0; i--){
			if (this.bundles[this.bundleOrder[i]] && this.bundles[this.bundleOrder[i]][key]){
				return this.bundles[this.bundleOrder[i]][key]; 
			}
		}
		
		return undefined;
	};
	
	/**
	 * Try to find a value for a key in a certain bundle.
	 */
	I18n.prototype.findByKeyInBundle = function(key, bundleName){
		if (this.bundles[bundleName] && this.bundles[bundleName][key]) {
			return this.bundles[bundleName][key];
		} else {
			return undefined;
		}
	};
	
	/**
	 * Get the translation of a certain key
	 * 
	 * @param {String} key - key from which you want the translation
	 * @param {...String} [replacements] - if extra parameters are given, they replace {0}, {1}, ... in the translated string
	 * @returns translation - if no translation is found, the key is returned
	 */
	I18n.prototype.t = function(key){
		var value = this.findByKey(key);
		
		if (value !== undefined){			
			// replace all given options
			for (var i = 1; i < arguments.length; i++){
				value = value.replace("{"+(i-1)+"}", arguments[i]);
			}
			
			return value;
		}
					
		// we didn't find any value for key, return key
		return key;
	};
	
	/**
	 * Get the translation of a date based on a key
	 */
	I18n.prototype.d = function(key, value){
		var dateFormat =  this.findByKey(key);
		if (dateFormat !== undefined){
			return value.toString(dateFormat);
		} else {
			return undefined;
		}
	};
	
	return I18n;	
});

/**
 * WebCenter Language Strings Loader for use in require.
 * 
 * Note: See wcr-component on how to use this component!
 * 
 * @author JEPE & KADA
 */
define('utils/webi18n',[ "utils/I18n", "jquery" ], function(I18n, $) {
	"use strict";
	
	/**
	 * For debugging purposes...
	 */
	function debuglog(msg){
		//console.log(msg);
	}
	
	var WebI18n = {};
	
	/**
	 * Public, could be used externally...
	 * 
	 * Makes the wrapper around a map of language strings...
	 * 
	 * Has useful methods like t, tFallback, replacing messages, ...
	 * 
	 * Used internally when using require("utils/webi18n!...") to load language strings.
	 * 
	 * @param langmap an Object which maps key on language string
	 */
	WebI18n.makeBundle = function(langmap) {
		/**
		 * A bundle loaded with utils/webi18n!&lt;bundlename&gt; has three methods 
		 */
		var bundle = {};
	
		/**
		 * t => get the translation of a key
		 * 
		 * Also possible to use parameterization!
		 * 
		 * Returns the key if the value does not exist
		 * 
		 * @param key the key to lookup
		 * @returns the translation
		 */
		bundle.t = function(key) {
			var args = [key, key].concat(Array.prototype.slice.call(arguments).slice(1));
			return bundle.tFallback.apply(bundle, args);
		};
		
		/**
		 * tFallback => get the translation of a key or use some other value if the key does not exist...
		 * 
		 * @param key the key to lookup
		 * @param defaultValue the value to use if the key does not exist
		 * @returns the translation
		 */
		bundle.tFallback = function(key, defaultValue) {
			var value = langmap[key];
			if (value !== undefined){			
				// replace all given options
				for (var i = 2; i < arguments.length; i++){
					value = value.replace("{"+(i-2)+"}", arguments[i]);
				}
				return value;
			} else {
				return defaultValue;
			}
		};
		
		/**
		 * asMap => get the values as an object...
		 * 
		 * @returns an object with key-value pairs...
		 */
		bundle.asMap = function() {
			return langmap;
		};
		
		return bundle;
	};
	
	/**
	 * @private
	 * Internal -- loading the language bundle.
	 * 
	 * Note: might try twice if you are not logged in! 
	 * (uses webloggedin! to log in in such a case)
	 * 
	 * @param name
	 * @param callback
	 * @param errorcallback
	 * @param isretry
	 */
	WebI18n._doActualLoad = function(name, callback, errorcallback, isretry){
		debuglog("Loading bundle... \""+name+"\"");
		
		// Load the bundle...
		window.webI18n.loadSingleBundle(name, function(objectbundle) {
			// Success!
			debuglog("Bundle loaded... \""+name+"\"");
			callback(objectbundle);
		}, function(errorcode){
			// Failed...
			debuglog("Bundle failed to load... \""+name+"\" (errorcode: \""+errorcode+"\")");
			errorcallback();
		});
	};
	
	
	/**
	 * Allow using utils/webi18n!<bundlename>
	 * 
	 * To understand this method, see RequireJS documentation on how to create custom loaders...
	 */
	WebI18n.load = function (name, req, onload, config) {
		if (config.isBuild) {
			// do not trace dependencies in this loader...
			onload();
		} else {
			
			// Check if no i18n instance exists yet ...
			// i18n is in global scope since we only want to have one to make use of it's caching abilities.
			if (!window.webI18n) {
				window.webI18n = new I18n(window.webcenterConfig.userLang, window.webcenterConfig.jspBaseUrl, window.webcenterConfig.languageLoadTimestamp);
			}
			
			WebI18n._doActualLoad(name, function(objectbundle){
				// success...
				var bundle = WebI18n.makeBundle(objectbundle);
				onload(bundle);
			}, function(){
				// Something bad happened... (or the session of the user expired)
				// We could do: onload.error(); to fail the loading of the require module using webi18n.
				// But let's just return no language strings instead, that way at least everything still works 
				// (but you will see the language keys instead of the translations) 
				onload(WebI18n.makeBundle({}));
			});
		}
	};
	
	return WebI18n;
});

/**
 * (The MIT License)
 * Copyright (c) 2011 hij1nx http://www.twitter.com/hij1nx
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the 'Software'), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
 * THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 * See https://github.com/hij1nx/EventEmitter2
 */
;!function(exports, undefined) {

  var isArray = Array.isArray ? Array.isArray : function _isArray(obj) {
    return Object.prototype.toString.call(obj) === "[object Array]";
  };
  var defaultMaxListeners = 10;

  function init() {
    this._events = {};
    if (this._conf) {
      configure.call(this, this._conf);
    }
  }

  function configure(conf) {
    if (conf) {
      
      this._conf = conf;
      
      conf.delimiter && (this.delimiter = conf.delimiter);
      conf.maxListeners && (this._events.maxListeners = conf.maxListeners);
      conf.wildcard && (this.wildcard = conf.wildcard);
      conf.newListener && (this.newListener = conf.newListener);

      if (this.wildcard) {
        this.listenerTree = {};
      }
    }
  }

  function EventEmitter(conf) {
    this._events = {};
    this.newListener = false;
    configure.call(this, conf);
  }

  //
  // Attention, function return type now is array, always !
  // It has zero elements if no any matches found and one or more
  // elements (leafs) if there are matches
  //
  function searchListenerTree(handlers, type, tree, i) {
    if (!tree) {
      return [];
    }
    var listeners=[], leaf, len, branch, xTree, xxTree, isolatedBranch, endReached,
        typeLength = type.length, currentType = type[i], nextType = type[i+1];
    if (i === typeLength && tree._listeners) {
      //
      // If at the end of the event(s) list and the tree has listeners
      // invoke those listeners.
      //
      if (typeof tree._listeners === 'function') {
        handlers && handlers.push(tree._listeners);
        return [tree];
      } else {
        for (leaf = 0, len = tree._listeners.length; leaf < len; leaf++) {
          handlers && handlers.push(tree._listeners[leaf]);
        }
        return [tree];
      }
    }

    if ((currentType === '*' || currentType === '**') || tree[currentType]) {
      //
      // If the event emitted is '*' at this part
      // or there is a concrete match at this patch
      //
      if (currentType === '*') {
        for (branch in tree) {
          if (branch !== '_listeners' && tree.hasOwnProperty(branch)) {
            listeners = listeners.concat(searchListenerTree(handlers, type, tree[branch], i+1));
          }
        }
        return listeners;
      } else if(currentType === '**') {
        endReached = (i+1 === typeLength || (i+2 === typeLength && nextType === '*'));
        if(endReached && tree._listeners) {
          // The next element has a _listeners, add it to the handlers.
          listeners = listeners.concat(searchListenerTree(handlers, type, tree, typeLength));
        }

        for (branch in tree) {
          if (branch !== '_listeners' && tree.hasOwnProperty(branch)) {
            if(branch === '*' || branch === '**') {
              if(tree[branch]._listeners && !endReached) {
                listeners = listeners.concat(searchListenerTree(handlers, type, tree[branch], typeLength));
              }
              listeners = listeners.concat(searchListenerTree(handlers, type, tree[branch], i));
            } else if(branch === nextType) {
              listeners = listeners.concat(searchListenerTree(handlers, type, tree[branch], i+2));
            } else {
              // No match on this one, shift into the tree but not in the type array.
              listeners = listeners.concat(searchListenerTree(handlers, type, tree[branch], i));
            }
          }
        }
        return listeners;
      }

      listeners = listeners.concat(searchListenerTree(handlers, type, tree[currentType], i+1));
    }

    xTree = tree['*'];
    if (xTree) {
      //
      // If the listener tree will allow any match for this part,
      // then recursively explore all branches of the tree
      //
      searchListenerTree(handlers, type, xTree, i+1);
    }
    
    xxTree = tree['**'];
    if(xxTree) {
      if(i < typeLength) {
        if(xxTree._listeners) {
          // If we have a listener on a '**', it will catch all, so add its handler.
          searchListenerTree(handlers, type, xxTree, typeLength);
        }
        
        // Build arrays of matching next branches and others.
        for(branch in xxTree) {
          if(branch !== '_listeners' && xxTree.hasOwnProperty(branch)) {
            if(branch === nextType) {
              // We know the next element will match, so jump twice.
              searchListenerTree(handlers, type, xxTree[branch], i+2);
            } else if(branch === currentType) {
              // Current node matches, move into the tree.
              searchListenerTree(handlers, type, xxTree[branch], i+1);
            } else {
              isolatedBranch = {};
              isolatedBranch[branch] = xxTree[branch];
              searchListenerTree(handlers, type, { '**': isolatedBranch }, i+1);
            }
          }
        }
      } else if(xxTree._listeners) {
        // We have reached the end and still on a '**'
        searchListenerTree(handlers, type, xxTree, typeLength);
      } else if(xxTree['*'] && xxTree['*']._listeners) {
        searchListenerTree(handlers, type, xxTree['*'], typeLength);
      }
    }

    return listeners;
  }

  function growListenerTree(type, listener) {

    type = typeof type === 'string' ? type.split(this.delimiter) : type.slice();
    
    //
    // Looks for two consecutive '**', if so, don't add the event at all.
    //
    for(var i = 0, len = type.length; i+1 < len; i++) {
      if(type[i] === '**' && type[i+1] === '**') {
        return;
      }
    }

    var tree = this.listenerTree;
    var name = type.shift();

    while (name) {

      if (!tree[name]) {
        tree[name] = {};
      }

      tree = tree[name];

      if (type.length === 0) {

        if (!tree._listeners) {
          tree._listeners = listener;
        }
        else if(typeof tree._listeners === 'function') {
          tree._listeners = [tree._listeners, listener];
        }
        else if (isArray(tree._listeners)) {

          tree._listeners.push(listener);

          if (!tree._listeners.warned) {

            var m = defaultMaxListeners;
            
            if (typeof this._events.maxListeners !== 'undefined') {
              m = this._events.maxListeners;
            }

            if (m > 0 && tree._listeners.length > m) {

              tree._listeners.warned = true;
              console.error('(node) warning: possible EventEmitter memory ' +
                            'leak detected. %d listeners added. ' +
                            'Use emitter.setMaxListeners() to increase limit.',
                            tree._listeners.length);
              console.trace();
            }
          }
        }
        return true;
      }
      name = type.shift();
    }
    return true;
  };

  // By default EventEmitters will print a warning if more than
  // 10 listeners are added to it. This is a useful default which
  // helps finding memory leaks.
  //
  // Obviously not all Emitters should be limited to 10. This function allows
  // that to be increased. Set to zero for unlimited.

  EventEmitter.prototype.delimiter = '.';

  EventEmitter.prototype.setMaxListeners = function(n) {
    this._events || init.call(this);
    this._events.maxListeners = n;
    if (!this._conf) this._conf = {};
    this._conf.maxListeners = n;
  };

  EventEmitter.prototype.event = '';

  EventEmitter.prototype.once = function(event, fn) {
    this.many(event, 1, fn);
    return this;
  };

  EventEmitter.prototype.many = function(event, ttl, fn) {
    var self = this;

    if (typeof fn !== 'function') {
      throw new Error('many only accepts instances of Function');
    }

    function listener() {
      if (--ttl === 0) {
        self.off(event, listener);
      }
      fn.apply(this, arguments);
    };

    listener._origin = fn;

    this.on(event, listener);

    return self;
  };

  EventEmitter.prototype.emit = function() {
    
    this._events || init.call(this);

    var type = arguments[0];

    if (type === 'newListener' && !this.newListener) {
      if (!this._events.newListener) { return false; }
    }

    // Loop through the *_all* functions and invoke them.
    if (this._all) {
      var l = arguments.length;
      var args = new Array(l - 1);
      for (var i = 1; i < l; i++) args[i - 1] = arguments[i];
      for (i = 0, l = this._all.length; i < l; i++) {
        this.event = type;
        this._all[i].apply(this, args);
      }
    }

    // If there is no 'error' event listener then throw.
    if (type === 'error') {
      
      if (!this._all && 
        !this._events.error && 
        !(this.wildcard && this.listenerTree.error)) {

        if (arguments[1] instanceof Error) {
          throw arguments[1]; // Unhandled 'error' event
        } else {
          throw new Error("Uncaught, unspecified 'error' event.");
        }
        return false;
      }
    }

    var handler;

    if(this.wildcard) {
      handler = [];
      var ns = typeof type === 'string' ? type.split(this.delimiter) : type.slice();
      searchListenerTree.call(this, handler, ns, this.listenerTree, 0);
    }
    else {
      handler = this._events[type];
    }

    if (typeof handler === 'function') {
      this.event = type;
      if (arguments.length === 1) {
        handler.call(this);
      }
      else if (arguments.length > 1)
        switch (arguments.length) {
          case 2:
            handler.call(this, arguments[1]);
            break;
          case 3:
            handler.call(this, arguments[1], arguments[2]);
            break;
          // slower
          default:
            var l = arguments.length;
            var args = new Array(l - 1);
            for (var i = 1; i < l; i++) args[i - 1] = arguments[i];
            handler.apply(this, args);
        }
      return true;
    }
    else if (handler) {
      var l = arguments.length;
      var args = new Array(l - 1);
      for (var i = 1; i < l; i++) args[i - 1] = arguments[i];

      var listeners = handler.slice();
      for (var i = 0, l = listeners.length; i < l; i++) {
        this.event = type;
        listeners[i].apply(this, args);
      }
      return (listeners.length > 0) || this._all;
    }
    else {
      return this._all;
    }

  };

  EventEmitter.prototype.on = function(type, listener) {
    
    if (typeof type === 'function') {
      this.onAny(type);
      return this;
    }

    if (typeof listener !== 'function') {
      throw new Error('on only accepts instances of Function');
    }
    this._events || init.call(this);

    // To avoid recursion in the case that type == "newListeners"! Before
    // adding it to the listeners, first emit "newListeners".
    this.emit('newListener', type, listener);

    if(this.wildcard) {
      growListenerTree.call(this, type, listener);
      return this;
    }

    if (!this._events[type]) {
      // Optimize the case of one listener. Don't need the extra array object.
      this._events[type] = listener;
    }
    else if(typeof this._events[type] === 'function') {
      // Adding the second element, need to change to array.
      this._events[type] = [this._events[type], listener];
    }
    else if (isArray(this._events[type])) {
      // If we've already got an array, just append.
      this._events[type].push(listener);

      // Check for listener leak
      if (!this._events[type].warned) {

        var m = defaultMaxListeners;
        
        if (typeof this._events.maxListeners !== 'undefined') {
          m = this._events.maxListeners;
        }

        if (m > 0 && this._events[type].length > m) {

          this._events[type].warned = true;
          console.error('(node) warning: possible EventEmitter memory ' +
                        'leak detected. %d listeners added. ' +
                        'Use emitter.setMaxListeners() to increase limit.',
                        this._events[type].length);
          console.trace();
        }
      }
    }
    return this;
  };

  EventEmitter.prototype.onAny = function(fn) {

    if(!this._all) {
      this._all = [];
    }

    if (typeof fn !== 'function') {
      throw new Error('onAny only accepts instances of Function');
    }

    // Add the function to the event listener collection.
    this._all.push(fn);
    return this;
  };

  EventEmitter.prototype.addListener = EventEmitter.prototype.on;

  EventEmitter.prototype.off = function(type, listener) {
    if (typeof listener !== 'function') {
      throw new Error('removeListener only takes instances of Function');
    }

    var handlers,leafs=[];

    if(this.wildcard) {
      var ns = typeof type === 'string' ? type.split(this.delimiter) : type.slice();
      leafs = searchListenerTree.call(this, null, ns, this.listenerTree, 0);
    }
    else {
      // does not use listeners(), so no side effect of creating _events[type]
      if (!this._events[type]) return this;
      handlers = this._events[type];
      leafs.push({_listeners:handlers});
    }

    for (var iLeaf=0; iLeaf<leafs.length; iLeaf++) {
      var leaf = leafs[iLeaf];
      handlers = leaf._listeners;
      if (isArray(handlers)) {

        var position = -1;

        for (var i = 0, length = handlers.length; i < length; i++) {
          if (handlers[i] === listener ||
            (handlers[i].listener && handlers[i].listener === listener) ||
            (handlers[i]._origin && handlers[i]._origin === listener)) {
            position = i;
            break;
          }
        }

        if (position < 0) {
          return this;
        }

        if(this.wildcard) {
          leaf._listeners.splice(position, 1)
        }
        else {
          this._events[type].splice(position, 1);
        }

        if (handlers.length === 0) {
          if(this.wildcard) {
            delete leaf._listeners;
          }
          else {
            delete this._events[type];
          }
        }
      }
      else if (handlers === listener ||
        (handlers.listener && handlers.listener === listener) ||
        (handlers._origin && handlers._origin === listener)) {
        if(this.wildcard) {
          delete leaf._listeners;
        }
        else {
          delete this._events[type];
        }
      }
    }

    return this;
  };

  EventEmitter.prototype.offAny = function(fn) {
    var i = 0, l = 0, fns;
    if (fn && this._all && this._all.length > 0) {
      fns = this._all;
      for(i = 0, l = fns.length; i < l; i++) {
        if(fn === fns[i]) {
          fns.splice(i, 1);
          return this;
        }
      }
    } else {
      this._all = [];
    }
    return this;
  };

  EventEmitter.prototype.removeListener = EventEmitter.prototype.off;

  EventEmitter.prototype.removeAllListeners = function(type) {
    if (arguments.length === 0) {
      !this._events || init.call(this);
      return this;
    }

    if(this.wildcard) {
      var ns = typeof type === 'string' ? type.split(this.delimiter) : type.slice();
      var leafs = searchListenerTree.call(this, null, ns, this.listenerTree, 0);

      for (var iLeaf=0; iLeaf<leafs.length; iLeaf++) {
        var leaf = leafs[iLeaf];
        leaf._listeners = null;
      }
    }
    else {
      if (!this._events[type]) return this;
      this._events[type] = null;
    }
    return this;
  };

  EventEmitter.prototype.listeners = function(type) {
    if(this.wildcard) {
      var handlers = [];
      var ns = typeof type === 'string' ? type.split(this.delimiter) : type.slice();
      searchListenerTree.call(this, handlers, ns, this.listenerTree, 0);
      return handlers;
    }

    this._events || init.call(this);

    if (!this._events[type]) this._events[type] = [];
    if (!isArray(this._events[type])) {
      this._events[type] = [this._events[type]];
    }
    return this._events[type];
  };

  EventEmitter.prototype.listenersAny = function() {

    if(this._all) {
      return this._all;
    }
    else {
      return [];
    }

  };

  if (typeof define === 'function' && define.amd) {
    define('lib/EventEmitter2',[],function() {
      return EventEmitter;
    });
  } else {
    exports.EventEmitter2 = EventEmitter; 
  }

}(typeof process !== 'undefined' && typeof process.title !== 'undefined' && typeof exports !== 'undefined' ? exports : window);

define('utils/WCEvent',["lib/EventEmitter2"],
	function (EventEmitter2) {
		"use strict";

		/**
		 * Object to create and handle events in WebCenter.
		 * The module immediately returns the instance of the event emitter, so it can be used like this:
		 * 	- WCEvent.on("xxx", function(){...})
		 *  - WCEvent.emit("xxx", arguments...)
		 */
		var WCEvent = (function () {

			function init() {
				var ebus = new EventEmitter2({
					wildcard: true, // should the event emitter use wildcards.
					delimiter: '.'
				});

				var maxListeners = 0;

				//>>excludeStart("PExclude", pragmas.productionExclude);
				// during development we want maximum 20 listeners per channel, in production mode this can be much more
				maxListeners = 20;

				// IE9 fix for EventEmitter (for when we exceed the maxlistener limit)
				if (!window.console) {
					window.console = {};
				}
				if (!window.console.trace) {
					window.console.trace = function () {};
				}
				//>>excludeEnd("PExclude");	

				ebus.setMaxListeners(maxListeners);

				return {
					on: function () {
						ebus.on.apply(ebus, arguments);
					},
					off: function () {
						ebus.off.apply(ebus, arguments);
					},
					emit: function () {
						ebus.emit.apply(ebus, arguments);
					}
				};
			};

			return {
				// Get the Singleton instance if one exists or create one if it doesn't
				getInstance: function () {
					if (!window.WCEvent) {
						window.WCEvent = init();
					}
					return window.WCEvent;
				}
			};

		})();

		return WCEvent.getInstance();
	});

define('utils/EventBasedValidator',["utils/webi18n!validatorStrings",
        "jquery",
        "underscore",
        "utils/WCEvent"], 
function(ValidatorStrings,
         $,
         _,
         WCEvent){
	"use strict";

	var EventBasedValidator = (function() {


		function init() {
			return {
				onValidateUniqueDocName: function(callback){
					WCEvent.on("validation.uniqueDocName", callback);
				},
				removeListener: function(callback){
					WCEvent.off("validation.uniqueDocName", callback);
				},

				///////////////////////
				//specific functions //
				///////////////////////
				
				//////////////////////////////
				// check unique document name:
				//////////////////////////////
				
				// create validation object
				createUniqueNamesObj: function(){
					return {
			    		errors : [],
			    		newDocs: [],
			    		newDocsDOMElems: {}, 
			    		newDocsMetadata: {}, 
			    		excludedProjects: {}, 
			    		localClashFound: false,
			    		emptyDocsFound: false,
			    		addDoc: function(docNameInput, projectID, metadata, elemToHighlight){
			    			if (!elemToHighlight) {
			    				// by default the DOM element to be highlighted in case of an error is the input field itself
			    				elemToHighlight = docNameInput;
			    			}
			    			var inputId = docNameInput.attr('id');
			    			if(!inputId){
			    				inputId = _.uniqueId();
			    				docNameInput.attr('id', inputId);
			    			}
			    			this.newDocsMetadata[inputId] = metadata;
			    			
			    			// register document for backend name validation and check for duplicate documents on the page 
			    	    	if(docNameInput.val()){
			    	    		// do everything in lower case
			    	    		var docName = docNameInput.val().toLowerCase();
			    	    		if(!this.newDocsDOMElems[projectID + '_' + docName]){
			    	        		// no duplicate document found yet. We'll add it to the list of existing docs
			    	    			this.newDocsDOMElems[projectID + '_' + docName] = elemToHighlight;
			    	    			
			    	    			// add for backend check, but only if there's a project and it's not excluded
			    	    			if(projectID && !this.excludedProjects[projectID]){
			    	    				this.newDocs.push({docName: docName, projectId: projectID});
			    	    			}
			    	        	} else {
			    	        		// document is already added to this project on this page, we have a duplicate
			    	        		// we'll add the element with the other one, so operations (like highlight) will affect both elements
			    	        		this.newDocsDOMElems[projectID + '_' + docName].push(elemToHighlight[0]);
			    	        		
			    	        		// we only want to add the error message once, as it is a general message (not parametrized)
			    	        		if(!this.localClashFound){
			    	        			// add error object, we want the validation to fail
				    	        		var errorObj = {
			    	        				msg : ValidatorStrings.t('PROJ_CREATION_SPECIFY_UNIQUE_DOCNAMES'),
			    	        				element : this.newDocsDOMElems[projectID + '_' + docName]
			    	        			};
				    	        		this.errors.splice(0,0,errorObj);
				    	        		this.localClashFound = true;
			    	        		}
			    	        	}
			    	    	} else {
			    	    		if(!this.emptyDocsFound){ // only add error once
			    	    			var errorObj = {
			    	        				msg : ValidatorStrings.t('INVALIDDOCNAMEMSG'),
			    	        				element : elemToHighlight
			    	        			};
					    	    		this.emptyDocsFound = true;
				    	        		this.errors.splice(0,0,errorObj);
			    	    		}
			    	    	}
			    		},
			    		excludeProject: function(projId){
			    			this.excludedProjects[projId] = true;
			    		},
			    		isOk: function(){
			    			return this.errors.length == 0;
			    		},
						getErrorMetadata: function(error){
							var res = [];
							_.each(error.element, _.bind(function(elem){
								if(elem.id && this.newDocsMetadata[elem.id]){
									res.push(this.newDocsMetadata[elem.id]);
								}
							}, this));
							return res;
						}
			    	};
				},
				
				// do the actual validation of unique document names
				validateNamesGlobally: function(validObj, noAlerts){
					// remove highlight
					$('.validationHighlight').removeClass('validationHighlight');
					
					// if no validation element was passed, create a new one
					if(!validObj){
						validObj = this.createUniqueNamesObj();
					}

					// use validation object and send it around to all listeners (which will handle this synchronously)
					WCEvent.emit("validation.uniqueDocName", validObj);
					
					// if backend check is needed, do the ajax call
					if(validObj.newDocs.length > 0){
						// check on backend for duplicates
		                var checkDocnamesUrl = "GetDocumentsByNameAndProjectID.jsp";
		                
		                $.ajax({
		        			url: checkDocnamesUrl,
		        			dataType : "json",
		        			method: "POST",
		        			async: false,
		        			data: { 
		        		        'data': JSON.stringify({docs: validObj.newDocs}) 
		        		    }
		        		}).done(function(jsonObj){
		        			if(jsonObj.docs && jsonObj.docs.length > 0){
		        				for(var i=0 ; i<jsonObj.docs.length ; i++){
		        					// conflict found, add errorObject
		        					var errorObj = {
		        						msg : ValidatorStrings.t('NAMECONFLICTEXCEPTION_ADV_TAG',jsonObj.docs[i].docName, jsonObj.docs[i].projectName),
		        						element : validObj.newDocsDOMElems[jsonObj.docs[i].projectId + '_' + jsonObj.docs[i].docName.toLowerCase()]
		                        	};
		                			validObj.errors.push(errorObj);
		        				}	
		        			}
		                });
					}
	                
					// now handle all errors
					if(!noAlerts){
						var errorMsg = '';
	        			for (var errIdx = 0; errIdx < validObj.errors.length; errIdx++) {
		        			var errObj = validObj.errors[errIdx];
		        			
		        			if (errObj.msg) {
		        				errorMsg += errObj.msg + '\n';
		        			}
		        			if(errObj.element){
		        				errObj.element.addClass('validationHighlight');
		        			}
		        		}
	        			if(errorMsg){
	        				alert(errorMsg);
	        			}
					}
				}
			};

		};

		return {
			// Get the Singleton instance if one exists
			// or create one if it doesn't
			getInstance : function() {
				if (!window.eventBasedValidator) {
					window.eventBasedValidator = init();
				}
				return window.eventBasedValidator;
			}
		};

	})();
	
	return EventBasedValidator;
});

define('utils/ExecutionQueue',["underscore"], function (_) {
	"use strict";


	/**
	 * ExecutionQueue can be used to execute heave operations that can't be executed in a WebWorker
	 * You can split up the work in smaller tasks using .run()
	 * When you need to process an array or map, use forEach
	 * it will split up the loop in smaller chunks and make sure the browser is not blocked for longer than 15 ms at a time
	 *
	 * @exports ExecutionQueue
	 * @class
	 *
	 * @param {Boolean} [synchronized=false] - if true, all operations will block EventQueue and be executed synchronously
	 * @param {Boolean} [updateLoadingIndicator=!synchronized] - Default value: true when assynchronous, false when synchronous
	 */
	var ExecutionQueue = function (synchronized, updateLoadingIndicator) {
		/**
		 * @member {Number} interval when deferring tasks
		 * @private
		 */
		this._interval = 1;
		/**
		 * @member {Number} maximum execution time of a chunk when doing foreach
		 */
		this._loopChunkTime = 15;
		/**
		 * @member {Array.<Array>} execution queue, [fn, context, assync, key, description]
		 * @private
		 */
		this._queue = [];
		/**
		 * @member {Boolean} is the queue executing?
		 * @private
		 */
		this._isRunning = false;
		/**
		 * @member {Number} timer ID (used to cancel the timer if needed
		 * @private
		 */
		this._timer = null;
		/**
		 * @member {Boolean} is the queue in synchronized mode?
		 */
		this._synchronized = synchronized;
		
		/**
		 * @member {Number} timestamp for when the runtime was blocked for the first time. this is used when the execution queue batches run calls
		 */
		this._blockingJSRuntimeSince = 0;
		/**
		 * @member {Boolean} batch run calls. By default consecutive run calls are deferred. If this is true, we try to run as much run calls as possible in the loopChunkTime. if we exceed the loopchunktime, we defer the next run call
		 */
		this._batchRunCalls = false;
		
		/**
		 * @member {Boolean} should start/stop be made to LoadingIndicator when starting/stopping tasks
		 * @private
		 */
		this._shouldUpdateLoadingIndicator = updateLoadingIndicator || !synchronized;

		/**
		 * @member {Array.<Function>} list of callbacks waiting for the queue to finish executing
		 * @private
		 */
		this._finishedCallbacks = [];
	};

	/**
	 * Put a function to the queue
	 * if the executor is not running, it will start runnning
	 * @param {Function} fn - task that needs to be executed, if assync is true the first parameter is a callback function that needs to be called once the operation has finished
	 * @param {Any} [context] - context for when executing fn
	 * @param {Boolean} [assync=false] - if true fn() can be assynchronous, once finish it should call the callback provided as first argument of fn
	 * @param {String} [key] - If LoadingIndicator should be shown during action you can provide a key for loading action here
	 * @param {String} [description} - If LoadingIndicator should be shown during action you can provide an optional description here
	 */
	ExecutionQueue.prototype.run = function (fn, context, assync, key, description) {
		this._queue.push([fn, context, assync, key, description]);
		if (!this._isRunning){
			this._startTimer();
		}
	};

	/**
	 * Run over a big list/map or do heave operations on a list
	 * Whenever execution takes more than loopChunkTime, the loop is stopped and processed further after a timeout
	 * This enables to execute heavy operations without blocking the browser too much
	 * @param {Array, Object} collection - collection that needs processing
	 * @param {Function} fn - arguments: (value, key/index)
	 * @param {Any} [context] - context for when calling fn
	 * @param {String} [key] - If LoadingIndicator should be shown during action you can provide a key for loading action here
	 * @param {String} [description} - If LoadingIndicator should be shown during action you can provide an optional description here
	 */
	ExecutionQueue.prototype.forEach = function (collection, fn, context, key, description) {
		var loopChunkTime = this._loopChunkTime;
		var queue = this;

		// run an assync task
		this.run(function (whenFinished) {
			var currentIndex = 0;
			var isArrayLike = _.isArray(collection);

			var keys;
			var maxIndex;

			if (isArrayLike) {
				maxIndex = collection.length;
			} else {
				keys = _.keys(collection);
				maxIndex = keys.length;
			}

			// execute a chunk
			var executeChunk = function () {
				var startTime = queue._getCurrentTime();
				while (queue._getCurrentTime() - startTime < loopChunkTime && currentIndex < maxIndex) {
				
					if (isArrayLike) {
						fn.call(context || window, collection[currentIndex], currentIndex);
						currentIndex++;
					} else {
						var key = keys[currentIndex];
						if (collection.hasOwnProperty(key)) {
							fn.call(context || window, collection[key], key);
						}
						currentIndex++;
					}
				}

				if (currentIndex < maxIndex) {
					// we are not finished yet, process another chunk
					queue._defer(executeChunk);
				} else {
					// we are finished!
					whenFinished();
				}
			};

			// process the first chunk
			executeChunk();
		}, window, true, key, description);
	};

	/**
	 * Start processing the queue if needed. Or notify that processing has finshed if there are no tasks anymore
	 */
	ExecutionQueue.prototype._startTimer = function () {
		if (!this._timer && this._queue.length > 0) {
			this._isRunning = true;
			this._blockingJSRuntimeSince = 0;
			this._timer = this._defer(_.bind(this._executeNext, this));
		} else if (this._queue.length === 0) {
			this._setFinished();
		}
	};
	/**
	 * Execute the next operation (and start the timer for the following operation)
	 */
	ExecutionQueue.prototype._executeNext = function () {
		if (this._queue.length > 0) {
			if (this._batchRunCalls && !this._blockingJSRuntimeSince ){
				this._blockingJSRuntimeSince = this._getCurrentTime();
			}
			var operation = this._queue.shift();

			if (operation[3]) {
				this._updateLoadingIndicator(true, operation[3], operation[4]);
			}

			if (operation[2]) {
				// assync operation
				operation[0].call(operation[1] || window, _.bind(this._operationFinished, this, operation[3]));
			} else {
				// synchronous operation
				operation[0].call(operation[1] || window);
				this._operationFinished(operation[3]);
			}
		} else {
			this._setFinished();
		}
	};

	/**
	 * Executed when an operation has finished
	 * @param {String} [key] - If LoadingIndicator should be shown during action you can provide a key for loading action here
	 */
	ExecutionQueue.prototype._operationFinished = function (key) {
		this._timer = null;		
		if (this._batchRunCalls && this._getCurrentTime() - this._blockingJSRuntimeSince < this._loopChunkTime){
			this._executeNext();
		} else {
			this._startTimer();
		}

		if (key) {
			this._updateLoadingIndicator(false, key);
		}
	};

	/**
	 * Clear the queue
	 */
	ExecutionQueue.prototype.clear = function () {
		this._queue = [];
		clearTimeout(this._timer);
		this._timer = null;
	};

	/**
	 * Wait till the queue has finished processing
	 * @param {Function} callback - called whenever the queue is finished and all tasks are processed
	 */
	ExecutionQueue.prototype.whenFinished = function (callback) {
		if (!this._isRunning) {
			callback();
		} else {
			this._finishedCallbacks.push(callback);
		}
	};

	/**
	 * mark the queue as finished (and execute needed callbacks)
	 */
	ExecutionQueue.prototype._setFinished = function () {
		this._isRunning = false;

		_.each(this._finishedCallbacks, function (callback) {
			callback();
		});

		this._finishedCallbacks = [];
	};

	/**
	 * Execute a task (deferred if queue is in asynchronous mode)
	 * @param {Function} fn
	 */
	ExecutionQueue.prototype._defer = function (fn) {
		if (!this._synchronized) {
			return setTimeout(fn, this._interval);
		} else {
			fn();
			return null;
		}
	};

	/**
	 * Update loading indicator if needed
	 * @param {Boolean} start - is an operation started or stopped
	 * @param {String} key - LoadingIndicator key
	 * @param {String} [description] - message to show in the LoadingIndicator
	 */
	ExecutionQueue.prototype._updateLoadingIndicator = function (start, key, description) {
		if (this._shouldUpdateLoadingIndicator) {
			requirejs(["component/loadingindicator/LoadingIndicator"], function (LoadingIndicator) {
				if (start) {
					LoadingIndicator.start(key, description);
				} else {
					LoadingIndicator.stop(key);
				}
			});
		}
	};
	
	/**
	 * Did the queue finish running
	 * @returns {Boolean}
	 */
	ExecutionQueue.prototype.isFinished = function(){
		return !this._isRunning;
	};
	
	/**
	 * When running in Batch mode, consecutive .run() calls will be ran in a non-deferred way if the executionqueue hasn't been blocking the queue for longer than 15ms
	 * If you are queuing a lot of small run tasks, this option can be very interesting to enable.
	 * By default all .run() calls are deferred
	 */
	ExecutionQueue.prototype.setBatchRunCalls = function(batchRunCalls){
		this._batchRunCalls = batchRunCalls;
	};
	
	/**
	 * Get the current time
	 * @returns {Number} current time in ms
	 */
	ExecutionQueue.prototype._getCurrentTime = function(){
		return Date.now();
	};
		

	return ExecutionQueue;
});

define('utils/extendedstring',[], function () {
"use strict";
    
    /**
     * Defines a replaceAll function on a string.
     * (source: http://stackoverflow.com/questions/1137436/what-are-useful-javascript-methods-that-extends-built-in-objects/1141816#1141816 )
     */
    String.prototype.replaceAll = function(search, replace) {
        if (replace === undefined) {
            return this.toString();
        }
        return this.split(search).join(replace);
    }
    
    return null;
});

define("utils/init", ["utils/ExecutionQueue", "jquery"], function (ExecutionQueue, $) {
	var exports = {};


	/**
	 * Initialize all wcr components on the page (or in a certain subtree)
	 * 
	 * (Calls wcrinit on the module defined by the data-init attribute) 
	 * 
	 * @param root the subtree to process...
	 * @param callback callback when done...
	 */
	exports.doinit = function (root, callback) {
		if (root !== undefined && root != null) {
			root = $(root);
		}
		var found = 0;
		var inited = 0;

		var execQueue = new ExecutionQueue();
		execQueue.setBatchRunCalls(true); // we rely on multiple .run() calls on the ExecutionQueue, by default these are all deferred, but we want to batch them so we work around 15ms without deferring to execute the next run call

		// Searches for all data-init attributes...
		$("*[data-init]", root).each(function (i, e) {
			// If that node was not initialized before... 
			if (!e.datainitdone) {
				e.datainitdone = true;
				found++;
				// look inside the attribute and get the module we need
				
				var module_to_init = $(e).attr("data-init");
			
				require([module_to_init], function (mod) {
					// make sure to run the module initializations in the execution queue
					execQueue.run((function (i, e, mod, module_to_init) {
						return function () {
							// now call the wcrinit(div) method on that module
							if (mod instanceof Object && mod.wcrinit) {
								// but save the function to destroy the element first...
								if (mod.wcrdestroy !== undefined) {
									e.destroyfunction = mod.wcrdestroy;
								}

								// initialize the actual component
								mod.wcrinit(e);

								// init was called...
								e.initcalled = true;

								// call init listeners on this element
								if (e.initlisteners !== undefined) {
									$.each(e.initlisteners, function (listeneri, listenerf) {
										listenerf.call(e);
									});
									e.initlisteners = undefined;
								}
							} else {
								require(["utils/wcrerror"], function (WCRError) {
									throw new WCRError(null, "Could not initialize module \"" + module_to_init + "\" on component. This module does not support node initialization...");
								});
							}

							// for the callback of this function
							inited++;
							if (inited === found && callback !== undefined) {
								callback();
							}
						};
					})(i, e, mod, module_to_init));
				});
			}

		});



		if (inited === found && callback !== undefined) {
			callback();
		}
	};

	/**
	 * Destroy all wcr components on the page (or in a certain subtree)
	 * ! Do this before removing html from the page...
	 * 
	 * (Calls wcrdestroy on the module defined by the data-init attribute)
	 * 
	 * This callback should be used by the module to remove listeners and stop timers...
	 * 
	 * @param root the subtree to process...
	 * @param callback callback when done...
	 */
	exports.dodestroy = function (root, callback) {
		if (root !== undefined && root != null) {
			root = $(root);
		}
		// Searches for all data-init attributes...
		$("*[data-init]", root).each(function (i, e) {
			// If that node was not initialized before... 
			if (e.datainitdone === true) {
				e.datainitdone = false;
				var destroy = function () {
					// check if there is a destroyfunction...
					if (e.destroyfunction !== undefined) {
						e.destroyfunction(e);
						delete e.destroyfunction;
					}
				};

				if (!e.initcalled) {
					// init has been called but is still loading the require module...
					if (e.initlisteners === undefined) {
						e.initlisteners = [];
					}
					e.initlisteners.push(destroy);
				} else {
					// the require module is loaded too...
					destroy();
				}
			}
		});
	};

	/**
	 * Wait for the component of a certain element to be initialized and call the callback if it is.
	 * 
	 * @param jQueryComponent a jQuery object (possibly containing multiple elements)
	 * @param callback the callback to execute (no arguments)
	 */
	exports.whenInitialized = function (jQueryComponent, callback) {
		if (jQueryComponent instanceof $) {
			var waitfor = jQueryComponent.length;
			if (waitfor === 0) {
				callback();
			}

			function onedonewaiting() {
				waitfor--;
				if (waitfor === 0) {
					callback();
				}
			}
			jQueryComponent.each(function (i, e) {
				if (e.datainitdone && e.initcalled) {
					// already initialized
					onedonewaiting();
				} else if (!$(e).attr("data-init")) {
					// not a component
					onedonewaiting();
				} else {
					// has data-init & not (fully) initialized yet
					if (e.initlisteners === undefined) {
						e.initlisteners = [];
					}
					e.initlisteners.push(onedonewaiting);
				}
			});
		} else {
			if (console && console.log){
				console.log("init.whenInitialized called with wrong first argument, expects jQuery selector");
			}
		}
	};

	return exports;
});

define('utils/Map',["underscore"],
	function (_) {
		"use strict";

		/**
		 * @class Map a Map is an implementation of the Map datastructure. It can contain unique keys (with a value assigned to it)
		 * @constructor
		 */
		var Map = function () {
			this._map = {};
		};

		/**
		 * Add an entry to the set
		 * @param {String} key - the entry name 
		 * @param {Any} value - the value for the entry (should not be undefined!)
		 */
		Map.prototype.add = function (key, value) {
			if (value === undefined) {
				throw new Error("value parameter should not be undefined!");
			}

			this._map[key] = value;
		};

		/**
		 * Check if an entry exists in the set
		 * @param {String} key - the entry name
		 * @returns {Boolean}
		 */
		Map.prototype.has = function (key) {
			return this._map.hasOwnProperty(key);
		};

		/**
		 * Get the value for an entry
		 * @param {String} key - the entry name
		 * @returns {Any} the value of the entry, or undefined if entry does not exist
		 */
		Map.prototype.get = function (key) {
			if(this._map.hasOwnProperty(key)) {
				return this._map[key];
			} else {
				return undefined;
			}
		};

		/**
		 * Remove an entry
		 * @param {String} key - the entry name to remove
		 * @returns {Object} the value stored for the key
		 */
		Map.prototype.remove = function (key) {
			var oldValue = this.get(key);
			delete this._map[key];
			return oldValue;
		};

		/**
		 * Clear the set
		 */
		Map.prototype.clear = function () {
			this._map = {};
		};

		/**
		 * Get a list of entry names
		 * @returns {Array.<String>}
		 */
		Map.prototype.entries = function () {
			return _.keys(this._map);
		};

		/**
		 * Get a list of values
		 * @returns {Array.<Any>}
		 * @returns
		 */
		Map.prototype.values = function () {
			return _.values(this._map);
		};

		/**
		 * Get the size of the map 
		 * @returns {Number}
		 */
		Map.prototype.size = function () {
			return _.size(this._map);
		};

		/**
		 * Is the map empty
		 * @returns {Boolean} True if there are no elements in the map (size = 0), false otherwise
		 */
		Map.prototype.isEmpty = function () {
			return this.size() === 0;
		};

		/**
		 * Get access to the map object with key/value pairs
		 * IMMUTABLE (only use for iterating)
		 * @returns {Object} 
		 */
		Map.prototype.asObject = function () {
			return this._map;
		};
		
		/**
		 * Loop through all key value pairs.
		 * @param {Function} callback - a function with two parameters: "key" & "value"
		 */
		Map.prototype.each = function(callback) {
			_.each(_.keys(this._map), _.bind(function(key, i) {
				var value = this.get(key);
				callback(key, value);
			}, this));
		};

		/**
		 * Change each key by the transform function and return as new Map
		 * @param {Function} transformFunction - Function(key) to transform the key
		 * @returns {Map} This map as new Map with all keys transformed by the given function                                  
		 */
		Map.prototype.transformKeys = function (transformFunction) {
			var newMap = new Map();
			this.each(function(key, value) {
				newMap.add(transformFunction(key), value);
			});
			return newMap;
		};

		return Map;
	});

define('utils/Mousewheel',["underscore"],
	function (_) {
		/**
		 * Handle mousewheel events
		 */
		var Mousewheel = new(function () {

			var listeners = [];

			this.enabled = true;

			var handleScrollEvent = _.bind(function (event) {
				if (this.enabled) {
					var delta = Math.max(-1, Math.min(1, (event.wheelDelta || -event.detail)));
					/**
					 * If delta is nonzero, handle it. Basically, delta is now
					 * positive if wheel was scrolled up, and negative, if wheel
					 * was scrolled down.
					 */
					if (delta) {
						_.each(listeners, function (callback) {
							callback(event, delta);
						});
					}
				}

			}, this);


			if (window.addEventListener && /Firefox/i.test(navigator.userAgent)) {
				/** DOMMouseScroll is for mozilla. */
				window.addEventListener('DOMMouseScroll', handleScrollEvent, false);
			} else {
				/** IE/chrome */
				document.onmousewheel = handleScrollEvent;
			}


			this.addListener = function (callback) {
				if (_.indexOf(listeners, callback) < 0) {
					listeners.push(callback);
				}
			};

			this.removeListener = function (callback) {
				listeners = _.without(listeners, callback);
			};
		})();

		return Mousewheel;
	});

define('utils/Observable',[
		"lib/EventEmitter2",
		"underscore"
	],
	function (
		EventEmitter2,
		_
	) {
		"use strict";

		/**
		 * Collection of instances for the observables installed on objects on the current page.
		 * Components can shooce to use the default event bus or specific private event bus.
		 * Collection is here for the singleton pattern.
		 */
		var observableInstances = {};


		/**
		 * Observable is a singleton defintion to install an event bus and make the object observable.
		 * The constructor of the class in not publically available; only possible to use mixin-method.
		 */
		var Observable = function (eventBusID) {
			this._eventBusID = eventBusID;

			this.ebus = new EventEmitter2({
				wildcard: true, // should the event emitter use wildcards.
				delimiter: '.', // the delimiter used to segment namespaces,
				newListener: true // if you want to emit the newListener event
			});

			var maxListeners = 0;

			//>>excludeStart("PExclude", pragmas.productionExclude);
			// during development we want maximum 20 listeners per channel, in production mode this can be much more
			maxListeners = 20;

			// IE9 fix
			if (!window.console) {
				window.console = {};
			}
			if (!window.console.trace) {
				window.console.trace = function () {};
			}
			//>>excludeEnd("PExclude");	

			this.ebus.setMaxListeners(maxListeners);
		};

		return {
			/**
			 * Mixin Observable functionality to an object
			 * @param {Object} object - the object in which the Observable functionality should be mixed in
			 * @param {String|Array} ID - the unique identifier of the observable object
			 * @param {String} [eventBusID] - If not provided a new EventBus is created; which instance of event bus should be used for the observable events?
			 */
			mixin: function (object, ID, eventBusID) {
				if (!ID) {
					throw new Error("ID should be defined");
				}

				if (_.isArray(ID)) {
					ID = ID.join('.');
				}

				var eventBus;

				if (eventBusID) {
					if (!observableInstances[eventBusID]) {
						observableInstances[eventBusID] = new Observable(eventBusID);
					}
					eventBus = observableInstances[eventBusID].ebus;
				} else {
					eventBus = new Observable(ID).ebus;
				}


				/**
				 * Subscribe to a channel on the Observable
				 * @param {String} channel - the channel to which to subscribe
				 * @param {Function} callback - the callback called when the channel event occurs
				 */
				object.on = function (channel, callback) {
					arguments[0] = [ID, channel].join('.');
					eventBus.on.apply(eventBus, arguments);
				};

				/**
				 * Emit an event on a certain sub channel
				 */
				object._emit = function (channel) {
					arguments[0] = [ID, channel].join('.');
					eventBus.emit.apply(eventBus, arguments);
				};

				/**
				 * Subscribe once to a channel on the Observable (after first event, the subscription is removed)
				 * @param {String} channel - the channel to which to subscribe
				 * @param {Function} callback - the callback called when the channel event occurs
				 */
				object.once = function (channel, callback) {
					arguments[0] = [ID, channel].join('.');
					eventBus.once.apply(eventBus, arguments);
				};

				/**
				 * Remove subscription to a specific channel
				 * callback should be identical as when registered with on or once
				 * @param {String} channel - the channel to which to subscribe
				 * @param {Function} callback - the callback called when the channel event occurs
				 */
				object.off = function (channel, callback) {
					arguments[0] = [ID, channel].join('.');
					eventBus.off.apply(eventBus, arguments);
				};

				/**
				 * Remove all observers that where registered on this observable.
				 * NOTE! if the same ID is used amongst other observables (which is a bad idea), these listeners are also removed...)
				 */
				object._destroyObservable = function () {
					eventBus.removeAllListeners([ID, '**'].join('.'));
				};

				/**
				 * Get the ID of the observable
				 * @returns {String}
				 */
				object.getObservableID = function () {
					return ID;
				};
			}
		};

	});

define('utils/Observer',["underscore"], function (_) {
	"use strict";

	function removeSubscriptions(observer, predicate) {
		// filter out all subscriptions to observable
		var tmp = _.partition(observer._observer_subscriptions, predicate);

		// remove subscriptions
		_.each(tmp[0], observer._stopObserveSubscription);

		// object observer subscription list
		observer._subscriptions = tmp[1];
	}


	return {
		/**
		 * Mixin Observer functionality to an object
		 * @param {Object} object - the object in which the Obser functionality should be installed
		 */
		mixin: function (object) {
			object._observer_subscriptions = [];

			/**
			 * Subscribe to a certain channel of a observable
			 * !Note! this automatically binds the callback to the observer (if not needed use _observeUnbound)
			 * @param {Observable} observable - the object to subscribe to
			 * @param {String} channel - the channel to subscribe to
			 * @param {Function} callback - the callback executed when message arrives in the channel. This callback is automatically bound to the observer!
			 * @returns {SubscriptionObject} this object can be used as an argument for _stopObserSubscription
			 */
			object._observe = function (observable, channel, callback) {
				return this._observeUnbound(observable, channel, _.bind(callback, object));
			};
			
			
			/**
			 * Subscribe to a certain channel of a observable
			 * @param {Observable} observable - the object to subscribe to
			 * @param {String} channel - the channel to subscribe to
			 * @param {Function} callback - the callback executed when message arrives in the channel.
			 * @returns {SubscriptionObject} this object can be used as an argument for _stopObserSubscription
			 */
			object._observeUnbound = function(observable, channel, callback){
				var subscription = {
					observable: observable,
					channel: channel,
					callback: callback
				};

				this._observer_subscriptions.push(subscription);

				observable.on(channel, callback);

				return subscription;
			};
			
			/**
			 * Stop a certain specific subscription 
			 * @param {SubscriptionObject} subscription - a subscription object as returned by _observe and _observeUnbound
			 */
			object._stopObserveSubscription = function (subscription) {
				subscription.observable.off(subscription.channel, subscription.callback);
			};

			/**
			 * Stop all subscriptions registered by the observer
			 */
			object._stopObserveAll = function () {
				_.each(this._observer_subscriptions, this._stopObserveSubscription);	   
			};
			
			/**
			 * Stop observing a specific object
			 * This will remove all subscriptions registered by the observer on the observable object
			 * @param {Observable} observable - to object for which all subscription should be removed
			 */
			object._stopObserveObject = function (observable) {
				removeSubscriptions(this, function (subscription) {
					return subscription.observable === observable;
				});
			};

			/**
			 * Stop observing a specific channel on a specific observable
			 * @param {Observable} observable - the object for which all subscription on the channel should be removed
			 * @param {String} channel - the channel for which subscriptions should be removed
			 */
			object._stopObserveObjectChannel = function (observable, channel) {
				removeSubscriptions(this, function (subscription) {
					return subscription.observable === observable && subscription.channel == channel;
				});
			};			
			
			/**
			 * Remove all subscriptions registerd by the observer on a certain channel
			 * It doesn't matter on which observable the subscription was made
			 * @param {String} channel - the channel for which all subscriptions should be removed
			 */
			object._stopObserveChannel = function(channel){
				removeSubscriptions(this, function(subscription){
					return subscription.channel == channel;
				});
			};
		}
	};
});

define('utils/Popup',["jquery", "jqueryUI"], function ($) {
	
	"use strict";

	var Popup = function (content, options) {
		
		// Content should always be wrapped by jQuery, do this if not ...
		if (!(content instanceof jQuery)) {
			content = $(content);
        }
		
		// Make a reference to this object.
		var self = this;
		
		// The array that will hold all the functions that need to be called on close.
		this._onCloseFunctions = [];
		
		// Create the content
		this._content = content.wrap($("<div></div>")).parent();
		
		// Define the default options.
		var defaultOptions = {
				modal: true,
				resizable: false,
				width:'auto',
	            autoOpen: false,
	            close: function( event, ui ) {
	            	// Execute the functions who are listening for on close value.
	                for (var i = 0; i < self._onCloseFunctions.length; i++) {
	                    self._onCloseFunctions[i]();
	                }
	            }
	        };
		
		// Create popup with right options
		if (options !== undefined) {
			
			// Merge the default options with the passed options. 
			$.extend(true, defaultOptions, options);
			
			// Overwrite the close option, to make sure that the onCloseFunctions are always called.
			defaultOptions.close = function( event, ui ) {
            	// Execute the functions who are listening for on close value.
                for (var i = 0; i < self._onCloseFunctions.length; i++) {
                    self._onCloseFunctions[i]();
                }
                // Call also the close function of the passed options (if defined).
                if (options.close !== undefined) {
                	options.close();
                }
            }
			
			// Make the popup.
			this._content.dialog(defaultOptions);
		} else {
			
			// Make the popup.
			this._content.dialog(defaultOptions);
		}
		
	}
	
	Popup.prototype.setTitle = function (newTitle) {
		this._content.dialog({
			title: newTitle
		});
	}
	
	Popup.prototype.close = function () {
		this._content.dialog( "close" );
	}
	
	Popup.prototype.open = function () {
		this._content.dialog( "open" );
	}
	
	Popup.prototype.onClose = function (onCloseFunction) {
		this._onCloseFunctions.push(onCloseFunction);
	}
	
	Popup.prototype.getJQElement = function () {
		return this._content;
	}
	
	return Popup;

});

define('utils/PriorityOrderedObject',[
		"underscore"
	],
	function (
		_
	) {
		"use strict";

		/**
		 * PriorityOrderedObjects keeps track of objects with a given priority. Multiple objects
		 * can have a the same priority and it's still possible to search for an object by key.
		 *
		 * @exports PriorityOrderedObject
		 * @class
		 */
		var PriorityOrderedObject = function () {
			// Priority -> Array with keys
			this._priority = [];
			// Key -> Item
			this._object = {};
		};

		/**
		 * Add the object with key with a given priority.
		 * @param {String} key   - Key of the item
		 * @param {Object} item  - Any item
		 * @param {Number} priority - the priority of the item
		 */
		PriorityOrderedObject.prototype.addItemWithPriority = function (key, item, priority) {
			if (this._object[key]) {
				throw new Error("Duplicate key exception");
			}
			if (this._priority[priority]) {
				this._priority[priority].push(key);
			} else {
				this._priority[priority] = [key];
			}
			this._object[key] = item;
		};

		/**
		 * Get the item by key
		 * @param {String} key - Key of the item
		 * @returns {Object} The item with the specified key
		 */
		PriorityOrderedObject.prototype.getItemByKey = function (key) {
			return this._object[key];
		};

		/**
		 * Check if  a certain key is present in the ordered object
		 * @param {String}  key - key of the item
		 * @returns {Boolean}
		 */
		PriorityOrderedObject.prototype.containsKey = function (key) {
			return this._object[key] !== undefined;
		};

		/**
		 * Remove the item with the given key
		 * @param {String} key - Key of the item
		 * @returns {Any} the deleted object, undefined if nothing was removed
		 */
		PriorityOrderedObject.prototype.removeByKey = function (key) {
			var deleted;
			if (this._object[key] !== undefined) {
				var array = _.find(this._priority, function (keyArray) {
					if (keyArray) {
						return keyArray.indexOf(key) >= 0;
					}
				});

				if (array) {
					array.splice(array.indexOf(key), 1);
					deleted = this._object[key];
					delete this._object[key];
				}
			}

			return deleted;
		};

		/**
		 * Return an array of the objects as they are ordered
		 * @returns {Array}
		 */
		PriorityOrderedObject.prototype.getOrderedObjectList = function () {
			var orderedObjectList = [];
			_.each(this._priority, function (keyArray) {
				_.each(keyArray, function (key) {
					orderedObjectList.push(this._object[key]);
				}, this);
			}, this);
			return orderedObjectList;
		};

		/**
		 * Get the items as object
		 * @returns {Object} The items as object
		 */
		PriorityOrderedObject.prototype.getItemsAsObject = function () {
			return _.clone(this._object);
		};

		/**
		 * Returns the number of items in the priority ordered object.
		 * @returns {Number}
		 */
		PriorityOrderedObject.prototype.getLength = function () {
			var length = 0;
			_.each(this._priority, function (keyArray) {
				if (keyArray) {
					length += keyArray.length;
				}
			}, this);
			return length;
		};

		/**
		 * Is the priority ordered object empty.
		 * @returns {Boolean}
		 */
		PriorityOrderedObject.prototype.isEmpty = function () {
			return _.isEmpty(this._object);
		};

		return PriorityOrderedObject;

	});

define('utils/Transformation',["underscore",
		"utils/ShapeUtils"
	],
	function (_,
		ShapeUtils) {
		"use strict";

		/**
		 * Create a new tranformation object
		 * See http://www.willamette.edu/~gorr/classes/GeneralGraphics/Transforms/transforms2d.htm#Translations
		 * and http://docs.geotools.org/latest/userguide/tutorial/affinetransform.html
		 *
		 * [a c e]
		 * [b d f]
		 * [0 0 1]
		 *
		 * @param {Number} a
		 * @param {Number} b
		 * @param {Number} c
		 * @param {Number} d
		 * @param {Number} e
		 * @param {Number} f
		 */
		var Transformation = function (a, b, c, d, e, f) {

			this.a = (a !== undefined ? a : 1);
			this.b = (b !== undefined ? b : 0);
			this.c = (c !== undefined ? c : 0);
			this.d = (d !== undefined ? d : 1);
			this.e = (e !== undefined ? e : 0);
			this.f = (f !== undefined ? f : 0);
		};

		/**
		 * Sets this transform to the matrix specified by the 6 numbers.
		 *
		 * [a c e]
		 * [b d f]
		 * [0 0 1]
		 *
		 * @param {Number} a
		 * @param {Number} b
		 * @param {Number} c
		 * @param {Number} d
		 * @param {Number} e
		 * @param {Number} f
		 */
		Transformation.prototype.setTransform = function (a, b, c, d, e, f) {
			this.a = (a ? a : 1);
			this.b = (b ? b : 0);
			this.c = (c ? c : 0);
			this.d = (d ? d : 1);
			this.e = (e ? e : 0);
			this.f = (f ? f : 0);
		};

		/**
		 * Create a copy of this transformation
		 * @returns {Transformation} Copy of this transformation with same a,b,c,d,e,f
		 */
		Transformation.prototype.copy = function () {
			return new Transformation(this.a, this.b, this.c, this.d, this.e, this.f);
		};

		/**
		 * Check if this transformation equals another transformation
		 * @param {Transformation} otherTransformation - Other transformation
		 * @returns {Boolean} True if a,b,c,d,e,f are equal, false otherwise
		 */
		Transformation.prototype.equals = function (otherTransformation) {
			return this.a == otherTransformation.a &&
				this.b == otherTransformation.b &&
				this.c == otherTransformation.c &&
				this.d == otherTransformation.d &&
				this.e == otherTransformation.e &&
				this.f == otherTransformation.f;
		};

		/**
		 * Create the inverse transformation of this transformation.
		 * @returns {Transformation} The inverse transformation of this transformation.
		 */
		Transformation.prototype.createInverse = function () {
			var inverse = this.copy();

			var d = 1 / (this.a * this.d - this.b * this.c);
			var m0 = this.d * d;
			var m1 = -this.b * d;
			var m2 = -this.c * d;
			var m3 = this.a * d;
			var m4 = d * (this.c * this.f - this.d * this.e);
			var m5 = d * (this.b * this.e - this.a * this.f);
			inverse.a = m0;
			inverse.b = m1;
			inverse.c = m2;
			inverse.d = m3;
			inverse.e = m4;
			inverse.f = m5;

			return inverse;
		};

		/**
		 * Sets this transform's scaling part. The matrix representing this transform becomes:
		 *          [   sx   c    e   ]
		 *          [   b    sy   f   ]
		 *          [   0    0    1   ]
		 * @param {Number} sx
		 * @param {Number} sy
		 */
		Transformation.prototype.setScale = function (sx, sy) {
			this.a = sx;
			this.d = sy;
		};

		/**
		 * Scale this transformation.
		 * The matrix representing this transform becomes:
		 *          [   a*sx   c*sy   e   ]
		 *          [   b*sx   d*sy   f   ]
		 *          [   0      0      1   ]
		 * @param {Number} sx
		 * @param {Number} sy
		 */
		Transformation.prototype.scale = function (sx, sy) {
			this.a *= sx;
			this.b *= sx;
			this.c *= sy;
			this.d *= sy;
		};

		/**
		 * Prescale the transformation
		 * The matrix representing this transform becomes:
		 *          [   a*sx   c*sy   e*sx   ]
		 *          [   b*sx   d*sy   f*sy   ]
		 *          [   0      0      1      ]
		 * @param {Number} sx
		 * @param {Number} sy
		 */
		Transformation.prototype.preScale = function (sx, sy) {
			this.a *= sx;
			this.b *= sx;
			this.e *= sx;
			this.c *= sy;
			this.d *= sy;
			this.f *= sy;
		};

		/**
		 * Sets this transform's translation part.
		 * The matrix representing this transform becomes:
		 *           [   a    c    dx  ]
		 *           [   b    d    dy  ]
		 *           [   0    0    1   ]
		 * @param {Number} dx
		 * @param {Number} dy
		 */
		Transformation.prototype.setTranslate = function (dx, dy) {
			this.e = dx;
			this.f = dy;
		};


		/**
		 * Translate this transformation.
		 * The matrix representing this transform becomes:
		 *           [   a    c    e += dx*a+dy*c  ]
		 *           [   b    d    f += dx*b+dy*d  ]
		 *           [   0    0    1   ]
		 * @param {Number} dx
		 * @param {Number} dy
		 */
		Transformation.prototype.translate = function (dx, dy) {
			this.e += dx * this.a + dy * this.c;
			this.f += dx * this.b + dy * this.d;
		};

		/**
		 * Pretranslate the transformation
		 * The matrix representing this transform becomes:
		 *          [   a   c    e+=dx   ]
		 *          [   b   d    d+=dy   ]
		 *          [   0   0    1       ]
		 * @param {Number} dx
		 * @param {Number} dy
		 */
		Transformation.prototype.preTranslate = function (dx, dy) {
			this.e += dx;
			this.f += dy;
		};

		/**
		 * Get a rotation transformation instance:
		 * [ cos(theta) -sin(theta) x-x*cos+y*sin ]
		 * [ sin(theta)  cos(theta) y-x*sin-y*cos ]
		 * [     0           0            1       ]
		 * @param {Number} theta - The rotation angle (positive in radians)
		 * @param {Number} [x = 0] - The x coordinate of the pivot point
		 * @param {Number} [y = 0] - The y coordinate of the pivot point
		 * @returns {Transformation}
		 */
		Transformation.prototype.rotationTransformation = function (theta, x, y) {
			x = x === undefined ? 0 : x;
			y = y === undefined ? 0 : y;
			var cos = Math.cos(theta);
			var sin = Math.sin(theta);
			return new Transformation(cos, sin, -sin, cos, x - x * cos + y * sin, y - x * sin - y * cos);

		};

		/**
		 * Get a translation transformation instance:
		 * [ 1 0 dx ]
		 * [ 0 1 dy ]
		 * [ 0 0 1  ]
		 * @param {Number} dx - the x translation distance
		 * @param {Number} dy - the y translation distance
		 * @returns {Transformation}
		 */
		Transformation.prototype.translateTransformation = function (dx, dy) {
			return new Transformation(1, 0, 0, 1, dx, dy);
		};


		/**
		 * Rotate this transformation
		 * @param {Number} theta - the rotation angle (in radians)
		 * @param {Number} [x = 0] - The x coordinate of the pivot point
		 * @param {Number} [y = 0] - The y coordinate of the pivot point
		 */
		Transformation.prototype.rotate = function (theta, x, y) {
			var rotationMatrix = this.rotationTransformation(theta, x, y);
			this.concatenate(rotationMatrix);
		};

		/**
		 * Pre-rotate this transformation
		 * @param {Number} theta - the rotation angle (in radians)
		 * @param {Number} [x = 0] - The x coordinate of the pivot point
		 * @param {Number} [y = 0] - The y coordinate of the pivot point
		 * @returns
		 */
		Transformation.prototype.preRotate = function (theta, x, y) {
			return this.preConcatenate(this.rotationTransformation(theta, x, y));
		};

		/**
		 * Get a flip transformation instance:
		 * [-1  0  x ]
		 * [ 0  1  y ]
		 * [ 0  0  1 ]
		 * @param {Number} x
		 * @param {Number} y
		 * @returns {Transformation}
		 */
		Transformation.prototype.flipTransformation = function (x, y) {
			return new Transformation(-1, 0, 0, 1, x, y);
		};

		/**
		 * Flip this transformation
		 * @param {Number} x
		 * @param {Number} y
		 */
		Transformation.prototype.flip = function (x, y) {
			this.concatenate(this.flipTransformation(x, y));
		};

		/**
		 * preFlip this transformation
		 * @param {Number} x
		 * @param {Number} y
		 */
		Transformation.prototype.preFlip = function (x, y) {
			this.preConcatenate(this.flipTransformation(x, y));
		};

		/**
		 * Create a string representation of this matrix
		 * @returns {String}
		 */
		Transformation.prototype.toString = function () {
			return "Transformation[[" + this.a + ", " + this.c + ", " + this.e + "], [" + this.b + ", " + this.d + ", " + this.f + "]]";
		};

		/**
		 * Concatenate the transformation tx to this transformation:
		 * In matrix form:
		 * [ this ] = [ this ] x [ tx ]
		 * @param {Transformation} tx - transformation to preconcatenate
		 */
		Transformation.prototype.concatenate = function (tx) {
			var m11 = this.a * tx.a + this.c * tx.b;
			var m12 = this.b * tx.a + this.d * tx.b;

			var m21 = this.a * tx.c + this.c * tx.d;
			var m22 = this.b * tx.c + this.d * tx.d;

			var dx = this.a * tx.e + this.c * tx.f + this.e;
			var dy = this.b * tx.e + this.d * tx.f + this.f;

			this.a = m11;
			this.b = m12;
			this.c = m21;
			this.d = m22;
			this.e = dx;
			this.f = dy;

		};

		/**
		 * Preconcatenate tx to this transformation:
		 * In matrix form:
		 * [ this ] = [ tx ] x [ this ]
		 * @param {Transformation} tx - transformation to preconcatenate
		 * @returns {Transformation} this
		 */
		Transformation.prototype.preConcatenate = function (tx) {
			var n00 = tx.a * this.a + tx.c * this.b;
			var n01 = tx.a * this.c + tx.c * this.d;
			var n02 = tx.a * this.e + tx.c * this.f + tx.e;
			var n10 = tx.b * this.a + tx.d * this.b;
			var n11 = tx.b * this.c + tx.d * this.d;
			var n12 = tx.b * this.e + tx.d * this.f + tx.f;

			this.a = n00;
			this.c = n01;
			this.e = n02;
			this.b = n10;
			this.d = n11;
			this.f = n12;

			return this;
		};

		/**
		 * Transform a point according to this transformation
		 * @param {ShapeUtils.Point} point - point to transform
		 * @returns {ShapeUtils.Point} transformed point
		 */
		Transformation.prototype.transformedPoint = function (point) {
			var x = point.x;
			var y = point.y;
			var px = x * this.a + y * this.c + this.e;
			var py = x * this.b + y * this.d + this.f;
			return new ShapeUtils.Point(px, py);
		};

		/**
		 * Pretransform a point according to this transformation
		 * @param {ShapeUtils.Point} point
		 * @returns {ShapeUtils.Point}
		 */
		Transformation.prototype.preTransformedPoint = function (point) {
			return new ShapeUtils.Point(this.a * point.x + this.b * point.y + this.e, this.c * point.x + this.d * point.y + this.f);
		};

		/**
		 * Transform a size according to this transformation
		 * @param {ShapeUtils.Size} size - size to transform
		 * @returns {ShapeUtils.Size} transformed size
		 */
		Transformation.prototype.transformedSize = function (size) {
			var p1 = this.transformedPoint(new ShapeUtils.Point());
			var p2 = this.transformedPoint(new ShapeUtils.Point(size.width, size.height));
			var s = new ShapeUtils.Size(p2.x - p1.x, p2.y - p1.y);

			return s;
		};

		/**
		 * Transform a rectangle according to thsi transformation
		 * @param {ShapeUtils.Rect} rect - rect to transform
		 * @returns {ShapeUtils.Rect} transformed rect
		 */
		Transformation.prototype.transformedRect = function (rect) {
			var worldRect = rect.copy();

			var topleft = this.transformedPoint(worldRect.origin);
			var p1x = topleft.x;
			var p1y = topleft.y;
			var p2x = topleft.x;
			var p2y = topleft.y;

			var topright = this.transformedPoint(new ShapeUtils.Point(worldRect.right(), worldRect.top()));
			p1x = Math.min(p1x, topright.x);
			p1y = Math.min(p1y, topright.y);
			p2x = Math.max(p2x, topright.x);
			p2y = Math.max(p2y, topright.y);

			var bottomleft = this.transformedPoint(new ShapeUtils.Point(worldRect.left(), worldRect.bottom()));
			p1x = Math.min(p1x, bottomleft.x);
			p1y = Math.min(p1y, bottomleft.y);
			p2x = Math.max(p2x, bottomleft.x);
			p2y = Math.max(p2y, bottomleft.y);

			var bottomright = this.transformedPoint(new ShapeUtils.Point(worldRect.right(), worldRect.bottom()));
			p1x = Math.min(p1x, bottomright.x);
			p1y = Math.min(p1y, bottomright.y);
			p2x = Math.max(p2x, bottomright.x);
			p2y = Math.max(p2y, bottomright.y);



			var p1 = new ShapeUtils.Point(p1x, p1y);
			var p2 = new ShapeUtils.Point(p2x, p2y);

			worldRect.origin = p1;
			worldRect.size = new ShapeUtils.Size(p2.x - p1.x, p2.y - p1.y);

			return worldRect;
		};

		/**
		 * Get the scale x
		 * @returns {Number} scale x calculated as Math.sqrt(a^2 + b^2)
		 */
		Transformation.prototype.getScaleX = function () {
			return Math.sqrt(this.a * this.a + this.b * this.b);
		};

		/**
		 * Get the scale y
		 * @returns {Number} scale y calculated as Math.sqrt(c^2 + d^2)
		 */
		Transformation.prototype.getScaleY = function () {
			return Math.sqrt(this.c * this.c + this.d * this.d);
		};

		/**
		 * Get the rotation that is represented by this transformation
		 * @returns {Number} the rotation in degrees
		 */
		Transformation.prototype.getRotation = function () {
			return Math.atan2(this.c, this.d);
		};

		return Transformation;
	});

define('utils/prefixTree',["underscore"], function (_) {

	"use strict";



	/**
	 * A simple implementation of a prefix tree - for quick prefix searching
	 * 
	 * @class
	 * @exports PrefixTree
	 */
	var PrefixTree = function () {
		this._root = new PrefixTreeEntry();
	};

	/**
	 * an entry in the prefix tree
	 */
	var PrefixTreeEntry = function () {
		this.objects = [];
		this.childs = {};
	};

	/**
	 * adds an object to the prefix tree
	 * @param {String}   key    - the key for this object
	 * @param {Object} object 
	 */
	PrefixTree.prototype.add = function (key, object) {
		addEntry(this._root, key.toLowerCase(), object);
	};


	/**
	 * Searches for all objects that have a key with the given prefix
	 * @param   {String}   prefix    
	 * @param   {Function} [filter] - filter the objects with this function, return true if it should be added, false if it should be ignored  
	 * @returns {Array.<Object>} 
	 */
	PrefixTree.prototype.find = function (prefix, filter) {
		var entries = [];
		findEntries(this._root, prefix.toLowerCase(), filter, entries);
		return entries;
	};

	/**
	 * help method to add an entry
	 * @param {PrefixTreeEntry}   prefixTreeEntry 
	 * @param {String}   prefix          
	 * @param {Object} object          
	 */
	var addEntry = function (prefixTreeEntry, prefix, object) {
		if (prefix === "") {
			prefixTreeEntry.objects.push(object);
		} else {
			var char = prefix.charAt(0);
			var remainder = prefix.substring(1);
			if (!prefixTreeEntry.childs[char]) {
				prefixTreeEntry.childs[char] = new PrefixTreeEntry();
			}
			addEntry(prefixTreeEntry.childs[char], remainder, object);
		}
	};

	/**
	 * help method to find entries
	 * @param   {Object}   prefixTreeEntry 
	 * @param   {PrefixTreeEntry}   prefix          
	 * @param   {Function} filter          
	 * @param   {Array.<Object>} entries - the array to which the entries are added          
	 */                                   
	var findEntries = function (prefixTreeEntry, prefix, filter, entries) {
		if (typeof prefixTreeEntry === "undefined") {
			return;
		}
		if (prefix === "") {
			_.each(prefixTreeEntry.objects, function (object) {
				if (!filter || filter(object)) {
					entries.push(object);
				}
			});
			_.each(prefixTreeEntry.childs, function (childTreeEntry) {
				findEntries(childTreeEntry, "", filter, entries);
			});
		} else {
			var char = prefix.charAt(0);
			var remainder = prefix.substring(1);
			findEntries(prefixTreeEntry.childs[char], remainder, filter, entries);
		}

	};




	return PrefixTree;


});

/**
 * @author PASH
 * SMSDrop stands for "Search Multiple Selection Drop-down".
 * SMSDrop is provides one drop-down for one id. <br>
 * Each drop-down is specifically created with the SMSDrop object and all the operations should be performed with the object created for specific drop-down.
 * By using this utility user can select multiple options from the drop-down by checking check-box.<br>
 * User can start typing to minimize the list of option, list will be populated according to the string entered in search input field.<br>
 * Selection for all or remove all the selection feature is also available.
 */
define('utils/SMSDrop',["jquery",
        "underscore",
        "utils/webi18n!SMSDropStrings",
        "wcr/hover"],
function($,
		_,
		webi18n) {
	"use strict";
	
	/**
	 * Constructor of SMSDrop
	 * @param : id {id is required to create element for drop-down}
	 * @param : separaterFunc {separaterFunc is a function to join selected values to get as a result. By default values will be separated with ' | '.}
	 */
	var SMSDrop = function(id, separaterFunc) {
		if (!id) {
			alert(webi18n.t("WRONGIDERRORMSG", id));
			return;
		}
		
		if (separaterFunc && $.type(separaterFunc) === "function") {
			this.separaterFunc = separaterFunc;
		} else {
			// Default function if function is not available in parameter
			this.separaterFunc = function(selectedOptions){
			    var stringValue = "";
			    $.each(selectedOptions, function(index, option) {
			    	stringValue += option.key;
			    	if (index < selectedOptions.length - 1) {
			    		stringValue += ' | ';
			    	}
			    });
			    return stringValue;
			};
		}
		
		this.id = id;
		this.optionValues = [];
		this.values = [];
		this.createEvent = true;
		this.onChangeCallBacks = [];
	};
	
	/**
	 * Getter method to get JQuery element for drop-down
	 * @returns JQuery element
	 */
	SMSDrop.prototype.getJQElement = function() {
		return this.renderSelectMultiple();
	};

	/**
	 * Method is to clear all the drop down options
	 */
	SMSDrop.prototype.clearAll = function() {
		this.optionValues = [];
		this.values = [];
		this.displaySelection();
	};
	
	/**
	 * Event listener to executes function whenever value for drop down is changing.
	 * @param callBack {callBack is a function which will execute on change of drop down}
	 */
	SMSDrop.prototype.onChange = function(callBack) {
		this.onChangeCallBacks.push(callBack);
		
	};
	
	/**
	 * Method to get input to generate options for the drop-down.
	 * @param optionValues {Array with key and value pair objects (like: optionValues.push({key:'value of option', value:'text of option'}); )}
	 * @param selectedValues {Boolean or an Array for the preselected values (like: selectedValues.push({key:'value of option', value:'text of option'})), 
	 * This parameter should keep values of the drop-down option or it should be true for all the preselected options in the drop down }
	 */
	SMSDrop.prototype.addElementOptions = function(optionValues, selectedValues) {
		if (optionValues && optionValues.length > 0) {
			this.optionValues = optionValues;
			this.optionValues.sort(this.sortByValue);
		}
		
		if (selectedValues) {
			if ($.isArray(selectedValues) && selectedValues.length > 0) {
				this.values = $.extend(true, [], selectedValues);
			} else if (typeof selectedValues === 'boolean' && selectedValues)  {
				this.values = $.extend(true, [], this.optionValues);
			}
		}
		this.displaySelection();
	};
	
	/**
	 *  This method is to get value for selected options in the drop-down.
	 * @returns {Array of values like - ({key: 'key', value: 'value'}) }
	 */
	SMSDrop.prototype.val = function() {
		return this.values;
	};
	
	/**
	 * This method is works as a compare method to help sorting based on the values.
	 * @param obj1 {key: 'key1', value: 'value1'}
	 * @param obj2 {key: 'key2', value: 'value2'}
	 * @returns
	 */
	SMSDrop.prototype.sortByValue = function(obj1, obj2) {
		if (val1 && val2 && val1.value & val2.value) {
			var val1 = obj1.value.toLowerCase();
			var val2 = obj2.value.toLowerCase();
			return (val1 == val2) ? 0 : (val1 > val2) ? 1 : -1;
		}
	};
	
	/**
	 * Method is to render field for selected options and a button to show drop-down <br>
	 * This method ensures that duplicate drop down is not populated with same id. <br>
	 * It removes previous drop down and create a fresh down each time.
	 * @returns {table as JQuery object}
	 */
	SMSDrop.prototype.renderSelectMultiple = function() {
		var outerDiv;
		if ($('#outdiv_' + this.id).length) {
			$('#outdiv_' + this.id).remove();
			this.createEvent = false;
		}
		outerDiv = $('<div>');
		outerDiv.attr('id', 'outdiv_' + this.id);
		
		var valueHolder = $('<input>');
		valueHolder.attr({'id': this.id, 'type': 'text'});
		valueHolder.css('display', 'none');
		
		var table = $('<table>');
		table.attr({'valign': 'top', 'padding': '0'});
		
		outerDiv.append(table);
		table.append(valueHolder);
		
		var row = $('<tr>');
		row.attr('padding', '0');
		table.append(row);

		var col1 = $('<td>');
		col1.css('padding', '0px');
		row.append(col1);
		
		var col2 = $('<td>');
		col2.css({'padding': '0 0 0 0', 'vertical-align': 'top'});
		row.append(col2);

		var img = $('<img id="' + this.id + '_popUpIcon" src="' + webcenterConfig.imageBaseUrl + 'ar_tp.gif">');
		col2.append(img);
		this.dropButtonEvent();

		var mvDiv = $('<div>');
		mvDiv.attr('id', 'mvdiv_' + this.id);
		mvDiv.css({'background-color': 'white', 'vertical-align': 'top', 
			'padding': '0 0 0 0', 'border': '1px solid grey', 
			'min-width': '230px', 'min-height': '15px',
			'max-height': '75px', 'overflow-y': 'auto'});
		col1.append(mvDiv);
		
		// executing all the call back function on inputChange event.
		valueHolder.on('inputChange', _.bind(function() {
			$.each(this.onChangeCallBacks, function(index, callBack) {
				callBack();
			});
		}, this));
		
		return outerDiv;
	};
	
	/**
	 * Initiating multiple selection drop-down.
	 * @param event {Existing object to know about event happened on the element.}
	 */
	SMSDrop.prototype.executeWithNotification = function(event) {
		"use strict";

		var dropdownItems = [];  // dropdownmenu expects and array of html code snippets, so we give it an array of 1 element
		dropdownItems.push('<img src="' + webcenterConfig.imageBaseUrl + 'ajax_loader.gif" />');
		var menuWidth = '400px', menuHeight = '300px', scrollHor = false, nohide = true, bgcolor = "#ffffff", nohiddonclick = false;
		
		$("#dropmenudiv").css('z-index', 10000);	// 10000 is an assumption that will be the highest z-index to display drop down.
		$("#dropmenudiv").addClass('showOverPopup'); // Adding class to deal with _allowInteraction for popup.
		
		dropdownmenu($('#mvdiv_' + this.id)[0], event, dropdownItems, menuWidth, menuHeight, scrollHor, nohide, bgcolor, nohiddonclick);
		
		this.displayMultiselect(event);				
	};

	/**
	 * Creates top section of drop-down and initiate method to populate options
	 * @param event
	 */
	SMSDrop.prototype.displayMultiselect = function(event) {
		var tableDBValue = $('<table>');
		tableDBValue.attr('id', 'dbValues' + this.id);
		tableDBValue.width($('#mvdiv_' + this.id).width());
		
		var rowSelectDeselectAll = $('<tr>');
		rowSelectDeselectAll.attr('class','darktext_greenlink_11');
		tableDBValue.append(rowSelectDeselectAll);
		
		var colSelectAll = $('<td>').css('display', 'flex');
		colSelectAll.attr('class', 'nobr');
		colSelectAll.css({'cursor': 'default', 'align': 'center'});
		rowSelectDeselectAll.append(colSelectAll);
		
		var linkSelectAll = $('<a>');
		linkSelectAll.attr({'class': 'linkSelectAll', 'href': '#'});
		linkSelectAll.css('padding-right', '30px');
		linkSelectAll.text(webi18n.t('SELECTALL'));
		colSelectAll.append(linkSelectAll);
		
		var linkDeselectAll = $('<a>');
		linkDeselectAll.attr({'class': 'linkDeselectAll', 'href': '#'});
		linkDeselectAll.text(webi18n.t('DESELECTALL'));
		colSelectAll.append(linkDeselectAll);
		
		var rowFilterInput = $('<tr>');
		rowFilterInput.attr('class','darktext_greenlink_11');
		tableDBValue.append(rowFilterInput);
		
		var colFilterInput = $('<td>');
		rowFilterInput.append(colFilterInput);
		
		var fieldFilterInput = $('<input>');
		fieldFilterInput.prop({'type': 'text', 'autocomplete': 'off', 'class': 'FieldInputFilter'});
		colFilterInput.append(fieldFilterInput);
		
		var rowDropDownOptions = $('<tr>');
		rowDropDownOptions.attr('class','darktext_greenlink_11');
		tableDBValue.append(rowDropDownOptions);
		
		var colDropDownOptions = $('<td>');
		rowDropDownOptions.append(colDropDownOptions);

		var dropmenucontainer = $('<div>');
		dropmenucontainer.attr('id', 'dropmenucontainer');
		dropmenucontainer.css({'width': '100%', 'max-height': '300px', 'display': 'block', 'overflow': 'auto', 'overflow-x': 'hidden'});
		colDropDownOptions.append(dropmenucontainer);
		
		// call a method to generate list of options with check box.
		dropmenucontainer.append(this.multiValueDropdownOptions());
		
		var selectDropmenucontainer = $('<div>');
		selectDropmenucontainer.attr('id', 'selectdropmenucontainer');
		selectDropmenucontainer.css({'width': '100%', 'display': 'block'});
		selectDropmenucontainer.append(tableDBValue);
		
		var dropdownItems = [];
		dropdownItems.push(selectDropmenucontainer.html());
		var menuWidth = '400px', menuHeight = '300px', scrollHor = false, nohide = false, bgcolor = "#ffffff", nohiddonclick = true;
		
		dropmenuobj.style.visibility = 'hidden';
		dropdownmenu($('#mvdiv_' + this.id)[0], event, dropdownItems, menuWidth, menuHeight, scrollHor, nohide, bgcolor, nohiddonclick);
		dropmenuobj.style.visibility = 'visible';
		
		this.addMultiSelectEvents();
	};
	
	/**
	 * Creates options with check box in drop-down
	 * @returns
	 */
	SMSDrop.prototype.multiValueDropdownOptions = function() {
		var tableDropDownOptions = $('<table>');
		
		var optionValues = this.optionValues;
		var values = this.values;
		
		$.each(optionValues, function(index, object) {
			var rowOption = $('<tr>');
			rowOption.attr({'class': 'darktext_greenlink_11', 'name': 'MultiValueSelectTr'});
			tableDropDownOptions.append(rowOption);
			
			var colOption = $('<td>');
			rowOption.append(colOption);
			
			var checkOption = $('<input>');
			checkOption.attr({'type': 'checkbox', 'value': object.key, 'class': 'multioption', 'id': object.key + '_' + this.id});
			checkOption.css('top', '-35px');
			
			var selected = $.grep(values, _.bind(function( n, i ) {
				return (n.key === object.key);
			}, this));
			
			if (selected.length > 0) {
				checkOption.attr("checked", true);
			}
			
			colOption.append(checkOption);
			
			var checkLable = $('<label>');
			checkLable.attr('for', object.key + '_' + this.id);
			checkLable.text(object.value);
			colOption.append(checkLable);
		});
	
		this.displaySelection();
		return tableDropDownOptions;
	};
	
	/**
	 * Adding click event for display or hide drop-down
	 */
	SMSDrop.prototype.dropButtonEvent = function() {
		var self = this;
		var dropButtenEventFunction = function(event) {
			if(!dropmenuobj || dropmenuobj.style.visibility !== 'visible') {
				self.executeWithNotification(event);
			} else {
				dropmenuobj.style.visibility = 'hidden';
			}
		};
		
		// if events are already bound then do not bind again.
		if (self.createEvent) {
			$(document).on("click", "#" + this.id + "_popUpIcon", dropButtenEventFunction);
			$(document).on("click", "#mvdiv_" + this.id, dropButtenEventFunction);
		}
	};
	
	/**
	 * Handles all other events for drop-down
	 */
	SMSDrop.prototype.addMultiSelectEvents = function() {
		this.eventSelectAll();
		this.eventDeSelectAll();
		this.eventCheckBox();
		this.eventFilterOptions();
	};
	
	/**
	 * Event handling for filter input field
	 */
	SMSDrop.prototype.eventFilterOptions = function() {
		$('#dbValues' + this.id).find('.FieldInputFilter').focus();
		$('#dbValues' + this.id).find('.FieldInputFilter').on("input change", _.bind(function() {
			this.showFilteredDropdownOptions($('#dbValues' + this.id).find('.FieldInputFilter')[0].value);
		}, this));
	};
	
	/**
	 * Event to select all the options by click on hyper link for "Select All"
	 */
	SMSDrop.prototype.eventSelectAll = function() {
		$('#dbValues' + this.id).find('.linkSelectAll').on("click", _.bind(function() {
			var list = $('#dbValues' + this.id).find('.multioption');
			$.each(list, _.bind(function(index, item) {
				var display = $(item).parent().css('display');
				// Ignore selection if not displayed in the drop down options
				if (display !== 'none') {
					item.checked = true;
					var value = item.value;
					var text = $("label[for='" + item.id + "']").text();
					this.updateValue(true, value, text);
				}
			}, this));
			this.displaySelection();
			
			// Trigger the event for inputChange.
			$('#' + this.id).trigger("inputChange", $('#' + this.id).val());
		}, this));
	};
	
	/**
	 * Method to deselect all the options by click on hyper link for "Deselect All"
	 */
	SMSDrop.prototype.eventDeSelectAll = function() {
		$('#dbValues' + this.id).find('.linkDeselectAll').on("click", _.bind(function() {
			var list = $('#dbValues' + this.id).find('.multioption');
			$.each(list, function(index, item) {
				item.checked = false;	// uncheck all the check boxes
			});
			this.values = [];	// Empty selected values
			this.displaySelection();
			
			updateHoverPosition($('#mvdiv_' + this.id)[0], 0, 0);	// updated hover position related to the div.
			$('#' + this.id).trigger("inputChange", '');
		}, this));
	};
	
	/**
	 * Displays selected values
	 */
	SMSDrop.prototype.displaySelection = function() {
		$('#mvdiv_' + this.id).empty();
		$('#' + this.id).val("");
		
		$('#' + this.id).attr("value", '');
		
		var innerTable = $('<ul>');
		innerTable.attr('class', 'darktext_darklink_11');
		innerTable.css({'padding': '0px', 'padding-top': '0px', 'padding-bottom': '0px', 'margin': '0px', 'list-style-type': 'none'});
		$('#mvdiv_' + this.id).append(innerTable);
		
		this.values.sort(this.sortByValue);
		
		var selectedValues = this.values;
		$.each(selectedValues, function(index, item) {
			var innerRow1 = $('<li>');
			innerRow1.append(item.value);
			innerTable.append(innerRow1);
		});
		var strValues = this.separaterFunc(this.values);
		if (strValues) {
			$('#' + this.id).attr('value', strValues);
			$('#' + this.id).val(strValues);
		}
	};
	
	/**
	 * Update result array for the selection or de selection
	 * @param checked {boolean value to know selection or de selection}
	 * @param value {value of the option}
	 * @param text {text fo the option}
	 */
	SMSDrop.prototype.updateValue = function(checked, key, value) {
		var index = this.getObjIndexByKey(this.values, key);
		if (checked) {
			if (index === -1) {
				this.values.push({key: key, value: value});
			}
		} else {
			if (index != -1) {
				this.values.splice(index, 1);
			}
		}
	};
	
	/**
	 * Method to get object index from the array based on the key
	 * @param array in structure of [{key: 'key', value: 'value'}]
	 * @param key as string
	 * @returns {Number} returns -1 if object is not found.
	 */
	SMSDrop.prototype.getObjIndexByKey = function(array, key) {
		var i = -1;
		$.each(array, function(index, value){
		   if(value.key === key){ i = index; return false; }
		});
		return i;
	};
	
	/**
	 * Handling change event for check box of the drop-down
	 */
	SMSDrop.prototype.eventCheckBox = function() {
		var self = this;
		$('#dbValues' + this.id).find('.multioption').on('change', function() {
			var value = this.value;
			var text = $("label[for='" + this.id + "']").text();
			self.updateValue(this.checked, value, text);
			self.displaySelection();
			
			$('#' + self.id).trigger("inputChange", $('#' + self.id).val());
		});
	};
	
	/**
	 * Method is used to display only filtered options based of requested term
	 * @param reqTerm {String provided in filter input field.}
	 */
	SMSDrop.prototype.showFilteredDropdownOptions = function(reqTerm) {
		var matchedValueHash = {};
		
		for (var i = 0; i < this.optionValues.length; i++) {
        	if (this.optionValues[i].value.toLowerCase().indexOf(reqTerm.toLowerCase()) !== -1) {
        		matchedValueHash[this.optionValues[i].key] = this.optionValues[i].value;
        	}
        }

		var list = $('#dbValues' + this.id).find('.multioption');
		$.each(list, _.bind(function(index, item) {
			var value = matchedValueHash[item.value];
			
			if (value || item.checked) {
				if (value) {
					var startHighLight = value.toLowerCase().indexOf(reqTerm.toLowerCase());
					var firstPart = value.substr(0, startHighLight);
					var highLightPart = value.substr(startHighLight, reqTerm.length);
					var lastPart = value.substr(startHighLight + reqTerm.length, value.length);
					
					var optionTextPart1 = $('<span>').text(firstPart);
					var optionTextPart2 = $('<b>').text(highLightPart);
					var optionTextPart3 = $('<span>').text(lastPart);
					
					$("label[for='" + item.id + "']").empty();
					$("label[for='" + item.id + "']").append(optionTextPart1).append(optionTextPart2).append(optionTextPart3);
				}
				
				$(item).parent().css('display', 'block');
			} else {
				$(item).parent().css('display', 'none');
			}
		}, this));
	};
	
	return SMSDrop;
});










	

define('utils/StringSet',["utils/Map"],
	function (Map) {
		/**
		 * @class StringSet is an implementation of the set datastructure for storing strings. It can contain unique strings
		 * @constructor
		 * @augments Utils.Map
		 */
		var StringSet = function () {
			Map.call(this);
		};

		StringSet.prototype = Object.create(Map.prototype);
		StringSet.prototype.constructor = StringSet;


		/**
		 * @override
		 * @param {String} key - add a key
		 */
		StringSet.prototype.add = function (key) {
			Map.prototype.add.call(this, key, true);
		};

		/**
		 * @override
		 * @param {String} key - the name of the entry
		 * @returns {String} the key if the key exists (or undefined)
		 */
		StringSet.prototype.get = function (key) {
			if (this.has(key)) {
				return key;
			} else {
				return undefined;
			}
		};

		/**
		 * returns the list of entry names
		 * @override
		 * @returns {Array.<String>}
		 */
		StringSet.prototype.values = function () {
			return this.entries();
		};

		return StringSet;
	});

define('utils/UndoRedoController',["utils/Observable", "underscore"], function (Observable, _) {
	"use strict";


	/**
	 * Implementation of a simple Undo Redo controller
	 * Using the Command pattern it is possible to execute commands that can be undone or redone
	 *
	 * @exports UndoRedoController
	 * @class
	 *
	 * @mixes Observable
	 * @param {Number} [stackSize=1000] stackSize - maximum size of the undo stack. If more actions are added, the oldes undo actions are removed from the stack
	 */
	var UndoRedoController = function (stackSize) {
		Observable.mixin(this, "undoRedoManager");

		if (!stackSize) {
			stackSize = 1000;
		}

		this._stackSize = stackSize;
		this._stackSizeLimitReached = false;
		this._undoStack = [];
		this._redoStack = [];
	};


	/**
	 * Execute a Command, this clears the redo stack
	 * @param {Command} command - the "do()" method of the command is executed, and the command is added to the undo stack
	 * @param {Boolean} [skipDo=false] - if true, the command.executeDo() is not called automatically
	 * @param {Boolean} [doNotClearRedoStack = false] - if true, the redoStack will not be emptied !! USE WITH CAUTION !! 
	 */
	UndoRedoController.prototype.execute = function (command, skipDo, doNotClearRedoStack) {
		if(!doNotClearRedoStack) {
			this._redoStack = [];
		}
		this._addToUndoStack(command);

		if (!skipDo) {
			command.executeDo();
		}


		this._emit("execute", command);
		this._emit("changed");
	};

	/**
	 * Undo the last executed command, the command is added to the redo stack
	 */
	UndoRedoController.prototype.undo = function () {
		var command = this._undoStack.pop();

		if (command) {
			this._redoStack.push(command);
			command.executeUndo();
			this._emit("undo", command);
			this._emit("changed");
		}
	};

	/**
	 * Redo the command that was last undone. The command is added to the undo stack
	 */
	UndoRedoController.prototype.redo = function () {
		var command = this._redoStack.pop();
		if (command) {
			this._addToUndoStack(command);
			command.executeDo();
			this._emit("redo", command);
			this._emit("changed");
		}
	};

	/**
	 * Add a command to undo stack
	 * @param {Command} command to add to the undo stack
	 */
	UndoRedoController.prototype._addToUndoStack = function (command) {
		if (this._undoStack.length >= this._stackSize) {
			this._undoStack = _.last(this._undoStack, this._stackSize - 1);
			this._stackSizeLimitReached = true;
		}

		this._undoStack.push(command);
	};

	/**
	 * Look at the next undoable command
	 * @param {Number} [peekLevel=0] - peek level (to look deeper down the stack)
	 * @returns {Command}
	 */
	UndoRedoController.prototype.peekUndo = function (peekLevel) {
		if (typeof peekLevel === "undefined") {
			peekLevel = 0;
		}
		return this._undoStack[this._undoStack.length - 1 - peekLevel];
	};

	/**
	 * Look at the next redoable command
	 * @param {Number} [peekLevel=0] - peek level (to look deeper down the stack)
	 * @returns {Command}
	 */
	UndoRedoController.prototype.peekRedo = function (peekLevel) {
		if (typeof peekLevel === "undefined") {
			peekLevel = 0;
		}
		return this._redoStack[this._redoStack.length - 1 - peekLevel];
	};


	/**
	 * Get the number of undo actions on the stack
	 * @returns {Number}
	 */
	UndoRedoController.prototype.undoCount = function () {
		return this._undoStack.length;
	};

	/**
	 * Get the number of redo actions on the stack
	 * @returns {Number}
	 */
	UndoRedoController.prototype.redoCount = function () {
		return this._redoStack.length;
	};

	/**
	 * Get the Undo Stack
	 * !!IMMUTABLE!!! don't change the return Array
	 * @returns {Array.<Command>}
	 */
	UndoRedoController.prototype.getUndoStack = function () {
		return this._undoStack;
	};
	/**
	 * Get the Redo Stack
	 * !!IMMUTABLE!!! don't change the return Array
	 * @returns {Array.<Command>}
	 */
	UndoRedoController.prototype.getRedoStack = function () {
		return this._redoStack;
	};

	/**
	 * If the maximum stack size was reached (and undo commands where thrown away), this returns true
	 * @returns {Boolean}
	 */
	UndoRedoController.prototype.didReachStackSizeLimit = function () {
		return this._stackSizeLimitReached;
	};
	return UndoRedoController;
});

define('utils/wcrerror',[], function () {
"use strict";
    
    var WCRError = function (langTag, message) {
        Error.call(this);
        this.errormessage = message;
        this.langTag = langTag;
        
        // when printing (firefox does not seem to support overriding toString for Error)
        this.message = (this.errormessage?this.errormessage+" ":"")+(this.langTag?"(langTag: "+this.langTag+")":"");
    };
    
    WCRError.prototype = Object.create(Error.prototype);
	WCRError.prototype.constructor = WCRError;
    WCRError.prototype.name = "WCRError";
    
    WCRError.prototype.getTag = function() {
    	return this.langTag;
    };
    
    return WCRError;
});

/* ***** BEGIN LICENSE BLOCK *****
 * Distributed under the BSD license:
 *
 * Copyright (c) 2010, Ajax.org B.V.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Ajax.org B.V. nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL AJAX.ORG B.V. BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ***** END LICENSE BLOCK ***** */

define('lib/dom',['require','exports','module'],function(require, exports, module) {
"use strict";

var XHTML_NS = "http://www.w3.org/1999/xhtml";

exports.getDocumentHead = function(doc) {
    if (!doc)
        doc = document;
    return doc.head || doc.getElementsByTagName("head")[0] || doc.documentElement;
}

exports.createElement = function(tag, ns) {
    return document.createElementNS ?
           document.createElementNS(ns || XHTML_NS, tag) :
           document.createElement(tag);
};

exports.hasCssClass = function(el, name) {
    var classes = (el.className || "").split(/\s+/g);
    return classes.indexOf(name) !== -1;
};

/*
* Add a CSS class to the list of classes on the given node
*/
exports.addCssClass = function(el, name) {
    if (!exports.hasCssClass(el, name)) {
        el.className += " " + name;
    }
};

/*
* Remove a CSS class from the list of classes on the given node
*/
exports.removeCssClass = function(el, name) {
    var classes = el.className.split(/\s+/g);
    while (true) {
        var index = classes.indexOf(name);
        if (index == -1) {
            break;
        }
        classes.splice(index, 1);
    }
    el.className = classes.join(" ");
};

exports.toggleCssClass = function(el, name) {
    var classes = el.className.split(/\s+/g), add = true;
    while (true) {
        var index = classes.indexOf(name);
        if (index == -1) {
            break;
        }
        add = false;
        classes.splice(index, 1);
    }
    if(add)
        classes.push(name);

    el.className = classes.join(" ");
    return add;
};

/*
 * Add or remove a CSS class from the list of classes on the given node
 * depending on the value of <tt>include</tt>
 */
exports.setCssClass = function(node, className, include) {
    if (include) {
        exports.addCssClass(node, className);
    } else {
        exports.removeCssClass(node, className);
    }
};

exports.hasCssString = function(id, doc) {
    var index = 0, sheets;
    doc = doc || document;

    if (doc.createStyleSheet && (sheets = doc.styleSheets)) {
        while (index < sheets.length)
            if (sheets[index++].owningElement.id === id) return true;
    } else if ((sheets = doc.getElementsByTagName("style"))) {
        while (index < sheets.length)
            if (sheets[index++].id === id) return true;
    }

    return false;
};

exports.importCssString = function importCssString(cssText, id, doc) {
    doc = doc || document;
    // If style is already imported return immediately.
    if (id && exports.hasCssString(id, doc))
        return null;
    
    var style;
    
    if (doc.createStyleSheet) {
        style = doc.createStyleSheet();
        style.cssText = cssText;
        if (id)
            style.owningElement.id = id;
    } else {
        style = doc.createElementNS
            ? doc.createElementNS(XHTML_NS, "style")
            : doc.createElement("style");

        style.appendChild(doc.createTextNode(cssText));
        if (id)
            style.id = id;

        exports.getDocumentHead(doc).appendChild(style);
    }
};

exports.importCssStylsheet = function(uri, doc) {
    if (doc.createStyleSheet) {
        doc.createStyleSheet(uri);
    } else {
        var link = exports.createElement('link');
        link.rel = 'stylesheet';
        link.href = uri;

        exports.getDocumentHead(doc).appendChild(link);
    }
};

exports.getInnerWidth = function(element) {
    return (
        parseInt(exports.computedStyle(element, "paddingLeft"), 10) +
        parseInt(exports.computedStyle(element, "paddingRight"), 10) + 
        element.clientWidth
    );
};

exports.getInnerHeight = function(element) {
    return (
        parseInt(exports.computedStyle(element, "paddingTop"), 10) +
        parseInt(exports.computedStyle(element, "paddingBottom"), 10) +
        element.clientHeight
    );
};


if (typeof document == "undefined")
    return;

if (window.pageYOffset !== undefined) {
    exports.getPageScrollTop = function() {
        return window.pageYOffset;
    };

    exports.getPageScrollLeft = function() {
        return window.pageXOffset;
    };
}
else {
    exports.getPageScrollTop = function() {
        return document.body.scrollTop;
    };

    exports.getPageScrollLeft = function() {
        return document.body.scrollLeft;
    };
}

if (window.getComputedStyle)
    exports.computedStyle = function(element, style) {
        if (style)
            return (window.getComputedStyle(element, "") || {})[style] || "";
        return window.getComputedStyle(element, "") || {};
    };
else
    exports.computedStyle = function(element, style) {
        if (style)
            return element.currentStyle[style];
        return element.currentStyle;
    };

exports.scrollbarWidth = function(document) {
    var inner = exports.createElement("ace_inner");
    inner.style.width = "100%";
    inner.style.minWidth = "0px";
    inner.style.height = "200px";
    inner.style.display = "block";

    var outer = exports.createElement("ace_outer");
    var style = outer.style;

    style.position = "absolute";
    style.left = "-10000px";
    style.overflow = "hidden";
    style.width = "200px";
    style.minWidth = "0px";
    style.height = "150px";
    style.display = "block";

    outer.appendChild(inner);

    var body = document.documentElement;
    body.appendChild(outer);

    var noScrollbar = inner.offsetWidth;

    style.overflow = "scroll";
    var withScrollbar = inner.offsetWidth;

    if (noScrollbar == withScrollbar) {
        withScrollbar = outer.clientWidth;
    }

    body.removeChild(outer);

    return noScrollbar-withScrollbar;
};

/*
 * Optimized set innerHTML. This is faster than plain innerHTML if the element
 * already contains a lot of child elements.
 *
 * See http://blog.stevenlevithan.com/archives/faster-than-innerhtml for details
 */
exports.setInnerHtml = function(el, innerHtml) {
    var element = el.cloneNode(false);//document.createElement("div");
    element.innerHTML = innerHtml;
    el.parentNode.replaceChild(element, el);
    return element;
};

if ("textContent" in document.documentElement) {
    exports.setInnerText = function(el, innerText) {
        el.textContent = innerText;
    };

    exports.getInnerText = function(el) {
        return el.textContent;
    };
}
else {
    exports.setInnerText = function(el, innerText) {
        el.innerText = innerText;
    };

    exports.getInnerText = function(el) {
        return el.innerText;
    };
}

exports.getParentWindow = function(document) {
    return document.defaultView || document.parentWindow;
};

});

/**
 * 
 * ########################################################################
 * #                                                                      #
 * #  A new way to load the css in WebCenter. (for re-usable components)  #
 * #                                                                      #
 * ########################################################################
 * 
 * ## Usage: 
 * define(["utils/webcss!./NameOfCssDocument.css"], function(){
 * 		// Your module which uses the css here.
 * });
 * 
 * OR (with less files)
 * 
 * define(["utils/webcss!./NameOfLessDocument.less"], function(){
 * 		// Your module which uses the less (css) here.
 * });
 * 
 * ## TECHNICAL DETAILS:
 * 
 * What this module does?
 *  - It loads the content of the css file using AMD
 *  - It makes all paths to images in the css relative to the page...
 *  - It puts the css file in the header of the document (inline)
 * 
 * Note: less is compiled by webcss-text... This module does not know anything about less...
 * 
 * An alternative would be:
 *  - To just add a &lt;link rel="stylesheet" src="sourceOfStylesheet"&gt; in the header of the document.
 * 
 * Why don't we use the alternative way? 
 *  [+] When optimizing wcr-static, the css is now bundled in the javascript component.
 *      (No need to do a separate call to load the css)
 *  [+] The css is available in the document before the component is executed
 *  [-] Paths in the css are relative to the stylesheet. 
 *      We wouldn't have a problem with that if we used the alternative way.
 *      However, if the css is put inline in the page, the browser does not know where the file came from...
 *      That is why this module needs to convert the urls. (make all paths to images in the css relative to the page...)
 * 
 * ## KNOWN LIMITATIONS:
 *  - does not support @ include in css (yet)
 *    If you need this, use less instead... (in less files, @ include is supported)
 * 
 * ## OPTIMIZATION NOTES:
 *  - if you don't want to bundle a certain css in the javascript, exclude "lib/webcss-text!<name of css>" when building.
 *    DO NOT use this for .less files (only .css). For .less files, when they are not bundled, they will be compiled in the browser!
 * 
 * @author JEPE
 */
define('utils/webcss',["lib/dom"], function(dom){
	var loader = {};
	
	/**
	 * Instead of a path relative to the css file, return an path relative to the document we are working in 
	 */
	function modifyPath(req, name, path){
		if (path.indexOf("/") === 0 || path.indexOf("data:") === 0) {
			// absolute path... no problem
			return path;
		}
		// Split the path to the image
		var pathPieces = path.split("/");
		
		// Split the URL to the css file
		var urlToCssFile = req.toUrl(name);
		var urlToCssFilePieces = urlToCssFile.split("/");
		// remove the file name from the path
		urlToCssFilePieces.splice(-1,1);
		
		// Join both paths...
		var newPath = [].concat(urlToCssFilePieces).concat(pathPieces);
		
		return newPath.join("/");
	}

	function makeCssLocal (req, name, content){
		// Make all paths in css local to page instead of stylesheet...
		// 1) Find all url(" ... ") things in the css.
		var urlPattern = /url\(('?"?)([^'")]*)('?"?)\)/g;
		newContent = content.replace(urlPattern, function(a, b, c, d){
			// 2) Modify the path to be local to the page
			return "url("+b+modifyPath(req, name, c) + d +")";
		});
		
		return newContent;
	}

	/**
	 * Implements a custom require loaded...
	 * 
	 * Allow using "utils/webcss!./nameOfCss.css"
	 * 
	 * See http://requirejs.org/docs/plugins.html for information about implementing custom loaders...
	 * @param name
	 * @param req
	 * @param onload
	 * @param config
	 */
	loader.load = function (name, req, onload, config) {
		// Load the css as text...
		req(["utils/webcss-text!"+name], function(cssContent){
			// Do not do further steps in build mode... 
			if(config.isBuild) {
				onload();
				return;
			}
			
			// Make the css local
			var newCssContent = makeCssLocal(req, name, cssContent);
			
			// Import the css (would crash if called in build mode)
			dom.importCssString(newCssContent);
			
			// Return something...
			onload({});
		});
	};

	return loader;
});

/**
 * Used by webcss to get the css text content...
 * 
 * Same as text, but BUILDS LESS FILES!!!
 * 
 * Note: see webcss-build for the version used during optimization...
 * 
 * @author JEPE
 */

define('utils/webcss-text',[], function(){
	var buildMap = {};
	
	/**
	 * Utility method
	 */
	function endsWith(str, suffix) {
	    return str.indexOf(suffix, str.length - suffix.length) !== -1;
	}
	
	
	
	var loader = {};
	
	/**
	 * Build the stylesheet (either less or css)
	 * 
	 * NOTE: Similar code in webcss-text & webcss-build !!!
	 * HOWEVER: Different versions of less + less for different environment...
	 */
	function buildStylesheet(req, isLessFile, name, content, config, callback, callbackerror) {
		// check if we did not already build this file...
		if(buildMap.hasOwnProperty(name)) {
			callback(buildMap[name]);
			return;
		}
		
		if(isLessFile) {
			// LESS - Build the less...
			require(["lib/less"], function(less){
				var baseUrl = config.baseUrl;
				var fileUrl = name;
				
				var cfg = config.less || {};
			    cfg.paths = [baseUrl];
			    cfg.filename = baseUrl+fileUrl;
			    cfg.async = true;
				cfg.syncImport = false;
			    
			    // Should only happen in development... But better add a warning...
			    console.log("Warning - not optimized - compiling less in browser...");
			    less.render(content, cfg, function(err, output) {
			    	if (err) {
			    		// There is an error in the less file...
			    		console.log(err + ' at ' + err.filename + ', line ' + err.line);
				        callbackerror();
				        return;
				    }
			    	
			    	var newContent = output.css;
			    	
			    	buildMap[name] = newContent;
				    callback(newContent);
			    });
			});
		} else {
			// CSS
			buildMap[name] = content;
			callback(content);
		}
	}

	/**
	 * Implements a custom require loader...
	 * 
	 * See http://requirejs.org/docs/plugins.html for information about implementing custom loaders...
	 * @param name
	 * @param req
	 * @param onload
	 * @param config
	 */
	loader.load = function (name, req, onload, config) {
		var isLessFile = endsWith(name.toLowerCase(), ".less");
		
		if(config.isBuild) {
			//
			// We are optimizing...
			//
			// Should NEVER reach here... Instead, "webcss-build.load(...)" should have been called...
		}else{
			//
			// We are just loading without optimization...
			//
			
			// 1. read file with css/less
			req(["lib/text!"+name], function(content){
				// 2. build file... (if needed)
				buildStylesheet(req, isLessFile, name, content, config, function(newContent){
					// 3. return
					onload(newContent);
				}, function(){
					// 3. an error occured...
					onload.error();
				});
			});
		}
	};
	
	loader.pluginBuilder = "utils/webcss-build";

	return loader;
});

define('utils/all',[
	"utils/Animatable",
	"utils/Cache",
	"utils/ColorUtils",
	"utils/ConvertUtils",
	"utils/Command",
	"utils/dateformatter",
	"utils/DomUtils",
	"utils/DashUtils",
	"utils/EventBasedValidator",
	"utils/ExecutionQueue",
	"utils/extendedstring",
	"utils/HTTPUtils",
	"utils/I18n",
	"utils/init",
	"utils/Map",
	"utils/Mousewheel",
	"utils/Observable",
	"utils/Observer",
	"utils/OrderedObject",
	"utils/Popup",
	"utils/PriorityOrderedObject",
	"utils/ShapeUtils",
	"utils/Transformation",
	"utils/prefixTree",
	"utils/ShapeUtils", 
	"utils/SMSDrop",
	"utils/StringSet",
	"utils/UndoRedoController",
	"utils/WCEvent",
	"utils/wcrerror",
	"utils/webcss",
	"utils/webcss-text",
	"utils/webi18n",
	"utils/XMLUtils"
], function () {});
