
define('web-viewer/annotation/AnnotationFilter',["underscore",
		"web-viewer/core/PrivateChannel",
		"web-viewer/annotation/AnnotationConstants"
	],
	function (_,
		PrivateChannel,
		AnnotationConstants) {
		"use strict";
		/**
		 * @class
		 * Controller for filtering annotation, saves the filtered state in the filter model
		 * @exports AnnotationFilter
		 * @memberof AnnotationModule
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {AnnotationFilterModel} annotationFilterModel
		 */
		var AnnotationFilter = function (f, annotationFilterModel) {
			PrivateChannel.installTo(this, f, ['annotationFilter']);

			this._annotationFilterModel = annotationFilterModel;
		};

		/**
		 * filter the annotation
		 * @param {Array.<Annotation>} annotations
		 * @returns {Array.<Annotation>}
		 */
		AnnotationFilter.prototype.filterAnnotations = function (annotations) {
			var filteredAnnotations = [];
			if (this._annotationFilterModel.isCustomFilterActive() && annotations) {
				filteredAnnotations = _.filter(annotations, _.bind(function (annotation) {
					return this.compliesToCustomFilter(annotation);
				}, this));
				this._annotationFilterModel.setNumberOfFilteredAnnotations(filteredAnnotations.length);
				return filteredAnnotations;
			} else if (annotations) {
				filteredAnnotations = _.filter(annotations, _.bind(function (annotation) {
					return this.compliesToFilter(annotation);
				}, this));
				this._annotationFilterModel.setNumberOfFilteredAnnotations(filteredAnnotations.length);
				return filteredAnnotations;

			} else {
				this._annotationFilterModel.setNumberOfFilteredAnnotations(0);
				return [];
			}
		};

		/**
		 * Filters on which annotations need te be shown (depending on the viewer wide filtering of annotation authors)
		 * Deletes review states and replies from the annotationModel of the page to be sure they are hidden in the annotation list
		 * @param   {Array.<Annotation>} annotations
		 * @returns {Array.<Annotation>}  array of the filtered annotations
		 */
		AnnotationFilter.prototype.filterIncomingAnnotations = function (annotations) {
			var filters = this._annotationFilterModel.getViewerUserFilter();
			if (filters.length > 0) {
				var annotsToRemove = []; //save top-level annotations that can be removed
				//now run over all our filtered annotations, and run through their replies/states to check the authors of those
				_.each(annotations, function (annotation) {
					//run through the review states and replies of the main annotation, the main annotation doesn't need be checked
					annotation.runThroughAnnotation(true, true, true, _.bind(function (annot) {
						//check if the author of the annotaiton complies to the filter (any of the filter authors is correct)
						var authorComplies = _.some(filters, function (filter) {
							return this._compliesToAuthor(annot, filter);
						}, this);
						if (!authorComplies) { //if it doesn't comply to the filter remove the review state/reply
							if (annot.inreplyto && !annot.state) { //it's a reply, remove the reply from the parent
								var index = _.indexOf(annot.parent.replies, annot);
								annot.parent.replies.splice(index, 1); //save to remove, runThroughAnnotation makes this possible
								return true; //no need to check the children of the reply (review states)
							} else if (annot.state && annot.parent) { //it's a review state
								var index = _.indexOf(annot.parent.states, annot);
								annot.parent.states.splice(index, 1);
								return true;
							} else if (!annot.parent || annot.linkedto) { //if top level annotation add it to array to remove after foreach
								annotsToRemove.push(annot);
								return true; //no need to run over the replies and states of the annotation
							}
						}
						return false;
					}, this));
				}, this);
				//remove all the annotation that do not comply from the annotations list
				_.each(annotsToRemove, function (removeAnnot) {
					var index = _.indexOf(annotations, removeAnnot);
					annotations.splice(index, 1);
				});

				return annotations; //return the collection of the filteredAnnotations
			} else { //no filter was defined.
				return annotations;
			}

		};

		/**
		 * check if the annotation needs to be shown with the user filter and state filter
		 * @param {Annotation} annotation
		 * @returns {Boolean}
		 */
		AnnotationFilter.prototype.compliesToFilter = function (annotation) {
			// If it isn't a user annotation, the filters don't really make sense
			var originType = this._annotationFilterModel.getOriginType();
			if (originType !== AnnotationConstants.AnnotationOriginTypes.user.id &&
				originType === annotation.originType) {
				return true;
			}
			return this._compliesToUser(annotation) && this._compliesToState(annotation) && this._compliesToType(annotation);
		};

		/**
		 * Does the annotation comply to the custom filter that has been set up
		 * @param   {Annotation} annotation
		 * @returns {Boolean}
		 */
		AnnotationFilter.prototype.compliesToCustomFilter = function (annotation) {
			var filters = this._annotationFilterModel.getFilterRows();

			if (this._annotationFilterModel.getFilterOperator() == AnnotationConstants.FilterOperators.ALL) {
				return _.every(filters, function (filter) {
					return this._checkCustomFilterRule(annotation, filter);
				}, this);
			} else if (this._annotationFilterModel.getFilterOperator() == AnnotationConstants.FilterOperators.ANY) {
				return _.some(filters, function (filter) {
					return this._checkCustomFilterRule(annotation, filter);
				}, this);
			}
		};

		/**
		 * Check if the annotation complies to the custom filter rule
		 * @param   {Annotation} annotation
		 * @param   {Object}   filter
		 * @returns {Boolean}
		 * @private
		 */
		AnnotationFilter.prototype._checkCustomFilterRule = function (annotation, filter) {
			if (filter.filterType == AnnotationConstants.FilterTypes.AUTHOR) {
				if (filter.filterCondition == AnnotationConstants.FilterCondition.IS) {
					return this._compliesToUser(annotation, filter.filterOption);
				} else {
					return !this._compliesToUser(annotation, filter.filterOption);
				}
			} else if (filter.filterType == AnnotationConstants.FilterTypes.REVIEWSTATE) {
				if (filter.filterCondition == AnnotationConstants.FilterCondition.IS) {
					return this._compliesToCustomFilterState(annotation, filter.filterOption);
				} else {
					return !this._compliesToCustomFilterState(annotation, filter.filterOption);
				}
			} else if (filter.filterType == AnnotationConstants.FilterTypes.REVIEWAUTHORANDSTATE) {
				if (filter.filterCondition == AnnotationConstants.FilterCondition.IS) {
					return this._compliesToReviewAuthorAndState(annotation, filter.filterOption);
				} else {
					return !this._compliesToReviewAuthorAndState(annotation, filter.filterOption);
				}
			}
		};

		/**
		 * does the annotation comply to the user filter
		 * @private
		 * @param {Annotation} annotation
		 * @returns {Boolean}
		 */
		AnnotationFilter.prototype._compliesToUser = function (annotation, user) {
			var complies = true;
			if (user === undefined) {
				user = this._annotationFilterModel.getSelectedUser();
			}
			if (user) {
				complies = false;
				annotation.runThroughAnnotation(true, true, true, _.bind(function (annot) {
					if (annot.username == user) {
						complies = true;
						return true;
					}
				}, this));
			}
			return complies;
		};

		AnnotationFilter.prototype._compliesToType = function (annotation) {
			return annotation.originType === this._annotationFilterModel.getOriginType();
		};

		/**
		 * Does the annotation comply to the author
		 * @private
		 * @param   {Annotation}   annotation
		 * @param   {String} author
		 * @returns {Boolean}
		 */
		AnnotationFilter.prototype._compliesToAuthor = function (annotation, author) {
			if (author) {
				if (annotation.username == author) {

					return true;
				}
			}
			return false;
		};

		/**
		 * Does the annotation comply to the author and state of the review
		 * @private
		 * @param   {Annotation} annotation
		 * @param   {String} reviewAuthorAndState
		 * @returns {Boolean}
		 */
		AnnotationFilter.prototype._compliesToReviewAuthorAndState = function (annotation, reviewAuthorAndState) {
			var complies = false;
			if (reviewAuthorAndState) {
				var reviewStateKeys = _.invert(AnnotationConstants.AnnotationReviewState);

				annotation.runThroughAnnotation(true, true, false, _.bind(function (annot) {
					var states = annot.states;
					complies = _.some(states, _.bind(function (state) {
						var annotReviewAuthorState = state.getUserName() + " " + reviewStateKeys[state.state];
						return reviewAuthorAndState === annotReviewAuthorState;
					}, this));
					if (complies) {
						return true;
					}
				}, this));
			}
			return complies;
		};

		/**
		 * does the annotation comply to the state filter
		 * @private
		 * @param {Annotation} annotation
		 * @returns {Boolean}
		 */
		AnnotationFilter.prototype._compliesToState = function (annotation) {
			var complies = true;
			var reviewState = this._annotationFilterModel.getSelectedState();
			if (reviewState) {
				complies = false;

				if (reviewState == "Any") {
					annotation.runThroughAnnotation(true, !this._annotationFilterModel.isInverseState(), false, _.bind(function (annot) {
						var states = annot.states;
						complies = complies || states.length > 0;
					}, this));
				} else {
					// only check replies if it is not an inverse State
					annotation.runThroughAnnotation(true, !this._annotationFilterModel.isInverseState(), false, _.bind(function (annot) {
						var states = annot.states;
						var annotComplies = _.some(states, _.bind(function (state) {
							return state.state == reviewState;
						}, this));
						if (annotComplies) {
							complies = true;
						}
					}, this));
				}

				if (this._annotationFilterModel.isInverseState()) {
					complies = !complies;
				}
			}
			return complies;
		};

		/**
		 * Does the annotation (with all its replies) comply to the custom filter state
		 * @private
		 * @param   {Annotation} annotation
		 * @param   {String} reviewState
		 * @returns {Boolean}
		 */
		AnnotationFilter.prototype._compliesToCustomFilterState = function (annotation, reviewState) {
			var complies = false;
			annotation.runThroughAnnotation(true, true, false, _.bind(function (annot) {
				var states = annot.states;
				var annotComplies = _.some(states, _.bind(function (state) {
					return state.state == AnnotationConstants.AnnotationReviewState[reviewState];
				}, this));
				if (annotComplies) {
					complies = true;
				}
			}, this));
			return complies;
		};

		return AnnotationFilter;
	});

define('web-viewer/annotation/Annotation',["web-viewer/utils/LineworkWrapper",
		"web-viewer/utils/LineworkPoint",
		"utils/ShapeUtils",
		"utils/XMLUtils",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/utils/GEOUtils",
		"web-viewer/utils/DrawUtils",
		"utils/StringSet",
		"web-viewer/annotation/AnnotationConstants",
		"underscore"
	],
	function (LineworkWrapper,
		LineworkPoint,
		ShapeUtils,
		XMLUtils,
		ViewerUtils,
		GEOUtils,
		DrawUtils,
		StringSet,
		AnnotationConstants,
		_) {
		"use strict";

		/**
		 * @class Represents an annotation. It includes the linework and the data of the annotation.
		 * @exports Annotation
		 * @memberof AnnotationModule
		 * @augments Objects.LineworkWrapper
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {String} originType - The origin of the annotation, defaults to "user", which indicates a user generated annotation
		 */
		var Annotation = function (f, originType) {
			LineworkWrapper.call(this);
			this.f = f;

			/** creation date */
			this.creationdate = new Date();
			/** modification date */
			this.date = new Date();
			/** name object of the annotation (will be used to construct username/firstname/lastname */
			this.name = null;
			/** page number */
			this.page = 0;
			/** subject of the annotation */
			this.subject = "";
			/** title */
			this.title = "";
			/** content as a string (without BODY tag) */
			this.content = "";

			/** origin of the annotation, this is derived from the xfdf file the annotations originates from 
			 *  AnnotationConstants.AnnotationOriginTypes has the set of the supported types
			 */
			this.originType = originType || AnnotationConstants.AnnotationOriginTypes.user.id;


			/**
			 * The 3D view camera position
			 * @example 
			 * {scale: 1.5,
			 * azimuth: 46.6,
			 * elevation: -52}
			 */
			this._view3DPosition = null;

			/** user name */
			this.username = "";
			/** first name */
			this.firstname = "";
			/** last name */
			this.lastname = "";
			/** message number. This is generated by the server. If it's a non-saved annotation, it will be temporarily assigned to the highest number */
			this.messagenumber = -1;
			/** Type of annotation {AnnotationConstants.AnnotationType} */
			this.annottype = null;

			/** parent {Annotation} object. In case of inreplyto or state or link */
			this.parent = null;

			/** Array of replies to this annotation */
			this.replies = [];
			/** {Annotation} the annotation that this annotation is in reply to. If null, there is no reply. The actual annotation object should be in the "parent" object */
			this.inreplyto = null;

			/** Array of links to this annotation */
			this.links = [];
			/** {Annotation} Indication if the annotation is linked to another annotation (will be in the parent object) */
			this.linkedto = null;

			/** Is this annotation saved? */
			this.saved = false;
			/** Is this annotation new (never saved on the server)? */
			this.isNew = true;

			/** Array of states for this annotation -> {Annotation} objects */
			this.states = [];
			/** State name if the annotation is a state-object */
			this.state = null;
			/** {String } State model, attribute in the annotation xml, if the annotation is a state-object */
			this.statemodel = null;

			/** {String} - attribute flags in the xml (by default just 'print') */
			this.flags = 'print';

			/** Rectangle of the popup - calculated from bottom left corner*/
			this.popuprect = new ShapeUtils.Rect();

			/** Page of the popup */
			this.popuppage = 0;
			/** Indication if the popup is/was opened when saved */
			this.popupIsOpen = false;
			/** this is the content of the reply while it is not yet saved */
			this.unsavedReplyText = null;
			/** Modified content of the annotation **/
			this.modifiedContent = null;
			/** Original annotation color, modified sits in this.annotationcolor **/
			this.originalAnnotationColor = null;

			/** keep track of what modifications have been done*/
			this._modifications = new StringSet();

			/** {ShapeUtils.Point} the scaling of the annotation, null if there is no scaling */
			this._scaling = null;

			/**
			 * If the rectangle is changed, set the annotation on modified
			 */
			this._override("setRect", function (rect, flip) {
				this._super(rect, flip);
				this.setModified("LINEWORK");
			});

			/**
			 * If the annotation is moved, set the annotation on modified
			 */
			this._override("move", function (size) {
				this._super(size);
				if (size.width !== 0 || size.height !== 0) {
					this.setModified("LINEWORK");
				}
			});

			/**
			 * If annotation's start and end point changed, set the annotation on modified
			 */
			this._override("setStartEnd", function (start, end) {
				this._super(start, end);
				this.setModified("LINEWORK");
			});


		};
		// inherit
		Annotation.prototype = Object.create(LineworkWrapper.prototype);
		// correct the constructor pointer 
		Annotation.prototype.constructor = Annotation;

		/**
		 * @returns {AnnotationConstants.AccessRight} access right object of the annotation
		 */
		Annotation.prototype.getPermission = function () {
			if (this.f.getCurrentUser().getUsername() == this.username) {
				return AnnotationConstants.AccessRight.READ_WRITE;
			} else {
				return AnnotationConstants.AccessRight.READ_ONLY;
			}
		};

		/**
		 * @returns {Boolean} permission for a certain action on the annotation
		 */
		Annotation.prototype.getPermissionFor = function (action) {
			return action !== AnnotationConstants.ActionPermissions.DUPLICATE_WITHOUT_TEXT;
		};


		/**
		 * @returns {String} sort by attribute of the annotation
		 */
		Annotation.prototype.getSortByAttr = function () {
			return this.content;
		};

		/**
		 * Get the outline of the rect of all annotations + its links
		 * @returns {ShapeUtils.Rect} The outline of the rect of all annotations + its links
		 */
		Annotation.prototype.getOutliningRect = function () {
			var annotationGroup = _.union(this.links, [this]); //annotation with all its linked annotations

			var x0 = _.min(annotationGroup, function (annotation) {
				return annotation.rect.left();
			}).rect.left();
			var y0 = _.max(annotationGroup, function (annotation) {
				return annotation.rect.top();
			}).rect.top();
			var x1 = _.max(annotationGroup, function (annotation) {
				return annotation.rect.right();
			}).rect.right();
			var y1 = _.min(annotationGroup, function (annotation) {
				return annotation.rect.bottom();
			}).rect.bottom();

			return new ShapeUtils.Rect(new ShapeUtils.Point(x0, y0), new ShapeUtils.Size(x1 - x0, y1 - y0));
		};

		/**
		 * @returns {String} class to represent the icon of the annotation (depends on its type)
		 */
		Annotation.prototype.getIconClass = function () {
			return AnnotationConstants.AnnotationIcons[this.annottype];
		};

		/**
		 * @returns {Boolean} true if creation date is the same as the modification date
		 */
		Annotation.prototype.isAnnotationModified = function () {
			return this.date - this.creationdate === 0;
		};


		/**
		 * Initialize an annotation based on a type, color and if it's selected
		 * @param {AnnotationConstants.AnnotationType} type
		 * @param {String} color - annotation color
		 * @param {Boolean} selected - if the annotation is initialized as being selected
		 * @param {AnnotationModel} [annotationModel] - needed when initializing annotation that has no name yet
		 * @param {ShapeUtils.Rect} [rect] - rectangle to initialize the annotation with, if not specified 0,0 rect will be used
		 */
		Annotation.prototype.initAnnotation = function (type, color, selected, annotationModel, rect) {
			if (!this.name || this.name.length === 0) {
				if (annotationModel === undefined) {
					this.f.emit("error.SEVERE", "Problem initializing annotation", "No model specified for generating unique name!");
				} else {
					this.name = annotationModel.generateUniqueName();
				}
			}

			if (!rect) {
				rect = new ShapeUtils.Rect();
			}
			this.rect = rect;
			if (annotationModel) {
				this._scaling = annotationModel.getCurrentScaling();
			}

			if (!color) {
				color = this.annotationcolor;
			}
			this.originalAnnotationColor = color;
			this.annotationcolor = color;
			if (this.annotationcolor === null) {
				this.annotationColor = this._getDefaultAnnotationColor(type);
			}

			this.color = color;

			this.annottype = type;
			this.setSelected(selected);
			if (!this.subject || this.subject.length === 0) {
				this.subject = ((this.inreplyto != null) ? "Re" + ": " : "") + AnnotationConstants.AnnotationReplyName[this.annottype];
			}


			switch (type) {
			case AnnotationConstants.AnnotationType.ANNOT_RECT:
				this.color = color;
				this.type = DrawUtils.LWTYPE_RECT;
				break;

			case AnnotationConstants.AnnotationType.ANNOT_ELLIPSE:
				this.color = color;
				this.type = DrawUtils.LWTYPE_ELLIPSE;
				break;

			case AnnotationConstants.AnnotationType.ANNOT_HIGHLIGHT:
				this.type = DrawUtils.LWTYPE_RECT;
				this.treatAsOutline = true;
				this.color = null;
				break;

			case AnnotationConstants.AnnotationType.ANNOT_LINE:
				this.type = DrawUtils.LWTYPE_LINE;
				this.color = color;

				this.pointArray[0] = new LineworkPoint();
				this.pointArray[0].makeMoveTo(this.rect.origin.x, this.rect.origin.y);
				this.pointArray[1] = new LineworkPoint();
				this.pointArray[1].makeLineTo(this.rect.origin.x + this.rect.size.width, this.rect.origin.y + this.rect.size.height);

				break;

			case AnnotationConstants.AnnotationType.ANNOT_ARROW:
				this.type = DrawUtils.LWTYPE_ARROW;
				this.color = color;

				this.pointArray[0] = new LineworkPoint();
				this.pointArray[0].makeMoveTo(this.rect.origin.x, this.rect.origin.y);
				this.pointArray[1] = new LineworkPoint();
				this.pointArray[1].makeLineTo(this.rect.origin.x + this.rect.size.width, this.rect.origin.y + this.rect.size.height);

				break;

			case AnnotationConstants.AnnotationType.ANNOT_PIN:
				this.type = DrawUtils.LWTYPE_IMAGE;
				this.imagehotspot = new ShapeUtils.Point(AnnotationConstants.AnnotationPin.imagehotspotx, AnnotationConstants.AnnotationPin.imagehotspoty);
				this.imagetouchspot = new ShapeUtils.Rect(new ShapeUtils.Point(AnnotationConstants.AnnotationPin.imagetouchspotx, AnnotationConstants.AnnotationPin.imagetouchspoty), new ShapeUtils.Size(AnnotationConstants.AnnotationPin.imagetouchspotw, AnnotationConstants.AnnotationPin.imagetouchspoth));
				break;

			case AnnotationConstants.AnnotationType.ANNOT_FREEFORM:
				this.type = DrawUtils.LWTYPE_LINEARRAY;
				this.color = color;
				break;

			}
		};

		/**
		 * Init an annotation from XML
		 * @param {Element} xml - The XML element that is used to initialize the annotation
		 * @param {Object} boxOffsets - The offsets of the media and trim boxes origins
		 */
		Annotation.prototype.initWithXML = function (xml, boxOffsets) {

			this.saved = true; //set saved to true
			this.isNew = false; //set new to false
			this.pointArray = []; //reset the point array to avoid duplication

			this.name = XMLUtils.attributeValue(xml, 'name');
			this.subject = XMLUtils.attributeValue(xml, 'subject');
			this.title = XMLUtils.attributeValue(xml, 'title');
			this.flags = XMLUtils.attributeValue(xml, 'flags');

			this._view3DPosition = {
				scale: ViewerUtils.parseFloat(XMLUtils.attributeValue(xml, 'view3Dscale')),
				azimuth: ViewerUtils.parseFloat(XMLUtils.attributeValue(xml, 'view3Dazimuth')),
				elevation: ViewerUtils.parseFloat(XMLUtils.attributeValue(xml, 'view3Delevation'))
			};

			if (isNaN(this._view3DPosition.scale) || isNaN(this._view3DPosition.azimuth) || isNaN(this._view3DPosition.elevation)) {
				this._view3DPosition = null;
			}

			/**
			 * Parse the title and split it into "messagenumber", last name, first name and username
			 * This can be different depending on the version of WebCenter this was created in
			 *
			 * Order of trying:
			 * 	- 11 Cagnie, Ruben (RUCA)
			 *  - 11 ADMIN (ADMIN)
			 *  - 11 USERNAME
			 *  - Cagnie, Ruben (RUCA)
			 */
			var numberExtraction = /((\d+)\s)?(.+)/.exec(this.title);
			if (numberExtraction) {
				this.messagenumber = numberExtraction[2];
				if (this.messagenumber === undefined) {
					this.messagenumber = -1;
				}

				var nameExtraction = /([^,]+)(,\s(.+))?\s\((.+)\)/.exec(numberExtraction[3]);
				if (nameExtraction) {
					this.username = nameExtraction[4];
					this.lastname = nameExtraction[1];
					this.firstname = nameExtraction[3] || "";
				} else {
					this.username = numberExtraction[3];
					this.lastname = "";
					this.firstname = "";
				}

			}

			this.messagenumber = parseInt(this.messagenumber, 10);

			this.page = XMLUtils.attributeValue(xml, 'page');

			this.date = ViewerUtils.dateFromString(XMLUtils.attributeValue(xml, 'date'));
			this.creationdate = ViewerUtils.dateFromString(XMLUtils.attributeValue(xml, 'creationdate'));


			this.state = XMLUtils.attributeValue(xml, 'state');
			this.statemodel = XMLUtils.attributeValue(xml, 'statemodel');

			this._parseContentsFromXML(xml);

			var replyto = XMLUtils.attributeValue(xml, 'inreplyto');
			if (replyto) {
				this.inreplyto = replyto;
			}

			var linkedto = XMLUtils.attributeValue(xml, 'linkedto');
			if (linkedto) {
				this.linkedto = linkedto;
			}

			var rectexp = new RegExp(/(.+),(.+),(.+),(.+)/);
			var rectsplit = rectexp.exec(XMLUtils.attributeValue(xml, 'rect'));

			var x1 = GEOUtils.convertPSPointsToMM(rectsplit[1]);
			var y2 = GEOUtils.convertPSPointsToMM(rectsplit[2]);
			var x2 = GEOUtils.convertPSPointsToMM(rectsplit[3]);
			var y1 = GEOUtils.convertPSPointsToMM(rectsplit[4]);
			var coords = this._modifyAnnotatationCoords(x1, y2, x2, y1, boxOffsets);
			x1 = coords[0];
			y2 = coords[1];
			x2 = coords[2];
			y1 = coords[3];

			var type = xml.nodeName;
			switch (type) {
			case 'square':
				this.initAnnotation(AnnotationConstants.AnnotationType.ANNOT_RECT);
				break;
			case 'circle':
				this.initAnnotation(AnnotationConstants.AnnotationType.ANNOT_ELLIPSE);
				break;
			case 'highlight':
				this.initAnnotation(AnnotationConstants.AnnotationType.ANNOT_HIGHLIGHT);
				break;
			case 'text':
				this.rect.size.width = 0;
				this.rect.size.height = 0;
				this.initAnnotation(AnnotationConstants.AnnotationType.ANNOT_PIN);
				break;
			case 'ink':
				this.initAnnotation(AnnotationConstants.AnnotationType.ANNOT_FREEFORM);

				if (!this.inreplyto) {
					var inklist = xml.getElementsByTagName('inklist')[0];
					if (inklist) {
						var gesture = inklist.getElementsByTagName('gesture')[0];
						var node = gesture;
						var pointArray = "";
						// Firefox splits large text regions into multiple Text objects (4096 chars in
						// each). Glue it back together.
						for (var x = 0; x < node.childNodes.length; x++) {
							pointArray += node.childNodes[x].nodeValue;
						}
						if (pointArray.length > 0) {
							var chuncs = pointArray.split(';');

							var startpoint = new LineworkPoint();
							var chunkpoint = chuncs[0].split(',');
							if (chunkpoint[0] !== undefined && chunkpoint[1] !== undefined) {
								startpoint.makeMoveTo(GEOUtils.convertPSPointsToMM(chunkpoint[0]), GEOUtils.convertPSPointsToMM(chunkpoint[1]));
								this.pointArray.push(startpoint);
								for (var i = 1; i < chuncs.length; i++) {
									var movepoint = new LineworkPoint();
									chunkpoint = chuncs[i].split(',');
									if (chunkpoint[0] !== undefined && chunkpoint[1] !== undefined) {
										var px = GEOUtils.convertPSPointsToMM(chunkpoint[0]);
										var py = GEOUtils.convertPSPointsToMM(chunkpoint[1]);
										movepoint.makeLineTo(px, py);
										this.pointArray.push(movepoint);
									} else {
										this.f.emit("error.DEBUG", "chunkpoint[0] || chunkpoint[1] is undefined...");
									}
								}
							} else {
								this.f.emit("error.DEBUG", "chunkpoint[0] || chunkpoint[1] is undefined...");
							}
						}
						this.calculateRectFromPointArray();
					}
				}
				break;
			case 'line':
				var subject = XMLUtils.attributeValue(xml, 'subject');
				if (subject && subject == 'Arrow') {
					this.initAnnotation(AnnotationConstants.AnnotationType.ANNOT_ARROW);
				} else {
					this.initAnnotation(AnnotationConstants.AnnotationType.ANNOT_LINE);
				}

				if (!this.inreplyto) {
					var pointexp = new RegExp(/(.+),(.+)/);

					var start = XMLUtils.attributeValue(xml, 'start');
					var end = XMLUtils.attributeValue(xml, 'end');

					var startsplit = pointexp.exec(start);
					var endsplit = pointexp.exec(end);

					var startx = GEOUtils.convertPSPointsToMM(startsplit[1]);
					var starty = GEOUtils.convertPSPointsToMM(startsplit[2]);
					var endx = GEOUtils.convertPSPointsToMM(endsplit[1]);
					var endy = GEOUtils.convertPSPointsToMM(endsplit[2]);

					var startpoint = new LineworkPoint();
					startpoint.makeMoveTo(startx, starty);
					var endpoint = new LineworkPoint();
					endpoint.makeLineTo(endx, endy);

					this.pointArray[0] = startpoint;
					this.pointArray[1] = endpoint;

					var sp = new ShapeUtils.Point(startpoint.x, startpoint.y);
					var ep = new ShapeUtils.Point(endpoint.x, endpoint.y);
					this.rect = new ShapeUtils.Rect(sp, sp.sizeTo(ep));
					this.rect.normalize();
				}
				break;
			}

			this.rect.origin.x = x1;
			this.rect.origin.y = y1;
			this.rect.size.width = x2 - x1;
			this.rect.size.height = y2 - y1;

			var color = XMLUtils.attributeValue(xml, 'color');
			if (!/^#?[a-f\d]{6}$/i.test(color)) {
				color = this._getDefaultAnnotationColor(this.annottype);
			}
			this.color = color;
			this.originalAnnotationColor = color;
			this.annotationcolor = color;

			var popup = xml.getElementsByTagName('popup')[0];
			if (popup) {
				this.popuppage = XMLUtils.attributeValue(popup, 'page');
				this.popupIsOpen = XMLUtils.attributeValue(popup, 'open') == 'yes';

				var popuprectsplit = rectexp.exec(XMLUtils.attributeValue(popup, 'rect'));

				var popupx1 = GEOUtils.convertPSPointsToMM(popuprectsplit[1]);
				var popupy1 = GEOUtils.convertPSPointsToMM(popuprectsplit[2]);
				var popupx2 = GEOUtils.convertPSPointsToMM(popuprectsplit[3]);
				var popupy2 = GEOUtils.convertPSPointsToMM(popuprectsplit[4]);

				this.popuprect.origin.x = popupx1;
				this.popuprect.origin.y = popupy1;
				this.popuprect.size.width = popupx2 - popupx1;
				this.popuprect.size.height = popupy2 - popupy1;
			}
		};

		/**
		 * Allows the annotation to modify it's coordinates when creating the annotation from XML
		 * This lets subclasses define other coordinates if for example the coordinates in the xml are relative to the media box instead of the trim box
		 * Defaults to coordinates relative to trim box
		 * @param {Number} x1 - bottom left x coordinate
		 * @param {Number} y2 - bottom left y coordinate
		 * @param {Number} x2 - top right x coordinate
		 * @param {Number} y1 - top right y coordinate
		 * @param {Object} boxOffests - object containing the mediaboxOffset and trimboxOffset
		 */
		Annotation.prototype._modifyAnnotatationCoords = function (x1, y2, x2, y1, boxOffsets) {
			return [x1, y2, x2, y1];
		};

		/**
		 * Duplicate an annotation
		 * @param {Boolean} keepReplies - True if you want to copy the replies to the duplicated annotation, false otherwise
		 * @param {Boolean} keepStates - True if you want to copy the states to the duplicated annotation, false otherwise
		 * @param {Boolean} keepInklist - True if you want to copy the pointArray to the duplicated annotation, false otherwise
		 * @param {Boolean} keepLinkedAnnotations - True if you want to copy the linked annotations to the duplicated annotation, false otherwise
		 * @param {AnnotationModel} annotationModel - The annotation model to ask a unique new name to
		 * @returns {Annotation} the duplicated annotation
		 */
		Annotation.prototype.duplicate = function (keepReplies, keepStates, keepInklist, keepLinkedAnnotations, annotationModel) {
			var newAnnotation = new Annotation(this.f);
			if (!keepInklist) {
				//copies properties of the annotation
				_.each(this, function (value, key) {
					newAnnotation[key] = value;
				});
				newAnnotation.modifiedContent = null;
				// make sure the rectangle is cloned, note copied
				if (newAnnotation.rect) {
					newAnnotation.rect = newAnnotation.rect.copy();
				}
				// same for the popuprect
				if (newAnnotation.popuprect) {
					newAnnotation.popuprect = newAnnotation.popuprect.copy();
				}
			} else {
				newAnnotation.initAnnotation(this.annottype, this.annotationcolor, true, annotationModel);
				newAnnotation.content = this.getTextContent();
			}

			newAnnotation.isNew = true;
			newAnnotation.name = annotationModel.generateUniqueName();

			if (!keepReplies) {
				newAnnotation.replies = [];
			}
			if (!keepStates) {
				newAnnotation.states = [];
			}
			if (!keepInklist) {
				newAnnotation.pointArray = [];
			} else {
				//check if the annotation is a reply, if this is the case use parent annotation's properties else is this the mainAnnotation
				var mainAnnotation = this.inreplyto ? this.parent : this;
				newAnnotation.pointArray = [];
				_.each(mainAnnotation.pointArray, function (point) {
					newAnnotation.pointArray.push(_.clone(point));
				});
				newAnnotation.color = _.clone(mainAnnotation.color);
				newAnnotation.rect = mainAnnotation.rect.copy();
				newAnnotation.popuprect = mainAnnotation.popuprect.copy();
			}
			if (keepLinkedAnnotations) {
				newAnnotation.links = [];
				_.each(this.links, function (link) {
					var newLink = link.duplicate(false, false, true, false, annotationModel);
					newLink.linkedto = newAnnotation.name;
					newLink.parent = newAnnotation;
					newAnnotation.links.push(newLink);
				});
			} else {
				newAnnotation.links = [];
			}

			return newAnnotation;
		};

		/**
		 * @returns {Boolean} true if the annotation needs a selection box (anchor points) to be edited
		 */
		Annotation.prototype.needsSelectionBox = function () {
			return (this.annottype != AnnotationConstants.AnnotationType.ANNOT_PIN);
		};

		/**
		 * Set the annotation selected
		 * @param {Boolean} selected - true if selected
		 */
		Annotation.prototype.setSelected = function (selected) {
			this.selected = selected;
			if (selected) {
				this.linewidth = 3;
			} else {
				this.linewidth = 2;
			}

			if (this.annottype === AnnotationConstants.AnnotationType.ANNOT_HIGHLIGHT) {
				this.fillcolor = "rgba(255, 255, 0, " + (selected ? ".8" : ".3") + ")";
			}
			if (this.annottype === AnnotationConstants.AnnotationType.ANNOT_PIN) {
				this.imageRef = selected ? AnnotationConstants.AnnotationPin.highlighted : AnnotationConstants.AnnotationPin.normal;
			}
		};

		/**
		 * Update the indication number (depending on the links) and the indication color
		 */
		Annotation.prototype.updateIndicator = function () {
			this.indicatorText = "" + this.messagenumber;
			if (this.linkedto) {
				if (this.parent) {
					this.indicatorText += "-" + (_.indexOf(this.parent.links, this) + 2); //add 2, because 1 is the main annotation and an array starts from 0...					
				}
			} else {
				var p = this.getGodfather();
				if (p.links.length > 0) {
					this.indicatorText += "-1";
				}
			}
			this.indicatorTextColor = this.getHeaderTextColor();
			this.indicatorColor = this.annotationcolor;
		};

		/**
		 * Get the full username of the user
		 * @returns {String} user name in following form: "Cagnie, Ruben (RUCA)" or "Admin (ADMIN")
		 */
		Annotation.prototype.getUserName = function () {
			if (this.firstname) {
				return this.lastname + ", " + this.firstname + " (" + this.username + ")";
			} else {
				return this.lastname + " (" + this.username + ")";
			}
		};

		/**
		 * @returns {Object|null} the annotation color in RGB colors
		 */
		Annotation.prototype.rgbColor = function () {
			var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(this.annotationcolor);
			return result ? {
				r: parseInt(result[1], 16),
				g: parseInt(result[2], 16),
				b: parseInt(result[3], 16)
			} : null;
		};

		/**
		 * @returns {String} 'white' if the annotation color is a dark color, 'black' if the annotation color is a light color
		 */
		Annotation.prototype.getHeaderTextColor = function () {
			var bg = this.rgbColor();
			var y = 0.30 * (bg.r / 255.0) + (0.59 * bg.g / 255.0) + (0.11 * bg.b / 255.0);

			if (y > 0.5) {
				return 'black';
			} else {
				return 'white';
			}
		};

		/**
		 * Indicate that the annotation has been modified. The modification date will be set to "now" as well
		 * @param {String} what - what is modified
		 */
		Annotation.prototype.setModified = function (what) {
			this._modifications.add(what);
			this.saved = this._modifications.size() === 0;
			this.date = new Date();
		};

		/**
		 * Set if the annotation is new or not, change
		 * @param {Boolean} isNew
		 */
		Annotation.prototype.setIsNew = function (isNew) {
			this.isNew = isNew;
		};

		/**
		 * Cancels the modified state.
		 * @param {String} [what] - which modification is canceled? if not provided, all modifications are cancelled
		 */
		Annotation.prototype.cancelModified = function (what) {
			if (what === "CONTENT") {
				this.modifiedContent = null;
			}
			if (what !== undefined) {
				this._modifications.remove(what);
			} else {
				this._modifications.clear();
			}

			this.saved = this._modifications.size() === 0;
		};

		/**
		 * The annotation is being modified, the changes are not yet saved
		 * The modification date is not yet updated.
		 */
		Annotation.prototype.setBeingModified = function (modifiedContent) {
			if (modifiedContent === undefined) {
				modifiedContent = null;
			}

			if (this.modifiedContent !== modifiedContent) {
				this._modifications.add("CONTENT");
				this.modifiedContent = modifiedContent;
				this.saved = false;
			}

		};

		/**
		 * Is the annotation being modified
		 * @returns {Boolean}
		 */
		Annotation.prototype.isBeingModified = function () {
			return !this.saved;
		};

		/**
		 * Run through an annotation to get all its children (replies and/or states)
		 * @param {Boolean} self - indicates if the callback function should also be called on the annotation itself
		 * @param {Boolean} replies - the callback will be called for every reply (and its replies)
		 * @param {Boolean} states - the callback will be called for every state
		 * @param {Function} callback - will be called (with the annotation) for every annotation that was found
		 */
		Annotation.prototype._runThroughAnnotation = function (annotation, replies, states, callback) {
			if (replies) {
				//making a shalow copy of the array makes it possible to remove it in the callback
				var annotationReplies = _.clone(annotation.replies);
				_.each(annotationReplies, _.bind(function (annot) {
					if (!callback(annot)) {
						this._runThroughAnnotation(annot, replies, states, callback);
					}
				}, this));
			}
			if (states) {
				var annotationStates = _.clone(annotation.states);
				_.each(annotationStates, _.bind(function (annot) {
					if (!callback(annot)) {
						this._runThroughAnnotation(annot, replies, states, callback);
					}
				}, this));
			}
		};

		/**
		 * Run through an annotation to get all its children (replies and/or states)
		 * @param {Boolean} self - indicates if the callback function should also be called on the annotation itself
		 * @param {Boolean} replies - the callback will be called for every reply (and its replies)
		 * @param {Boolean} states - the callback will be called for every state
		 * @param {Function} callback - will be called (with the annotation) for every annotation that was found
		 */
		Annotation.prototype.runThroughAnnotation = function (self, replies, states, callback) {
			if (self) {
				if (callback(this)) {
					return;
				}
			}
			this._runThroughAnnotation(this, replies, states, callback);
		};

		/**
		 * Makes an XML representation of the annotation
		 * @param {Document} document - the owner document that is needed to create nodes
		 * @returns {Element} element that represents the annotation (NOT appended to the document yet)
		 */
		Annotation.prototype.toXML = function (document) {
			/*
			<circle color="#FF0000" creationdate="D:20130608142640-04'00'" date="D:20130608223906-04'00'" flags="print" name="MfUwftyuPc4ZRLmCWavqpG" page="0" rect="116.65092240883808,193.72593822615414,247.69028039670718,291.9917703696743" subject="Oval" title="1 Cagnie, Ruben (RUCA)">
				<contents-richtext>
					<body>
						<p>Eggs</p>
					</body>
				</contents-richtext>
				<popup open="no" page="0" rect="255.2249117711603,158.17305818469373,405.22491177116024,270.67305818469373"/>
			</circle>
			*/
			var annot = document.createElementNS(null, AnnotationConstants.AnnotationElementName[this.annottype]);

			if (this.isNew && !this.linkedto) {
				annot.setAttribute('title', this.getUserName());
			} else {
				annot.setAttribute('title', this.messagenumber + " " + this.getUserName());
			}

			if (!this.name || this.name.length === 0) {
				this.f.emit("error.SEVERE", "Trying to convert an annotation without a name to XML");
			}

			annot.setAttribute('name', this.name);

			annot.setAttribute('color', this.annotationcolor);
			annot.setAttribute('creationdate', ViewerUtils.stringFromDate(this.creationdate));
			annot.setAttribute('date', ViewerUtils.stringFromDate(this.date));

			annot.setAttribute('flags', this.flags);

			annot.setAttribute('page', this.page);

			//rect="116.65092240883808,193.72593822615414,247.69028039670718,291.9917703696743"
			var rect = this.rect;
			if (this._scaling) {
				rect = new ShapeUtils.Rect(new ShapeUtils.Point(this.rect.origin.x * 1 / this._scaling.x, this.rect.origin.y * 1 / this._scaling.y), new ShapeUtils.Size(this.rect.size.width * 1 / this._scaling.x, this.rect.size.height * 1 / this._scaling.y));
			}
			annot.setAttribute('rect', ViewerUtils.rectToString(rect));

			annot.setAttribute('subject', this.subject);

			if (this.state) {
				annot.setAttribute('state', this.state);
			}
			if (this.statemodel) {
				annot.setAttribute('statemodel', this.statemodel);
			}
			if (this.inreplyto) {
				annot.setAttribute('inreplyto', this.inreplyto);
			}
			if (this.linkedto) {
				annot.setAttribute('linkedto', this.linkedto);
			}

			if (this._view3DPosition) {
				annot.setAttribute('view3Dscale', this._view3DPosition.scale);
				annot.setAttribute('view3Dazimuth', this._view3DPosition.azimuth);
				annot.setAttribute('view3Delevation', this._view3DPosition.elevation);
			}

			if (!this.inreplyto) {
				if (AnnotationConstants.AnnotationType.ANNOT_FREEFORM == this.annottype) {
					/*
			<inklist>
				<gesture>-285.60323374911894,598.6597735920734;-285.60323374911894,598.3248898662756;-286.94276865231024,596.4830293743876;-296.6543967004471,584.2597733827671;-302.84974562770685,575.0504709233269;-303.18462935350465,572.7062848427423;-302.51486190190906,570.0272150363597;-300.3381176842231,565.8411684638869;-297.15672228914383,562.8272149317065;-294.3102106198623,561.8225637543129;-291.29625708768197,559.6458195366272;-290.62648963608626,557.8039590447391;-290.62648963608626,554.1202380609631;-291.63114081347976,552.7807031577718;-294.64509434566014,550.2690752142881;-296.82183856334603,549.9341914884903;-304.0218386679992,547.7574472708044;-304.0218386679992,545.9155867789165;-303.18462935350465,536.8737261823752;-301.84509445031335,534.6969819646893;-294.64509434566014,524.6504701907547;-292.13346640217645,523.1434934246645;-289.4543965957939,519.9620980295851;-281.58462903954506,514.1016328281232;-278.4032336444658,513.0969816507296;-263.50090784646284,510.0830281185492;-255.9660240160117,510.0830281185492;-211.2590466220026,510.92023743304384;-196.3567208239996,512.7620979249318;-195.352069646606,514.436516553921;-195.352069646606,516.278377045809;-197.0264882755952,524.6504701907547;-198.53346504168533,527.1620981342382;-216.95206996056564,558.8086102221325;-221.64044212173508,568.6876801331682;-226.3288142829044,575.5527965120236;-231.6869538956696,581.5807035763844;-236.04044233104128,584.9295408343627;-249.43579136295418,593.971401430904;-254.12416352412376,595.813261922792;-258.4776519594954,596.4830293743876;-275.72416383808314,597.4876805517811;-276.89625687837554,596.4830293743876;-278.068349918668,594.3062851567017;-278.4032336444658,593.4690758422072;-278.4032336444658,593.4690758422072</gesture>
			</inklist>
				 */
					var inklist = document.createElementNS(null, "inklist");
					var gesture = document.createElementNS(null, "gesture");

					var pointarray = "";
					for (var i = 0; i < this.pointArray.length; i++) {
						var point = this.pointArray[i];
						if (i !== 0) {
							pointarray += ";";
						}
						if (this._scaling) {
							pointarray += GEOUtils.convertMMToPSPoints(point.x * 1 / this._scaling.x) + "," + GEOUtils.convertMMToPSPoints(point.y * 1 / this._scaling.y);
						} else {
							pointarray += GEOUtils.convertMMToPSPoints(point.x) + "," + GEOUtils.convertMMToPSPoints(point.y);
						}
					}
					gesture.textContent = pointarray;

					inklist.appendChild(gesture);
					annot.appendChild(inklist);
				} else if (AnnotationConstants.AnnotationType.ANNOT_LINE == this.annottype || AnnotationConstants.AnnotationType.ANNOT_ARROW == this.annottype) {
					//end="374.1086344293871,459.21259580886755" 
					//start="284.02491218977303,541.7614342180307"
					var startpoint = this.pointArray[0];
					var endpoint = this.pointArray[1];

					var rect;
					if (this._scaling) {
						rect = new ShapeUtils.Rect(new ShapeUtils.Point(rect.origin.x * 1 / this._scaling.x, rect.origin.y * 1 / this._scaling.y), new ShapeUtils.Size(rect.size.width * 1 / this._scaling.x, rect.size.height * 1 / this._scaling.y));
					} else {
						rect = new ShapeUtils.Rect(new ShapeUtils.Point(startpoint.x, endpoint.y), new ShapeUtils.Size(endpoint.x - startpoint.x, startpoint.y - endpoint.y));
					}
					annot.setAttribute('rect', ViewerUtils.rectToString(rect));

					var start = GEOUtils.convertMMToPSPoints(startpoint.x) + "," + GEOUtils.convertMMToPSPoints(startpoint.y);
					var end = GEOUtils.convertMMToPSPoints(endpoint.x) + "," + GEOUtils.convertMMToPSPoints(endpoint.y);
					if (this._scaling) {
						start = GEOUtils.convertMMToPSPoints(startpoint.x * 1 / this._scaling.x) + "," + GEOUtils.convertMMToPSPoints(startpoint.y * 1 / this._scaling.y);
						end = GEOUtils.convertMMToPSPoints(endpoint.x * 1 / this._scaling.x) + "," + GEOUtils.convertMMToPSPoints(endpoint.y * 1 / this._scaling.y);
					}
					annot.setAttribute('start', start);
					annot.setAttribute('end', end);

					if (this.annottype == AnnotationConstants.AnnotationType.ANNOT_ARROW) {
						annot.setAttribute('head', "None");
						annot.setAttribute('tail', "OpenArrow");
					}
				}
			}
			if (!this.linkedto || this.statemodel) {
				var content = document.createElement("contents-richtext");

				if (this.modifiedContent !== null) {
					this.content = this.modifiedContent;
				}
				if (this.content) {
					var markupElement = this._parseMarkupToXMLDocument(this.content);
					var bodyTag = markupElement.getElementsByTagName("body")[0];

					if (bodyTag) {
						content.appendChild(copyNode(bodyTag, document));
					} else {
						var newBody = document.createElement("body");
						// copy children
						if (markupElement.hasChildNodes && markupElement.hasChildNodes()) {
							for (var i = 0; i < markupElement.childNodes.length; i++) {
								var copy = copyNode(markupElement.childNodes[i], document);
								if (copy != null) {
									newBody.appendChild(copy);
								}
							}
						}
						content.appendChild(newBody);
					}
				}

				annot.appendChild(content);

			}


			if (this.popuprect.origin.x === 0 && this.popuprect.origin.y === 0) {
				this.popuprect.origin.x = this.rect.right();
				this.popuprect.origin.y = this.rect.top();
			}

			var popup = document.createElementNS(null, "popup");
			popup.setAttribute('rect', ViewerUtils.rectToString(this.popuprect));
			popup.setAttribute('page', this.popuppage);
			popup.setAttribute('open', this.popupIsOpen ? 'yes' : 'no');
			annot.appendChild(popup);

			return annot;
		};


		/**
		 * This helper function copies a node and al its attributes and children to a new node (that is created in the document given)
		 *
		 * In this way, copy without the conflicts of namespaces is possible
		 * Only works for TextNode and Element (otherwise null is returned)
		 * @param {Node} node - the node that needs to be copied
		 * @param {Document} document - the document in which the copy should be created
		 * @returns {Node} the copied node or null if it is not a text or element node
		 */
		function copyNode(node, document) {
			var newElement = null;

			if (node.nodeType === Node.TEXT_NODE) {
				newElement = document.createTextNode(node.textContent);
			} else if (node.nodeType === Node.ELEMENT_NODE) {
				newElement = document.createElement(node.nodeName.toLowerCase());

				// copy attributes if available
				if (node.hasAttributes && node.hasAttributes()) {
					for (var i = 0; i < node.attributes.length; i++) {
						newElement.setAttribute(node.attributes[i].name, node.attributes[i].value);
					}
				}

				// copy children
				if (node.hasChildNodes && node.hasChildNodes()) {
					for (var i = 0; i < node.childNodes.length; i++) {
						var childCopy = copyNode(node.childNodes[i], document);
						if (childCopy != null) {
							newElement.appendChild(childCopy);
						}
					}
				}
			}

			return newElement;
		}

		/**
		 * This function can parse an xhtml string in to a Document
		 *
		 * If DOMParser(text/html) is available, this is used, otherwise a workarround  is used
		 * (works in Chrome/Firefox/IE9 and up)
		 *
		 * @param {String} markup
		 * @returns {DocumentElement} the documentelement of the parsed markup
		 */
		Annotation.prototype._parseMarkupToXMLDocument = function (markup) {
			// replace non-breaking spaces with normal spaces, otherwise they will get lost in the parsers...
			markup = markup.replace(/\&nbsp;/g, ' ');

			// Firefox/Opera/IE throw errors on unsupported types
			try {
				// WebKit returns null on unsupported types
				if ((new DOMParser).parseFromString("", "text/html")) {
					// text/html parsing is natively supported
					var parser = new DOMParser();
					var markupDoc = parser.parseFromString(markup, "text/html");

					// on iOS when running the viewer in a UIWebView, documentElement is sometimes null
					// this is a very obscure bug by Apple
					// if documentElement is not defined/null, we fall back to method using document.innerHTML (after the try catch)
					if (markupDoc && markupDoc.documentElement) {
						return markupDoc.documentElement;
					}
				}
			} catch (ex) {}

			// the native DOMParser is not available... 
			// create HTMLDocument
			var doc = document.implementation.createHTMLDocument("");

			// find the contents of the <body> tag (or <html>-tag is no body tag is found)
			var matches = /<body>(.*)<\/body>/.exec(markup);
			if (matches && matches.length >= 1) {
				markup = matches[1];
			}

			matches = /<html>(.*)<\/html>/.exec(markup);
			if (matches && matches.length >= 1) {
				markup = matches[1];
			}


			doc.body.innerHTML = markup;

			return doc.documentElement;
		};

		/**
		 * Returns the state-annotation object that is for a given username
		 * @param {String} username
		 * @returns {Annotation} state annotation object that is used for the given user
		 */
		Annotation.prototype.getUserStateAnnotation = function (username) {
			return _.find(this.states, function (state) {
				return state.username === username;
			});
		};

		/**
		 * @param {Number} max - Maximum number of characters
		 * @returns {String} the content as a string (without HTML tags)
		 */
		Annotation.prototype.getStringContent = function (max) {
			var c = this.content.replace(/<([^<]*)>/g, "").trim();
			if (max && c.length > max) {
				c = c.substring(0, max) + "...";
			}
			return c;
		};

		/**
		 * Set the color of an annotation (and update all its links/replies/states)
		 * @param {String} color - new color of the annotation
		 */
		Annotation.prototype.setColor = function (color) {
			if (color === null) {
				color = this._getDefaultAnnotationColor(this.annottype);
			}
			this.runThroughAnnotation(true, true, true, function (annot) {
				if (annot.annotationcolor != color) {
					annot.annotationcolor = color;
					annot.color = color;
					annot.setModified("COLOR");
				}
			});
			var p = this.getGodfather();
			_.each(p.links, function (annot) {
				if (annot.annotationcolor != color) {
					annot.annotationcolor = color;
					annot.color = color;
					annot.setModified("COLOR");
				}
			});
		};

		/**
		 * Cancel the modified indication for color (also do this for all the links that are effected when doing color modification
		 */
		Annotation.prototype.cancelModifiedColor = function () {
			this.cancelModified("COLOR");
			this.runThroughAnnotation(true, true, true, function (annot) {
				annot.cancelModified("COLOR");
			});
			var p = this.getGodfather();
			_.each(p.links, function (annot) {
				annot.cancelModified("COLOR");
			});
		};

		/**
		 * Reset the annotation color to the last saved one
		 */
		Annotation.prototype.resetAnnotationColor = function () {
			if (this.originalAnnotationColor) {
				this.setColor(this.originalAnnotationColor);
			}
		};

		/**
		 * Get the godfather of this annotation (meaning the most main annotation)
		 * @returns {Annotation} - Godfather of this annotation
		 */
		Annotation.prototype.getGodfather = function () {
			var mainAnnotation = this;
			while (mainAnnotation.parent) {
				mainAnnotation = mainAnnotation.parent;
			}
			return mainAnnotation;
		};

		/**
		 * Get the type of the annotation
		 * @returns {AnnotationModule.AnnotationConstants.AnnotationType} type of the annotation
		 */
		Annotation.prototype.getType = function () {
			return this.annottype;
		};

		/**
		 * returns the text content of the annotation
		 * @return {String}
		 */
		Annotation.prototype.getTextContent = function () {
			return _.clone(this.content);
		};

		/**
		 * returns the message number of this annotation
		 * @return {Number}
		 */
		Annotation.prototype.getMessageNumber = function () {
			return this.messagenumber;
		};

		/**
		 * Set the content of the reply (but don't save it yet)
		 * @param {String} text
		 */
		Annotation.prototype.setUnsavedReply = function (text) {
			this.setModified("REPLY");
			this.unsavedReplyText = text;
			if (text === "") {
				this.cancelModified("REPLY");
			}
		};

		/**
		 * Reset the value of the unsaved reply text
		 */
		Annotation.prototype.resetUnsavedReply = function () {
			this.unsavedReplyText = null;
			this.cancelModified("REPLY");
		};

		/**
		 * Get the unsaved reply text
		 * @returns {String} the value of the unsaved reply
		 */
		Annotation.prototype.getUnsavedReply = function () {
			return this.unsavedReplyText;
		};

		/**
		 * Check if the annotation has an unsaved reply
		 * @returns {Boolean} true if there is an unsaved reply
		 */
		Annotation.prototype.hasUnsavedReply = function () {
			return this.unsavedReplyText !== null && this.unsavedReplyText !== "";
		};

		/**
		 * Is this annotation used as a state object
		 * @returns {Boolean}
		 */
		Annotation.prototype.isStateObject = function () {
			return this.state !== null;
		};

		/**
		 * Set the scaling of the annotation, all points of the annotation (linework) are scaled witht his value
		 * @param {ShapeUtils.Point|null} scaling
		 */
		Annotation.prototype.setScale = function (scaling) {
			this.scale(scaling);
			this._scaling = scaling;
		};

		/**
		 * Is the annotation scaled
		 * @returns {Boolean}
		 */
		Annotation.prototype.isScaled = function () {
			return this._scaling !== null && this._scaling !== undefined;
		};

		/**
		 * Get the scaling for this annotation
		 * @returns {ShapeUtils.Point|null}
		 */
		Annotation.prototype.getScale = function () {
			return this._scaling;
		};

		/**
		 * Returns the default annotation color for the given annotation type
		 * @param   {AnnotationConstants.AnnotationType} annotType
		 * @returns {String}
		 */
		Annotation.prototype._getDefaultAnnotationColor = function (annotType) {
			if (annotType == AnnotationConstants.AnnotationType.ANNOT_PIN) {
				return this.f.getCurrentUser().getColor("PREFERENCES_ANNOTATION_NOTE_COLOR");
			} else {
				return this.f.getCurrentUser().getColor("PREFERENCES_ANNOTATION_RECTANGLE_COLOR");
			}
		};

		/**
		 * parses xml and puts it in the contents
		 * @param {NodeList} xml - the full annotation XML
		 */
		Annotation.prototype._parseContentsFromXML = function (xml) {
			var contentEls = xml.getElementsByTagName('contents-richtext');

			this.content = "<p></p>";
			if (contentEls.length > 0) {
				var el = contentEls[0].getElementsByTagName("body")[0];
				if (!el) {
					el = contentEls[0];
				}

				if (typeof window.XMLSerializer != "undefined") {
					this.content = (new XMLSerializer()).serializeToString(el);
				} else if (typeof el.xml != "undefined") {
					this.content = el.xml; // internet explorer doesn't have XMLSerializer
				}
			}
		};

		/**
		 * Object with extra info to show before content or null
		 * @example Object {
		 *           element: myJQueryElement
		 *           tooltip: a tooltip for the element
		 * 			}
		 * @returns {Object}
		 */
		Annotation.prototype.getExtraContent = function () {
			return null;
		};


		/**
		 * Save the 3D View position in this annotation
		 * @param {Object} position 
		 *                 @example 
		 *                   {scale: 1.5,
		 *                   azimuth: 46.6,
		 *                   elevation: -52}
		 */
		Annotation.prototype.set3DViewPosition = function (position) {
			this._view3DPosition = position;
			this.setModified("3DVIEW");
		};

		/**
		 * Get the 3D View camera position of this annotation or null if there is none
		 * @returns {Object} position 
		 *                 @example 
		 *                   {scale: 1.5,
		 *                   azimuth: 46.6,
		 *                   elevation: -52}
		 */
		Annotation.prototype.get3DViewPosition = function () {
			return this._view3DPosition;
		};

		return Annotation;
	});

define('web-viewer/annotation/AnnotationFilterModel2',["web-viewer/core/PrivateChannel",
		"web-viewer/annotation/AnnotationConstants",
		"web-viewer/core/AbstractModel",
		"underscore",
		"jquery"
	],
	function (
		PrivateChannel,
		AnnotationConstants,
		AbstractModel,
		_,
		$
	) {
		"use strict";

		/**
		 * Model specific for the annotations filter
		 * @exports AnnotationFilterModel
		 * @memberof AnnotationModule
		 * @class
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f - Viewer facade
		 */
		var AnnotationFilterModel = function (f) {
			AbstractModel.call(this, f, "annotationFilterModel");
			this.f = f;

			/**
			 * @private
			 * @member {Array.<Object>} collection of all the filter rows
			 */
			this._filterRows = [];

			/**
			 * @private
			 * @member {AnnotationConstants.FilterOperators} the operator the combine different filter rows
			 */
			this._filterOperator = AnnotationConstants.FilterOperators.ALL;

			/**
			 * @private
			 * @member {Boolean} is the custom filter enabled or not
			 */
			this._isCustomFilterEnabled = false;

			/**
			 * @private
			 * @member {String|null} the selected user in the default filter
			 */
			this._selectedUser = null;

			/**
			 * @private
			 * @member {String|null} the selected state in the default filter
			 */
			this._selectedState = null;

			/**
			 * @private
			 * @member {Boolean} is the review state filter in the reverse state
			 */
			this._inverseState = false;

			/**
			 * @private
			 * @member {Number} the number of annotations after applying the filter
			 */
			this._numberOfFilteredAnnotations = 0;

			/**
			 * @private
			 * @member {String} the origin of annotation to filter on
			 */
			this._originType = AnnotationConstants.AnnotationOriginTypes.user.id;

			var viewerUserFilter = this.f.getProperty("showuser");
			if (viewerUserFilter) {
				this._viewerUserFilter = viewerUserFilter.split("|");
			} else {
				this._viewerUserFilter = [];
			}

			/**
			 * @private
			 * @member {Boolean} is the pinned state of the annotation filter enabled or not
			 */
			this._annotationFilterPinnedState = this.f.getOption("ANNOTATION_FILTER") !== undefined && this.f.getOption("ANNOTATION_FILTER") !== "";

			//if the pinned state is enabled, the option is set load the filter from this option
			if (this._annotationFilterPinnedState) {
				this.setFilterState(this.f.getOption("ANNOTATION_FILTER"));
			}
		};

		/**
		 * reset the annotation filter
		 */
		AnnotationFilterModel.prototype.resetFilter = function () {
			this._selectedUser = null;
			this._selectedState = null;
			this._inverseState = false;
			this._originType = AnnotationConstants.AnnotationOriginTypes.user.id;
			this._filterRows = [];
			this._filterRows.push(this.getDefaultFilterRow());
			this._filterOperator = AnnotationConstants.FilterOperators.ALL;
			this._isCustomFilterEnabled = false;
			this._numberOfFilteredAnnotations = 0;
			this._emit("filterReset", this);
			this._emit("filterChanged", this);
		};

		/**
		 * set the user to filter on
		 * @param {String|null} user
		 */
		AnnotationFilterModel.prototype.setSelectedUser = function (user) {
			this._selectedUser = user;
			this._isCustomFilterEnabled = false;
			this._emit("filterChanged", this);
			this.fireFilterStateChanged();
		};

		/**
		 * set the annotation origin type to filter on
		 * @param {String} type
		 */
		AnnotationFilterModel.prototype.setOriginType = function (type) {
			if (type !== this._originType) {
				this._originType = type;
				this._isCustomFilterEnabled = false;
				this._emit("filterChanged", this);
				this.fireFilterStateChanged();
			}
		};

		/**
		 * get the currently selected user
		 * @returns {String}
		 */
		AnnotationFilterModel.prototype.getSelectedUser = function () {
			return this._selectedUser;
		};

		/**
		 * get the currently selected origin
		 * @returns {String}
		 */
		AnnotationFilterModel.prototype.getOriginType = function () {
			return this._originType;
		};


		/**
		 * set the annotation state to filter on
		 * @param {String|null} state
		 * @param {Boolean} inverse - is it an inverse state, every but the state
		 */
		AnnotationFilterModel.prototype.setSelectedState = function (state, inverse) {
			this._selectedState = state;
			this._inverseState = inverse;
			this._isCustomFilterEnabled = false;
			this._emit("filterChanged", this);
			this.fireFilterStateChanged();
		};

		/**
		 * get the currectly selected state
		 * @returns {String}
		 */
		AnnotationFilterModel.prototype.getSelectedState = function () {
			return this._selectedState;
		};

		/**
		 * Is the state the inverse state
		 * @returns {Boolean}
		 */
		AnnotationFilterModel.prototype.isInverseState = function () {
			return this._inverseState;
		};

		/**
		 * Add a filter row to the annotation filters
		 *
		 * @param {Number} filterRowId
		 * @param {AnnotationConstants.FilterTypes} filterType
		 * @param {AnnotationConstants.FilterCondition} filterCondition
		 * @param {String} filterOption
		 */
		AnnotationFilterModel.prototype.addFilterRow = function (filterRowId, filterType, filterCondition, filterOption) {
			var filterRow = {
				filterId: filterRowId,
				filterType: filterType,
				filterCondition: filterCondition,
				filterOption: filterOption
			};

			this._filterRows[filterRowId] = filterRow;
		};

		/**
		 * Add the default row to the annotation filter
		 * @returns {Number}
		 */
		AnnotationFilterModel.prototype.addEmptyRow = function () {
			this._filterRows.push(this.getDefaultFilterRow());
			this._emit("filterChanged", this);
			return this._filterRows.length - 1;
		};

		/**
		 * Update the filter type of a filter row, need to provide the default selected option for selected type
		 * @param {Number} filterId
		 * @param {AnnotationConstants.FilterTypes} filterType
		 * @param {String} defaultOption
		 */
		AnnotationFilterModel.prototype.updateFilterType = function (filterId, filterType, defaultOption) {
			if (this._filterRows[filterId]) {
				this._filterRows[filterId].filterType = filterType;
				this._filterRows[filterId].filterOption = defaultOption;
				this._emit("filterChanged", this);
			}
		};

		/**
		 * Update the filter option of a filter row
		 * @param {Number} filterId
		 * @param {String} filterOption
		 */
		AnnotationFilterModel.prototype.updateFilterOption = function (filterId, filterOption) {
			if (this._filterRows[filterId]) {
				this._filterRows[filterId].filterOption = filterOption;
				this._emit("filterChanged", this);
			}
		};

		/**
		 * Update the condition of a filter row
		 * @param {Number} filterId
		 * @param {AnnotationConstants.FilterCondition} filterCondition
		 */
		AnnotationFilterModel.prototype.updateFilterCondition = function (filterId, filterCondition) {
			if (this._filterRows[filterId]) {
				this._filterRows[filterId].filterCondition = filterCondition;
				this._emit("filterChanged", this);
			}
		};

		/**
		 * Remove a filter row from the model
		 * @param {Number} filterRowId
		 */
		AnnotationFilterModel.prototype.removeFilterRow = function (filterRowId) {
			delete this._filterRows[filterRowId];
			this._emit("filterChanged", this);
		};

		/**
		 * Remove all the filter rows from the model (including the default row)
		 */
		AnnotationFilterModel.prototype.removeAllFilterRows = function () {
			this._filterRows = [];
			this._emit("filterChanged", this);
		};

		/**
		 * Get the filter row information for the row with filterRowId
		 * @param   {Number} filterRowId
		 * @returns {Object} object with all the information of specific row
		 */
		AnnotationFilterModel.prototype.getFilterRow = function (filterRowId) {
			return this._filterRows[filterRowId];
		};

		/**
		 * Returns all the filter rows.
		 * @returns {Array.<Object>}
		 */
		AnnotationFilterModel.prototype.getFilterRows = function () {
			if (this._filterRows.length === 0) {
				this._filterRows.push(this.getDefaultFilterRow());
			}
			return _.without(this._filterRows, null, undefined);
		};

		/**
		 * Set all the filter rows
		 * @param {Array.<Object>} filterRows
		 */
		AnnotationFilterModel.prototype.setFilterRows = function (filterRows) {
			this._filterRows = filterRows;
			this._emit("filterChanged", this);
		};

		/**
		 * Get the default filter row (selected values are by default review state with option accepted)
		 * @returns {Object}
		 */
		AnnotationFilterModel.prototype.getDefaultFilterRow = function () {
			return {
				filterId: this._filterRows.length,
				filterType: AnnotationConstants.FilterTypes.REVIEWSTATE,
				filterCondition: AnnotationConstants.FilterCondition.IS,
				filterOption: "STATUS_ACCEPTED"
			};
		};

		/**
		 * Set the active state of the custom filter
		 * @param {Boolean} setActive
		 */
		AnnotationFilterModel.prototype.setCustomFilterActive = function (setActive) {
			this._isCustomFilterEnabled = setActive;
			if (setActive && this._filterRows.length === 0) {
				this._filterRows.push(this.getDefaultFilterRow());
			}
			this._emit("filterChanged", this);
		};

		/**
		 * Is the custom filter active
		 * @returns {Boolean}
		 */
		AnnotationFilterModel.prototype.isCustomFilterActive = function () {
			return this._isCustomFilterEnabled;
		};

		/**
		 * Set the filter operator.
		 * @param {AnnotationConstants.FilterOperators} filterOperator
		 */
		AnnotationFilterModel.prototype.setFilterOperator = function (filterOperator) {
			this._filterOperator = filterOperator;
			this._emit("filterChanged", this);
		};

		/**
		 * Returns the currently active filter operator.
		 * @returns {AnnotationConstants.FilterOperators}
		 */
		AnnotationFilterModel.prototype.getFilterOperator = function () {
			return this._filterOperator;
		};

		/**
		 * Set the number of filtered annotations.
		 * @param {Number} number
		 */
		AnnotationFilterModel.prototype.setNumberOfFilteredAnnotations = function (number) {
			this._numberOfFilteredAnnotations = number;
		};

		/**
		 * Get the number of filtered annotations.
		 * @returns {Number}
		 */
		AnnotationFilterModel.prototype.getNumberOfFilteredAnnotations = function () {
			return this._numberOfFilteredAnnotations;
		};

		/**
		 * Is the annotation filter pinned at this moment
		 * @returns {Boolean}
		 */
		AnnotationFilterModel.prototype.isAnnotationFilterPinned = function () {
			return this._annotationFilterPinnedState;
		};

		/**
		 * Toggle the annotation pinned state
		 * @param {Boolean} [isEnabled] set the pinned state to
		 */
		AnnotationFilterModel.prototype.toggleAnnotationPinnedState = function (isEnabled) {
			if (isEnabled !== undefined) {
				this._annotationFilterPinnedState = isEnabled;
			} else {
				this._annotationFilterPinnedState = !this._annotationFilterPinnedState;
			}

			if (this._annotationFilterPinnedState) {
				this.fireFilterStateChanged(); //fire a change in the filter state so it gets persisted
			} else {
				this._emit("filterStateChanged", false); //reset the persisted filter
			}
		};

		/**
		 * Get the state of the annotation filter
		 * @returns {String} JSON string to be saved as option
		 */
		AnnotationFilterModel.prototype.getFilterState = function () {
			var stateObj = {
				customFilterOperator: this._filterOperator,
				customFilterRows: _.compact(this._filterRows), //compact needed to remove falsy values (null) from the array
				isCustomFilterActive: this.isCustomFilterActive(),
				selectedUser: this._selectedUser,
				selectedState: this._selectedState,
				inverseState: this._inverseState
			};
			return JSON.stringify(stateObj);
		};

		/**
		 * Set the state of the annotation filter
		 * @param {String} filterState
		 */
		AnnotationFilterModel.prototype.setFilterState = function (filterState) {
			try {
				filterState = $.parseJSON(filterState);
				if (filterState) {
					this._isCustomFilterEnabled = filterState.isCustomFilterActive;
					this._filterOperator = filterState.customFilterOperator;
					this._filterRows = filterState.customFilterRows;
					_.each(this._filterRows, function (row, index) {
						row.filterId = index; //compensate for the compact in the get filter state
					}, this);
					this._selectedUser = filterState.selectedUser;
					this._selectedState = filterState.selectedState;
					this._inverseState = filterState.inverseState;
				}
			} catch (e) {
				// something wrong with the filter state
				// fall back to default filter
			}
			this._emit("filterChanged", this);
		};

		/**
		 * Fire a change in the filter state, this causes the filter state to be persisted
		 */
		AnnotationFilterModel.prototype.fireFilterStateChanged = function () {
			this._emit("filterStateChanged", true);
		};

		/**
		 * Get the viewer filter on users, only show annotations, replies and states of the users in this array
		 * @returns {Array.<String>}
		 */
		AnnotationFilterModel.prototype.getViewerUserFilter = function () {
			return this._viewerUserFilter;
		};

		return AnnotationFilterModel;
	});

define('web-viewer/annotation/AnnotationModel',["web-viewer/core/PrivateChannel",
		"web-viewer/annotation/AnnotationFilter",
		"web-viewer/annotation/Annotation",
		"web-viewer/annotation/AnnotationConstants",
		"web-viewer/annotation/AnnotationFilterModel2",
		"utils/ShapeUtils",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/layout/component/ModalPopup",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/layout/LayoutConstants",
		"utils/OrderedObject",
		"underscore"
	],
	function (PrivateChannel,
		AnnotationFilter,
		Annotation,
		AnnotationConstants,
		AnnotationFilterModel,
		ShapeUtils,
		ListenerUtil,
		ViewerUtils,
		ModalPopup,
		ViewerToolbarButton,
		LayoutConstants,
		OrderedObject,
		_) {
		"use strict";

		/**
		 * Constructs a new annotation model
		 * @exports AnnotationModel
		 * @memberof AnnotationModule
		 * @class
		 * The annotation model is the main model within the viewer that keeps track of the current annotations and their selection. The model has its main function "setAnnotations". It keeps track of the selected annotations
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {String} IID - Unique channel ID of the annotation model
		 */
		var AnnotationModel = function (f, IID) {
			PrivateChannel.installTo(this, f, ['annotationModel', IID]);
			ListenerUtil.mixin(this, f);

			/**
			 * Facade
			 */
			this.f = f;

			/**
			 * In here every reason to disable annotation editing is logged
			 * {
			 * 	reason: reason,
			 *	hideAnnotations: hideAnnotations,
			 *	dontRemove: dontRemove
			 *	}
			 */
			this._disabledReasons = new OrderedObject();

			/**
			 * The page too which the current annotations belong
			 * @private
			 * @type {Data.AbstractDocumentPage}
			 */
			this._page = null;

			/**
			 * Annotations of the main document
			 */
			this._annotations = [];
			/**
			 * Annotations that are removed by the user, but not yet saved
			 */
			this._removedannotations = [];

			/**
			 * Keeps the list of annotations that are selected
			 */
			this._selectedAnnotations = [];
			/**
			 * Keeps the list of the annotations that are filtered
			 */
			this._filteredAnnotations = [];

			/**
			 * for which layout type does the wiget need to be rendered
			 */
			this._layoutType = null;

			/**
			 * the model specific for the annotation filter
			 */
			this._annotationFilterModel = new AnnotationFilterModel(this.f);

			/**
			 * variable used for stepping through annotations
			 */
			this._orderedAnnotations = [];
			//index gets first increment before it is used, make sure the first one is index 0
			this._currentAnnotationIndex = -1;

			/**
			 *  The currently active scaling, all annotation are scaled with this scaling. This is used to initialize new annotations.
			 */
			this._currentScaling = null;

			/**
			 * Can be set to lock the selection of annotations
			 */
			this.lockSelection = false; // A tool/module can decide that the selection will be locked and take the ownership for deselecting annotations
			// This is needed to prevent 2 tools (e.g. AnnotationTool and AnnotationDrawTool to get in each other ways

			/**
			 * Listeners used in adding global annotation on load (from SDK) to make sure all of annotations are loaded.
			 */
			this._addAnnotationsEnabledListener = null;
			this._disabledReasonChangedListener = null;

			/**
			 * Indicates if the annotation table is visible
			 */
			this._annotationListVisible = false; // Is the list visible? Needed for visibility options of annotations & popups

			/**
			 * Indicates if the annotations are currently being saved.
			 */
			this._isSavingAnnotations = false;


			var currentUser = f.getCurrentUser();
			/**
			 * If the Sequence label visibility changes, redraw the annotations
			 */
			currentUser.on("options.changed.PREFERENCES_ANNOTATION_SEQLABEL_VISIBLE", _.bind(function (oldValue, newValue) {
				this.modelChanged();
			}, this));

			/**
			 * If the preferences changes, redraw the annotations
			 */
			currentUser.on("options.changed.PREFERENCES_ANNOTATION_NEXT_MODE", _.bind(function (oldValue, newValue) {
				this._orderedAnnotations.length = 0;
				this.modelChanged();
			}, this));

			// listen to the datesort mode
			currentUser.on("options.changed.PREFERENCES_ANNOTATION_DATESORT_MODE", _.bind(function (oldValue, newValue) {
				this.setSortingMode(newValue);
			}, this));

			currentUser.on("options.changed.PREFERENCES_ANNOTATION_SHOWSELECTEDONLY", _.bind(function () {
				this.modelChanged();
			}, this));

			this.setSortingMode(f.getOption("PREFERENCES_ANNOTATION_DATESORT_MODE"));

			/** 
			 * Variable that indicates if the annotations can be changed at all (e.g. depending on approval status)
			 */
			this._canChangeAnnotations = true;

			/**
			 * @private
			 * @type {AnnotationModule.AnnotationConstants.annotateMode}
			 *
			 */
			this._annotationmode = AnnotationConstants.annotateMode.NO_RESTRICTIONS;
			if (f.getProperty("annotationmode")) {
				this._annotationmode = parseInt(f.getProperty("annotationmode"), 10);
			}

			/**
			 * @private
			 * @type {AnnotationModule.AnnotationConstants.allowedAnnotators}
			 *
			 */
			this._allowedAnnotators = AnnotationConstants.allowedAnnotators.ALL;
			if (f.getProperty("annotatorsmode")) {
				this._allowedAnnotators = parseInt(f.getProperty("annotatorsmode"), 10);
			}


			if (this.editabilityDependsOnApproval()) {
				this._canChangeAnnotations = false;
				f.loadBundle("viewer", function () {
					this.setDisableReason("LOADING_APPROVAL", this.f.t("Loading"));
				}, this);
			}

			/**
			 * Variable that indicates if a warning should be shown if the annotations should changed
			 */
			this._warnWhenChangingAnnotations = false;

			/**
			 * Variable that remembers the confirmation of the warning
			 */
			this._confirmedOnce = false;

			/**
			 * Filter object that keeps track of the current filter
			 */
			this._filter = new AnnotationFilter(f, this._annotationFilterModel);

			/**
			 * If the filter changes, refilter all the annotations
			 */
			this._annotationFilterModel.on("filterChanged", _.bind(function (filter) {
				this._filterAllAnnotations();
				this._orderedAnnotations = [];
			}, this));

			/**
			 * The annotations can be temporarily disabled with a certain reason
			 * the reason is shown in the annotation list
			 * if enableAnnotationToggling is true, buttons to show either current document or reference anntoations are available
			 * if showingDifferenceAnnotations is true then the annotations of the differnce document are shown, these cannot be changed
			 */
			this._temporarilyDisabled = {
				isTemporarilyDisabled: false,
				reason: "",
				enableAnnotationToggling: false,
				showingCurrentAnnotations: false,
				showingDifferenceAnnotations: false
			};

		};

		/**
		 * Sets the variable that indicates if the annotation table is visible
		 * @param {Boolean} annotationListVisible
		 */
		AnnotationModel.prototype.setAnnotationListVisible = function (annotationListVisible) {
			if (this._annotationListVisible !== annotationListVisible) {
				this._annotationListVisible = annotationListVisible;
				this._emit("annotationListVisible", annotationListVisible);
			}
		};

		/**
		 * Is the annotation list/table visible?
		 * Used by e.g. AnnotationPopupsLayerModel to show the popups based on the visible state
		 * @returns {Boolean} True if the annotation list/table is visible
		 */
		AnnotationModel.prototype.isAnnotationListVisible = function () {
			return this._annotationListVisible;
		};

		/**
		 * Get the current ordering function
		 * @returns {Function} function to order the annotations
		 */
		AnnotationModel.prototype.currentOrdening = function () {
			return this.orderByCreationDate;
		};

		/**
		 * Toggle the annotation visibility
		 * @param {Boolean} [toggle] - force visibility
		 */
		AnnotationModel.prototype.toggleVisibility = function (toggle) {
			if (toggle === undefined) {
				toggle = this.f.getOption("PREFERENCES_ANNOTATIONTOOLS_VISIBLE") == 1;
				toggle = !toggle;
			} else if (toggle && this.f.getOption("PREFERENCES_ANNOTATIONTOOLS_VISIBLE") == 1) { // yes! we compare with == and not with === (we are not sure what is in the preference (true, false)
				// nothing changed
				return;
			}

			if (!toggle) {
				this.f.setOption("PREFERENCES_ANNOTATIONTOOLS_VISIBLE", 0);
				this.unselectAllAnnotations();
			} else {
				this.f.setOption("PREFERENCES_ANNOTATIONTOOLS_VISIBLE", 1);
			}
			this.modelChanged();
		};

		/**
		 * Temporarily disable the module
		 * @param {Boolean} temporarilyDisable - if true, annotations are temporarily disabled (this setting is not saved)
		 * @param {Boolean} nableTogglingAnnotations - if true the user will have the option to toggle annotation on and off
		 * @param {String} reason - the reason why annotation is temporarily disabled, this is displayed to the user
		 * @param {String} [whichToEnable] - [difference|current] if provided, either the difference or current annotations will be shown
		 */
		AnnotationModel.prototype.temporarilyDisable = function (temporarilyDisable, enableTogglingAnnotations, reason, whichToEnable) {
			this._temporarilyDisabled.isTemporarilyDisabled = temporarilyDisable;
			this._temporarilyDisabled.reason = reason;
			this._temporarilyDisabled.enableAnnotationToggling = enableTogglingAnnotations;

			this.unselectAllAnnotations();

			if (whichToEnable !== undefined) {
				// if which to enable is set, immediately show these temporary annotations
				if (whichToEnable == "difference") {
					this.showTemporarilyDifferencePageAnnotations();
				} else if (whichToEnable == "current") {
					this.showTemporarilyActivePageAnnotations();
				}
				this.f.setOption("PREFERENCES_ANNOTATIONTOOLS_VISIBLE", 1, true);
			} else {
				// just disable annotations, don't show anything special
				this._temporarilyDisabled.showingCurrentAnnotations = false;
				this._temporarilyDisabled.showingDifferenceAnnotations = false;
			}


			this.updateTemporarilyDisabledReason();

			this._emit("temporarilyDisableChanged");

			if (!temporarilyDisable) {
				this.setPage(this.f.getActivePage());
				this._temporarilyDisabled.showingCurrentAnnotations = true;
			}
			this.modelChanged();
		};

		/**
		 * Register the disable reason
		 */
		AnnotationModel.prototype.updateTemporarilyDisabledReason = function () {
			this.f.loadBundle("viewer", function () {
				if (this.isTemporarilyDisabled() && !this.isTemporarilyCurrentPageAnnotationsShown()) {
					this.setDisableReason("TEMPORARY_DISABLE", this._temporarilyDisabled.reason);

					if (this.isTemporarilyDifferencePageAnnotationsShown()) {
						this.setDisableReason("TEMPORARY_DISABLE", this.f.t("AnnDisabledDiffMode"));
					}
				} else {
					this.removeDisableReason("TEMPORARY_DISABLE");
				}


			}, this);
		};

		/**
		 * Check wether the annotation module is temporarily disabled and no annotations are visible
		 * @returns {Boolean}
		 */
		AnnotationModel.prototype.isTemporarilyDisabledAndNoAnnotationsVisible = function () {
			return (this._temporarilyDisabled.isTemporarilyDisabled &&
					!this._temporarilyDisabled.showingCurrentAnnotations &&
					!this._temporarilyDisabled.showingDifferenceAnnotations) ||
				!this.isAllowedToShowAnnotations();
		};

		/**
		 * Are annotations temporarily disabled, only checks if there are temporarily disabled reason, not other reasons!
		 * @returns {Boolean}
		 */
		AnnotationModel.prototype.isTemporarilyDisabled = function () {
			return this._temporarilyDisabled.isTemporarilyDisabled;
		};

		/**
		 * Are annotations temporarily disabled and the user cannot edit
		 * @returns {Boolean}
		 */
		AnnotationModel.prototype.isTemporarilyDisabledAndCannotEdit = function () {
			return (this._temporarilyDisabled.isTemporarilyDisabled &&
					!this._temporarilyDisabled.showingCurrentAnnotations) ||
				!this.isAllowedToShowAnnotations();
		};
		/**
		 * Get the mssage to be shown when no annotations are visible because of temporarily disable
		 * @returns {String}
		 */
		AnnotationModel.prototype.getTemporarilyDisabledMessage = function () {
			return this._temporarilyDisabled.reason;
		};

		/**
		 * Is the annotation module temporarily disabled and the annotations for the current document are shown
		 * @returns {Boolean}
		 */
		AnnotationModel.prototype.isTemporarilyCurrentPageAnnotationsShown = function () {
			return this._temporarilyDisabled.showingCurrentAnnotations &&
				this.isAllowedToShowAnnotations();
		};

		/**
		 * True if annotations for the difference document are shown
		 * @returns {Boolean}
		 */
		AnnotationModel.prototype.isTemporarilyDifferencePageAnnotationsShown = function () {
			return this._temporarilyDisabled.showingDifferenceAnnotations &&
				this.isAllowedToShowAnnotations();
		};

		/**
		 * Can the user re-enable annotations while the annotationmodule is temporarily disabled
		 * @returns {Boolean}
		 */
		AnnotationModel.prototype.isTemporarilyAnnotationDocumentTogglingEnabled = function () {
			return this._temporarilyDisabled.enableAnnotationToggling &&
				this.isAllowedToShowAnnotations();
		};

		/**
		 * Checks if there are disable reasons that don't allow to show annotations at all
		 * @returns {Boolean}
		 */
		AnnotationModel.prototype.isAllowedToShowAnnotations = function () {
			return !_.some(this._disabledReasons.getItemsAsObject(), function (item) {
				return item.hideAnnotations;
			});
		};

		/**
		 * Temporarily show the annotations for the difference document, only if there are no reasons that don't allow showing annotations
		 */
		AnnotationModel.prototype.showTemporarilyDifferencePageAnnotations = function () {
			if (this.isAllowedToShowAnnotations()) {
				var differencePage = this.f.getDocumentController().getDifferencePage();
				this.unselectAllAnnotations();

				this._temporarilyDisabled.showingDifferenceAnnotations = true;
				this._temporarilyDisabled.showingCurrentAnnotations = false;
				this.updateTemporarilyDisabledReason();

				this.setPage(differencePage, _.bind(function () {
					this.updateTemporarilyDisabledReason();
					this._emit("temporarilyDisableChanged");
					this.modelChanged();
				}, this));
			}
		};

		/**
		 * Temporarily show the annotations for the active document
		 */
		AnnotationModel.prototype.showTemporarilyActivePageAnnotations = function () {
			if (this.isAllowedToShowAnnotations()) {
				var activePage = this.f.getActivePage();
				this.unselectAllAnnotations();
				this.setPage(activePage, _.bind(function () {
					this._temporarilyDisabled.showingDifferenceAnnotations = false;
					this._temporarilyDisabled.showingCurrentAnnotations = true;

					this.updateTemporarilyDisabledReason();
					this._emit("temporarilyDisableChanged");
					this.modelChanged();
				}, this));
			}
		};

		/**
		 * Hide the temporarily shown annotations
		 */
		AnnotationModel.prototype.hideTemporarilyAnnotations = function () {
			this._temporarilyDisabled.showingDifferenceAnnotations = false;
			this._temporarilyDisabled.showingCurrentAnnotations = false;
			this.setPage(this.f.getDocumentController().getActivePage());

			this.updateTemporarilyDisabledReason();
			this._emit("temporarilyDisableChanged");
			this.modelChanged();
		};

		/**
		 * Get the page for which the annotations are visible
		 * @returns {Data.AbstractDocumentPage}
		 */
		AnnotationModel.prototype.getAnnotationsPage = function () {
			return this._page;
		};

		/**
		 * Check whether editability of the annotations depends on approval (based on annotation mode in WCR settings)
		 * @returns true if the annotation edit ability depends on the approval state
		 */
		AnnotationModel.prototype.editabilityDependsOnApproval = function () {
			return (this._annotationmode === AnnotationConstants.annotateMode.IF_APPROVED_SHOW_WARNING ||
				this._annotationmode === AnnotationConstants.annotateMode.IF_APPROVED_DISABLE ||
				this._annotationmode === AnnotationConstants.annotateMode.IF_APPROVED_OR_REJECTED_SHOW_WARNING ||
				this._annotationmode === AnnotationConstants.annotateMode.IF_APPROVED_OR_REJECTED_DISABLE ||
				this._allowedAnnotators !== AnnotationConstants.allowedAnnotators.ALL ||
				this._annotationmode === AnnotationConstants.annotateMode.ALWAYS_DISABLE);
		};

		/**
		 * Can the current user change the annotations?
		 * @param {Boolean} [dontConfirm] - if set to false, a warning message could be displayed
		 * @returns {Boolean} true if the annotations are editable
		 */
		AnnotationModel.prototype.canChangeAnnotations = function (dontConfirm) {
			if (this.isTemporarilyDisabledAndCannotEdit() ||
				this._cannotChangeAnnotationsOfCurrentPage() ||
				!this._canChangeAnnotations) {
				return false;
			} else if (!this._confirmedOnce && this._warnWhenChangingAnnotations && !dontConfirm) {
				var key = this._page.isApproved() ? "AnnAlreadyApproved" : "AnnAlreadyRejected";
				this._confirmedOnce = window.confirm(this.f.i18n.t(key));
				return this._confirmedOnce;
			} else {
				return true;
			}
		};

		/**
		 * Can the given annotation be changed
		 * @param {Annotation} annotation - The annotation to be checked
		 * @returns {Boolean} True if the current user can change the given annotation, false otherwise
		 */
		AnnotationModel.prototype.canChangeAnnotation = function (annotation) {
			if (this.isTemporarilyDisabledAndCannotEdit() || this._cannotChangeAnnotationsOfCurrentPage()) {
				return false;
			}

			var access = AnnotationConstants.AccessRight.READ_ONLY;
			if (this._canChangeAnnotations) {
				access = annotation.getPermission();
			}
			return access == AnnotationConstants.AccessRight.READ_WRITE;
		};

		/**
		 * Can the annotations NOT be changed of the current page.
		 * This is false if the page is not part of the latest document version or because of e.g. pharma settings
		 */
		AnnotationModel.prototype._cannotChangeAnnotationsOfCurrentPage = function () {
			return this._page && (!this._page.getDocumentVersion().isLatestVersion() || !this._page.canChangeAnnotations());
		};

		/**
		 * Check if an annotation can be deleted
		 * @param {Annotation} annotation - the annotation to be deleted
		 * @returns {Boolean}
		 */
		AnnotationModel.prototype.canDeleteAnnotation = function (annotation) {
			if (this.f.getProperty("annotationblockdeletesaved") && !annotation.isNew) {
				return false;
			} else {
				return this.canChangeAnnotation(annotation);
			}
		};

		/**
		 * Update permissions for adding/changing annotations based on the approval setup
		 */
		AnnotationModel.prototype.updateApprovalInfo = function () {
			if (this._page) {
				this.f.loadBundle("viewer", function () {
					var oldCanChangeAnnotations = this._canChangeAnnotations;

					this._canChangeAnnotations = true;
					this._warnWhenChangingAnnotations = false;
					var message = "";


					if (this._annotationmode === AnnotationConstants.annotateMode.ALWAYS_DISABLE) {
						this._canChangeAnnotations = false;
						message = this.f.t("AnnDisabled");
					} else {

						// check default approval settings
						if (this._page.isApproved()) {
							// document is approved
							if (this._annotationmode === AnnotationConstants.annotateMode.IF_APPROVED_SHOW_WARNING) {
								this._canChangeAnnotations = true;
								this._warnWhenChangingAnnotations = true;
							} else if (this._annotationmode === AnnotationConstants.annotateMode.IF_APPROVED_DISABLE) {
								this._canChangeAnnotations = false;
								this._warnWhenChangingAnnotations = false;
								message = this.f.t("AnnAlreadyApprovedBlocked");
							}
						}

						if (this._page.isApproved() || this._page.isRejected()) {
							// document is approved or rejected
							if (this._annotationmode === AnnotationConstants.annotateMode.IF_APPROVED_OR_REJECTED_SHOW_WARNING) {
								this._canChangeAnnotations = true;
								this._warnWhenChangingAnnotations = true;
							} else if (this._annotationmode === AnnotationConstants.annotateMode.IF_APPROVED_OR_REJECTED_DISABLE) {
								this._canChangeAnnotations = false;
								this._warnWhenChangingAnnotations = false;

								if (this._page.isApproved()) {
									message = this.f.t("AnnAlreadyApprovedBlocked");
								} else {
									message = this.f.t("AnnAlreadyRejectedBlocked");
								}
							}
						}

					}

					if (this._canChangeAnnotations && this._page.isUnderApproval()) {
						// the approval has started check the annotators mode
						if (this._allowedAnnotators !== AnnotationConstants.allowedAnnotators.ALL) {
							// check specific security setting
							if (this._allowedAnnotators === AnnotationConstants.allowedAnnotators.ALL_APPROVERS) {
								if (!this._page.isUserApprover()) {
									// the user is not an approver, so he can't annotate
									this._canChangeAnnotations = false;
									message = this.f.t("AnnNotApproverBlocked");
								}
							} else if (this._allowedAnnotators === AnnotationConstants.allowedAnnotators.ALL_APPROVERS_AND_MASTER) {
								if (!this._page.isUserApprover() && !this._page.isUserApprovalMaster(_.bind(this.updateApprovalInfo, this))) {
									// the user is not an approver, nor an approval master, so he can't annotate
									this._canChangeAnnotations = false;

									message = this.f.t("AnnNotApproverOrMasterBlocked");
								}
							} else if (this._allowedAnnotators === AnnotationConstants.allowedAnnotators.PENDING_APPROVERS) {
								if (!this._page.isUserPendingApprover()) {
									// the user is not a pending approver, so he can't annotate
									this._canChangeAnnotations = false;
									message = this.f.t("AnnNotPendingApproverBlocked");
								}
							} else if (this._allowedAnnotators === AnnotationConstants.allowedAnnotators.PENDING_APPROVERS_AND_MASTER) {
								if (!this._page.isUserPendingApprover() && !this._page.isUserApprovalMaster(_.bind(this.updateApprovalInfo, this))) {
									// the user is not a pending approver, nor an approval master, so he can't annotate
									this._canChangeAnnotations = false;

									message = this.f.t("AnnNotPendingApproverOrMasterBlocked");
								}
							}
						}
					}

					if (message !== "") {
						this.setDisableReason("ANNOTATORS_MODE", message);
						if (this.getUnsavedAnnotationCount() > 0) {
							this.reload();
						}
					} else {
						this.removeDisableReason("ANNOTATORS_MODE");
					}

					// Actually changed
					if (oldCanChangeAnnotations != this._canChangeAnnotations) {
						this._emit("readWriteChanged");
					}


				}, this);
			}
		};

		/**
		 * Called by the module when approval updated
		 * @param {Boolean} success - If successfully loaded approval info.
		 */
		AnnotationModel.prototype.onApprovalUpdated = function (success) {

			this.removeDisableReason("LOADING_APPROVAL");
			this.updateApprovalInfo();
		};


		/**
		 * Set the annotations objects for this model
		 * @param {Array<Annotation>} annots - List of annotations
		 */
		AnnotationModel.prototype.setAnnotations = function (annots) {
			var annotList = this._filter.filterIncomingAnnotations(annots);
			this.scaleAllAnnotations(null);
			this._annotations = annotList;
			//TODO allow the reference page the compensated, the Java Viewer does not allow this, as it returns wrong parameter values
			if (this._page && this._page !== this.f.getDocumentController().getDifferencePage()) {
				this.scaleAllAnnotations(this._page.getCurrentScaling());
			}
			this._confirmedOnce = false;
			this.setSortingMode(this.f.getOption("PREFERENCES_ANNOTATION_DATESORT_MODE"), true);
			this._filterAllAnnotations(true);

			this.modelContentChanged();
		};

		/**
		 * @returns {Array.<Annotation>} annotations of the model (unfiltered)
		 */
		AnnotationModel.prototype.getAnnotations = function () {
			return this._annotations;
		};

		/**
		 * @returns {Array.<String>} origin types of the annotations
		 */
		AnnotationModel.prototype.getAnnotationOrigins = function () {
			return _.uniq(_.map(this._annotations, function (annotation) {
				return annotation.originType;
			}));
		};

		/**
		 * Get the number of annotations (that are not linked)
		 * @returns {Number}
		 */
		AnnotationModel.prototype.getAnnotationCount = function () {
			return _.filter(this._annotations, function (annotation) {
				return annotation.getGodfather() === annotation;
			}, this).length;
		};

		/**
		 * Filter all annotations to the active filter
		 * @private
		 * @param {Boolean} [silent] - if true, no events are send out
		 */
		AnnotationModel.prototype._filterAllAnnotations = function (silent) {
			var annotations = _.filter(this._annotations, function (annotation) {
				return annotation.getGodfather() === annotation;
			}, this);
			this._filteredAnnotations = this._filter.filterAnnotations(annotations);
			this.currentOrdening();

			if (!silent) {
				this.modelChanged();
				this.filterChanged();
			}
		};

		/**
		 * Get the list of users that have made an annotation + the current user
		 * @param {String} [originType] - type of origin to get the users of 
		 * @returns {Object} where key is the user name and the value is the full user name
		 * {
		 * 	"RUCA" : "Cagnie, Ruben (RUCA)"
		 *  "BAMT" : "Matte, Bart (BAMT)"
		 * }
		 */
		AnnotationModel.prototype.getUsers = function (originType) {
			var users = {};
			_.each(this._annotations, function userIterator(annotation) {
				if (!users[annotation.username] && (!originType || annotation.originType === originType)) {
					users[annotation.username] = annotation.getUserName();
				}

				if (annotation.replies.length > 0) {
					_.each(annotation.replies, userIterator);
				}
			});

			if (users[this.f.getCurrentUser().getUsername()] === undefined) {
				users[this.f.getCurrentUser().getUsername()] = this.f.getCurrentUser().getFullName();
			}

			return users;
		};

		/**
		 * Delete an annotation OBJECT (can be annotation/reply/review state)
		 * @private
		 * @param {Annotation} annotation - Annotation to delete
		 */
		AnnotationModel.prototype._deleteOneAnnotation = function (annotation) {
			this._annotations.splice(_.indexOf(this._annotations, annotation), 1);
			if (_.indexOf(this._filteredAnnotations, annotation) >= 0) {
				this._filteredAnnotations.splice(_.indexOf(this._filteredAnnotations, annotation), 1);
			}

			if (!annotation.isNew) {
				this._removedannotations.push(annotation);
			}
		};

		/**
		 * Renumber all the not saved annotations so the numbers are continuous. This is called when a not saved annotation is removed
		 * @param deletedNumber - the sequencenumber of the removed annotation
		 */
		AnnotationModel.prototype._renumberNewAnnotations = function (deletedNumber) {
			var unsavedAnnotations = this.getUnsavedAnnotations();
			_.each(unsavedAnnotations, _.bind(function (ann) {
				if (ann.messagenumber > deletedNumber) {
					ann.messagenumber = ann.messagenumber - 1;
					ann.updateIndicator();
				}
			}, this));
			this._emit("modelContentChanged");
		};

		/**
		 * Delete an annotation/reply with all its replies, review states and links. If the annotation is the main annotation, a message will be shown ("Do you wish to delete this annotation thread?")
		 * If the annotations can't be changed, this function will not be executed
		 * @param {Annotation} annotation - The annotation to delete
		 */
		AnnotationModel.prototype.deleteAnnotation = function (annotation) {
			if (!this.canDeleteAnnotation(annotation)) {
				this.f.i18n.loadBundle("viewer", function () {
					this.f.emit("error.WARNING", this.f.i18n.t("AnnReadOnlyError"));
				}, this);
				return;
			}

			this.f.i18n.loadBundle("viewer", _.bind(function () {
				if (!annotation.inreplyto) {
					if (this._deleteAnnotationPopup === undefined) {
						this._deleteAnnotationPopup = new ModalPopup(this.f, "deleteAnnotationPopup", this.f.getContext());

						// Header + Content
						this._deleteAnnotationPopup.setHeader($("<span/>").addClass("popupTitle").text(this.f.i18n.t("DeleteAnnotBtnTT")));
						this._deleteAnnotationPopup.setContent($("<div/>").addClass("unsavedItemsContent").text(this.f.i18n.t("AnnDelete")));
						this._deleteAnnotationPopup.setOkBtnText(this.f.i18n.t("YesResponse"));
						this._deleteAnnotationPopup.setCancelBtnText(this.f.i18n.t("NoResponse"));
					}
					this._deleteAnnotationPopup.setOkAction(_.bind(function () {
						this._deleteAnnotationPopup.hide();
						this._deleteAnnotationWithoutConfirmation(annotation);
					}, this));

					this._deleteAnnotationPopup.show();
					this._deleteAnnotationPopup.recalculateSize(400);
				} else {
					this._deleteAnnotationWithoutConfirmation(annotation);
				}
			}, this));
		};


		/**
		 * Delete an annotation/reply with all its replies, review states and links.
		 * @param {Annotation} annotation - The annotation to delete
		 */
		AnnotationModel.prototype._deleteAnnotationWithoutConfirmation = function (annotation) {
			if (annotation.parent) {
				var replyIndex = _.indexOf(annotation.parent.replies, annotation);
				if (replyIndex > -1) {
					annotation.parent.replies.splice(replyIndex, 1);
				} else {
					var stateIndex = _.indexOf(annotation.parent.states, annotation);
					if (stateIndex > -1) {
						annotation.parent.states.splice(stateIndex, 1);
					} else {
						var linkIndex = _.indexOf(annotation.parent.links, annotation);
						if (linkIndex > -1) {
							this._deleteAnnotationWithoutConfirmation(annotation.parent);
						}
					}
				}

				if (!annotation.isNew) {
					this._removedannotations.push(annotation);
				}
			} else {
				this.unselectAllAnnotations(true);
				_.each(annotation.links, function (annot) {
					this._deleteOneAnnotation(annot);
				}, this);
				this._deleteOneAnnotation(annotation);
				this._filterAllAnnotations(true);
			}


			this._emit("annotationDeleted", annotation);

			if (annotation.isNew) {
				this._renumberNewAnnotations(annotation.messagenumber);
			}
			this.modelChanged();
		};



		/**
		 * Reload the list of annotations.
		 * If the annotations have been changed since, a messages will be shown if the user wants to save the annotations first ("Annotations have been changed. Do you wish to save these changes?")
		 * @param {Boolean} silent - if silent is true, the user won't be prompted to save unsaved annotations
		 */
		AnnotationModel.prototype.reload = function (silent) {
			var reloadFunction = _.bind(function () {
				this._emit("reload.started");
				this.unselectAllAnnotations();
				this.resetAllAnnotations();
				if (this._page) {
					this.modelChanged();
					this._page.reloadAnnotations(_.bind(function (success) {
						if (success) {
							this.setAnnotations(this._page.getAnnotations());
						}
						this.modelChanged();
						this._emit("reload.finished");
					}, this));
				} else {
					this._emit("reload.finished");
				}
			}, this);

			if (silent) {
				// just throw away changes
				reloadFunction();
			} else {
				// prompt
				var hasUnsavedAnnotations = this.promptToSaveUnsavedAnnotations(reloadFunction, function () {}, reloadFunction);
				if (!hasUnsavedAnnotations) { //no prompt for unsaved annotations is shown
					reloadFunction(); //reload the annotations
				}
			}

		};

		/**
		 * Save 1 or all annotations
		 * @param {Annotation} [annotation] if specified, only this annotation will be saved. If not, all annotations will be saved
		 * @param {Function} [callbackFunction] will be called when the annotations have been saved
		 */
		AnnotationModel.prototype.save = function (annotation, callbackFunction) {
			if (!this._isSavingAnnotations) {
				this._emit("save.started");
				this._isSavingAnnotations = true;
				if (this.canChangeAnnotations() && this._page) {
					this.unselectAllAnnotations();


					// check if there are unsaved replies, if so, add them in the model
					_.each(this._annotations, function (annotation) {
						if (annotation.hasUnsavedReply()) {
							var text = annotation.getUnsavedReply();
							annotation.resetUnsavedReply();
							this.replyToAnnotation(annotation, text, false);
						}
					}, this);


					var annotationsToBeSaved;
					var annotationToBeRemoved = this._removedannotations.slice(0);
					this._removedannotations = [];

					if (!annotation) {
						annotationsToBeSaved = this.getUnsavedAnnotations();
					} else {
						annotationToBeRemoved = [];
						annotationsToBeSaved = [];

						annotationsToBeSaved.push(annotation);

						var p = annotation.getGodfather();
						_.each(p.links, function (annot) {
							if (!annot.saved && _.indexOf(annotationsToBeSaved, annot) == -1) {
								annotationsToBeSaved.push(annot);
							}
						});
					}

					this._page.saveAnnotations(annotationsToBeSaved, annotationToBeRemoved, _.bind(function (page, annots) {
						this._isSavingAnnotations = false;
						if (this._page == page) {
							// we only update the annotation, if the page that was saved is still active, otherwise don't do anything
							this.setAnnotations(annots);
						}

						if (callbackFunction) {
							callbackFunction();
						}
						this._emit("save.finished");

						this.modelChanged();
					}, this, this._page));
				} else {
					this._isSavingAnnotations = false;
					// we didn't need to save, notify that the safe action is finsihed
					this._emit("save.finished");
				}
			} else {
				this.f.emit("error.WARNING", this.f.i18n.t("AnnAreAlreadyBeingSaved"), true);
			}
		};

		/**
		 *
		 * @returns {Number} the count of all changed annotations (deleted + unsaved)
		 */
		AnnotationModel.prototype.getUnsavedAnnotationCount = function () {
			var unsavedCount = this.getUnsavedAnnotations().length;
			var deletedCount = this.getDeletedAnnotations().length;
			return unsavedCount + deletedCount;
		};

		/**
		 * Get an array of unsaved annotations.
		 * @returns {Array.<Annotation>} Array of unsaved annotations
		 */
		AnnotationModel.prototype.getUnsavedAnnotations = function () {
			var unsavedAnnotations = [];
			if (this.canChangeAnnotations(true)) {
				_.each(this._annotations, _.bind(function (annotation) {
					annotation.runThroughAnnotation(true, true, true, _.bind(function (annot) {
						if ((!annot.saved || annot.hasUnsavedReply()) && _.indexOf(unsavedAnnotations, annot) == -1 && this.canChangeAnnotation(annot)) {
							unsavedAnnotations.push(annot);
						}
					}, this));

					var p = annotation.getGodfather();
					_.each(p.links, _.bind(function (annot) {
						if ((!annot.saved || annot.hasUnsavedReply()) && _.indexOf(unsavedAnnotations, annot) == -1 && this.canChangeAnnotation(annot) && !annot.isStateObject()) {
							unsavedAnnotations.push(annot);
						}
					}, this));
				}, this));
			}

			return unsavedAnnotations;
		};

		/**
		 * Get an array of saved annotations.
		 * @returns {Array.<Annotation>} - Array of saved annotations
		 */
		AnnotationModel.prototype.getSavedAnnotations = function () {
			var savedAnnotations = [];
			_.each(this._annotations, function (annotation) {
				if (annotation.saved) {
					savedAnnotations.push(annotation);
				}
			});
			return savedAnnotations;
		};

		/**
		 * Get an array of removed annotations
		 * @returns {Array.<Annotation>} Array of deleted annotations
		 */
		AnnotationModel.prototype.getDeletedAnnotations = function () {
			return this._removedannotations;
		};


		/**
		 * @returns {Array.<Annotation>} the visible annotations to be displayed on the viewport
		 */
		AnnotationModel.prototype.getVisibleAnnotations = function () {
			// if annotation tools are disabled, or the module is temporarily disabled, do not show any annotations
			if (this.f.getOption("PREFERENCES_ANNOTATIONTOOLS_VISIBLE") === 0 ||
				this.isTemporarilyDisabledAndNoAnnotationsVisible()) {
				return [];
			} else {
				if (this.f.getOption("PREFERENCES_ANNOTATION_SHOWSELECTEDONLY") && this._selectedAnnotations.length > 0) {
					return this._selectedAnnotations;
				} else {
					return this._filteredAnnotations;
				}
			}
		};

		/**
		 * Get the filtered annotations
		 * @returns {Array.<Annotation>} Array of filtered annotations
		 */
		AnnotationModel.prototype.getFilteredAnnotations = function () {
			return this._filteredAnnotations;
		};




		/**
		 * Select an annotation. All other selected annotations will be unselected first
		 * @param {Annotation} annot - Annotation to be selected
		 * @param {Boolean} force - makes the unselectAllAnnotations forced
		 */
		AnnotationModel.prototype.selectAnnotation = function (annot, force) {
			var i;
			if (this._selectedAnnotations.length === 1) {
				var sa = this._selectedAnnotations[0];
				if (annot == sa) {
					return;
				}
			}

			this.unselectAllAnnotations(true, force);

			this._selectedAnnotations.push(annot);
			var mainAnnotation = annot.getGodfather();
			if (mainAnnotation.links) {
				for (i = 0; i < mainAnnotation.links.length; i++) {
					this._selectedAnnotations.push(mainAnnotation.links[i]);
				}
			}
			if (annot.linkedto) {
				annot = this.getAnnotation(annot.linkedto);
				if (annot) {
					for (i = 0; i < annot.links.length; i++) {
						if (annot.links[i] != annot) {
							this._selectedAnnotations.push(annot.links[i]);
						}
					}
					this._selectedAnnotations.push(annot);
				}
			}

			this.ensureVisibility();

			if (this._layoutType == LayoutConstants.LayoutType.MOBILE) {
				this.f.emit("module.annotation.zoomToAnnotation", annot);
			}

			this._emit("selectedAnnotation", annot);
			this.modelChanged();
		};

		/**
		 * Make sure that the annotations are visible. If not, set the visible property back to 1
		 */
		AnnotationModel.prototype.ensureVisibility = function () {
			if (!this.f.getOption("PREFERENCES_ANNOTATIONTOOLS_VISIBLE")) {
				this.f.setOption("PREFERENCES_ANNOTATIONTOOLS_VISIBLE", 1);
			}
		};

		/**
		 * Get an annotation based on the annotation name
		 * @param {String} annotationName - internal name of the annotation e.g. name="kkIy7drJAKxuER0gspHfIS"
		 * @returns {Annotation} the annotation for a given name, null if no annotation was found
		 */
		AnnotationModel.prototype.getAnnotation = function (annotationName) {
			for (var i = 0; i < this._annotations.length; i++) {
				if (this._annotations[i].name == annotationName) {
					return this._annotations[i];
				}
			}
			return null;
		};

		/**
		 * Get the all the links of an annotation (also of the parents and the links of links)
		 * @param {Annotation} annot - reference annotation
		 * @returns {Array.<Annotation>}
		 */
		AnnotationModel.prototype.getAnnotationLinks = function (annot) {
			var i;
			var annots = [];
			if (annot.links) {
				for (i = 0; i < annot.links.length; i++) {
					annots.push(annot.links[i]);
				}
			}
			if (annot.linkedto) {
				for (i = 0; i < this._annotations.length; i++) {
					if (this._annotations[i].name == annot.linkedto) {
						annots.push(this._annotations[i]);
					}
				}
			}
			return annots;
		};

		/**
		 * @param {Annotation} annotation
		 * @returns {Array.<Annotation>} all the replies to one annotation
		 */
		AnnotationModel.prototype.getReplies = function (annotation) {
			var replies = [];
			annotation.runThroughAnnotation(false, true, false, function (annot) {
				replies.push(annot);
			});
			return replies;
		};

		/**
		 * Unselect all the annotations
		 * @param {Boolean} [donotbroadcast] - if set, the model will not broadcast that this happened (only used when the caller will call modelChanged() itself)
		 * @param {Boolean} [force] - if set, it will ignore the lockSelection flag
		 */
		AnnotationModel.prototype.unselectAllAnnotations = function (donotbroadcast, force) {
			if (this._selectedAnnotations.length <= 0 || (this.lockSelection && !force)) {
				return;
			}

			var selectedAnnotations = this._selectedAnnotations;
			this._selectedAnnotations = [];

			_.each(selectedAnnotations, _.bind(function (annotation) {
				this._emit("unselectedAnnotation", annotation);
			}, this));

			if (!donotbroadcast) {
				this.modelChanged();
			}
		};

		/**
		 * Resets all the annotations in the model, all changes to annotations are lost.
		 */
		AnnotationModel.prototype.resetAllAnnotations = function () {
			this.scaleAllAnnotations(null);
			this.unselectAllAnnotations(); //first unselect all annotations
			this._annotations = []; //set annonations of to 0
			this._removedannotations = []; //set no removed annotation
			this._page.resetAnnotations();
		};


		/**
		 * Check if an annotation or one of the replies of the annotations is selected.
		 * @param {Annotation} annot - Annotation to check
		 * @returns {Boolean} True if the annotation or one of its replies are selected, false otherwise
		 */
		AnnotationModel.prototype.isAnnotationOrReplySelected = function (annot) {
			if (annot !== null) {
				//first check if annotation is selected
				if (_.indexOf(this._selectedAnnotations, annot) > -1) {
					return true;
				}
				//if annotation is not selected first check if it has replies
				if (annot.replies.length === 0) {
					return false;
				} else {
					//returns true the first time a reply of annot is selected
					return _.some(annot.replies, _.bind(function (reply) {
						return _.indexOf(this._selectedAnnotations, reply) > -1;
					}, this));
				}
			} else {
				return false;
			}
		};

		/**
		 * Get the selected annotations
		 * @returns {Array.<Annotation>} list of selected annotations
		 */
		AnnotationModel.prototype.getSelectedAnnotations = function () {
			return this._selectedAnnotations;
		};

		/**
		 * Move an annotation
		 * @param {Annotation} annot- Annotation to move
		 * @param {ShapeUtils.Size} movesize - World size of how much the annotation should move
		 * @returns {Boolean} True if the annotation was moved, false otherwise (e.g. when you cannot edit the annotation)
		 */
		AnnotationModel.prototype.moveAnnotation = function (annot, movesize) {
			if (!this.canChangeAnnotation(annot)) {
				return false;
			}

			annot.move(movesize);
			this.modelChanged();

			return true;
		};



		/**
		 * Transform the frame of an annotation
		 * @param {Annotation} annot - Annotation to transform
		 * @param {ShapeUtils.Rect} rect - New world rect for the annotation
		 * @returns {Boolean} True if the annotation was transformed, false otherwise (e.g. when you cannot edit the annotation)
		 */
		AnnotationModel.prototype.transformAnnotation = function (annot, rect) {
			if (!this.canChangeAnnotation(annot)) {
				return false;
			}

			annot.setRect(rect);
			this.modelChanged();

			return true;
		};

		/**
		 * Change the start and end point of an annotation
		 * @param {Annotation} annot - annotation to change
		 * @param {ShapeUtils.Point} start - start point
		 * @param {ShapeUtils.Point} end - end point
		 */
		AnnotationModel.prototype.updateStartEndOfAnnotation = function (annot, start, end) {
			if (!this.canChangeAnnotation(annot)) {
				return false;
			}

			annot.setStartEnd(start, end);
			this.modelChanged();

			return true;
		};

		/**
		 * Add an annotation with a given rect
		 * e.g. used to create an annotation from the measure module or to create a global annotation
		 * @param {ShapeUtils.Rect}  rect - the new rect
		 * @param {String} [content] - Content for the annotation
		 * @param {AnnotationConstants.AnnotationType} [type] - type of the annotation to be added
		 */
		AnnotationModel.prototype.addAnnotationWithRect = function (rect, content, type) {
			var annotationColor;
			if (this.f.getProperty("annotationcolor")) {
				annotationColor = this.f.getProperty("annotationcolor");
			} else {
				annotationColor = this.f.getCurrentUser().getColor("PREFERENCES_ANNOTATION_RECTANGLE_COLOR");
			}

			var annot = new Annotation(this.f);
			annot.initAnnotation(type, annotationColor, false, this, rect);
			if (content) {
				annot.content = content;
			}

			this.addAnnotation(annot);
		};

		/**
		 * Get the next message number for a new annotation
		 * @private
		 * @returns {Number} <the highest messagenumber> + 1
		 */
		AnnotationModel.prototype._nextNumberForOriginType = function (originType) {
			var number = 0;
			for (var i = 0; i < this._annotations.length; i++) {
				if (this._annotations[i].messagenumber > number && this._annotations[i].originType === originType) {
					number = this._annotations[i].messagenumber;
				}
			}
			number++;
			return number;
		};

		/**
		 * Add an annotation. If the annotation is not a main annotation (e.g. reply/review state) the main annotation should be set
		 * @param {Annotation} annot - the annotation to add
		 * @param {Annotation} [mainannotation] - the main annotation (the godfather annotation when adding a reply or review state)
		 * @param {Boolean} [silent = false] - don't send events
		 */
		AnnotationModel.prototype.addAnnotation = function (annot, mainannotation, silent) {
			if (this.checkCanChangeAnnotationsAndDisplayError()) {

				if (!annot.linkedto) {
					annot.messagenumber = this._nextNumberForOriginType(annot.originType);
				}
				annot.username = this.f.getCurrentUser().getUsername();
				annot.firstname = this.f.getCurrentUser().getFirstName();
				annot.lastname = this.f.getCurrentUser().getLastName();
				annot.creationdate = new Date();
				annot.date = new Date();


				annot.setModified("NEW");

				this.ensureVisibility();

				if (!mainannotation) {
					this._annotations.push(annot);
					this._filterAllAnnotations(true);
					var compliesWithFilter = this._filter.compliesToFilter(annot);
					var compliesWithCustomFilter = true;
					if (this._annotationFilterModel.isCustomFilterActive()) {
						compliesWithCustomFilter = this._filter.compliesToCustomFilter(annot);
					}
					if (compliesWithFilter && compliesWithCustomFilter) {
						if (!annot.linkedto && !silent) {
							this._emit("annotationAdded", annot, false);
						}
						this._filteredAnnotations.push(annot);
						if (!silent) {
							this.selectAnnotation(annot);
						}
					} else {
						/**
						 * The current annotation does not match with the current filter... Inform the user, so they know...
						 */
						if (!silent) {
							this._emit("annotationAdded", annot, true);
						}
						this.f.i18n.loadBundle("viewer", function () {
							this.f.emit("userMessage", "warn", this.f.i18n.t("AnnNotViewable"), _.bind(function () {
								this._annotationFilterModel.resetFilter();
							}, this));
						}, this);
					}
				} else {
					if (!silent) {
						this.modelChanged(mainannotation, annot);
					}
				}

			}
		};

		/**
		 * Add a linked annotation to the currently selected annotation
		 * @param {Annotation} annotation - Annotation to be added
		 */
		AnnotationModel.prototype.addLinkedAnnotation = function (annotation) {
			var annot = this.getSelectedAnnotations()[0];

			if (annot) {
				if (!this.canChangeAnnotation(annot)) {
					// we cannot add a linked annotation, just add a new annotation
					this.f.i18n.loadBundle("viewer", function () {
						this.f.emit("userMessage", "warn", this.f.i18n.t("AnnCannotAddLinkedToNonOwnedAnnotation"), true);
					}, this);
				} else {
					if (this.checkCanChangeAnnotationsAndDisplayError()) {
						annot = annot.getGodfather();

						if (annot.linkedto) {
							annot = this.getAnnotation(annot.linkedto);
						}

						annotation.linkedto = annot.name;
						annotation.parent = annot;

						annot.links.push(annotation);

						annotation.messagenumber = annot.messagenumber;
						this.addAnnotation(annotation);

						annotation.annotationcolor = annot.color;
						annotation.color = annot.color;

						this.selectAnnotation(annot, true);
					}
				}

			} else {
				this.f.i18n.loadBundle("viewer", function () {
					this.f.emit("userMessage", "warn", this.f.i18n.t("NoAnnotationSelected"), true);
				}, this);
			}
		};

		/**
		 * Reply to an annotation
		 * @param {Annotation} annotation - Annotation to reply to
		 * @param {String} replyText - reply text
		 * @param {Boolean} save - If true, save the annotation directly
		 */
		AnnotationModel.prototype.replyToAnnotation = function (annotation, replyText, save) {
			if (this.checkCanChangeAnnotationsAndDisplayError()) {
				annotation.resetUnsavedReply();
				var annot = annotation.duplicate(false, false, false, false, this);
				annot.inreplyto = annotation.name;
				annot.content = replyText;
				annot.parent = annotation;

				annotation.replies.push(annot);

				this.addAnnotation(annot, annotation);

				if (save) {
					this.save(null, _.bind(function () {
						this.selectAnnotation(annotation, true);
						annot.cancelModified();
						annot.setIsNew(false);
					}, this));
				}
			} else {
				annotation.resetUnsavedReply();
				this.modelChanged();
			}
		};

		/**
		 * Set the state of an annotation (for the current user)
		 * If the state is already set, the state is changed
		 * If the state has not been set before, make a new review state object
		 *
		 * @param {Annotation} annotation - Annotation for which the state needs to be set
		 * @param {String} state - new state
		 */
		AnnotationModel.prototype.setState = function (annotation, state) {
			if (this.checkCanChangeAnnotationsAndDisplayError()) {
				this.f.i18n.loadBundle("viewer", _.bind(function () {
					var text = this.f.i18n.t("AnnStatusSetByText", state, this.f.getCurrentUser().getFullName());
					var bodytext = "<p>" + text + "</p>";

					var stateAnnotation = annotation.getUserStateAnnotation(this.f.getCurrentUser().getUsername());
					if (stateAnnotation) {
						if (stateAnnotation.state != state) {
							stateAnnotation.state = state;
							stateAnnotation.content = bodytext;
							stateAnnotation.setModified("STATE");
							this.modelChanged(annotation);
						}
					} else {
						var annot = annotation.duplicate(false, false, false, false, this);

						annot.inreplyto = annotation.name;
						annot.content = bodytext;
						annot.parent = annotation;
						annot.state = state;
						annot.statemodel = "Review";
						annot.parent = annotation;
						annot.linkedto = annotation.name;
						annotation.states.push(annot);

						this.addAnnotation(annot, annotation);
					}

				}, this));
			}
		};

		/**
		 * Check if the user is allowed to change annotations, display an error if needed
		 * @returns {Boolean} canChangeAnnotations
		 */
		AnnotationModel.prototype.checkCanChangeAnnotationsAndDisplayError = function () {
			if (!this.canChangeAnnotations()) {
				if (this.getDisabledReason().length > 0) { // only show a warning message, if we have something to say...
					this.f.emit("userMessage", "warn", this.getDisabledReason());
				}

				return false;
			} else {
				return true;
			}
		};


		/**
		 * Inform the listeners that the model has changed.
		 * If a mainannotation or annotation is provided, the function will also emit an "annotationUpdated" message
		 * @param {Annotation} [mainannotation] - the "main" annotation that has been updated
		 * @param {Annotation} [annotation] - the "reference" annotation (reply/review state/link) that has been updated
		 */
		AnnotationModel.prototype.modelChanged = function (mainannotation, annotation) {
			if (annotation || mainannotation) {
				this._emit("annotationUpdated", mainannotation, annotation);
			}
			this._emit("modelChanged");
		};

		/**
		 * Inform the listeners that the complete content has changed for the model (i.o.w, the _annotations are changed)
		 */
		AnnotationModel.prototype.modelContentChanged = function () {
			this._orderedAnnotations.length = 0;
			this._emit("modelContentChanged");
		};

		/**
		 * Inform that the filter has changed (so the GUI can be updated)
		 */
		AnnotationModel.prototype.filterChanged = function () {
			this._emit("filterChanged");
		};

		/**
		 * Orders the annotations by modification date. Only main annotation get ordered but the last modified date of all the
		 * replies and states are taken into account.
		 * @private
		 */
		AnnotationModel.prototype.orderByModificationDate = function () {
			this._filteredAnnotations = _.sortBy(this._filteredAnnotations, function (ann) {
				var lastModifiedDate = new Date(0);
				ann.runThroughAnnotation(true, true, true, function (annotation) {
					if (annotation.date > lastModifiedDate) {
						lastModifiedDate = annotation.date;
					}
				});
				return lastModifiedDate;
			});
			this.currentOrdening = this.orderByModificationDate;
		};

		/**
		 * Orders the annotations by creation date.
		 * @private
		 */
		AnnotationModel.prototype.orderByCreationDate = function () {
			this._filteredAnnotations = _.sortBy(this._filteredAnnotations, function (ann) {
				return ann.creationdate;
			});
			this.currentOrdening = this.orderByCreationDate;
		};

		/**
		 * Calculates the _orderedAnnotations array according to the step by option and sets the index to 0.
		 */
		AnnotationModel.prototype.orderAnnotationsForStep = function () {
			var stepByCreationDate = this.f.getOption("PREFERENCES_ANNOTATION_DATESORT_MODE") === AnnotationConstants.AnnotationSortType.SORT_BY_CREATION_DATE;

			var annotationList = [];

			//when step by creation date is selected (also replies)
			if (this.f.getOption("PREFERENCES_ANNOTATION_NEXT_MODE") === 0) {
				_.each(this._filteredAnnotations, function (ann) {
					//go through all replies of the annotation and add them to the temp collection
					ann.runThroughAnnotation(true, true, false, function (annotation) {
						annotationList.push(annotation);
					});
				});

			} else if (this.f.getOption("PREFERENCES_ANNOTATION_NEXT_MODE") === 3) {
				// step by area/position, each part of a linked annotation needs to be added
				_.each(this._filteredAnnotations, function (annotation) {
					annotationList.push(annotation);
					annotationList = annotationList.concat(this.getAnnotationLinks(annotation));
				}, this);

			} else {
				annotationList = this._filteredAnnotations;
			}


			//Order the temp collection
			this._orderedAnnotations = _.sortBy(annotationList, function (ann) {
				if (stepByCreationDate) {
					return -ann.creationdate;
				} else {
					return -ann.date;
				}
			});

			this._currentAnnotationIndex = _.indexOf(this._orderedAnnotations, this._selectedAnnotations[0]);
		};

		/**
		 *	Gets the next annotation according to Step by setting. When the orderedAnnotations collection is empty it
		 *	calculates the new collection by calling orderAnnotationsForStep.
		 *	@private
		 *
		 *	@returns {Annotation} the next annotation
		 */
		AnnotationModel.prototype.getNextAnnotation = function () {
			if (this._orderedAnnotations.length === 0) {
				this.orderAnnotationsForStep();
			}
			if (this._selectedAnnotations.length > 0) {
				this._currentAnnotationIndex = _.indexOf(this._orderedAnnotations, this._selectedAnnotations[0]);
			}
			return this._orderedAnnotations[++this._currentAnnotationIndex % this._orderedAnnotations.length];
		};

		/**
		 *	Gets the previous annotation according to Step by setting. When the orderedAnnotations collection is empty it
		 *	calculates the new collection by calling orderAnnotationsForStep.
		 *	@private
		 *
		 *	@returns {Annotation} the previous annotation
		 */
		AnnotationModel.prototype.getPreviousAnnotation = function () {
			if (this._orderedAnnotations.length === 0) {
				this.orderAnnotationsForStep();
			}
			if (this._selectedAnnotations.length > 0) {
				this._currentAnnotationIndex = _.indexOf(this._orderedAnnotations, this._selectedAnnotations[0]);
			}

			if (--this._currentAnnotationIndex < 0) {
				this._currentAnnotationIndex = this._orderedAnnotations.length - 1;
			}

			return this._orderedAnnotations[this._currentAnnotationIndex % this._orderedAnnotations.length];
		};

		/**
		 * Selects the next annotation.
		 *
		 * @param {Boolean} isNext - should it go to the next annotation, if false it goes to the previous annotation
		 */
		AnnotationModel.prototype.goToAnnotation = function (isNext) {
			var nextAnnotation = isNext ? this.getNextAnnotation() : this.getPreviousAnnotation();
			this.selectAnnotation(nextAnnotation, true);
			var annotationNextMode = this.f.getOption("PREFERENCES_ANNOTATION_NEXT_MODE");
			//when zoom to annotation is needed
			if (annotationNextMode === 2 || annotationNextMode === 3) {
				this.f.emit("module.annotation.zoomToAnnotation", nextAnnotation);
			}
		};


		/**
		 * Update the annotation color of the current user to use the colors defined in his settings
		 */
		AnnotationModel.prototype.updateAllAnnotationColors = function () {
			var pinColor = this.f.getCurrentUser().getColor("PREFERENCES_ANNOTATION_NOTE_COLOR");
			var otherColor = this.f.getCurrentUser().getColor("PREFERENCES_ANNOTATION_RECTANGLE_COLOR");

			_.each(this._annotations, _.bind(function (annotation) {
				if (this.canChangeAnnotation(annotation)) {
					if (annotation.annottype == AnnotationConstants.AnnotationType.ANNOT_PIN) {
						annotation.setColor(pinColor);
					} else {
						annotation.setColor(otherColor);
					}
					this._emit("annotationUpdated", annotation);
				}
			}, this));

			this.modelChanged();
		};


		/**
		 * Set the sorting type of the annotations
		 * @param {Number} mode
		 * @param {Boolean} [silent] - if true, no events are sent out
		 */
		AnnotationModel.prototype.setSortingMode = function (mode, silent) {
			if (mode === AnnotationConstants.AnnotationSortType.SORT_BY_CREATION_DATE) {
				this.orderByCreationDate();
			} else if (mode === AnnotationConstants.AnnotationSortType.SORT_BY_MODIFICATION_DATE) {
				this.orderByModificationDate();
			}

			if (!silent) {
				this.filterChanged();
			}
		};

		/**
		 * Generate a unique name (looking to all existing annotations)
		 * This method will try up to 100 times to generate an unique name
		 * @returns {String} Unique random name for an annotation containing 22 characters from [0-9a-zA-Z]
		 */
		AnnotationModel.prototype.generateUniqueName = function () {
			var trialsLeft = 100;
			var finalName;

			while (finalName === undefined && trialsLeft > 0) {
				var candidate = ViewerUtils.randomString(22);

				var match = _.find(this._annotations, function (annotation) {
					return annotation.name === candidate;
				});

				if (match === undefined) {
					finalName = candidate;
				}

				trialsLeft--;
			}

			if (finalName === undefined) {
				this.f.emit("error.SEVERE", "Error while generating ID for new annotation");
			}

			return finalName;
		};

		/**
		 * Set the page for which annotations should be displayed (this method will fetch the annotation for the page if necessary)
		 * @param {Data.AbstractDocumentPage} newPage - page for whichannotatiosn should be displayed
		 * @param {Function} [callback] - called when the page has changed
		 */
		AnnotationModel.prototype.setPage = function (newPage, callback) {
			if (newPage != this._page) {
				// the page has changed, so we need to do something

				var oldPage = this._page;
				if (this._approvalUpdatedListener && oldPage) {
					if (oldPage.getDocumentVersion().isPageList()) {
						oldPage.off("approval.loaded", this._approvalUpdatedListener);
					} else {
						oldPage.getDocumentVersion().off("approval.update", this._approvalUpdatedListener);
					}
				}

				// reset annotations
				this.setAnnotations([]);
				this._emit("changed.page");
				this.unselectAllAnnotations(false, true);
				this._page = newPage;
				if (this._addAnnotationsEnabledListener) {
					this._unregisterListener(this._addAnnotationsEnabledListener);
				}
				if (this._disabledReasonChangedListener) {
					this._unregisterListener(this._disabledReasonChangedListener);
				}

				this._clearDisableReasons();

				if (newPage) {
					// load the annotations for the newPage
					newPage.requestAnnotations(_.bind(function (success) {
						this._page = newPage;
						this.f.loadBundle("viewer", function () {
							if (!newPage.canChangeAnnotations()) {
								this.setDisableReason("CANCHANGEANNOTATIONS", this.f.t("AnnReadOnlyCannotChangeAnnotations"));
							} else {
								this.removeDisableReason("CANCHANGEANNOTATIONS");
							}

							if (!newPage.getDocumentVersion().isLatestVersion()) {
								this.setDisableReason("NOT_LATEST", this.f.t("AnnReadOnlyNotLatestVersion"));
							} else {
								this.removeDisableReason("NOT_LATEST");
							}
						}, this);

						if (success) {
							this.setAnnotations(newPage.getAnnotations());

							if (this.getAnnotationCount() === 0 && this.f.getConfig().get("annotationaddglobal") === "1") {
								//if annotations are not completly done loading (either still disabled reason or user cannot edit yet)
								if (this.getDisabledReason().length > 0 || !this.canChangeAnnotations()) {
									//listen to both change is disabled reasons (either during loading or when opened in compare) and edit rights to annotations (load time)
									this._addAnnotationsEnabledListener = this._registerListener("readWriteChanged", function () {
										if (this.getDisabledReason().length === 0 && this.canChangeAnnotations() && this.getAnnotationCount() === 0) {
											this._emit("addGlobalAnnotation");
											this._unregisterListener(this._addAnnotationsEnabledListener);
											this._unregisterListener(this._disabledReasonChangedListener);
										}
									}, this, this);
									this._disabledReasonChangedListener = this._registerListener("disabledReasonChanged", function () {
										if (this.getDisabledReason().length === 0 && this.canChangeAnnotations() && this.getAnnotationCount() === 0) {
											this._emit("addGlobalAnnotation");
											this._unregisterListener(this._addAnnotationsEnabledListener);
											this._unregisterListener(this._disabledReasonChangedListener);
										}
									}, this, this);
								} else {
									this._emit("addGlobalAnnotation");
								}

							}
						} else {
							this.f.debug.error("Annotations were NOT fetched succesfully");
						}

						this.updateTemporarilyDisabledReason();

						if (callback) {
							callback();
						}
					}, this));
				}

				if (!this._appovalUpdatedListener) {
					this._approvalUpdatedListener = _.bind(function (success) {
						this.onApprovalUpdated(success);
					}, this);
				}


				// Dependency on Approval Module if needed
				if (this.editabilityDependsOnApproval()) {
					this.f.loadBundle("viewer", function () {
						this.setDisableReason("LOADING_APPROVAL", this.f.t("Loading"));
					}, this);
					this.f.requestModule("approval", _.bind(function (approval) {
						var docVersion = newPage.getDocumentVersion();
						if (docVersion.isPageList()) {
							newPage.on("approval.loaded", this._approvalUpdatedListener);
							newPage.requestPageApprovalSetup(this._approvalUpdatedListener);
						} else {
							docVersion.on("approval.update", this._approvalUpdatedListener);
							docVersion.requestApprovalSetup(this._approvalUpdatedListener);
						}
					}, this));
				}
			} else {
				if (callback !== undefined) {
					callback();
				}

			}

			if (!this._annotationFilterModel.isAnnotationFilterPinned()) {
				this._annotationFilterModel.resetFilter();
			}
			this.modelChanged();
		};

		/**
		 * Get the page for which annotations are shown currently
		 * @returns {Data.AbstractDocumentPage}
		 */
		AnnotationModel.prototype.getPage = function () {
			return this._page;
		};

		/**
		 * Returns the offset of the page
		 * @returns {ShapeUtils.Size}
		 */
		AnnotationModel.prototype.getAnnotationsOffset = function () {
			var annotPageOffset = this._page.getAnnotationsOffset();
			return annotPageOffset;
		};


		/**
		 * Get the value of the warning message
		 * @returns {String}
		 */
		AnnotationModel.prototype.getDisabledReason = function () {
			if (this._disabledReasons.getLength() > 0) {
				var reason = this._disabledReasons.getLast().reason;
				if (reason === "") {
					_.some(this._disabledReasons.getItemsAsObject(), function (item) {
						reason = item.reason;
						return reason !== "";
					});
				}
				return reason;
			} else {
				return "";
			}
		};

		/**
		 * Add a new reason why the annotationmodule is blocked
		 * @param {String} id
		 * @param {String} reason - message shown to the user
		 * @param {Boolean} [hideAnnotations] - do the annotations need to be completly hidden?
		 * @param {Boolean} [dontRemove] - don't remove the disabled reason from the collection when the page changes, the module who added the reason is responsible of removing it again
		 */
		AnnotationModel.prototype.setDisableReason = function (id, reason, hideAnnotations, dontRemove) {
			var disabledReason = {
				reason: reason,
				hideAnnotations: hideAnnotations,
				dontRemove: dontRemove
			};
			this._disabledReasons.addOrReplaceItem(id, disabledReason);
			this._disabledReasons.moveItemByKeyToEnd(id);
			this._emit("disabledReasonChanged");
		};

		/**
		 * Remove a reason why the annotaitonmodule was blocked
		 * @param id
		 */
		AnnotationModel.prototype.removeDisableReason = function (id) {
			this._disabledReasons.removeByKey(id);
			this._emit("disabledReasonChanged");
		};

		/**
		 * Clear out all disabled reasons, with exeption of the reasons set by other modules, they are responsible to clear those reasons
		 */
		AnnotationModel.prototype._clearDisableReasons = function () {
			var keepReasons = []; //construct an array with all the keys of reason that don't need to be cleared out of the reasons object
			_.each(this._disabledReasons.getKeys(), function (key) { //run over all the reasons in the disabledReasons collection
				if (this._disabledReasons.getItemByKey(key).dontRemove) { //if the don't remove flag is set, push the key in the array
					keepReasons.push(key);
				}
			}, this);
			this._disabledReasons.omitOthers(keepReasons); //omit all the items from the ordered object but the reasons that need to be kept
			this._emit("disabledReasonChanged");
		};

		/**
		 * Prompt to save unsaved annotations if there are any
		 * if there are unsaved anntations, save them
		 *
		 * NOTE: this is not a pure model function, but to limit regressions, we keep it this way...
		 * @param {Function} callbackOnOK - function which is called when the popup is closed and annotations are saved
		 * @param {Function} [callbackOnCancel] - if provided, this callback is called when the user presses cancel
		 * @param {Function} [callbackOnDontSave] - adds the possibility to not save unsaved annotation but to continue with the action, on the contrary, cancel will cancel the action and don't save. Note that reloading the annotations should be done by the caller!
		 * @returns {Boolean} true if there are unsaved items and the popup is shown, false if not
		 */
		AnnotationModel.prototype.promptToSaveUnsavedAnnotations = function (callbackOnOK, callbackOnCancel, callbackOnDontSave) {
			if (callbackOnCancel === undefined) {
				callbackOnCancel = callbackOnOK;
			}

			if (this._page && this.getUnsavedAnnotationCount() > 0 && this.canChangeAnnotations(true)) {
				this.f.loadBundle("viewer", function () {
					// show popup if there are unsaved items
					var modalPopup = new ModalPopup(this.f, "unsavedAnnotationsPopup", this.f.getContext());
					modalPopup.setRemoveFromDomAfterUsage(true);
					modalPopup.setOkAction(_.bind(function () {
						modalPopup.enableOkBtn(false);

						// save annotations
						this.save(null, _.bind(function () {
							// saving succeeded, so hide
							modalPopup.hide();
							if (callbackOnOK) {
								callbackOnOK();
							}
							this.toggleVisibility(true);
						}, this));
					}, this));

					modalPopup.setCancelAction(_.bind(function () {
						if (callbackOnCancel) {
							callbackOnCancel();
						}
						this.toggleVisibility(true);
					}, this));

					if (callbackOnDontSave) {
						var dontSaveButton = new ViewerToolbarButton(this.f, "dontSaveAnnotations", $("<span>").append(this.f.t("RevertAnnChanges")), false, _.bind(function () {
							modalPopup.hide();
							this.toggleVisibility(true);

							if (callbackOnDontSave) {
								callbackOnDontSave();
							}
						}, this));
						modalPopup.addButton(dontSaveButton);
					}

					// Header + Content
					modalPopup.setHeader($("<span/>").addClass("popupTitle").text(this.f.i18n.t("AnnTitle")));
					modalPopup.setContent($("<div/>").addClass("unsavedItemsContent").text(this.f.i18n.t("AnnSavePrompt")));
					modalPopup.setOkBtnText(this.f.i18n.t("Save"));
					this.toggleVisibility(false);
					modalPopup.show();
					modalPopup.recalculateSize(600);
				}, this);

				return true;
			} else {
				return false;
			}
		};

		/**
		 * Set to lock the selection (true) of annotations or not (false)
		 * @param {Boolean} lockSelection - True to lock the selection, false to not lock the selection
		 */
		AnnotationModel.prototype.setLockSelection = function (lockSelection) {
			this.lockSelection = lockSelection;
		};

		/**
		 * Set the layout type, the widget needs to be rendered for
		 * @param layoutType
		 */
		AnnotationModel.prototype.setLayoutType = function (layoutType) {
			this._layoutType = layoutType;
		};

		/**
		 * Get the layout type the widget should be rendered for
		 * @returns {LayoutConstants.LayoutType)
		 */
		AnnotationModel.prototype.getLayoutType = function () {
			return this._layoutType;
		};

		/**
		 * Scale all the annotation of the annotation model with a given factor, both horizontal as vertical, changes all the points and rectangle of the annotation,
		 * first undos the scaling and than applies the new scaling, if no scaleFactor is provided the current scaling is undone
		 * @param {ShapeUtils.Point} scaleFactor
		 */
		AnnotationModel.prototype.scaleAllAnnotations = function (scaleFactor) {
			if (this._currentScaling) {
				_.each(this._annotations, function (annotation) {
					if (annotation.isScaled()) {
						annotation.scale(new ShapeUtils.Point(1 / annotation.getScale().x, 1 / annotation.getScale().y));
					}
				}, this);
			}
			this._currentScaling = scaleFactor;
			_.each(this._annotations, function (annotation) {
				annotation.setScale(scaleFactor);
			});
		};

		/**
		 * Get the currently active scaling
		 * @returns {ShapeUtils.Point}
		 */
		AnnotationModel.prototype.getCurrentScaling = function () {
			return this._currentScaling;
		};

		/**
		 * Get the model for the annotation filter
		 * @returns {AnnotationFilterModel}
		 */
		AnnotationModel.prototype.getAnnotationFilterModel = function () {
			return this._annotationFilterModel;
		};

		return AnnotationModel;
	});

define('web-viewer/annotation/AnnotationLayerModel',["web-viewer/viewport/model/ViewportLayerModel",
		"utils/ShapeUtils",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/viewControls/ViewControlsConstants",
		"underscore"
	],
	function (ViewportLayerModel,
		ShapeUtils,
		ListenerUtil,
		ViewControlsConstants,
		_
	) {
		"use strict";

		/**
		 * Create a new annotation layer model
		 * @exports AnnotationLayerModel
		 * @memberof AnnotationModule
		 *
		 * @class This class is responsible for drawing annotations	 *
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewerfacade
		 * @param {String} ID - Id of the layer
		 * @param {AnnotationModel} annotationModel - Model of the annotations.
		 */
		var AnnotationLayerModel = function (f, ID, annotationModel) {
			ViewportLayerModel.call(this, f, ID);
			ListenerUtil.mixin(this, f);

			this.f = f;
			this.annotationModel = annotationModel;

			this._registerListener("modelChanged", this._calculateTransformation, this, this.annotationModel);
			this._registerListener("modelContentChanged", this.modelChanged, this, this.annotationModel);
			this._registerListener("parameters.changed", this._calculateTransformation, this, this.f.getDocumentController());
			this._registerListener("transform.viewportWidget.transformChanged", this._calculateTransformation, this);
		};

		AnnotationLayerModel.prototype = Object.create(ViewportLayerModel.prototype);
		AnnotationLayerModel.prototype.constructor = AnnotationLayerModel;

		/**
		 * Calculates the transformation for the annotation layer, takes compensation for distortion and scaling into account
		 */
		AnnotationLayerModel.prototype._calculateTransformation = function () {
			var page = this.annotationModel.getAnnotationsPage();
			if (page !== null && page !== undefined) {
				this.resetTransformation();
				this.getTransform().scale(1, -1);

				var rotation = page.getRotation();

				var worldBounds = this.f.getModule("viewport").getWorldBounds();

				var offset = new ShapeUtils.Size(); //start constructing the offset needed for the origin of the annotations
				offset.add(this.annotationModel.getAnnotationsOffset()); //add the annotation offset (used in compare)

				//calculate the rest of the tranformation, for rotation, flipped state
				this.calculateTransformation(worldBounds, page.getPageSize(), offset, rotation, page.isFlipped());
			}
			this.modelChanged();
		};

		/**
		 * @override
		 */
		AnnotationLayerModel.prototype.getLinework = function (animated, rotationanimation) {
			var visibleAnnotations = this.annotationModel.getVisibleAnnotations();
			var linkedAnnotations = [];
			_.each(visibleAnnotations, function (annotation) {
				_.each(annotation.links, function (linkedAnnot) {
					linkedAnnotations.push(linkedAnnot);
				});
			});

			visibleAnnotations = _.union(visibleAnnotations, linkedAnnotations);

			/**
			 * Box with the anchor points to allow resizing of the annotation
			 */
			var selectionBoxes = [];

			/**
			 * Show sequence number
			 */
			var showNumber = this.f.getOption("PREFERENCES_ANNOTATION_SEQLABEL_VISIBLE") && !rotationanimation;

			/**
			 * Iterate over all visible annotations and add the correct info to it
			 */
			for (var i = 0; i < visibleAnnotations.length; i++) {
				var annot = visibleAnnotations[i];
				if (this.annotationModel.isAnnotationOrReplySelected(annot)) {
					annot.setSelected(true);
					if (annot.needsSelectionBox() && this.annotationModel.canChangeAnnotation(annot)) {
						selectionBoxes[i] = annot.getSelectionBox();
					}
				} else {
					annot.setSelected(false);
				}

				if (showNumber) {
					annot.updateIndicator();
					annot.showIndicator = true;
				} else {
					annot.showIndicator = false;
				}
			}

			//Return the union of the annotations + their selection boxes (flatten to make it 1 dimensional array + remove undefined and null values)
			return _.union(visibleAnnotations, linkedAnnotations, _.without(_.flatten(selectionBoxes), undefined, null));
		};

		return AnnotationLayerModel;
	});

define('web-viewer/viewport/model/ViewportPopupModel',["underscore",
		"web-viewer/core/PrivateChannel"
	],
	function (_,
		PrivateChannel) {
		/**
		 * Construct a new viewport popup model
		 * @exports ViewportPopupModel
		 * @memberof ViewportModule
		 * @class
		 * This model is responsible for the state (shown/not shown) of the popup.
		 * It also contains the popup itself.
		 * @param {Core.ViewerFacade} f - Viewerfacade
		 * @param {String} id - Id of the viewport popup model
		 */
		var ViewportPopupModel = function (f, id) {
			this.f = f;
			this.id = id;
			this.shown = true;
		};

		/**
		 * Get the id of the popup model
		 * @returns {String} The id of the popup model
		 */
		ViewportPopupModel.prototype.getId = function () {
			return this.id;
		};

		/**
		 * Indicate that the popup should be shown.
		 */
		ViewportPopupModel.prototype.show = function () {
			this.shown = true;
		};

		/**
		 * Indicate that the popup should be hidden.
		 */
		ViewportPopupModel.prototype.hide = function () {
			this.shown = false;
		};

		/**
		 * Check whether the popup is shown.
		 * @returns {Boolean} True if the popup is shown, false otherwise/
		 */
		ViewportPopupModel.prototype.isShown = function () {
			return this.shown;
		};

		/**
		 * Get the popup belonging to this model.
		 * Override this function.
		 * @param {ViewportLayer} viewportLayer - The layer where the popups should be shown.
		 * @returns {Popup} The popup belonging to this model.
		 */
		ViewportPopupModel.prototype.getPopup = function (viewportLayer) {
			// override
			return undefined;
		};

		/**
		 * Called when the popup model is destroyed.
		 * Override this function.
		 * Useful to do some cleanup.
		 */
		ViewportPopupModel.prototype.onDestroy = function () {
			// override to do some cleanup
		};

		/**
		 * Return the world origin of the popup
		 * Override this function.
		 * @returns {ShapeUtils.Point} - Origin of the popup.
		 */
		ViewportPopupModel.prototype.getWorldOrigin = function () {
			// override
			return undefined;
		};

		/**
		 * Returns the world size of the popup
		 * Override this function.
		 * @returns {ShapeUtils.Size} - Size of the popup.
		 */
		ViewportPopupModel.prototype.getWorldSize = function () {
			// override
			return undefined;
		};

		/**
		 * When there needs to be added an offset to the popup location (in scene pixels), return the offset here
		 * @returns {ShapeUtils.Size} offset in pixels that needs to be added to the popuplocation
		 */
		ViewportPopupModel.prototype.getSceneOffset = function () {
			return undefined;
		};

		/**
		 * Function called by the layer after translating the scene rectangle to world coordinates.
		 * Override this function.
		 * @param {ShapeUtils.Rect} worldRect - Rectangle in world coordinates
		 * @param {Boolean} autoAdapted - True if the event is fired because the content was auto adapted (e.g. from a makeMeVisible trigger from an element in the content), false otherwise
		 */
		ViewportPopupModel.prototype.onResized = function (worldRect, autoAdapted) {
			// override
		};

		/**
		 * Function called by the layer after translating the scene point to world coordinates.
		 * Override this function.
		 * @param {ShapeUtils.Point} worldPoint - Point in world coordinates
		 */
		ViewportPopupModel.prototype.onDragged = function (worldPoint) {
			// override
		};

		return ViewportPopupModel;
	});

define('web-viewer/annotation/AnnotationPopup',["web-viewer/annotation/AnnotationConstants",
		"web-viewer/layout/component/Popup",
		"web-viewer/layout/component/MiniButton",
		"jquery",
		"underscore"
	],
	function (AnnotationConstants,
		Popup,
		MiniButton,
		$,
		_) {
		"use strict";

		/**
		 * Create an annotation popup. 
		 * Initial (re)paint should be called by caller
		 * 
		 * @exports AnnotationPopup
		 * @memberof AnnotationModule
		 * @class
		 * The annotation popup controls the popup for annotation details
		 * @constructor	  
		 * @augments Layout.Popup
		 * @example
		 * Emitted events:
		 * 		"pinned", {Boolean} - Fired when the popup is pinned/unpinned. The argument indicates whether the new state is pinned (true) or unpinned (false).
		 * @param {Core.ViewerFacade} f - Viewerfacade
		 * @param {String} id - Id of the popup used for its private channel
		 * @param {AnnotationModule.Annotation} annotation - Annotation this popup belongs to
		 * @param {JQueryObj} container - The container where the popup should stay in
		 */
		var AnnotationPopup = function (f, id, annotation, container) {
			this.f = f;
			Popup.call(this, f, id, container);

			this.annotation = annotation;
			this.element.addClass("annotationPopup");

			// Annotation number
			this.annotationNumber = $("<span />").addClass("annotationNumber");


			// Annotation icon
			this.icon = $("<i />");

			// Title
			this.title = $("<span />").addClass("annotationTitle");
		};

		AnnotationPopup.prototype = Object.create(Popup.prototype);
		AnnotationPopup.prototype.constructor = AnnotationPopup;


		/**
		 * Initialize the popup, create needed DOM events, listeners
		 * 
		 * @private
		 * @override
		 */
		AnnotationPopup.prototype._initialize = function () {
			if (!this._isInitialized) {
				Popup.prototype._initialize.call(this);

				// Styling
				this.headerWrapper.addClass("annotationPopupHeader");

				this.header.append(this.annotationNumber);
				this.header.append(this.icon);
				this.header.append(this.title);

				// Pin Icon
				this.pinIcon = new MiniButton(this.f, "pin", "pinned icon-WCR_Pin12", true, _.bind(function (e) {
					// Pin if unpinned
					this._emit("pinned", !this.annotation.popupIsOpen);
					e.stopPropagation();
				}, this));
				this.headerIcons.prepend(this.pinIcon.element);

				// Initial paint
				AnnotationPopup.prototype.repaint.call(this);
			}
		};

		/**
		 * Repaint the popup.
		 * The pin icon is repainted.
		 * The header is repainted (annotation number, title, annotation type icon).
		 */
		AnnotationPopup.prototype.repaint = function () {
			if (this._isInitialized) {
				// Pin icon
				if (this.annotation.popupIsOpen) {
					this.pinIcon.setIcon("pinned icon-WCR_Pin12");
					this.pinIcon.setTooltip("viewer", "UnpinPopupTT");
				} else {
					this.pinIcon.setIcon("unpinned icon-WCR_Pin12");
					this.pinIcon.setTooltip("viewer", "PinPopupTT");
				}

				// Set color
				this.setAnnotationNumber(this.annotation.messagenumber);
				this.annotationNumber.css({
					'color': this.annotation.getHeaderTextColor(),
					'background-color': this.annotation.annotationcolor
				});

				// Title
				this.setTitle(this.annotation.getUserName());

				// Icon
				this.setIcon(AnnotationConstants.AnnotationIcons[this.annotation.annottype]);
			}
		};

		/**
		 * Set the title in the header
		 * @param {String} title - Title to set in the header of the popup 
		 */
		AnnotationPopup.prototype.setTitle = function (title) {
			this.title.text(title);
		};

		/**
		 * Remove this popup
		 * @override
		 */
		AnnotationPopup.prototype.remove = function () {
			this.removeAllListeners("pinned");
			Popup.prototype.remove.call(this);
		};

		/**
		 * Set the icon (annotation type) in the header
		 * @param {String} icon - Icon (css class) to set in the header of the popup
		 */
		AnnotationPopup.prototype.setIcon = function (icon) {
			this.icon.removeClass().addClass("icon").addClass(icon);
		};

		/**
		 * Set the annotation number in the header
		 * @param {String} number - Annotation number to set in the header of the popup 
		 */
		AnnotationPopup.prototype.setAnnotationNumber = function (number) {
			this.annotationNumber.text(number);
		};

		return AnnotationPopup;
	});

define('web-viewer/layout/component/RichTextEditor',["web-viewer/utils/ViewerUtils",
		"jquery",
		"underscore"
	],
	function (ViewerUtils,
		$,
		_) {
		"use strict";

		/**
		 * Create a new abstract rich text editor. The richtext editor is not shown by default, it will be initialized by calling .init()
		 * @class
		 * A rich text editor has a title, a rich text area and an action bar with some buttons.
		 * This class hangs on the DOM element in the property "richTextObj"
		 * @exports RichTextEditor
		 */
		var RichTextEditor = function (f) {
			this.f = f;
			this.onTextChangeCallbacks = [];

			// Wrapper
			this.id = ViewerUtils.generateUniqueDOMId("richText_");

			this.element = $("<div />").addClass("richText").attr("id", this.id);
			this.element.data("richTextObj", this);

			// Title bar
			this.titleBar = $("<h3 />").addClass("title");
			this.titleBar.hide();

			// Action bar
			this.actionBar = $("<div />").addClass("actionBar").addClass("clearfix");

			// Init text area
			this.textAreaDiv = $("<div />").addClass("wrapper");
			this.textArea = $("<textarea />").addClass("text");
			this.textAreaId = "textArea_" + this.id;
			this.textArea.attr("id", this.textAreaId);

			// Is the editor initialized
			this.initialized = false;

			// Add them to the wrapper
			this.element.append(this.titleBar).append(this.textAreaDiv.append(this.textArea)).append(this.actionBar);

			// should fire text changes?
			this._fireTextChanges = true;
		};

		/**
		 * Set content
		 * @param {String} content - Content to set in the text area
		 */
		RichTextEditor.prototype.setContent = function (content) {
			// to be implemented 
		};

		/**
		 * Get content as HTML
		 * @returns {String} Content as HTML if the editor was initialized, otherwise ""
		 */
		RichTextEditor.prototype.getContent = function () {
			// to be implemented
			return "";
		};

		/**
		 * Set the buttons
		 * The previous action bar is erased.
		 * @param {Array<Obj.element>} buttons - Can be every JQueryObj with a .element. If it's undefined, it's not added.
		 */
		RichTextEditor.prototype.setButtons = function (buttons) {
			this.actionBar.empty();
			_.each(buttons, function (button) {
				if (button) {
					this.actionBar.append(button.element);
				}
			}, this);
		};

		/**
		 * Set the title
		 * @param {String} newTitle - New title
		 */
		RichTextEditor.prototype.setTitle = function (newTitle) {
			this.titleBar.text(newTitle);
			this.titleBar.show();
		};

		/**
		 * Init the rich text editor
		 * Should be called after the element is added to the DOM
		 * @param {Function} [callback] - Callback when the editor is initialized (also fired when it was already initialized). From then on, you can safely use methods like getContent().
		 */
		RichTextEditor.prototype.init = function (callback) {
			// initialize the editor
		};

		/**
		 * Fire a change of the contents
		 */
		RichTextEditor.prototype._fireTextChanged = function () {
			if (this._shouldFireTextChanged()) {
				_.each(this.onTextChangeCallbacks, function (onTextChangeCallback) {
					onTextChangeCallback();
				});
			}
		};

		/**
		 * should a text change be fired?
		 * @returns {Boolean}
		 */
		RichTextEditor.prototype._shouldFireTextChanged = function () {
			// override
			return this._fireTextChanges;
		};

		/**
		 * Check whether the rich text editor is already initialized
		 * @returns {Boolean} True if the editor was initialized, false otherwise
		 */
		RichTextEditor.prototype.isInitialized = function () {
			return this.initialized;
		};

		/**
		 * Destroy the rich text editor.
		 */
		RichTextEditor.prototype.destroy = function () {
			// to be implemented
		};

		/**
		 * Add a callback to the editor that is fired when the text changes
		 * It includes onKeyUp, onUndo, onRedo
		 * @param {Function} onTextChangeCallback - executed on text change of the editor
		 */
		RichTextEditor.prototype.onTextChanged = function (onTextChangeCallback) {
			this.onTextChangeCallbacks.push(onTextChangeCallback);
		};


		/**
		 * Should be called when an editor is detached from the DOM
		 */
		RichTextEditor.prototype.beforeDetach = function () {
			// to be implemented
		};

		/**
		 * Repair the TinyMCE instance by destroying it and reinializing it
		 */
		RichTextEditor.prototype.repair = function () {
			// to be implemented
		};

		/**
		 * Set the height of the editor
		 * @param {Number} newHeight - New height of the editor
		 */
		RichTextEditor.prototype.setHeight = function (newHeight) {
			if (this.isInitialized()) {
				var height = newHeight - this.actionBar.outerHeight(true);
				height = Math.max(height, 80); // min 80 px
				this.textAreaDiv.css("height", height);
			}
		};

		return RichTextEditor;
	});

define('web-viewer/layout/component/RichTextEditorTextarea',["web-viewer/layout/component/RichTextEditor",
		"web-viewer/utils/ViewerUtils",
		"utils/ConvertUtils",
		"jquery",
		"underscore"
	],
	function (RichTextEditor,
		ViewerUtils,
		ConvertUtils,
		$,
		_) {
		"use strict";


		/**
		 * Create a new rich text editor. The richtext editor is not shown by default, it will be initialized by calling .init()
		 * @class
		 * A rich text editor has a title, a rich text area and an action bar with some buttons.
		 * This class hangs on the DOM element in the property "richTextObj"
		 * @exports RichTextEditorTextarea
		 */
		var RichTextEditorTextarea = function (f) {
			RichTextEditor.call(this, f);


		};

		RichTextEditorTextarea.prototype = Object.create(RichTextEditor.prototype);
		RichTextEditorTextarea.prototype.constructor = RichTextEditorTextarea;


		/**
		 * Set content
		 * @param {String} content - Content to set in the text area
		 * @override
		 */
		RichTextEditorTextarea.prototype.setContent = function (content) {
			this.textArea.val(ConvertUtils.convertHTMLStringToRaw(content));
		};

		/**
		 * Get content as HTML
		 * @returns {String} Content as HTML if the editor was initialized, otherwise ""
		 * @override
		 */
		RichTextEditorTextarea.prototype.getContent = function () {
			if (this.isInitialized()) {
				return "<html><body>" + ConvertUtils.convertRawStringToHTML(this.textArea.val()) + "</body></html>";
			} else {
				return "";
			}
		};

		/**
		 * Init the rich text editor
		 * Should be called after the element is added to the DOM
		 * @param {Function} [callback] - Callback when the editor is initialized (also fired when it was already initialized). From then on, you can safely use methods like getContent().
		 * @override
		 */
		RichTextEditorTextarea.prototype.init = function (callback) {
			if (!this.initialized) {
				this.initialized = true;
				this.textArea.on("keyup change", _.bind(this._fireTextChanged, this));
				this.textArea.focus();
				if (callback) {
					callback();
				}

			} else {
				if (callback) {
					callback();
				}
				this.textArea.focus();
			}
		};

		/**
		 * Destroy the rich text editor.
		 * @override
		 */
		RichTextEditorTextarea.prototype.destroy = function () {
			this.textArea.off("keyup change");
			this.initialized = false;
		};


		return RichTextEditorTextarea;

	});

define('web-viewer/layout/component/RichTextEditorTinyMCE',["web-viewer/layout/component/RichTextEditor",
		"web-viewer/utils/ViewerUtils",
		"tinyMCE",
		"jquery",
		"underscore"
	],
	function (RichTextEditor,
		ViewerUtils,
		tinyMCE,
		$) {
		"use strict";

		/**
		 * This is a list of supported languages.
		 * 
		 * Whenever TinyMCE is updated, make sure these languages are available in the tinymce/langs folder.
		 * extra note: The language file for Czech should be cz.js and not cs.js (as it is provided by TinyMCE)
		 */
		var SUPPORTED_LANGUAGES = ["br", "cn", "cz", "de", "en", "es", "fr", "hu", "it", "jp", "nl", "pl", "ru", "th", "tr", "tw"];
		var UNSUPPORTED_LANGUAGES = [];
		var DEFAULT_LANGUAGE = "en";

		/**
		 * Create a new TinyMCE rich text editor. The richtext editor is not shown by default, it will be initialized by calling .init()
		 * @class
		 * A rich text editor has a title, a rich text area and an action bar with some buttons.
		 * This class hangs on the DOM element in the property "richTextObj"
		 * @exports RichTextEditorTinyMCE
		 */
		var RichTextEditorTinyMCE = function (f) {
			RichTextEditor.call(this, f);


			this.editor = undefined;
			
			// Check if there is a custom file for this layout type
			this._cssContentFile = "../wcr-static/web-viewer/layout/component/less/tinymce/tinymce_content_" + f.getLayoutType()+".css";
		};

		RichTextEditorTinyMCE.prototype = Object.create(RichTextEditor.prototype);
		RichTextEditorTinyMCE.prototype.constructor = Object.create(RichTextEditorTinyMCE);
		/**
		 * Set content
		 * @param {String} content - Content to set in the text area
		 * @override
		 */
		RichTextEditorTinyMCE.prototype.setContent = function (content) {
			this.textArea.val(content);
			if (this.initialized) {
				this.editor.setContent(content);
			}
		};

		/**
		 * Get content as HTML
		 * @returns {String} Content as HTML if the editor was initialized, otherwise ""
		 * @override
		 */
		RichTextEditorTinyMCE.prototype.getContent = function () {
			if (this.initialized) {
				return this.editor.getContent();
			} else {
				return "";
			}
		};


		/**
		 * Init the rich text editor
		 * Should be called after the element is added to the DOM
		 * @param {Function} [callback] - Callback when the editor is initialized (also fired when it was already initialized). From then on, you can safely use methods like getContent().
		 * @override
		 */
		RichTextEditorTinyMCE.prototype.init = function (callback) {
			if (!this.initialized) {
				var userLocale = this.f.getCurrentUser().getLocale();

				if (_.contains(SUPPORTED_LANGUAGES, userLocale)) {
					// we support the user locale for TinyMCE, initialize TinyMCE
					this._initMCE(callback, userLocale);
				} else {
					// we don't support this locale by default... check if the locale is available on the server
					if (_.contains(UNSUPPORTED_LANGUAGES, userLocale)) {
						// we already checked, and this language doesn't exist... load default language
						this._initMCE(callback, DEFAULT_LANGUAGE);
					} else {
						// check if the language file exist					
						this.f.ajax(requirejs.toUrl("lib/tinymce/langs/" + userLocale + ".js"), {
							dataType: "text",
							timeout: 2000,
							success: _.bind(function () {
								// this language is also supported, register it, and initialize TinyMCE
								SUPPORTED_LANGUAGES.push(userLocale);
								this._initMCE(callback, userLocale);
							}, this),
							error: _.bind(function () {
								// this language doesn't exist, register it as unsupported and initialize TinyMCE
								UNSUPPORTED_LANGUAGES.push(userLocale);
								this._initMCE(callback, DEFAULT_LANGUAGE);
							}, this)
						});
					}
				}
			} else {
				if (callback) {
					callback();
				}
				this.editor.focus();
			}
		};

		/**
		 * Start up TinyMCE
		 * @param {Function} callback - callback function to be called when TinyMCE is initialized 
		 * @param {String} locale - the locale in which TinyMCE should be started
		 */
		RichTextEditorTinyMCE.prototype._initMCE = function (callback, locale) {
			tinyMCE.init({
				selector: "#" + this.textAreaId,
				menubar: false,
				statusbar: false,
				plugins: "paste fullpage",
				toolbar: "bold italic underline strikethrough | subscript superscript | undo redo",
				toolbar_items_size: "small",
				skin: 'lightgrayflat',
				content_css: this._cssContentFile,
				element_format: "xhtml",
				doctype: "<!DOCTYPE html PUBLIC '-//W3C//DTD XHTML 1.0 Strict//EN' " + "'http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd'>",
				fullpage_default_doctype: "<!DOCTYPE html PUBLIC '-//W3C//DTD XHTML 1.0 Strict//EN' " + "'http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd'>",
				inline_styles: true,
				language: locale,
				valid_elements: "p[style],p[style]/li,div,span[style],span[style],br,nobr,sub,sup,strike/s,strong/b,i,u,s,a,h?,big,small,font,tt,code,var,kbd,em,samp,center,a[href|title|style]",
				auto_focus: this.textAreaId,
				setup: _.bind(function (ed) {
					this.editor = ed;
					ed.on("keyup", _.bind(function (e) {
						this._fireTextChanged();
					}, this));
					ed.on("redo", _.bind(function (e) {
						this._fireTextChanged();
					}, this));
					ed.on("undo", _.bind(function (e) {
						this._fireTextChanged();
					}, this));
					ed.on("change", _.bind(function (e) {
						this._fireTextChanged();
					}, this));
					ed.on('init', _.bind(function (e) {
						this.initialized = true;
						if (callback) {
							callback();
						}
					}, this));
				}, this),
				// this method is called before text is pasted in the richtext editor
				paste_preprocess: function (pl, o) {
					// convert the tables to div/p/span elements, as the java viewer doesn's support tables
					o.content = o.content.replace(/<(\/)?table[^>]*>/gi, "<$1div>");
					o.content = o.content.replace(/<(\/)?tr[^>]*>/gi, "<$1p>");
					o.content = o.content.replace(/<th[^>]*>/gi, "<span><strong>");
					o.content = o.content.replace(/<\/th[^>]*>/gi, "</strong></span>&nbsp;");
					o.content = o.content.replace(/<td[^>]*>/gi, "<span>");
					o.content = o.content.replace(/<\/td[^>]*>/gi, "</span>&nbsp;");
				},
				height: "100%"
			});
		};

		/**
		 * should a text change be fired?
		 * @returns {Boolean}
		 * @override
		 */
		RichTextEditorTinyMCE.prototype._shouldFireTextChanged = function () {
			return this.editor && this.editor.isDirty() && this._fireTextChanges;
		};


		/**
		 * Destroy the rich text editor.
		 * @override
		 */
		RichTextEditorTinyMCE.prototype.destroy = function () {
			if (this.initialized) {
				this.editor.destroy();
			}
			this.initialized = false;
		};

		/**
		 * Should be called when an editor is detached from the DOM
		 * @override
		 */
		RichTextEditorTinyMCE.prototype.beforeDetach = function () {
			if (this.isInitialized()) {
				this._fireTextChanges = false;
				this.destroy();
				this._fireTextChanges = true;
			}
		};

		/**
		 * Repair the TinyMCE instance by destroying it and reinializing it
		 * @override
		 */
		RichTextEditorTinyMCE.prototype.repair = function () {
			this.init();
		};

		/**
		 * @override
		 */
		RichTextEditorTinyMCE.prototype.setHeight = function (newHeight) {
			if (this.isInitialized()) {
				// just calculating it here doesn't work as tinymce loads its css asynchronously
				var toolbarHeight = this.textAreaDiv.find(".mce-toolbar-grp").outerHeight(true);
				// THIS IS A VERY UGLY HACK but there is no usable alternative known at the moment!
				// one toolbar row is 26px, 2px padding at top and bottom
				if (toolbarHeight % 26 !== 4) {
					toolbarHeight = 30;
				}
				var height = newHeight - this.actionBar.outerHeight(true) - toolbarHeight;
				height = Math.max(height, 80); // min 80 px
				var iframe = "#" + this.textAreaId + "_ifr";
				$(iframe).css("height", height);
			}
		};

		return RichTextEditorTinyMCE;
	});

define('web-viewer/layout/component/RichTextEditorFactory',["web-viewer/layout/component/RichTextEditorTextarea",
		"web-viewer/layout/component/RichTextEditorTinyMCE"
	],
	function (RichTextEditorTextarea,
		RichTextEditorTinyMCE) {
		"use strict";

		/**
		 * Create a new TinyMCE rich text editor. The richtext editor is not shown by default, it will be initialized by calling .init()
		 * @class
		 * A rich text editor has a title, a rich text area and an action bar with some buttons.
		 * This class hangs on the DOM element in the property "richTextObj"
		 * @exports RichTextEditor
		 */
		return {
			/**
			 * Create a new editor instance, depending on the used  device a TinyMCE or Textarea editor is created
			 * @param {ViewerFacade} f
			 * @returns {RichTextEditor}
			 */
			createEditor: function (f) {
				if (f.getFeatureDetector().isIOS()) { // on iOS tinyMCE has some serious (native) issues, load a textarea editor instead
					return new RichTextEditorTextarea(f);
				} else {
					return new RichTextEditorTinyMCE(f);
				}
			}
		};

	});

define('web-viewer/annotation/AnnotationCharLineCount',["web-viewer/utils/ViewerUtils",
		"component/tooltip/Tooltip",
		"utils/ConvertUtils",
		"jquery",
		"underscore"
	],
	function (ViewerUtils,
		Tooltip,
		ConvertUtils,
		$,
		_) {
		"use strict";
		/**
		 * @exports AnnotationCharLineCount
		 * @memberof AnnotationModule
		 * Constructs a new AnnotationCharLineCount element
		 * @class
		 * Annotation Char Line Count is an element that shows the character/line count of an annotation, including tooltips.
		 * It can be shown/hidden and the text can be updated.
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 */
		var AnnotationCharLineCount = function (f) {
			this.element = $("<div />").addClass("charLineCount");
			this.showing = false;
			this.f = f;
			this.tooltip = null;
		};

		/**
		 * Update the character line count with the newest count for the given rawHTMLString
		 * @param {String} rawHTMLString - HTML string to count the characters/lines from
		 */
		AnnotationCharLineCount.prototype.update = function (rawHTMLString) {
			if (this.showing) {
				// Update the count
				var charLineCount = ConvertUtils.getCharacterLineCount(rawHTMLString);
				this.element.text(charLineCount.charCount + "/" + charLineCount.lineCount);
				// Update the tooltip
				if (this.tooltip) {
					this.tooltip.setTooltipContent("viewer", "charLineCount", [charLineCount.charCount, charLineCount.lineCount]);
				} else {
					this.tooltip = new Tooltip(this.f.i18n, this.element, this.f.getContext(), this.f.getContext(), this.f.isMobile(), "viewer", "charLineCount", [charLineCount.charCount, charLineCount.lineCount]);
				}
			}
		};

		/**
		 * Show the character line count element
		 */
		AnnotationCharLineCount.prototype.show = function () {
			this.element.show();
			this.showing = true;
		};

		/**
		 * Hide the character line count
		 */
		AnnotationCharLineCount.prototype.hide = function () {
			this.element.hide();
			this.showing = false;
		};

		return AnnotationCharLineCount;
	});

define('web-viewer/layout/AutoTimeLabel',["component/tooltip/Tooltip",
		"web-viewer/utils/ViewerUtils",
		"utils/ConvertUtils",
		"jquery"
	],
	function (Tooltip,
		ViewerUtils,
		ConvertUtils,
		$) {
		"use strict";
		var AutoTimeLabel = function (f, date) {
			this.editOnTop = true;

			this.element = $("<span>");

			this.date = date;

			this.tooltip = new Tooltip(f.i18n, this.element, f.getContext(), f.getContext(), f.isMobile(), ConvertUtils.getLangFormattedDateAndTime(f.i18n, this.date));

			this.updateText = function (isCreationDate) {
				f.i18n.loadBundle("viewer", _.bind(function () {
					this.element.text(userFriendlyDate(this.date));
					var dateString = "";
					if (isCreationDate) {
						dateString = f.i18n.t("createdOn");
					} else {
						dateString = f.i18n.t("modifiedOn");
					}
					this.tooltip.setTooltipContent(dateString + ConvertUtils.getLangFormattedDateAndTime(f.i18n, this.date));
				}, this));
			};

			this.updateText();

			function userFriendlyDate(date) {
				var seconds = Math.floor((new Date().getTime() - date.getTime()) / 1000);

				var days = Math.floor(seconds / (3600 * 24));
				seconds -= days * 3600 * 24;

				var hours = Math.floor(seconds / 3600);
				seconds -= hours * 3600;

				var minutes = Math.floor(seconds / 60);
				seconds -= minutes * 60;

				var ago;
				var text = null;
				if (days > 0) {
					if (days == 1) {
						text = f.i18n.t("dateYesterday");
					} else if (days < 5) {
						ago = days + f.i18n.t("daysAbbreviation");
					} else {
						text = ConvertUtils.getLangFormattedDate(f.i18n, date);
					}
				} else if (hours > 0) {
					ago = hours + f.i18n.t("hoursAbbreviation");
				} else if (minutes > 0) {
					ago = minutes + f.i18n.t("minutesAbbreviation");
				} else {
					ago = seconds + f.i18n.t("secondsAbbreviation");
					ago = "< 1" + f.i18n.t("minutesAbbreviation");
				}

				return text ? text : f.i18n.t("timeAgo", ago);
			}


		};


		return AutoTimeLabel;
	});

define('web-viewer/annotation/AnnotationSingleRow',["web-viewer/layout/AutoTimeLabel",
		"web-viewer/annotation/AnnotationConstants",
		"web-viewer/annotation/AnnotationCharLineCount",
		"web-viewer/layout/component/MiniButton",
		"web-viewer/layout/component/DropdownMiniButton",
		"web-viewer/layout/component/ViewerToolbarButton",
		"component/toolbar/ButtonGroup",
		"component/tooltip/Tooltip",
		"web-viewer/layout/component/RichTextEditorFactory",
		"web-viewer/layout/component/ColorPicker",
		"jquery",
		"underscore",
		"HTMLTruncator"
	],
	function (AutoTimeLabel,
		AnnotationConstants,
		AnnotationCharLineCount,
		MiniButton,
		DropdownMiniButton,
		ViewerToolbarButton,
		ButtonGroup,
		Tooltip,
		RichTextEditorFactory,
		ColorPicker,
		$,
		_) {
		"use strict";

		/**
		 * @exports AnnotationSingleRow
		 * @memberof AnnotationModule
		 * @class An annotation div is an interface element which provides one line for an annotation
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewerfacade
		 * @param {Annotation} annotation - Annotation this single row belongs to
		 * @param {Boolean} main - indicates if this is the main annotation
		 * @param {AnnotationModel} annotationModel - Annotation Model
		 * @param {Boolean} hideHeader - True if you want to hide the header, false otherwise
		 * @param {JQueryObj} extraElement - Extra element to be added to the actions div
		 * @param {Boolean} isForMobile
		 */
		var AnnotationSingleRow = function (f, annotation, main, annotationModel, hideHeader, extraElement, isForMobile) {
			this.element = $("<div/>").addClass(main ? "main" : "reply");
			this.selected = false;

			this.f = f;

			this._MAX_CHARS = 300;

			this._hideHeader = hideHeader;

			this._annotation = annotation;

			this._main = main;

			this._annotationModel = annotationModel;

			this._isForMobile = isForMobile;

			this._visible = true;


			/**
			 * Header
			 */
			if (!this._hideHeader) {
				var header = $("<div/>").addClass("annotHeader");
				this.element.append(header);
				if (this._main) {
					header.append($("<i>").addClass("icon").addClass(this._annotation.getIconClass()));
				}
				var userName = $("<span />").addClass("userName").text(this._annotation.getUserName());
				header.append(userName);

				if (this._main && !this._isForMobile) {
					/**
					 * 3D View button for desktop
					 */
					this._view3D = new DropdownMiniButton(this.f, "3DView", "icon icon-WCR_Actions-3D12");
					this._view3D.setTooltip("viewer", "annot3DView");
					header.append(this._view3D.element.addClass("view3d"));
				}

				this._timelabel = new AutoTimeLabel(this.f, this._annotation.date);
				this._timelabel.element.addClass("date").addClass("pull-right");
				header.append(this._timelabel.element);

				this.updateTimer = _.bind(function () {
					this._timelabel.date = this._annotation.date;
					this._timelabel.updateText(this._annotation.isAnnotationModified());
				}, this);
			}

			/**
			 * Content
			 */
			this._content = $("<div/>").addClass("annotContent");
			this.element.append(this._content);

			/**
			 * Actions
			 */
			this._actionsDiv = $("<div/>").addClass("annotActions").addClass("clearfix");


			// Edit + delete actions	
			this._moreActions = new DropdownMiniButton(this.f, "moreActions", "icon-WCR_More-horizontal");
			this._moreActions.addButtonClass("annotAction");
			this._moreActions.setTooltip("viewer", "moreActions");
			// Rich text
			this._richTextEditor = undefined;
			this._annotationColorPicker = undefined;

			this._actionsDiv.append(this._moreActions.element);

			// Buttons in the editor
			// Save button
			var saveEdit = new ViewerToolbarButton(this.f, "saveEdit", "icon-WCR_Save12 marginRight5", true, _.bind(function (e) {
				this._annotation.content = this._richTextEditor.getContent();
				this._annotation.setModified("CONTENT");
				this._annotationModel.save();
				this.closeRichText();
				e.stopPropagation();
			}, this));
			// Cancel button
			var cancelEdit = new ViewerToolbarButton(this.f, "cancelReply", "icon-WCR_Revert12 marginRight5", true, _.bind(function (e) {
				this._annotation.resetAnnotationColor();
				this._annotationModel.modelChanged(this._annotation);
				this._annotation.cancelModifiedColor();
				this._richTextEditor.setContent("");
				this._annotation.cancelModified("CONTENT");
				this._annotationModel.modelChanged();
				this.closeRichText();
				e.stopPropagation();
			}, this));
			// Save and cancel are a buttongroup
			this._editActionsGroup = new ButtonGroup("editActionsGroup", [cancelEdit, saveEdit]);
			this._editActionsGroup.element.addClass("pull-right");

			if (annotation.getPermissionFor(AnnotationConstants.ActionPermissions.DUPLICATE_WITHOUT_TEXT)) {
				// duplicateWithoutText
				this._duplicateWithoutText = new MiniButton(this.f, "annotationCopyWithoutText", "icon-WCR_Annotation-Duplicate-NoText", true, _.bind(function (e) {
					this.f.emit("module.annotation.duplicateWithoutText", this._annotation);
					e.stopPropagation();
				}, this));
				this._duplicateWithoutText.addButtonClass("annotAction");
				this._duplicateWithoutText.setTooltip("viewer", "DuplicateWithoutTextAnnotBtnTT");
				this._actionsDiv.append(this._duplicateWithoutText.element);
			}

			// duplicate
			if (annotation.getPermissionFor(AnnotationConstants.ActionPermissions.DUPLICATE)) {
				this._duplicateAction = new MiniButton(this.f, "annotationDuplicate", "icon-WCR_Annotation-Duplicate", true, _.bind(function (e) {
					this.f.emit("module.annotation.duplicateAnnotation", this._annotation);
					e.stopPropagation();
				}, this));
				this._duplicateAction.addButtonClass("annotAction");
				if (this._annotation.originType === AnnotationConstants.AnnotationOriginTypes.user.id) {
					this._duplicateAction.setTooltip("viewer", "DuplicateAnnotBtnTT");
				} else {
					this._duplicateAction.setTooltip("viewer", "DuplicateToUserAnnotBtnTT");
				}
				this._actionsDiv.append(this._duplicateAction.element);
			}

			if (this._main) {
				// 3D View mobile
				if (this._isForMobile) {
					this._view3DMobile = new MiniButton(this.f, "3DView", "icon icon-WCR_Actions-3D12", true, _.bind(function (e) {
						this.f.emit("module.annotation.show3DView", this._annotation);
					}, this));
					this._view3DMobile.addButtonClass("annotAction");
					this._view3DMobile.setTooltip("viewer", "annot3DView");
					this._actionsDiv.append(this._view3DMobile.element);
				}

				// Zoom
				this._zoomAction = new MiniButton(this.f, "annotationZoom", "icon-WCR_ZoomFit", true, _.bind(function (e) {
					this.f.emit("module.annotation.zoomToAnnotation", this._annotation);
					e.stopPropagation();
				}, this));
				this._zoomAction.addButtonClass("annotAction");
				this._zoomAction.setTooltip("viewer", "ZoomAnnotBtnTT");
				this._actionsDiv.append(this._zoomAction.element);
			}

			// Review status
			this._statusAction = new DropdownMiniButton(this.f, "setReviewStatus", "icon-WCR_Approve-submenu");
			this._statusAction.addButtonClass("annotAction");
			this._statusAction.setTooltip("viewer", "SetReviewStateBtnTT");
			this._actionsDiv.append(this._statusAction.element);

			if (this._isForMobile && extraElement !== undefined) {
				this._actionsDiv.append(extraElement);
			}

			// Character/Line count for annotation
			this._charLineCount = new AnnotationCharLineCount(this.f);
			this._actionsDiv.append(this._charLineCount.element);

			this.element.append(this._actionsDiv);

			//reviews
			this._reviews = $("<ul/>").addClass("Review").hide();
			this.element.append(this._reviews);

			this.element.mouseenter(_.bind(function () {
				this.mouseEntered = true;
				this.showOrHideOptions();
			}, this)).mouseleave(_.bind(function () {
				this.mouseEntered = false;
				this.showOrHideOptions();
			}, this));

			this.f.i18n.loadBundle("viewer", _.bind(function () {
				this._moreActions.addAction({
					name: "annotationEdit",
					content: this.f.i18n.t("EditAnnotBtnTT"),
					icon: "icon-WCR_Annotate",
					action: _.bind(function () {
						if (!this._annotation.isNew) {
							this._initializeRichTextEditor();
							if (!this._richTextEditor.isInitialized()) {
								// only add first time
								this._content.after(this._richTextEditor.element);
								this._richTextEditor.element.trigger("makeMeVisible");
							}
							// On click, show rich text
							this._richTextEditor.element.show();
							this._content.hide();

							// Hide the actions div, move the char line counter in the editor
							this._actionsDiv.hide();
							this._charLineCount.element.insertAfter(this._editActionsGroup.element);

							// update
							this._richTextEditor.setContent("<html>" + this._annotation.content + "</html>");
							this.updateColorPicker();

							this._richTextEditor.init();
						} else {
							this._annotationModel.selectAnnotation(this._annotation, true);
						}
					}, this)
				});

				this._moreActions.addAction({
					name: "duplicateAnnotation",
					content: this.f.i18n.t("DuplicateAnnotBtnTT"),
					icon: "icon-WCR_Annotation-Duplicate",
					action: _.bind(function (e) {
						if (e.which) {
							this.f.emit("module.annotation.duplicateAnnotation", this._annotation);
							e.stopPropagation();
						}
					}, this)
				});

				if (this._annotation.getPermissionFor(AnnotationConstants.ActionPermissions.DUPLICATE_WITHOUT_TEXT)) {
					this._moreActions.addAction({
						name: "duplicateWithoutText",
						content: this.f.i18n.t("DuplicateWithoutTextAnnotBtnTT"),
						icon: "icon-WCR_Annotation-Duplicate-NoText",
						action: _.bind(function (e) {
							if (e.which) {
								this.f.emit("module.annotation.duplicateWithoutText", this._annotation);
								e.stopPropagation();
							}
						}, this)
					});
				}

				this._moreActions.addAction({
					name: "annotationSave",
					content: this.f.i18n.t("SaveAnnotBtnTT"),
					icon: "icon-WCR_Save",
					action: _.bind(function (e) {
						this._annotationModel.save();
					}, this)
				});


				this._moreActions.addAction({
					name: "annotationDelete",
					content: this.f.i18n.t("DeleteAnnotBtnTT"),
					icon: "icon-WCR_Trash",
					action: _.bind(function (e) {
						this._annotationModel.deleteAnnotation(this._annotation);
					}, this)
				});

				// 3D button actions in more actions
				if (this._main) {
					if(!this._isForMobile) {
						this._moreActions.addAction({
							name: "update3DView",
							content: this.f.i18n.t("annot3DViewUpdate"),
							icon: "icon icon-WCR_Annotation-3D-Update",
							action: _.bind(function (e) {
								this.f.emit("module.annotation.update3DView", this._annotation);
							}, this)
						});
					
					
						this._moreActions.addAction({
							name: "delete3DView",
							content: this.f.i18n.t("annot3DViewDelete"),
							icon: "icon icon-WCR_Annotation-3D-Delete",
							action: _.bind(function (e) {
								this.f.emit("module.annotation.delete3DView", this._annotation);
							}, this)
						});
					}
				}

				saveEdit.element.addClass("btn-mini").addClass("btn-success").append(this.f.i18n.t("Save"));
				cancelEdit.element.addClass("btn-mini").append(this.f.i18n.t("CancelButton"));

				// Create buttons
				_.each(AnnotationConstants.AnnotationReviewStateLocs, _.bind(function (loc, name, i) {
					if (name != "STATUS_ANY") {
						this._statusAction.addAction({
							name: name,
							content: this.f.i18n.t(loc),
							action: _.bind(function () {
								this._annotationModel.setState(this._annotation, AnnotationConstants.AnnotationReviewState[name]);
							}, this),
							icon: AnnotationConstants.AnnotationReviewStateIcons[name]
						});
					}
				}, this));

				// 3D button actions
				if (this._view3D) {
					this._view3D.addAction({
						name: "update3DView",
						content: this.f.i18n.t("annot3DViewUpdate"),
						icon: "icon icon-WCR_Annotation-3D-Update",
						action: _.bind(function (e) {
							this.f.emit("module.annotation.update3DView", this._annotation);
						}, this)
					});

					this._view3D.addAction({
						name: "delete3DView",
						content: this.f.i18n.t("annot3DViewDelete"),
						icon: "icon icon-WCR_Annotation-3D-Delete",
						action: _.bind(function (e) {
							this.f.emit("module.annotation.delete3DView", this._annotation);
						}, this)
					});
				}

				this._updateView3DButtons();
			}, this));
		};

		/**
		 * Close rich text editor and replace with content
		 */
		AnnotationSingleRow.prototype.closeRichText = function () {
			// reset the buttons
			this._actionsDiv.show();
			this._actionsDiv.append(this._charLineCount.element);
			// on cancel, show input
			this._content.show();
			this._richTextEditor.element.hide();
		};

		/**
		 * Show or hide the character line count in the annotation row
		 * @param {Boolean} show - show the char count
		 */
		AnnotationSingleRow.prototype.showHideCharLineCount = function (show) {
			var mainAnnotation = this._annotation.getGodfather();
			if (show && this._annotationModel.isAnnotationOrReplySelected(mainAnnotation)) {
				this._charLineCount.show();
				this._charLineCount.update(this._annotation.content);
			} else {
				this._charLineCount.hide();
			}
		};

		/**
		 * Repaint the entire row
		 */
		AnnotationSingleRow.prototype.repaintRow = function () {
			if (!this._hideHeader) {
				this.updateTimer();
			}
			if (!this._annotation.isBeingModified() && this._richTextEditor !== undefined) {
				this.closeRichText();
			} else {
				this.updateColorPicker();
			}

			this._updateView3DButtons();

			this._content.empty();
			this._content.removeClass("noContent");

			// Extra content
			var extraContent = this._annotation.getExtraContent();
			if (extraContent) {
				if (extraContent.element) {
					this._content.append(extraContent.element);

					if (extraContent.tooltip) {
						if (this.extraContentTooltip) {
							this.extraContentTooltip.destroy();
						}
						this.extraContentTooltip = new Tooltip(this.f.i18n, extraContent.element, this.f.getContext(), this.f.getContext(), this.f.isMobile(), extraContent.tooltip);
					}
				}
			}

			if (this._annotation.content && this._annotation.content.length > 0) {
				var wrapper = $("<div>").html(this._annotation.content);
				this._content.append(wrapper);
				this.f.i18n.loadBundle("viewer", _.bind(function () {
					wrapper.truncate({
						max_length: this._MAX_CHARS,
						more: this.f.i18n.t("ReadMore"),
						less: this.f.i18n.t("ReadLess"),
						css_more_class: 'truncator-link truncator-more',
						css_less_class: 'truncator-link truncator-less',
						link_prefix: '',
						link_suffix: ''
					});
				}, this));
			} else {
				if (!extraContent) {
					this.f.i18n.loadBundle("viewer", function () {
						this._content.addClass("noContent");
						if (this._annotation.isNew) {
							this._content.text(this.f.t("AnnNewClickToEdit"));
						} else {
							this._content.text(this.f.t("AnnNoContent"));
						}
					}, this);
				}
			}

			this.showOrHideOptions();

			this._updateReviews(this._annotation.states);

			this.showHideCharLineCount(this.f.getOption("PREFERENCES_ANNOTATION_SHOWCOUNT"));

			if (this._annotationModel.canChangeAnnotations(true)) {
				if (this._annotationModel.canChangeAnnotation(this._annotation)) {
					this._moreActions.show();
					if (this._duplicateAction) {
						this._duplicateAction.hide();
					}
					if (this._duplicateWithoutText) {
						this._duplicateWithoutText.hide();
					}
				} else {
					this._moreActions.hide();
					if (this._duplicateAction) {
						this._duplicateAction.show();
					}
					if (this._duplicateWithoutText) {
						this._duplicateWithoutText.show();
					}
				}
				this._statusAction.show();

				this._moreActions.setActionEnabled("annotationDelete", this._annotationModel.canDeleteAnnotation(this._annotation));
			} else {
				if (this._duplicateAction) {
					this._duplicateAction.hide();
				}
				if (this._duplicateWithoutText) {
					this._duplicateWithoutText.hide();
				}
				this._moreActions.hide();
				this._statusAction.hide();
			}
		};

		/**
		 * Checks if the row should show all the option-buttons
		 * @returns {Boolean}
		 */
		AnnotationSingleRow.prototype.shouldShowOptions = function () {
			if (this._isForMobile) {
				return this.selected;
			} else {
				return this.selected || this.mouseEntered || this._moreActions.isOpen() || this._statusAction.isOpen();
			}
		};

		/**
		 * Shows or hides the options, uses shouldShowOptions to check whether to show or hide
		 */
		AnnotationSingleRow.prototype.showOrHideOptions = function () {
			this.showOptions(this.shouldShowOptions());
		};

		/**
		 * Shows the extra annotation action-buttons (options)
		 * @param {Boolean} show
		 */
		AnnotationSingleRow.prototype.showOptions = function (show) {
			if (show) {
				if (this._isForMobile) {
					this._actionsDiv.show();
				} else {
					this._actionsDiv.removeClass("hideOptions");
				}
			} else {
				if (this._isForMobile) {
					this._actionsDiv.hide();
				} else {
					this._actionsDiv.addClass("hideOptions");
				}
			}
		};

		/**
		 * Update all the annotation review states
		 * @param {Array.<Annotation>} states
		 */
		AnnotationSingleRow.prototype._updateReviews = function (states) {
			this.f.i18n.loadBundle("viewer", function () {
				this._reviews.empty();
				for (var i = 0; i < states.length; i++) {

					var review = $("<li/>");

					var container = $("<a/>");
					var image = $("<i/>").addClass("icon").addClass(AnnotationConstants.AnnotationReviewStateIcons[_.invert(AnnotationConstants.AnnotationReviewState)[states[i].state]]);
					var reviewStateKeys = _.invert(AnnotationConstants.AnnotationReviewState);
					var reviewState = reviewStateKeys[states[i].state];
					var user = $("<span/>").text("\"" + this.f.i18n.t(AnnotationConstants.AnnotationReviewStateLocs[reviewState]) + "\": " + states[i].getUserName());

					container.append(image);
					container.append(user);
					review.append(container);

					this._reviews.append(review);
				}
			}, this);
			this._reviews.toggle(states.length > 0);
		};

		/**
		 * initialize the rich text editor of the annotation row
		 */
		AnnotationSingleRow.prototype._initializeRichTextEditor = function () {
			if (!this._richTextEditor) {
				this._richTextEditor = RichTextEditorFactory.createEditor(this.f);

				// Show a color picker if no annotation color is set
				//color picker only added for main annotation
				if (!this.f.getProperty("annotationcolor") && this._main) {
					this._annotationColorPicker = new ColorPicker(_.bind(function (newColor) {
						this._annotation.setColor("#" + newColor);
						this._annotationModel.modelChanged(this._annotation);
					}, this));

					this._annotationColorPicker.setColor(this._annotation.annotationcolor);
					this._annotationColorPicker.element.addClass("annotationColorPicker");
					this._annotationColorPicker.setTooltip(this.f, "general", "ChangeColor");
				}

				// Add to rich text editor
				this._richTextEditor.setButtons([this._editActionsGroup, this._annotationColorPicker]);

				// Let the counter listen to the richtext editor
				this._richTextEditor.onTextChanged(_.bind(function () {
					this._charLineCount.update(this._richTextEditor.getContent());
					this._annotation.setBeingModified(this._richTextEditor.getContent());
					this._annotationModel.modelChanged();
				}, this));
			}
		};

		/**
		 * Update 3D view button
		 */
		AnnotationSingleRow.prototype._updateView3DButtons = function () {
			var has3Dview = this._annotation.get3DViewPosition() !== null;

			var view3DModule = this.f.getModule("view3D");
			var view3DWidgetVisible = view3DModule && view3DModule.is3DWidgetEnabled();

			var canChange = this._annotationModel.canChangeAnnotation(this._annotation);

			if (this._view3D) {
				this._view3D.toggleButton(has3Dview);
				this._view3D.toggleElementVisibility("update3DView", canChange && view3DWidgetVisible);
				this._view3D.toggleElementVisibility("delete3DView", canChange && has3Dview);

				if (canChange) {
					this._view3D.enableDropdown();
				} else {
					this._view3D.disableDropdown();
				}
			}

			if (this._view3DMobile) {
				if (has3Dview) {
					this._view3DMobile.show();
				} else {
					this._view3DMobile.hide();
				}
			}

			if (this._moreActions) {
				this._moreActions.toggleElementVisibility("delete3DView", canChange && has3Dview);
				this._moreActions.toggleElementVisibility("update3DView", canChange && view3DWidgetVisible);
			}
		};

		/**
		 * set if the annotation is selected, the row should be highlighted
		 * @param {Boolean} selected
		 */
		AnnotationSingleRow.prototype.setSelected = function (selected) {
			this.selected = selected;
			this.element.toggleClass("selected", this.selected);
			this.repaintRow();
		};

		/**
		 * Should be called when the annotation row is detached from the DOM
		 */
		AnnotationSingleRow.prototype.onDetachFromDOM = function () {
			if (this._richTextEditor) {
				this._richTextEditor.beforeDetach();
			}
			this._hideAllTooltips();
		};

		/**
		 * Hide all tooltips
		 */
		AnnotationSingleRow.prototype._hideAllTooltips = function () {
			if (this.extraContentTooltip) {
				this.extraContentTooltip.hide();
			}

			if (this._duplicateAction) {
				this._duplicateAction.hideTooltip();
			}

			if (this._duplicateWithoutText) {
				this._duplicateWithoutText.hideTooltip();
			}

			if (this._zoomAction) {
				this._zoomAction.hideTooltip();
			}

			this._timelabel.tooltip.hide();
			this._moreActions.hideTooltip();

		};

		/**
		 * Should be called when the annotation row gets retached to the DOM
		 */
		AnnotationSingleRow.prototype.onReattachToDOM = function () {
			if (this._richTextEditor) {
				this._richTextEditor.repair();
			}
		};

		/**
		 * Updates the color picker
		 */
		AnnotationSingleRow.prototype.updateColorPicker = function () {
			if (this._annotationColorPicker) {
				this._annotationColorPicker.setColor(this._annotation.annotationcolor);
			}
		};

		/**
		 * Set the row visible or not
		 * @param {Boolean} visible - True for show, false for hide
		 */
		AnnotationSingleRow.prototype.setVisible = function (visible) {
			this._visible = visible;
			this.element.toggle(visible);
		};

		/**
		 * Is the row currently visible or not
		 * @returns {Boolean} True if visible, false otherwise
		 */
		AnnotationSingleRow.prototype.isVisible = function () {
			return this._visible;
		};

		/**
		 * Set the Annotation for this row and repaint
		 * @param {Annotation} annotation - the new annotation object
		 */
		AnnotationSingleRow.prototype.setAnnotation = function (annotation) {
			this._annotation = annotation;
			this.repaintRow();
		};


		return AnnotationSingleRow;
	});

define('web-viewer/annotation/AnnotationRow',["web-viewer/annotation/AnnotationSingleRow",
		"web-viewer/layout/component/ViewerToolbarButton",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/RichTextEditorFactory",
		"web-viewer/annotation/AnnotationCharLineCount",
		"web-viewer/annotation/AnnotationConstants",
		"web-viewer/layout/component/MiniButton",
		"jquery",
		"underscore"
	],
	function (AnnotationSingleRow,
		ViewerToolbarButton,
		ButtonGroup,
		RichTextEditorFactory,
		AnnotationCharLineCount,
		AnnotationConstants,
		MiniButton,
		$,
		_) {
		"use strict";

		/**
		 * @exports AnnotationRow
		 * @memberof AnnotationModule
		 * @class An annotation div is an interface element which provides one line for an annotation
		 * @constructor
		 * @param {Annotation} annotation
		 * @param {AnnotationModel} annotationModel
		 * @param {Boolean} popup
		 * @param {Number} replyLimit
		 * @param {Boolean} isForMobile
		 */
		var AnnotationRow = function (f, annotation, annotationModel, popup, replyLimit, isForMobile) {

			this.f = f;
			this._replyLimit = replyLimit;
			if (!this._replyLimit) {
				this._replyLimit = 2;
			}

			this._annotation = annotation;
			this._annotationModel = annotationModel;

			this._limitReplyCount = true;
			this._shownReplyCount = 0;
			this._totalReplyCount = 0;
			this.name = this._annotation.name;
			this.element = $("<div>").addClass("annotation").toggleClass("cannot-reply", !this._annotation.getPermissionFor(AnnotationConstants.ActionPermissions.CAN_BE_REPLIED_TO));

			this.annotationRows = {};

			this.selected = false;

			this._isForMobile = isForMobile;

			/**
			 * Replies
			 */
			this._repliesDiv = $("<div/>").addClass("replies");


			/**
			 * More Replies
			 */
			this._moreReplies = $("<p/>").addClass("MoreReplies clearfix");
			this._moreRepliesLabel = $("<a/>");

			this._moreRepliesLabel.on('click', _.bind(function (e) {
				this.toggleReplyLimit();
				e.stopPropagation();
			}, this));
			this._moreReplies.append(this._moreRepliesLabel);
			this._moreReplies.hide();


			if (!popup) {
				this.title = $("<span/>").addClass("annotationNumber");
				this.element.append(this.title);
			}

			/**
			 * Reply input
			 */
			if (isForMobile) {
				var replyButton = new MiniButton(this.f, "annotationAddReply", "icon-WCR_Comment", true, _.bind(function (e) {
					this._showRichTextEditor(e);
				}, this));
				replyButton.addButtonClass("annotAction");
				this._replyText = replyButton.element;
			} else {
				this._replyText = $("<input/>");
				this.f.i18n.loadBundle("viewer", _.bind(function () {
					this._replyText.attr("placeholder", this.f.i18n.t("AnnReplyTT"));
				}, this));
				// block click bubbling
				this._replyText.on("click", function (e) {
					e.preventDefault();
					e.stopPropagation();
				});
				this._replyText.on("focus", _.bind(function (e) {
					this._showRichTextEditor(e);
				}, this));
			}
			this._reply = $("<div/>").append(this._replyText).addClass("addReply");

			// Rich text
			this._richTextEditor = RichTextEditorFactory.createEditor(this.f);




			// Buttons in the editor
			// Save button
			this._saveReply = new ViewerToolbarButton(this.f, "saveReply", "icon-WCR_Save12 marginRight5", true, _.bind(function (e) {
				this._annotationModel.replyToAnnotation(this._annotation, this._richTextEditor.getContent(), true);
				this._closeRichText();
				e.stopPropagation();
			}, this));
			// Cancel button
			this._cancelReply = new ViewerToolbarButton(this.f, "cancelReply", "icon-WCR_Revert12 marginRight5", true, _.bind(function (e) {
				this._closeRichText();
				this._richTextEditor.setContent("");
				this._annotation.resetUnsavedReply();
				this._annotationModel.modelChanged();
				e.stopPropagation();
			}, this));
			// Save and cancel are a buttongroup
			this._replyActionsGroup = new ButtonGroup("replyActionsGroup", [this._cancelReply, this._saveReply]);
			this._replyActionsGroup.element.addClass("pull-right");

			// Set editor/button texts
			f.i18n.loadBundle("viewer", _.bind(function () {
				this._saveReply.element.addClass("btn-mini").addClass("btn-success").append(this.f.i18n.t("Save"));
				this._cancelReply.element.addClass("btn-mini").append(this.f.i18n.t("CancelButton"));
			}, this));

			// Character/line count in reply	
			this._charLineCount = new AnnotationCharLineCount(f);
			this.showHideCharLineCount(this.f.getOption("PREFERENCES_ANNOTATION_SHOWCOUNT"));
			// On text change update charline count
			this._richTextEditor.onTextChanged(_.bind(function () {
				this._charLineCount.update(this._richTextEditor.getContent());
				this._annotation.setUnsavedReply(this._richTextEditor.getContent());
				this._annotationModel.modelChanged();
			}, this));

			// Set the buttons	
			this._richTextEditor.setButtons([this._replyActionsGroup, this._charLineCount]);

			/**
			 * Order of gui elements
			 */
			// if popup, don't show header
			this._mainRow = new AnnotationSingleRow(this.f, this._annotation, true, this._annotationModel, popup, this._replyText, this._isForMobile);
			this.element.append(this._mainRow.element);
			this.annotationRows[this._annotation.name] = this._mainRow;

			this.element.append(this._repliesDiv);
			this._repliesDiv.prepend(this._moreReplies);

			if (!isForMobile) {
				this.element.append(this._reply);
			}

			this.annotationUpdated();
			this.updateReplyLimitLabel();
		};

		/**
		 * Close rich text editor and replace with reply
		 */
		AnnotationRow.prototype._closeRichText = function () {
			// on cancel, show reply
			this._replyText.show();
			this._richTextEditor.setContent("");
			this._richTextEditor.element.hide();
		};

		/**
		 * show the richt text editor
		 * @param {Object} e - event object
		 */
		AnnotationRow.prototype._showRichTextEditor = function (e) {
			this._replyText.hide();
			if (!this._richTextEditor.isInitialized()) {
				// only add first time
				this._replyText.after(this._richTextEditor.element);
			}
			this._richTextEditor.element.show();
			this._annotation.setUnsavedReply("");
			this._richTextEditor.setContent("<html><body></body></html>");
			this._richTextEditor.init(_.bind(function () {
				this._charLineCount.update(this._richTextEditor.getContent());
				// Scroll rich text editor in view
				this._richTextEditor.element.trigger("makeMeVisible");
				this._richTextEditor.element.trigger("contentResized");
			}, this));
			e.preventDefault();
			e.stopPropagation();
		};

		/**
		 * show/hide the character line count
		 * @param {Boolean} show - show or hide the character line count
		 * @param {Boolean} updateSingleRows - do all single rows need to be updated
		 */
		AnnotationRow.prototype.showHideCharLineCount = function (show, updateSingleRows) {
			if (show && this._annotationModel.isAnnotationOrReplySelected(this._annotation)) {
				this._charLineCount.show();
				this._charLineCount.update(this._richTextEditor.getContent());
			} else {
				this._charLineCount.hide();
			}
			if (updateSingleRows) {
				_.each(this.annotationRows, function (row) {
					row.showHideCharLineCount(show);
				});
			}
		};

		/**
		 * Redraw the annotation row after the annotation was updated
		 * @param {Annotation} refannotation
		 */
		AnnotationRow.prototype.annotationUpdated = function (refannotation) {
			this.element.css({
				"border-color": this._annotation.annotationcolor
			});

			if (this.title) {
				this.title.text(this._annotation.getMessageNumber());
				this.title.css({
					'color': this._annotation.getHeaderTextColor(),
					'background-color': this._annotation.annotationcolor
				});
			}

			this._mainRow.setAnnotation(this._annotation);

			var replyStack = this._annotationModel.getReplies(this._annotation);
			this._totalReplyCount = replyStack.length;

			//Remove first the replies that might have been deleted
			var currentReplies = [];
			_.each(this.annotationRows, function (row, annotName) {
				currentReplies.push(annotName);
			});
			this._annotation.runThroughAnnotation(true, true, false, function (annot) {
				var i = _.indexOf(currentReplies, annot.name);
				if (i > -1) {
					currentReplies.splice(i, 1);
				}
			});
			_.each(currentReplies, function (reply) {
				this.annotationRows[reply].element.remove();
				delete this.annotationRows[reply];
			}, this);

			/**
			 * Now redraw the replies
			 */
			var currentLimit = this._totalReplyCount;
			if (this._limitReplyCount) {
				currentLimit = this._replyLimit;
			}

			if (refannotation) {
				var indexOfRefAnnotation = replyStack.length - _.indexOf(replyStack, refannotation.parent); //minimum go to the annotation that is ref'd
				if (indexOfRefAnnotation <= replyStack.length) {
					currentLimit = Math.max(currentLimit, indexOfRefAnnotation);
				}
			}

			this._shownReplyCount = Math.min(currentLimit, this._totalReplyCount);


			for (var i = 0; i < this._totalReplyCount; i++) {
				var annot = replyStack[i];
				var row = this.annotationRows[annot.name];
				// show
				if (!row) {
					row = new AnnotationSingleRow(this.f, annot, false, this._annotationModel, undefined, undefined, this._isForMobile);
					this._repliesDiv.append(row.element);
					this.annotationRows[annot.name] = row;
					row.setSelected(this.selected);
					row.setVisible(false);
				}

				if ((this._totalReplyCount - 1 - i) < currentLimit) {
					row.setAnnotation(annot);
					row.setVisible(true);
				} else {
					// hide
					if (row) {
						row.setVisible(false);
					}

				}
			}
			this._repliesDiv.prepend(this._moreReplies);
			if (this.shouldShowMoreRepliesLink()) {
				// show morereplies link
				this._moreReplies.show();
			} else {
				this._moreReplies.hide();
			}
			this.updateReplyLimitLabel();

			this.showHideCharLineCount(this.f.getOption("PREFERENCES_ANNOTATION_SHOWCOUNT"));

			if (this._annotationModel.canChangeAnnotations(true) && !this._annotation.isNew) {
				this._reply.show();

				if (this._annotation.getUnsavedReply() === null) {
					this._replyText.show();
					this._closeRichText();
				} else {
					this._richTextEditor.setContent(this._annotation.getUnsavedReply());
				}
			} else {
				this._reply.hide();
				this._replyText.hide();
			}
		};

		/**
		 * Should the more replies link be shown?
		 * @returns {Boolean}
		 */
		AnnotationRow.prototype.shouldShowMoreRepliesLink = function () {
			return this._totalReplyCount > this._replyLimit;
		};


		/**
		 * Toggle the reply limit, sets the setting so all annotation replies are shown/hidden
		 * @param {Boolean} toggle - possible to set if the replies should be shown or hidden
		 */
		AnnotationRow.prototype.toggleReplyLimit = function (toggle) {
			if (toggle === undefined) {
				this._limitReplyCount = !this._limitReplyCount;
			} else {
				this._limitReplyCount = toggle;
			}

			this.annotationUpdated();
			this.updateReplyLimitLabel();
			this._moreReplies.trigger("contentResized");
		};

		/**
		 * Update the reply limit label, changes the link to show/hide annotation replies
		 */
		AnnotationRow.prototype.updateReplyLimitLabel = function () {
			if (!this._limitReplyCount) {
				// showing all replies, show less link
				this._moreRepliesLabel.text(this.f.i18n.t("AnnShowLessReplies"));
			} else {
				// show only limited replies, show more link
				var repliesHidden = this._totalReplyCount - this._shownReplyCount;
				if (repliesHidden <= 1) {
					this._moreRepliesLabel.text(this.f.i18n.t("AnnShowOneMoreReply"));
				} else {
					this._moreRepliesLabel.text(this.f.i18n.t("AnnShowMoreReplies", repliesHidden));
				}
			}
		};

		/**
		 * updates the timers of the annotation row
		 */
		AnnotationRow.prototype.updateTimers = function () {
			_.each(this.annotationRows, function (row) {
				row.updateTimer();
			});
		};

		/**
		 * Set this row to be the selected row, this highlights the row in the annotation widget
		 * @param {Boolean} selected - does the row needs to be hightlighted or not
		 */
		AnnotationRow.prototype.setSelected = function (selected) {
			this.selected = selected;
			_.each(this.annotationRows, function (row) {
				row.setSelected(selected);
			});
			this.element.toggleClass("selected", selected);
			this.showHideCharLineCount(this.f.getOption("PREFERENCES_ANNOTATION_SHOWCOUNT"));
		};

		/**
		 * Detach the annotation row from the DOM
		 */
		AnnotationRow.prototype.onDetachFromDOM = function () {
			if (!this.detachedFromDOM) {
				this.detachedFromDOM = true;
				this._richTextEditor.beforeDetach();
				_.each(this.annotationRows, function (row) {
					row.onDetachFromDOM();
				});
			}
		};

		/**
		 * Reinitialize TinyMCE, needed after reattaching the editor to the dom
		 */
		AnnotationRow.prototype.onReattachToDOM = function () {
			if (this.detachedFromDOM) {
				this.detachedFromDOM = false;
				this._richTextEditor.repair();
				_.each(this.annotationRows, function (row) {
					row.onReattachToDOM();
				});
			}
		};

		/**
		 * Set the available height for the row (e.g. while resizing the popup)
		 * @param {Number} height - Available height.
		 */
		AnnotationRow.prototype.setAvailableHeight = function (height) {
			// Adapt the reply editor size
			if (this._annotation.getUnsavedReply() !== null) {
				var leftOverHeight = height - 10; // minus 1 for border on .richText div, minus 5 for margin and minus 2x2 for padding on .addReply div

				if (this.shouldShowMoreRepliesLink()) {
					// the more replies link is shown
					leftOverHeight -= this._moreReplies.outerHeight(true);
				}

				_.each(this.annotationRows, function (row) {
					if (row.isVisible()) {
						// the row is visible
						leftOverHeight -= row.element.outerHeight(true);
					}
				});
				this._richTextEditor.setHeight(leftOverHeight);
			}
		};

		/**
		 * Set the annotation for the AnnotationRow. This will trigger a render of the HTML
		 * @param {Annotation} annotation - the annotation]
		 */
		AnnotationRow.prototype.setAnnotation = function (annotation) {
			this._annotation = annotation;
			this.annotationUpdated();
		};

		/**
		 * Returns the annotation object assigned with the annotation row
		 * @returns {Annotation}
		 */
		AnnotationRow.prototype.getAnnotation = function () {
			return this._annotation;
		};

		return AnnotationRow;
	});

define('web-viewer/annotation/AnnotationPopupNew',["web-viewer/annotation/AnnotationPopup",
		"web-viewer/layout/component/ColorPicker",
		"web-viewer/layout/component/ViewerToolbarButton",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/RichTextEditorFactory",
		"web-viewer/annotation/AnnotationCharLineCount",
		"web-viewer/annotation/AnnotationRow",
		"underscore"
	],
	function (AnnotationPopup,
		ColorPicker,
		ViewerToolbarButton,
		ButtonGroup,
		RichTextEditorFactory,
		AnnotationCharLineCount,
		AnnotationRow) {
		"use strict";

		/**
		 * Create a new New Annotation Popup.
		 * @exports AnnotationPopupNew
		 * @memberof AnnotationModule
		 * @class
		 * This annotation popup is responsible for a new annotation popup and its content (the rich text editor).
		 * @augments AnnotationModule.AnnotationPopup
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewerfacade
		 * @param {String} id - Id of the popup.
		 * @param {Annotation} annotation - Annotation this popup belongs to.
		 * @param {AnnotationModel} annotationModel - Model of the annotations.
		 * @param {JQueryObj} container - The container where the popup should stay in
		 */
		var AnnotationPopupNew = function (f, id, annotation, annotationModel, container) {
			AnnotationPopup.call(this, f, id, annotation, container);
			this.element.removeClass("annotationPopup").addClass("annotationPopupNew");
			// Default Size (in world size)
			if (!this.annotation.popuprect.size.width) {
				this.annotation.popuprect.size.width = 72; // min size for tinymce to show toolbar on one line
			}
			if (!this.annotation.popuprect.size.height) {
				this.annotation.popuprect.size.height = 45;
			}
			// Default Location (top right corner of annotation rect)
			if (!this.annotation.popuprect.origin.x) {
				this.annotation.popuprect.origin.x = this.annotation.rect.right();
			}
			if (!this.annotation.popuprect.origin.y) {
				this.annotation.popuprect.origin.y = this.annotation.rect.bottom() - this.annotation.popuprect.size.height;
			}

			this.annotationModel = annotationModel;
		};

		AnnotationPopupNew.prototype = Object.create(AnnotationPopup.prototype);
		AnnotationPopupNew.prototype.constructor = AnnotationPopupNew;

		/**
		 * @override
		 */
		AnnotationPopupNew.prototype._initialize = function () {
			// Super call
			AnnotationPopup.prototype._initialize.call(this);

			// Rich text editor
			this.richTextEditor = RichTextEditorFactory.createEditor(this.f);
			this.richTextEditor.setContent(this.annotation.content);

			// Show a color picker if no annotation color is set
			var annotationColorPicker;
			if (!this.f.getProperty("annotationcolor")) {
				annotationColorPicker = new ColorPicker(_.bind(function (newColor) {
					this.annotation.setColor("#" + newColor);
					this.annotationModel.modelChanged(this.annotation);
				}, this));
				annotationColorPicker.setColor(this.annotation.annotationcolor);
				annotationColorPicker.element.addClass("annotationColorPicker");
				annotationColorPicker.setTooltip(this.f, "general", "ChangeColor");
			}

			// Save button
			var saveAnnot = new ViewerToolbarButton(this.f, "saveAnnot", "icon-WCR_Save12 marginRight5", false, _.bind(function (e) {
				this.annotation.content = this.richTextEditor.getContent();
				this.annotationModel.save();
				e.stopPropagation();
			}, this));
			// Delete button
			var deleteAnnot = new ViewerToolbarButton(this.f, "deleteAnnot", "icon-WCR_Trash12 marginRight5", false, _.bind(function (e) {
				this.annotationModel.deleteAnnotation(this.annotation);
				e.stopPropagation();
			}, this));
			// Save and delete are a buttongroup
			var annotationActionsGroup = new ButtonGroup("AnnotationActionsGroup", [deleteAnnot, saveAnnot]);
			annotationActionsGroup.element.addClass("pull-right");

			// Set editor/button texts
			this.f.i18n.loadBundle("viewer", _.bind(function () {
				saveAnnot.element.addClass("btn-mini").addClass("btn-success").append(this.f.i18n.t("Save"));
				deleteAnnot.element.addClass("btn-mini").append(this.f.i18n.t("Delete"));
			}, this));

			// Character/line count
			this.charLineCount = new AnnotationCharLineCount(this.f);

			// Set the buttons and char/line count in the editor
			this.richTextEditor.setButtons([annotationColorPicker, annotationActionsGroup, this.charLineCount]);

			// Update count on text change
			this.richTextEditor.onTextChanged(_.bind(function () {
				this.charLineCount.update(this.richTextEditor.getContent());
				this.annotation.setBeingModified(this.richTextEditor.getContent());
			}, this));

			// Set the content to the rich text editor
			this.setContent(this.richTextEditor.element);

			// Init rich text
			this.richTextEditor.init(_.bind(function () {
				this._repaintCharLineCount();

				// Resize the editor along with the popup
				this.on("contentResized", _.bind(function (size) {
					this.richTextEditor.setHeight(size.height);
				}, this));
				this.richTextEditor.setHeight(this._outerContent.height());
			}, this));
		};

		/**
		 * @override
		 */
		AnnotationPopupNew.prototype.repaint = function () {
			if (this._isInitialized) {
				// Super call
				AnnotationPopup.prototype.repaint.call(this);
				this._repaintCharLineCount();
			}
		};

		/**
		 * Repaint the char/line count
		 */
		AnnotationPopupNew.prototype._repaintCharLineCount = function () {
			if (this.f.getOption("PREFERENCES_ANNOTATION_SHOWCOUNT")) {
				this.charLineCount.show();
				this.charLineCount.update(this.richTextEditor.getContent());
			} else {
				this.charLineCount.hide();
			}
		};

		/**
		 * @override
		 */
		AnnotationPopupNew.prototype.remove = function () {
			if (this.annotationColorPicker) {
				this.annotationColorPicker.destroy();
			}
			if (this.richTextEditor) {
				this.richTextEditor.destroy();
			}
			// Super call
			AnnotationPopup.prototype.remove.call(this);
		};

		return AnnotationPopupNew;
	});

define('web-viewer/annotation/AnnotationPopupExisting',["web-viewer/annotation/AnnotationPopup",
		"web-viewer/annotation/AnnotationRow",
		"underscore"
	],
	function (AnnotationPopup,
		AnnotationRow,
		_) {
		"use strict";

		/**
		 * Create a new Annotation Popup for existing annotations.
		 * @exports AnnotationPopupExisting
		 * @memberof AnnotationModule
		 * @class
		 * The annotation popup is responsible for an existing annotation popup and its content (the annotation row).
		 * @augments AnnotationModule.AnnotationPopup
		 * @constructor 
		 * @param {Core.ViewerFacade} f - Viewerfacade
		 * @param {String} id - Id of the popup.
		 * @param {Annotation} annotation - Annotation this popup belongs to.
		 * @param {AnnotationModel} annotationModel - Model of the annotations.
		 * @param {JQueryObj} container - The container where the popup should stay in.
		 */
		var AnnotationPopupExisting = function (f, id, annotation, annotationModel, container) {
			AnnotationPopup.call(this, f, id, annotation, container);

			this.annotationModel = annotationModel;
		};

		AnnotationPopupExisting.prototype = Object.create(AnnotationPopup.prototype);
		AnnotationPopupExisting.prototype.constructor = AnnotationPopupExisting;

		/**
		 * @override
		 */
		AnnotationPopupExisting.prototype._initialize = function () {
			// Super call
			AnnotationPopup.prototype._initialize.call(this);

			// Annotation row (content)
			this.row = new AnnotationRow(this.f, this.annotation, this.annotationModel, true);
			this.row.setSelected(true);
			this.annotationExistingContent = $("<div>").addClass("AnnotationTable enableTextSelection").append(this.row.element);

			this.setContent(this.annotationExistingContent);

			// Resize the row along with the popup
			this.on("contentResized", _.bind(function (size) {
				this.row.setAvailableHeight(size.height);
			}, this));
			this.row.setAvailableHeight(this.content.height());
		};

		/**
		 * @override
		 * @param {AnnotationModule.Annotation} refannotation - Referenced annotation used in the row update
		 */
		AnnotationPopupExisting.prototype.repaint = function (refannotation) {
			if (this._isInitialized) {
				// Super call
				AnnotationPopup.prototype.repaint.call(this);
				this.row.annotationUpdated(refannotation);
			}
		};

		return AnnotationPopupExisting;
	});

define('web-viewer/annotation/AnnotationPopupModel',["web-viewer/viewport/model/ViewportPopupModel",
		"web-viewer/annotation/AnnotationPopupNew",
		"web-viewer/annotation/AnnotationPopupExisting",
		"web-viewer/annotation/AnnotationConstants",
		"utils/ShapeUtils",
		"underscore"
	],
	function (ViewportPopupModel,
		AnnotationPopupNew,
		AnnotationPopupExisting,
		AnnotationConstants,
		ShapeUtils) {
		/**
		 * Create a new Annotation Popup Model.
		 * @exports AnnotationPopupModel
		 * @memberof AnnotationModule
		 * @class
		 * The annotation popup model is responsible for an annotation popup and its content.
		 * It binds the events of the popup and reacts on them.
		 * This popup model is the model for both a new annotation and an existing one.
		 * It will create the appropriate popup: AnnotationPopupNew if it's a new or AnnotationPopupExisting if it's an existing annotation.
		 * @augments Viewport.ViewportPopupModel
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewerfacade
		 * @param {String} id - Id of the popup model.
		 * @param {Annotation} annotation - Annotation this model belongs to.
		 * @param {AnnotationModel} annotationModel - Model of the annotations.
		 */
		var AnnotationPopupModel = function (f, id, annotation, annotationModel) {
			ViewportPopupModel.call(this, f, id);

			this.annotation = annotation;
			this.annotationModel = annotationModel;

			this.popup = null;
		};

		AnnotationPopupModel.prototype = Object.create(ViewportPopupModel.prototype);
		AnnotationPopupModel.prototype.constructor = AnnotationPopupModel;

		/**
		 * Repaint the popup
		 */
		AnnotationPopupModel.prototype.repaint = function (refannotation) {
			if (this.popup != null) {
				this.popup.repaint(refannotation);
			}
		};

		/**
		 * Get the Annotation popup
		 * @param {ViewportLayer} viewportLayer - The layer where the popups should be shown.
		 * @returns {AnnotationPopupNew | AnnotationPopupExisting} The annotation popup of this model.
		 */
		AnnotationPopupModel.prototype.getPopup = function (viewportLayer) {
			var container = viewportLayer.getCanvas();
			if (this.popup === null) {
				this._createPopup(container);
			} else {
				this.popup.setContainment(container);
			}
			return this.popup;
		};

		/**
		 * Create an annotation popup {AnnotationPopupNew | AnnotationPopupExisting} for the given layer
		 * and bind all events.
		 * @param {JQueryObj} container - Container for the popup
		 */
		AnnotationPopupModel.prototype._createPopup = function (container) {
			if (this.annotation.isNew) {
				this.popup = new AnnotationPopupNew(this.f, this.id, this.annotation, this.annotationModel, container);
			} else {
				this.popup = new AnnotationPopupExisting(this.f, this.id, this.annotation, this.annotationModel, container);
			}
			// we want the resized in world rect instead of scene rect, so don't bind them here, but in the layer, and let the layer call the model functions
			this.popup.on("closed", _.bind(this.onClosed, this));
			this.popup.on("pinned", _.bind(this.onPinned, this));
			this.popup.on("clicked", _.bind(this.onClicked, this));
		};

		/**
		 * Method called when the model is destroyed.
		 * Useful for some cleanup in the popup/content
		 */
		AnnotationPopupModel.prototype.onDestroy = function () {
			if (this.popup != null) {
				this.popup.remove();
			}
		};

		/**
		 * @override
		 * This method saves the rectangle in the annotation.
		 * @param {ShapeUtils.Rect} rect - Rectangle (origin/size) of the popup in world coordinates
		 * @param {Boolean} autoAdapted - True if the event is fired because the content was auto adapted (e.g. from a makeMeVisible trigger from an element in the content), false otherwise
		 */
		AnnotationPopupModel.prototype.onResized = function (worldRect, autoAdapted) {
			var copy = worldRect.copy();
			// compensate for origin in left bottom corner
			copy.origin.y -= copy.size.height;

			if (!this.annotation.popuprect.rectEquals(copy)) {
				this.annotation.popuprect = copy;

				if (!autoAdapted) {
					this._fireAnnotationChanged("POPUP_SIZE");
				}
			}
		};

		/**
		 * @override
		 * This method saves the origin in the annotation.
		 * @param {ShapeUtils.Point} point - Point (origin) of the popup in world coordinates
		 */
		AnnotationPopupModel.prototype.onDragged = function (worldPoint) {
			var point = worldPoint.copy();
			point.y -= this.annotation.popuprect.size.height;

			if (!this.annotation.popuprect.origin.equals(point)) {
				this.annotation.popuprect.origin = point;
				this._fireAnnotationChanged("POPUP_POSITION");
			}
		};

		/**
		 * This function is bind to the "closed" popup event and called when the popup is closed.
		 */
		AnnotationPopupModel.prototype.onClosed = function () {
			this.annotationModel.unselectAllAnnotations(true, true);

			if (this.annotation.popupIsOpen) {
				this.annotation.popupIsOpen = false;
				this.annotation.setModified("POPUP_PINNED");
			}
			this.annotationModel.modelChanged(this.annotation);
		};


		/**
		 * This function is bind to the "pinned" annotation popup event and called when the popup is pinned/unpinned.
		 */
		AnnotationPopupModel.prototype.onPinned = function (pinned) {
			if (this.annotation.popupIsOpen !== pinned) {
				this.annotation.popupIsOpen = pinned;
				if (!this._fireAnnotationChanged("POPUP_PINNED")) {
					// trigger repaint for pinned icon if we cannot change annotations
					this.popup.repaint();
				}
			}
		};

		/**
		 * This function is bind to the "clicked" annotation popup event and called when the popup is clicked.
		 */
		AnnotationPopupModel.prototype.onClicked = function () {
			this.annotationModel.selectAnnotation(this.annotation);
		};

		/**
		 * Fire that an annotation has changed
		 * @param {String} what - indicate what was changed
		 * @returns {Boolean} True when fired, false otherwise
		 */
		AnnotationPopupModel.prototype._fireAnnotationChanged = function (what) {
			if (this.annotationModel.canChangeAnnotation(this.annotation)) {
				this.annotation.setModified(what);
				this.annotationModel.modelChanged(this.annotation);
				return true;
			} else {
				return false;
			}
		};

		/**
		 * Get the origin of the popup in world coordinates
		 * @returns {ShapeUtils.Point} - The origin of the popup in world coordinates
		 */
		AnnotationPopupModel.prototype.getWorldOrigin = function () {
			// origin in left bottom corner
			return new ShapeUtils.Point(this.annotation.popuprect.left(), this.annotation.popuprect.bottom());
		};

		/**
		 * Get the size of the popup in world coordinates
		 * @returns {ShapeUtils.Size} - The size of the popup in world coordinates
		 */
		AnnotationPopupModel.prototype.getWorldSize = function () {
			return this.annotation.popuprect.size;
		};

		/**
		 * When the popup is for a pinned annotation, add an offset so the popup doesn't show above the icon
		 * @returns {ShapeUtils.Size}
		 */
		AnnotationPopupModel.prototype.getSceneOffset = function () {
			if (this.annotation.getType() == AnnotationConstants.AnnotationType.ANNOT_PIN) {
				return new ShapeUtils.Size(AnnotationConstants.AnnotationPin.imagewidth, 0);
			}
		};
		return AnnotationPopupModel;
	});

define('web-viewer/annotation/AnnotationPopupsLayerModel',["web-viewer/viewport/model/ViewportLayerModel",
		"web-viewer/annotation/AnnotationPopupModel",
		"underscore"
	],
	function (ViewportLayerModel,
		AnnotationPopupModel,
		_) {
		"use strict";

		/**
		 * @exports AnnotationPopupsLayerModel
		 * @memberof AnnotationModule
		 * @class
		 * Create a new annotation layer model for annotation popups
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewerfacade
		 * @param {String} ID - Id of the layer
		 * @param {AnnotationModel} annotationModel - Model of the annotations.
		 */
		var AnnotationPopupsLayerModel = function (f, ID, annotationModel) {
			ViewportLayerModel.call(this, f, ID);

			this.f = f;
			this.annotationModel = annotationModel;

			// New annotations received
			this._createModels();
			this.annotationModel.on("modelContentChanged", _.bind(function () {
				this.removeAllPopupModels();
				this._createModels();
			}, this));

			// Annotation model changed events
			this.annotationModel.on("modelChanged", _.bind(function () {
				var page = this.annotationModel.getAnnotationsPage();
				if (page !== null && page !== undefined) {
					this.resetTransformation();
					this.getTransform().scale(1, -1);

					var rotation = page.getRotation();

					var worldBounds = this.f.getModule("viewport").getWorldBounds();

					this.calculateTransformation(worldBounds, page.getPageSize(), this.annotationModel.getAnnotationsOffset(), rotation, page.isFlipped());
				}
				this.modelChanged();
			}, this));

			// Annotation selected
			this.annotationModel.on("selectedAnnotation", _.bind(function (annotation) {
				// if show count, repaint the annotation popup when selected to draw the char/line count
				if (this.f.getOption("PREFERENCES_ANNOTATION_SHOWCOUNT")) {
					var mainAnnotation = annotation.getGodfather();
					var popupModel = this.getPopupModel(mainAnnotation.name);
					if (popupModel !== undefined) {
						if (mainAnnotation == annotation) {
							// the main annotation is selected
							popupModel.repaint();
						} else {
							// a reply is selected
							popupModel.repaint(annotation);
						}
					}
				}
				this._updateModels();
				this.modelChanged();
			}, this));

			// Annotation unselected		
			this.annotationModel.on("unselectedAnnotation", _.bind(function (annotation) {
				// if show count, repaint the annotation popup when selected to hide the char/line count
				if (this.f.getOption("PREFERENCES_ANNOTATION_SHOWCOUNT")) {
					var popupModel = this.getPopupModel(annotation.getGodfather().name);
					if (popupModel !== undefined) {
						popupModel.repaint();
					}
				}
				this._updateModels();
			}, this));

			// Annotation deleted
			this.annotationModel.on("annotationDeleted", _.bind(function (annotation) {
				var mainAnnotation = annotation.getGodfather();
				if (annotation == mainAnnotation) {
					// deleted main annotation
					this.removePopupModel(annotation.name);
				} else {
					// deleted a sub annotation, so just repaint
					var popupModel = this.getPopupModel(mainAnnotation.name);
					if (popupModel !== undefined) {
						popupModel.repaint();
					}
				}
			}, this));

			// Annotation added
			this.annotationModel.on("annotationAdded", _.bind(function (annotation) {
				this._createModel(annotation);
			}, this));

			// Annotation updated
			this.annotationModel.on("annotationUpdated", _.bind(function (annotation, refannotation) {
				var popupModel = this.getPopupModel(annotation.getGodfather().name);
				if (popupModel !== undefined) {
					popupModel.repaint(refannotation);
				}
				this._updateModels();
			}, this));

			// Annotation list visible
			this.annotationModel.on("annotationListVisible", _.bind(function (annotationListVisible) {
				this._updateModels();
				this.modelChanged();
			}, this));

			// Read/write changed
			this.annotationModel.on("readWriteChanged", _.bind(function () {
				_.each(this.getPopupModels(), function (popupModel) {
					popupModel.repaint();
				}, this);
				this.modelChanged();
			}, this));

			// Annotations disabled
			this.annotationModel.on("temporarilyDisableChanged", _.bind(function () {
				this._updateModels();
				//this.modelChanged(); already fired by the annotation model (avoid flickering)
			}, this));

			// Show selected annotation only
			this.f.getCurrentUser().on("options.changed.PREFERENCES_ANNOTATION_SHOWSELECTEDONLY", _.bind(function (oldValue, newValue) {
				this._updateModels();
				this.modelChanged();
			}, this));

			// Show selected annotation popup only
			this.f.getCurrentUser().on("options.changed.PREFERENCES_ANNOTATION_SHOWSELECTEDPOPUPONLY", _.bind(function (oldValue, newValue) {
				this._updateModels();
				this.modelChanged();
			}, this));

			// Show annotation char/line count
			this.f.getCurrentUser().on("options.changed.PREFERENCES_ANNOTATION_SHOWCOUNT", _.bind(function (oldValue, newValue) {
				_.each(this.getPopupModels(), function (popupModel) {
					var annotation = this.annotationModel.getAnnotation(popupModel.getId());
					if (this.annotationModel.isAnnotationOrReplySelected(annotation)) {
						popupModel.repaint();
					}
				}, this);
				this.modelChanged();
			}, this));

			// Annotations visible
			this.f.getCurrentUser().on("options.changed.PREFERENCES_ANNOTATIONTOOLS_VISIBLE", _.bind(function (oldValue, newValue) {
				this._updateModels();
				//this.modelChanged(); already fired by the annotation model (avoid flickering)
			}, this));
		};

		AnnotationPopupsLayerModel.prototype = Object.create(ViewportLayerModel.prototype);
		AnnotationPopupsLayerModel.prototype.constructor = AnnotationPopupsLayerModel;

		/**
		 * Update the state of the models based on the options.
		 * Does the show/hide logic for annotation popups
		 */
		AnnotationPopupsLayerModel.prototype._updateModels = function () {
			if (this.f.getOption("PREFERENCES_ANNOTATIONTOOLS_VISIBLE")) {
				// Annotations visible
				if (this.annotationModel.isAnnotationListVisible()) {
					// options apply
					if (this.f.getOption("PREFERENCES_ANNOTATION_SHOWSELECTEDONLY")) {
						if (this.annotationModel.getSelectedAnnotations().length > 0) {
							// there are selected annotations, so show only the selected ones
							_.each(this.getPopupModels(), function (popupModel) {
								this._showSelected(popupModel);
							}, this);
						} else {
							// there are no selected annotations, show the pinned ones
							_.each(this.getPopupModels(), function (popupModel) {
								this._showPinnedOrSelected(popupModel);
							}, this);
						}
					} else {
						// else because it can only apply if PREFERENCES_ANNOTATION_SHOWSELECTEDONLY is turned off
						if (this.f.getOption("PREFERENCES_ANNOTATION_SHOWSELECTEDPOPUPONLY")) {
							_.each(this.getPopupModels(), function (popupModel) {
								this._showSelected(popupModel);
							}, this);
						} else {
							// show all popups that are pinned/selected if no (preference) options selected
							_.each(this.getPopupModels(), function (popupModel) {
								this._showPinnedOrSelected(popupModel);
							}, this);
						}
					}
				} else {
					// options not valid if list not visible, just check pinned state (or selected)
					_.each(this.getPopupModels(), function (popupModel) {
						this._showPinnedOrSelected(popupModel);
					}, this);
				}
			} else {
				// Annotations not visible, hide all popups
				_.each(this.getPopupModels(), function (popupModel) {
					popupModel.hide();
				}, this);
			}
		};

		/**
		 * Show the popup if it is selected
		 * This method relies on the fact that there are no annotations selected if the annotations are disabled
		 * @private
		 * @param {AnnotationPopupModel} popupModel - The model of the popup to show if pinned or selected
		 */
		AnnotationPopupsLayerModel.prototype._showSelected = function (popupModel) {
			var annotation = this.annotationModel.getAnnotation(popupModel.getId());
			if (this.annotationModel.isAnnotationOrReplySelected(annotation)) {
				popupModel.show();
			} else {
				popupModel.hide();
			}
		};

		/**
		 * Show the popup if it is pinned/selected and if the annotations are not disabled
		 * @private
		 * @param {AnnotationPopupModel} popupModel - The model of the popup to show if pinned or selected
		 */
		AnnotationPopupsLayerModel.prototype._showPinnedOrSelected = function (popupModel) {
			var annotation = this.annotationModel.getAnnotation(popupModel.getId());
			// if the annotations are not disabled and the annotation popup is open or selected, show it, else hide it
			if (!this.annotationModel.isTemporarilyDisabledAndNoAnnotationsVisible() && annotation && (annotation.popupIsOpen || this.annotationModel.isAnnotationOrReplySelected(annotation))) {
				popupModel.show();
			} else {
				popupModel.hide();
			}
		};

		/**
		 * Create the annotation popup models
		 */
		AnnotationPopupsLayerModel.prototype._createModels = function () {
			// Create all models
			_.each(this.annotationModel.getAnnotations(), function (annotation) {
				this._createModel(annotation);
			}, this);
			this._updateModels();
			this.modelChanged();
		};

		/**
		 * Create the annotation popup model for the given annotation
		 * @param {Annotation} annotation - Annotation to create the model of
		 */
		AnnotationPopupsLayerModel.prototype._createModel = function (annotation) {
			// If annotation is linked, the other one already exists, 
			// so don't create a new model
			if (annotation.linkedto) {
				return;
			} else {
				// This will create the appropriate popup (AnnotationPopupNew vs AnnotationPopupExisting)
				var popupModel = new AnnotationPopupModel(this.f, annotation.name, annotation, this.annotationModel);
				this.addPopupModel(popupModel);
			}
		};

		return AnnotationPopupsLayerModel;
	});

define('web-viewer/annotation/AnnotationTool',["underscore",
		"web-viewer/viewport/tools/AbstractTool",
		"utils/ShapeUtils",
		"web-viewer/utils/DrawUtils",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/annotation/AnnotationConstants",
		"web-viewer/annotation/Annotation",
		"web-viewer/core/CoreConstants"
	],
	function (_,
		AbstractTool,
		ShapeUtils,
		DrawUtils,
		ViewerUtils,
		AnnotationConstants,
		Annotation,
		CoreConstants) {
		"use strict";
		/**
		 * Construct a new Annotation Tool
		 * @exports AnnotationTool
		 * @memberof AnnotationModule
		 *
		 * @class Tool to select and edit annotations
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {AnnotationModel} annotationModel - Annotation model
		 */
		var AnnotationTool = function (f, annotationModel) {
			AbstractTool.call(this, f, CoreConstants.TOOLS.ANNOTATION);

			this.f = f;

			this._annotationModel = annotationModel;

			/**
			 * Different states constants
			 */
			this._defaultState = "defaultState";
			this._hoverState = "hoverState";
			this._draggingState = "draggingState";
			this._anchorState = "anchorState";

			/**
			 * Variable that contain the current situation
			 */
			this._anchor = undefined;
			this._targetAnnotation = null;

			/**
			 * Variable to keep track of where interaction started
			 */
			this._touchDown = null;

			/**
			 * The default state
			 */
			this.registerState(this._defaultState, (function (that) {
				return {
					"mousemove": function (e) {
						var l = that._getAnnotationLayer(e);
						if (!l || !l.isVisible()) {
							return;
						}

						that._verifyObjects(e, l);
						if (that._targetAnnotation || that._anchor) {
							that.requestActivation();
						}
					},
					"mouseup": function (e) {

					},
					"mousedown": function (e) {
						var l = that._getAnnotationLayer(e);
						if (!l) {
							return;
						}

						annotationModel.unselectAllAnnotations();
						e.originalEvent.preventDefault();
					},
					"mouseout": function (e) {
						if (that.isActive()) {
							f.emit("toolManager.request.deactivation", that);
						}
					},
					"mouseleave": function () {
						if (that.isActive()) {
							f.emit("toolManager.request.deactivation", that);
						}
					},
					"contextmenu": function (e) {
						e.preventDefault();
						e.stopPropagation();
						return false;
					}
				};
			}(this)));

			this.registerState(this._hoverState, _.defaults(function (that) {
				return {
					"mousedown": function (e) {
						if (e.which === 1) {
							that._verifyObjects(e);
							that._touchDown = that._getAnnotationLayer(e).getWorldPointFromScene(that.getScenePoint(e));
							if (that._anchor && that._targetAnnotation.annottype != AnnotationConstants.AnnotationType.ANNOT_FREEFORM) {
								f.emit("viewport.startInteractive");
								that.changeState(that._anchorState);
							} else if (that._targetAnnotation) {

								f.emit("viewport.startInteractive");
								annotationModel.selectAnnotation(that._targetAnnotation);
								that.changeState(that._draggingState);
							}
						}
						e.originalEvent.preventDefault();
					},
					"mousemove": function (e) {
						that._verifyObjects(e);
						that.cursorShouldChange();
					}
				};
			}(this), this.getState(this._defaultState)));

			/**
			 * Dragging state is when an annotation is selected and every mousemove the annotation is moved
			 */
			this.registerState(this._draggingState, _.defaults(function (that) {
				return {
					"mousemove": function (e) {
						that.cursorShouldChange();
						var worldPoint = that._getAnnotationLayer(e).getWorldPointFromScene(that.getScenePoint(e));
						var worldBounds = that._widget.getWorld().getWorldBounds(); //get the world bounds
						//get the converted worldbounds (with origin (0,0)), the origin gets added later on to not intervere with transforming the worldbounds to annotation coordinate system
						var worldRect = that._getAnnotationLayer(e).convertWorldRectToLayer(new ShapeUtils.Rect(new ShapeUtils.Point(), worldBounds.size));
						worldRect.origin.y = -(worldRect.size.height + worldRect.origin.y) - worldBounds.origin.y; //calculate the origin in annotation coordinate system
						worldRect.origin.x += worldBounds.origin.x; //add the origin to the worldRect (this is not converted as it stays a constant)
						var moveSize = that.keepMoveSizeWithinRect(worldRect, that._targetAnnotation.rect, that._touchDown.sizeTo(worldPoint)); //keep move size within world bounds
						annotationModel.moveAnnotation(that._targetAnnotation, moveSize);
						that._touchDown = worldPoint;
					},
					"mouseup": function (e) {
						f.emit("viewport.stopInteractive");
						that.changeState(that._hoverState);
					},
					"mouseout": function (e) {
						f.emit("viewport.stopInteractive");
						that.changeState(that._hoverState);
					},
					"enterState": function (e) {
						f.emit("viewport.startInteractive");
					},
					"leaveState": function (e) {
						f.emit("viewport.stopInteractive");
					}
				};
			}(this), this.getState(this._defaultState)));


			/**
			 * Anchor state is when clicked and an anchor is selected
			 */
			this.registerState(this._anchorState, _.defaults(function (that) {
				/**
				 * @returns a suffix for the anchor point, so the right action can be calculated
				 */
				function suffixForAnchor() {
					var suffix = null;
					switch (that._anchor.anchor) {
					case DrawUtils.LWANCHOR_TOPLEFT:
						suffix = "NW";
						break;
					case DrawUtils.LWANCHOR_TOPCENTER:
						suffix = "N";
						break;
					case DrawUtils.LWANCHOR_TOPRIGHT:
						suffix = "NE";
						break;
					case DrawUtils.LWANCHOR_BOTTOMLEFT:
						suffix = "SW";
						break;
					case DrawUtils.LWANCHOR_BOTTOMCENTER:
						suffix = "S";
						break;
					case DrawUtils.LWANCHOR_BOTTOMRIGHT:
						suffix = "SE";
						break;
					case DrawUtils.LWANCHOR_CENTERLEFT:
						suffix = "W";
						break;
					case DrawUtils.LWANCHOR_CENTERRIGHT:
						suffix = "E";
						break;
					default:
						break;
					}
					return suffix;
				}

				return {
					"mousemove": function (e) {
						var worldPoint = that._getWorldPoint(e);
						var suffix = suffixForAnchor();
						if (suffix != null) {
							var newrect = that._anchor.linework.rect.copy();

							//Figure out x-axis
							if (suffix.indexOf("E") > -1 || suffix.indexOf("W") > -1) {
								if (suffix.indexOf("W") > -1) {
									var sizeToX = that._anchor.linework.rect.left() - worldPoint.x;
									newrect.origin.x -= sizeToX;
									newrect.size.width += sizeToX;
								} else {
									var sizeToX = that._anchor.linework.rect.right() - worldPoint.x;
									newrect.size.width -= sizeToX;
								}
							}

							//Figure out y-axis
							if (suffix.indexOf("N") > -1 || suffix.indexOf("S") > -1) {
								if (suffix.indexOf("N") > -1) {
									var sizeToY = that._anchor.linework.rect.top() - worldPoint.y;
									newrect.origin.y -= sizeToY;
									newrect.size.height += sizeToY;
								} else {
									var sizeToY = that._anchor.linework.rect.bottom() - worldPoint.y;
									newrect.size.height -= sizeToY;
								}
							}

							annotationModel.transformAnnotation(that._anchor.linework, newrect);
						} else {
							if (that._anchor.anchor == DrawUtils.LWANCHOR_START) {
								annotationModel.updateStartEndOfAnnotation(that._anchor.linework, worldPoint, null);
							} else if (that._anchor.anchor == DrawUtils.LWANCHOR_END) {
								annotationModel.updateStartEndOfAnnotation(that._anchor.linework, null, worldPoint);
							}
						}
						that._touchDown = worldPoint;
					},
					"mouseup": function (e) {
						f.emit("viewport.stopInteractive");
						that.cursorShouldChange();
						that.changeState(that._hoverState);
					},
					"mouseleave": function (e) {
						f.emit("viewport.stopInteractive");
						that.cursorShouldChange();
						that.changeState(that._hoverState);
					},
					"mouseout": function (e) {

					}
				};
			}(this), this.getState(this._defaultState)));


			this._override("initialize", _.bind(function (widget, toolManager) {
				this._super(widget, toolManager);

				// on pressing delete key, request to delete the currently selected annotation
				this.persistentKeyEvents.push(["del", _.bind(function (e) {
					if (this._annotationModel.getSelectedAnnotations().length > 0) {
						_.each(this._annotationModel.getSelectedAnnotations(), _.bind(function (annot) {
							this._annotationModel.deleteAnnotation(annot);
						}, this));
					}

				}, this), "keyup"]);
				this.changeState(this._defaultState);
				this.registerPassiveEvents(["mousemove", "mousedown"]);
				this.registerActiveEvents(["mousemove", "mousedown", "mouseup", "mouseout", "mouseleave", "contextmenu"]);
			}, this));

			this._override("activate", _.bind(function (interruptRequest) {
				this._super(interruptRequest);
				this.changeState(this._hoverState);
			}, this));

			this._override("deactivate", _.bind(function (interruptRequest) {
				this._super(interruptRequest);
				this.changeState(this._defaultState);
			}, this));

		};

		// inherit
		AnnotationTool.prototype = Object.create(AbstractTool.prototype);
		// correct the constructor pointer 
		AnnotationTool.prototype.constructor = AnnotationTool;

		/**
		 * Calculate the default cursor
		 * 	- when an anchor is selected:
		 * 		- line or arrow annotation: move-cursor
		 * 		- show correct resize-cursor
		 *  - when an annotation is selected
		 *  	- show move-cursor unless the annotation can't be moved -> then show a not-allowed cursor
		 *  - when nothing is selected: show select arrow
		 */
		AnnotationTool.prototype.getDefaultCursor = function () {
			if (this._anchor) {
				return this._cursorForAnchor();
			} else if (this.getCurrentStateName() == this._draggingState) {
				if (this._annotationModel.canChangeAnnotation(this._targetAnnotation)) {
					return "cursor-moveObject";
				} else {
					return "cursor-notAllowed";
				}
			} else {
				return "cursor-select";
			}
		};

		/**
		 * Deactivate the current Annotation tool
		 * @param that
		 * @param e
		 */
		AnnotationTool.prototype._deActivate = function (e) {
			if (this.isActive()) {
				this.changeState(this._defaultState);
				this.f.emit("toolManager.request.deactivation", this);
			}
		};

		/**
		 * @returns {String} the cursor for the anchor state
		 */
		AnnotationTool.prototype._cursorForAnchor = function () {
			var suffix = "cursor-moveObject";

			//Show anchor cursor, unless it's an arrow or line, then just show the move cursor
			if (this._anchor.linework.annottype == AnnotationConstants.AnnotationType.ANNOT_ARROW || this._anchor.linework.annottype == AnnotationConstants.AnnotationType.ANNOT_LINE) {
				return suffix;
			}

			var l = this._getAnnotationLayer('Main');
			if (!l) {
				return suffix;
			}


			//because it's a static size, use the actual size of the rectangle
			var center = l.getScenePointFromWorld(this._anchor.rect.center());
			var size = new ShapeUtils.Size(this._anchor.rect.size.width, this._anchor.rect.size.height);
			var anchorrect = new ShapeUtils.Rect(new ShapeUtils.Point(center.x - size.width / 2, center.y - size.height / 2), size);

			var lwrect = l.getSceneRectFromWorld(this._anchor.linework.rect);

			if (anchorrect.containsPoint(lwrect.topleft())) {
				suffix = "NW";
			} else if (anchorrect.containsPoint(lwrect.topcenter())) {
				suffix = "N";
			} else if (anchorrect.containsPoint(lwrect.topright())) {
				suffix = "NE";
			} else if (anchorrect.containsPoint(lwrect.centerleft())) {
				suffix = "W";
			} else if (anchorrect.containsPoint(lwrect.centerright())) {
				suffix = "E";
			} else if (anchorrect.containsPoint(lwrect.bottomleft())) {
				suffix = "SW";
			} else if (anchorrect.containsPoint(lwrect.bottomcenter())) {
				suffix = "S";
			} else if (anchorrect.containsPoint(lwrect.bottomright())) {
				suffix = "SE";
			}

			return "cursor-resize" + suffix;
		};

		/**
		 * Get the annotation layer based on a point
		 * @param {String|Event} either the name of the view or an event that will calculate the view of the annotation layer
		 * @returns {ViewportLayer} Annotations layer of null if not found
		 */
		AnnotationTool.prototype._getAnnotationLayer = function (e) {
			var v = this._widget.requestView(e);
			if (v) {
				var l = v.getLayer("Annotations");
				if (l) {
					return l;
				}
			}
			return null;
		};

		/**
		 * Get the closest linework from a given point
		 * @param {Event} e - mouse event to calcuate the point to
		 * @param {ViewportLayer} [layer] - if the layer was already fetched
		 * @returns {LineworkWrapper} the closest lineworkwrapper to a certain point
		 */
		AnnotationTool.prototype._getClosestLineworkFromPoint = function (e, layer) {
			var l = layer;
			if (!l) {
				l = this._getAnnotationLayer(e);
			}

			if (l) {
				return l.getClosestLineworkFromPoint(this.getScenePoint(e));
			} else {
				return null;
			}
		};

		/**
		 * Converts a point from an event to the world, makes sure the point is containt in the document page
		 * @param {Event} e - mouse event
		 * @returns {ShapeUtils.Point} point in world coordinates
		 */
		AnnotationTool.prototype._getWorldPoint = function (e) {
			var activePage = this._annotationModel.getAnnotationsPage();
			var worldPoint = this.keepPointWithinWorldBounds(this.getWorldPoint(e), false);

			//convert the coordinate system from world coordinates to annotation coordinates
			worldPoint.y = activePage.getPageSize().height - worldPoint.y;

			var origin = activePage.getDocumentPageOrigin();

			worldPoint.offset(new ShapeUtils.Size(-origin.x, origin.y));

			var trimboxSize = activePage.getTrimboxRect().size;
			var missingHeight = 0; //pages with wrong XMP-data (page size) can have a missing part in the height, annotationlocation needs to be compensated for this
			if (trimboxSize.width !== 0 && trimboxSize.height !== 0) {
				missingHeight = activePage.getPageSize().height - (trimboxSize.height + activePage.getMargins().top + activePage.getMargins().bottom);
			}

			var margin = activePage.getMargins();
			if (activePage.isReaderSpread() && activePage.get("isRightPage")) { //check if the active page is on the right side of reader spread
				margin.left = -margin.left;
			}
			worldPoint.offset(new ShapeUtils.Size(-margin.left, -margin.bottom - missingHeight));

			return worldPoint;
		};

		/**
		 * Verify the current situation, so calcutate where the mouse is for the moment on the canvas
		 * @param {Event} e - mouse event to calcuate the point to
		 * @param {ViewportLayer} [layer] - if the layer was already fetched
		 */
		AnnotationTool.prototype._verifyObjects = function (e, layer) {
			if (!layer) {
				layer = this._getAnnotationLayer(e);
				if (!layer) {
					return;
				}
			}
			var object = this._getClosestLineworkFromPoint(e, layer);

			this._targetAnnotation = null;
			this._anchor = null;
			if (object) {
				if (object instanceof Annotation) {
					this._targetAnnotation = object;
				} else if (object.anchor) {
					this._anchor = object;
					this._targetAnnotation = object.linework;
				}
			}
			if (!object && !this._targetAnnotation) {
				this._deActivate(e);
			}
		};

		return AnnotationTool;
	});

define('web-viewer/annotation/AnnotationDrawTool',["underscore",
		"web-viewer/viewport/tools/AbstractTool",
		"utils/ShapeUtils",
		"web-viewer/utils/GEOUtils",
		"web-viewer/utils/DrawUtils",
		"web-viewer/annotation/Annotation",
		"web-viewer/utils/LineworkPoint",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/annotation/AnnotationConstants",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/core/CoreConstants"
	],
	function (_, AbstractTool,
		ShapeUtils,
		GEOUtils,
		DrawUtils,
		Annotation,
		LineworkPoint,
		ListenerUtil,
		AnnotationConstants,
		ViewerUtils,
		CoreConstants) {
		"use strict";

		/**
		 * This is the tool for drawing annotations.
		 * Also see it's related tool AnnotationNoteDrawTool.
		 * @exports AnnotationDrawTool
		 * @memberof AnnotationModule
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f
		 * @param {AnnotationModel} annotationModel - the model used by the module for annotations
		 * @param {AnnotationModule} annotationModule - the annotation module reference
		 */
		var AnnotationDrawTool = function (f, annotationModel, annotationModule) {
			AbstractTool.call(this, f, CoreConstants.TOOLS.ANNOTATION_DRAW);
			ListenerUtil.mixin(this, f);

			var DEFAULTSTATE = "default";
			var DRAWSTATE = "draw";
			var DRAWINGSTATE = "drawing";

			var threshold = 0; //use this threshold to debug the freeform tool. This allows to only draw a couple of segments. When set to 0, it's harmless

			var newAnnotation;

			this.drawLinkedAnnotation = false;

			this._annotationModule = annotationModule;

			this._annotationModel = annotationModel;

			/**
			 * Set the type of annotation to draw
			 */
			this.setAnnotationType = function (type) {
				f.setOption("PREFERENCES_ANNOTATION_TOOLMODE", type);
			};

			/**
			 * Get the type of annotation to draw
			 */
			this.getAnnotationType = function () {
				return f.getOption("PREFERENCES_ANNOTATION_TOOLMODE");
			};

			/**
			 * Set the annotation color to draw with
			 */
			this.setAnnotationColor = function (color) {
				f.getCurrentUser().setColor("PREFERENCES_ANNOTATION_RECTANGLE_COLOR", color);
			};

			/**
			 * Get the annotation color to draw with
			 */
			this.getAnnotationColor = function () {
				if (f.getProperty("annotationcolor")) {
					return f.getProperty("annotationcolor");
				} else {
					return f.getCurrentUser().getColor("PREFERENCES_ANNOTATION_RECTANGLE_COLOR");
				}
			};

			/**
			 * @override
			 */
			this.getDefaultCursor = function () {
				return "cursor-drawAnnotation";
			};

			/**
			 * Adds a global annotation to the opened document(s)
			 */
			this.addGlobalAnnotation = function () {
				if (!this.drawLinkedAnnotation) {
					annotationModel.unselectAllAnnotations(true, true);
				}
				var worldBounds = f.getModule("viewport").getWorldBounds(); //get the world bounds, this is the rect of the annotation
				var l = getAnnotationLayer("Main"); //get the annotation layer for the Main-View
				if (l) {
					newAnnotation = new Annotation(f);
					newAnnotation.initAnnotation(this.getAnnotationType(), this.getAnnotationColor(), false, annotationModel);
					//set the rect to the world rectangle converted to scene coordinates to make sure it get converted back to the correct annotation world coordinates
					newAnnotation.rect = f.getModule("viewport").getWorld().getSceneRectFromWorld(worldBounds);
					doneDrawing("Main"); //trigger a done drawing and use the annotation layer of the Main-view
				}
			};
			//register the listener to the model to create global annotations
			this._registerListener("addGlobalAnnotation", function () {
				if (this.f.isParameterUpdateOngoing()) {
					var paramChangedListener = this._registerListener("parameters.changed", function () {
						this._unregisterListener(paramChangedListener);
						this.addGlobalAnnotation();
					}, this, this.f.getDocumentController());
				} else {
					this.addGlobalAnnotation();
				}
			}, this, this._annotationModel);

			/**
			 * Get the annotation layer
			 */
			var getAnnotationLayer = _.bind(function (e) {
				var v = this._widget.requestView(e);
				if (v) {
					var l = v.getLayer("Annotations");
					if (l) {
						return l;
					}
				}
				return null;
			}, this);

			/**
			 * This is the mode when the tool is active,
			 * it will zoom on click and go the couldStartDraggingMode if left mousedown is clicked
			 */
			this.registerState(DEFAULTSTATE, {
				"mousedown": function (e) {},
				"mouseup": function (e) {},
				"mousemove": function (e) {},
				"mouseout": function () {},
				"contextmenu": function (e) {
					// prevent the context menu
					e.preventDefault();
					e.stopPropagation();
					return false;
				},
				"mouseleave": function () {},
				"click": function () {},
				canBeInterrupted: true
			});

			/**
			 * Draw state
			 */
			this.registerState(DRAWSTATE, _.defaults({
				'mousedown': _.bind(function (e) {
					if (e.which === 1) {
						// Annotation created by this tool are always user annotation, so set the filter to user annotations
						this._annotationModel.getAnnotationFilterModel().setOriginType(AnnotationConstants.AnnotationOriginTypes.user.id);
						if (!this.drawLinkedAnnotation) {
							annotationModel.unselectAllAnnotations(true, true);
						}

						this.requestCanvasLayer(e);

						var touchPoint = this.keepPointWithinWorldBounds(this.getWorldPoint(e), true);
						newAnnotation = new Annotation(f);

						newAnnotation.initAnnotation(this.getAnnotationType(), this.getAnnotationColor(), false, annotationModel);
						newAnnotation.rect = new ShapeUtils.Rect(touchPoint, new ShapeUtils.Size());

						this.changeState(DRAWINGSTATE);
					}
					e.originalEvent.preventDefault();
				}, this)
			}, this.getState(DEFAULTSTATE)));

			/**
			 * Function that should be called when the drawing is finished (does correct cleanups etc)
			 */
			var doneDrawing = _.bind(function (e) {
				cleanupCanvas(); //clear the canvas

				if (!e) {
					newAnnotation = null; //if there was no event passes, put the newAnnotation to null
				}

				//If the new annotation is a line or an arrow (pointArray==2), make sure that the points are not on the same location
				if (newAnnotation && newAnnotation.pointArray.length === 2) {
					if (newAnnotation.pointArray[0].x == newAnnotation.pointArray[1].x && newAnnotation.pointArray[0].y == newAnnotation.pointArray[1].y) {
						newAnnotation = null;
					}
				}

				//Make sure the annotation has at least some width or some height
				if (newAnnotation !== null && (Math.abs(newAnnotation.rect.size.width) === 0 || Math.abs(newAnnotation.rect.size.height) <= 0)) {
					//If both the width and height are 0, just return (not really useful to have a 0-sized annotation)
					if (Math.abs(newAnnotation.rect.size.width) === 0 && Math.abs(newAnnotation.rect.size.height) <= 0) {
						newAnnotation = null;
					} else {
						//If the box has either a 0-width or 0-height AND it's not a line/arrow, do not save the annotation
						if (newAnnotation.annottype != AnnotationConstants.AnnotationType.ANNOT_ARROW && newAnnotation.annottype != AnnotationConstants.AnnotationType.ANNOT_LINE) {
							newAnnotation = null;
						}
					}
				}

				//If we have an annotation, let's save it
				if (newAnnotation) {
					var l = getAnnotationLayer(e);
					if (l) {
						//Convert the drawed annotation from Scene (pixels) to World (mm)
						newAnnotation.rect = l.getWorldRectFromScene(newAnnotation.rect); //first the box
						if (this._annotationModel.getPage().isReaderSpread()) {
							this._annotationModule.promptIfAnnotationOutsidePage(newAnnotation.rect, _.bind(
									function (isCreatingLinkedAnnot) {
										var shiftPressed = this.drawLinkedAnnotation;
										if (isCreatingLinkedAnnot && !shiftPressed) {
											this.drawLinkedAnnotation = true;
										}
										this.createAnnotation(newAnnotation, l);
										if (isCreatingLinkedAnnot && !shiftPressed) {
											this.drawLinkedAnnotation = false;
										}
									}, this),
								function () {
									newAnnotation = null;
								}, this.drawLinkedAnnotation);
						} else {
							this.createAnnotation(newAnnotation, l);
						}

					} else {
						newAnnotation = null;
					}
				}

				this.changeState(DRAWSTATE);
			}, this);

			/**
			 * creates the annotation, makes a new annotation object, this will not save the annotation to the back-end
			 */
			this.createAnnotation = function (newAnnotation, annotationLayer) {
				for (var i = 0; i < newAnnotation.pointArray.length; i++) { //then the points
					var point = newAnnotation.pointArray[i];
					var p = new ShapeUtils.Point(point.x, point.y);
					var newp = annotationLayer.getWorldPointFromScene(p);
					point.x = newp.x;
					point.y = newp.y;

				}

				//Check if it need to be a linked annotation or a single new annotation
				if (this.drawLinkedAnnotation) {
					annotationModel.addLinkedAnnotation(newAnnotation);
				} else {
					annotationModel.addAnnotation(newAnnotation);
					if (this.isActive()) {
						this.requestDeactivation();
					}
				}
				newAnnotation = null;
			};

			/**
			 * Render the annotation via the draw utils
			 */
			var renderAnnotation = _.throttle(_.bind(function () {
				var ctx = this.getCanvasCtx();
				if (ctx) { // could be that the layer is already removed (caused by throttle)
					var csize = this.getCanvasSize();
					ctx.clearRect(0, 0, csize.width, csize.height);

					if (newAnnotation) {
						DrawUtils.drawOverlay(this.f, ctx, [newAnnotation]);
					}
				}
			}, this), 40);

			/**
			 * Drawing state
			 */
			this.registerState(DRAWINGSTATE, _.defaults({
				'enterState': function () {
					f.emit("viewport.startInteractive");
					f.emit("layout.disableTextSelection");
				},
				'leaveState': function () {
					f.emit("viewport.stopInteractive");
					setTimeout(function () {
						f.emit("layout.enableTextSelection");
					}, 500);
				},
				'mousemove': _.bind(function (e) {
					var newRect = newAnnotation.rect.copy();
					var touchPoint = this.keepPointWithinWorldBounds(this.getWorldPoint(e), true);

					if (this.getAnnotationType() == AnnotationConstants.AnnotationType.ANNOT_FREEFORM) {
						var point = new LineworkPoint();
						if (newAnnotation.pointArray.length === 0) {
							point.makeMoveTo(newAnnotation.rect.origin.x, newAnnotation.rect.origin.y);
						} else {
							var previousPoint = newAnnotation.pointArray[newAnnotation.pointArray.length - 1];
							if (Math.abs(previousPoint.x - touchPoint.x) < threshold || Math.abs(previousPoint.y - touchPoint.y) < threshold) {
								return;
							}
							point.makeLineTo(touchPoint.x, touchPoint.y);
						}
						newAnnotation.pointArray.push(point);

						//Check if the width needs to be updated
						var width = Math.abs(point.x - newRect.origin.x);
						if (point.x < newRect.origin.x) {
							var diff = Math.abs(newRect.origin.x - point.x);
							newRect.origin.x = point.x;
							newRect.size.width += diff;
						} else if (width > newRect.size.width) {
							newRect.size.width = width;
						}

						//Check if the height needs to be updated
						var height = Math.abs(point.y - newRect.origin.y);
						if (point.y < newRect.origin.y) {
							var diff = Math.abs(newRect.origin.y - point.y);
							newRect.origin.y = point.y;
							newRect.size.height += diff;
						} else if (height > newRect.size.height) {
							newRect.size.height = height;
						}
					} else {
						newRect.size = newAnnotation.rect.origin.sizeTo(touchPoint);
					}
					newAnnotation.setRect(newRect, true);

					renderAnnotation();
				}, this),
				'mouseclick': function (e) {

				},
				'mouseup': function (e) {
					doneDrawing(e);
				},
				'mouseleave': function (e) {
					doneDrawing();
				},
				'mousedown': function (e) {
					e.originalEvent.preventDefault();
				},
				canBeInterrupted: false
			}, this.getState(DEFAULTSTATE)));

			/**
			 * Can the tool be interrupted ?
			 */
			this.canBeInterrupted = function () {
				return false;
			};

			this.activeKeyEvents = [];
			this.persistentKeyEvents = [];
			/**
			 * @override
			 */
			this._override("initialize", function (widget, toolManager) {
				this._super(widget, toolManager);
				this.changeState(DEFAULTSTATE);
				this.registerActiveEvents(["mousedown", "mousemove", "mouseup", "mouseout", "mouseleave", "contextmenu"]);

				this.activeKeyEvents.push(["shift", _.bind(function (e) {
					this.drawLinkedAnnotation = true;
				}, this), "keydown"]);

				this.activeKeyEvents.push(["shift", _.bind(function (e) {
					this.drawLinkedAnnotation = false;
				}, this), "keyup"]);
			});

			/**
			 * Cleanup canvas
			 */
			var cleanupCanvas = _.bind(function () {
				this.releaseCanvasLayer();
			}, this);


			/**
			 * @override
			 */
			this.canBeActivated = function () {
				return annotationModel.canChangeAnnotations();
			};

			/**
			 * @override
			 */
			this._override("activate", function (interruptRequest) {
				this._super(interruptRequest);

				/*
				 * Initialize defaults
				 */
				this.getAnnotationType();
				this.getAnnotationColor();

				annotationModel.setLockSelection(true);

				/*
				 * Reconsider the state of the tool
				 */
				this.changeState(DRAWSTATE);
			});

			/**
			 * @override
			 */
			this._override("deactivate", function (interruptRequest) {
				this._super(interruptRequest);
				this.changeState(DEFAULTSTATE);
				cleanupCanvas();

				this.drawLinkedAnnotation = false;
				annotationModel.setLockSelection(false);
			});

			f.getCurrentUser().on("options.changed.PREFERENCES_ANNOTATIONTOOLS_VISIBLE", _.bind(function (oldValue, newValue) {
				if (!newValue && this.isActive()) {
					this.requestDeactivation();
				}
			}, this));

		};

		// inherit
		AnnotationDrawTool.prototype = Object.create(AbstractTool.prototype);
		// correct the constructor pointer 
		AnnotationDrawTool.prototype.constructor = AnnotationDrawTool;


		return AnnotationDrawTool;
	});

define('web-viewer/annotation/AnnotationIndicatorTool',["underscore",
		"web-viewer/viewport/tools/AbstractTool",
		"web-viewer/viewport/model/ViewportLayerModel",
		"utils/ShapeUtils",
		"web-viewer/annotation/AnnotationConstants",
		"web-viewer/core/CoreConstants"
	],
	function (_,
		AbstractTool,
		ViewportLayerModel,
		ShapeUtils,
		AnnotationConstants,
		CoreConstants) {
		"use strict";
		/**
		 * @class Tool to highlight annotations on the viewport
		 * @exports AnnotationIndicatorTool
		 * @memberof AnnotationModule
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f
		 * @param {AnnotationModel} annotationModel - the model used by the module for annotations
		 */
		var AnnotationIndicatorTool = function (f, annotationModel) {
			AbstractTool.call(this, f, CoreConstants.TOOLS.ANNOTATION_INDICATOR);
			this.f = f;

			this.layerModel = new ViewportLayerModel(f, "annotationIndication");
			this.layerModel.annotation = null;
			this.layerModel.tool = this;
			this.layerModel.annotationModel = annotationModel;
			this.layerModel.alpha = 0.7;

			/**
			 * Render the annotation indicator, highlights the annotation
			 * @param {Context2D} ctx
			 * @param {ShapeUtils.Rect} drawrect
			 */
			this.layerModel.render = function (ctx, drawrect) {
				ctx.fillStyle = "rgba(0, 0, 0, " + this.alpha + ")";

				ctx.clearRect(drawrect.origin.x, drawrect.origin.y, drawrect.size.width, drawrect.size.height);

				ctx.beginPath();
				ctx.rect(drawrect.origin.x, drawrect.origin.y, drawrect.size.width, drawrect.size.height);

				ctx.fill();

				ctx.save();
				if (this.annotation) {
					var inset = Math.abs(this.tool.sceneToWorldSize(new ShapeUtils.Size(10, 10)).width);
					var page = this.annotationModel.getAnnotationsPage();
					if (page !== null && page !== undefined) {
						this.resetTransformation();
						this.getTransform().scale(1, -1);

						var rotation = page.getRotation();

						var worldBounds = this.f.getModule("viewport").getWorldBounds();

						this.calculateTransformation(worldBounds, page.getPageSize(), this.annotationModel.getAnnotationsOffset(), rotation, page.isFlipped());
					}

					var r = this.annotation.rect.copy();

					r.normalize();

					ctx.clearRect(r.origin.x - inset, r.origin.y + r.size.height + inset, r.size.width + 2 * inset, -r.size.height - 2 * inset);

					var links = this.annotationModel.getAnnotationLinks(this.annotation);
					for (var i = 0; i < links.length; i++) {
						r = links[i].rect.copy();

						r.normalize();

						ctx.clearRect(r.origin.x - inset, r.origin.y + r.size.height + inset, r.size.width + 2 * inset, -r.size.height - 2 * inset);

					}


				}
				ctx.restore();
			};

			this._override("activate", function () {
				this._super();

				this.f.getModule("viewport").addLayer("annotationIndication", this.layerModel);
			});

			this._override("deactivate", function (interruptRequest) {
				this._super(interruptRequest);

				this.f.getModule("viewport").removeLayer("annotationIndication");
			});
		};

		// inherit
		AnnotationIndicatorTool.prototype = Object.create(AbstractTool.prototype);
		// correct the constructor pointer 
		AnnotationIndicatorTool.prototype.constructor = AnnotationIndicatorTool;

		/**
		 * Can the tool be interrupted ?
		 */
		AnnotationIndicatorTool.prototype.canBeInterrupted = function () {
			return false;
		};

		/**
		 * Set the currently active annotation
		 * @param {Annotation} annotation
		 */
		AnnotationIndicatorTool.prototype.setAnnotation = function (annotation) {
			this.layerModel.annotation = annotation;
		};


		return AnnotationIndicatorTool;
	});

 define('web-viewer/annotation/AnnotationOriginSelector',[
 	"jquery",
 	"web-viewer/annotation/AnnotationConstants",
 	"component/toolbar/ButtonGroup",
 	"component/toolbar/ToolbarButton",
 	"component/tooltip/Tooltip"
 ], function ($,
 	AnnotationConstants,
 	ButtonGroup,
 	ToolbarButton,
 	Tooltip
 ) {


 	"use strict";

 	/**
 	 * @class
 	 * @exports AnnotationOriginSelector
 	 * @param {Core.ViewerFacade} f
 	 * @param {AnnotationFilterModel2} filterModel
 	 * @param {AnnotationModel} annotationModel
 	 */
 	var AnnotationOriginSelector = function (f, filterModel, annotationModel, context, isMobile) {

 		/**
 		 * @member {AnnotationFilterModel2}
 		 * @private
 		 */
 		this._filterModel = filterModel;

 		/**
 		 * @member {AnnotationModel}
 		 * @private
 		 */
 		this._annotationModel = annotationModel;

 		this._annotationModel.on("modelChanged", _.bind(this._enableToolbarButtons, this));
 		this._annotationModel.on("modelContentChanged", _.bind(this._enableToolbarButtons, this));
 		this._annotationModel.on("modelContentChanged", _.bind(this._activateToolbarButtons, this));
 		this._filterModel.on("filterChanged", _.bind(this._activateToolbarButtons, this));


 		/**
 		 * @member {Core.ViewerFacade}
 		 * @private
 		 */
 		this.f = f;

 		/**
 		 * @member {ButtonGroup}
 		 * @private
 		 */
 		this._buttonGroup = new ButtonGroup("annotationTypeButtonGroup");

 		/**
 		 * @member {JQueryElement}
 		 * @private
 		 */
 		this.element = $("<div class='annotation-origin-selector'/>").append(this._buttonGroup.element);

 		/**
 		 * @member {JQueryElement}
 		 * @private
 		 */
 		this._context = context;

 		/**
 		 * @member {Boolean}
 		 * @private
 		 */
 		this._isMobile = isMobile;

 		/**
 		 * @member {Object} id -> ToolbarButton
 		 * @private
 		 */
 		this._buttons = {};


 		this._initTypeFilters();

 	};

 	/**
 	 * creates the origin buttons from the annotation types defined in AnnotationConstants.AnnoationTypes
 	 */
 	AnnotationOriginSelector.prototype._initTypeFilters = function () {
 		var annotationTypes = AnnotationConstants.AnnotationOriginTypes;
 		_.each(annotationTypes, function (annotationType) {
 			var id = annotationType.id;

 			var annotationIcon = annotationType.icon;
 			if (annotationType.subIcon) {
 				annotationIcon = $("<i class='icon containerIcon icon-normal'/>").addClass(annotationIcon);
 				annotationIcon.append($("<i class='icon subIcon icon-normal'/>").addClass(annotationType.subIcon));
 			}

 			var button = new ToolbarButton(this.f.i18n, this._context, this._isMobile, id, annotationIcon, this._isMobile, _.bind(function () {
 				this._filterModel.setOriginType(id);
 				this._annotationModel.unselectAllAnnotations();
 			}, this), this.f.i18n.t(annotationType.langKey));
 			this._buttonGroup.addButton(button);
 			this._buttons[id] = button;
 		}, this);
 		this._enableToolbarButtons();
 		this._activateToolbarButtons();
 	};

 	/**
 	 * enables the buttons depending on the availability of the types in the annotation model
 	 */
 	AnnotationOriginSelector.prototype._enableToolbarButtons = function () {
 		var annotationsTypes = _.uniq(_.map(this._annotationModel.getAnnotations(), function (annotation) {
 			return annotation.originType;
 		}));
 		var annotationGroups = _.uniq(_.map(annotationsTypes, function (type) {
 			return AnnotationConstants.AnnotationOriginTypes[type].group;
 		}));
 		_.each(this._buttons, function (button, id) {
 			var annotationsOfIdExist = _.find(annotationsTypes, function (type) {
 				return id === type;
 			});
 			var annotationsOfGroupExist = _.find(annotationGroups, function (group) {
 				return AnnotationConstants.AnnotationOriginTypes[id].group === group;
 			});
 			button.setEnabled(annotationsOfIdExist || id === AnnotationConstants.AnnotationOriginTypes.user.id);
 			var originType = AnnotationConstants.AnnotationOriginTypes[id];
 			if (originType && originType.hideBtnOnEmpty) {
 				if (annotationsOfIdExist) {
 					button.show();
 				} else {
 					button.hide();
 				}
 			} else if (originType && originType.hideBtnOnGroupEmpty) {
 				if (annotationsOfGroupExist) {
 					button.show();
 				} else {
 					button.hide();
 				}
 			}
 		});
 	};

 	/**
 	 * activates the buttons depending on the filter model
 	 */
 	AnnotationOriginSelector.prototype._activateToolbarButtons = function () {
 		var id = this._filterModel.getOriginType();
 		_.each(this._buttons, function (button, i) {
 			button.setActive(button.name === id);
 		});
 	};

 	return AnnotationOriginSelector;
 });

define('tpl!web-viewer/annotation/AnnotationCustomFilterPopup', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='\r\n    <div class="filterOperator">\r\n        <span class="operatorText">'+
((__t=(d.i18n("FilterMatch")))==null?'':_.escape(__t))+
'</span>\r\n        <select class="filterOperatorSelector">\r\n            <option ';
 if(d.filterOperator == 0){ 
__p+='selected="selected"';
 } 
__p+=' value="0">'+
((__t=(d.i18n("AnnotFilterAllOperator")))==null?'':_.escape(__t))+
'</option>    \r\n            <option ';
 if(d.filterOperator == 1){ 
__p+='selected="selected"';
 } 
__p+=' value="1">'+
((__t=(d.i18n("AnnotFilterAnyOperator")))==null?'':_.escape(__t))+
'</option>\r\n        </select>\r\n        <span class="operatorText">'+
((__t=(d.i18n("FilterOperator")))==null?'':_.escape(__t))+
'</span>\r\n    </div>\r\n    <div class="filterRows">\r\n        ';
 _.each(d.getFilterRows(), function(filterRow, index){ 
__p+='\r\n            <div class="" data-bind="'+
((__t=(index))==null?'':_.escape(__t))+
'">\r\n                '+
((__t=( d.callTemplate("web-viewer/annotation/AnnotationCustomFilterRow", filterRow, "filter") ))==null?'':__t)+
'\r\n            </div>	\r\n        ';
 }); 
__p+='\r\n    </div>\r\n    <a class="addFilterRow">\r\n        <i class="icon-WCR_Filter-Add12"></i><span class="addFilterRowText">'+
((__t=(d.i18n("AnnotFilterAddRow")))==null?'':_.escape(__t))+
'</span>\r\n    </a>';
return __p;
}});
define('tpl!web-viewer/annotation/AnnotationCustomFilterRow', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<div class="filterRow" data-bind="'+
((__t=(d.filter.filterId ))==null?'':_.escape(__t))+
'">\r\n    <select class="filterType">\r\n        ';
 _.each(d.getFilterTypes(), function(filterType, index){ 
__p+='\r\n            <option ';
 if(d.filter.filterType == index){ 
__p+='selected="selected"';
 } 
__p+=' value="'+
((__t=(index))==null?'':_.escape(__t))+
'">'+
((__t=(d.i18n(filterType)))==null?'':_.escape(__t))+
'</option>\r\n        ';
 }) 
__p+='\r\n    </select>\r\n    <select class="filterCondition">\r\n        <option ';
 if(d.filter.filterCondition == 0){ 
__p+='selected="selected"';
 } 
__p+=' value="0">'+
((__t=(d.i18n("AnnotFilterIs") ))==null?'':_.escape(__t))+
'</option>\r\n        <option ';
 if(d.filter.filterCondition == 1){ 
__p+='selected="selected"';
 } 
__p+=' value="1">'+
((__t=(d.i18n("AnnotFilterIsNot") ))==null?'':_.escape(__t))+
'</option>\r\n    </select>\r\n    <select class="filterOption">\r\n        ';
 _.each(d.getOptions(d.filter.filterType, d.filter.filterOption), function(option, index){ 
__p+='\r\n            <option ';
 if(d.filter.filterOption === index){ 
__p+='selected="selected"';
 } 
__p+=' ';
 if(index === "hidden"){ 
__p+='selected hidden';
 } 
__p+=' value="'+
((__t=(index ))==null?'':_.escape(__t))+
'">\r\n                '+
((__t=( option ))==null?'':_.escape(__t))+
'\r\n            </option>\r\n        ';
 }) 
__p+='  \r\n    </select>\r\n    ';
 if(d.filter.filterId !== 0){ 
__p+='\r\n        <a class="removeFilterRow" data-bind="'+
((__t=(d.filter.filterId ))==null?'':_.escape(__t))+
'">\r\n            <i class="icon-WCR_Filter-Remove12"></i>\r\n        </a>\r\n    ';
 } 
__p+='\r\n</div>';
return __p;
}});
define('web-viewer/annotation/AnnotationCustomFilterPopup',["web-viewer/core/PrivateChannel",
		"web-viewer/layout/component/Popup",
		"web-viewer/layout/component/ViewerToolbarButton",
		"Template",
		"web-viewer/annotation/AnnotationConstants",
		"tpl!web-viewer/annotation/AnnotationCustomFilterPopup",
		"tpl!web-viewer/annotation/AnnotationCustomFilterRow",
		"jquery",
		"underscore"
	],
	function (
		PrivateChannel,
		Popup,
		ViewerToolbarButton,
		Template,
		AnnotationConstants,
		mainTpl,
		subTpl,
		$,
		_
	) {
		"use strict";

		/**
		 * Create a popup for creating a custom annotation filter
		 * @exports AnnotationCustomFilterPopup
		 * @memberof AnnotationModule
		 * @constructor
		 * @augments Layout.Popup
		 *
		 * @param {Core.ViewerFacade} f - Viewerfacade
		 * @param {String} popupId - Id of the popup used for its private channel
		 */
		var AnnotationCustomFilterPopup = function (f, popupId, annotationModel) {
			Popup.call(this, f, popupId, f.getContext());
			PrivateChannel.installTo(this, f, ['annotationFilterPopup']);

			this.element.addClass("annotation-custom-filter-popup");
			
			/**
			 * @member {AnnotationModel}
			 * @private
			 */
			this._annotationModel = annotationModel;

			/**
			 * @member {AnnotationFilterModel}
			 * @private
			 */
			this._filterModel = this._annotationModel.getAnnotationFilterModel();

			/**
			 * @member {Array.<Object>} saves the state of the filter when the popup was opened to be able to restore it
			 * @private
			 */
			this._savedFilterState = null;

			/**
			 * @member {Boolean} remember if the custom filter was enabled when the popup was opened, to be able to restore the filter correctly
			 * @private
			 */
			this._wasCustomFilterActive = false;

			/**
			 * @member {JQueryObject} content of the entire filter in the popup (not the action buttons)
			 * @private
			 */
			this._filterContent = $("<div>").addClass("annotationFilter");

			/**
			 * @member {ShapeUtils.Point} location the popup should be rendered
			 * @private
			 */
			this._location = null;

			this.content.append(this._filterContent);

			/**
			 * @member {ViewerToolbarButton}
			 * @private
			 */
			this._okButton = new ViewerToolbarButton(this.f, "okButton", $("<span/>"), false, _.bind(function () {
				this._filterModel.setCustomFilterActive(true); //set the filter to be active
				//to make sure the state doesn't get persisted with every small change in the popup, only persist when the popup is closed
				this._filterModel.fireFilterStateChanged(); //filter is applied and the popup gets closed.
				this.hide();
			}, this));
			this._okButton.addClass("btn-success");

			/**
			 * @member {ViewerToolbarButton}
			 * @private
			 */
			this._cancelButton = new ViewerToolbarButton(this.f, "cancelButton", $("<span/>"), false, _.bind(function () {
				this._filterModel.setFilterState(this._savedFilterState);
				this._savedFilterState = null;
				if (!this._wasCustomFilterActive) {
					this._filterModel.setCustomFilterActive(false);
				}
				this.hide();
			}, this));
			this._cancelButton.addClass("btn");

			//append the buttons to a general container present in the popup (bottom right corner, location to add buttons)
			this.addButton(this._cancelButton.element);
			this.addButton(this._okButton.element);

			//initialize the template to be used for this popup
			this.t = new Template(this.f.i18n, mainTpl);
			this.t.setContext(this._filterContent); //the content is the pre-defined div

			//register several functions on the template
			//returns all the filter rows, adds the default filter row if there are none
			this.t.d.getFilterRows = _.bind(function () {
				var annotationFilters = this._filterModel.getFilterRows();

				return annotationFilters;
			}, this);

			//get the filter types that are available for the annotation filter
			this.t.d.getFilterTypes = _.bind(function () {
				return _.values(AnnotationConstants.FilterTypeNameLocs);
			}, this);

			//get all the filter options for a specific filter type
			this.t.d.getOptions = _.bind(function (selectedFilter, selectedOption) {
				var users = this._annotationModel.getUsers(AnnotationConstants.AnnotationOriginTypes.user.id);
				if (selectedFilter == AnnotationConstants.FilterTypes.AUTHOR) {
					if (selectedOption && users[selectedOption] === undefined) {
						users["hidden"] = this.f.i18n.t("AnnotFilterUserNotAvailable");
					}
					return users;
				} else if (selectedFilter == AnnotationConstants.FilterTypes.REVIEWSTATE || selectedFilter == AnnotationConstants.FilterTypes.REVIEWAUTHORANDSTATE) {
					//for both review state and review author and state, translate review state string are needed
					var reviewStates = {}; //construct an object with same keys but translated review states
					_.each(AnnotationConstants.AnnotationReviewStateLocs, function (value, key) {
						reviewStates[key] = this.f.i18n.t(value); //translation possible, template rendering is only done if the language bundle is loaded
					}, this);

					if (selectedFilter == AnnotationConstants.FilterTypes.REVIEWSTATE) {
						return reviewStates;
					} else { //REVIEWAUTHORANDSTATE
						var reviewAuthorAndState = {}; //construct object with a combination of the user name and review state
						_.each(users, function (user, index) {
							_.each(reviewStates, function (reviewState, key) {
								reviewAuthorAndState[user + " " + key] = user + " " + reviewState;
							});

							if (index !== _.last(_.keys(users))) { //between all review states of an user add a delimiter
								reviewAuthorAndState[user + "Delimiter"] = ("──────────");
							}
						}, this);
						if (selectedOption && reviewAuthorAndState[selectedOption] === undefined) {
							reviewAuthorAndState["hidden"] = this.f.i18n.t("AnnotFilterUserNotAvailable");
						}
						return reviewAuthorAndState;
					}
				}

			}, this);

			//add the listener to the popup, to be able to use the close button on the top right
			this.on("closed", _.bind(function () {
				this.hide();
			}, this));

			//add listeneres to key events to selectors and buttons of the by template rendered popup
			this.t.bind("click", ".addFilterRow", function (source, event) {
				var filterId = this._filterModel.addEmptyRow();
				this._addFilterRow(filterId);
			}, this);

			this.t.bind("click", ".removeFilterRow", function (source, event) {
				var filterId = $(source).attr("data-bind");
				this._removeFilterRow(filterId);
			}, this);

			//value in first dropdown changes, the third dropdownlist needs to be updated
			this.t.bind("change", ".filterType", function (source, event) {
				var filterId = $(source).parent().attr("data-bind");
				var selectedValue = $(source).val(); //get the currently selected option of the dropdown
				var options = this.t.d.getOptions(selectedValue); //get the options for the selected filter type (to be able to set default option)
				this._filterModel.updateFilterType(filterId, selectedValue, _.first(_.keys(options))); //take the first options as the default selected option
				//rerender the changed filter tow to make sure the options dropdown is filled with correct values
				this._rerenderFilterRow(filterId);
			}, this);

			this.t.bind("change", ".filterCondition", function (source, event) {
				var filterId = $(source).parent().attr("data-bind");
				var selectedValue = $(source).val();
				this._filterModel.updateFilterCondition(filterId, selectedValue);
			}, this);

			this.t.bind("change", ".filterOption", function (source, event) {
				var filterId = $(source).parent().attr("data-bind");
				var selectedValue = $(source).val();
				this._filterModel.updateFilterOption(filterId, selectedValue);
			}, this);

			this.t.bind("change", ".filterOperatorSelector", function (source, event) {
				var selectedValue = $(source).val(); //operator (ALL or ANY) changed
				this._filterModel.setFilterOperator(selectedValue);
			}, this);

			//load the language bundle to set the correct popup title and contents of the ok and cancel buttons
			this.f.i18n.loadBundle("viewer", function () {
				this.setHeader($("<span/>").addClass("popupTitle").html(this.f.i18n.t("CustomAnnotationFilter")));

				this._okButton.element.html(this.f.i18n.t("ApplyCustomFilter"));
				this._cancelButton.element.html(this.f.i18n.t("AppDialogCancel"));
			}, this);
		};

		AnnotationCustomFilterPopup.prototype = Object.create(Popup.prototype);
		AnnotationCustomFilterPopup.prototype.constructor = AnnotationCustomFilterPopup;

		/**
		 * Renders the popup template
		 */
		AnnotationCustomFilterPopup.prototype.render = function () {
			this.f.i18n.loadBundle("viewer", function () {
				this.t.d.filterOperator = this._filterModel.getFilterOperator();
				this.t.render();
			}, this);
		};

		/**
		 * Rerender a specific filter row (uses the subTemplate)
		 * @param {Number} filterId
		 */
		AnnotationCustomFilterPopup.prototype._rerenderFilterRow = function (filterId) {
			var filterRow = this._filterModel.getFilterRow(filterId);
			this.content.find(".filterRow[data-bind='" + filterId + "']").html(this.t.d.callTemplate("web-viewer/annotation/AnnotationCustomFilterRow", filterRow, "filter"));
		};

		/**
		 * Add a row the filter, adds a row in the popup
		 * @param {Number} filterId
		 */
		AnnotationCustomFilterPopup.prototype._addFilterRow = function (filterId) {
			var filterRow = this._filterModel.getFilterRow(filterId);
			this.content.find(".filterRows").append(this.t.d.callTemplate("web-viewer/annotation/AnnotationCustomFilterRow", filterRow, "filter"));
			this.recalculateSize(this._location, 475);
		};

		/**
		 * Remove a filter row from the popup
		 * @param {Number} filterId
		 */
		AnnotationCustomFilterPopup.prototype._removeFilterRow = function (filterId) {
			this.content.find(".filterRow[data-bind='" + filterId + "']").remove();
			this._filterModel.removeFilterRow(filterId);
			this.recalculateSize(this._location, 475);
		};

		/**
		 * Show the popup, opening the popup sets the costom filter active
		 * @param {ShapeUtils.Point} location The location of the popup
		 */
		AnnotationCustomFilterPopup.prototype.show = function (location) {
			this._savedFilterState = this._filterModel.getFilterState(); //save the custom filter state to be able to restore it
			this._wasCustomFilterActive = this._filterModel.isCustomFilterActive(); //save the custom filter state
			this.render(); //render the template
			this._location = location;
			this._filterModel.setCustomFilterActive(true);
			Popup.prototype.show.call(this, location, 475);
		};

		return AnnotationCustomFilterPopup;
	});

define('web-viewer/annotation/AnnotationWidget',["web-viewer/layout/AbstractWidget",
		"web-viewer/annotation/AnnotationRow",
		"web-viewer/annotation/AnnotationOriginSelector",
		"web-viewer/annotation/AnnotationCustomFilterPopup",
		"jquery",
		"web-viewer/annotation/AnnotationConstants",
		"utils/ShapeUtils",
		"web-viewer/layout/component/MiniButton",
		"component/button/Button",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/DropdownMiniButton",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/layout/component/ViewerToolbarButton",
		"underscore"
	],
	function (AbstractWidget,
		AnnotationRow,
		AnnotationOriginSelector,
		AnnotationCustomFilterPopup,
		$,
		AnnotationConstants,
		ShapeUtils,
		MiniButton,
		Button,
		ButtonGroup,
		DropdownMiniButton,
		LayoutConstants,
		ViewerToolbarButton,
		_) {
		"use strict";


		/**
		 * Construct the AnnotationWidget
		 * @exports AnnotationWidget
		 * @memberof AnnotationModule
		 * @class
		 * The AnnotationWidget displays the list of annotations and gives the user options to change the way annotations are displayed. It is possible to filter, order and step through annotations. Annotation can be edited in the widget
		 * @augments Layout.AbstractWidget
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {AnnotationModule} annotationModule
		 * @param {AnnotationModel} annotationModel - The annotation model
		 * @param {String} layoutType - the layout type the widget needs to be rendered for
		 */
		var AnnotationWidget = function (f, annotationModule, annotationModel, layoutType) {
			AbstractWidget.call(this, f, LayoutConstants.Widget.ANNOTATION);

			this.f = f;
			this.annotationModel = annotationModel;
			this.annotationModule = annotationModule;
			/**
			 * In this hash set all annotationRow objects are stored
			 */
			this.annotationRows = {};

			this._isForMobile = layoutType === LayoutConstants.LayoutType.MOBILE;


			/**
			 * Details
			 */
			this.detailsDiv = $("<div>").addClass("annotationDetails enableTextSelection");

			this.nonScrollablePart = $("<div>").addClass("nonScrollablePart");

			this._annotationOriginSelector = new AnnotationOriginSelector(f, annotationModel.getAnnotationFilterModel(), annotationModel, this.detailsDiv, this._isForMobile);
			this.nonScrollablePart.append(this._annotationOriginSelector.element);
			this.detailsDiv.append(this.nonScrollablePart);

			this.warningMessage = $("<div/>").addClass("alert").text(this.annotationModel.getDisabledReason()).toggle(this.annotationModel.getDisabledReason().length > 0);
			this.nonScrollablePart.append(this.warningMessage);

			// this div is shown when annotations are temporarily disabled
			this.temporarilyDisabledDiv = $("<div/>").addClass("annotationsTemporarilyDisabled").hide();
			this.nonScrollablePart.append(this.temporarilyDisabledDiv);

			// note i18n is done a bit later, I wanted the references to be available already (even if te language is not loaded yet)
			this.showCurrentFileAnnotationsButton = new Button(this.f.i18n, this.f.getContext(), this.f.isMobile(), "currentDocument", $("<span/>").text("current").prepend($("<i>").addClass("icon icon-WCR_Compare-Current")), true, _.bind(function () {
				this.annotationModule.toggleCurrentPageAnnotations();
			}, this));

			this.showCurrentFileAnnotationsButton.element.addClass("btn");
			// note i18n is done a bit later, I wanted the references to be available already (even if te language is not loaded yet)
			this.showDifferenceFileAnnotationsButton = new Button(this.f.i18n, this.f.getContext(), this.f.isMobile(), "differenceDocument", $("<span/>").text("reference").prepend($("<i>").addClass("icon icon-WCR_Compare-Reference")), true, _.bind(function () {
				this.annotationModule.toggleDifferencePageAnnotations();
			}, this));
			this.showDifferenceFileAnnotationsButton.element.addClass("btn");

			this.annotationToggleGroup = new ButtonGroup("annotationToggleBetweenCurrentAndDifference", [this.showCurrentFileAnnotationsButton, this.showDifferenceFileAnnotationsButton]);
			this.temporaryAnnotationToggler = $("<div/>").addClass("temporaryAnnotationToggle");

			// i18n for temporarily disabling 
			f.i18n.loadBundle("viewer", function () {
				this.temporaryAnnotationToggler.append($("<span/>").text(this.f.t("AnnShowAnnotationsFor") + ": "))
					.append($("<br/>"))
					.append(this.annotationToggleGroup.element);

				this.showCurrentFileAnnotationsButton.setContent(
					$("<span/>").text(this.f.t("CaptionCurrent")).prepend($("<i>").addClass("icon icon-WCR_Compare-Current"))
				);

				this.showDifferenceFileAnnotationsButton.setContent(
					$("<span/>").text(this.f.t("CaptionPrevious")).prepend($("<i>").addClass("icon icon-WCR_Compare-Reference"))
				);

			}, this);
			this.temporarilyDisabledDiv.append(this.temporaryAnnotationToggler);
			this.temporaryAnnotationToggler.hide();


			// filter
			this.header = $("<div>").addClass("annotationFilter");
			this._annotationFilters = $("<div>").addClass("annotationFilterSelectors");
			this._customFilters = $("<div>").addClass("customFilterControls");
			this.header.append(this._annotationFilters);
			this._numberOfSelectedThreads = $("<span/>").addClass("threadsCount");
			this.header.append(this._customFilters);
			this._customFilters.append(this._numberOfSelectedThreads);

			this.annotationModel.getAnnotationFilterModel().on("filterStateChanged", _.bind(function () {
				if (this.annotationModel.getAnnotationFilterModel().getOriginType() !== AnnotationConstants.AnnotationOriginTypes.user.id) {
					if (this._annotationFilters) {
						this._annotationFilters.hide();
					}
					if (this._pinFilterButton) {
						this._pinFilterButton.element.hide();
					}
					if (this._openFilterPopupButton) {
						this._openFilterPopupButton.element.hide();
					}
					if (this._duplicateFilteredAnnotationsButton) {

						if (_.all(this.annotationModel.getFilteredAnnotations(), function (annotation) {
								return annotation.getPermissionFor(AnnotationConstants.ActionPermissions.DUPLICATE);
							})) {
							this._duplicateFilteredAnnotationsButton.element.show();
						} else {
							this._duplicateFilteredAnnotationsButton.element.hide();
						}
					}
					if (this._copyFilteredAnnotationsWithoutText) {
						this._copyFilteredAnnotationsWithoutText.element.show();
					}
				} else {
					if (this._annotationFilters) {

						this._annotationFilters.show();
					}
					if (this._pinFilterButton) {
						this._pinFilterButton.element.show();
					}
					if (this._openFilterPopupButton) {
						this._openFilterPopupButton.element.show();
					}
					if (this._duplicateFilteredAnnotationsButton) {
						this._duplicateFilteredAnnotationsButton.element.hide();
					}
					if (this._copyFilteredAnnotationsWithoutText) {
						this._copyFilteredAnnotationsWithoutText.element.hide();
					}
				}
			}, this));

			if (!this._isForMobile) {
				this._customFilterPopup = new AnnotationCustomFilterPopup(this.f, "customFilter", this.annotationModel);

				this.annotationModel.getAnnotationFilterModel().on("filterReset", _.bind(function () {
					if (this._customFilterPopup.isVisible()) {
						this._customFilterPopup.hide();
					}
				}, this));

				//add a button to pin the annotation filter
				this._pinFilterButton = new MiniButton(this.f, "pinFilter", "icon-WCR_Pin12", true, _.bind(function () {
					this.annotationModel.getAnnotationFilterModel().toggleAnnotationPinnedState();
					this._updateFilterPinnedIcon();
				}, this));
				this._updateFilterPinnedIcon();
				this._pinFilterButton.element.addClass("pinAnnotationFilter");

				//add a button to copy without text all filtered annotations
				this._copyFilteredAnnotationsWithoutText = new MiniButton(this.f, "duplicateAllFilteredAnnotationWithoutText", " icon icon-WCR_Annotation-Duplicate-NoText12", true, _.bind(function () {
					this.f.emit("module.annotation.duplicateAllFilteredAnnotationWithoutText");
				}, this));

				this._copyFilteredAnnotationsWithoutText.setTooltip("viewer", "duplicateAllFilteredAnnotationWithoutText");
				this._copyFilteredAnnotationsWithoutText.element.addClass("pinAnnotationFilter");
				this._copyFilteredAnnotationsWithoutText.element.hide();

				//add a button to copy all filtered annotations
				this._duplicateFilteredAnnotationsButton = new MiniButton(this.f, "duplicateAllFiltered", "icon icon-WCR_Annotation-Duplicate12", true, _.bind(function () {
					this.f.emit("module.annotation.duplicateAllFilteredAnnotations");
				}, this));
				this._duplicateFilteredAnnotationsButton.setTooltip("viewer", "duplicateAllFilteredAnnotations");
				this._duplicateFilteredAnnotationsButton.element.addClass("pinAnnotationFilter");
				this._duplicateFilteredAnnotationsButton.element.hide();

				this._openFilterPopupButton = new MiniButton(this.f, "customFilter", "icon-WCR_Filter12", true, _.bind(function () {
					var left = this.context.offset().left < 250 ? this.context.width() : this.context.offset().left - 250;
					var location = new ShapeUtils.Point(left, this.context.offset().top);
					this._customFilterPopup.show(location);
				}, this));
				this._openFilterPopupButton.element.addClass("customFilterBtn");
				this._openFilterPopupButton.setTooltip("viewer", "OpenCustomFilterPopup");

				this._customFilters.append(this._copyFilteredAnnotationsWithoutText.element);
				this._customFilters.append(this._duplicateFilteredAnnotationsButton.element);

			}

			// the list wrapper is used to only have a scrollbar for the annotationlist and not for the filter
			this.annotationsListWrapper = $("<div>").addClass("annotationListWrapper");
			this.annotationsListWrapper.on("makeMeVisible", _.partial(function (that, e) {
				// add support for makeMeVisible trigger on the scrollable pane

				// element
				var et = $(e.target).offset().top;
				var eh = $(e.target).outerHeight();
				// content
				var ct = $(this).offset().top;
				var ch = $(this).outerHeight();

				// check whether we need to scroll
				if (et - ct < 0 || et + eh > ct + ch) {
					var top = $(this).scrollTop() + et - ct;
					$(this).scrollTop(top);
				}

				that._visibleItem = $(e.target);

				e.preventDefault();
				e.stopPropagation();
			}, this));

			if (!this._isForMobile) {
				this._customFilters.append(this._openFilterPopupButton.element);
				this._customFilters.append(this._pinFilterButton.element);
			}

			this.detailsDiv.append(this.annotationsListWrapper);

			this.annotationList = $("<div>").addClass("annotationList");

			if (this._isForMobile) {
				this.annotationsListWrapper.append(this.header);
			} else {
				this.nonScrollablePart.append(this.header);
			}
			this.annotationsListWrapper.append(this.annotationList);


			f.i18n.loadBundle("viewer", function () {
				this.setTitle(this.f.i18n.t("AnnWidgetTitle"));
			}, this);


			this.setIcon("icon icon-WCR_Comment");

			/**
			 * In here for every annotation row that is in the DOM is noted
			 * @private
			 */
			this.inDOM = {};
			/**
			 * In this object all elements that should be removed from the DOM are noted
			 * @private
			 */
			this.toBeRemovedFromDOM = {};

			/**
			 * The button in the header to save the annotation
			 * @private
			 */
			this.saveButton = null;
			/**
			 * Should the button be disabled?
			 * @private
			 */
			this.disableSaveButton = false;

			this.annotationModel.on("disabledReasonChanged", _.bind(function () {
				this.warningMessage.text(this.annotationModel.getDisabledReason());
				this.warningMessage.toggle(this.annotationModel.getDisabledReason().length > 0);
				this.resized();
			}, this));
			/**
			 * When annotations are saved, update the save button
			 */
			this.annotationModel.on("save.finished", _.bind(function () {
				this.disableSaveButton = false;
				this.updateSaveButton();
				this.reloadButton.setEnabled(true);
			}, this));

			/**
			 * When the annotation has changed, update all rows
			 */
			this.annotationModel.on("readWriteChanged", _.bind(function () {
				_.each(this.annotationRows, function (row) {
					row.annotationUpdated();
				});
			}, this));

			/**
			 * Save annotations button
			 */
			this.saveButton = new MiniButton(f, "saveAnnotations", "icon-WCR_Save", true, _.bind(function () {
				// start saving annotations
				this.annotationModel.save();
			}, this), "fontIconDisabled icon-WCR_Save");

			this.updateSaveButton();

			/**
			 * Step through annotation sbutton
			 */
			this.stepThroughButton = new MiniButton(f, "stepThroughAnn", "icon-WCR_Step", true, _.bind(function (e) {
				if (e.shiftKey) { //check if the shift key is pressed	
					this.annotationModel.goToAnnotation(false);
				} else {
					this.annotationModel.goToAnnotation(true);
				}
			}, this));
			this.stepThroughButton.setDisabledIcon("icon-WCR_Step fontIconDisabled");
			// on right click, step through backwards
			this.stepThroughButton.element.on("contextmenu", _.bind(function (e) {
				e.preventDefault();
				if (this.stepThroughButton.isEnabled()) {
					this.annotationModel.goToAnnotation(false);
				}
			}, this));
			this.stepThroughButton.setTooltip("viewer", "StepThroughAnnTT");

			/**
			 * Reload annotations button
			 */
			this.reloadButton = new MiniButton(f, "reloadAnnotations", "icon-WCR_Refresh", true, _.bind(function () {
				this.annotationModel.reload();
			}, this), "icon-WCR_Refresh fontIconDisabled");
			this.reloadButton.setTooltip("viewer", "ReloadMiniBtnTT");

			// while saving, disable reload button
			this.annotationModel.on("save.started", _.bind(function () {
				this.reloadButton.setEnabled(false);
				this.disableSaveButton = true;
				this.updateSaveButton();
			}, this));

			// while reloading, disable reload button
			this.annotationModel.on("reload.started", _.bind(function () {
				this.reloadButton.setEnabled(false);
			}, this));
			this.annotationModel.on("reload.finished", _.bind(function () {
				this.reloadButton.setEnabled(true);
			}, this));

			this.annotationModel.on("temporarilyDisableChanged", _.bind(this.updateListVisibility, this));
			this.annotationModel.on("disabledReasonChanged", _.bind(this.updateListVisibility, this));
			this.updateListVisibility();

			/**
			 * Listen to user settings
			 */

			// set sort by mode
			f.getCurrentUser().on("options.changed.PREFERENCES_ANNOTATION_DATESORT_MODE", _.bind(function (oldValue, newValue) {
				this.setSortByMode(newValue);
			}, this));

			// step through mode
			f.getCurrentUser().on("options.changed.PREFERENCES_ANNOTATION_NEXT_MODE", _.bind(function (oldValue, newValue) {
				this.setStepThroughMode(newValue);
			}, this));

			// show only selected annotation
			f.getCurrentUser().on("options.changed.PREFERENCES_ANNOTATION_SHOWSELECTEDONLY", _.bind(function (oldValue, newValue) {
				this.setShowSelectedAnnotationOnly(newValue > 0);
			}, this));

			// show only selected annotation popup
			f.getCurrentUser().on("options.changed.PREFERENCES_ANNOTATION_SHOWSELECTEDPOPUPONLY", _.bind(function (oldValue, newValue) {
				this.setShowSelectedAnnotationPopupOnly(newValue > 0);
			}, this));

			// show sequence label
			f.getCurrentUser().on("options.changed.PREFERENCES_ANNOTATION_SEQLABEL_VISIBLE", _.bind(function (oldValue, newValue) {
				this.setShowSequenceLabel(newValue > 0);
			}, this));

			// show char/line counter
			f.getCurrentUser().on("options.changed.PREFERENCES_ANNOTATION_SHOWCOUNT", _.bind(function (oldValue, newValue) {
				this.setShowCharLineCounter(newValue > 0);
			}, this));

			/**
			 * Make the options dropdown
			 */
			this.optionButton = new DropdownMiniButton(f, "annotationSettings", "icon-WCR_Settings-submenu");

			f.i18n.loadBundle("viewer", function () {

				// step through mode
				function toggleStepThrougMode(mode, toggle) {
					if (toggle) {
						this.setStepThroughMode(mode, true);
					}
				}
				this.optionButton.addOption({
					name: 'stepByThread',
					content: this.f.i18n.t("AnnSortThread"),
					onchange: _.bind(toggleStepThrougMode, this, AnnotationConstants.StepThroughMode.BY_THREAD),
					selected: this.f.getOption("PREFERENCES_ANNOTATION_NEXT_MODE") == AnnotationConstants.StepThroughMode.BY_THREAD,
					userCannotUncheck: true
				});

				this.optionButton.addOption({
					name: 'stepWithReplies',
					content: this.f.i18n.t("AnnSortDate"),
					onchange: _.bind(toggleStepThrougMode, this, AnnotationConstants.StepThroughMode.WITH_REPLIES),
					selected: this.f.getOption("PREFERENCES_ANNOTATION_NEXT_MODE") == AnnotationConstants.StepThroughMode.WITH_REPLIES,
					userCannotUncheck: true
				});

				this.optionButton.addOption({
					name: 'stepWithZoom',
					content: this.f.i18n.t("AnnSortThreadFit"),
					onchange: _.bind(toggleStepThrougMode, this, AnnotationConstants.StepThroughMode.WITH_ZOOM),
					selected: this.f.getOption("PREFERENCES_ANNOTATION_NEXT_MODE") == AnnotationConstants.StepThroughMode.WITH_ZOOM,
					userCannotUncheck: true
				});

				this.optionButton.addOption({
					name: 'stepWithLinked',
					content: this.f.i18n.t("AnnSortAreaFit"),
					onchange: _.bind(toggleStepThrougMode, this, AnnotationConstants.StepThroughMode.WITH_LINKED),
					selected: this.f.getOption("PREFERENCES_ANNOTATION_NEXT_MODE") == AnnotationConstants.StepThroughMode.WITH_LINKED,
					userCannotUncheck: true
				});



				this.optionButton.addDivider();


				// show selected annotation only
				this.optionButton.addOption({
					name: 'showselann',
					content: this.f.i18n.t("AnnListShowSel"),
					onchange: _.bind(function (toggle) {
						this.setShowSelectedAnnotationOnly(toggle, true);
					}, this),
					selected: this.f.getOption("PREFERENCES_ANNOTATION_SHOWSELECTEDONLY") > 0
				});

				// show selected annotation popup only
				this.optionButton.addOption({
					name: 'showselannpopup',
					content: this.f.i18n.t("AnnListShowSelPopup"),
					onchange: _.bind(function (toggle) {
						this.setShowSelectedAnnotationPopupOnly(toggle, true);
					}, this),
					selected: this.f.getOption("PREFERENCES_ANNOTATION_SHOWSELECTEDPOPUPONLY") > 0
				});

				// show sequence number
				this.optionButton.addOption({
					name: 'shownumber',
					content: this.f.i18n.t("AnnSeqLabelShow"),
					onchange: _.bind(function (toggle) {
						this.setShowSequenceLabel(toggle, true);
					}, this),
					selected: this.f.getOption("PREFERENCES_ANNOTATION_SEQLABEL_VISIBLE") > 0
				});

				// show char/line counter
				this.optionButton.addOption({
					name: 'showCharLineCount',
					content: this.f.i18n.t("AnnCountShow"),
					onchange: _.bind(function (toggle) {
						this.setShowCharLineCounter(toggle, true);
					}, this),
					selected: this.f.getOption("PREFERENCES_ANNOTATION_SHOWCOUNT") > 0
				});



				this.optionButton.addDivider();


				// sort by options
				function toggleSortByMode(mode, toggle) {
					if (toggle) {
						this.setSortByMode(mode, true);
					}
				}
				this.optionButton.addOption({
					name: 'sortbycreationdate',
					content: this.f.i18n.t("DateSortCreation"),
					onchange: _.bind(toggleSortByMode, this, AnnotationConstants.AnnotationSortType.SORT_BY_CREATION_DATE),
					selected: this.f.getOption("PREFERENCES_ANNOTATION_DATESORT_MODE") === AnnotationConstants.AnnotationSortType.SORT_BY_CREATION_DATE,
					userCannotUncheck: true
				});

				this.optionButton.addOption({
					name: 'sortbymodificationdate',
					content: this.f.i18n.t("DateSortModification"),
					onchange: _.bind(toggleSortByMode, this, AnnotationConstants.AnnotationSortType.SORT_BY_MODIFICATION_DATE),
					selected: this.f.getOption("PREFERENCES_ANNOTATION_DATESORT_MODE") === AnnotationConstants.AnnotationSortType.SORT_BY_MODIFICATION_DATE,
					userCannotUncheck: true
				});

				// make sure the show selected annotation popup only is disable when show selected annotation only is true
				this.setShowSelectedAnnotationOnly(this.f.getOption("PREFERENCES_ANNOTATION_SHOWSELECTEDONLY") > 0, false);
				this.updateWarningBadge();
			}, this);



			// Add callbacks for moving: find all rich text editors and re-init them
			this.on("movingStarted", _.bind(function () {
				if (this.context) {
					this.context.find("[id^=richText_]").each(function () {
						var richTextObj = $(this).data("richTextObj");
						// Only destroy the already initialized ones
						if (richTextObj && richTextObj.isInitialized()) {
							richTextObj.destroy();
							richTextObj.isDestroyed = true;
						}
					});
				}
			}, this));

			this.on("movingStopped", _.bind(function () {
				if (this.context) {
					this.context.find("[id^=richText_]").each(function () {
						var richTextObj = $(this).data("richTextObj");
						// Only re-init the ones we destroyed ourselves.
						if (richTextObj && richTextObj.isDestroyed) {
							_.defer(function () {
								richTextObj.init();
							});
						}
					});
				}
			}, this));



			/**
			 * Widget visibility
			 */
			this.on("visible", function (isVisible) {
				annotationModel.setAnnotationListVisible(isVisible);
			});
			annotationModel.setAnnotationListVisible(this.isVisible());


			annotationModel.on("selectedAnnotation", _.bind(this.selectAnnotation, this));
			annotationModel.on("unselectedAnnotation", _.bind(this.deselectAnnotation, this));
			annotationModel.on("annotationAdded", _.bind(function (annotation, hidden) {
				if (!hidden) {
					this.addAnnotation(annotation, true);
				}
			}, this));
			annotationModel.on("annotationUpdated", _.bind(this.updateAnnotation, this));
			annotationModel.on("annotationDeleted", _.bind(this.deleteAnnotation, this));


			/**
			 * Render the complete widget
			 */
			var renderWidget = _.bind(function () {
				this._updateButtons();
				this.renderCompleteWidget();
			}, this);

			/**
			 * Update both save and step through buttons
			 */
			var updateButtons = _.bind(function () {
				this._updateButtons();
			}, this);

			/**
			 * The widget gets initialized with its context
			 */
			this._override("setContext", function (context, headerExtra) {
				// call the old implementation of activate() (the implementation of AbstractWidget.js)
				this._super(context, headerExtra);

				//render the widget
				this.renderCompleteWidget();
				this.context.append(this.detailsDiv);

				// add buttons to the widget header
				this.headerExtra.append(this.saveButton.element).append(this.stepThroughButton.element).append(this.reloadButton.element).append(this.optionButton.element);
				this._updateButtons();


				// start listening to the model
				this.annotationModel.on("filterChanged", renderWidget);
				this.annotationModel.on("modelContentChanged", renderWidget);
				this.annotationModel.on("modelChanged", updateButtons);
				this.annotationModel.on("changed.page", _.bind(function () {
					// when a page changes, make sure to remove all DOM elements + clear the cache of AnnotationRow objects
					var toBeRemovedFromDOM = _.keys(this.inDOM);
					_.each(toBeRemovedFromDOM, function (name) {
						this._removeFromDOM(name);
					}, this);
					this.annotationRows = {};
				}, this));
				// heart beat for the timers (note PJPR, would be nice if this isn't managed by the widget, but by the autotimelabels themselves)
				this.startHeartBeat();

				// adjust the layout of the widget
				this.resized();

			});

			//Dectivation of the widget
			this._override("destroy", function () {
				// call the old implementation of activate() (the implementation of AbstractWidget.js)
				this._super();

				annotationModel.off("filterChanged", renderWidget);
				annotationModel.off("modelContentChanged", renderWidget);
				annotationModel.off("modelChanged", updateButtons);
				this.stopHeartBeat();
			});
		};

		AnnotationWidget.prototype = Object.create(AbstractWidget.prototype);
		AnnotationWidget.prototype.constructor = AnnotationWidget;
		AnnotationWidget.widgetName = LayoutConstants.Widget.ANNOTATION;



		/**
		 * Get the minimum size of the annotation details widget
		 * @override
		 * @returns {ShapeUtils.Size}
		 */
		AnnotationWidget.prototype.getMinimumSize = function () {
			return new ShapeUtils.Size(250, 150);
		};

		/**
		 * The annotation widget is greedy, and wants to have all available extra space
		 * @override
		 * @returns {Boolean} true
		 */
		AnnotationWidget.prototype.willFill = function () {
			return true;
		};

		/**
		 * Updates the save and step through buttons
		 */
		AnnotationWidget.prototype._updateButtons = function () {
			this.updateSaveButton();
			this.updateStepThroughButton(true);
		};

		/**
		 * Update the save button display (disable/enable the button, change badge, change tooltip
		 * @private
		 */
		AnnotationWidget.prototype.updateSaveButton = function () {
			if (this.saveButton) {
				if (this.disableSaveButton) {
					// the button is disabled
					this.saveButton.setEnabled(false);
				} else {
					// get the amount of unsaved annotations
					var unsavedCount = this.annotationModel.getUnsavedAnnotationCount();
					if (unsavedCount > 0) {
						this.saveButton.setEnabled(true);
						this.saveButton.setBadge(unsavedCount);
						if (this.annotationModel.getUnsavedAnnotations().length > 0) {
							if (this.annotationModel.getDeletedAnnotations().length > 0) {
								this.saveButton.setTooltip("viewer", "SaveMiniBtnTT", [this.annotationModel.getUnsavedAnnotations().length, this.annotationModel.getDeletedAnnotations().length]);
							} else {
								this.saveButton.setTooltip("viewer", "SaveMiniBtnAnnTT", [this.annotationModel.getUnsavedAnnotations().length]);
							}
						} else {
							this.saveButton.setTooltip("viewer", "SaveMiniBtnDelTT", [this.annotationModel.getDeletedAnnotations().length]);
						}
					} else {
						this.saveButton.setEnabled(false);
					}
				}
			}
		};

		/**
		 * Show or hide the warning badge, update the tool tip
		 */
		AnnotationWidget.prototype.updateWarningBadge = function () {
			var showSelAnn = this.optionButton.getOptionValue("showselann");
			var showSelAnnPopup = this.optionButton.getOptionValue("showselannpopup");
			if (showSelAnn || showSelAnnPopup) {
				// show the warning badge
				this.optionButton.setWarning();
				this.f.i18n.loadBundle("viewer", function () {
					// build up the tooltip text
					var tooltip = this.f.i18n.t("AnnWidgetOptionsTT") + ".";
					tooltip += "<br />" + this.f.i18n.t("Warning");
					if (showSelAnn) {
						tooltip += "  " + this.f.i18n.t("OnlySelectedAnnotationWarningTT");
					}
					if (showSelAnnPopup && this.optionButton.isActionEnabled("showselannpopup")) {
						if (showSelAnn) {
							// should not happen
							tooltip += "<br />";
						}
						tooltip += "  " + this.f.i18n.t("OnlySelectedAnnotationPopupWarningTT");
					}
					this.optionButton.setTooltip(tooltip);
				}, this);
			} else {
				// hide warning badge and restore tooltip
				this.optionButton.removeWarning();
				this.optionButton.setTooltip("viewer", "AnnWidgetOptionsTT");
			}
		};

		/**
		 * update the step through button, makes sure the button in disabled when there are no annotations
		 * @param {Boolean} enable - should the button be enabled when there are annotations
		 */
		AnnotationWidget.prototype.updateStepThroughButton = function (enable) {
			if (_.size(this.annotationModel.getFilteredAnnotations()) === 0 || !enable) {
				this.stepThroughButton.setEnabled(false);
			} else {
				this.stepThroughButton.setEnabled(true);
			}
		};

		/**
		 * Set the sort mode
		 * @param {Number} sorting mode - toggle the option
		 * @param {Boolean} [persist] - true if option needs to be persisted
		 */
		AnnotationWidget.prototype.setSortByMode = function (sortMode, persist) {
			if (persist) {
				this.f.setOption("PREFERENCES_ANNOTATION_DATESORT_MODE", sortMode);
			}

			this.optionButton.toggleOption("sortbycreationdate", sortMode == AnnotationConstants.AnnotationSortType.SORT_BY_CREATION_DATE);
			this.optionButton.toggleOption("sortbymodificationdate", sortMode == AnnotationConstants.AnnotationSortType.SORT_BY_MODIFICATION_DATE);
		};

		/**
		 * Enable the option to show only the selected annotation
		 * @param {Boolean} toggle - toggle the option
		 * @param {Boolean} [persist] - true if option needs to be persisted
		 */
		AnnotationWidget.prototype.setShowSelectedAnnotationOnly = function (toggle, persist) {
			if (persist) {
				this.f.setOption("PREFERENCES_ANNOTATION_SHOWSELECTEDONLY", toggle ? 1 : 0);
			}

			// if "show only selected annotation" is enabled, disable the "show only selected annotation popup" option
			if (toggle) {
				this.optionButton.toggleOption("showselannpopup", false);
				this.f.setOption("PREFERENCES_ANNOTATION_SHOWSELECTEDPOPUPONLY", 0);
			}

			// display warning if needed
			this.updateWarningBadge();
		};

		/**
		 * Enable the option to show only the selected annotation popup
		 * @param {Boolean} toggle - toggle the option
		 * @param {Boolean} [persist] - true if option needs to be persisted
		 */
		AnnotationWidget.prototype.setShowSelectedAnnotationPopupOnly = function (toggle, persist) {
			if (persist) {
				this.f.setOption("PREFERENCES_ANNOTATION_SHOWSELECTEDPOPUPONLY", toggle ? 1 : 0);
			}

			// if "show selected annotation popup only" is enabled, disable "show only selected annotation"
			if (toggle) {
				this.optionButton.toggleOption("showselann", false);
				this.f.setOption("PREFERENCES_ANNOTATION_SHOWSELECTEDONLY", 0);
			}


			// display warning if needed
			this.updateWarningBadge();
		};

		/**
		 * Set the step through mode
		 * @param {Number} mode - set AnnotationConstants.StepTroughModemode
		 * @param {Boolean} [persist] -  true if option needs to be persisted
		 */
		AnnotationWidget.prototype.setStepThroughMode = function (mode, persist) {
			if (persist) {
				this.f.setOption("PREFERENCES_ANNOTATION_NEXT_MODE", mode);
			}

			// update toggles
			this.optionButton.toggleOption("stepWithReplies", mode == AnnotationConstants.StepThroughMode.WITH_REPLIES);
			this.optionButton.toggleOption("stepByThread", mode == AnnotationConstants.StepThroughMode.BY_THREAD);
			this.optionButton.toggleOption("stepWithZoom", mode == AnnotationConstants.StepThroughMode.WITH_ZOOM);
			this.optionButton.toggleOption("stepWithLinked", mode == AnnotationConstants.StepThroughMode.WITH_LINKED);
		};

		/**
		 * Enable the option to show the sequence label
		 * @param {Boolean} toggle - toggle the option
		 * @param {Boolean} [persist] - true if option needs to be persisted
		 */
		AnnotationWidget.prototype.setShowSequenceLabel = function (toggle, persist) {
			if (persist) {
				this.f.setOption("PREFERENCES_ANNOTATION_SEQLABEL_VISIBLE", toggle ? 1 : 0);
			}

			this.optionButton.toggleOption("shownumber", toggle);
		};

		/**
		 * Enable the option to show the char/line counter
		 * @param {Boolean} toggle - toggle the option
		 * @param {Boolean} [persist] - true if option needs to be persisted
		 */
		AnnotationWidget.prototype.setShowCharLineCounter = function (toggle, persist) {
			if (persist) {
				this.f.setOption("PREFERENCES_ANNOTATION_SHOWCOUNT", toggle ? 1 : 0);
			}

			this.optionButton.toggleOption("showCharLineCount", toggle);

			_.each(this.annotationRows, function (annotationRow) {
				annotationRow.showHideCharLineCount(toggle, true);
			});
		};

		/**
		 * Render all the interface element of the widget
		 */
		AnnotationWidget.prototype.renderCompleteWidget = function () {
			if (!this._isDirty) {
				this._isDirty = true;
				this.f.i18n.loadBundle("viewer", function () {
					// defer makes the priority of the full redraw lower (all current events on the stack are first processed before redraw begins
					_.defer(_.bind(function () {
						this._isDirty = false;
						this.renderHeader();
						this.renderAnnotationList();
					}, this));
				}, this);
			}
		};

		/**
		 * render the annotation details header
		 */
		AnnotationWidget.prototype.renderHeader = function () {
			this._annotationFilters.empty();
			var annotationsOrigins = this.annotationModel.getAnnotationOrigins();
			if (annotationsOrigins.length > 1 || (annotationsOrigins.length === 1 && annotationsOrigins[0] !== AnnotationConstants.AnnotationOriginTypes.user.id)) {
				this._annotationOriginSelector.element.show();
			} else {
				this._annotationOriginSelector.element.hide();
			}

			/**
			 * User selector
			 */
			this._usercombo = $("<select>");
			var all = $("<option/>").text(this.f.i18n.t("AnnAllUsers")).val("All");
			var customEnabled = $("<option selected disabled hidden value=''/>").text(this.f.i18n.t("AnnotFilterCustomEnabled"));

			if (this.annotationModel.getAnnotationFilterModel().isCustomFilterActive()) {
				this._usercombo.append(customEnabled);
			}

			this._usercombo.append(all);

			var users = this.annotationModel.getUsers(AnnotationConstants.AnnotationOriginTypes.user.id);
			_.each(users, function (loc, name) {
				var option = $("<option/>").text(loc).val(name);
				if (selectedUser == name) {}
				this._usercombo.append(option);
			}, this);

			var selectedUser = this.annotationModel.getAnnotationFilterModel().getSelectedUser();
			if (selectedUser && !this.annotationModel.getAnnotationFilterModel().isCustomFilterActive()) {
				var hasSelectedUser = this._usercombo.find("option").filter(function(i, elem) {
					return $(elem).attr("value") === selectedUser;																
				}).length > 0;
				if (hasSelectedUser) {
					this._usercombo.val(selectedUser);
				} else {
					var newUserOption = $("<option hidden selected/>").text(this.f.i18n.t("AnnotFilterUserNotAvailable"));
					this._usercombo.append(newUserOption);
				}
			}

			this._annotationFilters.append(this._usercombo);

			this._usercombo.on("change", _.bind(function (e) {
				if (e.target.value != "All") {
					this.annotationModel.getAnnotationFilterModel().setSelectedUser(e.target.value);
				} else {
					this.annotationModel.getAnnotationFilterModel().setSelectedUser(null);
				}
			}, this));

			/**
			 * State selector
			 */
			this._statecombo = $("<select>");
			var stateall = $("<option/>").text(this.f.i18n.t("AnnAllReviewState")).val("All");
			var customEnabledState = $("<option selected disabled hidden value=''/>").text(this.f.i18n.t("AnnotFilterCustomEnabled"));

			if (this.annotationModel.getAnnotationFilterModel().isCustomFilterActive()) {
				this._statecombo.append(customEnabledState);
			}

			this._statecombo.append(stateall);

			var sep = "------------------------------";
			this._statecombo.append($("<option>").text(sep).attr("disabled", "disabled"));

			var notprefix = "NOT_";

			var option = $("<option/>").text(this.f.i18n.t("AnnStatusAny")).val("Any");
			this._statecombo.append(option);
			var notoption = $("<option/>").text(this.f.i18n.t("AnnNoReviewState")).val("NOT_Any");
			this._statecombo.append(notoption);

			this._statecombo.append($("<option>").text(sep).attr("disabled", "disabled"));
			_.each(AnnotationConstants.AnnotationReviewState, function (state, key) {
				var option = $("<option/>").text(this.f.i18n.t(AnnotationConstants.AnnotationReviewStateLocs[key])).val(AnnotationConstants.AnnotationReviewState[key]);
				this._statecombo.append(option);
			}, this);

			this._statecombo.append($("<option>").text(sep).attr("disabled", "disabled"));
			_.each(AnnotationConstants.AnnotationReviewState, function (state, key) {
				var notoption = $("<option/>").text(this.f.i18n.t("AllExcept", this.f.i18n.t(AnnotationConstants.AnnotationReviewStateLocs[key]))).val(notprefix + AnnotationConstants.AnnotationReviewState[key]);
				this._statecombo.append(notoption);
			}, this);


			this._annotationFilters.append(this._statecombo);
			this._statecombo.on("change", _.bind(function (e) {
				if (e.target.value != "All") {
					var inverse = e.target.value.indexOf(notprefix) > -1;
					var status = e.target.value;
					if (inverse) {
						status = status.substring(notprefix.length);
					}
					this.annotationModel.getAnnotationFilterModel().setSelectedState(status, inverse);
				} else {
					this.annotationModel.getAnnotationFilterModel().setSelectedState(null, false);
				}
				return true;
			}, this));

			var selectedState = this.annotationModel.getAnnotationFilterModel().getSelectedState();
			if (selectedState && !this.annotationModel.getAnnotationFilterModel().isCustomFilterActive()) {
				this._statecombo.val((this.annotationModel.getAnnotationFilterModel().isInverseState() ? notprefix : "") + selectedState);
			}


			/**
			 * The custom filter popup
			 */
			if (this._customFilterPopup && this._customFilterPopup.isVisible()) {
				this._customFilterPopup.render();
			}

			this._updateNumberOfSelectedAnnotations();
		};

		/**
		 * Update the count of currently visbile and total number of annotations.
		 * @private
		 */
		AnnotationWidget.prototype._updateNumberOfSelectedAnnotations = function () {
			this.f.i18n.loadBundle("viewer", function () {
				var selectedThreads = this.f.i18n.t("SelectedThreadsCount", this.annotationModel.getAnnotationFilterModel().getNumberOfFilteredAnnotations(), this.annotationModel.getAnnotationCount());
				this._numberOfSelectedThreads.text(selectedThreads);
			}, this);
		};

		/**
		 * Render/update the complete annotation list
		 */
		AnnotationWidget.prototype.renderAnnotationList = function () {
			var filteredAnnots = this.annotationModel.getFilteredAnnotations();

			// mark all elements as elements that probably should be removed
			this.toBeRemovedFromDOM = _.clone(this.inDOM);

			// readd every annotation row from the filtered annotation list
			// if a row gets added to the DOM it is removed from the "toBeRemovedFromDOM" list
			_.each(filteredAnnots, function (annotation) {
				if (annotation.linkedto && annotation.linkedto.length > 0) {
					//do not show linked annotations in the table
					return;
				} else if (this.annotationRows[annotation.name]) {
					// this annotation row already exists, readd to DOM
					var row = this.annotationRows[annotation.name];
					row.element.next("hr").remove();
					row.setAnnotation(annotation);
					this._addToDom(row);
					row.element.show();
				} else {
					// there is no annotationRow for this annotation, create a new one
					this.addAnnotation(annotation);
				}
			}, this);

			// remove all unneeded rows
			_.each(this.toBeRemovedFromDOM, function (value, name) {
				this._removeFromDOM(name);
			}, this);

			this.toBeRemovedFromDOM = {};
			this._fireResized();
		};

		/**
		 * Add an annotation row to the list
		 * @private
		 * @param {AnnotationRow} annotationRow
		 */
		AnnotationWidget.prototype._addToDom = function (annotationRow) {
			this.inDOM[annotationRow.name] = 1;
			delete this.toBeRemovedFromDOM[annotationRow.name];

			annotationRow.onDetachFromDOM();

			this.annotationList.append(annotationRow.element).append($("<hr/>"));
			annotationRow.onReattachToDOM();
		};

		/**
		 * Remove an annotation row from the DOM
		 * @private
		 * @param {String} name - the name of the annotation row
		 */
		AnnotationWidget.prototype._removeFromDOM = function (name) {
			if (this.inDOM[name]) {
				var row = this.annotationRows[name];
				if (row) {
					row.element.next("hr").remove();
					row.onDetachFromDOM();
					row.element.detach();
				}
				delete this.toBeRemovedFromDOM[name];
				delete this.inDOM[name];
			}
		};

		/**
		 * Create an annotationRow for a given Annotation, the annotation row is added to the DOM
		 * @param {Annotation} annotation - the annotation that should be added
		 * @param {Boolean} [fireResized] - if true, a resized event will be fired from the widget
		 */
		AnnotationWidget.prototype.addAnnotation = function (annotation, fireResized) {
			if (annotation.linkedto && annotation.linkedto.length > 0) {
				return; //do not show linked annotations in the table
			}

			var row = new AnnotationRow(this.f, annotation, this.annotationModel, undefined, undefined, this._isForMobile);
			this.annotationRows[annotation.name] = row;
			var annotationEl = row.element;

			this._addToDom(row);

			annotationEl.on("click", _.bind(function (e) {
				this.annotationModel.selectAnnotation(row.getAnnotation(), true);
				e.stopPropagation();
			}, this));

			var viewpressed = false;
			var viewactive = false;
			annotationEl.on('mousedown', _.bind(function () {
				viewpressed = true;
				setTimeout(_.bind(function () {
					if (viewpressed) {
						viewactive = true;
						this.f.emit("module.annotation.highlightAnnotation", row.getAnnotation());
					}
				}, this), 500);
			}, this));

			annotationEl.on('mouseup', _.bind(function () {
				if (viewpressed && viewactive) {
					this.f.emit("toolManager.deactivateTool", "annotationindicatortool");
				}
				viewpressed = false;
				viewactive = false;
			}, this));

			annotationEl.on('mouseout', _.bind(function () {
				if (viewpressed && viewactive) {
					this.f.emit("toolManager.deactivateTool", "annotationindicatortool");
				}
				viewpressed = false;
				viewactive = false;
			}, this));

			annotationEl.on('mouseleave', _.bind(function () {
				if (viewpressed && viewactive) {
					this.f.emit("toolManager.deactivateTool", "annotationindicatortool");
				}
				viewpressed = false;
				viewactive = false;
			}, this));


			if (this.annotationModel.isAnnotationOrReplySelected(annotation)) {
				this.selectAnnotation(annotation);
			}

			this._updateNumberOfSelectedAnnotations();
			if (fireResized) {
				this._fireResized();
			}
		};

		/**
		 * Select an annotation
		 * @param {Annotation} annotation - the annotation that is selected
		 */
		AnnotationWidget.prototype.selectAnnotation = function (annotation) {
			if (this.annotationRows[annotation.inreplyto]) {
				this.annotationRows[annotation.inreplyto].toggleReplyLimit(false);
			}

			if (this.annotationRows[annotation.name] || this.annotationRows[annotation.inreplyto]) {
				var annotationRow = this.annotationRows[annotation.name] || this.annotationRows[annotation.inreplyto].annotationRows[annotation.name];
				annotationRow.setSelected(true);

				var element = annotationRow.element;
				// assure it is visible in the scrollable pane
				element.trigger("makeMeVisible");
			}
		};

		/**
		 * Deselect an annotation
		 * @param {Annotation} annotation - the annotation that is deselected
		 */
		AnnotationWidget.prototype.deselectAnnotation = function (annotation) {
			var annotationRow;

			if (this.annotationRows[annotation.name]) {
				annotationRow = this.annotationRows[annotation.name];
			}

			if (this.annotationRows[annotation.inreplyto]) {
				this.annotationRows[annotation.inreplyto].toggleReplyLimit();
				if (annotationRow === undefined) {
					annotationRow = this.annotationRows[annotation.inreplyto].annotationRows[annotation.name];
				}
			}

			if (annotationRow) {
				annotationRow.setSelected(false);
			}
		};

		/**
		 * Update an annotation
		 * @param {Annotation} annotation
		 * @param {Annotation} [referenceAnnotation]
		 */
		AnnotationWidget.prototype.updateAnnotation = function (annotation, referenceAnnotation) {
			if (this.annotationRows[annotation.name]) {
				this.annotationRows[annotation.name].annotationUpdated(referenceAnnotation);
			} else if (annotation.linkedto && !annotation.parent && this.annotationRows[annotation.linkedto]) {
				this.annotationRows[annotation.linkedto].annotationUpdated();
			} else {
				var annot = annotation.getGodfather();
				if (annot !== annotation) {
					this.updateAnnotation(annot, annotation);
				}
			}
		};

		/**
		 * Delete an annotation
		 * @param {Annotation} annotation - Annotation to delete
		 */
		AnnotationWidget.prototype.deleteAnnotation = function (annotation) {
			if (this.annotationRows[annotation.name]) {
				this._removeFromDOM(annotation.name);
				delete this.annotationRows[annotation.name];
			} else if (annotation.linkedto && !annotation.parent && this.annotationRows[annotation.linkedto]) {
				this._removeFromDOM(annotation.linkedto);
				delete this.annotationRows[annotation.linkedto];
			} else {
				annotation = annotation.getGodfather();
				this.updateAnnotation(annotation);
			}

			this._updateNumberOfSelectedAnnotations();
			this._fireResized();
		};

		/**
		 * Gets called when the widget was resized,
		 * Update the height of the annotationsListWrapper
		 * @override
		 * @private
		 */
		AnnotationWidget.prototype.resized = function () {
			this.annotationsListWrapper.css("height", "");
			if (this.isContextAvailable() && this.detailsDiv.css("display") !== "flex") {
				_.defer(_.bind(function () {
					this.context.css("overflow", "hidden");
					this.annotationsListWrapper.css("height", "auto");

					var height = this.context.outerHeight(true);
					if (!this._isForMobile) {
						height -= this.header.outerHeight(true);
					}
					if (this.warningMessage.is(":visible")) {
						height -= this.warningMessage.outerHeight(true);
					}
					if (this.temporarilyDisabledDiv.is(":visible")) {
						height -= this.temporarilyDisabledDiv.outerHeight(true);
					}
					if (this._annotationOriginSelector.element.is(":visible")) {
						height -= this._annotationOriginSelector.element.outerHeight(true);
					}
					if (height > 0) {
						this.annotationsListWrapper.css("height", height);
					}

					if (this._visibleItem) {
						this._visibleItem.trigger("makeMeVisible");
					}
					this.context.css("overflow", "auto");
				}, this));
			}
		};

		/**
		 * Get the preferred size
		 * @override
		 */
		AnnotationWidget.prototype.getPreferredSize = function () {
			return new ShapeUtils.Size(this.annotationsListWrapper.outerWidth(true), this.annotationsListWrapper.outerHeight(true) + this.nonScrollablePart.outerHeight(true));
		};

		/**
		 * Start the heartbeat for the auto time labels
		 */
		AnnotationWidget.prototype.startHeartBeat = function () {
			this.canRunTimer = true;
			this.updateTimers();
		};

		/**
		 * Stop the heartbeat for the auto time labels
		 */
		AnnotationWidget.prototype.stopHeartBeat = function () {
			this.canRunTimer = false;
		};

		/**
		 * Update the filter pinned icon
		 */
		AnnotationWidget.prototype._updateFilterPinnedIcon = function () {
			if (this.annotationModel.getAnnotationFilterModel().isAnnotationFilterPinned()) {
				this._pinFilterButton.setIcon("pinned icon-WCR_Pin12");
				this._pinFilterButton.setTooltip("viewer", "AnnotFilterPinOn");
			} else {
				this._pinFilterButton.setIcon("unpinned icon-WCR_Pin12");
				this._pinFilterButton.setTooltip("viewer", "AnnotFilterPinOff");
			}
		};

		/**
		 * Update all autotimelabels
		 */
		AnnotationWidget.prototype.updateTimers = function () {
			if (!this.canRunTimer) {
				return;
			}

			var timer = 30000;
			_.each(this.annotationRows, function (row) {
				row.updateTimers();
			});

			setTimeout(_.bind(function () {
				this.updateTimers();
			}, this), timer);
		};

		/**
		 * Update the visible of the annotation list + header
		 * and show messages or toggling buttons for when in compare mode
		 */
		AnnotationWidget.prototype.updateListVisibility = function () {
			if (this.annotationModel.isTemporarilyDisabled()) {
				// temporarily disabled, show toggle buttons if possible
				// hide list if no annotations are shown, and show a message
				this.temporarilyDisabledDiv.toggle(this.annotationModel.isTemporarilyAnnotationDocumentTogglingEnabled());

				this.temporaryAnnotationToggler.toggle(this.annotationModel.isTemporarilyAnnotationDocumentTogglingEnabled());
				this.showCurrentFileAnnotationsButton.setActive(this.annotationModel.isTemporarilyCurrentPageAnnotationsShown());
				this.showDifferenceFileAnnotationsButton.setActive(this.annotationModel.isTemporarilyDifferencePageAnnotationsShown());


				if (this.annotationModel.isTemporarilyDisabledAndNoAnnotationsVisible()) {
					//no annotations are visible, show a message is one is provided
					this.header.hide();
					this.annotationsListWrapper.hide();
					this.updateStepThroughButton(false);
					this.reloadButton.setEnabled(false);
				} else {
					//there are some annotations, we don't need to show a message
					this.header.show();
					this.annotationsListWrapper.show();
					this.updateStepThroughButton(true);
					this.reloadButton.setEnabled(true);
				}
			} else {
				// module is not disabled, so show all things and hide the temporarilydisabled div
				this.header.show();
				this.annotationsListWrapper.show();
				this.temporarilyDisabledDiv.hide();
				this.updateStepThroughButton(true);
				this.reloadButton.setEnabled(true);
			}

			this.resized();
		};

		return AnnotationWidget;
	});

define('web-viewer/annotation/GVAnnotationConstants',{
	GVTypes: {
		barcode: {
			contentProperties: {
				"Decoded Value:": {
					langKey: "BarcodeValue"
				},
				"Decoded ValueLeft/Bottom:": {
					langKey: "BarcodeValueLeftBottom"
				}, // current formatting
				"Decoded Value Left/Bottom:": {
					langKey: "BarcodeValueLeftBottom"
				},
				"Decoded ValueRight/Top:": {
					langKey: "BarcodeValueRightTop"
				}, // current formatting
				"Decoded Value Right/Top:": {
					langKey: "BarcodeValueRightTop"
				},
				"Type:": {
					langKey: "BarcodeType"
				},
				"Grade:": {
					langKey: "BarcodeGrade"
				},
				"Magnification:": {
					langKey: "BarcodeMagnification"
				},
				"BWR:": {
					langKey: "BarcodeBWR"
				}
			}
		},
		braille: {
			contentProperties: {
				"Language:": {
					langKey: "BrailleSelLang"
				},
				"Translation:": {
					langKey: "BrailleTranslation",
					stop: "EOF"
				}
			}
		},
		text: {
			contentProperties: {
				"Type:": {
					langKey: "TextCompareType"
				},
				"Master:": {
					langKey: "TextCompareMaster"
				},
				"Sample:": {
					langKey: "TextCompareSample"
				}
			},
			valueProperties: {
				"Bold": "TextCompareTypeBold",
				"Case": "TextCompareTypeCase",
				"Change": "TextCompareTypeChange",
				"Color": "TextCompareTypeColor",
				"Deletion": "TextCompareTypeDeletion",
				"Size": "TextCompareTypeSize",
				"Space": "TextCompareTypeSpace",
				"Sub/Super": "TextCompareTypeSubSuper",
				"Font Type": "TextCompareTypeFontType",
				"Hyphen": "TextCompareTypeHyphen",
				"Insertion": "TextCompareTypeInsertion",
				"Italic": "TextCompareTypeItalic",
				"Location": "TextCompareTypeLocation",
				"Underline": "TextCompareTypeUnderline"
			}
		}
	}

});

define('tpl!web-viewer/annotation/AnnotationGV', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<p>';
 _.each(d.lines, function(line){ 
__p+='\r\n	'+
((__t=( line.key ))==null?'':_.escape(__t))+
' '+
((__t=( line.value ))==null?'':_.escape(__t))+
'<br/>\r\n';
}) 
__p+='</p>';
return __p;
}});
define('web-viewer/annotation/AnnotationGV',["web-viewer/annotation/Annotation",
	"web-viewer/annotation/AnnotationConstants",
	"web-viewer/annotation/GVAnnotationConstants",
	"utils/XMLUtils",
	"tpl!./AnnotationGV",
	"jquery"
], function (Annotation,
	AnnotationConstants,
	GVAnnotationConstants,
	XMLUtils,
	AnnotationGVTemplate,
	$) {

	"use strict";

	/**
	 * @class
	 * @augments Annotation
	 * @exports AnnotationGV
	 * An annotation created by AE's GV nodes, these are read_only
	 * @param {Core.ViewerFacade}
	 * @param {String} originType - origin of the annotation, in this case "braille", "barcode", "text", "spelling" and "artwork" are correct.
	 */
	var AnnotationGV = function (f, originType) {
		Annotation.call(this, f, originType);

		/**
		 * @member{String} a wrapper containing images if there are any
		 */
		this.imageWrapper = null;
		/**
		 * @member{String} the tooltip for the image wrapper if any
		 */
		this.imageTooltip = null;
	};

	AnnotationGV.prototype = Object.create(Annotation.prototype);
	AnnotationGV.prototype.constructor = AnnotationGV;

	/**
	 * @override
	 */
	AnnotationGV.prototype._parseContentsFromXML = function (xml) {
		var contentEls = xml.getElementsByTagName('contents-richtext');

		if (contentEls.length === 0) {
			contentEls = xml.getElementsByTagName('contents');
		}


		// Content
		var dataToRender = [];
		if (contentEls.length > 0) {
			var el = contentEls[0].getElementsByTagName("body")[0];
			if (!el) {
				el = contentEls[0];
			}

			// The contents of the file is one big string. 
			// To parse it we go looking for certain Keys and create a substring up until the next newLine or up until a certain stop character
			var text = el.textContent;
			if (GVAnnotationConstants.GVTypes[this.originType] && GVAnnotationConstants.GVTypes[this.originType]["contentProperties"]) {

				// GVAnnotationConstants has defined the keys to display

				_.each(GVAnnotationConstants.GVTypes[this.originType]["contentProperties"], function (obj, key) {

					var begin = text.indexOf(key);

					if (begin === -1) {
						return;
					}
					var substr = text.substr(begin + key.length);

					var substrLength;

					// Check how far the substring should go, if no 'stop' property was defined, go to the first newline character
					if (!obj.stop) {
						substrLength = substr.indexOf("\n");
						if (substrLength === -1) {
							substrLength = undefined;
						}

					}
					// Else just keep the rest of the string


					var value = substr.substr(0, substrLength).trim();

					// Check for I18n of the value  
					if (GVAnnotationConstants.GVTypes[this.originType]["valueProperties"] &&
						GVAnnotationConstants.GVTypes[this.originType]["valueProperties"][value]) {
						value = this.f.i18n.t(GVAnnotationConstants.GVTypes[this.originType]["valueProperties"][value]);
					}

					dataToRender.push({
						key: this.f.i18n.t(obj.langKey),
						value: value
					});


				}, this);
				this.content = AnnotationGVTemplate({
					lines: dataToRender
				});
			} else {
				this.content = "<p>" + el.textContent + "</p>";
			}

		}

		// Images
		var tMaster = XMLUtils.attributeValue(xml, 'tMaster'); // returns null when not present
		var tSample = XMLUtils.attributeValue(xml, 'tSample');
		if (tMaster !== null && tSample !== null) {

			var masterImage = "<img class='masterImage' src='data:image/png;base64," + tMaster + "' alt='Master'/>";
			var sampleImage = "<img class='sampleImage' src='data:image/png;base64," + tSample + "' alt='Sample'/>";

			this.imageWrapper = "<div class='imageWrapper'>" + masterImage + sampleImage + "</div>";

			// if both present, use image and add tooltip
			this.content = "";

			var tooltip = "";
			_.each(dataToRender, function (data, index) {
				if (index > 0) {
					tooltip += "<br />";
				}
				tooltip += data.key + " " + data.value;
			});
			this.imageTooltip = tooltip;
		} else {
			this.imageWrapper = null;
			this.imageTooltip = null;
		}

		// Set the title
		this.lastname = this.f.i18n.t(this.originType + "Title");
	};

	/**
	 * @override
	 */
	AnnotationGV.prototype.initWithXML = function (xml, boxOffsets, msgnr) {
		Annotation.prototype.initWithXML.call(this, xml, boxOffsets, msgnr);
		this.messagenumber = msgnr;
	};

	/**
	 * @override
	 */
	AnnotationGV.prototype._modifyAnnotatationCoords = function (x1, y2, x2, y1, boxOffsets) {
		// GV Annotations are relative to the mediabox
		// y offset doesn't get a minus because the world origin is top left, while the trim and mediaboxes origin is bottom left
		var xOffset = -(boxOffsets.trimboxOffset.width - boxOffsets.mediaboxOffest.width);
		var yOffset = boxOffsets.trimboxOffset.height - boxOffsets.mediaboxOffest.height;
		return [x1 + xOffset, y2 + yOffset, x2 + xOffset, y1 + yOffset];
	};



	/**
	 * @override
	 */
	AnnotationGV.prototype.getPermission = function () {
		return AnnotationConstants.AccessRight.READ_ONLY;
	};

	/**
	 * override
	 */
	AnnotationGV.prototype.getPermissionFor = function (action) {
		switch (action) {
		case AnnotationConstants.ActionPermissions.DUPLICATE:
				return this.originType !== AnnotationConstants.AnnotationOriginTypes.artwork.id;
		case AnnotationConstants.ActionPermissions.DUPLICATE_WITHOUT_TEXT:
			return true;
		case AnnotationConstants.ActionPermissions.CAN_BE_REPLIED_TO:
			return false;
		}
	};



	/**
	 * @override
	 */
	AnnotationGV.prototype.getExtraContent = function () {
		if (this.imageWrapper) {
			return {
				element: $(this.imageWrapper),
				tooltip: this.imageTooltip
			};
		}
		return null;
	};

	/**
	 * @override
	 */
	AnnotationGV.prototype.getTextContent = function () {
		if (this.imageTooltip) {
			return this.imageTooltip;
		} else {
			return Annotation.prototype.getTextContent.call(this);
		}
	};

	/**
	 * @override
	 */
	AnnotationGV.prototype.getIconClass = function () {
		if (this.originType === AnnotationConstants.AnnotationOriginTypes.braille.id ||
			this.originType === AnnotationConstants.AnnotationOriginTypes.barcode.id ||
			this.originType === AnnotationConstants.AnnotationOriginTypes.text.id ||
			this.originType === AnnotationConstants.AnnotationOriginTypes.artwork.id ||
			this.originType === AnnotationConstants.AnnotationOriginTypes.spelling.id) {
			return AnnotationConstants.AnnotationOriginTypes[this.originType].subIcon;
		} else {
			return Annotation.prototype.getIconClass.call(this);
		}
	};

	return AnnotationGV;

});

define('web-viewer/annotation/AnnotationDataExtensions',["web-viewer/annotation/Annotation", "web-viewer/annotation/AnnotationGV",
		"web-viewer/data/AbstractDocumentPage",
		"utils/ShapeUtils",
		"utils/XMLUtils",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/annotation/AnnotationConstants",
		"underscore"
	],
	function (Annotation,
		AnnotationGV,
		AbstractDocumentPage,
		ShapeUtils,
		XMLUtils,
		ViewerUtils,
		AnnotationConstants,
		_) {
		"use strict";

		/**
		 * Inject annotation functionality in the data model
		 * The AbstractDocumentPage prototype gets extended with methods for retrieving and storing annotations
		 * The annotations module only supports the WebCenter data adapter (in the future, this implementation here can be changed to work together with other data adapters too)
		 *
		 * @exports AnnotationDataExtensions
		 * @memberof AnnotationModule
		 * @param {Core.ViewerFacade} f - Viewer facade
		 */
		var AnnotationDataExtensions = function (f) {
			var dataAdapter = f.getDataAdapter();

			// only the WebCenter data-adapter is supported
			if (!dataAdapter.WebCenter) {
				f.emit("error.SEVERE", "Can't initialize Annotations on a non-webcenter data-adapter");
				return;
			}

			/**
			 * Initialize the page with needed data object, so the annotation module can work correctly
			 * @private
			 */
			AbstractDocumentPage.prototype._initAnnotationModule = function () {
				if (!this._annotData) {
					/**
					 * dataStatus: [notAvailable | available | loading]
					 * onAnnotationsAvailableCallbacks: array of callbacks for when the annotations are available
					 * annotations: {Array.<Annotation>}
					 */
					this._annotData = {
						dataStatus: 'notAvailable',
						onAnnotationsAvailableCallbacks: [],
						annotations: []
					};
				}
			};

			/**
			 * Load the annotation information for this page
			 * If the information is already available, it won't be reloaded (use reloadAnnotations if this is needed)
			 *
			 * @param {Function} onAnnotationsAvailable - Callback. First argument {Boolean} indicates whether the annotations are loaded successfully or not
			 */
			AbstractDocumentPage.prototype.requestAnnotations = function (onAnnotationsAvailable) {
				this._initAnnotationModule();

				if (this._annotData.dataStatus === "available") {
					onAnnotationsAvailable(true);
				} else {
					if (this._annotData.dataStatus === "loading") {
						this._annotData.onAnnotationsAvailableCallbacks.push(onAnnotationsAvailable);
					} else {
						// start loading information if needed
						if (this.hasAnnotations()) {
							this._annotData.onAnnotationsAvailableCallbacks.push(onAnnotationsAvailable);
							this._fetchAnnotationsFromServer();
						} else {
							onAnnotationsAvailable(true);
						}
					}
				}
			};

			/**
			 * (re)load the annotation information for this page.
			 *
			 * If the information is already available, the information will be reloaded
			 *
			 * @param {Function} onAnnotationsAvailable - Callback. First argument {Boolean} indicates whether the annotations are loaded successfully or not
			 */
			AbstractDocumentPage.prototype.reloadAnnotations = function (onAnnotationsAvailable) {
				this._initAnnotationModule();

				if (this._annotData.dataStatus === "loading") {
					// the system is still loading, register callback
					this._annotData.onAnnotationsAvailableCallbacks.push(onAnnotationsAvailable);
				} else {
					// normal reload
					// start loading information
					this._annotData.onAnnotationsAvailableCallbacks.push(onAnnotationsAvailable);
					this._fetchAnnotationsFromServer();
				}
			};

			/**
			 * Reset the annotations so they will need to be reloaded next time
			 */
			AbstractDocumentPage.prototype.resetAnnotations = function () {
				this._annotData.dataStatus = "notAvailable";
				this._annotData.annotations = [];
			};

			/**
			 * Fire all onAnnotationsAvailableCallbacks with "result" as parameter
			 * @private
			 * @param {Boolean} result- True when the annotations were successfully loaded, false otherwise
			 */
			AbstractDocumentPage.prototype._fireOnAnnotationsAvailableCallbacks = function (result) {
				_.each(this._annotData.onAnnotationsAvailableCallbacks, function (callback) {
					if (callback) {
						callback(result);
					}
				});

				this._annotData.onAnnotationsAvailableCallbacks = [];
				this._annotData.dataStatus = result ? "available" : "notAvailable";
			};


			/**
			 * Fetch annotation information from the server
			 * @private
			 */
			AbstractDocumentPage.prototype._fetchAnnotationsFromServer = function () {
				var params = {};
				if (this.hasGraphics()) {
					params.file = this.get("file");
				} else if (this.hasCad()) {
					params.cad = this.get("cad");
					params.style = this.get("style");
				}

				this._annotData.dataStatus = "loading";



				var promises = [];
				if (this.hasAnnotations(AnnotationConstants.AnnotationOriginTypes.user.id)) {
					promises.push(this._getUserAnnotations(params));
				}
				if (this._hasGVAnnotations()) {
					promises.push(this._getGVAnnotations(params));
				}
				var deferred = $.Deferred();
				var noErrors = true;
				$.when.apply($, promises).done(_.bind(function () {
					var annotations = [];
					_.each(arguments, function (annots) {
						annotations = annotations.concat(annots);
					});
					this._annotData.annotations = annotations;
					deferred.resolve(annotations);
				}, this)).fail(_.bind(function (data) {
					f.i18n.loadBundle("viewer", function () {
						f.emit("error.SEVERE", f.i18n.t("AnnOpenError", ""), data);
					});
					noErrors = false;
					deferred.reject();
				}, this)).always(_.bind(function () {
					this._fireOnAnnotationsAvailableCallbacks(noErrors);
				}, this));

				return deferred.promise();
			};

			/**
			 * fetches xfdf data from url and tries to parse it into annotations
			 * @param {String} url - url to get the xdfd from
			 * @param {Object} payload - extra options for the ajax call, success and error shouldn't be overridden, instead use the promise that is returned
			 * @param {AnnotationConstants.AnnotationOriginTypes.x.id} [originType] - the origin type of the annotations
			 * @return {$.Deferred.<Array.<Annotation>>}
			 */
			AbstractDocumentPage.prototype._getAnnotationsFromURL = function (url, payload, originType) {
				var deferred = $.Deferred();
				var annotations;
				f.ajax(url, _.extend({
					success: _.bind(function (data) {
						if (data) {
							var status = data.getElementsByTagName("status");
							if (status[0]) {
								var value = status[0].getElementsByTagName("value");
								if (value[0] && XMLUtils.textContent(value[0]) === "-1") {
									deferred.reject();
									return;
								}
							}
						}
						annotations = this._parseAnnotationData(data, this._annotData.annotations, originType);
						deferred.resolve(annotations);
					}, this),
					error: _.bind(function (jqXHR, textStatus, errorThrown) {
						//apparently JQuery has sometimes problems parsing the XML (although it is valid XML, try to parse it in another way)
						try {
							var data = new DOMParser().parseFromString(jqXHR.responseText, "text/xml");
							annotations = this._parseAnnotationData(data, this._annotData.annotations, originType);
							deferred.resolve(annotations);
							return;
						} catch (ex) {
							errorThrown = ex;
						}
						deferred.reject(jqXHR.responseText);
					}, this)
				}, payload));
				return deferred.promise();

			};

			/**
			 * fetches annotations created by the user
			 * @param {Object} param for url creation
			 * @return {$.Deferred.<Array.<Annotation>>}
			 */
			AbstractDocumentPage.prototype._getUserAnnotations = function (params) {
				var url = dataAdapter.ViewServer("getannotations.jsp", params);
				return this._getAnnotationsFromURL(url, {}, AnnotationConstants.AnnotationOriginTypes.user.id);
			};

			/**
			 * checks if there are GV annotations
			 */
			AbstractDocumentPage.prototype._hasGVAnnotations = function () {
				return this.hasAnnotations(AnnotationConstants.AnnotationOriginTypes.artwork.id) ||
					this.hasAnnotations(AnnotationConstants.AnnotationOriginTypes.braille.id) ||
					this.hasAnnotations(AnnotationConstants.AnnotationOriginTypes.barcode.id) ||
					this.hasAnnotations(AnnotationConstants.AnnotationOriginTypes.text.id) ||
					this.hasAnnotations(AnnotationConstants.AnnotationOriginTypes.spelling.id) ||
					this.hasAnnotations(AnnotationConstants.AnnotationOriginTypes.ext.id) ||
					this.hasAnnotations(AnnotationConstants.AnnotationOriginTypes.editor.id) ||
					this.hasAnnotations(AnnotationConstants.AnnotationOriginTypes.acrobat.id);
			};

			/**
			 * fetches annotations created by GV
			 * @param {Object} param for url creation
			 * @return {$.Deferred.<Array.<Annotation>>}
			 */
			AbstractDocumentPage.prototype._getGVAnnotations = function (params) {
				if (this.getDocumentVersion().isMultiPage()) {
					params.fmp = 1;
					params.page = this.getPagenum() - 1; // 0 based in XFDF
				}

				params.afs = 1; // external sources should be scanned

				var url = dataAdapter.ViewServer("getannotations.jsp", params);
				var deferred = $.Deferred();

				this._getAnnotationsFromURL(url).done(function (annotations) {
					deferred.resolve(annotations);
				}).fail(function () {
					// If the parsing fails, there aren't any gv annotations
					deferred.resolve([]);
				});
				return deferred;
			};


			/**
			 * Parse the received annotation data (xml)
			 * @param {XMLObject} data - XML object holding the annotation data in the tag annots
			 * @param {Array.<Annotation>} currentAnnotations - The annotations to check against if annotations already exist
			 * @param {AnnotationConstants.AnnotationOriginTypes.x.id} [originType] - The origin type of the annotations if known, otherwise it will be derived                                               
			 * @returns {Array.<Annotation>} annotations
			 */
			AbstractDocumentPage.prototype._parseAnnotationData = function (data, currentAnnotations, originType) {
				//Make sure the annotations are initialized (e.g. when there were no annotations before)
				this._initAnnotationModule();

				// keep track of the message number, mapped by source
				var messageNrs = {};
				if (!data) {
					return [];
				}

				var annotations = [];


				try {
					var elements = data.getElementsByTagName("annots");
					var annots;
					var child;
					if (elements && elements.length >= 1) {
						annots = data.getElementsByTagName("annots")[0];
					}

					if (annots) {
						child = annots.firstChild;
					}

					/**
					 * Make a temporary dictionary so see if the annotations are new or an update
					 *
					 * this will make a flat list of the annotation list, so not considering replies, states etc
					 */
					var tmpAnnotationDictionary = {};
					if (currentAnnotations != null) {
						for (var i = 0; i < currentAnnotations.length; i++) {
							currentAnnotations[i].runThroughAnnotation(true, true, true, function (annot) {
								tmpAnnotationDictionary[annot.name] = annot;
							});
						}
					}

					var rawAnnotationList = {};
					var source;
					while (child) {
						if (child.nodeType === 1) {
							var name = child.getAttribute('name');

							var annot = null;
							if (tmpAnnotationDictionary[name]) {
								annot = tmpAnnotationDictionary[name];

								/*
								 * Remove all the current states/replies/links
								 */
								annot.replies = [];
								annot.links = [];
								annot.states = [];
								annot.resetUnsavedReply();
								annot.cancelModified();
								annot._scaling = null;

								delete tmpAnnotationDictionary[name];
							} else {
								source = this._getAnnotationSourceFromXML(child, originType);
								if (source === AnnotationConstants.AnnotationOriginTypes.user.id) {
									// user annotations
									annot = new Annotation(f);
								} else {
									// external annotations
									annot = this._createAnnotation(source);
								}

							}
							source = annot.originType;
							messageNrs[source] = messageNrs[source] ? ++messageNrs[source] : 1;

							annot.initWithXML(child, {
								trimboxOffset: this.getAnnotationsOffset(),
								mediaboxOffest: this.getAnnotationsOffset(true)
							}, messageNrs[source]);
							rawAnnotationList[annot.name] = annot;
						}
						child = child.nextSibling;
					}

					tmpAnnotationDictionary = null;



					/**
					 * The structure of the xml is that all of the annotations AND their replies are seen as annotations
					 * Only, if an annotation is a reply, it has the "inreplyto" field and should NOT be drawn...
					 */
					for (var annotName in rawAnnotationList) {
						if (rawAnnotationList.hasOwnProperty(annotName)) {
							var rawAnnotation = rawAnnotationList[annotName];
							if (rawAnnotation.inreplyto) {
								// the annotation is a reply or review state, add it to its parent
								var inreplyAnnot = rawAnnotationList[rawAnnotation.inreplyto];
								if (inreplyAnnot) {
									rawAnnotation.parent = inreplyAnnot;

									if (rawAnnotation.state) {
										inreplyAnnot.states.push(rawAnnotation);
									} else {
										inreplyAnnot.replies.push(rawAnnotation);
									}
								}
							} else {
								// root level annoation, push it on the annoations list
								if (rawAnnotation.linkedto) {
									if (rawAnnotationList[rawAnnotation.linkedto]) {
										// the annotation is linked to another annotation, establish link
										rawAnnotationList[rawAnnotation.linkedto].links.push(rawAnnotation);
										annotations.push(rawAnnotationList[annotName]);
										rawAnnotation.parent = rawAnnotationList[rawAnnotation.linkedto];
										//set linked annotation sequence number correct
										rawAnnotation.messagenumber = rawAnnotation.parent.messagenumber;
										rawAnnotation.updateIndicator();
									}
								} else {
									annotations.push(rawAnnotationList[annotName]);
								}
							}
						}
					}
				} catch (err) {
					f.i18n.loadBundle("viewer", function () {
						f.emit("error.SEVERE", f.i18n.t("AnnDisplayError"), err);
					});
				}
				return annotations;
			};

			/**
			 * gets the source of the annotation
			 * @param {NodeList} xml
			 * @param {AnnotationConstants.AnnotationOriginTypes.x.id} originType - the origin of the annotations if already known, otherwise the source will be derived
			 * @return {AnnotationConstants.AnnotationOriginTypes.id}
			 */
			AbstractDocumentPage.prototype._getAnnotationSourceFromXML = function (xml, originType) {
				if (originType) {
					return originType;
				}

				var subject = XMLUtils.attributeValue(xml, subject);
				if (subject) {
					if (ViewerUtils.startsWith(subject, "GVBarcode") || ViewerUtils.startsWith(subject, "Barcode")) {
						return AnnotationConstants.AnnotationOriginTypes.barcode.id;
					}
					if (ViewerUtils.startsWith(subject, "GVBraille") || ViewerUtils.startsWith(subject, "BrailleProof")) {
						return AnnotationConstants.AnnotationOriginTypes.braille.id;
					}
					if (ViewerUtils.startsWith(subject, "GVSpell")) {
						return AnnotationConstants.AnnotationOriginTypes.spelling.id;
					}
					if (ViewerUtils.startsWith(subject, "GVArtwork")) {
						return AnnotationConstants.AnnotationOriginTypes.artwork.id;
					}
					if (ViewerUtils.startsWith(subject, "GVText")) {
						return AnnotationConstants.AnnotationOriginTypes.text.id;
					}
					if (ViewerUtils.startsWith(subject, "Global Vision")) {
						var title = XMLUtils.attributeValue(xml, title);
						if (title) {
							if (ViewerUtils.endsWith(title, "Spelling")) {
								return AnnotationConstants.AnnotationOriginTypes.spelling.id;
							}
							if (ViewerUtils.endsWith(title, "Change")) {
								return AnnotationConstants.AnnotationOriginTypes.text.id;
							}
							if (ViewerUtils.endsWith(title, "Difference")) {
								return AnnotationConstants.AnnotationOriginTypes.artwork.id;
							}
						}
					}
				}

				var source = XMLUtils.attributeValue(xml, "source");
				if (source) {
					var indexOfDash = source.lastIndexOf("-");
					if (indexOfDash > 0) {
						return source.substr(indexOfDash + 1, source.lastIndexOf(".") - indexOfDash - 1);
					}
				}

				// default - anything else
				return AnnotationConstants.AnnotationOriginTypes.ext.id;
			};

			/**
			 * creates an annotation from a particular source
			 * @param {String} source - source of the annotation
			 */
			AbstractDocumentPage.prototype._createAnnotation = function (source) {
				return new AnnotationGV(f, source);
			};


			/**
			 * Get annotations
			 * @returns {Array.<AnnotationModule.Annotation>} Raw annotations
			 */
			AbstractDocumentPage.prototype.getAnnotations = function () {
				this._initAnnotationModule();
				return this._annotData.annotations;
			};

			/**
			 * Get the offset of the annotations on the current page, this depends on the trimbox origin and the rotation/flipped state
			 * @param {Boolean} relativeToMediaBox - If true, the annotations are not relative to the trimbox, but relative to the mediabox
			 * @returns {ShapeUtils.Size}
			 */
			AbstractDocumentPage.prototype.getAnnotationsOffset = function (relativeToMediaBox) {
				var viewport = this.f.getModule("viewport");
				var rotation = this.getRotation() % 360;
				var isFlipped = this.isFlipped();
				var worldBounds = viewport.getWorldBounds();
				var pageSize = this.getPageSize();
				if (this.hasGraphics()) { //for document pages with graphics:

					var trimboxRect;
					if (relativeToMediaBox) {
						trimboxRect = this.getMediaBoxRect();
					} else {
						trimboxRect = this.getTrimboxRect();
					}

					var annotationOrigin;

					//get the correct origin for the annotations depending on the rotation and flipped state to get the correct trimbox corner 
					if ((!isFlipped && rotation === 0) || (isFlipped && rotation === 90)) {
						annotationOrigin = trimboxRect.bottomleft();
					} else if ((!isFlipped && rotation === 90) || (isFlipped && rotation === 180)) {
						annotationOrigin = trimboxRect.topleft();
					} else if ((!isFlipped && rotation === 180) || (isFlipped && rotation === 270)) {
						annotationOrigin = trimboxRect.topright();
					} else {
						annotationOrigin = trimboxRect.bottomright();
					}
					return new ShapeUtils.Size(annotationOrigin.x, -(worldBounds.size.height - annotationOrigin.y)); //make sure to invert the y-coordinate (annotation-coordinates start from bottom, not top)
				} else if (this.hasCad()) { //for CAD files use the origin location in the current world
					var origin = new ShapeUtils.Point();

					if (this.getCad() && this.getCad().getLocation()) { //if the location is defined (happens by the layer model) use it
						origin = this.getCad().getLocation().origin.copy();
					}
					//if the CAD is rotated use following calculations as the orign will always give the top left corner 
					if ((!isFlipped && rotation === 0)) {
						origin.y += pageSize.height;
					} else if ((!isFlipped && rotation === 90)) {
						//nothing needs to change here
					} else if ((!isFlipped && rotation === 180)) {
						origin.x += pageSize.width;
					} else if (!isFlipped) {
						origin.x += pageSize.height;
						origin.y += pageSize.width;
					}
					//if the CAD is flipped and rotated use following calculations
					if ((isFlipped && rotation === 90)) {
						origin.y += pageSize.width;
					} else if ((isFlipped && rotation === 180)) {
						//nothing needs to change
					} else if ((isFlipped && rotation === 270)) {
						origin.x += pageSize.height;
					} else if (isFlipped) {
						origin.x += pageSize.width;
						origin.y += pageSize.height;
					}

					return new ShapeUtils.Size(origin.x, -(worldBounds.size.height - origin.y));
				} else {
					return new ShapeUtils.Size();
				}
			};

			// ANNOTATION STORAGE
			/**
			 * Save annotations
			 * @param {Array.<AnnotationModule.Annotation>} annotations
			 * @param {Array.<AnnotationModule.Annotation>} removedannotations
			 * @param {Function} callback - is called after the annotations were successfully saved
			 */
			AbstractDocumentPage.prototype.saveAnnotations = function (annotations, removedannotations, callback) {
				//Create first the XML
				var xfdf = document.implementation.createDocument("http://ns.adobe.com/xfdf/", 'xfdf', null);

				// set character set (this is needed for IE (9))
				try {
					xfdf.characterSet = "UTF-8";
				} catch (exception) {
					// FireFox and possible other browsers might block this assignment as it is readonly-only property
					// but they assign the character set from the original document (which is UTF-8)
					// we can thus ignore this exception
				}

				try {
					xfdf.charset = "UTF-8";
				} catch (exception) {
					// FireFox and possible other browsers might block this assignment as it is readonly-only property
					// but they assign the character set from the original document (which is UTF-8)
					// we can thus ignore this exception
				}

				var annots = document.createElementNS(null, 'annots');
				xfdf.documentElement.appendChild(annots);

				var addedAnnotations = [];
				var modifiedAnnotations = [];

				for (var i = 0; i < annotations.length; i++) {
					if (annotations[i].isNew) {
						addedAnnotations.push(annotations[i]);
					} else {
						modifiedAnnotations.push(annotations[i]);
					}
					annots.appendChild(annotations[i].toXML(xfdf));
				}

				var remove = [];
				if (removedannotations && removedannotations.length > 0) {
					_.each(removedannotations, function (an) {
						an.runThroughAnnotation(true, true, true, function (annot) {
							remove.push(annot.name);
						});
					});
				}


				/*
        	http://eaw11dd413:8081/ViewServer/changeannotations.jsp?file=000000%5c00002_0000000975%5cbb1ff408c0a8019301550d2d51681e0b.0&ftbtlx=101.88&ftbtly=15.5&ftbbrx=363.88&ftbbry=225.5, version: null]
        	http://eaw11dd413:8081/wcr-web/updateapprovalstate.jsp;jsessionid=80BB4BB5D9CCAB768CED312BC55836C7?approvalcomments=Line%20annoataion%20%22Annotation%22%20added%20by%20Cagnie%2c%20Ruben%20(RUCA)%20on%20Aug%2020%2c%202013%20at%2008%3a32%20EDT%3cBR%3e&userid=00002_0000000001&loginid=02185fd6-5df2-4d3c-9dc9-542a2a8bbb5c&projectid=00002_0000000976&docid=00002_0000000896&docversionid=00002_0000000975
        	 */

				var params = {};

				if (this.get("file")) {
					params.file = this.get("file");
				} else if (this.get("cad")) {
					params.cad = this.get("cad");
					params.style = this.get("style");
				}

				if (remove && remove.length > 0) {
					params.remove = remove;
				}

				var url = dataAdapter.ViewServer("changeannotations.jsp", params);

				var promises = [this._getAnnotationsFromURL(url, {
					processData: false,
					contentType: 'text/xml; charset=UTF-8',
					data: xfdf,
					type: 'POST'
				})];
				if (this._hasGVAnnotations()) {
					promises.push(this._getGVAnnotations(params));
				}
				$.when.apply($, promises).done(_.bind(function () {
					var annotations = [];
					_.each(arguments, function (annots) {
						annotations = annotations.concat(annots);
					});
					this._annotData.annotations = annotations;
					this._fireOnAnnotationsAvailableCallbacks(true);
					this._processAfterParsingSavedAnnotations(addedAnnotations, modifiedAnnotations, removedannotations, callback);

				}, this)).fail(_.bind(function (data) {
					f.i18n.loadBundle("viewer", function () {
						f.emit("error.SEVERE", f.i18n.t("AnnSaveError", ""), data);
					});
				}, this));

			};



			/**
			 *
			 * Things that need to be done after parsing successful response from change annotations (in saveAnnotations).
			 * @private
			 * @param {Array.<Annotation>} addedAnnotations
			 * @param {Array.<Annotation>} modifiedAnnotations
			 * @param {Array.<Annotation>} removedAnnotations
			 * @param {Function} callback
			 */
			AbstractDocumentPage.prototype._processAfterParsingSavedAnnotations = function (addedAnnotations, modifiedAnnotations, removedAnnotations, callback) {
				// update the page that it has annotations or not
				var rawAnnotations = this.getAnnotations();
				var typeOfOrigins = _.pluck(rawAnnotations, "originType");
				var hasAnnotsObj = {};
				_.each(typeOfOrigins, function (origin) {
					hasAnnotsObj[origin] = true;
				});
				this.setHasAnnotations(_.size(rawAnnotations) > 0, hasAnnotsObj);

				var newCallback = function (status) {
					if (callback) {
						callback(rawAnnotations);
					}

					// only if everything was successfull, show this message
					if (status) {
						// SDK support
						var annotationCallbackUrl = f.getProperty("annotationcallbackurl");
						if (annotationCallbackUrl) {
							f.ajax(annotationCallbackUrl, {
								dataType: undefined, // intelligent guess of what will come back
								noAjaxParam: true
							});
						}

						f.i18n.loadBundle("viewer", function () {
							f.emit("userMessage", "success", f.i18n.t("AnnotationsSaved"), true);
						});
					}
				};

				this.reportAnnotations(addedAnnotations, modifiedAnnotations, removedAnnotations, newCallback);
			};

			/**
			 * Report to WebCenter that there was a modification on the annotations for this document page
			 *
			 * @param {Array.<Annotation>} newannotations - List of the newly added annotations
			 * @param {Array.<Annotation>} modifiedannotations - List of the modified annotations
			 * @param {Array.<Annotation>} removedannotations - List of the removed annotatinos
			 * @param {Function} [callback] - Callback with parameter boolean success (true) or error (false)
			 */
			AbstractDocumentPage.prototype.reportAnnotations = function (newannotations, modifiedannotations, removedannotations, callback) {
				// Aggregate the callback
				var expectedNumberOfCalls = 0;
				if (newannotations) {
					expectedNumberOfCalls += newannotations.length;
				}
				if (modifiedannotations) {
					expectedNumberOfCalls += modifiedannotations.length;
				}
				if (removedannotations) {
					expectedNumberOfCalls += removedannotations.length;
				}
				if (expectedNumberOfCalls === 0) {
					callback(true);
				}
				var numberOfCalls = 0;
				var status = true;
				var aggregatedCallback = function (success) {
					if (!success) {
						// one of the calls failed
						status = false;
					}

					// count the number of calls to this function
					numberOfCalls++;
					if (numberOfCalls === expectedNumberOfCalls) {
						if (callback) {
							callback(status);
						}
					}
				};

				f.i18n.loadBundle("viewer", _.bind(function () {
					if (newannotations) {
						_.each(newannotations, function (annotation) {
							annotation.cancelModified();
							this.alertOnAnnotationAdded(annotation, aggregatedCallback);
						}, this);
					}
					if (modifiedannotations) {
						_.each(modifiedannotations, function (annotation) {
							annotation.cancelModified();
							this.alertOnAnnotationModified(annotation, aggregatedCallback);
						}, this);
					}
					if (removedannotations) {
						_.each(removedannotations, function (annotation) {
							this.alertOnAnnotationDelete(annotation, aggregatedCallback);
						}, this);
					}
				}, this));
			};


			/**
			 * This function will determine the string for in the document history report
			 * For a reply: Reply (sequenceNrGodFater)
			 * For an annotation: Type annotation (sequenceNrIfKnown)
			 *
			 * @param {Annotation} annotation - the annotation for which to generate type string
			 * @return {String}
			 */
			function _getAnnotationType(annotation) {
				var sequenceNumber = "";
				if (annotation.inreplyto) {
					// the annotation is a Reply

					// find the godfather to determine the sequence number
					var godfather = annotation.getGodfather();


					if (godfather && godfather.messagenumber >= 0) {
						sequenceNumber = " (" + godfather.messagenumber + ")";
					}
					return f.i18n.t("AnnCommReply") + sequenceNumber;
				} else {
					if (annotation.messagenumber >= 0 && !annotation.isNew) {
						sequenceNumber = " (" + annotation.messagenumber + ")";
					}
					return f.i18n.t(AnnotationConstants.AnnotationComments[annotation.annottype]) + sequenceNumber;
				}
			}

			/**
			 * Add that an annotation was added to the document history
			 * @param {Annotation} annotation - Annotation that is added
			 * @param {Function} [callback] - Callback with parameter boolean success (true) or error (false)
			 * @example
			 * Line annoataion "Annotation" added by Cagnie, Ruben (RUCA) on Aug 21, 2013 at 10:15 EDT
			 * Rectangle annotation "http://www.esko.com" added by Cagnie, Ruben (RUCA) on Aug 21, 2013 at 10:42 EDT
			 */
			AbstractDocumentPage.prototype.alertOnAnnotationAdded = function (annotation, callback) {
				var sameDates = (annotation.date.getTime() == annotation.creationdate.getTime());
				var type = _getAnnotationType(annotation);
				var text = f.i18n.t(sameDates ? "AnnCommAdded" : "AnnCommAdded2", type, annotation.getStringContent(130), annotation.getUserName(), f.i18n.d("AnnCommDate", annotation.creationdate), f.i18n.d("AnnCommDate", annotation.date));
				if (this.getDocumentVersion().isMultiPage()) { //has multiple pages
					text = f.i18n.t("PagesString", this.getPagenum(), this.getPagenum(), this.getDocumentVersion().getPageCount()) + text;
				}
				this.updateApprovalState(text, callback);
			};

			/**
			 * Add that an annotation was deleted to the document history
			 * @param {Annotation} annotation - Annotation that is deleted
			 * @param {Function} [callback] - Callback with parameter boolean success (true) or error (false)
			 * @param {Boolean} [doNotReportReplies - If true, the deletion of replies to the deleted annotation is not reported (used internally)
			 * @example
			 * Rectangle annotation (27) "swaha" deleted by Cagnie, Ruben (RUCA) - was created by 27 Cagnie, Ruben (RUCA) on Aug 21, 2013 at 10:07 EDT and last modified on Aug 21, 2013 at 10:08 EDT
			 */
			AbstractDocumentPage.prototype.alertOnAnnotationDelete = function (annotation, callback, doNotReportReplies) {
				if (!doNotReportReplies && annotation.replies && annotation.replies.length > 0) {
					// we should report replies, and the annotation has 1 or more replies

					// generate a new callback which is only executed after the callback is executed for every reply
					var newCallback = _.after(annotation.replies.length, _.bind(function () {
						// report the main annotation, but don't report its replies 
						this.alertOnAnnotationDelete(annotation, callback, true);
					}, this));

					// for every reply, make a report
					_.each(annotation.replies, function (reply) {
						this.alertOnAnnotationDelete(reply, newCallback);
					}, this);
				} else {
					// report the deletion
					var sameDates = (annotation.date.getTime() == annotation.creationdate.getTime());
					var type = _getAnnotationType(annotation);
					var text = f.i18n.t(sameDates ? "AnnCommDeleted" : "AnnCommDeleted2", type, annotation.getStringContent(130), f.getCurrentUser().getFullName(), annotation.getUserName(), f.i18n.d("AnnCommDate", new Date()), f.i18n.d("AnnCommDate", annotation.creationdate), f.i18n.d("AnnCommDate", annotation.date));
					if (this.getDocumentVersion().isMultiPage()) { //has multiple pages
						text = f.i18n.t("PagesString", this.getPagenum(), this.getPagenum(), this.getDocumentVersion().getPageCount()) + text;
					}
					this.updateApprovalState(text, callback);
				}
			};

			/**
			 * Add that an annotation was deleted to the document history
			 * @param {Annotation} annotation - Annotation that is modified
			 * @param {Function} [callback] - Callback with parameter boolean success (true) or error (false)
			 * @example
			 * Oval annotation (1) "Eggs" modified by Cagnie, Ruben (RUCA) at Aug 21, 2013 at 10:30 EDT - was created by Cagnie, Ruben (RUCA) on Jun 08, 2013 at 14:26 EDT
			 */
			AbstractDocumentPage.prototype.alertOnAnnotationModified = function (annotation, callback) {
				var type = _getAnnotationType(annotation);
				var text = f.i18n.t("AnnCommModded", type, annotation.getStringContent(130), f.getCurrentUser().getFullName(), annotation.getUserName(), f.i18n.d("AnnCommDate", annotation.creationdate), f.i18n.d("AnnCommDate", new Date()));
				if (this.getDocumentVersion().isMultiPage()) { //has multiple pages
					text = f.i18n.t("PagesString", this.getPagenum(), this.getPagenum(), this.getDocumentVersion().getPageCount()) + text;
				}
				this.updateApprovalState(text, callback);
			};


			/**
			 * Generic function to write something to the document history
			 * If the call fails, an error is already shown.
			 * @param {Data.AbstractDocumentPage} page
			 * @param {Function} [callback] - Callback with parameter boolean success (true) or error (false)
			 * @example
			 * http://eaw11dd413:8081/wcr-web/updateapprovalstate.jsp?approvalcomments=Line%20annoataion%20%22Annotation%22%20added%20by%20Cagnie%2c%20Ruben%20(RUCA)%20on%20Aug%2020%2c%202013%20at%2008%3a32%20EDT%3cBR%3e&userid=00002_0000000001&loginid=02185fd6-5df2-4d3c-9dc9-542a2a8bbb5c&projectid=00002_0000000976&docid=00002_0000000896&docversionid=00002_0000000975
			 */
			AbstractDocumentPage.prototype.updateApprovalState = function (approvalState, callback) {
				var params = {
					approvalcomments: approvalState,
					projectid: this.getDocument().get("projectid"),
					docid: this.getDocument().get("docid"),
					docversionid: this.get("pageDocversionid"),
					pageno: this.getPagenum()
				};

				if (this.get("pageindex") !== undefined) {
					params.pageindex = this.get("pageindex");
				}
				if (this.get("pagename") !== undefined) {
					params.pagename = this.get("pagename");
				}

				var url = dataAdapter.WebCenter("updateapprovalstate.jsp");
				f.ajax(url, {
					success: function (data, textStatus, jqXHR) {
						if (callback) {
							callback(true);
						}
					},
					error: function (jqXHR, textStatus, errorThrown) {
						f.i18n.loadBundle("viewer", function () {
							f.emit("error.SEVERE", f.i18n.t("DocumentHistoryUpdateError"), jqXHR, textStatus, errorThrown);
						});
						if (callback) {
							callback(false);
						}
					},
					data: params,
					type: 'POST'
				});
			};

		};

		return AnnotationDataExtensions;
	});

define('web-viewer/annotation/AnnotationNoteDrawTool',["underscore",
		"web-viewer/viewport/tools/AbstractTool",
		"utils/ShapeUtils",
		"web-viewer/annotation/Annotation",
		"web-viewer/annotation/AnnotationConstants",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/core/CoreConstants"
	],
	function (_,
		AbstractTool,
		ShapeUtils,
		Annotation,
		AnnotationConstants,
		ViewerUtils,
		CoreConstants) {

		/**
		 * This is the tool for drawing note annotations.
		 * Also see it's related tool AnnotationDrawTool.
		 * @exports AnnotationNoteDrawTool
		 * @memberof AnnotationModule
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f
		 * @param {AnnotationModel} annotationModel - the model used by the module for annotations
		 * @param {AnnotationModule} annotationModule - the annotation module reference
		 */
		var AnnotationNoteDrawTool = function (f, annotationModel, annotationModule) {
			AbstractTool.call(this, f, CoreConstants.TOOLS.ANNOTATION_NOTE_DRAW);

			var defaultState = "default";

			this.drawLinkedAnnotation = false;

			this._annotationModel = annotationModel;

			this._annotationModule = annotationModule;

			/**
			 * Set the annotation color of the note annotation to draw
			 */
			this.setAnnotationColor = function (color) {
				f.getCurrentUser().setColor("PREFERENCES_ANNOTATION_NOTE_COLOR", color);
			};

			/**
			 * Get the annotation color of the note annotation to draw
			 */
			this.getAnnotationColor = function () {
				if (f.getProperty("annotationcolor")) {
					return f.getProperty("annotationcolor");
				} else {
					return f.getCurrentUser().getColor("PREFERENCES_ANNOTATION_NOTE_COLOR");
				}
			};

			/**
			 * @override
			 */
			this.getDefaultCursor = function () {
				return "cursor-drawComment";

			};

			/**
			 * Get the annotation layer
			 */
			var getAnnotationLayer = _.bind(function (e) {
				var v = this._widget.requestView(e);
				if (v) {
					var l = v.getLayer("Annotations");
					if (l) {
						return l;
					}
				}
				return null;
			}, this);

			/**
			 * This is the mode when the tool is active,
			 * it will zoom on click and go the couldStartDraggingMode if left mousedown is clicked
			 */
			this.registerState(defaultState, {
				"contextmenu": function (e) {
					// prevent the context menu
					e.preventDefault();
					e.stopPropagation();
					return false;
				},
				"click": _.bind(function (e) {
					var l = getAnnotationLayer(e);
					if (l) {
						//first get the current touchpoint, the point where was clicked
						var touchPoint = this.keepPointWithinWorldBounds(this.getWorldPoint(e), true);
						var newAnnotRect = new ShapeUtils.Rect(touchPoint, new ShapeUtils.Size()); //make a rect out of it

						//create the new annotation
						var newAnnotation = new Annotation(f);
						newAnnotation.initAnnotation(AnnotationConstants.AnnotationType.ANNOT_PIN, this.getAnnotationColor(), false, annotationModel);
						//make sure the rectangle is in world coordinates and rounded so detection of inside active page is correct
						newAnnotRect = l.getWorldRectFromScene(newAnnotRect);
						newAnnotRect.origin.x = ViewerUtils.parseFloat(newAnnotRect.origin.x.toFixed(2));
						newAnnotRect.origin.y = ViewerUtils.parseFloat(newAnnotRect.origin.y.toFixed(2));
						newAnnotation.rect = newAnnotRect;

						if (this._annotationModel.getPage().isReaderSpread()) {
							this._annotationModule.promptIfAnnotationOutsidePage(newAnnotation.rect, _.bind(
									function () {
										this.addAnnotation(newAnnotation);
									}, this),
								function () {
									newAnnotation = null;
								});
						} else {
							this.addAnnotation(newAnnotation);
						}
					}
				}, this)
			});

			/**
			 * @override
			 */
			this.canBeInterrupted = function () {
				return true;
			};

			this.activeKeyEvents = [];
			this.persistentKeyEvents = [];
			/**
			 * @override
			 */
			this._override("initialize", function (widget, toolManager) {
				this._super(widget, toolManager);
				this.changeState(defaultState);
				this.registerActiveEvents(["contextmenu", "click"]);
				this.activeKeyEvents.push(["shift", _.bind(function (e) {
					this.drawLinkedAnnotation = true;
				}, this), "keydown"]);

				this.activeKeyEvents.push(["shift", _.bind(function (e) {
					this.drawLinkedAnnotation = false;
				}, this), "keyup"]);
			});

			/**
			 * @override
			 */
			this._override("activate", function (interruptRequest) {
				this._super(interruptRequest);
				annotationModel.setLockSelection(true);
			});

			/**
			 * @override
			 */
			this.canBeActivated = function () {
				return annotationModel.canChangeAnnotations();
			};

			/**
			 * @override
			 */
			this._override("deactivate", function (interruptRequest) {
				this._super(interruptRequest);
				this.drawLinkedAnnotation = false;
				annotationModel.setLockSelection(false);
			});

			f.getCurrentUser().on("options.changed.PREFERENCES_ANNOTATIONTOOLS_VISIBLE", _.bind(function (oldValue, newValue) {
				if (!newValue && this.isActive()) {
					this.requestDeactivation();
				}
			}, this));
		};

		// inherit
		AnnotationNoteDrawTool.prototype = new AbstractTool();
		// correct the constructor pointer 
		AnnotationNoteDrawTool.prototype.constructor = AnnotationNoteDrawTool;

		/**
		 * Adds a created annotation to the annotationmodel
		 * @param {Annotation} the new annotation to be added
		 */
		AnnotationNoteDrawTool.prototype.addAnnotation = function (newAnnotation) {
			if (this.drawLinkedAnnotation) {
				this._annotationModel.addLinkedAnnotation(newAnnotation);
			} else {
				this._annotationModel.addAnnotation(newAnnotation);
				this.requestDeactivation();
			}
		};

		return AnnotationNoteDrawTool;
	});

define('web-viewer/annotation/AnnotationTouchDrawTool',["underscore",
		"web-viewer/viewport/tools/AbstractTool",
		"utils/ShapeUtils",
		"web-viewer/utils/GEOUtils",
		"web-viewer/utils/DrawUtils",
		"web-viewer/utils/TouchUtil",
		"web-viewer/annotation/Annotation",
		"web-viewer/utils/LineworkPoint",
		"web-viewer/annotation/AnnotationConstants",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/core/CoreConstants"
	],
	function (_, AbstractTool,
		ShapeUtils,
		GEOUtils,
		DrawUtils,
		TouchUtil,
		Annotation,
		LineworkPoint,
		AnnotationConstants,
		ViewerUtils,
		CoreConstants) {
		"use strict";

		var DEFAULTSTATE = "default";
		var DRAWSTATE = "draw";
		var DRAWINGSTATE = "drawing";

		/**
		 * This is the tool for drawing annotations with touch interaction
		 * @exports AnnotationTouchDrawTool
		 * @memberof AnnotationModule
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f
		 * @param {AnnotationModel} annotationModel - the model used by the module for annotations
		 * @param {AnnotationModule} annotationModule - the annotation module reference
		 */
		var AnnotationTouchDrawTool = function (f, annotationModel, annotationModule) {
			AbstractTool.call(this, f, CoreConstants.TOOLS.ANNOTATION_TOUCH_DRAW);



			var threshold = 0; //use this threshold to debug the freeform tool. This allows to only draw a couple of segments. When set to 0, it's harmless

			var newAnnotation = undefined;

			this.drawLinkedAnnotation = false;
			this._annotationModel = annotationModel;
			this._annotationModule = annotationModule;

			/**
			 * Set the type of annotation to draw
			 */
			this.setAnnotationType = function (type) {

			};

			/**
			 * Get the type of annotation to draw
			 */
			this.getAnnotationType = function () {
				return AnnotationConstants.AnnotationType.ANNOT_FREEFORM;
			};

			/**
			 * Set the annotation color to draw with
			 */
			this.setAnnotationColor = function (color) {
				f.getCurrentUser().setColor("PREFERENCES_ANNOTATION_RECTANGLE_COLOR", color);
			};

			/**
			 * Get the annotation color to draw with
			 */
			this.getAnnotationColor = function () {
				if (f.getProperty("annotationcolor")) {
					return f.getProperty("annotationcolor");
				} else {
					return f.getCurrentUser().getColor("PREFERENCES_ANNOTATION_RECTANGLE_COLOR");
				}
			};

			/**
			 * @override
			 */
			this.getDefaultCursor = function () {
				return "cursor-drawAnnotation";
			};

			/**
			 * Get the annotation layer
			 */
			var getAnnotationLayer = _.bind(function (e) {
				var v = this._widget.requestView(e);
				if (v) {
					var l = v.getLayer("Annotations");
					if (l) {
						return l;
					}
				}
				return null;
			}, this);

			/**
			 * This is the mode when the tool is active,
			 * it will zoom on click and go the couldStartDraggingMode if left mousedown is clicked
			 */
			this.registerState(DEFAULTSTATE, {
				"panstart": function (e) {},
				"panmove": function (e) {},
				"panend": function (e) {},
				canBeInterrupted: true
			});

			/**
			 * Draw state
			 */
			this.registerState(DRAWSTATE, _.defaults({
				'panstart': _.bind(function (e) {
					this.requestCanvasLayer(e.pointers[0]);
					var touchPoint = this.keepPointWithinWorldBounds(this.getWorldPoint(e.pointers[0]), true);
					newAnnotation = new Annotation(f);
					newAnnotation.initAnnotation(AnnotationConstants.AnnotationType.ANNOT_FREEFORM, this.getAnnotationColor(), false, annotationModel);
					newAnnotation.rect = new ShapeUtils.Rect(touchPoint, new ShapeUtils.Size());

					this.changeState(DRAWINGSTATE);
				}, this)
			}, this.getState(DEFAULTSTATE)));

			/**
			 * Function that should be called when the drawing is finished (does correct cleanups etc)
			 */
			var doneDrawing = _.bind(function (e) {
				cleanupCanvas(); //clear the canvas

				if (!e) {
					newAnnotation = null; //if there was no event passes, put the newAnnotation to null
				}

				//If the new annotation is a line or an arrow (pointArray==2), make sure that the points are not on the same location
				if (newAnnotation && newAnnotation.pointArray.length === 2) {
					if (newAnnotation.pointArray[0].x == newAnnotation.pointArray[1].x && newAnnotation.pointArray[0].y == newAnnotation.pointArray[1].y) {
						newAnnotation = null;
					}
				}

				//Make sure the annotation has at least some width or some height
				if (newAnnotation !== null && (Math.abs(newAnnotation.rect.size.width) === 0 || Math.abs(newAnnotation.rect.size.height) <= 0)) {
					//If both the width and height are 0, just return (not really useful to have a 0-sized annotation)
					newAnnotation = null;
				}

				//If we have an annotation, let's save it
				if (newAnnotation) {
					var l = getAnnotationLayer(e);
					if (l) {
						//Convert the drawed annotation from Scene (pixels) to World (mm)
						newAnnotation.rect = l.getWorldRectFromScene(newAnnotation.rect); //first the box

						if (this._annotationModel.getPage().isReaderSpread()) {
							this._annotationModule.promptIfAnnotationOutsidePage(newAnnotation.rect, _.bind(
									function () {
										this.createAnnotation(newAnnotation, l);
									}, this),
								function () {
									newAnnotation = null;
								});
						} else {
							this.createAnnotation(newAnnotation, l);
						}

					} else {
						newAnnotation = null;
					}
				}

				this.changeState(DRAWSTATE);
			}, this);

			/**
			 * creates the annotation, makes a new annotation object, this will not save the annotation to the back-end
			 */
			this.createAnnotation = function (newAnnotation, annotationLayer) {
				for (var i = 0; i < newAnnotation.pointArray.length; i++) { //then the points
					var point = newAnnotation.pointArray[i];
					var p = new ShapeUtils.Point(point.x, point.y);
					var newp = annotationLayer.getWorldPointFromScene(p);
					point.x = newp.x;
					point.y = newp.y;

				}

				//Check if it need to be a linked annotation or a single new annotation
				annotationModel.addAnnotation(newAnnotation);
				this.requestDeactivation();

				newAnnotation = null;
			};

			/**
			 * Render the annotation via the draw utils
			 */
			var renderAnnotation = _.throttle(_.bind(function () {
				var ctx = this.getCanvasCtx();
				if (ctx) { // could be that the layer is already removed (caused by throttle)
					var csize = this.getCanvasSize();
					ctx.clearRect(0, 0, csize.width, csize.height);

					if (newAnnotation) {
						DrawUtils.drawOverlay(this.f, ctx, [newAnnotation]);
					}
				}
			}, this), 40);

			/**
			 * Drawing state
			 */
			this.registerState(DRAWINGSTATE, _.defaults({
				'enterState': function () {
					f.emit("viewport.startInteractive");
					f.emit("layout.disableTextSelection");
				},
				'leaveState': function () {
					f.emit("viewport.stopInteractive");
					setTimeout(function () {
						f.emit("layout.enableTextSelection");
					}, 500);
				},
				'panmove': _.bind(function (e) {
					var newRect = newAnnotation.rect.copy();
					var touchPoint = this.keepPointWithinWorldBounds(this.getWorldPoint(e.pointers[0]), true);


					var point = new LineworkPoint();
					if (newAnnotation.pointArray.length === 0) {
						point.makeMoveTo(newAnnotation.rect.origin.x, newAnnotation.rect.origin.y);
					} else {
						var previousPoint = newAnnotation.pointArray[newAnnotation.pointArray.length - 1];
						if (Math.abs(previousPoint.x - touchPoint.x) < threshold || Math.abs(previousPoint.y - touchPoint.y) < threshold) {
							return;
						}
						point.makeLineTo(touchPoint.x, touchPoint.y);
					}
					newAnnotation.pointArray.push(point);

					//Check if the width needs to be updated
					var width = Math.abs(point.x - newRect.origin.x);
					if (point.x < newRect.origin.x) {
						var diff = Math.abs(newRect.origin.x - point.x);
						newRect.origin.x = point.x;
						newRect.size.width += diff;
					} else if (width > newRect.size.width) {
						newRect.size.width = width;
					}

					//Check if the height needs to be updated
					var height = Math.abs(point.y - newRect.origin.y);
					if (point.y < newRect.origin.y) {
						var diff = Math.abs(newRect.origin.y - point.y);
						newRect.origin.y = point.y;
						newRect.size.height += diff;
					} else if (height > newRect.size.height) {
						newRect.size.height = height;
					}

					newAnnotation.setRect(newRect, true);

					renderAnnotation();
				}, this),
				'panend': function (e) {
					doneDrawing(e.pointers[0]);
				},
				canBeInterrupted: false
			}, this.getState(DEFAULTSTATE)));

			/**
			 * Can the tool be interrupted ?
			 */
			this.canBeInterrupted = function () {
				return false;
			};

			this.activeKeyEvents = [];
			this.persistentKeyEvents = [];

			/**
			 * Cleanup canvas
			 */
			var cleanupCanvas = _.bind(function () {
				this.releaseCanvasLayer();
			}, this);


			/**
			 * @override
			 */
			this.canBeActivated = function () {
				return annotationModel.canChangeAnnotations();
			};


			f.getCurrentUser().on("options.changed.PREFERENCES_ANNOTATIONTOOLS_VISIBLE", _.bind(function (oldValue, newValue) {
				if (!newValue && this.isActive()) {
					this.requestDeactivation();
				}
			}, this));

		};

		// inherit
		AnnotationTouchDrawTool.prototype = Object.create(AbstractTool.prototype);
		// correct the constructor pointer 
		AnnotationTouchDrawTool.prototype.constructor = AnnotationTouchDrawTool;


		/**
		 * Initialize the annotation touch draw tool
		 * @override
		 */
		AnnotationTouchDrawTool.prototype.initialize = function (widget, toolManager) {
			AbstractTool.prototype.initialize.apply(this, arguments);
			// note this only works on the viewportwidget
			this._touchUtil = new TouchUtil(widget.getElement()[0]);
			this.changeState(DEFAULTSTATE);
		};

		/**
		 * Activate
		 * @override
		 */
		AnnotationTouchDrawTool.prototype.doActivation = function () {
			AbstractTool.prototype.doActivation.call(this);

			this._annotationModel.setLockSelection(true);

			this._touchUtil.addPanStartListener(_.bind(function (e) {
				this.getCurrentState().panstart(e);
			}, this));

			this._touchUtil.addPanMoveListener(_.bind(function (e) {
				this.getCurrentState().panmove(e);
			}, this));

			this._touchUtil.addPanEndListener(_.bind(function (e) {
				this.getCurrentState().panend(e);
			}, this));

			/*
			 * Reconsider the state of the tool
			 */
			this.changeState(DRAWSTATE);
			this.drawLinkedAnnotation = false;

		};

		/**
		 * deactivate
		 * @override
		 */
		AnnotationTouchDrawTool.prototype.doDeactivation = function () {
			AbstractTool.prototype.doDeactivation.call(this);

			this.changeState(DEFAULTSTATE);
			this.releaseCanvasLayer();

			this._touchUtil.removeAllListeners();

			this._annotationModel.setLockSelection(false);
		};

		return AnnotationTouchDrawTool;
	});

define('web-viewer/annotation/AnnotationTouchTool',["underscore",
		"web-viewer/viewport/tools/AbstractTool",
		"utils/ShapeUtils",
		"web-viewer/utils/TouchUtil",
		"web-viewer/annotation/Annotation",
		"web-viewer/core/CoreConstants"
	],
	function (_,
		AbstractTool,
		ShapeUtils,
		TouchUtil, Annotation,
		CoreConstants) {
		"use strict";
		/**
		 * Construct a new Annotation touch Tool (a touch optimized version to select and move annotations)
		 * @exports AnnotationTouchTool
		 * @memberof AnnotationModule
		 *
		 * @class Tool to select and edit annotations
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {AnnotationModel} annotationModel - Annotation model
		 */
		var AnnotationTouchTool = function (f, annotationModel) {
			AbstractTool.call(this, f, CoreConstants.TOOLS.ANNOTATION_TOUCH);

			this.f = f;

			this._annotationModel = annotationModel;

			/**
			 * Different states constants
			 */
			this._defaultState = "defaultState";
			this._annotationSelectedState = "annotationSelectedState";
			this._annotationSelectedPanningState = "annotationSelectedPanningState";

			// no annotation is selected
			this.registerState(this._defaultState, function (that) {
				return {
					"enterState": function () {
						that._annotationModel.unselectAllAnnotations();
						that._activeAnnotation = null;
					},
					"panstart": function (e) {},
					"panmove": function (e) {},
					"panend": function (e) {},
					"singletap": function (event) {
						var location = that.getScenePointFromTouch(event);
						if (location.view) {
							var annotation = that.getClosestAnnotation(location.view, location.point);

							if (annotation) {
								that._annotationModel.selectAnnotation(annotation);
							} else {
								that._annotationModel.unselectAllAnnotations();
							}
						}
					}
				};

			}(this));

			// an annotation was selected
			this.registerState(this._annotationSelectedState, function (that) {
				return _.defaults({
					"enterState": function () { },
					"panstart": function (event) {
						var location = that.getScenePointFromTouch(event);
						if (location.view) {
							var annotation = that.getClosestAnnotation(location.view, location.point);
							if (that._activeAnnotation === annotation && that._annotationModel.canChangeAnnotation(annotation)) {
								that.changeState(that._annotationSelectedPanningState);

								that._previousPoint = that._getAnnotationLayer(location.view).getWorldPointFromScene(that.getScenePoint(event.pointers[0]));
								that.requestActivation();
							}
						}
					},
					"panmove": function (e) { },
					"panend": function (e) { }
				}, that.getState(that._defaultState));
			}(this));

			// an annotation was selected, and a panning operation started in the selected annotation
			this.registerState(this._annotationSelectedPanningState, function (that) {

				return {
					enterState: function () {
						that.f.emit("viewport.startInteractive");
						that.f.emit("layout.disableTextSelection");
					},
					leaveState: function () {
						if (that.isActive()) {
							that.requestDeactivation();
						}
						that.f.emit("viewport.stopInteractive");
						setTimeout(function () {
							that.f.emit("layout.enableTextSelection");
						}, 500);
					},
					"panstart": function (e) {},
					"panmove": function (event) {
						if (that.isActive()) {
							var view = that._widget.requestView(event.pointers[0]);
							if (view) {
								var newPoint = that._getAnnotationLayer(view).getWorldPointFromScene(that.getScenePoint(event.pointers[0]));
								var worldBounds = that._widget.getWorld().getWorldBounds(); //get the world bounds
								//get the converted worldbounds (with origin (0,0)), the origin gets added later on to not intervere with transforming the worldbounds to annotation coordinate system
								var worldRect = that._getAnnotationLayer(view).convertWorldRectToLayer(new ShapeUtils.Rect(new ShapeUtils.Point(), worldBounds.size));
								worldRect.origin.y = -(worldRect.size.height + worldRect.origin.y) - worldBounds.origin.y; //calculate the origin in annotation coordinate system
								worldRect.origin.x += worldBounds.origin.x; //add the origin to the worldRect (this is not converted as it stays a constant)
								var moveSize = that.keepMoveSizeWithinRect(worldRect, that._activeAnnotation.rect, that._previousPoint.sizeTo(newPoint)); //keep move size within world bounds
								annotationModel.moveAnnotation(that._activeAnnotation, moveSize);
								that._previousPoint = newPoint;
							}
						} else {
							that.changeState(that._annotationSelectedState);
						}
					},
					"panend": function (e) {
						that.changeState(that._annotationSelectedState);
					}

				};
			}(this));


			this._annotationModel.on("unselectedAnnotation", _.bind(function () {
				this.changeState(this._defaultState);
			}, this));

			this._annotationModel.on("selectedAnnotation", _.bind(function (annot) {
				if (annot != this._activeAnnotation) {
					this._activeAnnotation = annot;
					this.changeState(this._annotationSelectedState);
				}
			}, this));
		};

		// inherit
		AnnotationTouchTool.prototype = Object.create(AbstractTool.prototype);
		// correct the constructor pointer 
		AnnotationTouchTool.prototype.constructor = AnnotationTouchTool;

		/**
		 * Get the center scene point of a TouchEvent
		 * @param {TouchEvent} event - the touch event
		 * @param {ViewportWidget} widget - the ViewportWidget
		 * @returns {Object} {view: reference to the View, point: scenePoint}
		 */
		AnnotationTouchTool.prototype.getScenePointFromTouch = function (event) {
			return this._touchUtil.getScenePointFromTouch(event, this._widget);
		};

		/**
		 * Get the annotation near the given scenepoint
		 * @param {ViewportView} view - the view in which the annotation should be searched
		 * @param {ShapeUtils.Point} scenePoint - the scene point where the annotation should be searched
		 * @returns {Annotation} the annotation near the scenePoint or null
		 */
		AnnotationTouchTool.prototype.getClosestAnnotation = function (view, scenePoint) {
			var closestLinework = view.getClosestLineworkFromPoint(scenePoint, ["Annotations"], 30);
			if (closestLinework) {
				if (closestLinework instanceof Annotation) {
					return closestLinework;
				} else if (closestLinework.anchor && closestLinework.linework instanceof Annotation) {
					return closestLinework.linework;
				}
			}
			return null;
		};

		/**
		 * @override
		 */
		AnnotationTouchTool.prototype.initialize = function (widget, toolManager) {
			AbstractTool.prototype.initialize.apply(this, arguments);


			this._touchUtil = new TouchUtil(widget.getElement()[0]);

			this.changeState(this._defaultState);

			this._touchUtil.addSingleTapListener(_.bind(function (event) {
				this.getCurrentState().singletap(event);
			}, this));
			this._touchUtil.addPanStartListener(_.bind(function (event) {
				this.getCurrentState().panstart(event);
			}, this));
			this._touchUtil.addPanMoveListener(_.bind(function (event) {
				this.getCurrentState().panmove(event);
			}, this));
			this._touchUtil.addPanEndListener(_.bind(function (event) {
				this.getCurrentState().panend(event);
			}, this));
		};


		/**
		 * Get the annotation layer based on a point
		 * @param {ViewportView} view
		 * @returns {ViewportLayer} Annotations layer of null if not found
		 */
		AnnotationTouchTool.prototype._getAnnotationLayer = function (view) {

			if (view) {
				var l = view.getLayer("Annotations");
				if (l) {
					return l;
				}
			}
			return null;
		};
		return AnnotationTouchTool;
	});

define('web-viewer/annotation/AnnotationToolbarComponent',["component/toolbar/ToolbarComponent",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/layout/component/SubtoolbarButton",
		"web-viewer/layout/component/ColorPicker",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/annotation/AnnotationConstants"
	],
	function (ToolbarComponent,
		ButtonGroup,
		ViewerToolbarButton,
		SubtoolbarButton,
		ColorPicker,
		LayoutConstants,
		AnnotationConstants) {
		"use strict";

		/**
		 * @exports AnnotationToolbarComponent
		 * @memberof AnnotationModule
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {AnnotationModel} annotationsModel - Annotation model
		 * @param {AnnotationDrawTool} drawTool - The annotation draw tool
		 * @param {AnnotationNoteDrawTool} noteDrawTool - The annotation note draw tool
		 */
		var AnnotationToolbarComponent = function (f, annotationsModel, drawTool, noteDrawTool) {
			ToolbarComponent.call(this, LayoutConstants.ToolbarComponent.ANNOTATION, f);
			this.f = f;
			this.annotationModel = annotationsModel;
			/**
			 * Group: annotate
			 */
			var annotateGroup = new ButtonGroup("AnnotateGroup");
			annotateGroup.setTitle(f.i18n, "viewer", "AnnBtnGroupTitle");
			this.addButtonGroup(annotateGroup);

			// Annotate ViewerToolbarButton
			var hideAnnotationsButton = new ViewerToolbarButton(f, "HideAnnotationsButton", f.getOption("PREFERENCES_ANNOTATIONTOOLS_VISIBLE") ? "icon-WCR_Annotation-Hide" : "icon-WCR_Annotation-Show", true, function () {
				annotationsModel.toggleVisibility();
			});
			// set initial tooltip
			hideAnnotationsButton.setTooltip("viewer", f.getOption("PREFERENCES_ANNOTATIONTOOLS_VISIBLE") ? "HideAnnotationsBtnTT" : "ShowAnnotationsBtnTT");
			// react on preference changes
			f.getCurrentUser().on("options.changed.PREFERENCES_ANNOTATIONTOOLS_VISIBLE", function (oldValue, newValue) {
				if (newValue) {
					hideAnnotationsButton.setTooltip("viewer", "HideAnnotationsBtnTT");
					hideAnnotationsButton.setIcon("icon-WCR_Annotation-Hide");
				} else {
					hideAnnotationsButton.setTooltip("viewer", "ShowAnnotationsBtnTT");
					hideAnnotationsButton.setIcon("icon-WCR_Annotation-Show");
				}
				updateAnnotationButtonsUsability();
			});
			annotateGroup.addButton(hideAnnotationsButton);

			// Annotate ViewerToolbarButton
			var annotateButton = new ViewerToolbarButton(f, "AnnotateButton", "icon icon-WCR_Annotation-Global-cont icon-danger", true, function () {
				//make sure annotations are visible
				annotationsModel.toggleVisibility(true);
			});
			annotateButton.setTooltip("viewer", "AddAnnotationBtnTT");
			annotateButton.listenToTool(drawTool, true);
			annotateGroup.addButton(annotateButton);
			// update the initial icon
			if (f.getOption("PREFERENCES_ANNOTATION_TOOLMODE") != AnnotationConstants.ANNOT_PIN) {
				annotateButton.setIcon(AnnotationConstants.AnnotationToolIcons[f.getOption("PREFERENCES_ANNOTATION_TOOLMODE")].icon,
					AnnotationConstants.AnnotationToolIcons[f.getOption("PREFERENCES_ANNOTATION_TOOLMODE")].sub);
			}
			// update the icon on events
			f.getCurrentUser().on("options.changed.PREFERENCES_ANNOTATION_TOOLMODE", function (oldValue, newValue) {
				if (newValue != AnnotationConstants.ANNOT_PIN) {
					annotateButton.setIcon(AnnotationConstants.AnnotationToolIcons[newValue].icon,
						AnnotationConstants.AnnotationToolIcons[newValue].sub);
				}
			});

			/**
			 * Subtoolbar Component: Annotation Selector
			 */
			var annotationSelectorSubtoolbar = new ToolbarComponent("AnnotationSelectorSubtoolbar");
			annotateButton.addSubtoolbarComponent(annotationSelectorSubtoolbar);



			/**
			 * Group: rectangle + oval + line + arrow + freeform + highlight + comment
			 */
			var annotateSubGroup = new ButtonGroup("AnnotateSubGroup");
			annotationSelectorSubtoolbar.addButtonGroup(annotateSubGroup);
			var annotationColorPicker = undefined;
			var modeType = "PREFERENCES_ANNOTATION_TOOLMODE";
			// Rectangle
			var rectangleButton = new SubtoolbarButton(f, "RectangleButton", AnnotationConstants.AnnotationToolIcons[AnnotationConstants.AnnotationType.ANNOT_RECT].icon, true, function () {
				drawTool.setAnnotationType(AnnotationConstants.AnnotationType.ANNOT_RECT);
				if (annotationColorPicker) {
					annotationColorPicker.setColor(drawTool.getAnnotationColor());
				}
			});
			rectangleButton.listenToToolOption("annotationdrawtool", modeType, AnnotationConstants.AnnotationType.ANNOT_RECT);
			rectangleButton.setTooltip("viewer", "AnnRectTT");
			rectangleButton.setIcon(AnnotationConstants.AnnotationToolIcons[AnnotationConstants.AnnotationType.ANNOT_RECT].icon,
				AnnotationConstants.AnnotationToolIcons[AnnotationConstants.AnnotationType.ANNOT_RECT].sub);
			annotateSubGroup.addButton(rectangleButton);
			// Oval
			var ovalButton = new SubtoolbarButton(f, "OvalButton", "addOval", true, function () {
				drawTool.setAnnotationType(AnnotationConstants.AnnotationType.ANNOT_ELLIPSE);
				if (annotationColorPicker) {
					annotationColorPicker.setColor(drawTool.getAnnotationColor());
				}
			});
			ovalButton.listenToToolOption("annotationdrawtool", modeType, AnnotationConstants.AnnotationType.ANNOT_ELLIPSE);
			ovalButton.setTooltip("viewer", "AnnOvalTT");
			ovalButton.setIcon(AnnotationConstants.AnnotationToolIcons[AnnotationConstants.AnnotationType.ANNOT_ELLIPSE].icon,
				AnnotationConstants.AnnotationToolIcons[AnnotationConstants.AnnotationType.ANNOT_ELLIPSE].sub);
			annotateSubGroup.addButton(ovalButton);
			// Line
			var lineButton = new SubtoolbarButton(f, "LineButton", "addLine", true, function () {
				drawTool.setAnnotationType(AnnotationConstants.AnnotationType.ANNOT_LINE);
				if (annotationColorPicker) {
					annotationColorPicker.setColor(drawTool.getAnnotationColor());
				}
			});
			lineButton.listenToToolOption("annotationdrawtool", modeType, AnnotationConstants.AnnotationType.ANNOT_LINE);
			lineButton.setTooltip("viewer", "AnnLineTT");
			lineButton.setIcon(AnnotationConstants.AnnotationToolIcons[AnnotationConstants.AnnotationType.ANNOT_LINE].icon,
				AnnotationConstants.AnnotationToolIcons[AnnotationConstants.AnnotationType.ANNOT_LINE].sub);
			annotateSubGroup.addButton(lineButton);
			// Arrow
			var arrowButton = new SubtoolbarButton(f, "ArrowButton", "addArrow", true, function () {
				drawTool.setAnnotationType(AnnotationConstants.AnnotationType.ANNOT_ARROW);
				if (annotationColorPicker) {
					annotationColorPicker.setColor(drawTool.getAnnotationColor());
				}
			});
			arrowButton.listenToToolOption("annotationdrawtool", modeType, AnnotationConstants.AnnotationType.ANNOT_ARROW);
			arrowButton.setTooltip("viewer", "AnnArrowTT");
			arrowButton.setIcon(AnnotationConstants.AnnotationToolIcons[AnnotationConstants.AnnotationType.ANNOT_ARROW].icon,
				AnnotationConstants.AnnotationToolIcons[AnnotationConstants.AnnotationType.ANNOT_ARROW].sub);

			annotateSubGroup.addButton(arrowButton);
			// Freeform
			var freeformButton = new SubtoolbarButton(f, "FreeformButton", "addFreehand", true, function () {
				drawTool.setAnnotationType(AnnotationConstants.AnnotationType.ANNOT_FREEFORM);
				if (annotationColorPicker) {
					annotationColorPicker.setColor(drawTool.getAnnotationColor());
				}
			});
			freeformButton.listenToToolOption("annotationdrawtool", modeType, AnnotationConstants.AnnotationType.ANNOT_FREEFORM);
			freeformButton.setTooltip("viewer", "AnnFreehandTT");
			freeformButton.setIcon(AnnotationConstants.AnnotationToolIcons[AnnotationConstants.AnnotationType.ANNOT_FREEFORM].icon,
				AnnotationConstants.AnnotationToolIcons[AnnotationConstants.AnnotationType.ANNOT_FREEFORM].sub);
			annotateSubGroup.addButton(freeformButton);
			// Highlight
			var highlightButton = new SubtoolbarButton(f, "HighlightButton", "addHighlight", true, function () {
				drawTool.setAnnotationType(AnnotationConstants.AnnotationType.ANNOT_HIGHLIGHT);
				if (annotationColorPicker) {
					annotationColorPicker.setColor(drawTool.getAnnotationColor());
				}
			});
			highlightButton.listenToToolOption("annotationdrawtool", modeType, AnnotationConstants.AnnotationType.ANNOT_HIGHLIGHT);
			highlightButton.setTooltip("viewer", "AnnHighlightTT");
			highlightButton.setIcon(AnnotationConstants.AnnotationToolIcons[AnnotationConstants.AnnotationType.ANNOT_HIGHLIGHT].icon,
				AnnotationConstants.AnnotationToolIcons[AnnotationConstants.AnnotationType.ANNOT_HIGHLIGHT].sub);
			annotateSubGroup.addButton(highlightButton);

			/**
			 * Other options
			 */

			var annotateSubGroupGlobAction = new ButtonGroup("AnnotateSubGroupGlobAction");
			annotationSelectorSubtoolbar.addButtonGroup(annotateSubGroupGlobAction);
			var addGlobalAnnotationButton = new SubtoolbarButton(f, "GlobalAnnotation", "addGlobalAnnotation", true, function (e) {
				drawTool.setAnnotationType(AnnotationConstants.AnnotationType.ANNOT_RECT);
				drawTool.addGlobalAnnotation(e);
			});
			addGlobalAnnotationButton.setTooltip("viewer", "AnnNewRectGlobal");
			addGlobalAnnotationButton.setIcon("icon-WCR_Annotation-Global-cont icon-danger", ["icon icon-WCR_Annotation-Global-sub icon-white"]);
			annotateSubGroupGlobAction.addButton(addGlobalAnnotationButton);

			/**
			 * Subtoolbar Component: Annotation Color selector
			 */
			if (!f.getProperty("annotationcolor")) {
				var annotateSubGroupColor = new ButtonGroup("AnnotateSubGroupColor");
				annotationSelectorSubtoolbar.addButtonGroup(annotateSubGroupColor);

				annotationColorPicker = new ColorPicker(function (newColor) {
					drawTool.setAnnotationColor("#" + newColor);
				});

				f.getCurrentUser().on("options.changed.PREFERENCES_ANNOTATION_RECTANGLE_COLOR", function (oldValue, newValue) {
					annotationColorPicker.setColor(f.getCurrentUser().getColor("PREFERENCES_ANNOTATION_RECTANGLE_COLOR"));
				});

				annotationColorPicker.setColor(f.getCurrentUser().getColor("PREFERENCES_ANNOTATION_RECTANGLE_COLOR"));

				var colorSelectorButton = new SubtoolbarButton(f, "ColorSelector", annotationColorPicker.element, true);
				colorSelectorButton.setTooltip("viewer", "AnnColorDefaultRect");

				var colorResetButton = new SubtoolbarButton(f, "ColorReset", "icon icon-WCR_WF-Undo", true, _.bind(function () {
					f.resetOption("PREFERENCES_ANNOTATION_RECTANGLE_COLOR");

					this.askToChangeAnnotationColors();
				}, this));
				colorResetButton.setIcon("icon icon-WCR_WF-Undo");
				colorResetButton.setTooltip("viewer", "AnnResetDefaultColor");


				annotateSubGroupColor.addButton(colorSelectorButton);
				annotateSubGroupColor.addButton(colorResetButton);

			}



			// Comment
			var commentButton = new ViewerToolbarButton(f, "CommentButton", "addComment", true, function () {
				// make sure annotations are visible
				annotationsModel.toggleVisibility(true);
			});
			commentButton.listenToTool(noteDrawTool, true);
			commentButton.setTooltip("viewer", "AnnCommNote");
			commentButton.setIcon("icon icon-WCR_Annotation-Note-sub");
			annotateGroup.addButton(commentButton);

			/**
			 * Subtoolbar Component: Annotation Color selector
			 */
			if (!f.getProperty("annotationcolor")) {
				/**
				 * Subtoolbar Component: Annotation Selector
				 */
				var annotationNoteSelectorSubtoolbar = new ToolbarComponent("AnnotationNoteSelectorSubtoolbar");
				commentButton.addSubtoolbarComponent(annotationNoteSelectorSubtoolbar);

				/**
				 * Group: rectangle + oval + line + arrow + freeform + highlight + comment
				 */
				var annotateNoteSubGroup = new ButtonGroup("AnnotateNoteSubGroup");
				annotationNoteSelectorSubtoolbar.addButtonGroup(annotateNoteSubGroup);

				var annotationNoteColorPicker = new ColorPicker(function (newColor) {
					noteDrawTool.setAnnotationColor("#" + newColor);
				});

				f.getCurrentUser().on("options.changed.PREFERENCES_ANNOTATION_NOTE_COLOR", function (oldValue, newValue) {
					annotationNoteColorPicker.setColor(noteDrawTool.getAnnotationColor());
				});
				annotationNoteColorPicker.setColor(noteDrawTool.getAnnotationColor());

				var noteColorSelectorButton = new SubtoolbarButton(f, "ColorSelector", annotationNoteColorPicker.element, true);
				noteColorSelectorButton.setTooltip("viewer", "AnnColorDefaultNote");

				var noteColorResetButton = new SubtoolbarButton(f, "ColorReset", "icon icon-WCR_WF-Undo", true, _.bind(function () {
					f.resetOption("PREFERENCES_ANNOTATION_NOTE_COLOR");

					this.askToChangeAnnotationColors();
				}, this));
				noteColorResetButton.setTooltip("viewer", "AnnResetNoteColor");

				annotateNoteSubGroup.addButton(noteColorSelectorButton);
				annotateNoteSubGroup.addButton(noteColorResetButton);
			}

			/**
			 * Disable/Enable the buttons depending on:
			 * 	1. If the user can't change the annotations (e.g. document is approved and annotationmode is set to "readonly when approved"
			 *  2. If "hideAllAnnotations" is true
			 */
			var updateAnnotationButtonsUsability = function () {
				if (!annotationsModel.canChangeAnnotations(true)) {
					var reason = "";
					if (annotationsModel.getDisabledReason().length > 0) {
						reason = annotationsModel.getDisabledReason();
					}

					// set the tooltip for when the button is disabled
					annotateButton.setDisabledTooltip(reason);
					commentButton.setDisabledTooltip(reason);
					// disable buttons
					if (drawTool.isActive()) {
						drawTool.requestDeactivation();
					}
					if (noteDrawTool.isActive()) {
						noteDrawTool.requestDeactivation();
					}
					annotateButton.setEnabled(false);
					commentButton.setEnabled(false);
				} else {
					// the buttons should be enabled
					annotateButton.setEnabled(true);
					commentButton.setEnabled(true);
				}

				hideAnnotationsButton.setEnabled(!annotationsModel.isTemporarilyDisabledAndNoAnnotationsVisible());
			};
			updateAnnotationButtonsUsability();

			annotationsModel.on("readWriteChanged", updateAnnotationButtonsUsability);
			annotationsModel.on("temporarilyDisableChanged", updateAnnotationButtonsUsability);
			annotationsModel.on("disabledReasonChanged", updateAnnotationButtonsUsability);

		};

		AnnotationToolbarComponent.prototype = Object.create(ToolbarComponent.prototype);
		AnnotationToolbarComponent.prototype.constructor = AnnotationToolbarComponent;
		AnnotationToolbarComponent.toolbarName = LayoutConstants.ToolbarComponent.ANNOTATION;



		/**
		 * Ask the user if the current annotations should be updated with newly set defaults
		 * @private
		 */
		AnnotationToolbarComponent.prototype.askToChangeAnnotationColors = function () {
			this.f.i18n.loadBundle("viewer", _.bind(function () {
				if (confirm(this.f.i18n.t("AnnUpdateColors"))) {
					this.annotationModel.updateAllAnnotationColors();
				}
			}, this));
		};


		return AnnotationToolbarComponent;
	});

define('web-viewer/annotation/AnnotationTouchToolbarComponent',["component/toolbar/ToolbarComponent",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/layout/LayoutConstants"
	],
	function (ToolbarComponent,
		ButtonGroup,
		ViewerToolbarButton,
		LayoutConstants) {

		/**
		 * @exports AnnotationTouchToolbarComponent
		 * @memberof AnnotationModule
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {AnnotationModel} annotationsModel - Annotation model
		 * @param {AnnotationTouchDrawTool} drawTool - The annotation draw tool
		 */
		var AnnotationTouchToolbarComponent = function (f, annotationsModel, drawTool) {
			ToolbarComponent.call(this, LayoutConstants.ToolbarComponent.ANNOTATION, f);
			this.f = f;
			this.annotationModel = annotationsModel;
			/**
			 * Group: annotate
			 */
			var annotateGroup = new ButtonGroup("AnnotateGroup");
			annotateGroup.setTitle(f.i18n, "viewer", "AnnBtnGroupTitle");
			this.addButtonGroup(annotateGroup);



			// Annotate ViewerToolbarButton
			var annotateButton = new ViewerToolbarButton(f, "AnnotateButton", "icon-WCR_Add", true, function () {
				//make sure annotations are visible
				annotationsModel.toggleVisibility(true);
			});
			annotateButton.listenToTool(drawTool, true);
			annotateGroup.addButton(annotateButton);


			/**
			 * Disable/Enable the buttons depending on:
			 * 	1. If the user can't change the annotations (e.g. document is approved and annotationmode is set to "readonly when approved"
			 *  2. If "hideAllAnnotations" is true
			 */
			var updateAnnotationButtonsUsability = function () {
				if (!annotationsModel.canChangeAnnotations(true)) {
					var reason = "";
					if (annotationsModel.getDisabledReason().length > 0) {
						reason = annotationsModel.getDisabledReason();
					}

					// set the tooltip for when the button is disabled
					annotateButton.setDisabledTooltip(reason);

					// disable buttons
					if (drawTool.isActive()) {
						drawTool.requestDeactivation();
					}

					annotateButton.setEnabled(false);
				} else {
					// the buttons should be enabled
					annotateButton.setEnabled(true);
				}
			};
			updateAnnotationButtonsUsability();

			annotationsModel.on("readWriteChanged", updateAnnotationButtonsUsability);
			annotationsModel.on("temporarilyDisableChanged", updateAnnotationButtonsUsability);
			annotationsModel.on("disabledReasonChanged", updateAnnotationButtonsUsability);

		};

		AnnotationTouchToolbarComponent.prototype = Object.create(ToolbarComponent.prototype);
		AnnotationTouchToolbarComponent.prototype.constructor = AnnotationTouchToolbarComponent;
		AnnotationTouchToolbarComponent.toolbarName = LayoutConstants.ToolbarComponent.ANNOTATION;

		return AnnotationTouchToolbarComponent;
	});

/**
 *  @namespace AnnotationModule
 */
define('web-viewer/annotation/AnnotationModule',["underscore",
		"web-viewer/core/AbstractModule",
		"web-viewer/annotation/AnnotationModel",
		"web-viewer/annotation/AnnotationLayerModel",
		"web-viewer/annotation/AnnotationPopupsLayerModel",
		"web-viewer/annotation/AnnotationTool",
		"web-viewer/annotation/AnnotationDrawTool",
		"web-viewer/annotation/AnnotationIndicatorTool",
		"web-viewer/annotation/AnnotationWidget",
		"utils/Transformation",
		"utils/ShapeUtils",
		"web-viewer/annotation/AnnotationDataExtensions",
		"web-viewer/annotation/AnnotationNoteDrawTool",
		"web-viewer/annotation/AnnotationTouchDrawTool",
		"web-viewer/annotation/AnnotationTouchTool",
		"web-viewer/annotation/AnnotationToolbarComponent",
		"web-viewer/annotation/AnnotationTouchToolbarComponent",
		"web-viewer/annotation/AnnotationConstants",
		"web-viewer/layout/component/ModalPopup",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/utils/ListenerUtil"
	],
	function (_,
		AbstractModule,
		AnnotationModel,
		AnnotationLayerModel,
		AnnotationPopupsLayerModel,
		AnnotationTool,
		AnnotationDrawTool,
		AnnotationIndicatorTool,
		AnnotationWidget,
		Transformation,
		ShapeUtils,
		AnnotationDataExtensions,
		AnnotationNoteDrawTool,
		AnnotationTouchDrawTool,
		AnnotationTouchTool,
		AnnotationToolbarComponent,
		AnnotationTouchToolbarComponent,
		AnnotationConstants,
		ModalPopup,
		LayoutConstants,
		ListenerUtil) {
		"use strict";

		/**
		 * @exports AnnotationModule
		 * @memberof AnnotationModule
		 * @class The annotation module is the main module that controls the loading and displaying of the annotations
		 * @augments Core.AbstractModule
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {Boolean} silenced - if true, the module shouldn't publish tools/widgets/toolbarcomponents
		 * @constructor
		 */
		var AnnotationModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "annotation");
			ListenerUtil.mixin(this, f);

			/**
			 * Current annotation model
			 */
			this.annotationModel = new AnnotationModel(this.f, "annotation");

			/**
			 * Layer Models
			 */
			this.layerModels = {};

			/**
			 * Inject annotation functionality in data objects
			 */
			AnnotationDataExtensions(f);

			/**
			 * Tools
			 */
			// NOTE we should only create the needed tools... based on the device mode 
			this._drawTool = new AnnotationDrawTool(f, this.annotationModel, this);
			this._noteDrawTool = new AnnotationNoteDrawTool(f, this.annotationModel, this);
			this._indicatorTool = new AnnotationIndicatorTool(f, this.annotationModel);



			this._publishTool(this._indicatorTool);

			if (this.f.isMobile()) {
				// only publish the annotation touch draw tool
				this._touchDrawTool = new AnnotationTouchDrawTool(f, this.annotationModel, this);
				this._publishTool(this._touchDrawTool);
				this._selectTool = new AnnotationTouchTool(f, this.annotationModel);
				this._publishTool(this._selectTool);
			} else {
				// desktop
				// publish both the draw tool and not draw tool
				this._publishTool(this._drawTool);
				this._publishTool(this._noteDrawTool);

				this._selectTool = new AnnotationTool(f, this.annotationModel);
				this._publishTool(this._selectTool);
			}

			/**
			 * Component: annotate
			 */
			this._annotateComponent = null; // set the annotation tool bar component as unavailable

			// publish annotate tool bar component
			this._publishToolbarComponent(AnnotationToolbarComponent.toolbarName, _.bind(function (layoutType) {
				if (layoutType === LayoutConstants.LayoutType.MOBILE) {
					this._annotateComponent = new AnnotationTouchToolbarComponent(f, this.annotationModel, this._touchDrawTool);
				} else {
					this._annotateComponent = new AnnotationToolbarComponent(f, this.annotationModel, this._drawTool, this._noteDrawTool);
				}
				return this._annotateComponent;
			}, this));

			/**
			 * Widget(s)
			 */
			this._annotationWidget = null; // set the annotation widget as unavailable

			// publish annotation widget
			this._publishWidget(AnnotationWidget.widgetName, _.bind(function (layoutType) {
				this.annotationModel.setLayoutType(layoutType);
				this._annotationWidget = new AnnotationWidget(f, this, this.annotationModel, layoutType);
				return this._annotationWidget;
			}, this));



			this.on("highlightAnnotation", _.bind(function (annotation) {
				this.annotationModel.selectAnnotation(annotation, true);
				this._indicatorTool.setAnnotation(annotation);

				f.emit("toolManager.request.activation", "annotationindicatortool");
			}, this));

			this.on("duplicateAnnotation", _.bind(function (annotation) {
				if (annotation.getPermissionFor(AnnotationConstants.ActionPermissions.DUPLICATE)) {
					this.annotationModel.getAnnotationFilterModel().setOriginType(AnnotationConstants.AnnotationOriginTypes.user.id);
					this._duplicateAnnotation(annotation, false, false, true, true);
				}
			}, this));

			this.on("duplicateWithoutText", _.bind(function (annotation) {
				if (annotation.getPermissionFor(AnnotationConstants.ActionPermissions.DUPLICATE_WITHOUT_TEXT)) {
					this.annotationModel.getAnnotationFilterModel().setOriginType(AnnotationConstants.AnnotationOriginTypes.user.id);
					this._duplicateAnnotation(annotation, false, false, true, true, true);
				}
			}, this));

			this.on("duplicateAllFilteredAnnotations", _.bind(function () {
				var filteredAnnotations = _.filter(this.annotationModel.getFilteredAnnotations(), function (annotation) {
					return annotation.getPermissionFor(AnnotationConstants.ActionPermissions.DUPLICATE);
				});
				if (filteredAnnotations.length > 0) {
					this.annotationModel.getAnnotationFilterModel().setOriginType(AnnotationConstants.AnnotationOriginTypes.user.id);
					_.each(filteredAnnotations, function (annotation) {
						this._duplicateAnnotation(annotation, false, false, true, true, false, true);
					}, this);
					this.annotationModel.modelChanged();
					this.annotationModel.modelContentChanged();
				}

			}, this));

			this.on("duplicateAllFilteredAnnotationWithoutText", _.bind(function () {
				var filteredAnnotations = _.filter(this.annotationModel.getFilteredAnnotations(), function (annotation) {
					return annotation.getPermissionFor(AnnotationConstants.ActionPermissions.DUPLICATE_WITHOUT_TEXT);
				});
				if (filteredAnnotations.length > 0) {
					this.annotationModel.getAnnotationFilterModel().setOriginType(AnnotationConstants.AnnotationOriginTypes.user.id);
					_.each(filteredAnnotations, function (annotation) {
						this._duplicateAnnotation(annotation, false, false, true, true, true, true);
					}, this);
					this.annotationModel.modelChanged();
					this.annotationModel.modelContentChanged();
				}
			}, this));

			this.on("zoomToAnnotation", _.bind(function (annotation) {
				var options = {};

				/**
				 * Constrain the zoom so it never goes lower than 2 cm
				 */
				var zoomRect = annotation.rect.copy();
				if (f.getOption("PREFERENCES_ANNOTATION_NEXT_MODE") !== 3) {
					zoomRect = annotation.getOutliningRect();
				}
				var minSize = 20;

				var diff = null;
				if (Math.abs(zoomRect.size.width) < minSize) {
					diff = (minSize - zoomRect.size.width) / 2;
					zoomRect.origin.x -= diff;
					zoomRect.size.width = minSize;
				}
				if (Math.abs(zoomRect.size.height) < minSize) {
					diff = (minSize - zoomRect.size.height) / 2;
					zoomRect.origin.y -= diff;
					zoomRect.size.height = minSize;
				}

				options.rect = convertAnnotationRectToWorld(zoomRect);
				options.constrain = true;
				options.animated = true;

				this.annotationModel.ensureVisibility();

				// Select annotation
				if (this.annotationModel.getLayoutType() !== LayoutConstants.LayoutType.MOBILE) {
					this.annotationModel.selectAnnotation(annotation, true);
				}

				f.emit("viewport.fitInRect", options);
			}, this));

			// with this event annotation module can be temporarily disabled
			this.on("temporarilyDisable", _.bind(function (disable, enableTogglingAnnotations, reason, whichToEnable) {
				this.annotationModel.temporarilyDisable(disable, enableTogglingAnnotations, reason, whichToEnable);
			}, this));

			var getAnnotationTransform = _.bind(function () {
				return this.layerModels['AnnotationLayer'].getTransform().copy();
			}, this);

			function convertAnnotationRectToWorld(annotationRect) {
				return getAnnotationTransform().transformedRect(annotationRect);
			}

			function convertWorldRectToAnnotationLayer(worldRect) {
				var transform = getAnnotationTransform().createInverse();
				return new ShapeUtils.Rect(transform.transformedPoint(worldRect.origin), transform.transformedSize(worldRect.size));
			}

			this.on("createAnnotation", _.bind(function (rect, content, type) {
				if (type === undefined) {
					type = AnnotationConstants.AnnotationType.ANNOT_RECT;
				}
				this.annotationModel.addAnnotationWithRect(convertWorldRectToAnnotationLayer(rect), content, type);
			}, this));

			this._registerListener("intercept.activePageChange", function (context) {
				var newActivePage = context.newActivePage;
				var oldActivePage = context.oldActivePage;
				if (newActivePage !== oldActivePage) {
					var hasUnsavedAnnotations = this.promptToSaveUnsavedAnnotations(_.bind(function () {
						this.f.getDocumentController().changeActivePage(newActivePage);
					}, this), _.bind(function () {
						//Don't change the active page, cancel doesn't change the page and doesn't save the annotations
					}, this), _.bind(function () {
						this.annotationModel.resetAllAnnotations(); //don't save annotations removes all the unsaved annotations
						this.f.getDocumentController().changeActivePage(newActivePage); //change the active page, the annotations of the previous page are not saved
					}, this));
					if (hasUnsavedAnnotations) {
						context.newActivePage = oldActivePage;
					}
				}
			}, this);

			this._registerListener("viewControls.compensationChanged", function () {
				if (this.annotationModel) {
					var page = this.annotationModel.getAnnotationsPage();

					//TODO allow the reference page the compensated, the Java Viewer does not allow this, as it returns wrong parameter values
					if (page !== this.f.getDocumentController().getDifferencePage()) {
						var scaling = page.getCurrentScaling();

						this.annotationModel.scaleAllAnnotations(scaling);
					}
				}
			}, this, this.f);

			this._registerListener("filterStateChanged", function (doPersist) {
				if (doPersist) {
					this.f.setOption("ANNOTATION_FILTER", this.annotationModel.getAnnotationFilterModel().getFilterState());
				} else {
					this.f.resetOption("ANNOTATION_FILTER");
				}
			}, this, this.annotationModel.getAnnotationFilterModel());


			// React on annotation added to save 3D View
			this._registerListener("annotationAdded", function (annotation) {
				if(!this.f.isMobile()) {
					this._save3DView(annotation);
				}
			}, this, this.annotationModel);

			this.on("update3DView", _.bind(function (annotation) {
				this._save3DView(annotation);
			}, this));

			this.on("delete3DView", _.bind(function (annotation) {
				this._save3DView(annotation, true);
			}, this));

			this.on("show3DView", _.bind(function (annotation) {
				this._show3DView(annotation);
			}, this));

			this._registerListener("selectedAnnotation", function (annotation) {
				if (this.annotationModel.getLayoutType() !== LayoutConstants.LayoutType.MOBILE) {
					this._show3DView(annotation.getGodfather(), false);
				}
			}, this, this.annotationModel);


			// When the 3D widget is available repaint the rows to update action visiblity
			this.f.whenModuleAvailable("view3D", _.bind(function (view3DModule) {
				view3DModule.on("3DWidget.enabled", _.bind(function (enabled) {
					this.annotationModel.modelContentChanged();
				}, this));
			}, this));


			// React on active page changes
			this.f.getDocumentController().whenActiveOrDifferencePageChangedAndParametersAvailable(this._whenActiveOrDifferencePageChangedAndParametersAvailable, this);
			this.f.getDocumentController().whenActiveOrDifferencePageChangedAndParametersAvailableOrChange(this._whenActiveOrDifferencePageChangedAndParametersAvailableOrChange, this);
		};

		// inherit
		AnnotationModule.prototype = Object.create(AbstractModule.prototype);
		// correct the constructor pointer 
		AnnotationModule.prototype.constructor = AnnotationModule;


		/**
		 * Show the 3D view of the annotation if the 3D module is available
		 * @param {Annotation} annotation - The annotation to show the 3D view of
		 * @param {Boolean} [doSelect = true] - Select the annotation also
		 */
		AnnotationModule.prototype._show3DView = function (annotation, doSelect) {
			var view3DModule = this.f.getModule("view3D");
			var position = annotation.get3DViewPosition();
			if (view3DModule && position) {
				if (this.annotationModel.getLayoutType() === LayoutConstants.LayoutType.MOBILE) {
					view3DModule.show3D(position);
				} else {
					view3DModule.setCameraPosition(position);
				}
				if (doSelect === undefined || doSelect) {
					this.annotationModel.selectAnnotation(annotation, true); // also select
				}
			}
		};

		/**
		 * Save the current 3D view in the annotation
		 * @param {Annotation} annotation - The annotation to set the 3D view of
		 * @param {Boolean} doDelete - Indicating to clear the current 3D view rather than setting a new one                               
		 */
		AnnotationModule.prototype._save3DView = function (annotation, doDelete) {
			// only for main annotations
			if (annotation.getGodfather() === annotation) {
				if (doDelete) {
					annotation.set3DViewPosition(null);
					this.annotationModel.selectAnnotation(annotation, true); // also select
					this.annotationModel.modelChanged(annotation);
				} else {
					var view3DModule = this.f.getModule("view3D");
					if (view3DModule) {
						var position = view3DModule.getCameraPosition();
						if (position) {
							annotation.set3DViewPosition(position);
							this.annotationModel.selectAnnotation(annotation, true); // also select
							this.annotationModel.modelChanged(annotation);
						}
					}
				}
			}
		};


		/**
		 * Are there still unsaved element?
		 * @override
		 * @returns {Boolean}
		 */
		AnnotationModule.prototype.hasUnsavedElements = function () {
			return this.annotationModel.getUnsavedAnnotationCount() > 0;
		};


		/**
		 * Save all unsaved elements
		 * @override
		 * @param callbackFunction
		 */
		AnnotationModule.prototype.saveAllUnsavedElements = function (callbackFunction) {
			this.annotationModel.save(null, callbackFunction);
		};


		/**
		 * When the active page has changed, and the parameters are availabled for this page, reinitialize annotations model
		 * @param {data.AbstractDocumentPage} page
		 */
		AnnotationModule.prototype._whenActiveOrDifferencePageChangedAndParametersAvailableOrChange = function (page, difference) {
			//handle new page
			if (this.layerModels['AnnotationLayer'] === undefined) {
				// Create the annotations layer
				this.layerModels['AnnotationLayer'] = new AnnotationLayerModel(this.f, "Annotations", this.annotationModel);
				// Create the annotation popups layer
				this.layerModels['AnnotationPopupsLayer'] = new AnnotationPopupsLayerModel(this.f, "AnnotationPopups", this.annotationModel);

				var viewportModule = this.f.getModule("viewport");
				// Global layer
				viewportModule.addLayer("Annotations", this.layerModels['AnnotationLayer']);
				// Main layer only
				viewportModule.addLayer("AnnotationPopups", this.layerModels['AnnotationPopupsLayer'], "Main");
			}

			if (this.annotationModel.isTemporarilyDisabled()) {
				if (this.annotationModel.isTemporarilyCurrentPageAnnotationsShown()) {
					this.annotationModel.setPage(page);
					this.annotationModel.modelChanged();
				} else if (this.annotationModel.isTemporarilyDifferencePageAnnotationsShown()) {
					this.annotationModel.setPage(difference);
					this.annotationModel.modelChanged();
				}
			} else {
				this.annotationModel.setPage(page);
			}
		};

		/**
		 * When active or difference page changes, calculate the annotation offsets for both active and/or reference CAD page, only for when those pages are in compare
		 * @param {AbstractDocumentPage} activePage
		 * @param {AbstractDocumentPage} referencePage
		 * @param {AbstractDocumentPage|null} oldPage
		 * @param {Boolean} isCurrentChanged
		 */
		AnnotationModule.prototype._whenActiveOrDifferencePageChangedAndParametersAvailable = function (activePage, referencePage, oldPage, isCurrentChanged) {
			if (this._annotateComponent) {
				if (activePage.isBlankPage()) {
					this._annotateComponent.hide();
				} else {
					this._annotateComponent.show();
				}
			}
		};

		/**
		 * Toggle whether the difference page annotations should be shown temporarily
		 * @param {Boolean} [toggle]
		 */
		AnnotationModule.prototype.toggleDifferencePageAnnotations = function (toggle) {
			if (toggle === undefined) {
				toggle = !this.annotationModel.isTemporarilyDifferencePageAnnotationsShown();
			}
			if (toggle) {
				this.f.setOption("PREFERENCES_ANNOTATIONTOOLS_VISIBLE", 1, true);
				this.annotationModel.showTemporarilyDifferencePageAnnotations();
			} else {
				// hide them again
				this.annotationModel.hideTemporarilyAnnotations();
			}

		};

		/**
		 * Toggle whether the active/current page annotations should be shown temporarily
		 * @param {Boolean} [toggle]
		 */
		AnnotationModule.prototype.toggleCurrentPageAnnotations = function (toggle) {
			if (toggle === undefined) {
				toggle = !this.annotationModel.isTemporarilyCurrentPageAnnotationsShown();
			}
			if (toggle) {
				this.f.setOption("PREFERENCES_ANNOTATIONTOOLS_VISIBLE", 1, true);
				this.annotationModel.showTemporarilyActivePageAnnotations();
			} else {
				this.annotationModel.hideTemporarilyAnnotations();
			}
		};

		/**
		 * @override
		 * @returns {String}
		 */
		AnnotationModule.prototype.getUnsavedElementsName = function () {
			// note we don't wait for the bundle to be loaded... but it's not even possible in this situation
			// this language var should normally be loaded already...
			return this.f.i18n.t("Annotations");
		};


		/**
		 * Are annotations for the current document currently shown (annotations are temporarily disabled because of compare)
		 * @borrows {AnnotationModule.AnnotationModel~isTemporarilyCurrentPageAnnotationsShown}
		 * @returns {Boolean}
		 */
		AnnotationModule.prototype.isTemporarilyCurrentPageAnnotationsShown = function () {
			return this.annotationModel.isTemporarilyCurrentPageAnnotationsShown();
		};

		/**
		 * Are annotations for the difference document currently shown (annotations are temporarily disabled because of compare)
		 * @borrows {AnnotationModule.AnnotationModel~isTemporarilyDifferencePageAnnotationsShown}
		 * @returns {Boolean}
		 */
		AnnotationModule.prototype.isTemporarilyDifferencePageAnnotationsShown = function () {
			return this.annotationModel.isTemporarilyDifferencePageAnnotationsShown();
		};

		/**
		 * @override
		 */
		AnnotationModule.prototype.getState = function () {
			var state = {
				// Annotation model
				annotationModel: {
					temporarilyDisabled: {
						temporarilyDisabled: this.annotationModel.isTemporarilyDisabled(),
						temporarilyCurrentPageAnnotationsShown: this.annotationModel.isTemporarilyCurrentPageAnnotationsShown(),
						temporarilyDifferencePageAnnotationsShown: this.annotationModel.isTemporarilyDifferencePageAnnotationsShown(),
						temporarilyDisabledMessage: this.annotationModel.getTemporarilyDisabledMessage(),
						temporaryAnnotationDocumentTogglingEnabled: this.annotationModel.isTemporarilyAnnotationDocumentTogglingEnabled()
					},
					annotationVisibility: this.f.getOption("PREFERENCES_ANNOTATIONTOOLS_VISIBLE") == 1,
					filter: this.annotationModel.getAnnotationFilterModel().getFilterState()
				}
			};
			return state;
		};

		/**
		 * @override
		 * @param {Object} state
		 */
		AnnotationModule.prototype.setState = function (state) {
			this.f.whenModuleAvailable("annotation", function () {
				// This module is ready
				if (state !== undefined) {
					var annotationModelState = state.annotationModel;
					if (state.disableSelectTool) {
						this._selectTool = null;
						this._unpublishTool("annotationtool");
						this._indicatorTool = null;
						this._unpublishTool("annotationindicatortool");
					}
					// Restore Annotation model
					if (annotationModelState !== undefined) {
						var temporarilyDisabledState = annotationModelState.temporarilyDisabled;
						if (temporarilyDisabledState !== undefined) {
							this.annotationModel.temporarilyDisable(temporarilyDisabledState.temporarilyDisabled, temporarilyDisabledState.temporaryAnnotationDocumentTogglingEnabled, temporarilyDisabledState.temporarilyDisabledMessage);
							if (temporarilyDisabledState.temporarilyCurrentPageAnnotationsShown) {
								this.annotationModel.showTemporarilyActivePageAnnotations();
							} else if (temporarilyDisabledState.temporarilyDifferencePageAnnotationsShown) {
								this.annotationModel.showTemporarilyDifferencePageAnnotations();
							}
						}
						this.annotationModel.toggleVisibility(annotationModelState.annotationVisibility);

						if (annotationModelState.filter) {
							this.annotationModel.getAnnotationFilterModel().setFilterState(annotationModelState.filter);
						}
					}
				}
			}, this);
		};

		/**
		 * Prompt to save unsaved annotations if there are any
		 * if there are unsaved annotations, save them
		 *
		 * @param {Function} callbackOnOK - function which is called when the popup is closed and annotations are saved
		 * @param {Function} [callbackOnCancel] - if provided, this callback is called when the user presses cancel (if not defined, callbackOnOK is executed)
		 * @param {Function} [callbackOnDontSave] - adds the possibility to not save unsaved annotation but to continue with the action, instead cancal will cancel the action and dnon't save
		 * @returns {Boolean} true if there are unsaved items and the popup is shown, false if not
		 */
		AnnotationModule.prototype.promptToSaveUnsavedAnnotations = function (callbackOnOK, callbackOnCancel, callbackOnDontSave) {
			return this.annotationModel.promptToSaveUnsavedAnnotations(callbackOnOK, callbackOnCancel, callbackOnDontSave);
		};

		/**
		 * Check if the annotation is outside the currently active page, if this is the case prompt to check if the user still wants to add the annotation
		 * @param {ShapeUtils.Rect} rect
		 * @param {Function} callbackOnOk
		 * @param {Function} callbackOnCancel
		 * @param {Boolean} [isCreatingLinkedAnnot]
		 */
		AnnotationModule.prototype.promptIfAnnotationOutsidePage = function (rect, callbackOnOk, callbackOnCancel, isCreatingLinkedAnnot) {
			var page = this.f.getDocumentController().getActivePage();
			var isRectOnPage = true;
			if (!page.hasCad()) {
				isRectOnPage = page.isRectOnPage(rect);
			}

			//show modal popup with the question if the user wants to add an annotation that's not on the currently active page (if needed)
			if (!isRectOnPage) {
				var modalPopup = new ModalPopup(this.f, "annotationOnOtherPage", this.f.getContext());
				modalPopup.setRemoveFromDomAfterUsage(true);
				modalPopup.setOkAction(_.bind(function () {
					callbackOnOk(isCreatingLinkedAnnot);
					modalPopup.hide();
				}, this));

				modalPopup.setCancelAction(_.bind(function () {
					callbackOnCancel();
				}, this));

				// Header + Content
				modalPopup.setHeader($("<span/>").addClass("popupTitle").text(this.f.i18n.t("AnnSaveTitle")));
				modalPopup.setContent($("<div/>").addClass("unsavedItemsContent").text(this.f.i18n.t("AnnSaveOutsidePrompt")));
				modalPopup.setOkBtnText(this.f.i18n.t("YesResponse"));
				modalPopup.setCancelBtnText(this.f.i18n.t("NoResponse"));

				modalPopup.show();
			} else {
				callbackOnOk();
			}
		};

		/**
		 * Reload the annotations (throwing away unsaved actions!)
		 */
		AnnotationModule.prototype.reload = function () {
			this.annotationModel.reload(true);
		};

		/**
		 * Disable annotations with a given reason
		 * @param {Boolean} disable - disable or enable the annotations for given reason (key is used as id for the reason)
		 * @param {String} key - the key used in the reasons
		 * @param {String} disabledText - text used to display in the annotationWidget for which reason the annotations were disabled
		 */
		AnnotationModule.prototype.disableAnnotations = function (disable, key, disabledText) {
			if (disable) {
				this.annotationModel.setDisableReason(key, disabledText, true, true);
			} else {
				this.annotationModel.removeDisableReason(key);
			}
		};

		/**
		 * duplicates the annotation
		 * @param {Annotation} annotation - the annotation to duplicate
		 * @param {Boolean} keepReplies - True if you want to copy the replies to the duplicated annotation, false otherwise
		 * @param {Boolean} keepStates - True if you want to copy the states to the duplicated annotation, false otherwise
		 * @param {Boolean} keepInklist - True if you want to copy the pointArray to the duplicated annotation, false otherwise
		 * @param {Boolean} keepLinkedAnnotations - True if you want to copy the linked annotations to the duplicated annotation, false otherwise
		 * @param {Boolean} [duplicateWithoutText = false] - whether or not to remove the content of the annotation
		 * @param {Boolean} [silent = false] - whether or not to keep silent about the changes

		 */
		AnnotationModule.prototype._duplicateAnnotation = function (annotation, keepReplies, keepStates, keepInklist, keepLinkedAnnotations, duplicateWithoutText, silent) {

			var ann = annotation.getGodfather().duplicate(keepReplies, keepStates, keepInklist, keepLinkedAnnotations, this.annotationModel);
			if (duplicateWithoutText) {
				ann.content = "";
			}

			if (annotation.inreplyto) {
				ann.content = _.clone(annotation.content);
			}

			// fix the color of the duplicate (to get the current drawing color we refer to the draw tools)
			var color = null;
			if (ann.getType() == AnnotationConstants.AnnotationType.ANNOT_PIN) {
				color = this._noteDrawTool.getAnnotationColor();
			} else {
				color = this._drawTool.getAnnotationColor();
			}
			ann.setColor(color);

			this.annotationModel.unselectAllAnnotations(true, true);
			// add annotation so it gets a number
			this.annotationModel.addAnnotation(ann, false, silent);
			if (ann.links.length > 0) {
				// copy all the links
				_.each(ann.links, function (linkedAnnotation) {
					linkedAnnotation.messagenumber = ann.messagenumber;
					this.annotationModel.addAnnotation(linkedAnnotation);
				}, this);
			}
			// make sure the linked annotation are also in the correct color
			ann.setColor(color);
			if (!silent) {
				this.annotationModel.modelChanged(ann);
			}

		};

		return AnnotationModule;
	});
