
define('web-viewer/compare/CompareModel',["web-viewer/core/AbstractModel",
		"web-viewer/compare/CompareConstants",
		"utils/ShapeUtils",
		"underscore"
	],
	function (AbstractModel,
		CompareConstants,
		ShapeUtils,
		_) {
		"use strict";

		/**
		 * @memberof CompareModule
		 * @exports CompareModel
		 *
		 * @class The CompareModel class keeps track of the current compare mode and the needed settings
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Array.<String>} channel
		 */
		var CompareModel = function (f, channel, diffMode) {
			AbstractModel.call(this, f, channel);

			/**
			 * @member The currently activate compare mode
			 * @private
			 * @type CompareModule.CompareMode
			 */
			this._currentMode = null;

			/**
			 * @member The current DMode
			 * @private
			 * @type CompareModule.CompareConstants.diffMode
			 */
			this._diffMode = diffMode ? diffMode : CompareConstants.diffMode.UNDEFINED;

			/**
			 * @member The available compare modes
			 * @private
			 */
			this._availableModes = {};

			/**
			 * @member {ShapeUtils.Point} offset of the reference document page for compare
			 */
			this._offset = new ShapeUtils.Point(0, 0);

			/**
			 * @member {ShapeUtils.Point} offset of the cad overlay document page for compare
			 */
			this._cadOverlayOffset = new ShapeUtils.Point(0, 0);

			/**
			 * @member {CompareConstants.cornerAutoAlign}
			 */
			this._cornerAlign = CompareConstants.cornerAutoAlign.TOPLEFT;

			/**
			 * @member {Boolean} is the reference document page flipped
			 */
			this._referenceFlipped = false;

			/**
			 * @member {Boolean} is the cad overlay document page flipped
			 */
			this._cadOverlayFlipped = false;

			/**
			 * @member {Boolean} is the reference document page negated
			 */
			this._referenceNegated = false;

			/**
			 * @member {CompareConstants.referenceRotation}
			 */
			this._referenceRotation = CompareConstants.referenceRotation.NOTROTATED;

			/**
			 * @member {CompareConstants.referenceRotation}
			 */
			this._cadOverlayRotation = CompareConstants.referenceRotation.NOTROTATED;

			/**
			 * @member {Boolean} use trim box align
			 */
			this._trimBoxAlign = false;

			/**
			 * The name of the most recently activated (or activating) compare mode
			 */
			this._newModeName = null;


			/**
			 * @member {Boolean} used to delay the firing of view server parameters changed
			 */
			this._silenceParametersChanged = false;
			this._fireParametersChanged = false;
		};

		CompareModel.prototype = Object.create(AbstractModel.prototype);
		CompareModel.prototype.constructor = CompareModel;


		/**
		 * Change the current mode by name.
		 * SHOULD BE CALLED FROM THE MODULE ONLY
		 * @param {String} name - the name of the new mode
		 * @param {CompareModule.CompareConstants.diffMode} [diffMode] - The diff mode that the mode should activate
		 */
		CompareModel.prototype.activateModeByName = function (name, diffMode) {
			if (this._availableModes[name]) {

				var oldMode = this._currentMode;
				var newMode = this._availableModes[name];

				if (newMode !== oldMode) {
					this.toggleCurrentMode(false);

					this._newModeName = name;

					//activate the new mode
					newMode.toggle(true, diffMode);

					this._currentMode = newMode;

					this._fireChange(false, ["mode"], oldMode, newMode);
				}

			}
		};

		/**
		 * De-activate the current active mode
		 * SHOULD BE CALLED FROM THE MODULE ONLY
		 */
		CompareModel.prototype.deactivateCurrentMode = function () {
			this.toggleCurrentMode(false);
			if (this._currentMode) {
				var oldMode = this._currentMode;
				this._newModeName = "";
				this._currentMode = null;
				this._fireChange(false, ["mode"], oldMode, null);
			}
		};

		/**
		 * Toggle the current active mode
		 * SHOULD BE CALLED FROM THE MODULE ONLY
		 */
		CompareModel.prototype.toggleCurrentMode = function (toggle) {
			if (this._currentMode && this._currentMode.isEnabled() !== toggle) {
				this._currentMode.toggle(toggle);
			}
		};

		/**
		 * Get the mode name based on the given diff mode
		 * @param {CompareModule.CompareConstants.diffMode} diffMode - The diff mode of which you want to get the mode name
		 * @returns {String} diffMode
		 */
		CompareModel.prototype.getModeNameByDiffMode = function (diffMode) {
			// Get to the mode name based on the diffMode: first get the name of the diff mode
			var diffModeName = _.invert(CompareConstants.diffMode)[diffMode];
			// then use it in the diffModeToMode mapping to get the mode constant name
			var mode = CompareConstants.diffModeToMode[diffModeName];
			// and then get the value of the mode constant, meaning the mode name.
			return CompareConstants.mode[mode];
		};

		/**
		 * Get a Compare mode by name
		 * @param {String|CompareModule.CompareConstants.mode} name
		 * @returns {CompareModule.CompareMode}  the requested mode or undefined if the mode doesn't exist
		 */
		CompareModel.prototype.getModeByName = function (name) {
			return this._availableModes[name];
		};

		/**
		 * sets the offset of the reference document page for compare, set by alignment of the document page
		 * @param {ShapeUtils.Point} offset
		 */
		CompareModel.prototype.setOffset = function (offset) {
			var oldOffset = this._offset;

			this._offset = offset.copy();

			this._fireChange(false, ["offset"], oldOffset, offset);
		};

		/**
		 * Get the offset of the reference page for compare
		 * @returns {ShapeUtils.Point}
		 */
		CompareModel.prototype.getOffset = function () {
			return this._offset.copy();
		};

		/**
		 * reset the offset of the reference document page
		 */
		CompareModel.prototype.resetOffset = function () {
			this.setOffset(new ShapeUtils.Point(0, 0));
		};

		/**
		 * Set the offset of the cad overlay in the model
		 * @param {ShapeUtils.Point} offset
		 */
		CompareModel.prototype.setCadOverlayOffset = function (offset) {
			var oldOffset = this._cadOverlayOffset;

			this._cadOverlayOffset = offset.copy();

			this._fireChange(false, ["cadOverlayOffset"], oldOffset, offset);
		};

		/**
		 * Get the offset of the CAD overlay
		 * @returns {ShapeUtils.Point}
		 */
		CompareModel.prototype.getCadOverlayOffset = function () {
			return this._cadOverlayOffset.copy();
		};

		/**
		 * Reset the offset of the CAD overlay
		 */
		CompareModel.prototype.resetCadOverlayOffset = function () {
			this.setCadOverlayOffset(new ShapeUtils.Point(0, 0));
		};

		/**
		 * Set the corner alignment
		 * @param {CompareConstants.cornerAutoAlign} corner
		 */
		CompareModel.prototype.setCornerAlign = function (corner) {
			this._cornerAlign = corner;
		};

		/**
		 * Get the corner alignment
		 * @returns {CompareConstants.cornerAutoAlign}
		 */
		CompareModel.prototype.getCornerAlign = function () {
			return this._cornerAlign;
		};

		/**
		 * reset the corner alignment
		 */
		CompareModel.prototype.resetCornerAlign = function () {
			this._cornerAlign = CompareConstants.cornerAutoAlign.TOPLEFT;
		};

		/**
		 * Get the rotation of the reference document page
		 * @returns {CompareConstants.referenceRotation}
		 */
		CompareModel.prototype.getReferenceRotation = function () {
			return this._referenceRotation;
		};

		/**
		 * Set the rotation of the reference document page
		 * @param {CompareConstants.referenceRotation}
		 */
		CompareModel.prototype.setReferenceRotation = function (rotation) {
			this._referenceRotation = rotation;
		};

		/**
		 * Flip the reference document page
		 */
		CompareModel.prototype.flipReference = function () {
			this._referenceFlipped = !this._referenceFlipped;
		};

		/**
		 * Flip the CAD overlay
		 * @param {Boolean} [flip] - if undefined just flip from the current flip state, otherwhise if true : page is flipped, if false page is not flipped
		 */
		CompareModel.prototype.flipCadOverlay = function (flip) {
			if (flip === undefined) {
				flip = !this._cadOverlayFlipped;
			}
			this._cadOverlayFlipped = flip;
		};

		/**
		 * Is the reference document page flipped
		 * @returns {Boolean}
		 */
		CompareModel.prototype.isReferenceFlipped = function () {
			return this._referenceFlipped;
		};

		/**
		 * Is the CAD overlay flipped
		 * @returns {Boolean}
		 */
		CompareModel.prototype.isCadOverlayFlipped = function () {
			return this._cadOverlayFlipped;
		};

		/**
		 * negate the reference document page
		 */
		CompareModel.prototype.negateReference = function () {
			this._referenceNegated = !this._referenceNegated;
			this.parametersChanged();
		};

		/**
		 * Is the reference document page nagated
		 * @returns {Boolean}
		 */
		CompareModel.prototype.isReferenceNegated = function () {
			return this._referenceNegated;
		};

		/**
		 * Reset all the orientation options of the reference document page (flip, negation and rotation)
		 */
		CompareModel.prototype.resetReferenceOrientation = function () {
			this._referenceFlipped = false;
			this._referenceNegated = false;
			this._referenceRotation = CompareConstants.referenceRotation.NOTROTATED;
		};

		/**
		 * Reset the orientation options of the CAD overlay (this is the rotation and the flipped state)
		 */
		CompareModel.prototype.resetCadOverlayOrientation = function () {
			this._cadOverlayRotation = CompareConstants.referenceRotation.NOTROTATED;
			this._cadOverlayFlipped = false;
		};

		/**
		 * toggle the trim box align option
		 */
		CompareModel.prototype.toggleTrimBoxAlign = function () {
			this._trimBoxAlign = !(this._trimBoxAlign);
		};

		/**
		 * Is the trim box align option on
		 * @returns {Boolean}
		 */
		CompareModel.prototype.isTrimBoxAlign = function () {
			return this._trimBoxAlign;
		};

		/**
		 * Reset the trim box algin option (not active)
		 */
		CompareModel.prototype.resetTrimBoxAlign = function () {
			this._trimBoxAlign = false;
		};

		/**
		 * Set the dmode
		 * @param {CompareModule.CompareConstants.diffMode} dmode
		 */
		CompareModel.prototype.setDiffMode = function (dmode) {
			var oldMode = this._diffMode;
			this._diffMode = dmode;

			if (this._currentMode === null) {
				oldMode = CompareConstants.diffMode.UNDEFINED;
			}

			if (oldMode != dmode) {
				this.parametersChanged();
			}

			this._fireChange(false, ["diffMode"], oldMode, this._diffMode);
		};

		/**
		 * Get the configured dmode
		 * @returns {CompareModule.CompareConstants.diffMode}
		 */
		CompareModel.prototype.getDiffMode = function () {
			return this._diffMode;
		};

		/**
		 * Get the current active compare mode mode
		 * @returns {CompareModule.CompareMode} or undefined when no comparison is active
		 */
		CompareModel.prototype.getActiveMode = function () {
			return this._currentMode;
		};

		/**
		 * Register a new compare mode
		 * @param {CompareModule.CompareMode} mode
		 */
		CompareModel.prototype.registerMode = function (mode) {
			if (this._availableModes[mode.getName()]) {
				throw new Error("Trying to register mode which already exists!", mode.getName(), mode);
			}

			var oldList = _.keys(this._availableModes);
			this._availableModes[mode.getName()] = mode;

			this._fireChange(false, ["availableModes"], oldList, _.keys(this._availableModes));
		};

		/**
		 * Notify the viewport that parameters have changed
		 */
		CompareModel.prototype.parametersChanged = function () {
			if (this._silenceParametersChanged) {
				// fire it later
				this._fireParametersChanged = true;
			} else {
				this.f.emit("viewport.viewserverParametersChanged");
			}
		};

		/**
		 * Check if the given dmode is a "real" (not current or reference or readerspread diff mode) compare mode.
		 * @param {CompareModule.CompareConstants.diffMode} diffMode - The diff mode to check
		 * @returns {Boolean} True if the given dmode is a "real" compare mode, false otherwise.
		 */
		CompareModel.prototype.isRealCompareMode = function (diffMode) {
			return (diffMode >= CompareConstants.diffMode.SHOWBOTHCHANGES && diffMode <= CompareConstants.diffMode.SHOWBOTHSIDEBYSIDECHANGESONLY && diffMode !== CompareConstants.diffMode.READERSPREAD);
		};

		/**
		 * is the currently active compare mode read spread
		 * @returns {Boolean}
		 */
		CompareModel.prototype.isReaderSpreadActive = function () {
			return this._newModeName === CompareConstants.mode.READERSPREAD;
		};

		/**
		 * Silence parameters changed or not
		 * @param {Boolean} toggle - True to silence, false to unsilence
		 */
		CompareModel.prototype.toggleSilenceParametersChanged = function (toggle) {
			if (toggle) {
				this._silenceParametersChanged = true;
			} else {
				this._silenceParametersChanged = false;
				if (this._fireParametersChanged) {
					this.parametersChanged();
					this._fireParametersChanged = false;
				}
			}
		};

		/**
		 * Set the rotation of the CAD overlay
		 * @param {Number} rotation The rotation in degrees
		 */
		CompareModel.prototype.setCadOverlayRotation = function (rotation) {
			this._cadOverlayRotation = rotation;
		};

		/**
		 * Get the rotation of the CAD overlay
		 * @returns {Number} the rotation in degrees
		 */
		CompareModel.prototype.getCadOverlayRotation = function () {
			return this._cadOverlayRotation;
		};


		return CompareModel;
	});

define('web-viewer/compare/CompareMode',["web-viewer/core/AbstractModel",
		"underscore"
	],
	function (AbstractModel,
		_) {
		"use strict";

		/**
		 * @exports CompareMode
		 * @memberof CompareModule
		 * 
		 * @class CompareMode is a base class for compare modes
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {String} name - name of the compare mode
		 * @param {CompareModule.CompareModel} compareModel - the compare model
		 * @param {Boolean} enableTogglingAnnotations - Enable the toggling of annotation or not
		 * @param {String} annotationTooltipKey - Key (in the general bundle) of the reason message that should be shown when annotations are disabled
		 * @param {String} [whichToEnable] - Which of the show annotation buttons to enable?
		 */
		var CompareMode = function (f, name, compareModel, enableTogglingAnnotations, annotationTooltipKey, whichToEnable) {
			var channel = compareModel.getPrivateChannelNameAsArray();
			channel.push("mode");
			channel.push(name);
			AbstractModel.call(this, f, channel);
			this.f = f;
			this.name = name;
			this._enabled = false;
			this._compareModel = compareModel;

			this._enableTogglingAnnotations = enableTogglingAnnotations;
			this._annotationTooltipKey = annotationTooltipKey;
			this._whichToEnable = whichToEnable;

			this._annotationModuleAvailable = false;
			this.f.whenModuleAvailable("annotation", function (annotationModule) {
				this._annotationModuleAvailable = true;
				// by default annotation are shown, so only fire if not enabled
				if (this._enabled) {
					this._fireTemporarilyDisableAnnotations();
				}
			}, this);
		};

		CompareMode.prototype = Object.create(AbstractModel.prototype);
		CompareMode.prototype.constructor = CompareMode;

		/**
		 * Get the name of the mode
		 * @returns {String}
		 */
		CompareMode.prototype.getName = function () {
			return this.name;
		};

		/**
		 * Enable or disable the compare mode
		 * @param {Boolean} toggle 
		 * @param {CompareModule.CompareConstants.diffMode} [diffMode] - The diff mode in the mode that should be activated
		 */
		CompareMode.prototype.toggle = function (toggle, diffMode) {
			if (toggle !== this._enabled) {
				this._enabled = toggle;
				// enable/disable annotations
				if (this._annotationModuleAvailable) {
					this._fireTemporarilyDisableAnnotations();
				}
			}
		};

		/**
		 * Is the mode active or not?
		 * @returns {Boolean}
		 */
		CompareMode.prototype.isEnabled = function () {
			return this._enabled;
		};

		/**
		 * Intercept fragmentURL generation. inject the correct parameters
		 * @param context
		 */
		CompareMode.prototype.interceptFragmentURL = function (context) {
			// do nothing;
		};

		/**
		 * Fire the temporarily disable annotations options
		 */
		CompareMode.prototype._fireTemporarilyDisableAnnotations = function () {
			this.f.loadBundle("viewer", function () {
				this.f.emit("module.annotation.temporarilyDisable", this._enabled, this._enableTogglingAnnotations, this.f.t(this._annotationTooltipKey), this._whichToEnable);
			}, this);
		};

		return CompareMode;
	});

define('web-viewer/compare/CompareShowCurrentMode',["underscore",
		"web-viewer/compare/CompareMode",
		"web-viewer/compare/CompareConstants"
	],
	function (_,
		CompareMode,
		CompareConst) {
		"use strict";

		/**
		 * @memberof CompareModule
		 * @exports CompareShowCurrentMode
		 * @class Deadsimple compare mode just showing the current document
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Compare.CompareModel} compareModel - model for compare
		 */
		var CompareShowCurrentMode = function (f, compareModel) {
			CompareMode.call(this, f, CompareConst.mode.SHOW_CURRENT, compareModel, false, "", "current");
		};

		CompareShowCurrentMode.prototype = Object.create(CompareMode.prototype);
		CompareShowCurrentMode.prototype.constructor = CompareShowCurrentMode;

		/**
		 * Start/stop show compare mode
		 */
		CompareShowCurrentMode.prototype.toggle = function (toggle) {
			CompareMode.prototype.toggle.call(this, toggle);
			this._compareModel.setDiffMode(toggle ? CompareConst.diffMode.CURRENTVERSION : CompareConst.diffMode.UNDEFINED);
		};

		return CompareShowCurrentMode;
	});

define('web-viewer/compare/CompareShowDifferenceMode',["underscore",
		"web-viewer/compare/CompareMode",
		"web-viewer/compare/CompareConstants"
	],
	function (_,
		CompareMode,
		CompareConst) {
		"use strict";

		/**
		 * @memberof CompareModule
		 * @exports CompareShowDifferenceMode
		 * @class Deadsimple compare mode just showing the difference document
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {CompareModule.CompareModel} compareModel - model for compare
		 */
		var CompareShowDifferenceMode = function (f, compareModel) {
			CompareMode.call(this, f, CompareConst.mode.SHOW_DIFFERENCE, compareModel, false, "AnnDisabledDiffMode", "difference");
		};

		CompareShowDifferenceMode.prototype = Object.create(CompareMode.prototype);
		CompareShowDifferenceMode.prototype.constructor = CompareShowDifferenceMode;

		/**
		 * Start/stop show compare mode
		 * @override
		 */
		CompareShowDifferenceMode.prototype.toggle = function (toggle) {
			CompareMode.prototype.toggle.call(this, toggle);
			this._compareModel.setDiffMode(toggle ? CompareConst.diffMode.DIFFVERSION : CompareConst.diffMode.UNDEFINED);
		};

		return CompareShowDifferenceMode;
	});

define('web-viewer/compare/CompareBlinkingMode',["underscore",
		"web-viewer/compare/CompareMode",
		"web-viewer/compare/CompareConstants"
	],
	function (_,
		CompareMode,
		CompareConst) {
		"use strict";
		/**
		 * @memberof CompareModule
		 * @exports CompareBlinkingMode
		 * @class Compare two documents by switching continuously between the two documents 
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Compare.CompareModel} compareModel - model for compare
		 */
		var CompareBlinkingMode = function (f, compareModel) {
			CompareMode.call(this, f, CompareConst.mode.BLINKING, compareModel, false, "DiffBlinkingAnnotationsHidden");

			/**
			 * @private the interval for the switch between documents
			 */
			this._blinkInterval = CompareConst.blinkingSpeed;

			/**
			 * @private is the difference visible?
			 */
			this._differenceVisible = false;

			/**
			 * internal function used to execute the blinking mode
			 */
			this._toggleDifference = _.bind(function () {
				this._differenceVisible = !this._differenceVisible;
				// instruct the graphics module
				this._updateCompareModel();

				if (this.isEnabled()) {
					this._timer = setTimeout(this._toggleDifference, this._blinkInterval);
				}
			}, this);

		};

		CompareBlinkingMode.prototype = Object.create(CompareMode.prototype);
		CompareBlinkingMode.prototype.constructor = CompareBlinkingMode;

		/**
		 * Start/stop blinking mode
		 * @param {Boolean} [toggle] - should the mode be blinking or not, if undefined the mode will go to the other state than it is in
		 */
		CompareBlinkingMode.prototype.toggle = function (toggle) {
			CompareMode.prototype.toggle.call(this, toggle);

			if (toggle) {
				this._toggleDifference();
			} else {
				if (this._timer) {
					clearTimeout(this._timer);
				}
				this._differenceVisible = false;

				this._updateCompareModel();
			}
		};

		/**
		 * Update the state of the compare model
		 */
		CompareBlinkingMode.prototype._updateCompareModel = function () {
			var newDiffMode = this.isDifferenceVisible() ? CompareConst.diffMode.DIFFVERSION : CompareConst.diffMode.CURRENTVERSION;
			this._compareModel.setDiffMode(newDiffMode);
		};


		/**
		 * Is the difference visible
		 * @returns {Boolean}
		 */
		CompareBlinkingMode.prototype.isDifferenceVisible = function () {
			return this._differenceVisible;
		};


		/**
		 * Is the current visible
		 * @returns
		 */
		CompareBlinkingMode.prototype.isCurrentVisible = function () {
			return !this._differenceVisible;
		};


		return CompareBlinkingMode;
	});

define('web-viewer/compare/CompareDifferenceMode',["underscore",
		"web-viewer/compare/CompareMode",
		"web-viewer/compare/CompareConstants",
		"web-viewer/utils/ViewerUtils"
	],
	function (_,
		CompareMode,
		CompareConst,
		ViewerUtils) {
		"use strict";

		/**
		 * @memberof CompareModule
		 * @exports CompareDifferenceMode
		 * @class Compare two documents in difference compare (two colors for added and removed parts)
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Compare.CompareModel} compareModel - model for compare
		 */
		var CompareDifferenceMode = function (f, compareModel) {
			CompareMode.call(this, f, CompareConst.mode.DIFFERENCE, compareModel, true, "DiffDifferenceAnnotationsHidden");

			this.threshold = 0;
			this.setThreshold(parseInt(this.f.getCurrentUser().getOption("PREFERENCES_DIFFMODE_THRESHOLD"), 10));
			
			
			this.f.getCurrentUser().on("options.changed.PREFERENCES_DIFFMODE_THRESHOLD", _.bind(function (oldValue, newValue) {
				// not newValue for translations to hex
				this.setThreshold(newValue);
			}, this));
			
			//the setting is the same as in the javaviewer:
			// 0=DEFAULT, 1=CHANNELS, 2=HSV, 3=AUTOMATIC
			this.colorSpace = this.f.getOption("CONFIG_DIFF_COMPAREMODE");

			this.differenceMode = compareModel.getDiffMode();

			if (this.differenceMode < CompareConst.diffMode.SHOWBOTHCHANGES || this.differenceMode > CompareConst.diffMode.SHOWBOTHCHANGESONLY) {
				var diffMode = CompareConst.sideBySideToDiffMode[this.differenceMode];
				if (diffMode === undefined) {
					this.differenceMode = CompareConst.diffMode.SHOWBOTHCHANGES;
				} else {
					this.differenceMode = diffMode;
				}
			}

			// hex value with #
			this.addedHighlightColor = this.f.getCurrentUser().getColor("PREFERENCES_HIGHLIGHTCOLORS_DIFF_ADD");
			this.f.getCurrentUser().on("options.changed.PREFERENCES_HIGHLIGHTCOLORS_DIFF_ADD", _.bind(function (oldValue, newValue) {
				// not newValue for translations to hex
				this.setAddedHighlightColor(this.f.getCurrentUser().getColor("PREFERENCES_HIGHLIGHTCOLORS_DIFF_ADD"));
			}, this));

			// hex value with #
			this.subtractedHighlightColor = this.f.getCurrentUser().getColor("PREFERENCES_HIGHLIGHTCOLORS_DIFF_SUB");
			this.f.getCurrentUser().on("options.changed.PREFERENCES_HIGHLIGHTCOLORS_DIFF_SUB", _.bind(function (oldValue, newValue) {
				// not newValue for translations to hex
				this.setSubtractedHighlightColor(this.f.getCurrentUser().getColor("PREFERENCES_HIGHLIGHTCOLORS_DIFF_SUB"));
			}, this));

			this.on("changed.**", function () {
				compareModel.parametersChanged();
			});
		};

		CompareDifferenceMode.prototype = Object.create(CompareMode.prototype);
		CompareDifferenceMode.prototype.constructor = CompareDifferenceMode;

		/**
		 * Start/stop show compare mode
		 */
		CompareDifferenceMode.prototype.toggle = function (toggle, diffMode) {
			CompareMode.prototype.toggle.call(this, toggle);
			if (toggle) {
				if (diffMode != undefined &&
					(diffMode === CompareConst.diffMode.SHOWBOTHCHANGES || diffMode === CompareConst.diffMode.SHOWBOTHCHANGESHIGHLIGHT || diffMode === CompareConst.diffMode.SHOWBOTHCHANGESONLY || diffMode === CompareConst.diffMode.SHOWBOTHONTOP)) {
					// we only allow diffModes of this mode
					this.setDifferenceMode(diffMode);
				} else {
					this._compareModel.setDiffMode(this.differenceMode);
				}
			} else {
				this._compareModel.setDiffMode(CompareConst.diffMode.UNDEFINED);
			}
		};

		/**
		 * get the DType parameter for the viewserver
		 */
		CompareDifferenceMode.prototype._getDType = function () {
			// the DType parameter is formed with diffHsvMode, diffEnhanche and diffChannelsMode (but we use default value for those: 0)
			// original formula: diffHsvMode*256 + diffEnhance*64 + colorSpace*8 + diffChannelsMode
			// we only use colorspace setting

			return this.colorSpace * 8;
		};

		/**
		 * Intercept fragment url generation
		 * @param {Object} context
		 */
		CompareDifferenceMode.prototype.interceptFragmentURL = function (context) {
			context.params = _.extend(context.params, this.getOverwriteOptions());
		};

		/**
		 * Get the overwrite options for fragmentURL generation
		 * @returns {Object} object with the parameters which should be set
		 */
		CompareDifferenceMode.prototype.getOverwriteOptions = function () {
			return {
				dtype: this._getDType(),
				dlevel: this.threshold,
				hc0: ViewerUtils.rgbIntToBgrInt(ViewerUtils.intFromHex(this.addedHighlightColor)), // the viewer want it in BGR
				hc1: ViewerUtils.rgbIntToBgrInt(ViewerUtils.intFromHex(this.subtractedHighlightColor)) // the viewer want it in BGR
			};
		};
		/**
		 * Set a new threshold for difference compare
		 * @param {Number} newThreshold - number between 0 and 100
		 */
		CompareDifferenceMode.prototype.setThreshold = function (newThreshold) {
			var oldThreshold = this.threshold;			
			
			this.threshold = newThreshold;
			if  (isNaN(this.threshold)){
				this.threshold = 0;
			}
			this.threshold = Math.min(Math.max(this.threshold, 0), 100);
			
			this.f.setOption("PREFERENCES_DIFFMODE_THRESHOLD", this.threshold);
			this._fireChange(false, ["threshold"], oldThreshold, newThreshold);
		};

		/**
		 * Get the currently used threshold
		 * @returns {Number} threshold
		 */
		CompareDifferenceMode.prototype.getThreshold = function () {
			return this.threshold;
		};

		/**
		 * Set the colorspace 
		 * @param {CompareModule.CompareConstants.colorSpace} newColorspace
		 */
		CompareDifferenceMode.prototype.setColorSpace = function (newColorspace) {
			var oldColorspace = this.colorSpace;
			this.colorSpace = newColorspace;
			this._fireChange(false, ["colorspace"], oldColorspace, newColorspace);
			//the setting is set the same way as in the java viewer, so making sure the setting is the same
			// 0=default, 1=channels, 2=hsv, 3=automatic (using the 4th and 5th bit)
			this.f.setOption("CONFIG_DIFF_COMPAREMODE", newColorspace);
		};

		/**
		 * Get the currently used colorspace
		 * @param {CompareModule.CompareConstants.colorSpace} colorspace
		 */
		CompareDifferenceMode.prototype.getColorSpace = function () {
			return this.colorSpace;
		};

		/**
		 * Set the difference mode
		 * @param {CompareModule.CompareConstants.showDifferenceMode} newDifferenceMode
		 */
		CompareDifferenceMode.prototype.setDifferenceMode = function (newDifferenceMode) {
			var oldDifferenceMode = this.differenceMode;
			this.differenceMode = newDifferenceMode;

			if (this.isEnabled()) {
				// only update the compare mode if active...
				this._compareModel.setDiffMode(this.differenceMode);
			}
			this._fireChange(false, ["differenceMode"], oldDifferenceMode, newDifferenceMode);
		};

		/**
		 * Get the difference mode
		 * @returns {CompareModule.CompareConstants.showDifferenceMode}
		 */
		CompareDifferenceMode.prototype.getDifferenceMode = function () {
			return this.differenceMode;
		};

		/**
		 * Set the added highlight color
		 * @param {String} addedHighlightColor - hex value with #
		 */
		CompareDifferenceMode.prototype.setAddedHighlightColor = function (addedHighlightColor) {
			var oldAddedHighlightColor = this.addedHighlightColor;
			this.f.getCurrentUser().setColor("PREFERENCES_HIGHLIGHTCOLORS_DIFF_ADD", addedHighlightColor);
			this.addedHighlightColor = this.f.getCurrentUser().getColor("PREFERENCES_HIGHLIGHTCOLORS_DIFF_ADD");
			this._fireChange(false, ["addedHighlightColor"], oldAddedHighlightColor, this.addedHighlightColor);
		};

		/**
		 * Get the added highlight color
		 * @returns {String} hex value with #
		 */
		CompareDifferenceMode.prototype.getAddedHighlightColor = function () {
			return this.addedHighlightColor;
		};

		/**
		 * Set the subtracted highlight color
		 * @param {String} subtractedHighlightColor - hex value with #
		 */
		CompareDifferenceMode.prototype.setSubtractedHighlightColor = function (subtractedHighlightColor) {
			var oldSubtractedHighlightColor = this.subtractedHighlightColor;
			this.f.getCurrentUser().setColor("PREFERENCES_HIGHLIGHTCOLORS_DIFF_SUB", subtractedHighlightColor);
			this.subtractedHighlightColor = this.f.getCurrentUser().getColor("PREFERENCES_HIGHLIGHTCOLORS_DIFF_SUB");
			this._fireChange(false, ["subtractedHighlightColor"], oldSubtractedHighlightColor, this.subtractedHighlightColor);
		};

		/**
		 * Get the subtracted highlight color
		 * @returns {String} hex value with #
		 */
		CompareDifferenceMode.prototype.getSubtractedHighlightColor = function () {
			return this.subtractedHighlightColor;
		};

		/**
		 * Reset highlight colors
		 */
		CompareDifferenceMode.prototype.resetHighlightColors = function () {
			this.f.getCurrentUser().resetOption("PREFERENCES_HIGHLIGHTCOLORS_DIFF_ADD");
			this.f.getCurrentUser().resetOption("PREFERENCES_HIGHLIGHTCOLORS_DIFF_SUB");
			// it is automatically fired back to setAddedHighlightColor and setSubtractedHighlightColor
		};

		return CompareDifferenceMode;
	});

define('web-viewer/compare/CompareSideBySideMode',["underscore",
		"web-viewer/compare/CompareMode",
		"web-viewer/compare/CompareConstants",
		"web-viewer/viewport/ViewportConstants",
		"web-viewer/utils/ListenerUtil"
	],
	function (_,
		CompareMode,
		CompareConst,
		ViewportConstants,
		ListenerUtil) {
		"use strict";

		/**
		 * @memberof CompareModule
		 * @exports CompareSideBySideMode
		 * @class This mode shows a side by side view of the current and reference document. It enables the user to show a difference panel too (therefor it relies on compareShowDifferenceMode)
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {CompareModule.CompareModel} compareModel - model for compare
		 * @param {CompareModule.CompareDifferenceMode} differenceMode - the difference mode implementation
		 */
		var CompareSideBySideMode = function (f, compareModel, differenceMode) {
			CompareMode.call(this, f, CompareConst.mode.SIDEBYSIDE, compareModel, true, "DiffSideBySideAnnotationsHidden");
			ListenerUtil.mixin(this, f);

			this.differenceCompareMode = differenceMode;

			/**
			 * Is the difference compare panel shown?
			 * @private
			 */
			this._showDifference = true;

			// is the mode already listening to the graphics model
			this._isListeningToGraphics = false;

			/**
			 * How are the panels layouted
			 * @private
			 */
			this._panelsLayoutMode = ViewportConstants.PanelsLayoutMode.AUTOMATIC;

			// load the user preference
			this.updateSettingsFromUserPreference(this.f.getOption("PREFERENCES_DIFF_SPLITMODE"));

			this.currentFilePanel = null;
			this.differenceFilePanel = null;
			this.differenceModePanel = null;

			this.viewport = this.f.getModule("viewport");

			f.getCurrentUser().on("options.changed.PREFERENCES_DIFF_SPLITMODE", _.bind(function (oldValue, newValue) {
				this.updateSettingsFromUserPreference(newValue);
			}, this));

		};

		CompareSideBySideMode.prototype = Object.create(CompareMode.prototype);
		CompareSideBySideMode.prototype.constructor = CompareSideBySideMode;

		/**
		 * Start/stop show compare mode
		 * @param {Boolean} toggle 
		 * @override
		 */
		CompareSideBySideMode.prototype.toggle = function (toggle, diffMode) {
			if (!this.isEnabled() && toggle) {
				CompareMode.prototype.toggle.call(this, toggle);

				if (diffMode === undefined) {
					diffMode = this.differenceCompareMode.getDifferenceMode();
					diffMode = parseInt(_.invert(CompareConst.sideBySideToDiffMode)[diffMode]);
				}

				// when enabled, set the diff mode and add the needed panels
				this._compareModel.setDiffMode(diffMode);

				this.f.getDocumentController().whenParametersAvailable(function () {
					if (this.isEnabled()) {
						if (!this.graphicsModule) {
							this.graphicsModule = this.f.getModule("graphics");
						}



						// set panel layout mode
						this.viewport.setPanelsLayoutMode(this._panelsLayoutMode);

						// get a reference to the default panel
						this.currentFilePanel = this.viewport.getPanelModel("default");
						this.currentFilePanel.toggleLayer("referenceCad", false);
						this.currentFilePanel.toggleLayer("currentDiffCad", false);
						this.currentFilePanel.toggleLayer("referenceDiffCad", false);


						// add difference file panel
						this.differenceFilePanel = this.viewport.addPanel("compare_difference", this._getDifferenceFileOverwriteOptions(), ["Cad", "currentDiffCad", "referenceDiffCad"]);
						// show difference compare mode panel if needed
						this.toggleDifferencePanel(this._showDifference);

						// update the overwrite options for all panels
						this.updateOverwriteOptions();

						// make sure to react to channel changes to update the overwrite options
						this.f.whenModuleAvailable("graphics", function (module) {
							this.graphicsModule = module;

							if (this.isEnabled() && !this._isListeningToGraphics) {
								this._registerListener("model.changed.channels", _.debounce(_.bind(this.updateOverwriteOptions, this), 40), undefined, this.graphicsModule);
								this._registerListener("model.changed.channel.**", _.debounce(_.bind(this.updateOverwriteOptions, this), 40), undefined, this.graphicsModule);
								this._isListeningToGraphics = true;
								this.updateOverwriteOptions();
							}
						}, this);


						this._registerListener("changed.**", this.updateOverwriteOptions, this, this.differenceCompareMode);
						this._registerListener("annotationModel.annotation.temporarilyDisableChanged", this.updateOverwriteOptions, this);
						this._registerListener("doc.page.active", this.updatePanelHeaders, this);
						this._registerListener("doc.page.difference", this.updatePanelHeaders, this);
						this.updatePanelHeaders();
					}
				}, this);
			} else if (this.isEnabled() && !toggle) {
				CompareMode.prototype.toggle.call(this, toggle);

				// remove panels
				this.viewport.removePanel("compare_difference");
				this.viewport.removePanel("compare_differenceMode");

				// the side by side compare is disabled, restore everything and remove the previously created panels
				this._compareModel.setDiffMode(CompareConst.diffMode.UNDEFINED);

				// reset overwrite options
				this.currentFilePanel.deleteOverwriteOption("dmode");
				this.currentFilePanel.deleteOverwriteOption("colorBITWISE");
				this.currentFilePanel.toggleLayer("Annotations", true);
				this.currentFilePanel.toggleLayer("AnnotationPopups", true);
				this.currentFilePanel.toggleLayer("referenceCad", true);
				this.currentFilePanel.toggleLayer("currentDiffCad", true);
				this.currentFilePanel.toggleLayer("referenceDiffCad", true);
				this.currentFilePanel.setHeaderText(null);

				// reset references
				this.currentFilePanel = null;
				this.differenceFilePanel = null;
				this.differenceModePanel = null;

				// set panel layout to automatic
				this.viewport.setPanelsLayoutMode(ViewportConstants.PanelsLayoutMode.AUTOMATIC);

				// unregister all listeners
				this._unregisterAllListeners();
				this._isListeningToGraphics = false;
			}
		};

		/**
		 * Is the side by side mode showing the difference compare mode panel
		 * @returns {Boolean}
		 */
		CompareSideBySideMode.prototype.isShowingDifference = function () {
			return this._showDifference;
		};

		/**
		 * Show/hide the difference compare mode panel
		 * @param {Boolean} toggle
		 */
		CompareSideBySideMode.prototype.setShowingDifference = function (toggle) {
			var oldValue = this._showDifference;
			this._showDifference = toggle;

			if (this.isEnabled() && oldValue !== toggle) {
				// if the mode is enabled, show/hide the panel if needed
				this.toggleDifferencePanel(toggle);
			}

			this.storeUserPreference();
			this._fireChange(false, ["showDifference"], oldValue, toggle);
		};

		/**
		 * Add/remove the difference compare mode panel
		 * @param {Boolean} toggle
		 */
		CompareSideBySideMode.prototype.toggleDifferencePanel = function (toggle) {
			if (toggle) {
				this.differenceModePanel = this.viewport.addPanel("compare_differenceMode", this._getDifferenceModeOverwriteOptions(), ["Annotations", "AnnotationPopups", "Cad", "referenceCad"]);
				this.f.loadBundle("viewer", function () {
					this.differenceModePanel.setHeaderText(this.f.t("CaptionCompared"));
				}, this);

			} else {
				this.viewport.removePanel("compare_differenceMode");
				this.differenceModePanel = null;
			}

			// update the overwrite options
			this.updateOverwriteOptions();
		};

		/**
		 * Configure the overwrite options for all created panels
		 */
		CompareSideBySideMode.prototype.updateOverwriteOptions = function () {
			if (this.isEnabled()) {
				if (this.graphicsModule === undefined) {
					this.graphicsModule = this.f.getModule("graphics");
				}
				if (this.annotationModule === undefined) {
					this.annotationModule = this.f.getModule("annotation");
				}

				this.currentFilePanel.setOverwriteOptions(this._getCurrentFileOverwriteOptions());



				this.differenceFilePanel.setOverwriteOptions(this._getDifferenceFileOverwriteOptions());
				if (this.differenceModePanel !== null) {
					this.differenceModePanel.setOverwriteOptions(this._getDifferenceModeOverwriteOptions());
				}



				if (this.annotationModule !== undefined) {
					this.currentFilePanel.toggleLayer("Annotations", this.annotationModule.isTemporarilyCurrentPageAnnotationsShown());
					this.currentFilePanel.toggleLayer("AnnotationPopups", this.annotationModule.isTemporarilyCurrentPageAnnotationsShown());
					this.differenceFilePanel.toggleLayer("Annotations", this.annotationModule.isTemporarilyDifferencePageAnnotationsShown());
					this.differenceFilePanel.toggleLayer("AnnotationPopups", this.annotationModule.isTemporarilyDifferencePageAnnotationsShown());
				}
			}
		};

		/**
		 * Generate the overwrite options for the current file panel
		 * @returns {Object}
		 */
		CompareSideBySideMode.prototype._getCurrentFileOverwriteOptions = function () {
			var colormask = 0;
			if (this.graphicsModule !== undefined) {
				colormask = this.graphicsModule.getChannelSelectionMaskForDiffMode(CompareConst.diffMode.CURRENTVERSION);
			}

			return {
				dmode: CompareConst.diffMode.CURRENTVERSION,
				colorBITWISE: colormask
			};
		};

		/**
		 * Generate the overwrite options for the difference file panel
		 * @returns {Object}
		 */
		CompareSideBySideMode.prototype._getDifferenceFileOverwriteOptions = function () {
			var colormask = 0;
			if (this.graphicsModule !== undefined) {
				colormask = this.graphicsModule.getChannelSelectionMaskForDiffMode(CompareConst.diffMode.DIFFVERSION);
			}

			return {
				dmode: CompareConst.diffMode.DIFFVERSION,
				colorBITWISE: colormask,
				isDefaultView: false
			};
		};

		/**
		 * Generate the overwrite options for the difference compare mode panel
		 * @returns {Object}
		 */
		CompareSideBySideMode.prototype._getDifferenceModeOverwriteOptions = function () {
			var colormask = 0;
			if (this.graphicsModule !== undefined) {
				colormask = this.graphicsModule.getChannelSelectionMaskForDiffMode(this.differenceCompareMode.getDifferenceMode());
			}

			return _.extend(this.differenceCompareMode.getOverwriteOptions(), {
				dmode: this.differenceCompareMode.getDifferenceMode(),
				colorBITWISE: colormask,
				isDefaultView: false
			});
		};

		/**
		 * Restore the settings of the compare mode from the user preferences 
		 * @param {Number} value - some magical bitmasked number :) (see below)
		 */
		CompareSideBySideMode.prototype.updateSettingsFromUserPreference = function (value) {
			// the option is build up as follows"
			// value & 3 --> gives the panel layout mode (0=auto, 1=vertical, 2= horizontal)
			// value & 4 --> determines whether the differencecompare is shown or not

			this.setPanelsLayoutMode(+value & 3); // the + makes it a number
			this.setShowingDifference((value & 4) === 4);
		};

		/**
		 * Save the user preference
		 */
		CompareSideBySideMode.prototype.storeUserPreference = function () {
			var value = this.getPanelsLayoutMode();
			if (this.isShowingDifference()) {
				value += 4;
			}
			this.f.setOption("PREFERENCES_DIFF_SPLITMODE", value);
		};

		/**
		 * Change the panels layoutmode 
		 * @param {ViewportConstants.PanelsLayoutMode} mode
		 */
		CompareSideBySideMode.prototype.setPanelsLayoutMode = function (mode) {
			var oldValue = this._panelsLayoutMode;
			this._panelsLayoutMode = mode;
			if (this.isEnabled()) {
				// if side by side is enabled enabled, also update the viewport
				this.viewport.setPanelsLayoutMode(mode);
			}
			this.storeUserPreference();
			this._fireChange(false, ["panelLayout"], oldValue, this._panelsLayoutMode);
		};

		/**
		 * Get the currently configure panels layout mode for side by side compare
		 * @returns {ViewportConstants.PanelsLayoutMode}
		 */
		CompareSideBySideMode.prototype.getPanelsLayoutMode = function () {
			return this._panelsLayoutMode;
		};

		/**
		 * Update the header text for the panels
		 */
		CompareSideBySideMode.prototype.updatePanelHeaders = function () {
			this.f.loadBundle("viewer", function () {
				if (this.isEnabled()) {
					var current = "";
					var difference = "";

					var docController = this.f.getDocumentController();
					var activePage = docController.getActivePage();
					var activeDocVersion = activePage.getDocumentVersion();
					var differencePage = docController.getDifferencePage();
					var differenceDocVersion = differencePage.getDocumentVersion();

					if (activeDocVersion.getDocument() !== differenceDocVersion.getDocument()) {
						// we need to add document info too
						current += activeDocVersion.getDocument().get("docname") + ", ";
						difference += differenceDocVersion.getDocument().get("docname") + ", ";
					}

					current += this.f.t("WCHistVersion") + " " + activeDocVersion.getVersionId();
					difference += this.f.t("WCHistVersion") + " " + differenceDocVersion.getVersionId();



					current += " - (" + this.f.t("CaptionCurrent") + ")";
					difference += " - (" + this.f.t("CaptionPrevious") + ")";

					this.currentFilePanel.setHeaderText(current);
					this.differenceFilePanel.setHeaderText(difference);
				}
			}, this);
		};

		/**
		 * Set the dmode parameter correctly
		 * @param {Object} c intercept context
		 */
		CompareSideBySideMode.prototype.interceptFragmentURL = function (c) {
			c.params.dmode = CompareConst.diffMode.CURRENTVERSION;
		};


		return CompareSideBySideMode;
	});

define('web-viewer/compare/CompareShowBothOnTopMode',["underscore",
		"web-viewer/compare/CompareMode",
		"web-viewer/compare/CompareConstants"
	],
	function (_,
		CompareMode,
		CompareConst) {
		"use strict";

		/**
		 * @memberof CompareModule
		 * @exports CompareShowBothOnTopMode
		 * @class Deadsimple compare mode that shows the two documents on top of eachother
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Compare.CompareModel} compareModel - model for compare
		 */
		var CompareShowBothOnTopMode = function (f, compareModel) {
			CompareMode.call(this, f, CompareConst.mode.SHOWBOTHONTOP, compareModel, true, "DiffShowBothOnTopAnnotationsHidden");
		};

		CompareShowBothOnTopMode.prototype = Object.create(CompareMode.prototype);
		CompareShowBothOnTopMode.prototype.constructor = CompareShowBothOnTopMode;

		/**
		 * Start/stop compare mode
		 */
		CompareShowBothOnTopMode.prototype.toggle = function (toggle) {
			CompareMode.prototype.toggle.call(this, toggle);
			this._compareModel.setDiffMode(toggle ? CompareConst.diffMode.SHOWBOTHONTOP : CompareConst.diffMode.UNDEFINED);
		};

		return CompareShowBothOnTopMode;
	});

define('web-viewer/core/ViewerMode',["underscore"],
	function (_) {
		/**
		 * @class ViewerMode - A ViewerMode is a modal modus which gets control over the viewer. Only 1 viewermode can be activate a certain point in time 
		 * @export ViewerMode
		 * @memberof Core
		 * 
		 * @param {Core.ViewerFacade} f 
		 * @param {String} name - the name of the mode
		 */
		var ViewerMode = function (f, name) {
			this.f = f;
			this._name = name;

			this._active = false;
		};

		/**
		 * Start or stop this mode
		 * 
		 * NOTE!! this should only be called by the ViewerModeManager
		 * @param toggle
		 */
		ViewerMode.prototype.toggle = function (toggle) {
			// the mode is being started
			if (!this._active && toggle) {
				this._active = true;

				this._doStart();
			} else if (this._active && !toggle) {
				this._active = false;

				this._doStop();
			}
		};

		/**
		 * Start the mode
		 */
		ViewerMode.prototype._doStart = function () {
			// implemented by the specific mode: execute all actions on start
		};

		/**
		 * Stop the mode
		 */
		ViewerMode.prototype._doStop = function () {
			// implemented by the specific mode: execute all actions on stop (cleanup!)
		};


		/**
		 * Check if the mode is active
		 * @returns {Boolean}
		 */
		ViewerMode.prototype.isActive = function () {
			return this._active;
		};



		/**
		 * Request the modemanager to stop this mode
		 */
		ViewerMode.prototype.requestStop = function () {
			if (this._active) {
				this.f.stopMode(this);
			}
		};

		/**
		 * Allow all tools in the system
		 */
		ViewerMode.prototype._allowAllTools = function () {
			this.f.emit("toolManager.request.allowAllTools");
		};

		/**
		 * Allow all tools to be used, except for the tools listed in disallowedTools
		 * @param {Array.<String>} [disallowedTools] - an array of tools that should be disallowed
		 */

		ViewerMode.prototype._allowAllToolsExcept = function (disallowedTools) {
			this.f.emit("toolManager.request.allowAllToolsExcept", disallowedTools);
		};

		/**
		 * Disallow all tools, except for the tools lited in allowedTools
		 * @param {Array.<String>} [allowedTools] - an array of tools that are still allowed
		 */
		ViewerMode.prototype._disallowAllToolsExcept = function (allowedTools) {
			this.f.emit("toolManager.request.disallowAllToolsExcept", allowedTools);
		};

		return ViewerMode;
	});

define('web-viewer/compare/alignment/CompareAlignmentMode',["underscore",
		"web-viewer/compare/CompareMode",
		"web-viewer/compare/CompareConstants",
		"web-viewer/viewport/ViewportConstants",
		"web-viewer/utils/ListenerUtil",
		"utils/ShapeUtils"
	],
	function (_,
		CompareMode,
		CompareConst,
		ViewportConstants,
		ListenerUtil,
		ShapeUtils) {
		"use strict";

		/**
		 * @memberof CompareModule
		 * @exports CompareAlignmentMode
		 * @class This mode shows a side by side view of the current and reference document. It enables the user to quickly align two files. (the result of alignment is shown on the right)
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {CompareModule.CompareModel} compareModel - model for compare
		 * @param {CompareModule.CompareAlignmentAnchorLayerModel} currentAnchorLayerModel - the layer model which displays the current anchor
		 * @param {CompareModule.CompareAlignmentAnchorLayerModel} referenceAnchorLayerModel - the layer model which displays the reference anchor
		 */
		var CompareAlignmentMode = function (f, compareModel, currentAnchorLayerModel, referenceAnchorLayerModel) {
			CompareMode.call(this, f, CompareConst.mode.ALIGNMENT, compareModel, false, "DiffSideBySideAnnotationsHidden");
			ListenerUtil.mixin(this, f);

			// is the mode already listening to the graphics model
			this._isListeningToGraphics = false;

			this._currentFilePanel = null;
			this._differenceFilePanel = null;
			this._resultPanel = null;

			this._currentAnchorLayerModel = currentAnchorLayerModel;
			this._referenceAnchorLayerModel = referenceAnchorLayerModel;

			this._forCadOverlay = false;

			this.viewport = this.f.getModule("viewport");
		};

		CompareAlignmentMode.prototype = Object.create(CompareMode.prototype);
		CompareAlignmentMode.prototype.constructor = CompareAlignmentMode;

		/**
		 * Start/stop show compare mode
		 * @param {Boolean} toggle
		 * @override
		 */
		CompareAlignmentMode.prototype.toggle = function (toggle) {
			if (!this.isEnabled() && toggle) {
				CompareMode.prototype.toggle.call(this, toggle);

				// add layers for drawing the anchors
				this.viewport.addLayer("compare_alignment_currentAnchor", this._currentAnchorLayerModel);
				this.viewport.addLayer("compare_alignment_referenceAnchor", this._referenceAnchorLayerModel);

				if (!this.graphicsModule) {
					this.graphicsModule = this.f.getModule("graphics");
				}

				// when enabled, set the diff mode and add the needed panels
				this._compareModel.setDiffMode(CompareConst.diffMode.SHOWBOTHSIDEBYSIDEHIGHLIGHT);

				// set panel layout mode
				this.viewport.setPanelsLayoutMode(ViewportConstants.PanelsLayoutMode.VERTICAL);

				// get a reference to the default panel
				this._currentFilePanel = this.viewport.getPanelModel("default");
				this._currentFilePanel.toggleLayer("referenceCad", false);
				this._currentFilePanel.toggleLayer("currentDiffCad", false);
				this._currentFilePanel.toggleLayer("referenceDiffCad", false);
				this._currentFilePanel.toggleLayer("Annotations", false);
				this._currentFilePanel.toggleLayer("AnnotationPopups", false);
				this._currentFilePanel.toggleLayer("compare_alignment_referenceAnchor", false);
				this._currentFilePanel.toggleLayer("CadOverlay", !this._forCadOverlay);


				// add difference panel
				var disallowedLayersForDifferencePanel = ["Cad",
					"currentDiffCad",
					"referenceDiffCad",
					"Annotations",
					"AnnotationPopups",
					"compare_alignment_currentAnchor"
				];

				if (!this._forCadOverlay) {
					disallowedLayersForDifferencePanel.push("CadOverlay");
				}

				this._differenceFilePanel = this.viewport.addPanel("compare_difference", this._getDifferenceFileOverwriteOptions(), disallowedLayersForDifferencePanel);
				// show the result panel
				this._resultPanel = this.viewport.addPanel("compare_align_result", this._getResultOverwriteOptions(), ["Annotations",
					"AnnotationPopups",
					"Cad",
					"referenceCad",
					"compare_alignment_referenceAnchor"
				]);

				this.updatePanelHeaders();

				// update the overwrite options for all panels
				this.updateOverwriteOptions();

				// make sure to react to channel changes to update the overwrite options
				this.f.whenModuleAvailable("graphics", function (module) {
					this.graphicsModule = module;

					if (this.isEnabled() && !this._isListeningToGraphics) {
						this._registerListener("model.changed.channels", _.debounce(_.bind(this.updateOverwriteOptions, this), 40), undefined, this.graphicsModule);
						this._registerListener("model.changed.channel.**", _.debounce(_.bind(this.updateOverwriteOptions, this), 40), undefined, this.graphicsModule);
						this._isListeningToGraphics = true;
						this.updateOverwriteOptions();
					}
				}, this);


				this._registerListener("doc.page.active", this.updatePanelHeaders, this);
				this._registerListener("doc.page.difference", this.updatePanelHeaders, this);
				this.updatePanelHeaders();
			} else if (this.isEnabled() && !toggle) {
				CompareMode.prototype.toggle.call(this, toggle);

				// remove panels
				this.viewport.removePanel("compare_difference");
				this.viewport.removePanel("compare_align_result");

				// the side by side compare is disabled, restore everything and remove the previously created panels
				this._compareModel.setDiffMode(CompareConst.diffMode.UNDEFINED);

				// reset overwrite options
				this._currentFilePanel.deleteOverwriteOption("dmode");
				this._currentFilePanel.deleteOverwriteOption("colorBITWISE");
				this._currentFilePanel.toggleLayer("Annotations", true);
				this._currentFilePanel.toggleLayer("AnnotationPopups", true);
				this._currentFilePanel.toggleLayer("referenceCad", true);
				this._currentFilePanel.toggleLayer("currentDiffCad", true);
				this._currentFilePanel.toggleLayer("referenceDiffCad", true);
				this._currentFilePanel.toggleLayer("CadOverlay", true);
				this._currentFilePanel.setHeaderText(null);

				// reset references
				this._currentFilePanel = null;
				this._differenceFilePanel = null;
				this._resultPanel = null;

				// set panel layout to automatic
				this.viewport.setPanelsLayoutMode(ViewportConstants.PanelsLayoutMode.AUTOMATIC);

				// unregister all listeners
				this._unregisterAllListeners();
				this._isListeningToGraphics = false;


				this.viewport.removeLayer("compare_alignment_currentAnchor");
				this.viewport.removeLayer("compare_alignment_referenceAnchor");
			}
		};

		/**
		 * Configure the overwrite options for all created panels
		 */
		CompareAlignmentMode.prototype.updateOverwriteOptions = function () {
			if (this.isEnabled()) {
				if (this.graphicsModule === undefined) {
					this.graphicsModule = this.f.getModule("graphics");
				}
				if (this.annotationModule === undefined) {
					this.annotationModule = this.f.getModule("annotation");
				}


				this._currentFilePanel.setOverwriteOptions(this._getCurrentFileOverwriteOptions());
				this._differenceFilePanel.setOverwriteOptions(this._getDifferenceFileOverwriteOptions());
				this._resultPanel.setOverwriteOptions(this._getResultOverwriteOptions());
			}
		};

		/**
		 * Generate the overwrite options for the current file panel
		 * @returns {Object}
		 */
		CompareAlignmentMode.prototype._getCurrentFileOverwriteOptions = function () {
			var colormask = 0;
			if (this.graphicsModule !== undefined) {
				colormask = this.graphicsModule.getChannelSelectionMaskForDiffMode(CompareConst.diffMode.CURRENTVERSION);
			}

			return {
				dmode: CompareConst.diffMode.CURRENTVERSION,
				colorBITWISE: colormask
			};
		};

		/**
		 * Generate the overwrite options for the difference file panel
		 * @returns {Object}
		 */
		CompareAlignmentMode.prototype._getDifferenceFileOverwriteOptions = function () {
			var options = {
				dmode: CompareConst.diffMode.DIFFVERSION,
				colorBITWISE: 0,
				isDefaultView: false
			};

			if (this.graphicsModule !== undefined) {
				options.colorBITWISE = this.graphicsModule.getChannelSelectionMaskForDiffMode(CompareConst.diffMode.DIFFVERSION);
			}

			// do not show graphics for middle panel when aligning cad overlay
			if (this._forCadOverlay) {
				options.colorBITWISE = 0;
			}

			return options;
		};

		/**
		 * Generate the overwrite options for the difference compare mode panel
		 * @returns {Object}
		 */
		CompareAlignmentMode.prototype._getResultOverwriteOptions = function () {

			var params = {
				dmode: CompareConst.diffMode.SHOWBOTHONTOP
			};

			if (this.graphicsModule !== undefined) {
				params.colorBITWISE = this.graphicsModule.getChannelSelectionMaskForDiffMode(CompareConst.diffMode.SHOWBOTHCHANGES);
			}
			return params;
		};


		/**
		 * Update the header text for the panels
		 */
		CompareAlignmentMode.prototype.updatePanelHeaders = function () {
			this.f.loadBundle("viewer", function () {
				if (this.isEnabled()) {
					if (!this._forCadOverlay) {
						var current = "";
						var difference = "";

						var docController = this.f.getDocumentController();
						var activePage = docController.getActivePage();
						var activeDocVersion = activePage.getDocumentVersion();

						var differencePage = docController.getDifferencePage();
						var differenceDocVersion = differencePage.getDocumentVersion();

						if (activeDocVersion.getDocument() !== differenceDocVersion.getDocument()) {
							// we need to add document info too
							current += activeDocVersion.getDocument().get("docname") + ", ";
							difference += differenceDocVersion.getDocument().get("docname") + ", ";
						}

						current += this.f.t("WCHistVersion") + " " + activeDocVersion.getVersionId();
						current += " - (" + this.f.t("CaptionCurrent") + ")";

						difference += this.f.t("WCHistVersion") + " " + differenceDocVersion.getVersionId();
						difference += " - (" + this.f.t("CaptionPrevious") + ")";

						this._currentFilePanel.setHeaderText(current);
						this._differenceFilePanel.setHeaderText(difference);

						if (this._resultPanel) {
							this._resultPanel.setHeaderText(this.f.t("AnchorAlignWizardResultPanelHeader"));
						}
					} else {
						// set headers for cad overlay
					}

				}
			}, this);
		};

		/**
		 * Enable the "CAD OVERLAY" alignment mode (needs to be done before mode is started
		 * @param {Boolean} forCadOverlay - if true, enable cad overlay alignment mode
		 */
		CompareAlignmentMode.prototype.setForCadOverlay = function (forCadOverlay) {
			this._forCadOverlay = forCadOverlay;
		};

		return CompareAlignmentMode;
	});

define('web-viewer/compare/alignment/CompareAlignmentAnchorModel',["web-viewer/core/AbstractModel",
		"utils/ShapeUtils",
		"underscore"
	],
	function (AbstractModel,
		ShapeUtils,
		_) {
		"use strict";

		/**
		 * @exports CompareAlignmentAnchorModel
		 * @memberof CompareModule
		 * @class
		 * This model contains 2 anchors, used for anchor alignment (both anchors can be null)
		 * These points are stored in document coordinates (activePage) relative to activePage.documentPageOrigin()
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewerfacade
		 * @param {String,Array} channelName - the name of the private channel
		 */
		var CompareAlignmentAnchorModel = function (f, channelName) {
			AbstractModel.call(this, f, channelName);


			this._currentAnchor = null;
			this._referenceAnchor = null;

			// by default the reference anchor is active
			this._isCurrentActive = false;
		};

		CompareAlignmentAnchorModel.prototype = Object.create(AbstractModel.prototype);
		CompareAlignmentAnchorModel.prototype.constructor = CompareAlignmentAnchorModel;

		/**
		 * Change the position of the current anchor
		 * @param {ShapeUtils.Point} point
		 */
		CompareAlignmentAnchorModel.prototype.setCurrentAnchor = function (point) {
			var oldCurrent = this._currentAnchor;
			this._currentAnchor = point !== null ? point.copy() : null;
			this._fireChange(false, ["current"], oldCurrent, this._currentAnchor !== null ? this._currentAnchor.copy() : null);
		};

		/**
		 * Move the current Anchor
		 * @param {Number} x - x offset
		 * @param {Number} y - y offset
		 */
		CompareAlignmentAnchorModel.prototype.moveCurrentAnchor = function (x, y) {
			var oldCurrent = this._currentAnchor;

			if (this._currentAnchor === null) {
				this._currentAnchor = new ShapeUtils.Point();
			}
			this._currentAnchor = new ShapeUtils.Point(this._currentAnchor.x + x, this._currentAnchor.y + y);
			this._fireChange(false, ["current"], oldCurrent, this._currentAnchor.copy());
		};

		/**
		 * Change the position of the reference anchor
		 * @param {ShapeUtils.Point} point
		 */
		CompareAlignmentAnchorModel.prototype.setReferenceAnchor = function (point) {
			var oldReference = this._referenceAnchor;
			this._referenceAnchor = point !== null ? point.copy() : null;
			this._fireChange(false, ["reference"], oldReference, this._referenceAnchor !== null ? this._referenceAnchor.copy() : null);
		};

		/**
		 * Move the reference anchor
		 * @param {Number} x - x offset
		 * @param {Number} y - y offset
		 */
		CompareAlignmentAnchorModel.prototype.moveReferencAnchor = function (x, y) {
			var oldReference = this._referenceAnchor;

			if (this._referenceAnchor === null) {
				this._referenceAnchor = new ShapeUtils.Point();
			}

			this._referenceAnchor = new ShapeUtils.Point(this._referenceAnchor.x + x, this._referenceAnchor.y + y);
			this._fireChange(false, ["reference"], oldReference, this._referenceAnchor.copy());
		};


		/**
		 * Calculate the vector from current to reference anchor
		 * @returns {ShapeUtils.Size}
		 */
		CompareAlignmentAnchorModel.prototype.getOffset = function () {
			if (this._currentAnchor !== null && this._referenceAnchor !== null) {
				return new ShapeUtils.Size(this._currentAnchor.x - this._referenceAnchor.x, this._currentAnchor.y - this._referenceAnchor.y);
			} else {
				return undefined;
			}
		};


		/**
		 * Get the position of the current anchor (anchor 1)
		 * @returns {ShapeUtils.Point}
		 */
		CompareAlignmentAnchorModel.prototype.getCurrentAnchor = function () {
			return this._currentAnchor !== null ? this._currentAnchor.copy() : null;
		};


		/**
		 * Get the position of the reference anchor (anchor 2)
		 * @returns {ShapeUtils.Point}
		 */
		CompareAlignmentAnchorModel.prototype.getReferenceAnchor = function () {
			return this._referenceAnchor !== null ? this._referenceAnchor.copy() : null;
		};


		/**
		 * Get the position of the currently active anchor (depending on isCurrentActive)
		 * @returns {ShapeUtils.Point}
		 */
		CompareAlignmentAnchorModel.prototype.getActiveAnchor = function () {
			if (this._isCurrentActive) {
				return this.getCurrentAnchor();
			} else {
				return this.getReferenceAnchor();
			}
		};


		/**
		 * Change which anchor is active
		 * @param {Boolean} isCurrentActive - true if the current anchor is active, false if the reference anchor is active
		 */
		CompareAlignmentAnchorModel.prototype.setCurrentActive = function (isCurrentActive) {
			var before = this._isCurrentActive;
			this._isCurrentActive = isCurrentActive;
			this._fireChange(false, ["isCurrentActive"], before, isCurrentActive);
		};

		/**
		 * Check which anchor is active
		 * @returns {Boolean} true if the current anchor is active
		 */
		CompareAlignmentAnchorModel.prototype.isCurrentActive = function () {
			return this._isCurrentActive;
		};

		/**
		 * Change the position of the currently activated anchor
		 * @param {ShapeUtils.Point} point
		 */
		CompareAlignmentAnchorModel.prototype.setActiveAnchorPosition = function (point) {
			if (this.isCurrentActive()) {
				this.setCurrentAnchor(point);
			} else {
				this.setReferenceAnchor(point);
			}
		};

		/**
		 * Move the currently active anchor
		 * @param {Number} x - x offset
		 * @param {Number} y - y offset
		 */
		CompareAlignmentAnchorModel.prototype.moveActiveAnchor = function (x, y) {
			if (this.isCurrentActive()) {
				this.moveCurrentAnchor(x, y);
			} else {
				this.moveReferencAnchor(x, y);
			}
		};

		/**
		 * Reset the anchor model (this resets the position of both anchors)
		 */
		CompareAlignmentAnchorModel.prototype.reset = function () {
			this.setCurrentAnchor(null);
			this.setReferenceAnchor(null);
		};

		return CompareAlignmentAnchorModel;
	});

define('web-viewer/utils/SnapIndicator',["web-viewer/utils/LineworkWrapper",
		"web-viewer/utils/DrawUtils",
		"utils/ShapeUtils"
	],
	function (LineworkWrapper, DrawUtils, ShapeUtils) {
		"use strict";

		var IMG_SRC = "/wcr-static/web-viewer/layout/img/crosshair.png";
		var IMG_OFFSET = new ShapeUtils.Size(7, 7);

		/**
		 * @exports SnapIndicator
		 * @memberof MeasureModule
		 * @class SnapIndicator is a LineworkWrapper object that is used to display a hint to snap to
		 * @augments Utils.LineworkWrapper
		 * @constructor
		 * @param {String} name - Name of the oneup
		 * @param {ShapeUtils.Point} [center] - center for where the indication should be
		 */


		var SnapIndicator = function (center) {
			LineworkWrapper.call(this);

			this.type = DrawUtils.LWTYPE_IMAGE;
			this.linewidth = 0;
			this.imageRef = IMG_SRC;
			this.staticsize = true;
			this.imageOffset = IMG_OFFSET;

			if (center) {
				this.setCenter(center);
			}



		};

		// inherit
		SnapIndicator.prototype = Object.create(LineworkWrapper.prototype);
		// correct the constructor pointer 
		SnapIndicator.prototype.constructor = SnapIndicator;

		/**
		 * Set the center point of the snap indicator
		 * @param {ShapeUtils.Point} center - center of the indicator
		 */
		SnapIndicator.prototype.setCenter = function (center) {
			this.setRect(new ShapeUtils.Rect(new ShapeUtils.Point(center.x, center.y)));
		};


		return SnapIndicator;

	});

define('web-viewer/compare/alignment/CompareAlignmentAnchorLayerModel',["underscore",
		"web-viewer/viewport/model/ViewportLayerModel",
		"web-viewer/compare/CompareConstants",
		"web-viewer/utils/LineworkWrapper",
		"web-viewer/utils/DrawUtils",
		"utils/ShapeUtils",
		"web-viewer/utils/SnapIndicator"
	],
	function (_,
		ViewportLayerModel,
		CompareConstants,
		LineworkWrapper,
		DrawUtils,
		ShapeUtils,
		SnapIndicator

	) {
		"use strict";
		/**
		 * @exports CompareAlignmentAnchorLayerModel
		 * @memberof Compare
		 * @augments ViewportLayerModel
		 * @class This model holds 1 anchor point to draw in a layer
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {String} ID - the ID of the layer model
		 */

		var CompareAlignmentAnchorLayerModel = function (f, ID) {
			ViewportLayerModel.call(this, f, ID);
			this._anchor = null;
			this._isActive = false;
			this._snapIndicator = null;
			this.f.getDocumentController().whenActivePageParametersChange(_.bind(this._updateTransformation, this));

		};

		// inherit
		CompareAlignmentAnchorLayerModel.prototype = Object.create(ViewportLayerModel.prototype);
		// correct the constructor pointer
		CompareAlignmentAnchorLayerModel.prototype.constructor = CompareAlignmentAnchorLayerModel;

		/**
		 * Set the position of the anchor
		 * @param {ShapeUtils.Point} anchor
		 */
		CompareAlignmentAnchorLayerModel.prototype.setAnchor = function (anchor) {
			this._anchor = anchor;
			this.modelChanged();
		};

		/**
		 * Move the anchor
		 * @param {Number} x
		 * @param {Number} y
		 */
		CompareAlignmentAnchorLayerModel.prototype.moveAnchor = function (x, y) {
			this._anchor.x += x;
			this._anchor.y += y;
			this.modelChanged();
		};


		/**
		 * Change how the anchor should be displayed (active or non-active)
		 * @param {Boolean} isActive
		 */
		CompareAlignmentAnchorLayerModel.prototype.setActive = function (isActive) {
			this._isActive = isActive;
		};

		/**
		 * Update the transformation of the layer so that (0,0) is (0,0) in the current document
		 */
		CompareAlignmentAnchorLayerModel.prototype._updateTransformation = function () {
			this.resetTransformation();
			var page = this.f.getActivePage();
			var origin = page.getDocumentPageOrigin();
			this.transform.translate(origin.x, origin.y);
		};

		/**
		 * Indicate CAD snap point
		 * @param {ShapeUtils.Point} [point] - snap point to indicate
		 */
		CompareAlignmentAnchorLayerModel.prototype.indicateSnapPoint = function (point) {
			if (this._snapPoint !== point) {
				this._snapPoint = point;
				this.modelChanged();
			}
		};



		/**
		 * @override
		 */
		CompareAlignmentAnchorLayerModel.prototype.getLinework = function () {
			var linework = [];

			if (this._anchor) {
				var anchor = new LineworkWrapper();
				anchor.type = DrawUtils.LWTYPE_IMAGE;
				anchor.linewidth = 1;
				anchor.imageRef = CompareConstants.anchorAlignment[this._isActive ? 'ACTIVE' : 'DEFAULT'];
				anchor.staticsize = true;
				anchor.setRect(new ShapeUtils.Rect(new ShapeUtils.Point(this._anchor.x, this._anchor.y)));
				anchor.imageOffset = new ShapeUtils.Size(CompareConstants.anchorAlignment.OFFSET.width, CompareConstants.anchorAlignment.OFFSET.height);

				linework.push(anchor);

			}

			if (this._snapPoint) {
				if (!this._snapIndicator) {
					this._snapIndicator = new SnapIndicator(this._snapPoint);
				} else {
					this._snapIndicator.setCenter(this._snapPoint);
				}

				linework.push(this._snapIndicator);
			}

			return linework;
		};



		return CompareAlignmentAnchorLayerModel;
	});

define('web-viewer/compare/alignment/CompareAnchorTool',["underscore",
		"web-viewer/viewport/tools/AbstractTool",
		"utils/ShapeUtils",
		"web-viewer/core/CoreConstants"
	],
	function (_,
		AbstractTool,
		ShapeUtils,
		CoreConstants) {
		"use strict";

		/**
		 * @exports CompareAnchorTool
		 * @memberof Compare
		 * @class
		 * This tool is used to align two documents
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {CompareAlignmentAnchorModel} compareAlignmentAnchorModel - model used for storing alignment anchors
		 * @param {CompareAlignmentAnchorLayerModel} currentLayerModel - layer model for the current anchor
		 * @param {CompareAlignmentAnchorLayerModel} referenceLayerModel - layer model for the reference anchor
		 */
		var CompareAnchorTool = function (f, compareAlignmentAnchorModel, currentLayerModel, referenceLayerModel) {
			AbstractTool.call(this, f, CoreConstants.TOOLS.COMPARE_ANCHOR_ALIGNMENT);

			/**
			 * The model used for the alignment anchors
			 */
			this._alignmentAnchorModel = compareAlignmentAnchorModel;

			/**
			 * The layer model for the current anchor
			 */
			this._currentLayerModel = currentLayerModel;

			/**
			 * The layer model for the reference anchor
			 */
			this._referenceLayerModel = referenceLayerModel;

			/**
			 * Is the tool used for CAD overlay Alignment
			 **/
			this._forCadOverlay = false;

			/* on Firefox on Mac there is a problem with cursor display none and custom cursors... so we show a default crosshair cursor in this case (by adding MacFF to the ViewerApp)*/
			if (window.mozInnerScreenX !== null && navigator.platform.indexOf('Mac') >= 0) {
				this._macFF = true;
				this.f.getContext().addClass("MacFF");
			} else {
				this._macFF = false;
			}

			/**
			 * DEFAULT STATE
			 */
			this.registerState("default", {
				/*
				 * on mouse down, check if the mouse is in either the current or reference panel (and set the correct anchor as active)
				 */
				mousedown: _.bind(function (e) {
					var panel = this.getPanelAt(e);
					var panelName = panel.getModel().getName();
					if (panelName === "default") {
						this._alignmentAnchorModel.setCurrentActive(true);
						this.changeState("validPanel");
					} else if (panelName === "compare_difference") {
						this._alignmentAnchorModel.setCurrentActive(false);
						this.changeState("validPanel");
					}
					e.preventDefault();
				}, this),

				mouseup: function () {},

				mousemove: _.bind(function (e) {
					// we don't need to use the result, but the snapping should be indicated
					this._getAndIndicateSnapPoint(e);
				}, this),

				/**
				 * no context menu!
				 */
				contextmenu: _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
				}, this)
			});

			/**
			 * validPanel - the interaction is happening in a valid panel, move the anchors
			 */
			this.registerState("validPanel", _.defaults({
				mouseup: _.bind(function (e) {
					var anchorPoint = this._getAndIndicateSnapPoint(e);
					if (!anchorPoint) {
						var scenePoint = this.getScenePoint(e);
						anchorPoint = this.convertScenePointToWorldWithCorrectTransform(scenePoint);
					}

					this._alignmentAnchorModel.setActiveAnchorPosition(anchorPoint);
					this.changeState("default");
				}, this),
				mousemove: _.bind(function (e) {
					var anchorPoint = this._getAndIndicateSnapPoint(e);
					if (!anchorPoint) {
						var scenePoint = this.getScenePoint(e);
						anchorPoint = this.convertScenePointToWorldWithCorrectTransform(scenePoint);
					}
					this._liveSetAnchor(anchorPoint);
				}, this)

			}, this.getState("default")));


			this.registerActiveEvents(["mousedown", "mousemove", "mouseup", "contextmenu"]);

			/**
			 * Also support key bindins
			 */
			_.each(["up", "down", "left", "right"], function (key) {
				this.activeKeyEvents.push([key, _.bind(this._handleArrowKey, this, key), "keyup"]);
			}, this);


			/**
			 * @type {ShapeUtils.Size} the movement the anchor should make at the end of key input
			 */
			this._keyMove = null;

			/**
			 * When the keyinput stopped, move the active anchor
			 */
			this._handleEndOfKeyInput = _.debounce(_.bind(function () {
				if (this._keyMove) {
					this._alignmentAnchorModel.moveActiveAnchor(this._keyMove.width, this._keyMove.height);
					this._keyMove = null;
				}

				this.f.emit("viewport.stopInteractive");
			}, this), 500);


		};

		CompareAnchorTool.prototype = Object.create(AbstractTool.prototype);
		CompareAnchorTool.prototype.constructor = CompareAnchorTool;


		/**
		 * The tool is activated
		 * @param interruptRequest
		 */
		CompareAnchorTool.prototype.doActivation = function (interruptRequest) {
			AbstractTool.prototype.doActivation.call(this, interruptRequest);
			this.changeState("default");
		};

		/**
		 * Get the layer model for the active anchor
		 * @returns {CompareAlignmentAnchorLayerModel}
		 */
		CompareAnchorTool.prototype._getActiveAnchorLayerModel = function () {
			return this._alignmentAnchorModel.isCurrentActive() ? this._currentLayerModel : this._referenceLayerModel;
		};

		/**
		 * Live update the layer model, but not the anchor model
		 * @param destination
		 */
		CompareAnchorTool.prototype._liveSetAnchor = function (destination) {
			var layerModel = this._getActiveAnchorLayerModel();
			layerModel.setAnchor(destination);
		};

		/**
		 * Returns the default cursor for the tool
		 * @returns {String}
		 */
		CompareAnchorTool.prototype.getDefaultCursor = function () {
			return "cursor-anchor";
		};

		/**
		 * Convert a scene point to the correct location relative to (0,0) of the current document
		 * @param {ShapeUtils.Point} point
		 * @returns {ShapeUtils.Point}
		 */
		CompareAnchorTool.prototype.convertScenePointToWorldWithCorrectTransform = function (point) {
			var world = this._widget.getWorld();
			/* World transform */
			var t = world.getTransformation();

			var docOrigin = this.f.getActivePage().getDocumentPageOrigin();
			t.translate(docOrigin.x, docOrigin.y);

			return t.createInverse().transformedPoint(point);
		};

		/**
		 * Get the Point to snap to (and indicate it)
		 * @param   {Object}           e - mouse event
		 * @returns {ShapeUtils.Point} point to select (could be snapped or the original scenepoint) in active page coordinates (relative to the activepage DocumentOrigin)
		 */
		CompareAnchorTool.prototype._getAndIndicateSnapPoint = function (e) {
			var panel = this.getPanelAt(e);
			if (panel) {
				var panelName = panel.getModel().getName();

				if (panelName === "default" || panelName === "compare_difference") {
					var view = this._widget.requestView(e);
					var refpoint = this.getScenePoint(e);
					if (view) {
						var searchFilter;
						if (panelName === "default") {
							searchFilter = ["Cad"];
							if (!this._forCadOverlay) {
								searchFilter.push("CadOverlay");
							}
						} else if (panelName === "compare_difference") {
							if (this._forCadOverlay) {
								searchFilter = ["CadOverlay"];
							} else {
								searchFilter = ["referenceCad"];
							}
						}
						var point = view.getClosestLineworkPointFromPoint(refpoint, 16, searchFilter);

						if (point) {
							point = this.convertScenePointToWorldWithCorrectTransform(point); // convert to world coordinates (relative to the activepage DocumentOrigin)
							this._currentLayerModel.indicateSnapPoint((panelName === "default") ? point : null);
							this._referenceLayerModel.indicateSnapPoint((panelName === "compare_difference") ? point : null);
							this.setCursor("cursor-none");
						} else {
							point = this.convertScenePointToWorldWithCorrectTransform(refpoint);
							this._currentLayerModel.indicateSnapPoint(null);
							this._referenceLayerModel.indicateSnapPoint(null);
							this.setCursor("cursor-anchor");
						}
						return point;
					}
				}
			}
			return null;
		};

		/**
		 * An arrow key is pressed, update the keymove size, and launch the debounced handleendofkeyinput
		 * @param key
		 * @param e
		 */
		CompareAnchorTool.prototype._handleArrowKey = function (key, e) {
			if (!this._keyMove) {
				this.f.emit("viewport.startInteractive");
				this._keyMove = new ShapeUtils.Size(0, 0);
			}

			// we want to move the anchor with 1 pixel, so based on the zoomlevel we know how much mm in the document it is
			var distanceInMM = 1 / this._widget.getWorld().getZoom();

			var keyIndex = _.indexOf(["up", "right", "down", "left"], key);

			var rotation = this._widget.getWorld().getRotation() % 360;
			var flip = this._widget.getWorld().isFlip();
			var isViewFromBack = this._widget.getWorld().isFlipViewFromBack();
			if (isViewFromBack) {
				if (rotation === 90 || rotation === 270) {
					rotation = (rotation + 180) % 360;
				}
				flip = !flip;
			}

			// reverse rotation
			keyIndex = (keyIndex + 4 - ((rotation / 90) % 4)) % 4;

			// reverse flip
			if (flip) {
				if (keyIndex == 1) {
					keyIndex = 3;
				} else if (keyIndex == 3) {
					keyIndex = 1;
				}
			}

			var x = 0,
				y = 0;

			switch (keyIndex) {
			case 0:
				// UP
				y -= distanceInMM;
				break;
			case 1:
				// RIGHT
				x += distanceInMM;
				break;
			case 2:
				// DOWN
				y += distanceInMM;
				break;
			case 3:
				// LEFT
				x -= distanceInMM;
				break;
			}

			// update the layer model
			var layerModel = this._getActiveAnchorLayerModel();
			layerModel.moveAnchor(x, y);

			// update key move
			this._keyMove.width += x;
			this._keyMove.height += y;

			// launch debounced handle end of key input
			this._handleEndOfKeyInput();
		};

		/*
		 * Is the tool being used for CAD overlay or not?
		 * @param {Boolean} forCadOverlay
		 */
		CompareAnchorTool.prototype.setForCadOverlay = function (forCadOverlay) {
			this._forCadOverlay = forCadOverlay;
		};

		/**
		 * @override
		 */
		CompareAnchorTool.prototype.getIconOffset = function () {
			if (!this._macFF) {
				return new ShapeUtils.Size(5, 5);
			} else {
				// default crosshair is shown, no icon offset
				return new ShapeUtils.Size(0, 0);
			}
		};

		return CompareAnchorTool;
	});

define('tpl!web-viewer/compare/alignment/CompareAlignmentWizardWidget', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<h3>'+
((__t=(d.i18n('AnchorAlignWizardTitle')))==null?'':_.escape(__t))+
'</h3>\r\n<p>'+
((__t=(d.getExplanationText()))==null?'':_.escape(__t))+
'<br />\r\n'+
((__t=(d.i18n('AnchorAlignWizardArrowKeys')))==null?'':_.escape(__t))+
'</p>';
return __p;
}});
define('web-viewer/compare/alignment/CompareAlignmentWizardWidget',["web-viewer/layout/AbstractWidget",
		"Template",
		"tpl!web-viewer/compare/alignment/CompareAlignmentWizardWidget",
		"web-viewer/layout/component/ViewerToolbarButton",
		"component/toolbar/ButtonGroup",
		"component/popover/Popover",
		"jquery",
		"underscore",
		"web-viewer/compare/CompareConstants",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/core/CoreConstants",
		"web-viewer/utils/ListenerUtil"
	],
	function (AbstractWidget,
		Template,
		tpl,
		ViewerToolbarButton,
		ButtonGroup,
		Popover,
		$,
		_,
		CompareConstants,
		LayoutConstants,
		CoreConstants,
		ListenerUtil) {
		"use strict";

		/**
		 * @constant {Number} the speed to render the widget
		 */
		var RENDER_SPEED = 40;

		/**
		 * @exports CompareAlignmentWizardWidget
		 * @augments Layout.AbstractWidget
		 * @class The CompareAlignmentWizardWidget displays the currently loaded document and logged in user
		 * @constructor
		 * @mixes {Utils.ListenerUtilMixin}
		 * @param {Core.ViewerFacade} f
		 */
		var CompareAlignmentWizardWidget = function CompareAlignmentWizardWidget(f, wizard, anchorTool) {
			AbstractWidget.call(this, f, LayoutConstants.Widget.ALIGNWIZARD);
			ListenerUtil.mixin(this, f);

			this._wizard = wizard;
			this._anchorTool = anchorTool;

			this.templateContext = $("<td/>");

			// initialize template
			this.t = new Template(f.i18n, tpl);
			this.t.d.getExplanationText = function () {
				if (!wizard.isForCadOverlay()) {
					return this.i18n("AnchorAlignWizardExplanation");
				} else {
					return this.i18n("AnchorAlignWizardExplanationCadGraphics");
				}
			};

			this.render = _.throttle(_.bind(this._render, this), RENDER_SPEED);



			// the button bar at the bottom of the screen
			this._buttonBar = $("<td/>").addClass("buttons");



			this._anchorToolButton = new ViewerToolbarButton(this.f, "anchorTool", "icon-WCR_Align-Point", true);
			// tooltip shown when anchor tool is not active
			var tooltip = new Popover(this._anchorToolButton.element, this.f.getContext(), this.f.getContext(), "bottom", "#333", "tooltip", 5);
			this._tooltip = tooltip;
			var tmpAnchorSetActive = this._anchorToolButton.setActive;
			// when the button is inactive (because the tool is inactive) show the popover
			this._anchorToolButton.setActive = _.bind(function (widget, isActive) {
				tmpAnchorSetActive.call(this, isActive);
				if (!isActive && widget.isVisible() && widget.isEnabled()) {
					tooltip.show();
				} else {
					tooltip.hide();
				}
			}, this._anchorToolButton, this);

			this._anchorToolButton.listenToTool(this._anchorTool, true);
			this._anchorToolButton.setTooltip("viewer", "AnchorAlignWizardAnchorTool");



			this._autoAlignButton = new ViewerToolbarButton(this.f, "autoAlign", "icon-WCR_Align-Auto", true, _.bind(function () {
				this._wizard.doAutoAlign();
			}, this));

			this._autoAlignButton.setTooltip("viewer", "AlignAuto");
			this._autoAlignButton.element.addClass("autoAlign");


			this._alignGroup = new ButtonGroup("alignButtons", [this._anchorToolButton, this._autoAlignButton]);
			this._buttonBar.append(this._alignGroup.element);
			// Buttons in the editor
			// Cancel button
			this._cancelButton = new ViewerToolbarButton(this.f, "cancelAlignment", "icon-WCR_Revert", true, _.bind(function (e) {
				this._tooltip.hide();
				this._wizard.cancel();
			}, this));
			// Save button
			this._alignButton = new ViewerToolbarButton(this.f, "saveAlignment", "icon-WCR_Save", true, _.bind(function (e) {
				this._tooltip.hide();
				this._wizard.alignAndExit();
			}, this));
			this._alignButton.element.addClass("btn-success");

			// Save and cancel are a buttongroup
			this._cancelSaveGroup = new ButtonGroup("editActionsGroup", [this._cancelButton, this._alignButton]);
			this._buttonBar.append(this._cancelSaveGroup.element);

			// load language if needed
			this.f.loadBundle("viewer", function () {
				this._tooltip.setContent($("<div/>").text(this.f.t('AnchorAlignWizardReactivateTool')));
				this._alignButton.setContent($("<span/>").text(this.f.t("AnchorAlignWizardAlign")));
				this._cancelButton.setContent($("<span/>").text(this.f.t("CancelButton")));

				this.render();
			}, this);

		};

		CompareAlignmentWizardWidget.prototype = Object.create(AbstractWidget.prototype);
		CompareAlignmentWizardWidget.prototype.constructor = CompareAlignmentWizardWidget;
		CompareAlignmentWizardWidget.widgetName = LayoutConstants.Widget.ALIGNWIZARD;

		/**
		 * @override
		 */
		CompareAlignmentWizardWidget.prototype.setContext = function (context) {
			AbstractWidget.prototype.setContext.call(this, context);

			this.context.addClass("enableTextSelection");
			this.context.append($("<table>").append($("<tr>").append(this.templateContext).append(this._buttonBar)));

			this.t.setContext(this.templateContext);

			this.context.addClass("CompareAlignmentWizardWidget");
			this.render();
		};


		/**
		 * Update data and render
		 */
		CompareAlignmentWizardWidget.prototype._render = function () {
			if (!this._wizard.isAutoAlignEnabled()) {
				this._alignGroup.detachButton(this._autoAlignButton);
			} else {
				this._alignGroup.addButton(this._autoAlignButton);
			}
			this.t.render();
		};


		/**
		 * @override
		 * @returns {String}
		 */
		CompareAlignmentWizardWidget.prototype.getWidgetType = function () {
			return "wizard";
		};


		/**
		 * Hide the tooltip when the widget is disabled
		 * @override
		 */
		CompareAlignmentWizardWidget.prototype.setEnabled = function (isEnabled) {
			AbstractWidget.prototype.setEnabled.apply(this, arguments);

			if (!isEnabled) {
				this._tooltip.hide();
			}
		};

		/**
		 * hide the tooltip if the widget is made invisible
		 * @override
		 * @param isVisible
		 */
		CompareAlignmentWizardWidget.prototype.setVisible = function (isVisible) {
			AbstractWidget.prototype.setVisible.call(this, isVisible);
			if (!isVisible) {
				// hide the tooltip when the widget is hidden
				this._tooltip.hide();
			}

			this.render();
		};

		/**
		 * hide the tooltip if the widget is collapsed
		 * @override
		 * @param isVisible
		 */
		CompareAlignmentWizardWidget.prototype.setCollapsed = function (collapsed) {
			AbstractWidget.prototype.setCollapsed.call(this, collapsed);
			if (collapsed) {
				// hide the tooltip when the widget is hidden
				this._tooltip.hide();
			}
		};

		return CompareAlignmentWizardWidget;
	});

define('web-viewer/compare/alignment/CompareAlignmentWizard',["web-viewer/core/ViewerMode",
		"web-viewer/core/CoreConstants",
		"web-viewer/compare/alignment/CompareAlignmentMode",
		"web-viewer/compare/alignment/CompareAlignmentAnchorModel",
		"web-viewer/compare/alignment/CompareAlignmentAnchorLayerModel",
		"web-viewer/compare/alignment/CompareAnchorTool",
		"web-viewer/compare/alignment/CompareAlignmentWizardWidget",
		"web-viewer/utils/ListenerUtil",
		"utils/ShapeUtils",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/graphics/GraphicsConstants",
		"web-viewer/cad/CadConstants",
		"Mousetrap",
		"underscore"
	],
	function (ViewerMode,
		CoreConstants,
		CompareAlignmentMode,
		CompareAlignmentAnchorModel,
		CompareAlignmentAnchorLayerModel,
		CompareAnchorTool,
		CompareAlignmentWizardWidget,
		ListenerUtil,
		ShapeUtils,
		LayoutConstants,
		GraphicsConstants,
		CadConstants,
		Mousetrap,
		_) {
		"use strict";

		/**
		 * @class CompareAlignmentWizard the Compare Alignment Wizard is a ViewerMode which starts up an interactive anchor alignment for the compare module
		 * @memberof CompareModule
		 * @exports CompareAlignmentWizard
		 *
		 * @augments {Core.ViewerMode}
		 * @param {Core.ViewerFacade} f
		 * @param {CompareModule} compareModule - the compareModule
		 * @param {CompareModel} compareModel - the compare model
		 */
		var CompareAlignmentWizard = function (f, compareModule, compareModel) {
			ViewerMode.call(this, f, "compareAlignmentWizardMode");
			ListenerUtil.mixin(this, f);

			this._compareModule = compareModule;
			this._compareModel = compareModel;

			this._alignmentAnchorModel = new CompareAlignmentAnchorModel(this.f, ["module", "compare", "alignment", "anchor"]);
			this._currentAnchorLayerModel = new CompareAlignmentAnchorLayerModel(this.f, "compare_alignment_currentAnchor");
			this._referenceAnchorLayerModel = new CompareAlignmentAnchorLayerModel(this.f, "compare_alignment_referenceAnchor");

			this._compareAlignmentMode = new CompareAlignmentMode(this.f, this._compareModel, this._currentAnchorLayerModel, this._referenceAnchorLayerModel);
			this._compareModel.registerMode(this._compareAlignmentMode);
			this._compareAlignmentWizardWidget = null;
			this._middleToActiveAnchor = null;
			this._anchorTool = null;
			this._forCadOverlay = false;

			this._offsetBeforeStart = null;
			this._before = {};
			this._activeAnchorToWorldOffset = new ShapeUtils.Size();

			this._compareDiffMode = null;

			this._onPressA = _.bind(function () {
				this.f.emit("toolManager.activateTool", this._anchorTool);
			}, this);



		};

		CompareAlignmentWizard.prototype = Object.create(ViewerMode.prototype);
		CompareAlignmentWizard.prototype.constructor = CompareAlignmentWizard;


		/**
		 * The Wizard mode is started
		 */
		CompareAlignmentWizard.prototype._doStart = function () {
			this._before = {};

			// store the current compare offset (so we can restore it on cancel)	
			if (!this._forCadOverlay) {
				this._offsetBeforeStart = this._compareModel.getOffset();
			} else {
				this._offsetBeforeStart = this._compareModel.getCadOverlayOffset();
			}



			// Save the current Channel view mode and set it to composite view
			var graphicsModule = this.f.getModule("graphics");
			if (graphicsModule) {
				this._before.channelViewMode = graphicsModule.getChannelViewMode();
				graphicsModule.setChannelViewMode(GraphicsConstants.ChannelViewMode.COMPOSITE_VIEW);
			}

			// Save the current CAD+Graphics mode and change it to show both
			var cadModule = this.f.getModule("cad");
			if (cadModule) {
				this._before.cadGraphicsMode = cadModule.getCadGraphicsMode();
				cadModule.setCadGraphicsMode(CadConstants.CadGraphicsMode.BOTH);
			}

			// listen to anchor model changes
			this._registerListener("changed.*", this._anchorsUpdated, this, this._alignmentAnchorModel);

			// listen to reference offset changes
			this._registerListener("referenceOffset.beforeChange", this._storeMiddleToAnchorOffset, this, this._compareModule);
			this._registerListener("cadOverlayOffset.beforeChange", this._storeMiddleToAnchorOffset, this, this._compareModule);

			// listen to compare model offset changes
			this._registerListener("referenceOffset.changed", this._offsetChanged, this, this._compareModel);
			this._registerListener("cadOverlayOffset.changed", this._offsetChanged, this, this._compareModel);

			this._alignmentAnchorModel.reset();

			this._currentAnchorLayerModel._updateTransformation();
			this._referenceAnchorLayerModel._updateTransformation();

			this._disallowAllToolsExcept([CoreConstants.TOOLS.PAN,
				CoreConstants.TOOLS.ZOOM,
				CoreConstants.TOOLS.COMPARE_ANCHOR_ALIGNMENT
			]);

			// store the current compare mode so we can restore it later on
			this._compareDiffMode = this._compareModule.getCurrentDiffMode();

			this._compareAlignmentMode.setForCadOverlay(this._forCadOverlay);

			// enable the alignment compare mode
			this._compareModule.activateModeByName("alignment");

			// initialize the anchor tool if non-existent
			if (!this._anchorTool) {
				this._initializeAnchorTool();
			}

			this._anchorTool.setForCadOverlay(this._forCadOverlay);

			this.f.emit("toolManager.activateTool", this._anchorTool);



			if (!this._compareAlignmentWizardWidget) {
				// we directly create the widget, the wizard can't really work without it			
				this._compareAlignmentWizardWidget = new CompareAlignmentWizardWidget(this.f, this, this._anchorTool);
				// _publishWidget
				this.f.emit("layout.addWidget", CompareAlignmentWizardWidget.widgetName, _.bind(function (layoutType) {
					return this._compareAlignmentWizardWidget;
				}, this));
			}

			this._compareAlignmentWizardWidget.setEnabled(true);

			Mousetrap.bind("a", this._onPressA, "keypress");

			this.f.emit("layout.switchLayoutMode", LayoutConstants.LayoutMode.WIZARD);
			if (this._compareAlignmentWizardWidget) {
				this._compareAlignmentWizardWidget.render();
			}
		};

		/**
		 * The wizard mode is ended
		 */
		CompareAlignmentWizard.prototype._doStop = function () {
			this._unregisterAllListeners();

			// allow all tools again
			this._allowAllTools();

			// disable the anchor tool
			if (this._anchorTool.isActive()) {
				this._anchorTool.requestDeactivation();
			}

			// activate the previous diffmode
			this._compareModule.activateModeByDiffMode(this._compareDiffMode);
			this._compareDiffMode = null;
			Mousetrap.unbind("a", this._onPressA, "keypress");

			this._compareAlignmentWizardWidget.setEnabled(false);

			this.f.emit("layout.switchLayoutMode");


			// restore the channel view mode
			var graphicsModule = this.f.getModule("graphics");
			if (graphicsModule) {
				graphicsModule.setChannelViewMode(this._before.channelViewMode);
			}

			// restore the CAD + Graphics mode
			var cadModule = this.f.getModule("cad");
			if (cadModule && this._before.cadGraphicsMode !== undefined) {
				cadModule.setCadGraphicsMode(this._before.cadGraphicsMode);
			}
		};


		/**
		 * Update the anchors
		 */
		CompareAlignmentWizard.prototype._offsetChanged = function (oldValue, newValue) {
			// hide snapping indicator when chaning the transformation
			this._currentAnchorLayerModel.indicateSnapPoint(null);
			this._referenceAnchorLayerModel.indicateSnapPoint(null);
			this._currentAnchorLayerModel._updateTransformation();
			this._referenceAnchorLayerModel._updateTransformation();
			this._alignmentAnchorModel.setReferenceAnchor(this._alignmentAnchorModel.getCurrentAnchor());


			this._restoreMiddleToAnchorOffset();
		};

		/**
		 * Change the reference offset of the CompareModule
		 */
		CompareAlignmentWizard.prototype._setReferenceOffset = function () {
			var anchorOffset = this._alignmentAnchorModel.getOffset();
			var cadModule;
			if (anchorOffset && !(anchorOffset.width === 0 && anchorOffset.height === 0)) {

				if (!this._forCadOverlay) {
					var compareOffset = this._compareModel.getOffset();
					if (this.f.getActivePage().hasCad()) {
						// CAD compare alignment
						// we need to apply the inverse transform from the reference CAD to our offset difference

						cadModule = this.f.getModule("cad");
						anchorOffset = cadModule.getReferenceTransform().createInverse().transformedSize(anchorOffset);
						//we need to change the x-offset to the other way for CAD
						anchorOffset.width = -anchorOffset.width;
					}

					this._compareModule.changeReferenceOffset(new ShapeUtils.Point(compareOffset.x + anchorOffset.width, compareOffset.y + anchorOffset.height));
				} else {
					// CAD overlay
					var cadOverlayOffset = this._compareModel.getCadOverlayOffset();

					cadModule = this.f.getModule("cad");
					// apply the inverse transform for the CAD overlay to the offset difference
					anchorOffset = cadModule.getOverlayTransform().createInverse().transformedSize(anchorOffset);

					this._compareModule.changeCadOverlayOffset(new ShapeUtils.Point(cadOverlayOffset.x - anchorOffset.width, cadOverlayOffset.y + anchorOffset.height));
				}
			}

		};

		/**
		 * The anchors have changed positions, update the layermodel, and set the offset on the compare module
		 */
		CompareAlignmentWizard.prototype._anchorsUpdated = function () {
			this._referenceAnchorLayerModel.setAnchor(this._alignmentAnchorModel.getReferenceAnchor());
			this._currentAnchorLayerModel.setAnchor(this._alignmentAnchorModel.getCurrentAnchor());
			this._currentAnchorLayerModel.setActive(this._alignmentAnchorModel.isCurrentActive());
			this._referenceAnchorLayerModel.setActive(!this._alignmentAnchorModel.isCurrentActive());

			this._setReferenceOffset();
		};

		/**
		 * Initialize the anchor tool
		 */
		CompareAlignmentWizard.prototype._initializeAnchorTool = function () {
			this._anchorTool = new CompareAnchorTool(this.f, this._alignmentAnchorModel, this._currentAnchorLayerModel, this._referenceAnchorLayerModel);
			this.f.emit("toolManager.addTool", this._anchorTool);
		};

		/**
		 * Cancel the wizard
		 */
		CompareAlignmentWizard.prototype.cancel = function () {
			if (!this._forCadOverlay) {
				this._compareModule.changeReferenceOffset(this._offsetBeforeStart);
			} else {
				this._compareModule.changeCadOverlayOffset(this._offsetBeforeStart);
			}
			this.requestStop();
		};

		/**
		 * Save the alignment (actually, leave it like it is) and stop the viewermode
		 */
		CompareAlignmentWizard.prototype.alignAndExit = function () {
			this.requestStop();
		};

		/**
		 * Execute an auto-align action
		 */
		CompareAlignmentWizard.prototype.doAutoAlign = function () {
			this._compareModule.doAutoAlign();
		};

		/**
		 * Store the current vector between Middle and the active anchor (for later usage)
		 */
		CompareAlignmentWizard.prototype._storeMiddleToAnchorOffset = function () {
			var anchor = this._alignmentAnchorModel.getActiveAnchor();
			if (anchor && !this._middleToActiveAnchor) {
				// apply offset from active document to get position in the world
				var docOrigin = this.f.getActivePage().getDocumentPageOrigin();
				anchor.offset(new ShapeUtils.Size(docOrigin.x, docOrigin.y));


				// calculate middle to anchor
				var viewport = this.f.getModule("viewport");
				var middle = viewport.getMiddle();
				var zoom = viewport.getZoom();

				// we store the vector in pixel domain! the zoom level can change when the offsets are updated (and the document gets bigger/smaller)
				this._middleToActiveAnchor = new ShapeUtils.Size((anchor.x - middle.x) * zoom, (anchor.y - middle.y) * zoom);
				this.f.emit("module.graphics.model.needsFragmentCleanup");
			}
		};

		/**
		 * Restore the offset between the active anchor and the middle of the viewport
		 * (so that the anchor is displayed on the same location as saved by _storeMiddleToAnchorOffset)
		 */
		CompareAlignmentWizard.prototype._restoreMiddleToAnchorOffset = function () {
			if (this._middleToActiveAnchor) {
				var anchor = this._alignmentAnchorModel.getActiveAnchor();
				// apply offset from active document to get position in the world
				var docOrigin = this.f.getActivePage().getDocumentPageOrigin();
				anchor.offset(new ShapeUtils.Size(docOrigin.x, docOrigin.y));


				// set middle of the viewport
				var viewport = this.f.getModule("viewport");
				var zoom = viewport.getZoom();
				viewport.transform({
					middle: new ShapeUtils.Point(anchor.x - (this._middleToActiveAnchor.width / zoom), anchor.y - (this._middleToActiveAnchor.height / zoom)),
					animated: false,
					tmpDisableConstraints: true
				});
				this._middleToActiveAnchor = null;
			}
		};

		/**
		 * Is auto align possible?
		 * @returns {Boolean}
		 */
		CompareAlignmentWizard.prototype.isAutoAlignEnabled = function () {
			return !this.f.getActivePage().hasCad() && !this._forCadOverlay;
		};

		/**
		 * Check if the wizard is being used for Cad Overlay alignment
		 * @returns {Boolean}
		 */
		CompareAlignmentWizard.prototype.isForCadOverlay = function () {
			return this._forCadOverlay;
		};

		/**
		 * Enable the "CAD OVERLAY" alignment mode (needs to be done before mode is started
		 * @param {Boolean} forCadOverlay - if true, enable cad overlay alignment mode
		 */
		CompareAlignmentWizard.prototype.setForCadOverlay = function (forCadOverlay) {
			this._forCadOverlay = forCadOverlay;
		};


		return CompareAlignmentWizard;
	});

define('web-viewer/layout/component/Spinner',["component/tooltip/Tooltip",
		"web-viewer/utils/ViewerUtils",
		"underscore",
		"jquery",
		"jqueryUI"
	],
	function (Tooltip,
		ViewerUtils,
		_,
		$) {
		"use strict";

		/**
		 * @class A spinner is a component that has an input field with arrows next to it to spin through the values.
		 * You can also use page keys (page) or arrow keys (step).
		 * The wrapping element to add to a component sits in Spinner.element
		 * @exports Spinner
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {Object} options - put undefined for default values
		 *    {Number} min: minimum value of the spinner (Default: 0)
		 *    {Number} max: maximum value of the spinner (Default: 100)
		 *    {Number} step: step value of the spinner (Default: 1);
		 *    {Number} page: page size of the spinner (Default: 10);
		 *    {Function} onSpinnerChanged: called when the spinner value is changed. Arguments: value (Required)
		 *    {String} inputUnit: Unit of the input field (put behind the input field) (Default: %)
		 *    {String} tooltip: Tooltip text that should be shown when you hover over the component. (Optional)
		 */
		var Spinner = function (f, options) {
			this.f = f;

			// Set Options
			this._min = options.min === undefined ? 0 : options.min;
			this._max = options.max === undefined ? 100 : options.max;
			this._step = options.step === undefined ? 1 : options.step;
			this._page = options.page === undefined ? 10 : options.page;
			this._onSpinnerChanged = options.onSpinnerChanged;
			this._inputUnit = options.inputUnit === undefined ? "%" : options.inputUnit;

			// Wrapper element
			this.element = $("<div />").addClass("spinnerWrapper");

			// Add the input field if needed
			this._input = $("<input />").attr("size", ("" + this._max).length).addClass("input"); // the input field
			this._input.on("keydown", _.bind(this._valueChanged, this));
			this._input.on("blur", _.bind(this._valueChanged, this));
			this._labelUnit = $("<span>" + this._inputUnit + "</span>");
			this._label = $("<label />").addClass("unit").append(this._input).append(this._labelUnit); // the unit
			this.element.append(this._label);

			// spinner behavior
			this._input.spinner({
				min: this._min,
				max: this._max,
				step: this._step,
				page: this._page,
				spin: _.bind(function (event, ui) {
					this._fireSpinnerChanged(ui.value);
				}, this)
			});
			this.setValue(this._min);

			// Tooltip
			if (options.tooltip !== undefined) {
				this._tooltip = new Tooltip(this.f.i18n, this.element, this.f.getContext(), this.f.getContext(), this.f.isMobile(), options.tooltip);
			}
		};

		/**
		 * Change the value of the spinner (by pressing enter in the input or on blur)
		 * @param {Object} event
		 */
		Spinner.prototype._valueChanged = function (event) {
			if ((event.which == 13 || event.keyCode == 13) && event.type === "keydown") {
				// on enter, lose focus, let the blur event handle it
				this._input.blur();
			} else if (event.type !== "keydown") {
				var value = ViewerUtils.parseFloat($(event.target).val());
				if (isNaN(value)) { // if nothing is filled in or only letter(s), behave like entered the minimum
					value = this._min;
				}
				// limit to boundaries
				value = Math.max(Math.min(value, this._max), this._min);
				$(event.target).val(value); // by doing this letters added to the end will be filtered out and bounderies are enforced
				this._fireSpinnerChanged(value);
			}
		};

		/**
		 * @private
		 * Fire the value to the listener
		 * @param {Number} value - New Spinner value
		 */
		Spinner.prototype._fireSpinnerChanged = function (value) {
			this._onSpinnerChanged(value);
		};

		/**
		 * Set the value of the Spinner
		 * @param {Number} value - New value of the Spinner
		 */
		Spinner.prototype.setValue = function (value) {
			if (this.getValue() !== value) {
				this._input.spinner("value", value);
			}
		};

		/**
		 * Get the value of the Spinner
		 * @returns {Number} The value of the Spinner
		 */
		Spinner.prototype.getValue = function () {
			return this._input.spinner("value");
		};

		/**
		 * Is the Spinner disabled?
		 * @returns {Boolean} True if the Spinner is disabled, false otherwise
		 */
		Spinner.prototype.isDisabled = function () {
			return this._input.spinner("option", "disabled");
		};

		/**
		 * Set the Spinner disabled
		 * @param {Boolean} disabled - True if the Spinner should be disabled, false if the Spinner should be enabled.
		 */
		Spinner.prototype.setDisabled = function (disabled) {
			this._input.spinner("option", "disabled", disabled);
		};

		/**
		 * Set the tool tip of the Spinner
		 * @param {String} tooltip - Tool tip text
		 */
		Spinner.prototype.setTooltip = function (tooltip) {
			if (this._tooltip === undefined) {
				this._tooltip = new Tooltip(this.f.i18n, this.element, this.f.getContext(), this.f.getContext(), this.f.isMobile(), tooltip);
			} else {
				this._tooltip.setTooltipContent(tooltip);
			}
		};

		/**
		 * Update one of the initial options.
		 * Currently supported: "min","max","step","page".
		 * @param {String} optionName - Name of the option, e.g. "max"
		 * @param {Object} value - New value for the option
		 */
		Spinner.prototype.setOption = function (optionName, newValue) {
			var oldValue = this["_" + optionName];
			if (oldValue !== undefined && oldValue !== newValue) {
				// update the current value
				this["_" + optionName] = newValue;
				// update the spinner
				this._input.spinner("option", optionName, newValue);
			}
		};

		/**
		 * Set the label of the spinner and update the DOM element
		 * @param {String} newValuet - new value of the label
		 */
		Spinner.prototype.setInputUnit = function (newValuet) {
			this._inputUnit = newValuet;
			this._labelUnit.empty();
			this._labelUnit.append(this._inputUnit);
		};



		return Spinner;
	});

define('web-viewer/layout/component/Slider',["component/tooltip/Tooltip",
		"web-viewer/layout/component/Spinner",
		"web-viewer/utils/ViewerUtils",
		"underscore",
		"jquery",
		"jqueryUI",
		"jqueryUITouchPunch"
	],
	function (Tooltip,
		Spinner,
		ViewerUtils,
		_,
		$) {
		"use strict";

		/**
		 * @class A slider is a component that has a slider bar and optionally a spinner next to it.
		 * The wrapping element to add to a component sits in Slider.element
		 * @exports Slider
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {Object} options -
		 *    {Number} min: minimum value of the slider (Default: 0)
		 *    {Number} max: maximum value of the slider (Default: 100)
		 *    {Number} step: step value of the slider (Default: 1)
		 *    {Function} onSliderChanged: called when the slider value is changed. Arguments: value (Required)
		 *    {Boolean} showInput: Show an input field next to the slider (Default: true)
		 *    {String} inputUnit: Unit of the input field (put behind the input field) (Default: %)
		 *    {String} tooltip: Tooltip text that should be shown when you hover over the component. (Optional)
		 */
		var Slider = function (f, options) {
			this.f = f;

			// Set Options
			this._min = options.min === undefined ? 0 : options.min;
			this._max = options.max === undefined ? 100 : options.max;
			this._step = options.step === undefined ? 1 : options.step;
			this._onSliderChanged = options.onSliderChanged;
			this._showInput = options.showInput === undefined ? true : options.showInput;
			this._inputUnit = options.inputUnit === undefined ? "%" : options.inputUnit;

			// Wrapper element
			this.element = $("<div />").addClass("sliderWrapper");

			// Add slider
			this._slider = $("<div />").addClass("slider");
			this.element.append(this._slider);
			this._slider.slider({
				slide: _.bind(function (event, ui) {
					this._setInputValue(ui.value);
					this._fireSliderChanged(ui.value);
				}, this),
				min: this._min,
				max: this._max,
				step: this._step
			});

			// Add the input field if needed
			if (this._showInput) {
				this._spinner = new Spinner(f, {
					min: this._min,
					max: this._max,
					step: this._step,
					page: this._step * 10,
					onSpinnerChanged: _.bind(function (value) {
						this._setSliderValue(value);
					}, this),
					inputUnit: this._inputUnit
				});
				this.element.append(this._spinner.element);
			}

			// Tooltip
			if (options.tooltip !== undefined) {
				this._tooltip = new Tooltip(this.f.i18n, this.element, this.f.getContext(), this.f.getContext(), this.f.isMobile(), options.tooltip);
			}
		};

		/**
		 * @private
		 * Fire the value to the listener
		 * @param {Number} value - New slider value
		 */
		Slider.prototype._fireSliderChanged = function (value) {
			this._onSliderChanged(value);
		};

		/**
		 * @private
		 * Set the value of the slider.
		 * @param {Number} value - New value of the slider
		 */
		Slider.prototype._setSliderValue = function (value) {
			if (this.getValue() !== value) {
				this._slider.slider("value", value);
				this._fireSliderChanged(value);
			}
		};

		/**
		 * @private
		 * Set the value of the spinner. It doesn't fire yet.
		 * @param {Number} value - New value of the spinner
		 */
		Slider.prototype._setInputValue = function (value) {
			if (this._showInput) {
				this._spinner.setValue(value);
			}
		};

		/**
		 * Set the value of the slider
		 * @param {Number} value - New value of the slider
		 */
		Slider.prototype.setValue = function (value) {
			if (this.getValue() !== value) {
				this._setSliderValue(value);
				this._setInputValue(value);
			}
		};

		/**
		 * Get the value of the slider
		 * @returns {Number} The value of the slider
		 */
		Slider.prototype.getValue = function () {
			return this._slider.slider("value");
		};

		/**
		 * Is the slider disabled?
		 * @returns {Boolean} True if the slider is disabled, false otherwise
		 */
		Slider.prototype.isDisabled = function () {
			return this._slider.slider("option", "disabled");
		};

		/**
		 * Set the slider disabled
		 * @param {Boolean} disabled - True if the slider should be disabled, false if the slider should be enabled.
		 */
		Slider.prototype.setDisabled = function (disabled) {
			this._slider.slider("option", "disabled", disabled);
			if (this._showInput) {
				this._input.spinner("option", "disabled", disabled);
			}
		};

		/**
		 * Set the tool tip of the slider
		 * @param {String} tooltip - Tool tip text
		 */
		Slider.prototype.setTooltip = function (tooltip) {
			if (this._tooltip === undefined) {
				this._tooltip = new Tooltip(this.f.i18n, this.element, this.f.getContext(), this.f.getContext(), this.f.isMobile(), tooltip);
			} else {
				this._tooltip.setTooltipContent(tooltip);
			}
		};

		/**
		 * Update one of the initial options.
		 * Currently supported: "min" and "max".
		 * @param {String} optionName - Name of the option, e.g. "max"
		 * @param {Object} value - New value for the option
		 */
		Slider.prototype.setOption = function (optionName, newValue) {
			var oldValue = this["_" + optionName];
			if (oldValue !== undefined && oldValue !== newValue) {
				// update the current value
				this["_" + optionName] = newValue;
				// update the slider
				this._slider.slider("option", optionName, newValue);

				// update the spinner if needed
				if (this._showInput) {
					this._spinner.setOption(optionName, newValue);
				}
			}
		};

		/**
		 * Get one of the options.
		 * Currently supported: "min" and "max".
		 * @param {String} optionName - Name of the option, e.g. "max"
		 * @returns {Object} value - The value of the option or undefined if not found
		 */
		Slider.prototype.getOption = function (optionName) {
			return this["_" + optionName];
		};

		return Slider;
	});

define('web-viewer/layout/component/DoubleColorBlock',["jquery"],
	function (
		$
	) {

		/**
		 * @class This component is an element that consists of two color blocks that are glued together.
		 * @exports DoubleColorBlock
		 * @constructor
		 * @param {String} [initialLeftColor] - Initial color of the left block (in hex with # format)
		 * @param {String} [initialRightColor] - Initial color of the right block (in hex with # format)
		 */
		var DoubleColorBlock = function (initialLeftColor, initialRightColor) {
			this.element = $("<div />").addClass("doubleColorBlock");

			this._leftBlock = $("<div />").addClass("part");
			this._rightBlock = $("<div />").addClass("part");

			if (initialLeftColor !== undefined) {
				this.setLeftColor(initialLeftColor);
			}

			if (initialRightColor !== undefined) {
				this.setRightColor(initialRightColor);
			}

			this.element.append(this._leftBlock);
			this.element.append(this._rightBlock);
		};

		/**
		 * Set the color of the left color block.
		 * @param {String} leftColor - Color for the left block (in hex with # format)
		 */
		DoubleColorBlock.prototype.setLeftColor = function (leftColor) {
			this._leftBlock.css("background-color", leftColor);
		};

		/**
		 * Set the color of the right color block.
		 * @param {String} rightColor - Color for the right block (in hex with # format)
		 */
		DoubleColorBlock.prototype.setRightColor = function (rightColor) {
			this._rightBlock.css("background-color", rightColor);
		};

		return DoubleColorBlock;
	});

define('web-viewer/compare/CompareDifferenceSubToolbarComponent',["component/toolbar/ToolbarComponent",
		"web-viewer/layout/component/SubtoolbarButton",
		"web-viewer/layout/component/DropdownSubtoolbarButton",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/Slider",
		"web-viewer/layout/component/ColorPicker",
		"web-viewer/layout/component/DoubleColorBlock",
		"web-viewer/compare/CompareConstants",
		"web-viewer/utils/ListenerUtil",
		"underscore"
	],
	function (ToolbarComponent,
		SubtoolbarButton,
		DropdownSubtoolbarButton,
		ButtonGroup,
		Slider,
		ColorPicker,
		DoubleColorBlock,
		CompareConstants,
		ListenerUtil,
		_) {
		"use strict";

		/**
		 * @memberof CompareModule
		 * @exports CompareDifferenceSubToolbarComponent
		 * @class subtoolbarcomponent used for interaction with the difference compare mode
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {CompareModule.CompareDifferenceMode} compareDiffMode - the mode that should be controlled
		 * @param {Layout.ViewerToolbarButton} parent - model for compareparent button
		 */
		var CompareDifferenceSubToolbarComponent = function (f, compareDiffMode, parent) {
			ListenerUtil.mixin(this, f);
			ToolbarComponent.call(this, "DifferenceCompareOptions", f);
			this.f = f;
			this.compareDiffMode = compareDiffMode;

			this.parent = parent;

			this._initColorSpace();
			this._initThresholdSlider();
			this._initHighlightColor();
			this._initDiffmode();

			this.f.getDocumentController().whenActivePageChanged(_.bind(function (oldPage, newPage) {
				this._whenActivePageChanged(oldPage, newPage);
			}, this));
		};

		CompareDifferenceSubToolbarComponent.prototype = Object.create(ToolbarComponent.prototype);
		CompareDifferenceSubToolbarComponent.prototype.constructor = CompareDifferenceSubToolbarComponent;


		/**
		 * The active page has changed
		 * @param {Data.AbstractDocumentPage} oldPage
		 * @param {Data.AbstractDocumentPage} newPage
		 */
		CompareDifferenceSubToolbarComponent.prototype._whenActivePageChanged = function (oldPage, newPage) {
			if (newPage.hasCad()) {
				this.colorspaceGroup.hide();
				this.differenceShowModeGroup.hide();
				this.slider.element.addClass("displayNone");
			} else {
				this.colorspaceGroup.show();
				this.differenceShowModeGroup.show();
				this.slider.element.removeClass("displayNone");
			}
		};


		/**
		 * Initialize the colorspace
		 */
		CompareDifferenceSubToolbarComponent.prototype._initColorSpace = function () {
			this.colorSpaceButtons = [];
			var isMobile = this.f.isMobile();
			// colorspace 1: channels
			this.colorSpaceButtons[0] = new SubtoolbarButton(this.f,
				"colorspaceChannels",
				"icon-WCR_Compare-Channels",
				true,
				_.bind(this.compareDiffMode.setColorSpace, this.compareDiffMode, CompareConstants.colorSpace.CHANNELS)
			);


			// colorspace 2: HSV
			this.colorSpaceButtons[1] = new SubtoolbarButton(this.f,
				"colorspaceHSV",
				"icon-WCR_Compare-HSV",
				true,
				_.bind(this.compareDiffMode.setColorSpace, this.compareDiffMode, CompareConstants.colorSpace.HSV)
			);

			// colorspace 3: automatic
			this.colorSpaceButtons[2] = new SubtoolbarButton(this.f,
				"colorspaceAutomatic",
				"icon-WCR_Compare-Automatic",
				true,
				_.bind(this.compareDiffMode.setColorSpace, this.compareDiffMode, CompareConstants.colorSpace.AUTOMATIC)
			);


			// tool tips
			this.f.i18n.loadBundle("viewer", function () {
				var prefix = this.f.i18n.t("DiffColorSpace") + " ";
				this.colorSpaceButtons[0].setTooltip(prefix + this.f.i18n.t("DiffColorSpaceChannels"));
				this.colorSpaceButtons[1].setTooltip(prefix + this.f.i18n.t("DiffColorSpaceHsv"));
				this.colorSpaceButtons[2].setTooltip(prefix + this.f.i18n.t("DiffColorSpaceAutomatic"));
			}, this);

			this.colorspaceGroup = new ButtonGroup("colorspace", this.colorSpaceButtons);
			this.colorspaceGroup.element.addClass("toggleBtnGroup");
			this.addButtonGroup(this.colorspaceGroup);


			this.compareDiffMode.on("changed.colorspace", _.bind(this.updateColorSpaceButtons, this));
			this.updateColorSpaceButtons(0, this.compareDiffMode.getColorSpace());
		};

		/**
		 * Update the color space buttons
		 * @param oldColorSpace
		 * @param newColorSpace
		 */
		CompareDifferenceSubToolbarComponent.prototype.updateColorSpaceButtons = function (oldColorSpace, newColorSpace) {
			_.each(this.colorSpaceButtons, function (button, index) {
				// the colorspace options are 1,2,3 and array indexes are 0,1,2 thus +1
				if (newColorSpace === CompareConstants.colorSpace.DEFAULT) {
					// note we have the same icon for colorspace 0 (default) and 1 (channels)
					newColorSpace = CompareConstants.colorSpace.CHANNELS;
				}
				button.setActive(index + 1 === newColorSpace);
			}, this);
		};

		/**
		 * Initialize threshold slider
		 */
		CompareDifferenceSubToolbarComponent.prototype._initThresholdSlider = function () {
			this.slider = new Slider(this.f, {
				onSliderChanged: _.bind(_.debounce(function () {
					this.compareDiffMode.setThreshold(this.slider.getValue());
				}, 250), this)
			});

			this.slider.setValue(this.compareDiffMode.getThreshold()); // initial value

			// synchronize between model and slider
			this._registerListener("changed.threshold", function (oldValue, newValue) {
				this.slider.setValue(newValue);
			}, this, this.compareDiffMode);

			this.f.i18n.loadBundle("viewer", function () {
				this.slider.setTooltip(this.f.i18n.t("ChangeDifferenceThresholdTT"));
			}, this);
			this.element.append(this.slider.element);
		};



		/**
		 * Initialize the addSubtractedColorConfiguration
		 */
		CompareDifferenceSubToolbarComponent.prototype._initHighlightColor = function () {
			if (!this.f.isMobile()) {
				/**
				 * Highlight Color changes
				 */
				this.diffColorChangeGroup = new ButtonGroup("diffColorChangeGroup");

				this.addButtonGroup(this.diffColorChangeGroup);


				// Main button to change high light colors
				var mainDiffColorChangeIcon = new DoubleColorBlock(this.compareDiffMode.getAddedHighlightColor(), this.compareDiffMode.getSubtractedHighlightColor());
				var mainDiffColorChangeButton = new DropdownSubtoolbarButton(this.f, "ChangeDiffColor", mainDiffColorChangeIcon.element, true);
				mainDiffColorChangeButton.setTooltip("viewer", "ChangeDifferenceHighlightColorsTT");
				this.diffColorChangeGroup.addButton(mainDiffColorChangeButton);

				// Submenu options		

				// icons
				// added highlight color
				var changeAddedHighlightColorIcon = new ColorPicker(_.bind(this.compareDiffMode.setAddedHighlightColor, this.compareDiffMode));
				changeAddedHighlightColorIcon.setColor(this.compareDiffMode.getAddedHighlightColor());
				// subtracted highlight color
				var changeSubtractedHighlightColorIcon = new ColorPicker(_.bind(this.compareDiffMode.setSubtractedHighlightColor, this.compareDiffMode));

				changeSubtractedHighlightColorIcon.setColor(this.compareDiffMode.getSubtractedHighlightColor());
				// both highlight colors
				var changeBothHighlightColorBlock = new DoubleColorBlock(this.compareDiffMode.getAddedHighlightColor(), this.compareDiffMode.getSubtractedHighlightColor());
				var changeBothHighlightColorIcon = new ColorPicker(_.bind(function (newColor) {
					this.compareDiffMode.setAddedHighlightColor(newColor);
					this.compareDiffMode.setSubtractedHighlightColor(newColor);
				}, this), changeBothHighlightColorBlock.element);
				changeBothHighlightColorIcon.setColor(this.compareDiffMode.getAddedHighlightColor());


				// react on changes -> update all icons
				this.compareDiffMode.on("changed.addedHighlightColor", function (oldValue, newValue) {
					changeAddedHighlightColorIcon.setColor(newValue);
					mainDiffColorChangeIcon.setLeftColor(newValue);
					changeBothHighlightColorBlock.setLeftColor(newValue);
				});
				this.compareDiffMode.on("changed.subtractedHighlightColor", function (oldValue, newValue) {
					changeSubtractedHighlightColorIcon.setColor(newValue);
					mainDiffColorChangeIcon.setRightColor(newValue);
					changeBothHighlightColorBlock.setRightColor(newValue);
				});

				// options
				this.f.i18n.loadBundle("viewer", function () {
					mainDiffColorChangeButton.addAction({
						name: "changeAddedHighlightColor",
						icon: changeAddedHighlightColorIcon.element,
						content: this.f.i18n.t('DiffAddColor'),
						enabled: true,
						action: function () {
							changeAddedHighlightColorIcon.show();
						}
					});

					mainDiffColorChangeButton.addAction({
						name: "changeSubtractedHighlightColor",
						icon: changeSubtractedHighlightColorIcon.element,
						content: this.f.i18n.t('DiffSubColor'),
						enabled: true,
						action: function () {
							changeSubtractedHighlightColorIcon.show();
						}
					});

					mainDiffColorChangeButton.addAction({
						name: "changeBothHighlightColor",
						icon: changeBothHighlightColorIcon.element,
						content: this.f.i18n.t('DiffBothColors'),
						enabled: true,
						action: function () {
							changeBothHighlightColorIcon.show();
						}
					});
				}, this);


				// Reset highlight color changes
				var resetDiffColorButton = new SubtoolbarButton(this.f, "ResetDiffColor", "icon-WCR_WF-Undo", true, _.bind(this.compareDiffMode.resetHighlightColors, this.compareDiffMode));
				resetDiffColorButton.setTooltip("viewer", "DiffResetColor");
				this.diffColorChangeGroup.addButton(resetDiffColorButton);
			}
		};

		/**
		 * Initialize the Difference mode selection (show all, highlight & only difference)
		 */
		CompareDifferenceSubToolbarComponent.prototype._initDiffmode = function () {
			/**
			 * Mode buttons
			 */
			this.showModeButtons = [];

			var isMobile = this.f.isMobile();

			// dmode 2: view both
			this.showModeButtons[0] = new SubtoolbarButton(this.f,
				"differenceShow",
				"icon-WCR_Compare-BothDiff",
				true,
				_.bind(this.setDifferenceMode, this, CompareConstants.showDifferenceMode.SHOW)
			);

			// dmode 3: view both but highlight difference
			this.showModeButtons[1] = new SubtoolbarButton(this.f,
				"differenceHighlight",
				"icon-WCR_Compare-HighlightDiff",
				true,
				_.bind(this.setDifferenceMode, this, CompareConstants.showDifferenceMode.HIGHLIGHT)
			);

			// dmode 4: view both but only difference
			this.showModeButtons[2] = new SubtoolbarButton(this.f,
				"differenceOnly",
				"icon-WCR_Compare-ShowDiff",
				true,
				_.bind(this.setDifferenceMode, this, CompareConstants.showDifferenceMode.ONLY)
			);

			// tool tips
			this.f.i18n.loadBundle("viewer", function () {
				this.showModeButtons[0].setTooltip(this.f.i18n.t("DiffBothTT"));
				this.showModeButtons[1].setTooltip(this.f.i18n.t("DiffBothHighlightTT"));
				this.showModeButtons[2].setTooltip(this.f.i18n.t("DiffBothShowTT"));
			}, this);

			this.differenceShowModeGroup = new ButtonGroup("differenceShowMode", this.showModeButtons);
			this.differenceShowModeGroup.element.addClass("toggleBtnGroup");
			this.addButtonGroup(this.differenceShowModeGroup);

			this.compareDiffMode.on("changed.differenceMode", _.bind(this.updateDifferenceModeButtons, this));
			this.updateDifferenceModeButtons(0, this.compareDiffMode.getDifferenceMode());
		};

		/**
		 * Update the difference mode buttons (activate/deactivate them)
		 * @param {CompareConstants.showDifferenceMode} oldMode
		 * @param {CompareConstants.showDifferenceMode} newMode
		 */
		CompareDifferenceSubToolbarComponent.prototype.updateDifferenceModeButtons = function (oldMode, newMode) {
			var diffModeIcons = ["icon-WCR_Compare-BothDiff", "icon-WCR_Compare-HighlightDiff", "icon-WCR_Compare-ShowDiff"];

			// difference mode 0 --> 2, 1 --> 3, 2 --> 4 thus +2
			_.each(this.showModeButtons, function (button, index) {
				button.setActive(index + 2 === newMode);
				button.element.removeClass("displayNone");
			});

			this.updateParentIcon(diffModeIcons[newMode - 2]);
		};

		/**
		 * Update the icon of the parent button
		 * @param {String} icon
		 */
		CompareDifferenceSubToolbarComponent.prototype.updateParentIcon = function (icon) {
			if (this.parent) {
				this.parent.setIcon(icon);
			}
		};

		/**
		 * Change the difference mode
		 * @param mode
		 */
		CompareDifferenceSubToolbarComponent.prototype.setDifferenceMode = function (mode) {
			this.compareDiffMode.setDifferenceMode(mode);
		};


		return CompareDifferenceSubToolbarComponent;
	});

define('web-viewer/compare/CompareSideBySideSubToolbarComponent',["web-viewer/compare/CompareDifferenceSubToolbarComponent",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/SubtoolbarButton",
		"web-viewer/layout/component/DropdownSubtoolbarButton",
		"web-viewer/viewport/ViewportConstants",
		"jquery",
		"underscore"
	],
	function (CompareDifferenceSubToolbarComponent,
		ButtonGroup,
		SubtoolbarButton,
		DropdownSubtoolbarButton,
		ViewportConstants,
		$,
		_) {
		"use strict";
		/**
		 * @memberof CompareModule
		 * @exports CompareSideBySideSubToolbarComponent
		 * @class A subtoolbare component for the side by side compare
		 * @augments CompareModule.CompareDifferenceSubToolbarComponent
		 * @constructor
		 * @param {Core.ViewerFacade}
		 * @param {CompareModule.CompareDifferenceMode} compareDiffMode - compare difference mode
		 * @param {CompareModule.CompareSideBySideMode} compareSideBySideMode - compare side by side mode
		 * @param {Layout.ToolbarComponent} parent - parent toolbar component
		 *
		 */
		var CompareSideBySideSubToolbarComponent = function (f, compareDiffMode, compareSideBySideMode, parent) {
			this._sideBySideMode = compareSideBySideMode;
			CompareDifferenceSubToolbarComponent.call(this, f, compareDiffMode, parent);

			// panel tiling options
			this._tileIcons = ["icon-WCR_Tiling-Auto", "icon-WCR_Tiling-Vertical", "icon-WCR_Tiling-Horizontal"];
			this._tileButton = new DropdownSubtoolbarButton(this.f, "sideBySideMode", this.getTileIcon(), true);
			this._tileButton.setTooltip("viewer", "diffChangeSideBySideTilingMode");

			// options
			this.f.i18n.loadBundle("viewer", function () {
				this._tileButton.addAction({
					name: "automaticTiling",
					icon: "icon-WCR_Tiling-Auto",
					content: this.f.i18n.t('DiffSplitTileAuto'),
					enabled: true,
					action: _.bind(function () {
						this._sideBySideMode.setPanelsLayoutMode(ViewportConstants.PanelsLayoutMode.AUTOMATIC);
					}, this)
				});

				this._tileButton.addAction({
					name: "verticalTiling",
					icon: "icon-WCR_Tiling-Vertical",
					content: this.f.i18n.t('DiffSplitTileVer'),
					enabled: true,
					action: _.bind(function () {
						this._sideBySideMode.setPanelsLayoutMode(ViewportConstants.PanelsLayoutMode.VERTICAL);
					}, this)
				});

				this._tileButton.addAction({
					name: "horizontalTiling",
					icon: "icon-WCR_Tiling-Horizontal",
					content: this.f.i18n.t('DiffSplitTileHor'),
					enabled: true,
					action: _.bind(function () {
						this._sideBySideMode.setPanelsLayoutMode(ViewportConstants.PanelsLayoutMode.HORIZONTAL);
					}, this)
				});
			}, this);



			if (!this.f.isMobile()) {
				this._tilingModeButtonGroup = new ButtonGroup("tilingmode", [this._tileButton]);
				this.prependButtonGroup(this._tilingModeButtonGroup);
			}


			// add hide difference panel button
			this.toggleDifferenceSidePanelButton = new SubtoolbarButton(this.f,
				"toggleDifferencePanel",
				"icon-WCR_Compare-HideDiff",
				false,
				_.bind(this.toggleDifferencePanel, this)
			);

			this.toggleDifferenceSidePanelButton.setTooltip("viewer", "compareDisableDifferenceView");
			// tool tips
			this.f.i18n.loadBundle("viewer", function () {
				var showDiff = this.f.i18n.t("DiffBothSplitShowDiff") + ": <br />\r\n";
				this.showModeButtons[0].setTooltip(showDiff + this.f.i18n.t("DiffBothTT"));
				this.showModeButtons[1].setTooltip(showDiff + this.f.i18n.t("DiffBothHighlightTT"));
				this.showModeButtons[2].setTooltip(showDiff + this.f.i18n.t("DiffBothShowTT"));

			}, this);

			this.differenceShowModeGroup.prependButton(this.toggleDifferenceSidePanelButton);

			// when side by side mode has changed, update the controls
			this._sideBySideMode.on("changed.*", _.bind(function () {
				this.updateControls();
			}, this));


			this.updateControls();
		};

		CompareSideBySideSubToolbarComponent.prototype = Object.create(CompareDifferenceSubToolbarComponent.prototype);
		CompareSideBySideSubToolbarComponent.prototype.constructor = CompareSideBySideSubToolbarComponent;

		/**
		 * @override
		 */
		CompareSideBySideSubToolbarComponent.prototype.updateParentIcon = function () {
			// do not update the parent icon for this mode...
		};

		/**
		 * Update the controls for the toolbar component
		 *
		 * If difference compare mode is not shown, hide the controls for this mode
		 */
		CompareSideBySideSubToolbarComponent.prototype.updateControls = function () {
			this.differenceShowModeGroup.show();

			if (!this._sideBySideMode.isShowingDifference()) {
				// disabled difference controls
				this.colorspaceGroup.hide();
				this.slider.element.addClass("displayNone");
				if (this.diffColorChangeGroup) {
					this.diffColorChangeGroup.hide();
				}
			} else {
				//diff panel is visible
				if (this.diffColorChangeGroup) {
					this.diffColorChangeGroup.show();
				}
				var page = this.f.getDocumentController().getActivePage();
				if (page && !page.hasCad()) {
					this.colorspaceGroup.show();
					this.slider.element.removeClass("displayNone");
				}
			}

			if (this._tileButton) {
				this._tileButton.setIcon(this.getTileIcon());
			}

			this.updateDifferenceModeButtons();
			this.element.trigger("contentResized");
		};

		/**
		 * When the difference mode is changed, interfere and disable show difference compare mode if the mode being set is the same as the current one
		 * @override
		 * @param {CompareConstants.showDifferenceMode} mode
		 */
		CompareSideBySideSubToolbarComponent.prototype.setDifferenceMode = function (mode) {
			this._sideBySideMode.setShowingDifference(true);

			// old behavior of setDifferenceMode
			CompareDifferenceSubToolbarComponent.prototype.setDifferenceMode.call(this, mode);
		};

		/**
		 * When the acitve page changes, we also want to update the controls...
		 * @override
		 * @param oldPage
		 * @param newPage
		 */
		CompareSideBySideSubToolbarComponent.prototype._whenActivePageChanged = function (oldPage, newPage) {
			CompareDifferenceSubToolbarComponent.prototype._whenActivePageChanged.call(this, oldPage, newPage);
			this.updateControls();
		};

		/**
		 * Update the difference compare buttons
		 * (set all buttons unactive if there is no compare difference shown)
		 * @override
		 */
		CompareSideBySideSubToolbarComponent.prototype.updateDifferenceModeButtons = function () {
			if (this._sideBySideMode.isShowingDifference()) {
				CompareDifferenceSubToolbarComponent.prototype.updateDifferenceModeButtons.call(this, undefined, this.compareDiffMode.getDifferenceMode());
			} else {
				_.each(this.showModeButtons, function (button, index) {
					button.setActive(false);
				});
			}

			if (this.toggleDifferenceSidePanelButton) {
				this.toggleDifferenceSidePanelButton.setActive(!this._sideBySideMode.isShowingDifference());
			}

			// hide the difference mode buttons for non cad files
			var page = this.f.getDocumentController().getActivePage();
			if (page && !page.hasCad()) {
				// graphics
				_.each(this.showModeButtons, function (el) {
					el.element.removeClass("displayNone");
					el.element.removeClass("endRoundedCorner");
				});
			} else {
				// CAD
				_.each(this.showModeButtons, function (el, index) {
					if (index > 0) {
						el.element.addClass("displayNone");
					} else {
						el.element.addClass("endRoundedCorner");
					}
				});
			}
		};

		/**
		 * Get the icon name for the currently active layout mode
		 * @returns {String}
		 */
		CompareSideBySideSubToolbarComponent.prototype.getTileIcon = function () {
			return this._tileIcons[this._sideBySideMode.getPanelsLayoutMode()];
		};


		/**
		 * The toggle button for the difference side by side panel is clicked
		 */
		CompareSideBySideSubToolbarComponent.prototype.toggleDifferencePanel = function () {
			if (this._sideBySideMode.isShowingDifference()) {
				this._sideBySideMode.setShowingDifference(false);
			}
		};

		return CompareSideBySideSubToolbarComponent;
	});

define('web-viewer/compare/CompareToolbarComponent',["component/toolbar/ToolbarComponent",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/compare/CompareConstants",
		"web-viewer/compare/CompareDifferenceSubToolbarComponent",
		"web-viewer/compare/CompareSideBySideSubToolbarComponent",
		"underscore"
	],
	function (ToolbarComponent,
		ButtonGroup,
		ViewerToolbarButton,
		LayoutConstants,
		CompareConstants,
		CompareDifferenceSubToolbarComponent,
		CompareSideBySideSubToolbarComponent,
		_) {
		"use strict";

		/**
		 * create a new compare toolbar component
		 * @memberof CompareModule
		 * @exports CompareToolbarComponent
		 * @class The toolbar component for the compare module
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {CompareModule.CompareModel} compareModel
		 */
		var CompareToolbarComponent = function (f, compareModule) {
			ToolbarComponent.call(this, LayoutConstants.ToolbarComponent.COMPARE, f);
			this.f = f;
			this.compareModule = compareModule;
			this.compareModel = compareModule.getCompareModel();
			this.buttons = {};
			this.modesGroup = new ButtonGroup("Compare", []);
			this.modesGroup.setTitle(this.f.i18n, "viewer", "CompareToolsTitle");
			this.addButtonGroup(this.modesGroup);

			this.compareModel.on("changed.mode", _.bind(function (oldMode, newMode) {
				if (oldMode && this.buttons[oldMode.getName()] !== undefined) {
					this.buttons[oldMode.getName()].setActive(false);
				}

				if (newMode && this.buttons[newMode.getName()] !== undefined) {
					this.buttons[newMode.getName()].setActive(true);
				}
			}, this));

			this.addButtonForMode(CompareConstants.mode.SHOW_CURRENT, "icon-WCR_Compare-Current", "DiffCurrentTT");
			this.addButtonForMode(CompareConstants.mode.SHOW_DIFFERENCE, "icon-WCR_Compare-Reference", "DiffPrevTT");
			this.addButtonForMode(CompareConstants.mode.BLINKING, "icon-WCR_Compare-Blinking", "DiffBothBlinkingTT");
			this.addButtonForMode(CompareConstants.mode.SHOWBOTHONTOP, "icon-WCR_Compare", "DiffShowBothOnTopTT");

			var differenceCompareMode = this.compareModel.getModeByName(CompareConstants.mode.DIFFERENCE);
			if (differenceCompareMode !== undefined) {
				this.addDifferenceModeButton(differenceCompareMode);

				var updateBadges = _.debounce(_.bind(this._updateBadges, this), 0);

				differenceCompareMode.on("changed.threshold", updateBadges);
				this.compareModel.on("changed.mode", updateBadges);
				this.f.getDocumentController().whenActivePageChanged(updateBadges);

				var sideBySideCompareMode = this.compareModel.getModeByName(CompareConstants.mode.SIDEBYSIDE);
				if (sideBySideCompareMode !== undefined) {
					this.addSideBySideModeButton(differenceCompareMode, sideBySideCompareMode);
					sideBySideCompareMode.on("changed.showDifference", updateBadges);
				}

				this._updateBadges();
			}



		};

		CompareToolbarComponent.prototype = Object.create(ToolbarComponent.prototype);
		CompareToolbarComponent.prototype.constructor = CompareToolbarComponent;
		CompareToolbarComponent.toolbarName = LayoutConstants.ToolbarComponent.COMPARE;


		/**
		 * Add a button to active/deactivate a mode
		 * @param {String|CompareModule.CompareConstants.mode} name - name of the compare mode
		 * @param {String} icon - name of the icon
		 * @param {String} tooltip - tooltip
		 */
		CompareToolbarComponent.prototype.addButtonForMode = function (name, icon, tooltip) {
			this.buttons[name] = new ViewerToolbarButton(this.f, "compare_" + name, icon, true, _.partial(function (that) {
				if (this.isActive()) {
					that.compareModule.activateModeByName();
				} else {
					that.compareModule.activateModeByName(name);
				}
			}, this));
			this.buttons[name].setTooltip("viewer", tooltip);
			this.modesGroup.addButton(this.buttons[name]);
		};


		/**
		 * Difference compare mode button
		 * @param differenceCompareMode
		 */
		CompareToolbarComponent.prototype.addDifferenceModeButton = function (differenceCompareMode) {
			this.addButtonForMode(CompareConstants.mode.DIFFERENCE, "icon-WCR_Compare-BothDiff", "compareDifferenceMode");

			var button = this.buttons[CompareConstants.mode.DIFFERENCE];
			button.addSubtoolbarComponent(new CompareDifferenceSubToolbarComponent(this.f, differenceCompareMode, button));

			this._differenceCompareMode = differenceCompareMode;
		};


		/**
		 * Side by side compare mode button
		 * @param {CompareModule.CompareDifferenceMode} differenceCompareMode
		 * @param {CompareModule.CompareSideBySideMode} sideBySideCompareMode
		 */
		CompareToolbarComponent.prototype.addSideBySideModeButton = function (differenceCompareMode, sideBySideCompareMode) {
			this.addButtonForMode(CompareConstants.mode.SIDEBYSIDE, "icon-WCR_Compare2-SideBySide", "DiffBothSplitTT");

			var button = this.buttons[CompareConstants.mode.SIDEBYSIDE];
			button.addSubtoolbarComponent(new CompareSideBySideSubToolbarComponent(this.f, differenceCompareMode, sideBySideCompareMode, button));
			this._sideBySideCompareMode = sideBySideCompareMode;
		};


		/**
		 * Update the Warning Badges for the Threshold for difference and side by side mode
		 */

		CompareToolbarComponent.prototype._updateBadges = function () {
			var button;

			if (this.buttons[CompareConstants.mode.DIFFERENCE] && this._differenceCompareMode) {
				// difference compare mode is available
				button = this.buttons[CompareConstants.mode.DIFFERENCE];
				if (this._differenceCompareMode.isEnabled() && this._differenceCompareMode.getThreshold() > 0 && !this.f.getActivePage().hasCad()) {
					// show warning
					button.setBadge("!", "warning");
					button.setTooltip("viewer", "compareDifferenceThresholdWarning");
				} else {
					button.setBadge();
					button.setTooltip("viewer", "compareDifferenceMode");
				}

			}

			if (this.buttons[CompareConstants.mode.SIDEBYSIDE]) {
				// side by side compare mode is available
				button = this.buttons[CompareConstants.mode.SIDEBYSIDE];

				if (this._sideBySideCompareMode.isEnabled() && this._sideBySideCompareMode.isShowingDifference() && this._differenceCompareMode.getThreshold() > 0 && !this.f.getActivePage().hasCad()) {
					button.setBadge("!", "warning");
					button.setTooltip("viewer", "compareDifferenceThresholdWarning");
				} else {
					button.setBadge();
					button.setTooltip("viewer", "DiffBothSplitTT");
				}
			}
		};


		return CompareToolbarComponent;
	});

define('web-viewer/compare/CompareTouchToolbarComponent',["component/toolbar/ToolbarComponent",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/compare/CompareConstants",
		"underscore"
	],
	function (ToolbarComponent,
		ButtonGroup,
		ViewerToolbarButton,
		LayoutConstants,
		CompareConstants,
		_) {
		"use strict";

		/**
		 * create a new compare toolbar component
		 * @memberof CompareModule
		 * @exports CompareTouchToolbarComponent
		 * @class The toolbar component for the compare module
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {CompareModule.CompareModel} compareModel
		 */
		var CompareTouchToolbarComponent = function (f, compareModule) {
			ToolbarComponent.call(this, LayoutConstants.ToolbarComponent.COMPARE, f);
			this.f = f;
			this._compareModule = compareModule;
			this._compareModel = compareModule.getCompareModel();
			this._buttons = {};
			this._modesGroup = new ButtonGroup("Compare", []);
			this._modesGroup.element.removeClass("btn-group");
			this._modesGroup.element.addClass("compareModes");
			this._modesGroup.setTitle(this.f.i18n, "viewer", "CompareToolsTitle");
			this.addButtonGroup(this._modesGroup);



			var differenceCompareMode = this._compareModel.getModeByName(CompareConstants.mode.DIFFERENCE);
			this.addDifferenceModeButton(differenceCompareMode);

			this.addButtonForMode(CompareConstants.mode.BLINKING, "icon-WCR_Compare-Blinking", "DiffBothBlinkingTT");


			var sideBySideCompareMode = this._compareModel.getModeByName(CompareConstants.mode.SIDEBYSIDE);
			this.addSideBySideModeButton(differenceCompareMode, sideBySideCompareMode);



			this.addButtonForMode(CompareConstants.mode.SHOWBOTHONTOP, "icon-WCR_Compare-Both", "DiffShowBothOnTopTT");

			this.addButtonForMode(CompareConstants.mode.SHOW_CURRENT, "icon-WCR_Compare-Current", "DiffCurrentTT");
			this.addButtonForMode(CompareConstants.mode.SHOW_DIFFERENCE, "icon-WCR_Compare-Reference", "DiffPrevTT");


			this._compareModel.on("changed.mode", _.bind(this._updateActiveStatus, this));
			this._updateActiveStatus(undefined, this._compareModule.getActiveMode());



		};

		CompareTouchToolbarComponent.prototype = Object.create(ToolbarComponent.prototype);
		CompareTouchToolbarComponent.prototype.constructor = CompareTouchToolbarComponent;
		CompareTouchToolbarComponent.toolbarName = LayoutConstants.ToolbarComponent.COMPARE;


		/**
		 * Update the active flag for the compare mode buttons
		 * @param {String} oldMode 
		 * @param {String} newMode 
		 */
		CompareTouchToolbarComponent.prototype._updateActiveStatus = function (oldMode, newMode) {
			if (oldMode && this._buttons[oldMode.getName()] !== undefined) {
				this._buttons[oldMode.getName()].setActive(false);
			}

			if (newMode && this._buttons[newMode.getName()] !== undefined) {
				this._buttons[newMode.getName()].setActive(true);
			}
		};

		/**
		 * Add a button to active/deactivate a mode
		 * @param {String|CompareModule.CompareConstants.mode} name - name of the compare mode
		 * @param {String} icon - name of the icon
		 * @param {String} tooltip - tooltip
		 */
		CompareTouchToolbarComponent.prototype.addButtonForMode = function (name, icon, tooltip) {
			this._buttons[name] = new ViewerToolbarButton(this.f, "compare_" + name, icon, true, _.partial(function (that) {
				if (this.isActive()) {
					that._compareModule.activateModeByName();
				} else {
					that._compareModule.activateModeByName(name);
				}
			}, this));
			this._buttons[name].setTooltip("viewer", tooltip);
			this._modesGroup.addButton(this._buttons[name]);
		};


		/**
		 * Difference compare mode button
		 * @param differenceCompareMode
		 */
		CompareTouchToolbarComponent.prototype.addDifferenceModeButton = function (differenceCompareMode) {
			this.addButtonForMode(CompareConstants.mode.DIFFERENCE, "icon-WCR_Compare-BothDiff", "compareDifferenceMode");


		};


		/**
		 * Side by side compare mode button
		 * @param {CompareModule.CompareDifferenceMode} differenceCompareMode
		 * @param {CompareModule.CompareSideBySideMode} sideBySideCompareMode
		 */
		CompareTouchToolbarComponent.prototype.addSideBySideModeButton = function (differenceCompareMode, sideBySideCompareMode) {
			this.addButtonForMode(CompareConstants.mode.SIDEBYSIDE, "icon-WCR_Compare-SideBySide", "DiffBothSplitTT");

		};
		return CompareTouchToolbarComponent;
	});

define('web-viewer/layout/component/UserUnitInputField',["web-viewer/utils/ViewerUtils",
		"web-viewer/core/CoreConstants",
		"underscore",
		"jquery"
	],
	function (ViewerUtils,
		CoreConstants,
		_,
		$) {
		"use strict";
		/**
		 * An input field that adapts to the user preference UNIT SIZE
		 * It allows the easily get input from the users (as a mm value)
		 * @memberof LayoutModule
		 * @param {ViewerFacade}         f         - the ViewerFacade
		 * @param {Number|String}        value     - the initial mm value of the field
		 * @param {String}               direction - x or y the direction of the value (needed because resolutionx and resolutiony can be different in 1 file)
		 * @param {Function}             onChange  - callback called when the value has changed. Its parameters are oldValue, newValue (both in mm)
		 */
		var UserUnitInputField = function (f, value, direction, onChange) {
			this.f = f;
			this.element = $("<span>").addClass("userUnitInputField");
			this._input = $("<input>").attr("type", "number").attr("step", "any").appendTo(this.element);
			this._unitLabel = $("<span>").addClass("unitLabel").appendTo(this.element);

			this._direction = direction;


			value = ViewerUtils.parseFloat(value);

			this._mmValue = value;

			this.f.getCurrentUser().on("options.changed.PREFERENCES_UNIT_SIZE", _.bind(this._renderField, this));
			this._input.on("blur keydown", _.bind(this._inputChanged, this));
			this._renderField();

			this._onChange = onChange;
		};


		/**
		 * Set the value (in MM)
		 * @param {Number} value - value in mm
		 */
		UserUnitInputField.prototype.setValue = function (value) {
			var oldValue = this._mmValue;

			this._mmValue = ViewerUtils.parseFloat(value);
			this._renderField();

			if (oldValue !== this._mmValue) {
				this._onChange(oldValue, this._mmValue);
			}
		};

		/**
		 * Render the field
		 */
		UserUnitInputField.prototype._renderField = function () {
			var convertedValue = ViewerUtils.convertFromMmValue(this._mmValue, this.f.getUnits(), this.f.getUnitsFix(), this._getResolution());
			this._input.val(convertedValue);

			this.f.loadBundle("viewer", function () {
				this._unitLabel.text(this.f.t(this.f.getUnitsLocName()));
			}, this);
		};

		/**
		 * Get the resolution for unit conversion
		 * @returns {Number} resolution of the page for the needed conversion
		 */
		UserUnitInputField.prototype._getResolution = function () {
			var page = this.f.getActivePage();
			if (page) {
				if (this._direction === "y") {
					return ViewerUtils.parseFloat(page.getParam("resolutiony"));
				} else if (this._direction === "x") {
					return ViewerUtils.parseFloat(page.getParam("resolutionx"));
				}
			}

			// by default return 1
			return 1;
		};

		/**
		 * The user changed the input field, update the mmValue and send out correct event
		 * @param {Object} event - Event Object
		 */
		UserUnitInputField.prototype._inputChanged = function (event) {
			if (event.which == 13 || event.keyCode == 13 || event.type !== "keydown") {
				var value = ViewerUtils.parseFloat(this._input.val());
				if (!value) {
					value = 0;
				}

				this._input.val(value);
				var oldValue = this._mmValue;
				this._mmValue = ViewerUtils.parseFloat(ViewerUtils.convertToMmValue(value, this.f.getUnits(), this.f.getUnitsFix(CoreConstants.SizeUnits.MM), this._getResolution()));

				if (this._mmValue != oldValue) {
					this._onChange(oldValue, this._mmValue);
				}
				this._input.blur();
			}
		};

		/**
		 * Show/hide the unit label
		 * @param {Boolean} toggle
		 */
		UserUnitInputField.prototype.toggleLabel = function (toggle) {
			this._unitLabel.toggleClass("displayNone", !toggle);
		};

		/**
		 * Get the value of the field (in mm)
		 * @returns {Number} the value of the field in mm
		 */
		UserUnitInputField.prototype.getValue = function () {
			return this._mmValue;
		};

		return UserUnitInputField;
	});

define('web-viewer/compare/CompareWidget',[
		"web-viewer/layout/AbstractWidget",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/layout/component/DropdownToolbarButton",
		"web-viewer/layout/component/MiniButton",
		"web-viewer/layout/component/ModalPopup",
		"web-viewer/layout/component/UserUnitInputField",
		"web-viewer/compare/CompareDifferenceSubToolbarComponent",
		"web-viewer/compare/CompareSideBySideSubToolbarComponent",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/compare/CompareConstants",
		"jquery",
		"underscore"
	],
	function (
		AbstractWidget,
		ViewerToolbarButton,
		DropdownToolbarButton,
		MiniButton,
		ModalPopup,
		UserUnitInputField,
		CompareDifferenceSubToolbarComponent,
		CompareSideBySideSubToolbarComponent,
		LayoutConstants,
		CompareConstants,
		$,
		_) {
		"use strict";

		/**
		 * @exports CompareWidget
		 * @memberof Compare
		 * @augments Layout.AbstractWidget
		 * @class The CompareWidget displays the currently active page and the makes it possible to select the active and difference page from a dropdown menu.
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f
		 * @param {CompareModule} compareModule
		 * @param {CompareModel} compareModel
		 * @param {Layout.Component.DocumentSelection} documentSelection - contains the document selectors
		 */
		var CompareWidget = function CompareWidget(f, compareModule, compareModel, documentSelection) {
			AbstractWidget.call(this, f, LayoutConstants.Widget.COMPARE);

			/**
			 * @member {CompareModel} compareModel
			 */
			this._compareModel = compareModel;

			/**
			 * @member {CompareModule} compareModule
			 */
			this._compareModule = compareModule;

			/**
			 * @member {Core.ViewerFacade} the viewer facade
			 */
			this.f = f;

			/**
			 * @member {Layout.Component.DocumentSelection}
			 */
			this._documentSelection = documentSelection;

			this._compareOptionsInitialized = false;
			this._cadOverlayOptionsInitialized = false;

			/**
			 * Containers for all sections
			 **/
			this._extraControls = $("<div/>").addClass("extraControls displayNone");
			this._documentSelectionDiv = $("<div/>").addClass("documentSelection");
			this._compareOptions = $("<div/>").addClass("compareOptions displayNone");
			this._cadOverlay = $("<div>").addClass("cadOverlay displayNone");


			/**
			 * Reset compare pages
			 */
			this._resetLinkedComparePagesButton = new MiniButton(this.f, "resetLinkedComparePages", "icon-WCR_Revert", true, _.bind(function () {
				this.f.i18n.loadBundle("viewer", function () {
					var modalPopup = new ModalPopup(this.f, "resetLinkedComparePagesPopup", this.f.getContext());
					modalPopup.setRemoveFromDomAfterUsage(true);

					modalPopup.setOkAction(_.bind(function () {
						this.f.getActivePage().resetLinkedComparePage();
						this._updateResetLinkedComparePagesBtn();
						modalPopup.hide();
					}, this));

					modalPopup.addButton(new ViewerToolbarButton(this.f, "allPages", $("<span/>").append(f.i18n.t("AllPages")), true, _.bind(function () {
						_.each(this.f.getActivePage().getDocumentVersion().getAllPages(), function (page) {
							page.resetLinkedComparePage();
						});
						this._updateResetLinkedComparePagesBtn();
						modalPopup.hide();
					}, this)));

					// Header + Content
					modalPopup.setHeader($("<span/>").addClass("popupTitle").text(this.f.i18n.t("ResetLinkedComparePagesTT")));
					modalPopup.setContent($("<div/>").addClass("unsavedItemsContent").text(this.f.t("ResetLinkedComparePages")));
					modalPopup.setOkBtnText(this.f.i18n.t("PageOnly"));

					modalPopup.show();
				}, this);
			}, this), "icon-WCR_Revert fontIconDisabled");

			this._resetLinkedComparePagesButton.setTooltip("viewer", "ResetLinkedComparePagesTT");

			this._diffDocPageElement = $("<div/>");

			this.f.i18n.loadBundle("viewer", _.bind(function () {
				this._documentSelectionDiv.append($("<h4/>").append(this.f.t("DocumentListCurrentLabel") + " <br/>"));
				this._documentSelectionDiv.append(this._documentSelection.getActiveDocumentSelector().element);
				this._diffDocPageElement.append($("<h4/>").append(this.f.t("DocumentListDiffLabel") + " <br/>")).append(this._documentSelection.getDifferenceDocumentSelector().element);
				this._documentSelectionDiv.append(this._diffDocPageElement);

				// set the widget title
				this.setTitle(this.f.t("DocumentListTitle"));

			}, this));

			if (this.f.isMobile()) {
				// on mobile we want to update the title whenever the compare mode changes
				this.f.loadBundle("viewer", function () {
					this._compareModule.on("changed.mode", _.bind(this._compareModeChanged, this));
					this._compareModeChanged();
				}, this);
			}
			this.setIcon("icon icon-WCR_Document");
		};

		CompareWidget.prototype = Object.create(AbstractWidget.prototype);
		CompareWidget.prototype.constructor = CompareWidget;
		CompareWidget.widgetName = LayoutConstants.Widget.COMPARE;

		/**
		 * Update the title to the active compare mode
		 * And show the controls for difference or side by side compare
		 *
		 * NOTE: this method is only used on mobile
		 */
		CompareWidget.prototype._compareModeChanged = function () {
			var activeMode = this._compareModule.getActiveMode();
			var title;
			if (activeMode) {
				var activeModeName = activeMode.getName();
				title = this.f.t(CompareConstants.modeNameToLanguageKey[activeModeName]);

				if (this.f.isMobile()) {
					if (activeModeName === CompareConstants.mode.DIFFERENCE || activeModeName === CompareConstants.mode.SIDEBYSIDE) {
						var differenceCompareMode = this._compareModel.getModeByName(CompareConstants.mode.DIFFERENCE);
						if (activeModeName === CompareConstants.mode.DIFFERENCE) {
							if (!this._differenceCompareControls) {
								this._differenceCompareControls = new CompareDifferenceSubToolbarComponent(this.f, differenceCompareMode);
								this._extraControls.prepend(this._differenceCompareControls.element);
							}

							this._differenceCompareControls.element.removeClass("displayNone");
							if (this._sideBySideCompareControls) {
								this._sideBySideCompareControls.element.addClass("displayNone");
							}
						}

						if (activeModeName === CompareConstants.mode.SIDEBYSIDE) {
							if (!this._sideBySideCompareControls) {
								var sideBySideCompareMode = this._compareModel.getModeByName(CompareConstants.mode.SIDEBYSIDE);
								this._sideBySideCompareControls = new CompareSideBySideSubToolbarComponent(this.f, differenceCompareMode, sideBySideCompareMode);
								this._extraControls.prepend(this._sideBySideCompareControls.element);
							}

							this._sideBySideCompareControls.element.removeClass("displayNone");
							if (this._differenceCompareControls) {
								this._differenceCompareControls.element.addClass("displayNone");
							}
						}

						// show the compare mode extra controls
						this._extraControls.removeClass("displayNone");


					} else {
						// hide the extra controls
						this._extraControls.addClass("displayNone");

					}
					this._fireResized();
				}

			} else {
				// hide the extra controls
				this._extraControls.addClass("displayNone");
			}
			
			if (this.f.isMobile()){
				// update the title for mobile
				if (!title) {
					title = this.f.t("DocumentListTitle");
				}

				this.setTitle(title);
			}
		};

		/**
		 * initializes all the options for when both a current and difference page are selected, this are options for compare
		 */
		CompareWidget.prototype._initCompareOptions = function () {
			if (!this._compareOptionsInitialized) {
				this._compareOptionsInitialized = true;

				if (!this.f.isMobile()) {
					this._channelMatchingBtn = new ViewerToolbarButton(this.f, "channelMatching", "icon-WCR_Channel-Matching", false, _.bind(function () {
						var graphicsModule = this.f.getModule("graphics");
						if (graphicsModule !== undefined) {
							graphicsModule.showChannelMatchingPopup();
						}
					}, this));
				}

				// auto align button
				this._autoAlign = new ViewerToolbarButton(this.f, "autoAlign", "icon-WCR_Align-Auto", true, _.bind(function () {
					this._emit("doAutoAlign");
				}, this));
				this._autoAlign.element.addClass("autoAlign");

				//anchor align button
				this._anchorAlign = new ViewerToolbarButton(this.f, "anchorAlign", "icon-WCR_Align-Point", true, _.bind(function () {
					this._compareModule.startAlignmentWizard();
				}, this));
				this._anchorAlign.setTooltip("viewer", "AnchorAlignWizardTitle");

				// corner align button
				this._cornerAlign = new DropdownToolbarButton(this.f, "cornerAlign", "cornerAlign", true);

				// rotate reference button
				this._rotateReference = new ViewerToolbarButton(this.f, "rotateReference", "icon-WCR_Orientation-Rotate1", true, _.bind(function () {
					this._emit("rotateReference");
					this._updateRotateBtn();
				}, this));

				// flip reference button
				this._flipReference = new ViewerToolbarButton(this.f, "flipReference", "icon-WCR_Orientation-Mirror", true, _.bind(function () {
					this._emit("flipReference");
					this._updateFlipBtn();
				}, this));

				// negate reference button
				this._negateReference = new ViewerToolbarButton(this.f, "negateReference", "icon-WCR_Invert", true, _.bind(function () {
					this._emit("negateReference");
					this._updateNegateBtn();
				}, this));

				// trimbox align button
				this._trimBoxAlign = new ViewerToolbarButton(this.f, "trimBoxAlign", "icon-WCR_Align-Trimbox", true, _.bind(function () {
					this._emit("trimBoxAlign");
					this._trimBoxAlign.setActive(this._compareModel.isTrimBoxAlign());
				}, this));
				this._trimBoxAlign.element.addClass("toggleBtn");



				this._offsetX = new UserUnitInputField(this.f, 0, "x", _.bind(function (oldValue, newValue) {
					this._emit("offsetXChanged", newValue);
				}, this));

				this._offsetY = new UserUnitInputField(this.f, 0, "y", _.bind(function (oldValue, newValue) {
					this._emit("offsetYChanged", newValue);
				}, this));


				this.f.i18n.loadBundle("viewer", _.bind(function () {
					// initialize corner align actions
					this._cornerAlign.addAction({
						name: "topLeftAlign",
						content: this.f.t("AlignTopLeft"),
						icon: "icon-WCR_Align-TopLeft",
						action: _.bind(function () {
							this._emit("doCornerAlign", CompareConstants.cornerAutoAlign.TOPLEFT);
							this._cornerAlign.setIcon("icon-WCR_Align-TopLeft");
						}, this),
						enabled: true
					});

					this._cornerAlign.addAction({
						name: "topCenterAlign",
						content: this.f.t("AlignTopCenter"),
						icon: "icon-WCR_Align-TopCenter",
						action: _.bind(function () {
							this._emit("doCornerAlign", CompareConstants.cornerAutoAlign.TOPCENTER);
							this._cornerAlign.setIcon("icon-WCR_Align-TopCenter");
						}, this),
						enabled: true
					});

					this._cornerAlign.addAction({
						name: "topRightAlign",
						content: this.f.t("AlignTopRight"),
						icon: "icon-WCR_Align-TopRight",
						action: _.bind(function () {
							this._emit("doCornerAlign", CompareConstants.cornerAutoAlign.TOPRIGHT);
							this._cornerAlign.setIcon("icon-WCR_Align-TopRight");
						}, this),
						enabled: true
					});

					this._cornerAlign.addAction({
						name: "centerLeftAlign",
						content: this.f.t("AlignCenterLeft"),
						icon: "icon-WCR_Align-MiddleLeft",
						action: _.bind(function () {
							this._emit("doCornerAlign", CompareConstants.cornerAutoAlign.CENTERLEFT);
							this._cornerAlign.setIcon("icon-WCR_Align-MiddleLeft");
						}, this),
						enabled: true
					});

					this._cornerAlign.addAction({
						name: "centeredAlign",
						content: this.f.t("AlignCentered"),
						icon: "icon-WCR_Align-MiddleCenter",
						action: _.bind(function () {
							this._emit("doCornerAlign", CompareConstants.cornerAutoAlign.CENTERED);
							this._cornerAlign.setIcon("icon-WCR_Align-MiddleCenter");
						}, this),
						enabled: true
					});

					this._cornerAlign.addAction({
						name: "centerRightAlign",
						content: this.f.t("AlignCenterRight"),
						icon: "icon-WCR_Align-MiddleRight",
						action: _.bind(function () {
							this._emit("doCornerAlign", CompareConstants.cornerAutoAlign.CENTERRIGHT);
							this._cornerAlign.setIcon("icon-WCR_Align-MiddleRight");
						}, this),
						enabled: true
					});

					this._cornerAlign.addAction({
						name: "bottomLeftAlign",
						content: this.f.t("AlignBottomLeft"),
						icon: "icon-WCR_Align-BottomLeft",
						action: _.bind(function () {
							this._emit("doCornerAlign", CompareConstants.cornerAutoAlign.BOTTOMLEFT);
							this._cornerAlign.setIcon("icon-WCR_Align-BottomLeft");
						}, this),
						enabled: true
					});

					this._cornerAlign.addAction({
						name: "bottomCenterAlign",
						content: this.f.t("AlignBottomCenter"),
						icon: "icon-WCR_Align-BottomCenter",
						action: _.bind(function () {
							this._emit("doCornerAlign", CompareConstants.cornerAutoAlign.BOTTOMCENTER);
							this._cornerAlign.setIcon("icon-WCR_Align-BottomCenter");
						}, this),
						enabled: true
					});

					this._cornerAlign.addAction({
						name: "bottomRightAlign",
						content: this.f.t("AlignBottomRight"),
						icon: "icon-WCR_Align-BottomRight",
						action: _.bind(function () {
							this._emit("doCornerAlign", CompareConstants.cornerAutoAlign.BOTTOMRIGHT);
							this._cornerAlign.setIcon("icon-WCR_Align-BottomRight");
						}, this),
						enabled: true
					});

					// set tooltips
					if (this._channelMatchingBtn) {
						this._channelMatchingBtn.setTooltip(this.f.i18n.t("MapChanTitle"));
					}
					this._autoAlign.setTooltip(this.f.t("AlignAuto"));
					this._rotateReference.setTooltip(this.f.t("RotateReferenceTT"));
					this._flipReference.setTooltip(this.f.t("MirrorReferenceTT"));
					this._negateReference.setTooltip(this.f.t("ToolInvertReferenceTT"));
					this._cornerAlign.setTooltip(this.f.t("AlignCornerReferenceTT"));
					this._trimBoxAlign.setTooltip(this.f.t("AlignFromTrimBoxTT"));
					this._addCompareOptionsToDOM();
				}, this));
			}
		};

		/**
		 * Initialize the controls for CAD overlay
		 */
		CompareWidget.prototype._initCadOverlayOptions = function () {
			if (!this._cadOverlayOptionsInitialized) {
				this._cadOverlayOptionsInitialized = true;
				// initialize elements 
				this._cadOverlaySaveToEmpty = new MiniButton(this.f, "saveCadOverlayToEmpty", "icon-WCR_Save", true, _.bind(function () {
					this._emit("saveCadOverlay");
				}, this), "icon-WCR_Save fontIconDisabled");
				this._cadOverlaySaveToEmpty.element.addClass("pull-right");
				this._cadOverlaySaveToEmpty.setTooltip("viewer", "saveCadRefTT");
				this._cadOverlaySaveToEmpty.hide();

				this._cadOverlaySave = new ViewerToolbarButton(this.f, "saveCadOVerlay", "icon-WCR_Save", true, _.bind(function () {
					this._emit("saveCadOverlay");
				}, this));

				this._cadOverlaySave.setTooltip("viewer", "SaveCadOverlayTT");

				this._open3DView = new ViewerToolbarButton(this.f, "open3DView", "icon-WCR_3D-Create", false, _.bind(function () {
					this._compareModule.checkAligmentAndShow3D();
				}, this));
				this._open3DView.setTooltip("viewer", "Open3DViewTT");


				// orientation and alignment
				this._cadOverlayOrientation = $("<div>").addClass("alignment");

				// rotate reference button
				this._cadOverlayRotate = new ViewerToolbarButton(this.f, "rotateOverlay", "icon-WCR_Orientation-Rotate1", true, _.bind(function () {
					this._emit("rotateCadOverlay");
				}, this));
				this._cadOverlayRotate.setTooltip("viewer", "RotateOverlayTT");

				// flip reference button
				this._cadOverlayFlip = new ViewerToolbarButton(this.f, "flipOverlay", "icon-WCR_Orientation-Mirror", true, _.bind(function () {
					this._emit("flipCadOverlay");
				}, this));
				this._cadOverlayFlip.setTooltip("viewer", "FlipOverlayTT");

				// manual offset             
				this._cadOverlayOffsetX = new UserUnitInputField(this.f, 0, "x", _.bind(function (oldValue, newValue) {
					this._emit("cadOverlayOffsetXChanged", newValue);
				}, this));

				this._cadOverlayOffsetY = new UserUnitInputField(this.f, 0, "y", _.bind(function (oldValue, newValue) {
					this._emit("cadOverlayOffsetYChanged", newValue);
				}, this));


				this.f.loadBundle("viewer", function () {
					// insert into dom and do i18n


					// header
					if (this.f.isMobile()) {
						this._cadOverlay.append($("<h3>").text(this.f.t("DocumentListCadOverlayTitle")).append(this._cadOverlaySaveToEmpty.element));
					} else {
						this._cadOverlay.append(this._cadOverlaySaveToEmpty.element);
						this._cadOverlay.append($("<h4>").text(this.f.t("DocumentListCadLabel")));
					}

					this._cadOverlay.append($("<div>").addClass("documentSelection").append(this._documentSelection.getCadOverlayDocumentSelector().element));

					this._cadOverlay.append(this._cadOverlayOrientation);

					if (!this.f.isMobile()) {
						this._cadOverlayOrientation.append($("<span>").addClass("alignLabel").text(this.f.t("DiffOrientation")));
					}
					this._cadOverlayOrientation.append(this._cadOverlayRotate.element).append(this._cadOverlayFlip.element);
					this._cadOverlayOrientation.append("<br />");

					if (!this.f.isMobile()) {
						this._cadOverlayOrientation.append($("<span>").addClass("alignLabel").text(this.f.t("AlignLabel")));

						//anchor align button         
						this._cadOverlayAnchorAlign = new ViewerToolbarButton(this.f, "anchorAlign", "icon-WCR_Align-Point", true, _.bind(function () {
							this._compareModule.startAlignmentWizard(true);
						}, this));
						this._cadOverlayAnchorAlign.setTooltip("viewer", "AnchorAlignWizardTitle");
						this._cadOverlayOrientation.append(this._cadOverlayAnchorAlign.element);
					}

					var offsetGroup;
					if (this.f.isMobile()) {
						// mobile optimized html for offset fields
						offsetGroup = $("<div>").addClass("offsetFields");
						offsetGroup.append($("<p>").addClass("alignLabel").text(this.f.t("AlignOffset")));
						offsetGroup.append($("<p>").text("X: ").append(this._cadOverlayOffsetX.element));
						offsetGroup.append($("<p>").text("Y: ").append(this._cadOverlayOffsetY.element));

					} else {
						this._cadOverlayOffsetX.toggleLabel(false);
						offsetGroup = $("<div>").append($("<span>").addClass("alignLabel").text(this.f.t("AlignOffset")));
						offsetGroup.append($("<div>").addClass("offsetFields").append(this._cadOverlayOffsetX.element).append(this._cadOverlayOffsetY.element));
					}

					this._cadOverlayOrientation.append(this._cadOverlaySave.element);
					this._cadOverlayOrientation.append(this._open3DView.element);

					this._cadOverlayOrientation.append(offsetGroup);
				}, this);
			}
		};

		/**
		 * Add the compare buttons and input fields to the DOM
		 */
		CompareWidget.prototype._addCompareOptionsToDOM = function () {
			if (this._compareOptionsInitialized && this.context && !this._compareOptionsAddedToDOM) {
				this._compareOptionsAddedToDOM = true;
				this.f.loadBundle("viewer", function () {

					if (this.f.isMobile()) {
						// MOBILE LAYOUT
						this._documentSelectionTitle = $("<h3/>").text(this.f.t("DocumentListTitle"));
						this._extraControls.append(this._documentSelectionTitle);
					} else {
						// channel matchin is only available on desktop
						this._channelMatching = $("<div/>").addClass("channelMatching");
						this._channelMatching.append($("<h4/>").append(this.f.t("DocumentListChannelMatch") + ": "));
						this._channelMatching.append(this._channelMatchingBtn.element);
						this._compareOptions.append(this._channelMatching);
					}


					// ALIGNMENT AND ORIENTATION
					this._alignBtns = $("<div/>").addClass("alignment");

					var heading = $(this.f.isMobile() ? "<h3/>" : "<h4/>");

					this._alignBtns.append(heading.text(this.f.t("DocumentListAlignment")));

					if (!this.f.isMobile()) {
						this._alignBtns.append($("<span>").addClass("alignLabel").text(this.f.t("DiffOrientation")));
					}
					this._alignBtns.append(this._rotateReference.element);
					this._alignBtns.append(this._flipReference.element);
					this._alignBtns.append(this._negateReference.element);
					this._alignBtns.append("<br>");

					if (!this.f.isMobile()) {
						this._alignBtns.append($("<span>").addClass("alignLabel").text(this.f.t("AlignLabel")));
					}
					this._alignBtns.append(this._autoAlign.element);
					if (!this.f.isMobile()) {
						this._alignBtns.append(this._anchorAlign.element);
					}
					this._alignBtns.append(this._cornerAlign.element);
					this._alignBtns.append(this._trimBoxAlign.element);
					this._alignBtns.append("<br>");

					if (this.f.isMobile()) {
						// mobile optimized html for offset fields
						this._offsetFields = $("<div>").addClass("offsetFields");
						this._offsetFields.append($("<p>").addClass("alignLabel").text(this.f.t("AlignOffset")));
						this._offsetFields.append($("<p>").text("X: ").append(this._offsetX.element));
						this._offsetFields.append($("<p>").text("Y: ").append(this._offsetY.element));
						this._alignBtns.append(this._offsetFields);
					} else {

						this._alignBtns.append($("<span>").addClass("alignLabel").text(this.f.t("AlignOffset")));
						this._offsetFields = $("<div>").addClass("offsetFields");
						this._offsetX.toggleLabel(false);
						this._offsetFields.append(this._offsetX.element);
						this._offsetFields.append(this._offsetY.element);
						this._alignBtns.append(this._offsetFields).append(this._unitLabel);
					}
					this._compareOptions.append(this._alignBtns);

					this.headerExtra.append(this._resetLinkedComparePagesButton.element);

					this._compareModeChanged();
				}, this);
			}
		};

		/**
		 * Set the context of the widget
		 * @override
		 * @param context
		 * @param header
		 */
		CompareWidget.prototype.setContext = function (context, header) {
			AbstractWidget.prototype.setContext.call(this, context, header);
			this.context.addClass("compareWidget");
			this.context
				.append(this._extraControls)
				.append(this._documentSelectionDiv)
				.append(this._compareOptions)
				.append(this._cadOverlay);

			this._addCompareOptionsToDOM();
			this.headerExtra.addClass("flexEnd");
			this._updateResetLinkedComparePagesBtn();
		};

		/**
		 * Update the compare widget components
		 */
		CompareWidget.prototype.updateCompareWidget = function () {

			// show/hide align buttons        
			var refPage = this.f.getDocumentController().getDifferencePage();
			if (refPage === undefined || this._compareModel.isReaderSpreadActive()) {
				if (this._compareOptions !== null) {
					this._compareOptions.addClass("displayNone");
				}
			} else {
				this._initCompareOptions();
				this._compareOptions.removeClass("displayNone");


				// update state
				var offset = this._compareModel.getOffset();

				this._offsetX.setValue(offset.x);
				this._offsetY.setValue(offset.y);


				this._trimBoxAlign.setActive(this._compareModel.isTrimBoxAlign());

				var icon = CompareConstants.cornerAlignIcons[this._compareModel.getCornerAlign()];
				if (icon) {
					this._cornerAlign.setIcon(icon);
				} else {
					this._cornerAlign.setIcon("icon-WCR_Align-TopLeft");
				}

				this._updateRotateBtn();
				this._updateFlipBtn();
				this._updateNegateBtn();

				if (refPage.hasCad()) {
					this._negateReference.hide();
					this._trimBoxAlign.hide();
					this._cornerAlign.hide();
					this._autoAlign.hide();
					if (this._channelMatching) {
						this._channelMatching.hide();
					}
				} else {
					this._negateReference.show();
					this._trimBoxAlign.show();
					this._cornerAlign.show();
					this._autoAlign.show();
					if (this._channelMatching) {
						this._channelMatching.show();
					}
				}
			}

			// show/hide cad overlay
			if (this._cadOverlay) {
				this._cadOverlay.toggleClass("displayNone", !this._compareModule.isCadOverlayAllowed());
			}
			var cadOverlay = this.f.getDocumentController().getCadOverlay();
			if (cadOverlay) {
				this._initCadOverlayOptions();
				this._cadOverlayOrientation.removeClass("displayNone");

				// update state
				var cadOverlayOffset = this._compareModel.getCadOverlayOffset();

				this._cadOverlayOffsetX.setValue(cadOverlayOffset.x);
				this._cadOverlayOffsetY.setValue(cadOverlayOffset.y);

				if (this._compareModule.canSaveCadOverlay()) {
					this._cadOverlaySave.show();
					if (!this.f.getActivePage().getDocumentVersion().isMultiPage() &&
						this.f.getActivePage().get("canGenerate3D") && this.f.getActivePage().canEditAttributes()) {
						// only show "Open 3D button" when user has EditProperties permission and a 3D view can be generated
						this._open3DView.show();
					} else {
						this._open3DView.hide();
					}

					if (!this._compareModule.isCadOverlayModified()) {
						this._cadOverlaySave.setEnabled(false);
					} else {
						this._cadOverlaySave.setEnabled(true);
					}
				} else {
					this._cadOverlaySave.hide();
					this._open3DView.hide();
				}

				if (this.f.getActivePage().hasLinkedPage() &&
					this.f.getActivePage().get("canGenerate3D") && this.f.getActivePage().canEditAttributes()) {
					// only show "Open 3D button" when user has EditProperties permission and a 3D view can be generated
					this._open3DView.show();
				}

				this._cadOverlaySaveToEmpty.hide();
				this._updateCadOverlayButtons();

			} else {
				if (this._cadOverlayOrientation) {
					this._cadOverlayOrientation.addClass("displayNone");
				}
				if (this._compareModule.isCadOverlayAllowed()) {
					this._initCadOverlayOptions();
				}
				if (this._compareModule.canSaveCadOverlay()) {
					this._initCadOverlayOptions();
					if (this._cadOverlaySaveToEmpty) {
						this._cadOverlaySaveToEmpty.show();
						if (!this._compareModule.isCadOverlayModified()) {
							this._cadOverlaySaveToEmpty.setEnabled(false);
						} else {
							this._cadOverlaySaveToEmpty.setEnabled(true);
						}
					}
				} else {
					if (this._cadOverlaySaveToEmpty) {
						this._cadOverlaySaveToEmpty.hide();
					}
				}
			}

			this._updateResetLinkedComparePagesBtn();

			this._fireResized();
		};

		/**
		 * @private
		 * Update the rotate btn
		 */
		CompareWidget.prototype._updateRotateBtn = function () {
			var rotation = this._compareModel.getReferenceRotation();
			if (rotation === CompareConstants.referenceRotation.NOTROTATED) {
				this._rotateReference.setIcon("icon-WCR_Orientation-Rotate1");
			} else if (rotation === CompareConstants.referenceRotation.ROTATED90) {
				this._rotateReference.setIcon("icon-WCR_Orientation-Rotate2");
			} else if (rotation === CompareConstants.referenceRotation.ROTATED180) {
				this._rotateReference.setIcon("icon-WCR_Orientation-Rotate3");
			} else if (rotation === CompareConstants.referenceRotation.ROTATED270) {
				this._rotateReference.setIcon("icon-WCR_Orientation-Rotate4");
			}
		};

		/**
		 * @private
		 * update the flip btn
		 */
		CompareWidget.prototype._updateFlipBtn = function () {
			if (this._compareModel.isReferenceFlipped()) {
				this._flipReference.setIcon("icon-WCR_Orientation-MirrorReverse");
				this._flipReference.setTooltip(this.f.t("MirrorReferenceTT2"));
			} else {
				this._flipReference.setIcon("icon-WCR_Orientation-Mirror");
				this._flipReference.setTooltip(this.f.t("MirrorReferenceTT"));
			}
		};

		/**
		 * @private
		 * update the negate btn
		 */
		CompareWidget.prototype._updateNegateBtn = function () {
			if (this._compareModel.isReferenceNegated()) {
				this._negateReference.setIcon("icon-WCR_InvertReverse");
				this._negateReference.setTooltip(this.f.t("ToolInvertReferenceTT2"));
			} else {
				this._negateReference.setIcon("icon-WCR_Invert");
				this._negateReference.setTooltip(this.f.t("ToolInvertReferenceTT"));
			}
		};

		/**
		 * @private
		 * Update the reset linked compare pages button
		 */
		CompareWidget.prototype._updateResetLinkedComparePagesBtn = function () {
			var isMultiPage = this.f.getActivePage().getDocumentVersion().isMultiPage();
			this._resetLinkedComparePagesButton.element.toggle(isMultiPage);
			if (isMultiPage) {
				this._resetLinkedComparePagesButton.setEnabled(
					_.some(this.f.getActivePage().getDocumentVersion().getAllPages(), function (page) {
						return page.hasLinkedComparePage();
					}));
			}
		};

		/**
		 * Toggle the visibility of the difference document selector
		 * @param {Boolean} hide - show or hide the document selector
		 */
		CompareWidget.prototype.toggleDifferenceDocPageSelector = function (hide) {
			if (hide) {
				this._diffDocPageElement.hide();
			} else {
				this._diffDocPageElement.show();
			}
		};

		/**
		 * Update the orientation buttons for the cad overlay
		 */
		CompareWidget.prototype._updateCadOverlayButtons = function () {
			var rotation = this._compareModel.getCadOverlayRotation();
			if (rotation === CompareConstants.referenceRotation.NOTROTATED) {
				this._cadOverlayRotate.setIcon("icon-WCR_Orientation-Rotate1");
			} else if (rotation === CompareConstants.referenceRotation.ROTATED90) {
				this._cadOverlayRotate.setIcon("icon-WCR_Orientation-Rotate2");
			} else if (rotation === CompareConstants.referenceRotation.ROTATED180) {
				this._cadOverlayRotate.setIcon("icon-WCR_Orientation-Rotate3");
			} else if (rotation === CompareConstants.referenceRotation.ROTATED270) {
				this._cadOverlayRotate.setIcon("icon-WCR_Orientation-Rotate4");
			}

			if (this._compareModel.isCadOverlayFlipped()) {
				this._cadOverlayFlip.setIcon("icon-WCR_Orientation-MirrorReverse");
				this._cadOverlayFlip.setTooltip(this.f.t("MirrorCadOverlayTT2"));
			} else {
				this._cadOverlayFlip.setIcon("icon-WCR_Orientation-Mirror");
				this._cadOverlayFlip.setTooltip(this.f.t("MirrorCadOverlayTT"));
			}
		};


		return CompareWidget;
	});

define('web-viewer/compare/CompareShowReaderSpreadMode',["underscore",
		"web-viewer/compare/CompareMode",
		"web-viewer/compare/CompareConstants",
		"web-viewer/viewport/ViewportConstants",
		"web-viewer/pagelist/PagelistConstants",
		"web-viewer/utils/ListenerUtil",
		"utils/ShapeUtils"
	],
	function (_,
		CompareMode,
		CompareConstants,
		ViewportConstants,
		PagelistConstants,
		ListenerUtil,
		ShapeUtils) {
		"use strict";

		/**
		 * @memberof CompareModule
		 * @exports CompareShowReaderSpreadMode
		 * @class This mode is only available for multipage or pagelist files and shows two pages next to each other.
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {CompareModule.CompareModel} compareModel - model for compare
		 */
		var CompareShowReaderSpreadMode = function (f, compareModel) {
			CompareMode.call(this, f, CompareConstants.mode.READERSPREAD, compareModel, false, "", "current");
			ListenerUtil.mixin(this, f);

			/**
			 * @member {AbstractDocumentPage} - current diffpage
			 */
			this.diffPage = null;

			/**
			 * @member {Number} - parameter for url, 1 when the active page is on the left of reader spread, 2 if the active page is on the right of reader spread
			 */
			this.rs = CompareConstants.pagelistSide.RIGHTPAGE;
		};

		CompareShowReaderSpreadMode.prototype = Object.create(CompareMode.prototype);
		CompareShowReaderSpreadMode.prototype.constructor = CompareShowReaderSpreadMode;

		/**
		 * Start/stop show compare mode
		 *
		 * @param {Boolean} toggle - should the reader spread mode be toggled on or off
		 */
		CompareShowReaderSpreadMode.prototype.toggle = function (toggle) {
			CompareMode.prototype.toggle.call(this, toggle);
			this.f.getDocumentController().willNeedParameterUpdate("READER_SPREAD");
			if (toggle) {
				var isFetchingParameters = this.f.getDocumentController().getDifferencePage() !== undefined;
				this.f.getDocumentController().resetDifferencePage();
				this._compareModel.setDiffMode(CompareConstants.diffMode.READERSPREAD);
				var activePage = this.f.getDocumentController().getActivePage();
				activePage.setReaderSpread(true);
				var diffPageno = this._calculateReferencePageNumber(activePage);

				this.diffPage = activePage.getDocumentVersion().getPage(diffPageno);
				if (this.diffPage !== undefined) { //is undefined when the active page is the first page
					this.diffPage.resetParameters();
					activePage.resetParameters();
					this.f.getDocumentController().changeDifferencePage(this.diffPage);
				}
				if (this.rs === CompareConstants.pagelistSide.RIGHTPAGE) { // active page is shown on the right
					activePage.set("isRightPage", true); //set a property of the page to indicate if the active page is right or left in reader spread
					if (this.diffPage === undefined) { //if there is no reference page, calculate the width of the active page and set as offset
						if (isFetchingParameters) {
							activePage.resetParameters();
						}
					}
				} else { //for left sided pages, no offset is needed
					activePage.set("isRightPage", false); //set is right page to false, page is left side of reader spread
				}
				//register listener to when the active page is being changed, a difference pages needs to be set
				this._registerListener("intercept.activePageChange", this._whenActivePageChanges, this, this.f);
			} else { //turn the compare mode off
				var activePage = this.f.getDocumentController().getActivePage();
				activePage.reset("isRightPage");
				activePage.setReaderSpread(false); //make sure the readerSpread attribute of the page is set to false
				//reset the difference page, this page is not the default reference page wanted for other compare modes as its the previous or next page of the same version
				this.diffPage = null;
				this.f.getDocumentController().resetDifferencePage();
				this._compareModel.setDiffMode(CompareConstants.diffMode.UNDEFINED);
				this._unregisterAllListeners(); //unregister the listeners
				// see compare module for resetting diff page
			}
			this.f.getDocumentController().readyForParameterUpdate("READER_SPREAD");
		};

		/**
		 * Intercept the fragment URL for construction of the URL
		 * @param {Object} c - object with all the parameters for the URL
		 */
		CompareShowReaderSpreadMode.prototype.interceptFragmentURL = function (c) {
			c.params.rs = this.rs; //only set the rs parameters to make sure the active page is shown on the correct side of the reader spread

			c.params.daligntrim = 1;
			c.params.dmode = CompareConstants.diffMode.SHOWBOTHONTOP; // reader spread uses show both on top compare mode on the view server
		};

		/**
		 * When the active page changes, calculate the difference page and set the correct offset on the active page
		 *
		 * @param {AbstractDocumentPage} oldActivePage
		 * @param {AbstractDocumentPage} activePage
		 */
		CompareShowReaderSpreadMode.prototype._whenActivePageChanges = function (pages) {
			if (this.isEnabled()) { //first make sure the compare mode is activated
				if (pages.oldActivePage !== undefined) { //set the reader spread of the old active document page to false
					pages.oldActivePage.setReaderSpread(false);
					pages.oldActivePage.reset("isRightPage");
				}

				pages.newActivePage.setReaderSpread(true); //set the reader spread for the new active page
				var diffPageno = this._calculateReferencePageNumber(pages.newActivePage);
				this.diffPage = pages.newActivePage.getDocumentVersion().getPage(diffPageno);

				if (this.diffPage === undefined) {

					pages.diffPage = undefined;
				} else {
					this.diffPage.resetParameters();
					pages.diffPage = this.diffPage;
				}

				//set the offset of the active document page
				if (this.rs === CompareConstants.pagelistSide.RIGHTPAGE) {
					if (this.diffPage !== undefined) {
						pages.newActivePage.resetParameters();
					}
					pages.newActivePage.set("isRightPage", true);
				} else {
					pages.newActivePage.set("isRightPage", false);
				}

			}
		};

		/**
		 * Calculates the the number of the reference page for reader spread mode
		 * @param {AbstractDocumentPage} activePage - The current active document page
		 * @returns {Number}
		 */
		CompareShowReaderSpreadMode.prototype._calculateReferencePageNumber = function (activePage) {
			var activePagenum = activePage.getPagenum();
			var activePageBinding = activePage.getDocument().get("binding");
			var diffPagenum;
			if (activePagenum % 2 === 0) {
				// even pages
				diffPagenum = activePagenum + 1;
				if (activePageBinding === undefined || activePageBinding === PagelistConstants.binding.LEFT) {
					// even pages with left (or undefined) binding come left
					this.rs = CompareConstants.pagelistSide.LEFTPAGE;
				} else if (activePageBinding === PagelistConstants.binding.RIGHT) {
					// even pages with right binding come right
					this.rs = CompareConstants.pagelistSide.RIGHTPAGE;
				}
			} else {
				// odd pages
				diffPagenum = activePagenum - 1;
				if (activePageBinding === undefined || activePageBinding === PagelistConstants.binding.LEFT) {
					// odd pages with left (or undefined) binding come right
					this.rs = CompareConstants.pagelistSide.RIGHTPAGE;
				} else if (activePageBinding === PagelistConstants.binding.RIGHT) {
					// odd pages with right binding come left
					this.rs = CompareConstants.pagelistSide.LEFTPAGE;
				}
			}

			return diffPagenum;
		};

		return CompareShowReaderSpreadMode;
	});

define('tpl!web-viewer/compare/ComparePagelistPopup', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='';
 if(d.showPage){ 
__p+='\r\n	<div class="page">\r\n		'+
((__t=(d.i18n("PagelistComparePage")))==null?'':_.escape(__t))+
' '+
((__t=( d.page ))==null?'':_.escape(__t))+
'\r\n	</div>\r\n';
 } 
__p+='\r\n<div class="range">\r\n	<div>'+
((__t=(d.i18n("PagelistCompareRange")))==null?'':_.escape(__t))+
'</div>\r\n	<ul>\r\n		<li><input type="radio" name="range" value="0" ';
 if(d.range===0){ 
__p+=' checked ';
}
__p+=' /> '+
((__t=(d.i18n("PagelistCompareCurrent")))==null?'':_.escape(__t))+
'</li>\r\n		<li><input type="radio" name="range" value="1" ';
 if(d.range===1){ 
__p+=' checked ';
}
__p+=' /> '+
((__t=(d.i18n("PagelistCompareFollowing")))==null?'':_.escape(__t))+
'</li>\r\n		<li><input type="radio" name="range" value="2" ';
 if(d.range===2){ 
__p+=' checked ';
}
__p+=' /> '+
((__t=(d.i18n("PagelistCompareAll")))==null?'':_.escape(__t))+
'</li>\r\n	</ul>\r\n</div>';
return __p;
}});
define('web-viewer/compare/ComparePagelistPopup',["web-viewer/core/PrivateChannel",
		"web-viewer/layout/component/ModalPopup",
		"Template",
		"tpl!web-viewer/compare/ComparePagelistPopup",
		"web-viewer/compare/CompareConstants",
		"jquery",
		"underscore"
	],
	function (
		PrivateChannel,
		ModalPopup,
		Template,
		tpl,
		CompareConstants,
		$,
		_
	) {
		"use strict";
		/**
		 * Create a popup for selecting comparison setup for pagelists. 
		 * @exports ComparePagelistPopup
		 * @memberof CompareModule
		 * @constructor
		 * @augments Layout.ModalPopup
		 * 
		 * @param {Core.ViewerFacade} f - Viewerfacade
		 * @param {String} popupId - Id of the popup used for its private channel
		 */
		var ComparePagelistPopup = function (f, popupId) {
			ModalPopup.call(this, f, popupId, f.getContext());
			PrivateChannel.installTo(this, f, ['comparePagelist']);

			this._defaultPagelistCompareRange = CompareConstants.pagelistCompareRange.CURRENT;
			this._currentPagelistCompareRange = null;
			this._pagelistComparePage = null;

			this.t = new Template(this.f.i18n, tpl);
			this.t.setContext(this._popupContent);
			this.t.bind("change", "[name=range]", function (source, event) {
				this._currentPagelistCompareRange = parseInt(source.value, 10);
			}, this);

			this.f.i18n.loadBundle("viewer", function () {
				this.setHeader($("<span/>").addClass("popupTitle").html(f.i18n.t("MultipageCompare")));
				this.setOkBtnText(f.i18n.t("OkButton"));
			}, this);

			this.element.addClass("comparePagelistPopup");
		};

		ComparePagelistPopup.prototype = Object.create(ModalPopup.prototype);
		ComparePagelistPopup.prototype.constructor = ComparePagelistPopup;

		/**
		 * Renders the compare pagelist popup
		 */
		ComparePagelistPopup.prototype.render = function () {
			this.t.d.showPage = this._pagelistComparePage && this._pagelistComparePage.getDocumentVersion().isMultiPage();
			this.t.d.page = this._pagelistComparePage.getPagenum() || "";
			this.t.d.range = this._currentPagelistCompareRange;

			this.t.render();
		};

		/**
		 * Reset the popup to default settings
		 */
		ComparePagelistPopup.prototype._reset = function () {
			// reset to default
			this._currentPagelistCompareRange = CompareConstants.pagelistCompareRange.CURRENT;
		};

		/**
		 * Show the pagelist popup
		 */
		ComparePagelistPopup.prototype.show = function () {
			this._reset();
			this.render();
			ModalPopup.prototype.show.call(this);
			this.recalculateSize(300);
		};

		/**
		 * Set the 'compare to' page
		 * @param {AbstractDocumentPage} page
		 */
		ComparePagelistPopup.prototype.setComparePage = function (page) {
			this._pagelistComparePage = page;
		};

		return ComparePagelistPopup;
	});

define('tpl!web-viewer/layout/component/DocumentSelector', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<div class="documentItem"> \r\n	<img class="thumbnail pull-left" src="'+
((__t=(d.thumbnailUrl))==null?'':__t)+
'" />\r\n	<ul class="info">		\r\n		<li class="documentName">\r\n			'+
((__t=(d.docInfo.documentName))==null?'':_.escape(__t))+
'\r\n		</li>\r\n		\r\n		';
 if(d.docInfo.project!==undefined){ 
__p+='\r\n			<li class="projectName">\r\n				<span class="docSelectorKey">'+
((__t=(d.i18n("WCProject")))==null?'':_.escape(__t))+
'</span> '+
((__t=(d.docInfo.project))==null?'':_.escape(__t))+
'\r\n			</li>\r\n		';
 } 
__p+='	\r\n		\r\n		';
 if(!d.docInfo.isGroup){ 
__p+='				\r\n			';
 if(d.docInfo.pageNumber!== undefined && !d.docInfo.referencedFrom){ 
__p+='\r\n				<li class="docSelectorPageInfo">	\r\n					<span class="pageNumber">'+
((__t=(d.i18n("PageUC")))==null?'':_.escape(__t))+
'&nbsp;'+
((__t=(d.docInfo.pageNumber))==null?'':_.escape(__t))+
'</span>\r\n						';
 if(d.docInfo.pageCount!==null){ 
__p+='\r\n							'+
((__t=(d.i18n("OfKeyWord")))==null?'':_.escape(__t))+
' '+
((__t=(d.docInfo.pageCount))==null?'':_.escape(__t))+
'\r\n						';
 } 
__p+='\r\n				</li>\r\n			';
 } 
__p+='\r\n		\r\n			';
 if(d.docInfo.versionNumber!==undefined){
__p+='\r\n				<li class="versionNumber">\r\n					<span class="docSelectorKey">'+
((__t=(d.i18n("WCVersion")))==null?'':_.escape(__t))+
'</span> '+
((__t=(d.docInfo.versionNumber))==null?'':_.escape(__t))+
'\r\n				</li>\r\n			';
 } 
__p+='\r\n		';
 } else { 
__p+='\r\n			';
 if(d.docInfo.versionNumber!==undefined){
__p+='\r\n				<li class="versionNumber">\r\n					<span class="docSelectorKey">'+
((__t=(d.i18n("WCVersion")))==null?'':_.escape(__t))+
' </span>'+
((__t=(d.docInfo.versionId))==null?'':_.escape(__t))+
' \r\n				</li>\r\n			';
 } 
__p+='\r\n			<li class="showAllPages"  ';
if(d.docInfo.isOpen){
__p+='style="display: none;"';
}
__p+='>'+
((__t=(d.i18n("ShowAllPages")))==null?'':_.escape(__t))+
' </li>\r\n			<li class="hideAllPages"  ';
if(!d.docInfo.isOpen){
__p+='style="display: none;"';
}
__p+='>'+
((__t=(d.i18n("HideAllPages")))==null?'':_.escape(__t))+
' </li>\r\n		';
 } 
__p+='\r\n		\r\n			 ';
 if (d.docInfo.referencedFrom) { 
__p+=' \r\n				<li class="cadReferencedFrom">\r\n                	<i class="icon icon-WCR_Link"></i> '+
((__t=(d.docInfo.referencedFrom))==null?'':_.escape(__t))+
'\r\n				</li>\r\n            ';
 } 
__p+='\r\n	</ul>\r\n</div>';
return __p;
}});
define('web-viewer/layout/component/DocumentDropdownMenu',["component/dropdownmenu/DropdownMenu",
		"Template",
		"tpl!web-viewer/layout/component/DocumentSelector",
		"jquery",
		"underscore"
	],
	function (
		DropdownMenu,
		Template,
		tpl,
		$,
		_) {
		"use strict";

		/** 
		 *
		 * @class A DocumentDropdownMenu is a dropdownmenu that when clicked shows a dropdown with a list of document pages. New
		 * documents can be added to the list and the documents can be enabled/disabled. Changing the selected pages makes the previous
		 * selected page back available in the the dropdown. It is also possible to group document pages. Group elements will hide all its elements,
		 * when the group element is clicked all the pages of this group will become visible. Only document pages of the same type should be added
		 * the same group.
		 *
		 * @exports Layout.Component.DocumentDropdownMenu
		 * @constructor
		 * @augments Layout.Component.DropdownMenu
		 * @param {Core.ViewerFacade} f
		 * @param {Function}          onDocumentSelect      - function to execute when a document is selected
		 * @param {String}            [emptyListText]       - text shown in the list when no documents can be selected
		 * @param {String}            [nothingSelectedText] - Text shown in the dropdown when no document is selected
		 * @param {String}            [deselectText]        - Text used for the action to deselect the currently selected document, when not passed it is not possible to deselect the currently selected document
		 * @param {Function}          [deselectCallback]    - callback to execute when the document gets deselected, first parameter is the deselected pageId
		 * @param {Boolean}           [isMobile]            - adapt layout for mobile
		 * @param {Boolean}           [isCompact]           - if true version number is put behind the document (on the same line)
		 */
		var DocumentDropdownMenu = function (f, onDocumentSelect, emptyListText, nothingSelectedText, deselectText, deselectCallback, isMobile, isCompact) {
			this.f = f;
			DropdownMenu.call(this, this.f.i18n, this.f.getContext(), this.f.isMobile(), "bottom", "documentSelector", 300);

			/**
			 * @member {Function} function that's executed when a new document gets selected
			 * @private
			 */
			this._onDocChange = onDocumentSelect;

			/**
			 * @member {JQueryObj} visible part of the dropdownmenu with the currently selected document page
			 */
			this.element = $("<div/>").addClass("selectedDocument");
			this.attachTo(this.element);

			this._isMobile = isMobile;

			/**
			 * @member {JQueryObj} element for the currently selected element
			 */
			this._selectedDocumentElement = $("<div>");
			if (this._isMobile) {
				this.element.append('<i class="icon-WCR_Down"></i>');
			}
			this.element.append(this._selectedDocumentElement);
			if (!this._isMobile) {
				this.element.append($("<div>").addClass("docSelectorCaret").append($("<span/>").addClass("caret")));
			}

			/**
			 * @member {Object} the selected document
			 * @private
			 */
			this._selectedDocumentPage = null;

			/**
			 * @member {Number} the number of enabled documents
			 * @private
			 */
			this._enabledDocuments = 0;

			/**
			 * @member {Object} maps a group name to an array of element names
			 * @private
			 */
			this._elementGrouping = {};

			/**
			 * @member {String} the currently active group, group of which all elements are visible
			 * @private
			 */
			this._activeGroup = null;

			/**
			 * @member {DataConstants.documentPageType} - if the dropdown is restricted to show only document pages of this doc type
			 * @private
			 */
			this._enabledDocType = null;

			/**
			 * @member {String} id of the divider element in the dropdown
			 */
			this.dividerId = null;

			/**
			 * @member {Template} template used to generate each row of document page information
			 */
			this.t = new Template(this.f.i18n, tpl);

			this.unveil = _.debounce(_.partial(this._unveil, true), 100);
			this.t.d.compact = isCompact;
			this._deselectText = deselectText || "";
			this._nothingSelectedText = nothingSelectedText || "";
			this._deselectCallback = deselectCallback;

			if (this._deselectText !== "" && this._nothingSelectedText !== "") {
				var html = $("<div />").addClass("deselectElement").html(this._nothingSelectedText);
				this.deselectElement = {
					name: "deselectAction",
					content: html,
					icon: undefined,
					action: _.bind(function () {
						var selectedPageId = this.getSelectedDocumentPageId();
						this.deselectCurrentPage(true);
						this._deselectCallback(selectedPageId);
					}, this),
					enabled: true
				};
				this.addAction(this.deselectElement);
				this.disableDocumentPage(this.deselectElement.name);
				this._selectedDocumentPage = this.deselectElement;
				this._selectedDocumentElement.html(this._selectedDocumentPage.content.clone());
				this._enabledDocuments++;
			}
			if (emptyListText !== "") {
				var html = $("<div />").addClass("deselectElement").html(emptyListText);
				this.emptyListElement = {
					name: "emptyList",
					content: html,
					icon: undefined,
					action: function () {},
					enabled: true
				};
				this.addAction(this.emptyListElement);
			}
		};

		DocumentDropdownMenu.prototype = Object.create(DropdownMenu.prototype);
		DocumentDropdownMenu.prototype.constructor = DocumentDropdownMenu;

		/**
		 * Initialize the document dropdown menu
		 * @private
		 * @override
		 */
		DocumentDropdownMenu.prototype._initialize = function () {
			DropdownMenu.prototype._initialize.call(this);
			if (this._isMobile) {
				this.popover.setPlacementOptions(["bottomLeft"]);
			} else {
				this.popover.setPlacementOptions(["bottom", "top"]);
			}
		};

		/**
		 * Add a document to the dropdown menu
		 * @param {Object} documentInfo : description of the document
		 * @param {String} groupName - name of the group the document belongs to
		 * @example
		 * {
		 * 	pageId: 'uniquePageId',
		 * 	project: 'projectName',
		 * 	documentName: 'nameOfTheDocument',
		 * 	[versionNumber: number,]
		 * 	[pageNumber: page,]
		 * 	[pageCount: numberOfPages,]
		 *  thumbnailUrl: String,
		 *  docType: docTypeConstant,
		 *  [isVisible: Boolean,]
		 *  [group: String]
		 * }
		 */
		DocumentDropdownMenu.prototype.addDocumentPage = function (docInfo, groupName) {
			if (groupName !== undefined && this._elementGrouping[groupName] !== undefined) {
				if (this._elementGrouping[groupName].length === 0) { //no previous elements where added to the group, use the first page's information for the group element
					this.t.d.docInfo = _.clone(docInfo); //make sure to make copy, will be changing this object for generating html
					this.t.d.docInfo.isGroup = true; //identify this element as group
					this.t.d.docInfo.isOpen = this.items[groupName].isOpen;
					this.t.d.thumbnailUrl = docInfo.thumbnailUrl;
					var html = this.t.getRenderedHtml();
					this.setActionText(groupName, $(html)); //copy generated html in group element
					if (this._enabledDocType !== null && docInfo.docType !== this._enabledDocType) {
						this.disableDocumentPage(groupName);
					}
				}
				this._elementGrouping[groupName].push(docInfo.pageId);
			}
			if (this.items[docInfo.pageId] === undefined) { //to make sure a document page doesn't get added twice
				this.t.d.docInfo = docInfo;
				this.t.d.thumbnailUrl = "";
				var html = this.t.getRenderedHtml();
				var dropdownElement = {
					name: docInfo.pageId,
					content: html,
					icon: undefined,
					action: _.bind(function (pageId) {
						this._itemClicked(pageId);
					}, this, docInfo.pageId),
					enabled: true,
					docType: docInfo.docType,
					isVisible: (groupName === undefined //if page doesn't belong to a group, make it visible
						|| (this._activeGroup === groupName //if the groupName is the same as the currently active group 
							&& (this._enabledDocType === null || docInfo.docType === this._enabledDocType))), //and the enabled doctype is null or the doctypes are the same make the element visible
					group: groupName, //name of the group this element belongs to
					thumb: docInfo.thumbnailUrl,
					isUnveiled: false,
					after: (docInfo.prepend && this.deselectElement) ? this.deselectElement.name : undefined
				};
				this.addAction(dropdownElement);
				if (dropdownElement.isVisible) { //only if the element is visible at creation of the element add to the counter of currently active pages
					this._enabledDocuments++;
				}
				if (this.emptyListElement !== undefined && this._enabledDocuments > 0) {
					this.hideElement(this.emptyListElement.name);
					this.showElement(this.dividerId);
				}
			}
		};

		/**
		 * Remove a document page from the document dropdown menu.
		 * @param {String} pageId - the unique ID for the document page
		 */
		DocumentDropdownMenu.prototype.removeDocumentPage = function (pageId) {
			this.removeAction(pageId);
			delete this.items[pageId];
		};

		/**
		 * Add a divider to the dropdown menu, there can only be one divider added to the dropdown menu
		 */
		DocumentDropdownMenu.prototype.addDivider = function () {
			if (this.dividerId === null) {
				this.dividerId = DropdownMenu.prototype.addDivider.call(this); //same the id to be able to hide when the emptylist element is shown
				if (this.emptyListElement !== undefined && this.emptyListElement.isVisible) { //if the emptylist element is already visible, make sure to hide the divider
					this.hideElement(this.dividerId);
				}
			}
		};

		/**
		 * Add a document group to the dropdown menu. After adding the group document pages can be added to that group.
		 * The group element will be shown in the dropdown when the enabled document type is the same as the elements in the group and
		 * the elements of the group are not enabled in the dropdown. Only document pages of the same document type should be added to
		 * a document group.
		 * @param {String} groupName - name of the group, is the name for the element in the dropdown
		 * @param {Function} [beforeClickCallback] - callback will be called before the group element in the dropdown is clicked.
		 * @param {Function} [afterClickCallback] - callback will be called after the group element in the dropdown is clicked.
		 */
		DocumentDropdownMenu.prototype.addDocumentGroup = function (groupName, beforeClickCallback, afterClickCallback) {
			this._elementGrouping[groupName] = [];
			var html = $("<div />"); //first create an empty element for the group, when adding document pages to the group, the first added page will be used for the dropdown element
			this.groupElement = {
				name: groupName,
				content: html,
				icon: undefined,
				isOpen: false,
				action: _.bind(function () {
					if (beforeClickCallback !== undefined) {
						beforeClickCallback(); //call the callback at the beginning of the click
					}

					if (this.isGroupOpen(groupName)) {
						this.hideElementGroup(groupName);
					} else {
						this.showElementGroup(groupName); //show the clicked group
					}

					// dropdown size has changed, so trigger repositioning and unveiling
					this.show();

					if (afterClickCallback !== undefined) {
						afterClickCallback(); //call the callback at the end of the click
					}
				}, this),
				enabled: true,
				doNotHideOnClick: true, //to make sure the dropdown will not be hidden when the group element is clicked in the dropdown
				isUnveiled: true,
				isGroup: true
			};
			this._enabledDocuments++; //add as an enabled document, the element should be hidden later if that's needed
			this.addGroup(this.groupElement); //add the group element to the dropdown, prepend group elements to make sure they are above the divider
		};

		/**
		 * Set the selected document page
		 * @param {String} pageId
		 */
		DocumentDropdownMenu.prototype.setSelectedDocumentPage = function (pageId) {
			if (this._selectedDocumentPage !== this.deselectElement && this._selectedDocumentPage !== null) {
				this.enableDocumentPage(this._selectedDocumentPage.name);
			} else if (this._selectedDocumentPage !== null) {
				this.deselectElement.content.html(this._deselectText);
				this.enableDocumentPage(this.deselectElement.name);
			}
			this._selectedDocumentPage = this.items[pageId];
			if (this._selectedDocumentPage !== undefined) {
				this._selectedDocumentElement.html(this._selectedDocumentPage.content);
				this._selectedDocumentElement.find(".thumbnail").attr("src", this._selectedDocumentPage.thumb);
			} else {
				this._selectedDocumentPage = null;
			}

			this.disableDocumentPage(pageId);
		};

		/**
		 * Set a temp content for the document selector, this will not be added to the dropdown list when getting deactivated.
		 * It sets the currently selected document page to null.
		 * @param {Object} docInfo
		 */
		DocumentDropdownMenu.prototype.setTempSelectedDocumentPage = function (docInfo) {
			this.t.d.docInfo = docInfo;
			this.t.d.thumbnailUrl = docInfo.thumbnailUrl;
			var html = this.t.getRenderedHtml();
			if (this._selectedDocumentPage !== this.deselectElement && this._selectedDocumentPage !== null) {
				this.enableDocumentPage(this._selectedDocumentPage.name);
			} else if (this._selectedDocumentPage !== null) {
				this.deselectElement.content.html(this._deselectText);
				this.enableDocumentPage(this.deselectElement.name);
			}
			this._selectedDocumentElement.html(html);
			this._selectedDocumentPage = null;
		};

		/**
		 * Is called when an item in the dropdown is clicked
		 * @private
		 * @param {String} pageId - the id of the document page
		 */
		DocumentDropdownMenu.prototype._itemClicked = function (pageId) {
			this.hide();
			this.setSelectedDocumentPage(pageId);
			this._onDocChange(pageId);
		};

		/**
		 * Disable a document in the dropdown list, the document will no longer be visible
		 * @param {String} pageId
		 */
		DocumentDropdownMenu.prototype.disableDocumentPage = function (pageId) {
			//needed to check if it wasn't visible before, make sure the enabledDocuments count stays correct
			if (this.isElementVisible(pageId)) {
				this.hideElement(pageId);
				this._enabledDocuments--;
				if (this.emptyListElement !== undefined && this._enabledDocuments <= 0) {
					this.showElement(this.emptyListElement.name);
					this.hideElement(this.dividerId);
				}
			}
		};

		/**
		 * Enable a document in the dropdown list
		 * @param {String} pageId
		 */
		DocumentDropdownMenu.prototype.enableDocumentPage = function (pageId) {
			//needed to check if it wasn't visible before, make sure the enabledDocuments count stays correct
			if (this.isElementVisible(pageId) !== undefined && !this.isElementVisible(pageId)
				//test if either the element is in no group or the pages that needs to be enabled belongs to the active group
				&& (this.items[pageId].group === undefined || this.isGroupOpen(this.items[pageId].group))) {
				this.showElement(pageId);
				this._enabledDocuments++;
				if (this.emptyListElement !== undefined && this._enabledDocuments > 0) {
					this.hideElement(this.emptyListElement.name);
					this.showElement(this.dividerId);
				}
			}
		};

		/**
		 * enable all document page of specific document type
		 * @param {DataConstants.documentPageType} docTypeConstant
		 */
		DocumentDropdownMenu.prototype.enableOnlyPagesWithDocType = function (docTypeConstant) {
			_.each(this.items, _.bind(function (item) {
				//make sure the element doesn't belong to a group, this will be handled later as all members of a group are of the same document type
				if (item.docType !== undefined && item.docType === docTypeConstant && item.group === undefined) {
					this.enableDocumentPage(item.name);
				} else if (item.docType !== undefined && item.group === undefined) {
					this.disableDocumentPage(item.name);
				}
			}, this));
			//set the enabled document type, only document pages of this type should now be enabled in the dropdown menu
			this._enabledDocType = docTypeConstant;

			//for the groups:
			_.each(this._elementGrouping, _.bind(function (group, groupName) { //iterate over all groups
				var itemOfGroup = this.items[group[0]]; //get an item of the group, will be used to test document type, so doesn't matter which element
				if (itemOfGroup !== undefined && itemOfGroup.docType !== docTypeConstant) { //if the document type is not the same
					this.hideElementGroup(groupName); //first hide all elements of the group, this makes the group element visible
					this.disableDocumentPage(groupName); //make sure to hide the group element
				} else if (itemOfGroup !== undefined && !this.items[groupName].isVisible && !itemOfGroup.isVisible) { //if both group element and all elements of the group are not visible
					this.hideElementGroup(groupName); //make sure at least the group element is visible (doc types are correct)
				}
			}, this));
			//deselect the selected document pages if they are not of the same type
			if (this._selectedDocumentPage !== null && docTypeConstant !== this._selectedDocumentPage.docType) {
				this.deselectCurrentPage(false);
			} else if (this._selectedDocumentPage !== null) {
				this.disableDocumentPage(this._selectedDocumentPage.name);
			}

			if (this._selectedDocumentPage !== null && this.deselectElement !== undefined && this.deselectElement !== this._selectedDocumentPage) {
				this.enableDocumentPage(this.deselectElement.name);
			}
		};

		/**
		 * Returns the unique id of the currently selected document page
		 * @returns {String|undefined} - unique id of the document page or undefined if there is no selected document page
		 */
		DocumentDropdownMenu.prototype.getSelectedDocumentPageId = function () {
			if (this._selectedDocumentPage !== null && this._selectedDocumentPage !== this.deselectElement) {
				return this._selectedDocumentPage.name;
			} else {
				return undefined;
			}
		};

		/**
		 * deselect the currently selected document page
		 * @param {Boolean} enableDocument - should the deselected document be added to the dropdownlist
		 */
		DocumentDropdownMenu.prototype.deselectCurrentPage = function (enableDocument) {
			if (enableDocument && this._selectedDocumentPage !== null) {
				this.enableDocumentPage(this._selectedDocumentPage.name);
			}
			if (this._deselectText !== "" && this._nothingSelectedText !== "") {
				this._selectedDocumentPage = this.deselectElement;
				this.disableDocumentPage(this.deselectElement.name);
				this.deselectElement.content.html(this._nothingSelectedText);
				this._selectedDocumentElement.html(this._selectedDocumentPage.content.clone());
			} else {
				this._selectedDocumentElement.html("");
				this._selectedDocumentPage = null;
			}
		};

		/**
		 * Hide all the elements belonging to a group, this will active the group element if there is no enabled document type
		 * @param {String} groupName
		 */
		DocumentDropdownMenu.prototype.hideElementGroup = function (groupName) {
			if (groupName !== undefined) {
				if (this._listElements[groupName] !== undefined) {
					this._listElements[groupName].first().find(".showAllPages").show();
					this._listElements[groupName].first().find(".hideAllPages").hide();
				} else {
					this.items[groupName].content.find(".showAllPages").show();
					this.items[groupName].content.find(".hideAllPages").hide();
				}
				this.items[groupName].isOpen = false;
				_.each(this._elementGrouping[groupName], _.bind(function (groupElementName) {
					this.disableDocumentPage(groupElementName); //first hide all the members of the group
				}, this));
				var elementOfGroup = this._elementGrouping[groupName][0]; //get a representative of the group
				//if the docType restriction for the dropdown is not the same as the docType of the group make sure to also hide the group element
				if (this._enabledDocType !== null && elementOfGroup !== undefined && this._enabledDocType !== this.items[elementOfGroup].docType) {
					this.disableDocumentPage(groupName);
				} else {
					this.enableDocumentPage(groupName);
				}
			}
		};

		/**
		 * Shows all the elements belonging to a group, this will hide the group element
		 * @param {String} groupName
		 */
		DocumentDropdownMenu.prototype.showElementGroup = function (groupName) {
			if (groupName !== undefined) {
				if (this._listElements[groupName] !== undefined) {
					this._listElements[groupName].first().find(".showAllPages").hide();
					this._listElements[groupName].first().find(".hideAllPages").show();
				} else {
					this.items[groupName].content.find(".showAllPages").hide();
					this.items[groupName].content.find(".hideAllPages").show();
				}
				this.items[groupName].isOpen = true;
				this._activeGroup = groupName;
				_.each(this._elementGrouping[groupName], _.bind(function (groupElementName) {
					this.enableDocumentPage(groupElementName);
				}, this));
			}
			this._unveil();
		};

		/**
		 * Hides all elements belonging to a group, will show all the group elements
		 */
		DocumentDropdownMenu.prototype.hideAllGroups = function () {
			_.each(this._elementGrouping, _.bind(function (group, groupName) {
				this.hideElementGroup(groupName);
			}, this));
		};

		/**
		 * Has the dropdown menu the document page with docId
		 * @param {String} docId
		 * @returns {Boolean}
		 */
		DocumentDropdownMenu.prototype.hasDocumentPage = function (docId) {
			return this.items[docId] !== undefined;
		};

		/**
		 * Has the dropdown menu the document group with groupId
		 * @param {String} groupId
		 * @returns {Boolean}
		 */
		DocumentDropdownMenu.prototype.hasDocumentGroup = function (groupId) {
			return this.items[groupId] !== undefined;
		};

		/**
		 * Does the dropdown menu have any groups
		 * @returns {Boolean}
		 */
		DocumentDropdownMenu.prototype.hasGroups = function () {
			return !_.isEmpty(this._elementGrouping);
		};

		/**
		 * Is the group with group name open
		 * @param {String} groupName
		 * @returns {Boolean}
		 */
		DocumentDropdownMenu.prototype.isGroupOpen = function (groupName) {
			return this.items[groupName].isOpen;
		};

		/**
		 * Show the document dropdown menu
		 */
		DocumentDropdownMenu.prototype.show = function () {
			DropdownMenu.prototype.show.call(this); //show the drop down menu
			this._unveil(); //unveil thumbnails if needed
			if (!this.scrollListenerRegistered) {
				this.popover.getContentDiv().scroll(_.bind(this.unveil, this)); //register scroll listener
				this.scrollListenerRegistered = true;
			}
		};

		/**
		 * Checks if the dropdown menu is open and gets all the elements that are visible in the list. If there are visible
		 * elements where the thumbnail has not yet been unveiled sets the src-element to the correct value so the document
		 * selector requests the thumbnail.
		 *
		 * @param {Boolean} [keepList] - should the previously constructed list of visible elements of the document selector be used, this happens when
		 * 								no elements become visible/hidden (when scrolling for example)
		 */
		DocumentDropdownMenu.prototype._unveil = function (keepList) {
			//first order all the elements of the lsit so all the elements are in order of appearance in the docSelector
			//if the list doesn't exist yet(is empty) or the existing list shouldn't be used the check to unveil thumbnails, construct the list
			if (this.indexList === undefined || this.indexList.length === 0 || !keepList) {
				var orderedListElements = [];
				
				//run over all the elements that have been added to the DOM
				_.each(this._listElements, _.bind(function (element, key) {
					var item = this.items[key]; //get the item that contains the data of this DOM-element
					if (item.isVisible) { //check if the element in the document controller is visible
						var groupIndex;
						if (item.type === "group") { //if the element is a group
							groupIndex = element.closest(".group").index(); // The closest .group element is the encompassing group element
							orderedListElements[groupIndex] = []; //add an array at the location to fill with all elements of this group
							orderedListElements[groupIndex][0] = item; //add the group element to the array as first element
						} else if (item.group !== undefined) { //the element is part of a group and index is relative to the div-element of the group
							groupIndex = element.closest(".group").index();
							orderedListElements[groupIndex][element.index()] = item; //add the element to the array of the group element
						} else { //the element is not a group nor belonging to a group
							orderedListElements[element.index()] = item;
						}
					}
				}, this));
				//to get all the elements as a 1 level deep array, flatten the array so elements of groups are placed at the correct location
				this.indexList = _.without(_.flatten(orderedListElements), undefined, null); //array in the correct order of appearance in the document selector
			}
			//only if there are elements in the list, document selector is not empty continue
			if (this.indexList.length > 0) {
				var scrollValue = 0; //how many pixels are the top of the element (not visible due to scroll)
				if (this.popover !== undefined) { //check if the dropdown is open and the popover is visible
					scrollValue = this.popover.getContentDiv().scrollTop();
				}
				var elementHeight = Math.ceil(this.menu.outerHeight(true) / this.indexList.length); // we can't rely on outerheight of a single element (it could be a group, and margins are collapsed on eachother), we do now how many elements we have and how much space they take in total :)
				var startIndex = Math.floor(scrollValue / elementHeight); //start index of the visible elements, how many elements are above and not visible

				var visibleHeight = this.popover.getContentDiv().innerHeight(); //calculate the height of the dropdown menu
				var endIndex = startIndex + Math.ceil(visibleHeight / elementHeight); //get the index of the last visible element
				if (endIndex > this.indexList.length - 1) { //make sure we do not move outside the indexlist array when going over visible elements
					endIndex = this.indexList.length - 1;
				}

				//run over all the visible elements
				for (var i = startIndex; i <= endIndex; i++) {
					var item = this.indexList[i];
					if (item) {
						var listElement = this._listElements[item.name];
						if (!item.isUnveiled && listElement !== undefined) { //check whether the thumbnail is already unveiled
							listElement.find(".thumbnail").attr("src", item.thumb); //unveil thumbnail
							item.isUnveiled = true;
						}
					}
				}
			}
		};

		/**
		 * Unveil thumbnails if this is needed.
		 */
		DocumentDropdownMenu.prototype.unveilThumbnails = function () {
			this._unveil();
		};

		/**
		 * Get an ordered list of the items as they appear in the dropdown menu
		 * @returns {Array.<Object>}
		 * @private
		 */
		DocumentDropdownMenu.prototype._getOrderedItemList = function () {
			if (this.menu) { //the menu list was openend, all items are in the DOM, 
				//use this since opened groups add documents to the end fo the itemStack
				return _.map(this.menu.find("li[data-name]"), function (itemElement) {
					return this.items[$(itemElement).attr("data-name")];
				}, this);
			} else {
				return this.itemStack;
			}
		};

		/**
		 * Get the page id of the first selectable item in the dropdown list
		 * @returns {String|null} Page Id, e.g. "0_1_2"
		 */
		DocumentDropdownMenu.prototype.getFirstSelectablePageId = function () {
			var listItems = this._getOrderedItemList();
			var index = 1;
			var item = listItems[index];
			while (index < listItems.length && item && item.isGroup) {
				item = listItems[++index];
			}
			if (index < listItems.length && item) {
				return item.name;
			} else {
				return null;
			}
		};

		/**
		 * Get the page id of the last selectable item in the dropdown list
		 * @returns {String|null} Page Id, e.g. "0_1_2"
		 */
		DocumentDropdownMenu.prototype.getLastSelectablePageId = function () {
			var listItems = this._getOrderedItemList();
			var item = _.last(listItems);
			var index = _.indexOf(listItems, item);
			while (index > 0 && item && item.isGroup) {
				item = listItems[--index];
			}
			if (item && index > 0) {
				return item.name;
			} else {
				return null;
			}
		};

		/**
		 * Get the page id of the next selectable item, relative to the currently active document page, in the dropdown list
		 * @returns {String|null} Page Id, e.g. "0_1_2"
		 */
		DocumentDropdownMenu.prototype.getNextItemId = function () {
			var listItems = this._getOrderedItemList();
			var selectedPageId = this.getSelectedDocumentPageId();
			var index = _.indexOf(listItems, this.items[selectedPageId]);
			var item = listItems[++index];
			while (index < listItems.length && item && item.isGroup) {
				item = listItems[++index];
			}
			if (item && index < listItems.length) {
				return item.name;
			} else {
				return null;
			}
		};

		/**
		 * Get the page id of the previous selectable item, relative to the currently active document page, in the dropdown list
		 * @returns {String|null} Page Id, e.g. "0_1_2"
		 */
		DocumentDropdownMenu.prototype.getPreviousItemId = function () {
			var listItems = this._getOrderedItemList();
			var selectedPageId = this.getSelectedDocumentPageId();
			var index = _.indexOf(listItems, this.items[selectedPageId]);
			var item = listItems[--index];
			while (index > 0 && item && item.isGroup) {
				item = listItems[--index];
			}
			if (item && index > 0) {
				return item.name;
			} else {
				return null;
			}
		};


		return DocumentDropdownMenu;
	});

define('web-viewer/layout/component/DocumentSelection',["web-viewer/core/PrivateChannel",
		"web-viewer/layout/component/DocumentDropdownMenu",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/data/DataConstants",
		"underscore"
	],
	function (PrivateChannel,
		DocumentDropdownMenu,
		ListenerUtil,
		DataConstants,
		_) {
		"use strict";
		/**
		 * This component will take care of document selection.
		 * It contains a document selector for active documents and for difference document.
		 *
		 * @exports DocumentSelection
		 * @memberof Layout.Component
		 * @constructor
		 * @param {Core.ViewerFacade} f - The viewer facade
		 * @param {Compare.CompareModule} [compareModule] - compareModule, needed for difference document selector
		 * @param {Function} [onDocChangeCallback] - Called when active or difference document is changed
		 * @param {Boolean} [isMobile] - is mobile layout
		 * @param {Boolean} [isCompact] - if true version number is put behind the document (on the same line)
		 */
		var DocumentSelection = function (f, compareModule, onDocChangeCallback, isMobile, isCompact) {
			PrivateChannel.installTo(this, f, "documentSelection");
			this.f = f;
			ListenerUtil.mixin(this, f);

			//makes sure callbacks are only registered once when new documents are added
			this._callBacksAreRegistered = false;

			/**
			 * @Member {Function}
			 */
			this._onDocChangeCallback = onDocChangeCallback;

			/**
			 * @Member {compare.CompareModule}
			 */
			this.compareModule = compareModule;

			/**
			 * @member {DocumentDropdownMenu} dropdown menu for selecting the active document page
			 */
			this.activeDocPageSelector = null;

			/**
			 * @member {DocumentDropdownMenu} dropdown menu for selecting the difference document page
			 */
			this.differenceDocPageSelector = null;

			/**
			 * @member {DocumentDropdownMenu} dropdown menu for selecting the CAD overlay
			 */
			this.cadOverLaySelector = null;

			this.f.i18n.loadBundle("viewer", _.bind(function () {
				this.activeDocPageSelector = new DocumentDropdownMenu(f, _.bind(this.changeActivePage, this), this.f.t("EmptyDocumentList"), undefined, undefined, undefined, isMobile, isCompact);

				if (this.compareModule) {
					this.differenceDocPageSelector = new DocumentDropdownMenu(f, _.bind(this.changeDifferencePage, this), this.f.t("EmptyDocumentList"), this.f.t("EmptyDifferenceDocSelector"), this.f.t("DocumentListCompareTurnOff"), _.bind(function (deselectedPageId) {
						this.activeDocPageSelector.enableDocumentPage(deselectedPageId);

						// disable compare
						this.compareModule.activateModeByName();
					}, this), isMobile, isCompact);


					this.cadOverLaySelector = new DocumentDropdownMenu(f, _.bind(this.changeCadOverlay, this), this.f.t("EmptyDocumentList"), this.f.t("EmptyCadOverlayDocSelector"), this.f.t("DocumentListCadOverlayTurnOff"), _.bind(function (deselectedPageId) {
						// turn of CAD overlay
						this.f.getDocumentController().changeCadOverlay();
					}, this), isMobile, isCompact);
				}

				this.f.getDocumentController().whenDocumentsLoadedOrChange(this._whenDocumentsLoaded, this);
			}, this));
		};

		/**
		 * Get active document selector
		 * @return {DocumentDropdownMenu}
		 */
		DocumentSelection.prototype.getActiveDocumentSelector = function () {
			return this.activeDocPageSelector;
		};

		/**
		 * Get difference document selector
		 * @return {DocumentDropdownMenu}
		 */
		DocumentSelection.prototype.getDifferenceDocumentSelector = function () {
			return this.differenceDocPageSelector;
		};

		/**
		 * Get the CAD overlay document selector
		 * @returns {DocumentDropdownMenu} /
		 */
		DocumentSelection.prototype.getCadOverlayDocumentSelector = function () {
			return this.cadOverLaySelector;
		};


		/**
		 * Changes the active document page
		 * @param {String} id - unique id of the document page
		 */
		DocumentSelection.prototype.changeActivePage = function (id) {
			var newActivePage = this.f.getDocumentController().getDocumentPage(id);
			var activePage = this.f.getDocumentController().getActivePage();

			this._emit("changeActivePage", newActivePage, _.bind(function (isChanged) {
				if (isChanged && this.differenceDocPageSelector) {
					if (activePage !== undefined) {
						var pageId = activePage.getUniqueID();
						//make sure that the previous active document page can be selected as difference page
						this.differenceDocPageSelector.enableDocumentPage(pageId);
					}
					// reset difference when there is another type of document selected
					if (activePage.getDocument().getDocumentType() !== newActivePage.getDocument().getDocumentType()) {
						this.activeDocPageSelector.enableDocumentPage(this.differenceDocPageSelector.getSelectedDocumentPageId());
						this.compareModule.activateModeByName();
					}

					var docType = newActivePage.getDocument().getDocumentType();
					// enable as difference only documents of the same type
					this.differenceDocPageSelector.enableOnlyPagesWithDocType(docType);
					//the new active page can't be selected as difference document page
					this.differenceDocPageSelector.disableDocumentPage(id);
				}

				if (!isChanged) {
					this.activeDocPageSelector.setSelectedDocumentPage(this.f.getActivePage().getUniqueID());
				}
			}, this));
		};

		/**
		 * changes the difference document page
		 * @param {String} id - unique id of the document page
		 */
		DocumentSelection.prototype.changeDifferencePage = function (id) {
			var differencePage = this.f.getDocumentController().getDifferencePage();

			if (differencePage !== undefined) {
				var pageId = differencePage.getUniqueID();
				//make sure that the previous difference document page can be selected as active page
				this.activeDocPageSelector.enableDocumentPage(pageId);
			}
			this.activeDocPageSelector.disableDocumentPage(id);
			var newDiffPage = this.f.getDocumentController().getDocumentPage(id);
			this._emit("changeReferencePage", newDiffPage);
		};

		/**
		 * Changes the cad voerlay
		 * @param {String} id - unique id of the document page
		 */
		DocumentSelection.prototype.changeCadOverlay = function (id) {
			var docController = this.f.getDocumentController();
			var cadOverlayPage = docController.getDocumentPage(id);
			this._emit("changeCadOverlay", cadOverlayPage);

		};


		/**
		 * Called when documents are loaded or when new documents are added
		 * @param {Array.<AbstractDocument>} documents
		 */
		DocumentSelection.prototype._whenDocumentsLoaded = function (documents) {
			//this function is called when later on new documents are added or documents change (new version, new pages)
			//so make sure it takes this into account (check if page or group was already in the dropdown)
			_.each(documents, _.bind(function (document) {
				_.each(document.getAllDocumentVersions(), _.bind(function (docVersion) {
					if (docVersion) {
						if (docVersion.isMultiPage()) {
							var groupName = docVersion.getDocument().getIID() + "_" + docVersion.getVersionId() + "_Group";
							if (!this.activeDocPageSelector.hasDocumentGroup(groupName)) {
								//add a document group if it wasn't in the dropdown before, call loadPagesIfNeeded as callback to check wether pages still need to be loaded
								this.activeDocPageSelector.addDocumentGroup(groupName,
									// before show/hide
									_.bind(this._loadPagesIfNeeded, this, docVersion),
									// after show/hide
									_.bind(function () {
										// always disable the current visible active/diff page again if the whole group was shown again
										this.activeDocPageSelector.disableDocumentPage(this.activeDocPageSelector.getSelectedDocumentPageId());
										if (this.differenceDocPageSelector) {
											this.activeDocPageSelector.disableDocumentPage(this.differenceDocPageSelector.getSelectedDocumentPageId());
										}
									}, this));
							}
							if (this.differenceDocPageSelector && !this.differenceDocPageSelector.hasDocumentGroup(groupName)) {
								this.differenceDocPageSelector.addDocumentGroup(groupName,
									// before show/hide
									_.bind(this._loadPagesIfNeeded, this, docVersion),
									// after show/hide
									_.bind(function () {
										// always disable the current visible active/diff page again if the whole group was shown again
										this.differenceDocPageSelector.disableDocumentPage(this.differenceDocPageSelector.getSelectedDocumentPageId());
										this.differenceDocPageSelector.disableDocumentPage(this.activeDocPageSelector.getSelectedDocumentPageId());
									}, this));
							}
						}

						var pageCount = docVersion.getPageCount();
						for (var i = 1; i <= pageCount; i++) {
							var docPage = docVersion.getPage(i);
							if (docPage !== undefined) {
								var pageInfo = docPage.getInformationForDocumentSelector();

								var groupName;
								if (docVersion.isMultiPage()) {
									groupName = docVersion.getDocument().getIID() + "_" + docVersion.getVersionId() + "_Group";
								}
								if (!this.activeDocPageSelector.hasDocumentPage(pageInfo.pageId)) {
									this.activeDocPageSelector.addDocumentPage(pageInfo, groupName);
								}
								if (this.differenceDocPageSelector && !this.differenceDocPageSelector.hasDocumentPage(pageInfo.pageId) && !docPage.isBlankPage()) {
									this.differenceDocPageSelector.addDocumentPage(pageInfo, groupName);
								}

								if (this.cadOverLaySelector) {
									if (docPage.hasCad()) {
										this.cadOverLaySelector.addDocumentPage(pageInfo);
									} else if (docPage.hasLinkedPage()) {
										var linkedCadDocInfo = docPage.getLinkedPage().getInformationForDocumentSelector();
										linkedCadDocInfo.referencedFrom = docPage.getDocument().get("docname");
										this.cadOverLaySelector.addDocumentPage(linkedCadDocInfo);
									}
								}
							}
						}
					}
				}, this));
			}, this));
			//after elements have been added, unveil thumbnails if needed (doc selector might be open)
			this.activeDocPageSelector.unveilThumbnails();
			if (this.differenceDocPageSelector) {
				this.differenceDocPageSelector.unveilThumbnails();
			}
			this._registerCallbacksIfNeeded();
		};

		/**
		 * Registers the callbacks when the document selectors are initialized, makes sure they are only registered once.
		 * @private
		 */
		DocumentSelection.prototype._registerCallbacksIfNeeded = function () {
			if (!this._callBacksAreRegistered) {
				//To make sure first the document selectors are filled with data before the active page is initialized
				this.f.getDocumentController().whenActivePageChangedAndParametersAvailable(function (oldPage, page) {
					//check if the active document page was changed from somewhere else than the document selector
					if (this.activeDocPageSelector.getSelectedDocumentPageId() === undefined || page.getUniqueID() !== this.activeDocPageSelector.getSelectedDocumentPageId()) {
						if (page.getDocumentVersion().isMultiPage()) {
							this.activeDocPageSelector.hideAllGroups();
							var groupName = page.getDocument().getIID() + "_" + page.getDocumentVersion().getVersionId() + "_Group";
							this.activeDocPageSelector.showElementGroup(groupName);
						}
						this.activeDocPageSelector.setSelectedDocumentPage(page.getUniqueID());

						if (this.differenceDocPageSelector) {
							//if multi page first convert to graphics document type
							var docType = page.getDocument().getDocumentType();
							this.differenceDocPageSelector.enableOnlyPagesWithDocType(docType);
							this.differenceDocPageSelector.disableDocumentPage(page.getUniqueID());
							this.differenceDocPageSelector.hideAllGroups();

							if (page.getDocumentVersion().isPageList()) {
								_.each(page.getDocument().getAllDocumentVersions(), function (docVersion) {
									if (docVersion) {
										var diffPage = docVersion.getPage(page.getPagenum());
										if (diffPage) {
											this.differenceDocPageSelector.enableDocumentPage(diffPage.getUniqueID());
										}
									}
								}, this);
							}

							//for the first active page make sure to load pages of previous version for pagelist documents
							//only for this case is the oldPage === undefined
							if (oldPage === undefined && page.getDocumentVersion().isPageList()) {
								this.updateDocumentSelectorsForPageList(undefined, page);
							}
						}

					}

					if (this._onDocChangeCallback) {
						this._onDocChangeCallback();
					}
				}, this);
				if (this.differenceDocPageSelector) {
					this.f.getDocumentController().whenDifferencePageChangedAndParametersAvailable(function (oldPage, page) {
						//deselect the current page in the difference selector if reader spread is active (enabled) and a page is selected in the doc selector 
						var isReaderSpread = this.compareModule.isReaderSpreadActive();
						var activeMode = this.compareModule.getActiveMode();
						if (isReaderSpread && this.differenceDocPageSelector.getSelectedDocumentPageId() !== undefined && activeMode.isEnabled()) {
							this.differenceDocPageSelector.deselectCurrentPage(true);
							//if there is no active mode or the active mode is not enabled or the active mode is not reader spread
						} else if (activeMode === null || !activeMode.isEnabled() || !isReaderSpread) {
							if (page === undefined) {
								// set selected difference page to not selected
								this.differenceDocPageSelector.deselectCurrentPage(true);
								if (oldPage) {
									this.activeDocPageSelector.enableDocumentPage(oldPage.getUniqueID());
								}
							} else if (page.isBlankPage()) {
								// set selected difference page to not selected
								//this.differenceDocPageSelector.deselectCurrentPage(true);
								if (oldPage !== undefined) {
									this.activeDocPageSelector.enableDocumentPage(oldPage.getUniqueID());
								}
								this.differenceDocPageSelector.setTempSelectedDocumentPage(page.getInformationForDocumentSelector());
								this.f.emit("error.WARNING", this.f.i18n.t("ComparingWithBlankPage"));
							} else {
								//check if the difference document page is changed from somewhere other than the document selector
								if (this.activeDocPageSelector.getSelectedDocumentPageId() === undefined || page.getUniqueID() !== this.differenceDocPageSelector.getSelectedDocumentPageId()) {
									this.differenceDocPageSelector.setSelectedDocumentPage(page.getUniqueID());
									this.activeDocPageSelector.disableDocumentPage(page.getUniqueID());
									if (oldPage) {
										this.activeDocPageSelector.enableDocumentPage(oldPage.getUniqueID());
									}
								}
							}
						}
						if (this._onDocChangeCallback) {
							this._onDocChangeCallback();
						}
					}, this);
				}

				if (this.cadOverLaySelector) {
					this.f.getDocumentController().whenCadOverlayChangedAndParametersAvailable(function (oldPage, newPage) {
						this.cadOverLaySelector.deselectCurrentPage(true);
						if (newPage) {
							this.cadOverLaySelector.setSelectedDocumentPage(newPage.getUniqueID());
						}

						if (this._onDocChangeCallback) {
							this._onDocChangeCallback();
						}
					}, this);
				}

				this._callBacksAreRegistered = true;
			}
		};

		/**
		 * Set difference selector page to the given diff page. If undefined, the current page is de-selected and again added to the list of possible difference pages.
		 * If reader-spread is active, the page is deselected and added to the list also if diffpage is defined.
		 * @param {AbstractDocumentPage} [diffPage] - Diff page to set in the difference selector.
		 */
		DocumentSelection.prototype.setDifferenceSelectorPage = function (diffPage) {
			if (this.differenceDocPageSelector) {
				if (diffPage === undefined) {
					this.differenceDocPageSelector.deselectCurrentPage(true);
				} else {
					if (this.compareModule.isReaderSpreadActive()) {
						this.differenceDocPageSelector.deselectCurrentPage(true);
						this.activeDocPageSelector.enableDocumentPage(diffPage.getUniqueID());
					} else {
						this.differenceDocPageSelector.setSelectedDocumentPage(diffPage.getUniqueID());
						this.activeDocPageSelector.disableDocumentPage(diffPage.getUniqueID());
					}
				}
			}
		};

		/**
		 * Update the document selectors if the selected active document page is part of a page list document. Make sure to test if the document is
		 * a pagelist before calling this method! Since part is asynchronous, call this method before disabling/changing compare mode as takes
		 * it into account when changing to the correct difference document page. This only happens if compare was already active.
		 * @param {AbstractDocumentPage|undefined} activePage - if not provided, no previous pages will be disabled in the difference document selector
		 * @param {AbstractDocumentPage} newActivePage - the new active document page, used to enable the correct document pages in the difference document selector
		 */
		DocumentSelection.prototype.updateDocumentSelectorsForPageList = function (oldActivePage, newActivePage) {
			if (oldActivePage !== undefined) {
				var pageNumber = oldActivePage.getPagenum();
				var document = oldActivePage.getDocument();
				//run over all previous document versions of the document, to disable all pages that were enabled for the old active document page
				for (var i = 1; i < oldActivePage.getDocumentVersion().getVersionId(); i++) {
					var documentVersion = document.getDocumentVersion(i);
					if (documentVersion !== undefined && documentVersion.hasPage(pageNumber)) { //it is possible that the document version/page was not (yet) loaded
						var pageID = documentVersion.getPage(pageNumber).getUniqueID(); //unique id is used as key for the document selector
						this.differenceDocPageSelector.disableDocumentPage(pageID); //disable the document page 
					}
				}
			}
			//to make sure that the correct document page is set as difference page, if before changing active document page, a compare mode was active
			//this is needed because of asynchronous call to load version of the new active document page
			var wasComparing = false;
			if (this.compareModule) {
				wasComparing = this.compareModule.getActiveMode() !== null && !this.compareModule.isReaderSpreadActive() && !newActivePage.getLinkedComparePage();
			}

			//call to the server to request previous version of the document that has pages with the same page number as the active document page
			newActivePage.getDocument().loadVersionsForDocumentPage(newActivePage, _.bind(function (pages) {
				//execute callback for when the pages have been added to the document controller
				_.each(pages, _.bind(function (page) { //run over each pages to add to the difference document selector
					if (page) {
						if (this.differenceDocPageSelector.hasDocumentPage(page.getUniqueID())) { //first check if the page wasn't already added to the document selector
							this.differenceDocPageSelector.enableDocumentPage(page.getUniqueID()); //enable if that is the case
						} else {
							var docInfo = page.getInformationForDocumentSelector();
							docInfo.prepend = true; //pages need to be prepended to make sure they are added to the beginning of the document selector
							docInfo.docType = DataConstants.documentPageType.PAGELIST; //change docType to PageList to make sure they are not hidden by the document selector
							docInfo.pageCount = null; //set pageCount to null, this is not correct for page list documents as they don't load all pages of a version at the same time
							this.differenceDocPageSelector.addDocumentPage(docInfo);
						}
					}
				}, this));
				//check if the comparing was active before, if so set the difference document page to the same page of the last version
				if (wasComparing) {
					this.differenceDocPageSelector.setSelectedDocumentPage(_.last(pages).getUniqueID()); //_.last gets the page of the last version 
					this.changeDifferencePage(_.last(pages).getUniqueID());
				}
			}, this));
		};

		/**
		 * Load pages of the given document version if these pages are not yet loaded, this to prevent loading at initialization
		 * @private
		 * @param {AbstractDocumentVersion} docVersion - document version to load pages for
		 */
		DocumentSelection.prototype._loadPagesIfNeeded = function (docVersion) {
			if (docVersion.isMultiPage() && docVersion.getPageCount() <= 1) {
				this.f.getDocumentController().loadDocumentPages(docVersion.getDocument().getIID(), docVersion.getVersionId());
			}
		};

		/**
		 * Add a newly created document page in the cad overlay selector and remove an old cad overlay (linked document pages).
		 * It makes the newly added document page as the active (selected) document page.
		 * @param {AbstractDocumentPage} graphicsPage
		 * @param {String} oldCadOverlayId
		 */
		DocumentSelection.prototype.setNewlyCreatedCadOverlay = function (graphicsPage, oldCadOverlayId) {
			this.cadOverLaySelector.removeDocumentPage(oldCadOverlayId);
			var linkedCadPage = graphicsPage.getLinkedPage();
			if (linkedCadPage) {
				var linkedCadDocInfo = linkedCadPage.getInformationForDocumentSelector();
				linkedCadDocInfo.referencedFrom = graphicsPage.getDocument().get("docname"); //make sure the link is displayed in the document selector
				this.cadOverLaySelector.addDocumentPage(linkedCadDocInfo);
				if (this.f.getDocumentController().getCadOverlay()) {
					this.cadOverLaySelector.setSelectedDocumentPage(linkedCadDocInfo.pageId);
				}
			}
		};

		/**
		 * Get the first selectable page id
		 * @returns {String|null}
		 */
		DocumentSelection.prototype.getFirstSelectablePageId = function () {
			return this.activeDocPageSelector.getFirstSelectablePageId();
		};

		/**
		 * Get the last selectable page id
		 * @returns {String|null}
		 */
		DocumentSelection.prototype.getLastSelectablePageId = function () {
			return this.activeDocPageSelector.getLastSelectablePageId();
		};

		/**
		 * Get the next selectable page id, relative to the currently selected page in the active page selector
		 * @returns {String|null}
		 */
		DocumentSelection.prototype.getNextSelectablePageId = function () {
			return this.activeDocPageSelector.getNextItemId();
		};

		/**
		 * Get the previous selectable page id, relative to the currently selected page in the active page selector
		 * @returns {String|null}
		 */
		DocumentSelection.prototype.getPreviousSelectablePageId = function () {
			return this.activeDocPageSelector.getPreviousItemId();
		};

		return DocumentSelection;
	});

/**
 * @namespace CompareModule
 */
define('web-viewer/compare/CompareModule',["web-viewer/core/AbstractModule",
		"web-viewer/compare/CompareModel",
		"web-viewer/compare/CompareShowCurrentMode",
		"web-viewer/compare/CompareShowDifferenceMode",
		"web-viewer/compare/CompareBlinkingMode",
		"web-viewer/compare/CompareDifferenceMode",
		"web-viewer/compare/CompareSideBySideMode",
		"web-viewer/compare/CompareShowBothOnTopMode",
		"web-viewer/compare/alignment/CompareAlignmentWizard",
		"web-viewer/compare/CompareToolbarComponent",
		"web-viewer/compare/CompareTouchToolbarComponent",
		"web-viewer/compare/CompareWidget",
		"web-viewer/compare/CompareConstants",
		"web-viewer/compare/CompareShowReaderSpreadMode",
		"web-viewer/compare/ComparePagelistPopup",
		"web-viewer/layout/component/DocumentSelection",
		"web-viewer/layout/component/ModalPopup",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/utils/ListenerUtil",
		"utils/XMLUtils",
		"utils/ShapeUtils",
		"web-viewer/utils/ViewerUtils",
		"Mousetrap",
		"underscore"
	],
	function (AbstractModule,
		CompareModel,
		CompareShowCurrentMode,
		CompareShowDifferenceMode,
		CompareBlinkingMode,
		CompareDifferenceMode,
		CompareSideBySideMode,
		CompareShowBothOnTopMode,
		CompareAlignmentWizard,
		CompareToolbarComponent,
		CompareTouchToolbarComponent,
		CompareWidget,
		CompareConstants,
		CompareShowReaderSpreadMode,
		ComparePagelistPopup,
		DocumentSelection,
		ModalPopup,
		LayoutConstants,
		ListenerUtil,
		XMLUtils,
		ShapeUtils,
		ViewerUtils,
		Mousetrap,
		_) {
		"use strict";

		/**
		 * Initialize the Compare Module
		 *
		 * @exports CompareModule
		 * @memberof CompareModule
		 * @class
		 * The Compare module enables comparison in the viewer
		 * @constructor
		 * @augments {Core.AbstractModule}
		 * @param {Core.ViewerFacade} f
		 * @param {Boolean} silenced - if true, the module shouldn't publish tools/widgets/toolbarcomponents
		 * @param {String} name - unique name of the module
		 */
		var CompareModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "compare");
			ListenerUtil.mixin(this, f);

			var diffMode = this.f.getOption("PREFERENCES_DIFFMODE_COMPARE");
			this._compareModel = new CompareModel(this.f, this.getPrivateChannelNameAsArray(), diffMode);

			/**
			 * @member {DocumentSelection} Active and Difference document selectors
			 */
			this._documentSelection = new DocumentSelection(this.f, this, _.bind(function () {
				// update widget, if it's already loaded
				if (this._compareWidget) {
					this._compareWidget.updateCompareWidget();
				}
			}, this), this.f.isMobile());

			// REGISTER ALL AVAILABLE MODES
			this._compareModel.registerMode(new CompareShowCurrentMode(this.f, this._compareModel));
			this._compareModel.registerMode(new CompareShowDifferenceMode(this.f, this._compareModel));
			this._compareModel.registerMode(new CompareBlinkingMode(this.f, this._compareModel));
			this._compareModel.registerMode(new CompareShowReaderSpreadMode(this.f, this._compareModel));

			this._differenceMode = new CompareDifferenceMode(this.f, this._compareModel);
			this._compareModel.registerMode(this._differenceMode);

			this._sideBySideMode = new CompareSideBySideMode(this.f, this._compareModel, this._differenceMode);
			this._compareModel.registerMode(this._sideBySideMode);

			this._compareModel.registerMode(new CompareShowBothOnTopMode(this.f, this._compareModel));

			this._toolbarComponent = null;
			this._compareWidget = null;

			this._comparePagelistPopup = null;

			this._compareAlignmentWizard = null;

			this._publishToolbarComponentAndWidgetIfNeeded();

			this._docListener = this._registerListener("doc.loaded", this._publishToolbarComponentAndWidgetIfNeeded, this);
			this._registerListener("intercept.fragmentURL", this._interceptFragmentURL, this);

			this._registerListener("intercept.parameters", this._interceptFragmentURL, this);

			this._settingDefault = false;

			// Listen to the compare model for saving the diffMode in the user settings

			// first time, check if the viewer was opened in compare
			if (this.f.getProperty("compare") === '1' && this._compareModel.isRealCompareMode(diffMode)) {
				this.activateModeByDiffMode(this.f.getOption("PREFERENCES_DIFFMODE_COMPARE"));
				this._settingDefault = true;
			} else {
				this._compareModel.setDiffMode(CompareConstants.diffMode.UNDEFINED);
			}
			// listen to main modes
			this._registerListener("changed.mode", function (oldValue, newValue) {
				var currentDiffMode = this.getCurrentDiffMode();
				if (newValue === null && this.f.getDocumentController().getDifferencePage() !== undefined) {
					this.changeReferenceOffset(new ShapeUtils.Point(0, 0));
				}
				if (this._compareModel.isRealCompareMode(currentDiffMode)) {
					this.f.setOption("PREFERENCES_DIFFMODE_COMPARE", currentDiffMode);
				}
			}, this, this._compareModel);
			// also listen to difference mode sub modes
			this._registerListener("changed.differenceMode", function (oldValue, newValue) {
				var compareMode = newValue;
				//first check if side-by-side is active
				if (this._compareModel.getActiveMode().getName() === CompareConstants.mode.SIDEBYSIDE) {
					//map the difference mode to a side-by-side difference mode
					if (compareMode === CompareConstants.diffMode.SHOWBOTHCHANGESHIGHLIGHT) {
						compareMode = CompareConstants.diffMode.SHOWBOTHSIDEBYSIDEHIGHLIGHT;
					} else if (compareMode === CompareConstants.diffMode.SHOWBOTHCHANGESONLY) {
						compareMode = CompareConstants.diffMode.SHOWBOTHSIDEBYSIDECHANGESONLY;
					} else {
						compareMode = CompareConstants.diffMode.SHOWBOTHSIDEBYSIDE;
					}
				}
				this.f.setOption("PREFERENCES_DIFFMODE_COMPARE", compareMode);
			}, this, this._differenceMode);
			// don't listen to the user options (assuming this is the only place where the option will be changed, so no need to update on such events because they come from this module)	

			// When we were not setting a default reference page (meaning, the user selected the page himself) and when a second document is selected and there was no previous document selected, activate the last saved compare mode
			this.f.getDocumentController().whenDifferencePageChanged(function (oldPage, newPage) {
				if (!this._settingDefault && oldPage === undefined && newPage && !this._compareModel.isReaderSpreadActive()) {
					this.activateModeByDiffMode(this.f.getOption("PREFERENCES_DIFFMODE_COMPARE"));
				}
			}, this);
			this.f.getDocumentController().whenActiveOrDifferencePageChanged(this._whenActiveOrDifferencePageChanged, this);
			this.f.getDocumentController().whenActiveOrDifferencePageChangedAndParametersAvailableOrChange(this._whenActiveOrDifferencePageChangedAndParametersAvailableOrChange, this);


			// On changes in the document selector
			this._registerListener("changeReferencePage", _.bind(function (newDiffPage) {
				if (this.f.getActivePage().getDocumentVersion().isMultiPage()) {
					// only possible when difference selector is set, so comparePagelistPopup is available then
					this._comparePagelistPopup.setComparePage(newDiffPage);
					this._comparePagelistPopup.show();
				} else {
					this.activateModeByDiffMode(this.f.getOption("PREFERENCES_DIFFMODE_COMPARE"), newDiffPage);
				}
			}, this), this, this._documentSelection);
			this._registerListener("changeActivePage", _.bind(function (newActivePage, callback) {
				//first check if the new active page is a multipage and the current active mode is reader spread
				if (this._compareModel.isReaderSpreadActive() && !newActivePage.getDocumentVersion().isMultiPage()) {
					this.activateModeByName();
				}
				var isActivePageChanged = this.f.getDocumentController().changeActivePage(newActivePage);
				if (callback) {
					callback(isActivePageChanged);
				}
			}, this), this, this._documentSelection);

			// set the CAD overlay when a new cad overlay was selected in the document selector
			this._registerListener("changeCadOverlay", _.bind(function (newCadOverlay) {
				this.f.getDocumentController().changeCadOverlay(newCadOverlay);
			}, this), this, this._documentSelection);


			this._registerListener("intercept.activePageChange", this._interceptActivePageChange, this);

			this.f.getDocumentController().whenDifferencePageChanged(function (oldPage, newPage) {
				var navigationModule = this.f.getModule("navigation");
				if (navigationModule) {
					navigationModule.resetExtraRotation();
				}
				if (this._compareModel.isRealCompareMode(this._compareModel.getDiffMode())) {
					this._previousDifferencePage = newPage;
				}
			}, this);

			this.f.getDocumentController().whenCadOverlayChanged(function (oldPage, newPage) {
				if (newPage) { //make sure the same data of the compare model is set to the CAD overlay, we keep the same alignment as before
					this.changeCadOverlayOffset(this._compareModel.getCadOverlayOffset());
					this.flipCadOverlay(this._compareModel.isCadOverlayFlipped());
					this.setCadOverlayRotation(this._compareModel.getCadOverlayRotation());
				}
			}, this);

			this.f.getDocumentController().whenActivePageChanged(function (oldPage, newPage) {
				var navigationModule = this.f.getModule("navigation");
				if (navigationModule) {
					navigationModule.resetExtraRotation();
				}

				this._compareModel.resetCadOverlayOffset(); //first reset the alignment data
				this._compareModel.resetCadOverlayOrientation();
				if (newPage && newPage.hasLinkedPage()) { //if there is a linked page, go set the correct alignment data
					var cadOverlayPage = newPage.getLinkedPage();

					if (cadOverlayPage) {
						var overlayOffset = new ShapeUtils.Point();
						var overlayIsFlipped = false;
						var overlayRotation = 0;

						if (this.f.getDocumentController().getActivePage().hasLinkedPage() && this.f.getDocumentController().getActivePage().getLinkedPage() === cadOverlayPage) {
							overlayOffset = new ShapeUtils.Point(ViewerUtils.parseFloat(cadOverlayPage.get("offsetx")), ViewerUtils.parseFloat(cadOverlayPage.get("offsety")));
							overlayIsFlipped = cadOverlayPage.get("overlayIsFlipped");
							overlayRotation = cadOverlayPage.get("overlayRotation");
						}

						this.changeCadOverlayOffset(overlayOffset);
						this.flipCadOverlay(overlayIsFlipped);
						this.setCadOverlayRotation(overlayRotation);
					}
				}

				if (this._3DWidget) {
					this._3DWidget.setEnabled(false);
				}
			}, this);

			this._registerListener("newLinkedCad", function (oldCadOverlay, newCadOverlay) {
				if (oldCadOverlay !== newCadOverlay) {
					var oldCadId = null;
					if (oldCadOverlay) {
						oldCadId = oldCadOverlay.getUniqueID();
					}
					this._documentSelection.setNewlyCreatedCadOverlay(this.f.getActivePage(), oldCadId);
				}
			}, this, this.f.getDocumentController());

			// Keyboard shortcuts
			Mousetrap.bind("ctrl+l", _.bind(function (e) {
				var diffPage = this.f.getDocumentController().getDifferencePage();
				if (diffPage === undefined || this._compareModel.isReaderSpreadActive() || diffPage.hasCad()) {
					return;
				}

				this.doAutoAlign();

				e.preventDefault();
				e.stopPropagation();
			}, this), "keydown");

			Mousetrap.bind("home", _.bind(function (e) {
				var pageId = this._documentSelection.getFirstSelectablePageId();
				if (pageId) {
					var newActivePage = this.f.getDocumentController().getDocumentPage(pageId);
					if (newActivePage) {
						this.f.getDocumentController().changeActivePage(newActivePage);
					}
				}
				e.preventDefault();
				e.stopPropagation();
			}, this));

			Mousetrap.bind("end", _.bind(function (e) {
				var pageId = this._documentSelection.getLastSelectablePageId();
				if (pageId) {
					var newActivePage = this.f.getDocumentController().getDocumentPage(pageId);
					if (newActivePage) {
						this.f.getDocumentController().changeActivePage(newActivePage);
					}
				}
				e.preventDefault();
				e.stopPropagation();
			}, this));

			Mousetrap.bind("pagedown", _.bind(function (e) {
				var pageId = this._documentSelection.getNextSelectablePageId();
				if (pageId) {
					var newActivePage = this.f.getDocumentController().getDocumentPage(pageId);
					if (newActivePage) {
						this.f.getDocumentController().changeActivePage(newActivePage);
					}
				}
				e.preventDefault();
				e.stopPropagation();
			}, this));

			Mousetrap.bind("pageup", _.bind(function (e) {
				var pageId = this._documentSelection.getPreviousSelectablePageId();
				if (pageId) {
					var newActivePage = this.f.getDocumentController().getDocumentPage(pageId);
					if (newActivePage) {
						this.f.getDocumentController().changeActivePage(newActivePage);
					}
				}
				e.preventDefault();
				e.stopPropagation();
			}, this));

			Mousetrap.bind("ctrl+s", _.bind(function (e) {
				var currentDiffMode = this.getCurrentDiffMode();
				if (currentDiffMode === CompareConstants.diffMode.UNDEFINED || this.isReaderSpreadActive()) {
					return;
				}

				if (currentDiffMode === CompareConstants.diffMode.CURRENTVERSION) {
					// go to diff version
					this.activateModeByName(CompareConstants.mode.SHOW_DIFFERENCE);
				} else {
					// go to current version
					this.activateModeByName(CompareConstants.mode.SHOW_CURRENT);
				}

				e.preventDefault();
				e.stopPropagation();
			}, this), "keydown");
		};

		CompareModule.prototype = Object.create(AbstractModule.prototype);
		CompareModule.prototype.constructor = CompareModule;


		/**
		 * Get the current active diff mode in saveable format (meaning, blinking returns BLINKING_CURRENT instead of CURRENT/REFERENCE)
		 * @returns {CompareConstants.diffMode} The current active diff mode
		 */
		CompareModule.prototype.getCurrentDiffMode = function () {
			var compareMode = this._compareModel.getActiveMode();
			var diffMode = null;
			// blinking switches between current/reference but we should save BLINKING (otherwise we cannot restore blinking)
			if (compareMode !== null && compareMode.getName() === CompareConstants.mode.BLINKING) {
				diffMode = CompareConstants.diffMode.BLINKING_CURRENT;
			} else {
				diffMode = this._compareModel.getDiffMode();
			}
			return diffMode;
		};

		/**
		 * This method will activate the last used compare mode with the last used difference file
		 */
		CompareModule.prototype.activateLastUsedCompareMode = function () {
			var lastUsedDiffMode = this.f.getOption("PREFERENCES_DIFFMODE_COMPARE");
			if (this._compareModel.isRealCompareMode(lastUsedDiffMode)) {
				if (this._previousDifferencePage === this.f.getActivePage()) {
					delete this._previousDifferencePage;
				}

				this.activateModeByDiffMode(this.f.getOption("PREFERENCES_DIFFMODE_COMPARE"), this._previousDifferencePage);
				if (this._previousState) {
					this.setState(this._previousState);
					delete this._previousState;
				}
			}
		};

		/**
		 * Change the current mode by mode name
		 * @param {String|CompareModule.CompareConstants.mode} [name] - the name of the new mode, if undefined/null/falsy the current mode is unloaded
		 * @param {CompareModule.CompareConstants.diffMode} [diffMode] - Diff mode to set in the mode specified by name
		 * @param {AbstractDocumentPage} [diffPage] - Diff page to set if none is active
		 */
		CompareModule.prototype.activateModeByName = function (name, diffMode, diffPage) {
			if (name === undefined) {
				if (this._compareModel.isRealCompareMode(this._compareModel.getDiffMode())) {
					this._previousState = this.getState();
				} else {
					delete this._previousState;
				}

				// we deactivate the current mode
				this._compareModel.deactivateCurrentMode();
				this.f.getDocumentController().resetDifferencePage();
			} else {

				var activeMode = this._compareModel.getActiveMode();
				if (!activeMode || activeMode.getName() !== name) {

					this._compareModel.toggleSilenceParametersChanged(true);
					this._compareModel.toggleCurrentMode(false);

					if (name === CompareConstants.mode.READERSPREAD || name === CompareConstants.mode.ALIGNMENT) {
						// we go to readerspread; don't set diff page, reader spread will do it first
						// or we go to alignment mode, this doesn't need difference page when in cad overlay
						this._compareModel.activateModeByName(name, diffMode);

						diffPage = this.f.getDocumentController().getDifferencePage();
						if (diffPage && !diffPage.isBlankPage()) {
							diffPage.whenParametersAvailable(_.bind(function () {
								this._compareModel.toggleSilenceParametersChanged(false);
								if (this._compareWidget) {
									this._compareWidget.updateCompareWidget();
								}
							}, this));
						} else {
							this._compareModel.toggleSilenceParametersChanged(false);
						}
					} else {
						// to another mode
						// Make sure the diff page is filled in
						this.f.getDocumentController().whenDocumentsLoaded(function () {
							if (!diffPage) {
								diffPage = this.f.getDocumentController().getDifferencePage();
							}
							if (!diffPage) {
								// we don't know which one to set, so do default logic
								diffPage = this.getDefaultDiffPage();
							}

							// disable restore of last saved mode, see module constructor
							this._settingDefault = true;
							this.f.getDocumentController().changeDifferencePage(diffPage);
							this._settingDefault = false;

							diffPage = this.f.getDocumentController().getDifferencePage();
							if (diffPage) {
								this._compareModel.activateModeByName(name, diffMode);
								diffPage.whenParametersAvailable(_.bind(function () {

									this._compareModel.toggleSilenceParametersChanged(false);
									if (this._compareWidget) {
										this._compareWidget.updateCompareWidget();
									}
								}, this));
							} else {
								this._compareModel.toggleSilenceParametersChanged(false);
							}
						}, this);
					}
				} else {
					// the mode should not be change, but maybe we should update the diff page
					if (diffPage) {
						// disable restore of last saved mode, see module constructor
						this._settingDefault = true;
						this.f.getDocumentController().changeDifferencePage(diffPage);
						this._settingDefault = false;
					}
				}
			}
		};

		/**
		 * Change the current mode by diffMode
		 * @param {CompareModule.CompareConstants.diffMode} diffMode - The diff mode
		 * @param {AbstractDocumentPage} [diffPage] - Diff page that should be taken if none is active
		 */
		CompareModule.prototype.activateModeByDiffMode = function (diffMode, diffPage) {
			if (diffMode === CompareConstants.diffMode.UNDEFINED) {
				this.activateModeByName();
			} else {
				var modeName = this._compareModel.getModeNameByDiffMode(diffMode);
				this.activateModeByName(modeName, diffMode, diffPage);
			}
		};

		/**
		 * Get the default diff doc page for the current/given active document page
		 * Make sure all documents are loaded before calling this function!
		 * @param {AbstractDocumentPage} [activePage] - If defined, use this page to calculate the diff page; otherwise use the current active page
		 * @return {AbstractDocumentPage} The default diff doc page for the current/given active document page or undefined if nothing useful can be found
		 */
		CompareModule.prototype.getDefaultDiffPage = function (activePage) {
			if (!activePage) {
				activePage = this.f.getActivePage();
			}

			// First see if there is a linked compare page
			var linkedComparePage = activePage.getLinkedComparePage();
			if (linkedComparePage) {
				return linkedComparePage;
			} else {
				var diffVersion = this._getNextDiffDocumentVersion(activePage);

				if (diffVersion) {
					// for multi page only, get corresponding page
					if ((activePage.getDocumentVersion().isMultiPage() && diffVersion.isMultiPage()) ||
						(activePage.getDocumentVersion().isPageList() && diffVersion.isPageList())) {
						var correspondingDiffPage = diffVersion.getPage(activePage.getPagenum());
						if (correspondingDiffPage && !correspondingDiffPage.isBlankPage()) {
							if (correspondingDiffPage === activePage) {
								// same one as the active page, so search another one
								diffVersion = this._getNextDiffDocumentVersion(activePage, diffVersion);

								if (diffVersion) {
									correspondingDiffPage = diffVersion.getPage(activePage.getPagenum());
									if (correspondingDiffPage) {
										return correspondingDiffPage;
									} else {
										// if the active document has more pages than the diff, show the last
										// other case is a multipage active document with a normal diff document -> last = first in that case.
										return diffVersion.getLastPage();
									}
								} else { //can happen when pagelist has only 1 version
									// no other document found -> should not happen
									return undefined;
								}
							} else {
								return correspondingDiffPage;
							}
						} else {
							if (diffVersion.isPageList() && (correspondingDiffPage === undefined || correspondingDiffPage.isBlankPage())) { //check if blank or empty page
								var pageNum = activePage.getPagenum();
								pageNum++;
								//as long the page is empty or blank try the next page
								while (pageNum < diffVersion.getPageCount() && (!diffVersion.hasPage(pageNum) || diffVersion.getPage(pageNum).isBlankPage())) {
									pageNum++;
								}
								if (pageNum < diffVersion.getPageCount() && diffVersion.getPage(pageNum) !== activePage) { //there is a next page that's not empty or blank
									return diffVersion.getPage(pageNum);
								} else { //no next page is found, select the first non blank page for this document version
									return diffVersion.getFirstNotBlankPage();
								}
							}
							// if the active document has more pages than the diff, show the last
							// other case is a multipage active document with a normal diff document -> last = first in that case.
							return diffVersion.getLastPage();
						}
					} else {
						return diffVersion.getFirstPage();
					}
				} else {
					// no other document found, check if there are other pages
					if (activePage.getDocumentVersion().isMultiPage()) {
						// return the first if it's different
						var firstPage = activePage.getDocumentVersion().getFirstPage();
						if (activePage === firstPage) {
							var nextDiffPage = firstPage.getNextPage();
							while (nextDiffPage.isBlankPage() && nextDiffPage.getPagenum() < nextDiffPage.getDocumentVersion().getPageCount()) {
								nextDiffPage = nextDiffPage.getNextPage();
							}
							return nextDiffPage;
						} else {
							return firstPage;
						}
					}
					return undefined;
				}
			}
		};

		/**
		 * Get the next diff document version based on the current active document page.
		 * First another document is tried (latest version), then previous version.
		 * Make sure all documents are loaded before calling this function!
		 * @param {AbstractDocumentPage} activePage - Use this page to calculate the diff page
		 * @param {AbstractDocumentVersion} [excludeVersion] - Version to skip in the search of a new diff version (next to active document page)
		 * @return {AbstractDocumentVersion} Next diff document version or undefined if none found.
		 */
		CompareModule.prototype._getNextDiffDocumentVersion = function (activePage, excludeVersion) {
			var activeDoc = activePage.getDocument();

			var newDoc = _.find(this.f.getDocumentController().getAllDocuments(), function (document) {
				// if exclude version is defined, make sure we exclude it
				return (!excludeVersion || (excludeVersion && document !== excludeVersion.getDocument())) && document !== activeDoc && document.getDocumentType() === activeDoc.getDocumentType();
			});

			var diffVersion;

			if (newDoc !== undefined) {
				// Get the latest version of that document
				diffVersion = newDoc.getDocumentVersion();
			} else {
				// There is no other document, but maybe other versions
				var activeDocVersion = activePage.getDocumentVersion();
				// if multiple versions
				if (activeDoc.hasMultipleVersions() && activeDocVersion.getVersionId() > 1) {
					// previous version
					diffVersion = activeDoc.getDocumentVersion(activeDocVersion.getVersionId() - 1);
				}
			}

			var oldDiffVersion = diffVersion;
			//if it's a page list make sure only the same pages of previous version are used to compare to, since only these can be picked in dropdown menu
			if (activePage.getDocumentVersion().isPageList() && diffVersion !== undefined && !diffVersion.hasPage(activePage.getPagenum())) {
				while (diffVersion !== undefined && diffVersion.getVersionId() > 0 && !diffVersion.hasPage(activePage.getPagenum())) {
					diffVersion = activeDoc.getDocumentVersion(diffVersion.getVersionId() - 1);
				}
				//if the page has no previous versions, set the current version as diffVersion
				if (diffVersion !== undefined && !diffVersion.hasPage(activePage.getPagenum())) {
					diffVersion = activePage.getDocumentVersion();
				}
			}

			if (excludeVersion && diffVersion === excludeVersion) {
				// we exclude it if needed
				if (oldDiffVersion && oldDiffVersion === excludeVersion) {
					return undefined;
				} else {
					return oldDiffVersion;
				}
			} else {
				return diffVersion;
			}
		};

		/**
		 * @override
		 */
		CompareModule.prototype._interceptFragmentURL = function (c) {
			if (this._compareModel.getActiveMode()) {

				// Set the general settings
				var offset = this._compareModel.getOffset();
				if (offset !== null) {
					c.params.dox = offset.x;
					c.params.doy = offset.y;
				}
				var corner = this._compareModel.getCornerAlign();
				if (corner !== null) {
					c.params.dalign = corner;
				}
				if (this._compareModel.isReferenceNegated()) {
					c.params.dn = 1;
				}
				var rotation = this._compareModel.getReferenceRotation() / 90;
				if (this._compareModel.isReferenceFlipped()) {
					rotation += 4;
				}
				if (rotation !== 0) {
					c.params.dor = rotation;
				}
				if (this._compareModel.isTrimBoxAlign()) {
					c.params.daligntrim = 1;
				}

				c.params.dmode = this._compareModel.getDiffMode();

				// Do mode specific things
				this._compareModel.getActiveMode().interceptFragmentURL(c);
			}
		};

		/**
		 * Get the comparemodel
		 * @returns {CompareModule.CompareModel}
		 */
		CompareModule.prototype.getCompareModel = function () {
			return this._compareModel;
		};

		/**
		 * Get the diffMode from the model
		 * @returns {CompareConstants.diffMode}
		 */
		CompareModule.prototype.getDiffMode = function () {
			return this._compareModel.getDiffMode();
		};

		/**
		 * Get the active Compare mode
		 * @returns {CompareModule.CompareMode} the active mode (or null)
		 */
		CompareModule.prototype.getActiveMode = function () {
			return this._compareModel.getActiveMode();
		};

		/**
		 * Get the offset of the reference page for compare
		 * @returns {ShapeUtils.Point}
		 */
		CompareModule.prototype.getReferenceOffset = function () {
			return this._compareModel.getOffset();
		};

		/**
		 * Is it possible to use the compare functionallity in this session (based on which documents are loaded)
		 * @returns {Boolean}
		 */
		CompareModule.prototype.isCompareAvailable = function () {
			if (!this._compareWidget && !this._toolbarComponent) {
				var show = this._shouldShowWidgetOrTools();

				return show.widget || show.tools;
			} else {
				return true;
			}
		};

		/**
		 * Check if CAD overlay is allowed (graphical document is activated, and there is at least 1 other CAD file loaded (or graphics document with linked page))
		 * @returns {Boolean}
		 */
		CompareModule.prototype.isCadOverlayAllowed = function () {
			var activePage = this.f.getActivePage();


			if (!activePage.hasCad()) {
				if (activePage.canSaveCadOverlay() || activePage.hasLinkedPage()) {
					return true;
				}
				var activeDocument = this.f.getDocumentController().getActiveDocument();
				return _.some(this.f.getDocumentController().getDocuments(), function (document) {
					if (document != activeDocument) {
						var version = document.getDocumentVersion();
						if (version) {
							var page = version.getPage();
							if (page) {
								return page.hasCad() || page.hasLinkedPage();
							}
						}
					}
					return false;
				});
			}

			return false;
		};

		/**
		 * Check if the CAD overlay settings can be saved for the currently active page
		 * @returns {Boolean}
		 */
		CompareModule.prototype.canSaveCadOverlay = function () {
			var activePage = this.f.getActivePage();
			return activePage.canSaveCadOverlay();
		};

		/**
		 * Is the CAD overlay changed since the current page was loaded
		 * @returns {Boolean}
		 */
		CompareModule.prototype.isCadOverlayModified = function () {
			var activePage = this.f.getActivePage();
			var linkedCad = activePage.getLinkedPage();
			var cadOverlay = this.f.getDocumentController().getCadOverlay();
			return linkedCad != cadOverlay ||
				(linkedCad &&
					((linkedCad.getStateParam("compareOffset") !== undefined && ViewerUtils.parseFloat(linkedCad.get("offsetx")) !== linkedCad.getStateParam("compareOffset").x) ||
						(linkedCad.getStateParam("compareOffset") !== undefined && ViewerUtils.parseFloat(linkedCad.get("offsety")) !== linkedCad.getStateParam("compareOffset").y) ||
						(linkedCad.getStateParam("rotation") !== undefined && ViewerUtils.parseFloat(linkedCad.get("overlayRotation")) !== linkedCad.getStateParam("rotation")) ||
						(linkedCad.getStateParam("flipped") !== undefined && linkedCad.get("overlayIsFlipped") !== linkedCad.getStateParam("flipped"))));
		};


		/**
		 * Save the CAD overlay for the current active page
		 * @param {function} [callback] - called when the save either fails or is successful. The argument is an boolean indicating success
		 */
		CompareModule.prototype.saveCadOverlay = function (callback) {
			var graphicsPage = this.f.getActivePage();
			var context = {
				saveCadOverlay: true,
				callback: _.partial(_.bind(this.saveCadOverlay, this), callback)
			};

			this.f.emit("intercept.savingCadOverlay", context);

			if (context.saveCadOverlay) {
				this.f.getDataAdapter().saveCadOverlay(graphicsPage,
					this.f.getDocumentController().getCadOverlay(),
					this._compareModel.getCadOverlayRotation(),
					this._compareModel.isCadOverlayFlipped(),
					this._compareModel.getCadOverlayOffset(),
					_.bind(function (success) {

						if (success) {
							graphicsPage.set("isCadAlignmentSaved", true);
						}

						// update the compare widget
						if (this._compareWidget) {
							this._compareWidget.updateCompareWidget();
						}

						// execute callback
						if (callback) {
							callback(success);
						}
					}, this));
			}

		};

		/**
		 * Should the widget or compare tools be shown?
		 * @returns {Object} Object with a boolean for widget and tools (whether they should be shown)
		 */
		CompareModule.prototype._shouldShowWidgetOrTools = function () {
			// it is posisble that we only show the widget and not the compare tool
			var showWidget = false;
			var showCompareTools = false;
			if (this.f.getDocumentController().getDocumentCount() > 2) {
				// there's more than 2 documents, show everything!
				showWidget = true;
				showCompareTools = true;
			} else {
				// there are either 2 documents or 1 document
				var documents = this.f.getDocumentController().getAllDocuments();
				if (_.some(documents, function (doc) {
						return doc.hasMultipleVersions();
					})) {
					// at least 1 document has more than 1 version, so show all controls
					showWidget = true;
					showCompareTools = true;
				} else {
					if (documents.length == 2) {
						showWidget = true;

						// the compare tools are available if two graphics files or two cad files are opened. If a cad and graphics file is opened, compare tools should not be available
						var doc0hasCad = documents[0].getDocumentVersion().getPage().hasCad();
						var doc1hasCad = documents[1].getDocumentVersion().getPage().hasCad();
						if (doc0hasCad === doc1hasCad) {
							// if both documents are of the same type, we can use compare tools :)
							showCompareTools = true;
						}

					} else {
						// there is 1 document with 1 version
						if (documents[0].getDocumentVersion().getPageCount() > 1) {
							// there are multiple pages
							showWidget = true;
							showCompareTools = true;
						}
						if (this.f.getActivePage().hasLinkedPage()) {
							showWidget = true;
							showCompareTools = false;
						}

					}
				}

				if (!showWidget) {
					//maybe we still need to show the widget when the user can save CAD Overlay Alignment
					if (_.some(documents, function (document) {
							return _.some(document.getDocumentVersion().getAllPages(), function (page) {
								return page.canSaveCadOverlay();
							});
						})) {
						showWidget = true;
					}
				}
			}
			return {
				widget: showWidget,
				tools: showCompareTools
			};
		};


		/**
		 * Publish the toolbarcomponent if available
		 */
		CompareModule.prototype._publishToolbarComponentAndWidgetIfNeeded = function () {
			if (this._toolbarComponent === null || this._compareWidget === null) { //should be both either null or initialized

				var show = this._shouldShowWidgetOrTools();

				if (show.tools) {
					// publish the compare buttons, make the buttons work!
					this._publishToolbarComponent(CompareToolbarComponent.toolbarName, _.bind(function (layoutType) {
						if (layoutType === LayoutConstants.LayoutType.MOBILE) {
							this._toolbarComponent = new CompareTouchToolbarComponent(this.f, this);
						} else {
							this._toolbarComponent = new CompareToolbarComponent(this.f, this);
						}
						return this._toolbarComponent;
					}, this));
				}

				if (show.widget) {
					// publish the compare widget
					this._publishWidget(CompareWidget.widgetName, _.bind(function (layoutType) {
						this._compareWidget = new CompareWidget(this.f, this, this._compareModel, this._documentSelection);

						this._comparePagelistPopup = new ComparePagelistPopup(this.f, "comparePagelistPopup");
						this._comparePagelistPopup.setOkAction(_.partial(function (that) {
							// that = module, this = popup
							switch (this._currentPagelistCompareRange) {
							case CompareConstants.pagelistCompareRange.CURRENT:
								that._doCurrentCompareRange(this._pagelistComparePage);
								break;
							case CompareConstants.pagelistCompareRange.CURRENTANDFOLLOWING:
								that._doCurrentAndFollowingCompareRange(this._pagelistComparePage);
								break;
							case CompareConstants.pagelistCompareRange.ALL:
								that._doAllCompareRange(this._pagelistComparePage);
								break;
							}

							that.activateModeByDiffMode(that.f.getOption("PREFERENCES_DIFFMODE_COMPARE"), this._pagelistComparePage);
							this.hide();
						}, this));

						this._comparePagelistPopup.setCancelAction(_.bind(function () {
							this._documentSelection.setDifferenceSelectorPage(this.f.getDocumentController().getDifferencePage());
						}, this));

						//listen to the widget for auto align action
						this._registerListener("doAutoAlign", this.doAutoAlign, this, this._compareWidget);
						//listen to the widget for corner align
						this._registerListener("doCornerAlign", this._doCornerAlign, this, this._compareWidget);
						//listen to the widget for rotate, flip and negate reference document page
						this._registerListener("rotateReference", this.rotateReference, this, this._compareWidget);
						this._registerListener("flipReference", this.flipReference, this, this._compareWidget);
						this._registerListener("negateReference", _.bind(function () {
							this._compareModel.negateReference();
						}, this), this, this._compareWidget);
						//listen to set the trim box align on or off
						this._registerListener("trimBoxAlign", this._doTrimBoxAlign, this, this._compareWidget);
						//listen to when the offset is changed manually 
						this._registerListener("offsetXChanged", _.bind(function (offsetX) {
							var offset = this._compareModel.getOffset();
							offset.x = offsetX;
							this.changeReferenceOffset(offset);
						}, this), this, this._compareWidget);
						this._registerListener("offsetYChanged", _.bind(function (offsetY) {
							var offset = this._compareModel.getOffset();
							offset.y = offsetY;
							this.changeReferenceOffset(offset);
						}, this), this, this._compareWidget);
						//listen to when the cad overlay offset is changed manually
						this._registerListener("cadOverlayOffsetXChanged", _.bind(function (offsetX) {
							var offset = this._compareModel.getCadOverlayOffset();
							offset.x = offsetX;
							this.changeCadOverlayOffset(offset);
						}, this), this, this._compareWidget);
						this._registerListener("cadOverlayOffsetYChanged", _.bind(function (offsetY) {
							var offset = this._compareModel.getCadOverlayOffset();
							offset.y = offsetY;
							this.changeCadOverlayOffset(offset);
						}, this), this, this._compareWidget);
						this._registerListener("rotateCadOverlay", this.rotateCadOverlay, this, this._compareWidget);
						this._registerListener("flipCadOverlay", this.flipCadOverlay, this, this._compareWidget);
						this._registerListener("saveCadOverlay", this.saveCadOverlay, this, this._compareWidget);

						if (this._docListener) {
							this._unregisterListener(this._docListener);
							this._docListener = null;
						}

						this._compareWidget.updateCompareWidget();
						return this._compareWidget;
					}, this));

				}

				if (show.widget || show.tools) {
					this._emit("compareAvailable");
				}
			}
		};


		/**
		 * Executed when active or difference page has changed and before the parameters are fetched...
		 */
		CompareModule.prototype._whenActiveOrDifferencePageChanged = function () {
			this._compareModel.resetOffset();

			// reset corner align
			this._compareModel.resetCornerAlign();
			this._compareModel.resetReferenceOrientation();
			this._compareModel.resetTrimBoxAlign();


			if (this._toolbarComponent !== null) {
				if (this.f.getActivePage().isBlankPage()) {
					this._toolbarComponent.hide();
					if (this._compareWidget) {
						this._compareWidget.toggleDifferenceDocPageSelector(true);
					}
				} else {
					this._toolbarComponent.show();
					if (this._compareWidget) {
						this._compareWidget.toggleDifferenceDocPageSelector(false);
					}
				}
			}

			if (this._compareWidget) {
				this._compareWidget.updateCompareWidget();
			}
		};


		/**
		 * Executed when the active or difference page changed and the parameters are available
		 * @private
		 */
		CompareModule.prototype._whenActiveOrDifferencePageChangedAndParametersAvailableOrChange = function (page, referencePage, oldPage, isActivePageChanged) {
			if (this._compareWidget) {
				this._compareWidget.updateCompareWidget();
			}
		};

		/**
		 * align reference document page to a corner (or center) of the current document page
		 * @private
		 * @param {CompareConstants.cornerAutoAlign} corner - or null to reset
		 */
		CompareModule.prototype._doCornerAlign = function (corner) {
			var docController = this.f.getDocumentController();
			docController.willNeedParameterUpdate("CORNER_ALIGN");

			this._compareModel.resetOffset();
			if (this._compareWidget) {
				this._compareWidget.updateCompareWidget();
			}
			this._compareModel.setCornerAlign(corner);
			this.changeReferenceOffset(this._compareModel.getOffset());
			docController.readyForParameterUpdate("CORNER_ALIGN");
		};

		/**
		 * align the current document page and the reference document page on their trimboxes
		 */
		CompareModule.prototype._doTrimBoxAlign = function () {
			var docController = this.f.getDocumentController();
			docController.willNeedParameterUpdate("TRIMBOX_ALIGN");

			this._compareModel.toggleTrimBoxAlign();

			docController.readyForParameterUpdate("TRIMBOX_ALIGN");
		};

		/**
		 * auto align the two document page
		 */
		CompareModule.prototype.doAutoAlign = function () {
			var page = this.f.getActivePage();

			var viewport = this.f.getModule("viewport");
			var visibleRect = viewport.getVisibleRect();
			var context = {
				rectangle: visibleRect,
				// the servlet that should be used to fetch the image information
				servlet: "",
				// the parameters used for requesting the image
				params: {
					x1: 0,
					x2: 0,
					y1: 0,
					y2: 0,
					daligntrim: this._compareModel.isTrimBoxAlign() ? 1 : 0,
					opmask: 3
				}
			};

			// ask all interceptors to update the context. All listeners have access to the context object and can adapt the information that is in the object
			// there is no predefined order in which these interceptors are called, their actions thus should not overlap!
			this.f.emit("intercept.fragmentURL", context);

			var url = page.getInfoURL(context.params);
			this.f.ajax(url, {
				success: _.bind(function (data) {
					var alignInfo = XMLUtils.xml2obj(data);
					if (alignInfo.status.info !== undefined && alignInfo.status.info.autoalign !== undefined) {
						var currentOffset = this._compareModel.getOffset();
						var offset = new ShapeUtils.Size(ViewerUtils.parseFloat(alignInfo.status.info.autoalign.dx), ViewerUtils.parseFloat(alignInfo.status.info.autoalign.dy));
						currentOffset.offset(offset);
						this.changeReferenceOffset(currentOffset);
					}
				}, this),
				error: _.bind(function (jqXHR, textStatus, errorThrown) {

					this.f.i18n.loadBundle("viewer", function () {
						this.f.emit("error.SEVERE", this.f.i18n.t("AlignChangeError") + " " + textStatus, jqXHR, textStatus, errorThrown);
					}, this);
				}, this)
			});
		};

		/**
		 *
		 */
		CompareModule.prototype._getCornerAlignOffset = function () {
			var corner = this._compareModel.getCornerAlign();
			var cornerOffset = new ShapeUtils.Size();

			var page = this.f.getActivePage();
			var reference = this.f.getDocumentController().getDifferencePage();

			//get the size of the reference document page
			var diffSize = reference.getPageSize();

			//get the size of the current document page
			var pageSize = page.getPageSize();

			if (corner === CompareConstants.cornerAutoAlign.TOPLEFT || corner === CompareConstants.cornerAutoAlign.CENTERLEFT || corner === CompareConstants.cornerAutoAlign.BOTTOMLEFT) {
				//No offset for x
			} else if (corner === CompareConstants.cornerAutoAlign.TOPCENTER || corner === CompareConstants.cornerAutoAlign.CENTERED || corner === CompareConstants.cornerAutoAlign.BOTTOMCENTER) {
				cornerOffset.width = pageSize.width / 2 - diffSize.width / 2;
			} else if (corner === CompareConstants.cornerAutoAlign.TOPRIGHT || corner === CompareConstants.cornerAutoAlign.CENTERRIGHT || corner === CompareConstants.cornerAutoAlign.BOTTOMRIGHT) {
				cornerOffset.width = pageSize.width - diffSize.width;
			}

			if (corner === CompareConstants.cornerAutoAlign.TOPLEFT || corner === CompareConstants.cornerAutoAlign.TOPCENTER || corner === CompareConstants.cornerAutoAlign.TOPRIGHT) {
				//No offset for y
			} else if (corner === CompareConstants.cornerAutoAlign.CENTERLEFT || corner === CompareConstants.cornerAutoAlign.CENTERED || corner === CompareConstants.cornerAutoAlign.CENTERRIGHT) {
				cornerOffset.height = pageSize.height / 2 - diffSize.height / 2;
			} else if (corner === CompareConstants.cornerAutoAlign.BOTTOMLEFT || corner === CompareConstants.cornerAutoAlign.BOTTOMCENTER || corner === CompareConstants.cornerAutoAlign.BOTTOMRIGHT) {
				cornerOffset.height = pageSize.height - diffSize.height;
			}

			return cornerOffset;
		};

		/**
		 * Change the offset of the current and reference document page, so both document pages are visible
		 * @param {ShapeUtils.Point} offset
		 */
		CompareModule.prototype.changeReferenceOffset = function (referenceOffset) {
			this.f.getDocumentController().willNeedParameterUpdate("COMPARE_OFFSET");
			this._emit("referenceOffset.beforeChange");

			this._compareModel.setOffset(referenceOffset);

			if (this._compareWidget !== null) {
				this._compareWidget.updateCompareWidget();
			}

			var diff = this.f.getDocumentController().getDifferencePage();
			diff.setStateParam("compareOffset", referenceOffset);

			this.f.getDocumentController().readyForParameterUpdate("COMPARE_OFFSET");
			this.f.getDocumentController().whenParametersAvailable(function () {
				this._emit("referenceOffset.changed");
			}, this);
		};

		/**
		 * Change the offset between the CAD overlay and the active document page (Graphics), it's the offset between the CAD overlay and the Graphics top-left corner(if in CAD is upright positioned)
		 * @param {ShapeUtils.Point} cadOverlayerOffset
		 */
		CompareModule.prototype.changeCadOverlayOffset = function (cadOverlayerOffset) {
			this._compareModel.setCadOverlayOffset(cadOverlayerOffset);

			if (this._compareWidget !== null) {
				this._compareWidget.updateCompareWidget();
			}

			var cadOverlay = this.f.getDocumentController().getCadOverlay();
			if (cadOverlay) {
				this.f.getDocumentController().willNeedParameterUpdate("CAD_OVERLAY_OFFSET");
				this._emit("cadOverlayOffset.beforeChange");

				cadOverlay.setStateParam("compareOffset", cadOverlayerOffset);

				this.f.getDocumentController().readyForParameterUpdate("CAD_OVERLAY_OFFSET");
				this.f.getDocumentController().whenParametersAvailable(function () {
					this._emit("cadOverlayOffset.changed");
				}, this);
			}
		};


		/**
		 * @override
		 */
		CompareModule.prototype.getState = function () {
			var state = {
				// Compare model
				compareModel: {
					diffMode: this.getCurrentDiffMode(),
					cornerAlign: this._compareModel.getCornerAlign(),
					trimBoxAlign: this._compareModel.isTrimBoxAlign(),
					offset: this._compareModel.getOffset(),
					cadOverlayOffset: this._compareModel.getCadOverlayOffset(),
					cadOVerlayRotation: this._compareModel.getCadOverlayRotation(),
					referenceFlipped: this._compareModel.isReferenceFlipped(),
					referenceNegated: this._compareModel.isReferenceNegated(),
					referenceRotation: this._compareModel.getReferenceRotation()
				},
				// Difference mode
				differenceMode: {
					threshold: this._differenceMode.getThreshold(),
					colorSpace: this._differenceMode.getColorSpace(),
					addedColor: this._differenceMode.getAddedHighlightColor(),
					subtractedColor: this._differenceMode.getSubtractedHighlightColor()
				},
				// Side by side mode
				sideBySideMode: {
					panelsLayoutMode: this._sideBySideMode.getPanelsLayoutMode(),
					showingDifference: this._sideBySideMode.isShowingDifference()
				}
			};
			return state;
		};

		/**
		 * @override
		 * @param state
		 */
		CompareModule.prototype.setState = function (state) {
			this.f.whenModuleAvailable("compare", function () {
				if (state !== undefined) {
					// Compare model
					var compareModelState = state.compareModel;
					if (compareModelState !== undefined) {
						this.activateModeByDiffMode(compareModelState.diffMode);

						var reference = this.f.getDocumentController().getDifferencePage();
						if (reference !== undefined) {
							this._compareModel.setCornerAlign(compareModelState.cornerAlign);
							if (compareModelState.trimBoxAlign) {
								this._doTrimBoxAlign();
							}
							if (compareModelState.referenceFlipped) {
								this.flipReference();
							}
							if (compareModelState.referenceNegated) {
								this._compareModel.negateReference();
							}
							if (compareModelState.referenceRotation) {
								this.setReferenceRotation(compareModelState.referenceRotation);
							}
							if (!this._compareModel.isReaderSpreadActive()) {
								this.changeReferenceOffset(compareModelState.offset);
							}
							if (compareModelState.cadOverlayOffset) {
								this._compareModel.setCadOverlayOffset(compareModelState.cadOverlayOffset);
							}
							if (compareModelState.cadOverlayRotation) {
								this._compareModel.setCadOverlayRotation(compareModelState.cadOverlayRotation);
							}
						}
					}
					// Difference mode
					var differenceModeState = state.differenceMode;
					if (differenceModeState !== undefined) {
						this._differenceMode.setThreshold(differenceModeState.threshold);
						this._differenceMode.setColorSpace(differenceModeState.colorSpace);
						this._differenceMode.setAddedHighlightColor(differenceModeState.addedColor);
						this._differenceMode.setSubtractedHighlightColor(differenceModeState.subtractedColor);
					}
					// Side by side mode
					var sideBySideModeState = state.sideBySideMode;
					if (sideBySideModeState !== undefined) {
						this._sideBySideMode.setPanelsLayoutMode(sideBySideModeState.panelsLayoutMode);
						this._sideBySideMode.setShowingDifference(sideBySideModeState.showingDifference);
					}
				}
				this.f.debug.log("STATE", "compare state is injected.");
			}, this);
		};

		/**
		 * Is Reader spread active?
		 * @returns {Boolean} True if active, false otherwise.
		 */
		CompareModule.prototype.isReaderSpreadActive = function () {
			return this._compareModel.isReaderSpreadActive();
		};

		/**
		 * Do current compare range
		 * @param {AbstractDocumentPage} diffPage - The selected diff page to link to the active page
		 */
		CompareModule.prototype._doCurrentCompareRange = function (diffPage) {
			var activePage = this.f.getActivePage();

			activePage.setLinkedComparePage(diffPage);
		};

		/**
		 * Do current and following compare range
		 * @param {AbstractDocumentPage} diffPage - The selected diff page to link to the active page (and serves as base for following pages)
		 */
		CompareModule.prototype._doCurrentAndFollowingCompareRange = function (diffPage) {
			var activePage = this.f.getActivePage();
			var activeDocumentVersion = activePage.getDocumentVersion();
			var diffDocumentVersion = diffPage.getDocumentVersion();

			var activePageNumber = activePage.getPagenum();
			var diffPageNumber = diffPage.getPagenum();
			while (activePageNumber <= activeDocumentVersion.getPageCount()) {
				activePage = activeDocumentVersion.getPage(activePageNumber);

				// if there are no pages left, keep the last one
				if (diffPageNumber <= diffDocumentVersion.getPageCount()) {
					diffPage = diffDocumentVersion.getPage(diffPageNumber);
				}
				if (activePage !== undefined && !activePage.isBlankPage()) {
					activePage.setLinkedComparePage(diffPage);
				}

				activePageNumber++;
				diffPageNumber++;
			}
		};

		/**
		 * Do all compare range
		 * @param {AbstractDocumentPage} diffPage - The selected diff page to link to the active page (and serves as base for following/previous pages)
		 */
		CompareModule.prototype._doAllCompareRange = function (diffPage) {
			// do current and next
			this._doCurrentAndFollowingCompareRange(diffPage);

			// do previous
			var activePage = this.f.getActivePage();
			var activeDocumentVersion = activePage.getDocumentVersion();
			var diffDocumentVersion = diffPage.getDocumentVersion();

			var activePageNumber = activePage.getPagenum() - 1; // current already done
			var diffPageNumber = diffPage.getPagenum() - 1; // current already done
			while (activePageNumber >= 1) {
				activePage = activeDocumentVersion.getPage(activePageNumber);

				// if there are no pages left, keep the first one
				if (diffPageNumber >= 1) {
					diffPage = diffDocumentVersion.getPage(diffPageNumber);
				}

				if (activePage !== undefined && !activePage.isBlankPage()) {
					activePage.setLinkedComparePage(diffPage);
				}

				activePageNumber--;
				diffPageNumber--;
			}
		};

		/**
		 * Start up the auto alignment wizard
		 */
		CompareModule.prototype.startAlignmentWizard = function (cadOverlay) {
			if (!this._compareAlignmentWizard) {
				this._compareAlignmentWizard = new CompareAlignmentWizard(this.f, this, this._compareModel);
			}
			this._compareAlignmentWizard.setForCadOverlay(cadOverlay);

			this.f.startMode(this._compareAlignmentWizard);
		};

		/**
		 * Intercept active page change to make sure the diff page is set correctly.
		 * @param {Object} context - Context of the interceptor
		 */
		CompareModule.prototype._interceptActivePageChange = function (context) {
			var navigationModule = this.f.getModule("navigation");
			if (navigationModule) {
				navigationModule.resetExtraRotation();
			}

			var currentDiffPage = this.f.getDocumentController().getDifferencePage();
			if (currentDiffPage && !this.isReaderSpreadActive() && context.newActivePage.getDocumentVersion().isMultiPage()) {
				// there was already a diff page, and we are not in reader spread and the new one will be a multi page document, so update it if needed
				var newDiffPage = this.getDefaultDiffPage(context.newActivePage);
				if (newDiffPage) {
					// if nothing useful found, leave it like it is
					context.diffPage = newDiffPage;
				}
			}

			if (context.newActivePage.getDocumentVersion().isPageList()) {
				this._documentSelection.updateDocumentSelectorsForPageList(context.oldActivePage, context.newActivePage);
				if (!this.isReaderSpreadActive() && context.newActivePage.isBlankPage()) {
					context.diffPage = undefined;
					this.activateModeByName();
				}
			}

		};

		/**
		 * Flip the reference page
		 */
		CompareModule.prototype.flipReference = function () {
			var docController = this.f.getDocumentController();
			docController.willNeedParameterUpdate("FLIP");

			this._compareModel.flipReference();
			var diff = docController.getDifferencePage();
			diff.flipPage();

			docController.readyForParameterUpdate("FLIP");
		};

		/**
		 * Flip the cad overlay
		 * @param {Boolean} [flip] - if undefined just flip from the current flip state, otherwhise if true : page is flipped, if false page is not flipped
		 */
		CompareModule.prototype.flipCadOverlay = function (flip) {
			this._compareModel.flipCadOverlay(flip);

			var docController = this.f.getDocumentController();
			var cadOverlay = docController.getCadOverlay();
			if (cadOverlay) {
				docController.willNeedParameterUpdate("FLIP_OVERLAY");

				cadOverlay.flipPage(flip);

				docController.readyForParameterUpdate("FLIP_OVERLAY");
			}
		};

		/**
		 * Rotate the reference page with 1 step of 90 degrees
		 */
		CompareModule.prototype.rotateReference = function () {
			this.setReferenceRotation((this._compareModel.getReferenceRotation() + 90) % 360);
		};

		/**
		 * Set the rotation of the reference page
		 * @param {Number} rotation - rotation in degreees
		 */
		CompareModule.prototype.setReferenceRotation = function (rotation) {
			var docController = this.f.getDocumentController();
			docController.willNeedParameterUpdate("ROTATION");

			rotation = (rotation % 360);
			this._compareModel.setReferenceRotation(rotation);

			var diff = this.f.getDocumentController().getDifferencePage();
			diff.setRotation(rotation);
			var navigationModule = this.f.getModule("navigation");
			if (navigationModule && diff.hasCad()) {
				navigationModule.setExtraRotation(rotation);
			}

			docController.readyForParameterUpdate("ROTATION");
		};

		/**
		 * Rotate the CAD overlay
		 */
		CompareModule.prototype.rotateCadOverlay = function () {
			this.setCadOverlayRotation((this._compareModel.getCadOverlayRotation() + 90) % 360);
		};

		/**
		 * Set the CAD overlay to a given rotation
		 * @param {Number} rotation - rotation in degrees
		 */
		CompareModule.prototype.setCadOverlayRotation = function (rotation) {
			rotation = (rotation % 360);
			this._compareModel.setCadOverlayRotation(rotation);

			var docController = this.f.getDocumentController();
			var cadOverlay = docController.getCadOverlay();
			if (cadOverlay) {
				docController.willNeedParameterUpdate("ROTATION_OVERLAY");

				cadOverlay.setRotation(rotation);
				var navigationModule = this.f.getModule("navigation");
				if (navigationModule) {
					navigationModule.setExtraRotation(rotation);
				}
				docController.readyForParameterUpdate("ROTATION_OVERLAY");
			}
		};

		/**
		 * Publishes and shows the 3D widget
		 */
		CompareModule.prototype.checkAligmentAndShow3D = function () {
			if (this.isCadOverlayModified()) {
				this.f.loadBundle("viewer", function () {
					var promptSaveCADAlignPopup = new ModalPopup(this.f, "saveCADAlignment", this.f.getContext());
					promptSaveCADAlignPopup.setRemoveFromDomAfterUsage(true);

					// Header + Content
					if (this.canSaveCadOverlay()) {
						promptSaveCADAlignPopup.setOkAction(_.bind(function () {
							this.saveCadOverlay(_.bind(this._load3DView, this));
							promptSaveCADAlignPopup.hide();
						}, this));
						promptSaveCADAlignPopup.setHeader($("<span/>").addClass("popupTitle").text(this.f.i18n.t("CadOverlayPromptTitle")));
						promptSaveCADAlignPopup.setContent($("<div/>").addClass("unsavedItemsContent").text(this.f.i18n.t("CadOverlayPromptContent")));
						promptSaveCADAlignPopup.setOkBtnText(this.f.i18n.t("Save"));
					} else {
						promptSaveCADAlignPopup.setOkAction(_.bind(function () {
							this._load3DView();
							promptSaveCADAlignPopup.hide();
						}, this));
						promptSaveCADAlignPopup.setHeader($("<span/>").addClass("popupTitle").text(this.f.i18n.t("CadOverlayPromptOpen3D")));
						promptSaveCADAlignPopup.setContent($("<div/>").addClass("unsavedItemsContent").text(this.f.i18n.t("CadOverlayPromptOpen3DContent")));
						promptSaveCADAlignPopup.setOkBtnText(this.f.i18n.t("CadOverlayPromptOpen3D"));
					}

					promptSaveCADAlignPopup.setCancelBtnText(this.f.i18n.t("AppDialogCancel"));

					promptSaveCADAlignPopup.show();
				}, this);
			} else {
				if (!this.f.getActivePage().get("isCadAlignmentSaved") && this.canSaveCadOverlay()) {
					this.saveCadOverlay(_.bind(this._load3DView, this));
				} else {
					this._load3DView();
				}
			}
		};

		/**
		 * load the 3D view, loads the module which will publish the widget if needed
		 */
		CompareModule.prototype._load3DView = function () {
			this.f.requestModule("view3D", _.bind(function (view3DModule) {
				view3DModule.show3D();
			}, this), false);
		};

		return CompareModule;
	});
